/*
    TOMUSS: The Online Multi User Simple Spreadsheet
    Copyright (C) 2023 Thierry EXCOFFIER, Universite Claude Bernard

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

ALERT_RECORDS = { 'filter': 0, 'to': 0, 'cc': 0, 'subject': 0, 'body': 0 };

function configure_alert_record() {
    var new_config = [];
    var nbr = Object.keys(popup_column().alert).length + 1;
    for (var i = 0; i < nbr; i++) {
        var conf = {};
        var record = false;
        for (var name in ALERT_RECORDS) {
            var value = document.getElementById('alert_' + name + '_' + i).value.strip();
            if (value !== '') {
                conf[name] = value;
                record = true;
            }
        }
        if (record)
            new_config.push(conf);
    }
    column_attr_set(popup_column(), 'alert', new_config, undefined, true);
    popup_close();
}

function alert_colorize() {
    var nbr = Object.keys(popup_column().alert).length + 1;
    var errors = [];
    for (var i = 0; i < nbr; i++) {
        for (var name in ALERT_RECORDS) {
            var element = document.getElementById('alert_' + name + '_' + i);
            if (!element)
                continue;
            var value = element.value;
            element.style.color = "#000";
            value = value.split('[');
            for (var j in value) {
                if (j == 0)
                    continue;
                var title = value[j].split(']');
                if (title[1] !== undefined
                    && data_col_from_col_title(title[0]) === undefined) {
                    element.style.color = "#F00";
                    errors.push(_("TITLE_column_attr_alert")
                        + ' «' + html(popup_column().title) + '» : '
                        + _("ALERT_columns_unknown_title") + title[0] + '»<br>');
                }
            }
        }
    }
    set_message('column_alert', 2, errors.join(''));
}

function configure_alert() {
    var column = the_current_cell.column;
    var config = column.alert;
    config.push({});
    var content = [_('TIP_column_attr_alert'), '<table onkeyup="alert_colorize()">'];
    for (var name in ALERT_RECORDS) {
        content.push('</tr><tr><td colspan="' + config.length + '">'
            + _('MSG_alert_' + name) + '</tr><tr>');
        for (var i in config) {
            content.push('<td>');
            id = 'alert_' + name + '_' + i;
            if (name == 'body') {
                content.push('<textarea id="' + id + '" rows="10">');
                content.push(html(config[i][name] || ''));
                content.push('</textarea>');
            }
            else {
                content.push('<input id="' + id + '" value="');
                content.push(encode_value(config[i][name] || ''));
                content.push('">');
            }
        }
    }
    content.push('</tr></table>');
    create_popup('import_div',
        html(column.title) + ' <button onclick="configure_alert_record()">'
        + _('LABEL_save') + '</button>',
        content.join('').replace('</tr>', ''), '', false);
    alert_colorize();
    config.pop();
}