/*
  TOMUSS: The Online Multi User Simple Spreadsheet
  Copyright (C) 2011-2012 Thierry EXCOFFIER, Universite Claude Bernard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr
*/

function svg_rect(x, y, width, height, classe) {
    var r = document.createElementNS("http://www.w3.org/2000/svg", 'rect');
    r.setAttribute('x', x);
    r.setAttribute('y', y);
    if (width < 0)
        width = 0;
    r.setAttribute('width', width);
    r.setAttribute('height', height);
    if (classe)
        r.setAttribute('class', classe);
    return r;
}

function svg_text(text) {
    var r = document.createElementNS("http://www.w3.org/2000/svg", 'text');
    if (r.textContent !== undefined)
        r.textContent = text;
    else
        r.innerText = text;
    return r;
}

function histogram_bar(cls, x, dx, dy, maxmax, v, is_note, val_min, val_max,
    container) {
    var label, h = (dy * v) / maxmax;

    if (is_note) {
        label = (val_max - val_min) * cls / 20. + val_min;
        if (isNaN(label))
            label = '';
        else
            if (val_max - val_min >= 20)
                label = label.toFixed(0);
            else
                label = label.toFixed(1);
    }
    else
        label = cls;

    var r = svg_rect(x, dy - h, dx, h, 'a' + cls);
    container.appendChild(r);
    r = svg_text(label);
    r.setAttribute('transform', 'translate(' + (x + dx / 1.5) + ',0),rotate(-90)');
    container.appendChild(r);
}

var update_histogram_data_col = -1;
var update_histogram_id;
var svg_object, svg_style;
var t_column_histogram;
var t_column_graph;
var t_column_average;
var current_histogram;

function update_histogram_real() {
    do_update_histogram = false;
    if (the_current_cell.data_col == update_histogram_data_col)
        return;
    if (!t_column_histogram)
        return;
    update_histogram_data_col = the_current_cell.data_col;

    var dx = (t_column_histogram.the_width - 1) / 27;
    var dy = t_column_histogram.the_height;
    var font_size = Math.min((dx / 0.9).toFixed(0), (dy / 2.4).toFixed(0));
    var the_style =
        'g { pointer-events: none;}' +
        'rect { stroke: #000 ; stroke-opacity: 0.5 ; stroke-width:1 }' +
        '.a0, .a1, .a2, .a3, .a4 { fill: #F00 }' +
        '.a5, .a6, .a7, .a8, .a9 { fill: #FA0 }' +
        '.a10, .a11, .a12, .a13, .a14 { fill: #AFA }' +
        '.a15, .a16, .a17, .a18, .a19 { fill: #0F0 }' +
        '.appn { fill: #F8F }' +
        '.anan { fill: #FFF }' +
        '.aabi { fill: #F88 }' +
        '.aabj { fill: #88F }' +
        '.apre { fill: #8F8 }' +
        '.aoui { fill: #8FF }' +
        '.anon  { fill: #FF8 }' +
        'text { text-anchor:end; font-size:' + font_size + 'px; }';
    var stats = compute_histogram(the_current_cell.data_col);
    var i;
    var maxmax = stats.maxmax();

    if (!svg_object) {
        var d;
        try {
            d = document.createElementNS("http://www.w3.org/2000/svg", 'svg');
            d.setAttribute('width', 1000);
        }
        catch (err) {
            return;
        }
        t_column_histogram.appendChild(d);
        svg_style = document.createElementNS("http://www.w3.org/2000/svg", 'style');
        d.appendChild(svg_style);
        svg_object = document.createElementNS("http://www.w3.org/2000/svg", 'g');
        d.appendChild(svg_object);
    }
    if (svg_style.textContent !== undefined)
        svg_style.textContent = the_style;
    else
        svg_style.innerText = the_style;

    while (svg_object.firstChild)
        svg_object.removeChild(svg_object.firstChild);

    // allowed_grades?
    histogram_bar('ppn', 0 * dx, dx, dy, maxmax, stats.nr_ppn(), false, 0, 0, svg_object);
    histogram_bar('abi', 1 * dx, dx, dy, maxmax, stats.nr_abi(), false, 0, 0, svg_object);
    histogram_bar('abj', 2 * dx, dx, dy, maxmax, stats.nr_abj(), false, 0, 0, svg_object);
    histogram_bar('pre', 3 * dx, dx, dy, maxmax, stats.nr_pre(), false, 0, 0, svg_object);
    histogram_bar('oui', 4 * dx, dx, dy, maxmax, stats.nr_yes(), false, 0, 0, svg_object);
    histogram_bar('non', 5 * dx, dx, dy, maxmax, stats.nr_no(), false, 0, 0, svg_object);
    histogram_bar('nan', 6 * dx, dx, dy, maxmax, stats.nr_nan(), false, 0, 0, svg_object);

    for (i = 0; i < 20; i++)
        histogram_bar(i, (i + 7) * dx, dx, dy, maxmax, stats.histogram[i],
            true, stats.v_min, stats.v_max, svg_object
        );

    i = stats.average();
    if (i > 1)
        i = i.toFixed(1);
    else
        i = i.toFixed(2);
    t_column_average.innerHTML = i;
    current_histogram = stats;

    update_column_graph(t_column_graph, 256);
}

function graph_th(element) {
    column_graph_zoom_redraw_needed = [0, 0, false, element.cells[0].innerText];
}

var graph_just_selected = 'nothing is selected';

function update_column_graph(canvas, size, mouse_x, mouse_y, selected) {
    if (the_current_cell.data_col_previous === undefined)
        return '';
    canvas.width = size;
    canvas.height = size;
    var ctx = canvas.getContext("2d");
    ctx.lineWidth = 2;
    ctx.fillStyle = "#FFF";
    ctx.fillRect(0, 0, size, size);

    graph_just_selected = 'nothing is selected';
    var xmin = 1e50, xmax = -1e50, ymin = 1e50, ymax = -1e50;
    var selectors = {};
    var nrx = 0, sumx = 0, sumx2 = 0;
    var nry = 0, sumy = 0, sumy2 = 0;
    var nr_values = {}, sum_values = {};
    for (var line_id in filtered_lines) {
        var line = filtered_lines[line_id];
        var value_x_txt = line[the_current_cell.data_col_previous].value;
        var value_x = Number(value_x_txt)
        if (!isNaN(value_x) && value_x_txt !== '') {
            xmin = Math.min(xmin, value_x);
            xmax = Math.max(xmax, value_x);
            nrx++;
            sumx += value_x;
            sumx2 += value_x * value_x;
        }
        var value_y_txt = line[the_current_cell.data_col_current].value;
        value_y = Number(value_y_txt);
        if (!isNaN(value_y) && value_y_txt !== '') {
            ymin = Math.min(ymin, value_y);
            ymax = Math.max(ymax, value_y);
            nry++;
            sumy += value_y;
            sumy2 += value_y * value_y;
        }
        if (the_current_cell.data_col_previous_previous) {
            value = line[the_current_cell.data_col_previous_previous].value;
            selectors[value] = true;
            if (!isNaN(value_y) && value_y !== '') {
                if (nr_values[value] === undefined) {
                    nr_values[value] = 1;
                    sum_values[value] = value_y;
                }
                else {
                    nr_values[value]++;
                    sum_values[value] += value_y;
                }
            }
        }
    }
    var nr_selectors = Object.keys(selectors).length;
    if (nr_selectors < 32) {
        var i = 0;
        for (var selector in selectors)
            selectors[selector] = hls2rgb(i++ / nr_selectors, 0.5, 1);
    }
    else {
        for (var selector in selectors)
            selectors[selector] = "#000";
    }
    var radius = 10;
    var disks = [];
    var stddev_x = Math.pow((sumx2 / nrx) - (sumx / nrx) * (sumx / nrx), 0.5);
    var stddev_y = Math.pow((sumy2 / nry) - (sumy / nry) * (sumy / nry), 0.5);
    var stddev_xn = stddev_x * Math.pow(nrx / (nrx - 1), 0.5);
    var stddev_yn = stddev_y * Math.pow(nry / (nry - 1), 0.5);
    var infos = [
        '<style id="graphstyle"></style>',
        '<table class="colored" style="table-layout: fixed">',
        '<tr><th style="width:8em"><th><p style="overflow:auto">',
        html(columns[the_current_cell.data_col_previous].title),
        '<th><p style="overflow:auto">',
        html(columns[the_current_cell.data_col_current].title),
        '</tr>',
        '<tr><th>', _("B_s_minimum"), '<td>', xmin.toFixed(4), '<td>', ymin.toFixed(4), '</tr>',
        '<tr><th>', _("B_s_average"),
        '<td>', (sumx / nrx).toFixed(4),
        '<td>', (sumy / nry).toFixed(4), '</tr>',
        '<tr><th>', _("B_s_maximum"), '<td>', xmax.toFixed(4), '<td>', ymax.toFixed(4), '</tr>',
        '<tr><th>', _("B_s_stddev"), '<td>', stddev_x.toFixed(4), '<td>', stddev_y.toFixed(4), '</tr>',
        '<tr><th>', _("B_s_stddev_sample"), '<td>', stddev_xn.toFixed(4), '<td>', stddev_yn.toFixed(4), '</tr>',
        '</table>'
    ];

    if (Object.keys(selectors).length > 0 && Object.keys(selectors).length < nrx / 2) {
        infos.push(html(columns[the_current_cell.data_col_current].title) + ' :');
        infos.push('<table class="colored selector"><tr><th>')
        infos.push(html(columns[the_current_cell.data_col_previous_previous].title));
        infos.push('<th>');
        infos.push(_("B_s_nr_"));
        infos.push('<th>');
        infos.push(_("B_s_average_"));
        infos.push('</tr>');
        var sorted_keys = [];
        for (var i in nr_values)
            sorted_keys.push(i);
        sorted_keys.sort();
        for (var i in sorted_keys) {
            i = sorted_keys[i];
            infos.push('<tr class="_');
            infos.push(i.replace(/[^a-zA-Z0-9]/g, '_'));
            infos.push('" onmouseenter="graph_th(this)"><th style="background:');
            infos.push(selectors[i]);
            infos.push('"><p style="background:rgba(255,255,255,0.7)">')
            infos.push(i);
            infos.push('<td>');
            infos.push(nr_values[i]);
            infos.push('<td>');
            infos.push((sum_values[i] / nr_values[i]).toFixed(2));
            infos.push('</tr>');
        }
        infos.push('</table>');
    }

    if (xmax < columns[the_current_cell.data_col_previous].max)
        xmax = columns[the_current_cell.data_col_previous].max;
    if (ymax < columns[the_current_cell.data_col_current].max)
        ymax = columns[the_current_cell.data_col_current].max;
    xmin = Math.min(0, xmin);
    ymin = Math.min(0, ymin);

    for (var line_id in filtered_lines) {
        var line = filtered_lines[line_id];
        var x = line[the_current_cell.data_col_previous].value;
        var y = line[the_current_cell.data_col_current].value;
        var color = the_current_cell.data_col_previous_previous
            ? selectors[line[the_current_cell.data_col_previous_previous].value]
            : "#000";
        x = (x - xmin) / (xmax - xmin) * size;
        y = size - (y - ymin) / (ymax - ymin) * size;
        disks.push([line[the_current_cell.data_col_previous_previous].value, x, y]);
        if ((x - mouse_x) * (x - mouse_x) + (y - mouse_y) * (y - mouse_y) < radius * radius) {
            graph_just_selected = selected = line[the_current_cell.data_col_previous_previous].value.toString();
            ctx.fillStyle = "#000";
            ctx.font = '12px sans-serif';
            if (mouse_x < size * 0.7)
                label_x = mouse_x + 30;
            else
                label_x = mouse_x - 150;
            if (mouse_y < size * 0.9)
                label_y = mouse_y + 13;
            else
                label_y = mouse_y - 49;
            var lines = [
                line[1].value + ' ' + line[2].value,
                columns[the_current_cell.data_col_previous].title + '=' + line[the_current_cell.data_col_previous].value,
                columns[the_current_cell.data_col_current].title + '=' + line[the_current_cell.data_col_current].value,
                the_current_cell.data_col_previous_previous
                    ? columns[the_current_cell.data_col_previous_previous].title + '=' + line[the_current_cell.data_col_previous_previous].value
                    : ''
            ];
            ctx.fillText(lines[0], label_x, label_y);
            ctx.fillText(lines[1], label_x, label_y + 13);
            ctx.fillText(lines[2], label_x, label_y + 26);
            ctx.fillText(lines[3], label_x, label_y + 39);
            infos.push('<p>');
            for (var i in lines)
                infos.push('<div style="background:' + color + '2">' + html(lines[i]) + '</div>');
        }
        ctx.fillStyle = color + '3';
        if (isNaN(x) || isNaN(y))
            continue;
        ctx.beginPath();
        ctx.arc(x, y, radius, 0, 2 * Math.PI);
        ctx.fill();
    }
    if (selected !== undefined) {
        ctx.strokeStyle = "#000";
        ctx.lineWidth = 2;
        for (var i in disks) {
            if (disks[i][0] == selected) {
                ctx.beginPath();
                ctx.arc(disks[i][1], disks[i][2], radius - 2, 0, 2 * Math.PI);
                ctx.stroke();
            }
        }
    }
    ctx.fillStyle = "#000";
    ctx.font = '24px sans-serif';
    ctx.fillText(columns[the_current_cell.data_col_previous].title, size / 2, size - 10);
    ctx.fillText(xmin, 40, size - 10);
    ctx.fillText(xmax, 9 * size / 10, size - 10);

    ctx.fillText(columns[the_current_cell.data_col_current].title, 10, size / 2);
    ctx.fillText(ymin, 10, size - 40);
    ctx.fillText(ymax, 10, 20);

    ctx.strokeStyle = "#00F4";
    ctx.moveTo(0, size);
    ctx.lineTo(size, 0);
    ctx.stroke();

    return infos.join('');
}

var column_graph_zoom_redraw_needed;

function column_graph_zoom_redraw() {
    var canvas = document.getElementById("column_zoomed_graph");
    if (!canvas) {
        change_option('graph');
        return; // stop job
    }
    if (column_graph_zoom_redraw_needed == undefined)
        return true;
    var x = column_graph_zoom_redraw_needed[0];
    var y = column_graph_zoom_redraw_needed[1];
    var click = column_graph_zoom_redraw_needed[2];
    var selected = column_graph_zoom_redraw_needed[3];
    column_graph_zoom_redraw_needed = undefined;
    var infos = update_column_graph(canvas, window_height(), x, y, selected);
    if (click)
        document.getElementById('graph_infos').innerHTML = infos;
    document.getElementById('graphstyle').textContent =
     'TABLE.colored.selector TR._' + graph_just_selected.replace(/[^a-zA-Z0-9]/g, '_') + ' TD {background: #FF0}';

    document.getElementById('graphx').value = the_current_cell.data_col_previous;
    document.getElementById('graphy').value = the_current_cell.data_col_current;
    document.getElementById('graphc').value = the_current_cell.data_col_previous_previous;
    change_option('graph',
        the_current_cell.data_col_previous_previous
        + '=' + the_current_cell.data_col_previous
        + '=' + the_current_cell.data_col_current);
    return true;
}

function graph_change(select) {
    switch (select.id) {
        case 'graphx': the_current_cell.data_col_previous = Number(select.value); break;
        case 'graphy': the_current_cell.data_col_current = Number(select.value); break;
        case 'graphc': the_current_cell.data_col_previous_previous = Number(select.value); break;
    }
    column_graph_zoom_redraw_needed = [0, 0, true];
}

function column_graph_zoom() {
    var options = [];
    var cols = column_list_all();
    for (var i in cols)
        options.push('<option value="' + cols[i] + '">' + html(columns[cols[i]].title) + '</option>');
    options = options.join('');

    create_popup(
        'column_zoomed_graph', '',
        '<canvas id="column_zoomed_graph"></canvas>'
        + '<div>'
        + 'X : <select id="graphx" onchange="graph_change(this)">' + options + '</select><br>'
        + 'Y : <select id="graphy" onchange="graph_change(this)">' + options + '</select><br>'
        + 'C : <select id="graphc" onchange="graph_change(this)">' + options + '</select><br>'
        + '<div id="graph_infos"></div>'
        + '</div>'
        ,
        '',
        false);
    var canvas = document.getElementById("column_zoomed_graph");
    canvas.onmousemove = function (event) {
        event = the_event(event);
        column_graph_zoom_redraw_needed = [event.x, event.y];
    };
    canvas.onmousedown = function (event) {
        event = the_event(event);
        column_graph_zoom_redraw_needed = [event.x, event.y, true];
    };
    column_graph_zoom_redraw_needed = [0, 0, true];
    periodic_work_add(column_graph_zoom_redraw, 100);
}

function update_histogram(force) {
    t_column_histogram = document.getElementById('t_column_histogram');
    t_column_average = document.getElementById('t_column_stats');
    if (!t_column_average)
        return;
    if (force)
        update_histogram_data_col = -1;

    if (t_column_histogram && t_column_histogram.offsetWidth) {
        t_column_histogram.the_height = t_column_histogram.offsetHeight - 1;
        t_column_histogram.the_width = t_column_histogram.offsetWidth;
    }

    t_column_graph = document.getElementById('t_column_graph');

    if (update_histogram_id)
        clearTimeout(update_histogram_id);

    update_histogram_id = setTimeout(update_histogram_real, 300);
}
