/*
    TOMUSS: The Online Multi User Simple Spreadsheet
    Copyright (C) 2008-2015 Thierry EXCOFFIER, Universite Claude Bernard

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr
*/

function set_type(value, column, xcolumn_attr) {
    var checked = type_title_to_type(value);

    if (column.real_type
        && column.real_type.cell_compute !== undefined
        && checked.cell_compute === undefined
    ) {
        if (the_current_cell.line[column.data_col]._save !== undefined) {
            // Restore uncomputed values. XXX some may missing if never in client
            for (var line in lines)
                lines[line][column.data_col].restore();
        }
        else {
            // Make values user modifiable
            for (var line in lines) {
                var cell = lines[line][column.data_col];
                if (cell.author.length <= 1) {
                    cell = cell.set_value('')
                    lines[line][column.data_col] = cell
                    cell.date = '20000101010101'
                }
            }
        }
        table_fill(false, false, true);
    }
    if (column.real_type
        && column.real_type.cell_compute === undefined
        && checked.cell_compute !== undefined) {
        /* Save values */
        for (var line in lines)
            lines[line][column.data_col].save();
    }
    column.real_type = checked;
    column.need_update = true;

    if (xcolumn_attr === false && column.columns === '') {
        // Only here on local user interaction
        var use = _('B_' + column.real_type.title).split("(")[1];
        if (use == 'ID)' && data_col_from_col_title('ID') !== undefined)
            column_attr_set(column, 'columns', 'ID');
    }
    if (xcolumn_attr === false && column.real_type.type_change)
        column.real_type.type_change(column);

    setTimeout(grade_type_in_average, 100);
    return value;
}

function popup_type_choice_help(element) {
    var type = element.getAttribute('type');
    document.getElementById('type_help').innerHTML = '<b>' + _('B_' + type) + '</b> : '
        + _('H_' + type);
}

function popup_type_choice(t) {
    var type = t.title;
    var e = '<a type="' + type
        + '" onmouseenter="popup_type_choice_help(this)"'
        + ' onclick="javascript:popup_type_choosed(this)">'
        + _('B_' + type).replace(/(\(.*\))/,
            '<small><small>$1</small></small>')
        + '</a>';
    if (type === 'Note' || type === 'Prst' || type === 'Moy')
        e = '<b>' + e + '</b>';
    return e;
}

function type_filter(text) {
    var all_types = popup_get_element().getElementsByTagName('A');
    text = flat(text.toLowerCase());
    for(var type of all_types) {
        var code = type.getAttribute('type');
        if (flat(type.textContent).toLowerCase().indexOf(text) == -1 &&
            flat(_('H_' + code)).toLowerCase().indexOf(text) == -1)
            type.style.opacity = 0.2;
        else
        type.style.opacity = 1;
    }
    document.getElementById('type_help').innerHTML = '';
}

function popup_type_chooser(column, _value) {
    if (!column_change_allowed(column)) {
        Alert("ERROR_value_not_modifiable");
        return;
    }

    var cols = {};
    for (var i in types) {
        t = types[i];
        if (!cols[t.type_type])
            cols[t.type_type] = [];
        cols[t.type_type].push(popup_type_choice(t));
    }
    var t = '<style>#popup A:hover { color: blue; background: #FFF }</style><table class="colored" style="float:left"><tr>';
    for (var i in cols)
        t += '<th>' + _('TH_type_type_' + i);
    t += '</tr><tr>';
    for (var i in cols) {
        // Use try/catch for an unknown IE error
        try {
            cols[i].sort(function (a, b) {
                return a.human_priority
                    - b.human_priority;
            });
        }
        catch (error) {
        }
        t += '<td>' + cols[i].join('<br>');
    }
    t += '</table><div id="type_help"></div>';

    create_popup('type_chooser_div',
        _('TITLE_columntype') + column.title,
        _('MSG_columntype')
        + ' <input autofocus placeholder="' + _('filtre2.png') + '" onkeyup="type_filter(this.value)"><br>',
        t,
        false);
}

function popup_type_choosed(element) {
    var type = element.getAttribute('type');
    var t = type_title_to_type(type);
    if (t.type_type != 'data' && t.type_type != 'computed')
        if (!confirm(_('ALERT_columntype')))
            return;

    var button = document.getElementById('t_column_type');
    var td = the_td(button);
    column_attr_set(popup_column(), 'type', type, td, true);
    if (the_current_cell.column == popup_column()) {
        button.innerHTML = _('B_' + type);
        attr_update_user_interface(column_attributes['type'],
            the_current_cell.column);
    }
    popup_close();
    var n = 'ALERT_change_to_' + type;
    if (myindex(column_attributes['columns'].visible_for, type) >= 0) {
        if (the_current_cell.column.columns === '') {
            set_message('type_change', 1,
                _("COL_TITLE_type") + ' → ' + _('B_' + type) + '<br>'
                + (_(n) == n ? _("columns.png") : _(n).replace(/\n/g, '<br>')));
        }
        select_tab("column", _("TAB_formula"));
        document.getElementById('t_column_columns').focus();
        highlight_add(document.getElementById('t_column_columns'));
    }
    else if (_(n) != n)
        set_message('type_change', 1,
            _("COL_TITLE_type") + ' → ' + _('B_' + type) + '<br>'
            + _(n).replace(/\n/g, '<br>'));
    else
        set_message('type_change', 1);
}
