#!/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2023 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

import ast
from . import mail
from .. import utilities
from .. import configuration

class Abis(mail.Mail):
    """Compute the number of ABI for the current university year"""
    attributes_visible = ('columns',)
    def get_all_values(self, column, line_ids=None):
        from ..TEMPLATES import P # Avoid circular import
        uyear = utilities.university_year(column.table.year, column.table.semester)
        students = P.p_get_students(uyear)
        translate = configuration.all_all.get
        for line_id, user in self.users(column, line_ids):
            nr_abis = None
            resume = user.manage_key('resume')
            if resume:
                # ABI in the tables
                nr_abis = 0
                for _ue, semesters in ast.literal_eval(resume).items():
                    for stats in semesters:
                        if utilities.university_year(*stats[:2]) == uyear:
                            nr_abis += stats[3]
            if user.student_id in students:
                # ABI in the P templates
                nr_abis = nr_abis or 0
                for infos in students[user.student_id].values():
                    if translate(infos[0].rstrip('@'), '') == configuration.abi:
                        nr_abis += 1
            if nr_abis is not None:
                yield line_id, nr_abis
