/*
    TOMUSS: The Online Multi User Simple Spreadsheet
    Copyright (C) 2021 Thierry EXCOFFIER, Universite Claude Bernard

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

var analyser_current_attrs = {
    "script_G_upload": 0,
    "script_G_old_upload": 0,
    "script_G_on_upload": 0,
    "script_U_upload": 0,
    "script_U_old_upload": 0,
    "script_U_on_upload": 1,
    "script_G": '',
    "script_U": ''
};

function analyser_read_gui() {
    var value;
    for (var i in analyser_current_attrs) {
        if (document.getElementById(i).tagName == 'INPUT')
            value = Number(document.getElementById(i).checked);
        else
            value = document.getElementById(i).value;
        analyser_current_attrs[i] = value;
    }
}

function analyser_save(button) {
    analyser_read_gui();
    column_attr_set(popup_column(), 'analyser_config',
        JSON.stringify(analyser_current_attrs), button, true);
    analyser_update_gui();
}

function analyser_write_gui() {
    var config = JSON.parse(the_current_cell.column.analyser_config);
    for (var i in analyser_current_attrs) {
        analyser_current_attrs[i] = config[i];
        if (isNaN(config[i]))
            document.getElementById(i).value = config[i];
        else
            document.getElementById(i).checked = config[i];
    }
    analyser_update_gui();
}

function analyser_config_saved() {
    var config = JSON.parse(the_current_cell.column.analyser_config);
    analyser_read_gui();
    for (var i in analyser_current_attrs)
        if (analyser_current_attrs[i] !== config[i]) {
            return false;
        }
    return true;
}

function analyser_update_gui_real() {
    if (!document.getElementById('analyser_grade'))
        return; // Closed popup
    var config_saved = analyser_config_saved();
    document.getElementById('analyser_grade').disabled =
        !config_saved || analyser_grade[0] == 0 || !analyser_savable;
    document.getElementById('analyser_regrade').disabled =
        !config_saved || analyser_regrade[0] == 0 || !analyser_savable;
    document.getElementById('analyser_filtered').disabled =
        !config_saved || analyser_filtered[0] == 0 || !analyser_savable;
    document.getElementById('analyser_record').disabled = config_saved || !analyser_savable;
}

function analyser_update_gui() {
    setTimeout(analyser_update_gui_real, 300);
}

function analyser_do(element) {
    create_popup('import_div', 'Grade',
        '<style>.import_export { bottom: 25% }</style>'
        + '<iframe name="analyser_iframe" style="width:100%; height:calc(100% - 3em)"></iframe>', '', false);

    do_post_data(
        { 'line_ids': window[element.id][0].join(' ') },
        add_ticket(year + "/" + semester + "/" + ue + '/analyser_grade/' + popup_column().data_col),
        "analyser_iframe");
}

var analyser_savable;
var analyser_grade;
var analyser_regrade;
var analyser_filtered;

function analyser_config() {
    var why_not_savable = column_change_allowed_text(the_current_cell.column);
    analyser_savable = why_not_savable === true;

    // [line_ids_possible, line_ids_impossible]
    analyser_grade = [[], []];
    analyser_regrade = [[], []];
    analyser_filtered = [[], []];
    for (var line_id in lines) {
        var line = lines[line_id];
        if (line[0].value === '')
            continue
        var cell = line[the_current_cell.data_col];
        var index = Number(!cell.modifiable(line, the_current_cell.column));
        if (line.is_filtered)
            analyser_filtered[index].push(line_id);
        if (cell.value === '')
            analyser_grade[index].push(line_id);
        analyser_regrade[index].push(line_id);
    }

    function button(name, text) {
        var numbers = window[name];
        return '<button id="' + name + '" onclick="analyser_do(this)">'
            + _("MSG_analyser_grade"
            ).replace('{0}', numbers[0].length.toString()).replace('{1}', text)
            + (numbers[1].length
                ? '<br><span class="red">'
                + _("MSG_analyser_impossible").replace('{}', numbers[1].length.toString())
                + '</span>'
                : '')
            + '</button> ';
    }

    create_popup('import_div',
        html(the_current_cell.column.title)
        + ', '
        + (the_current_cell.column.columns.length
            ? _("MSG_analyser_columns") + ' ' + the_current_cell.column.columns
            : '<span class="red">' + _("MSG_analyser_no_columns") + '</span>'
        ),
        '<style>TABLE.analyser TD { vertical-align: top; padding: 1em }'
        + 'TABLE.analyser TD { width: 50% }'
        + 'TABLE.analyser BUTTON { margin-top: 0.5em }'
        + '.red { color: #F00 }'
        + '</style>'
        + _( "MSG_analyser_intro"),
        '<table class="colored analyser" onmousedown="analyser_update_gui()" onkeydown="analyser_update_gui()"><tr>'
        + '<th>' + _("TH_analyser_grading")
        + '<th>' + _("TH_analyser_checking") + '</tr><tr><td>'
        + '<label><input id="script_G_upload" type="checkbox"> ' + _("MSG_analyser_upload") + '</label><br>'
        + '<label><input id="script_G_old_upload" type="checkbox"> ' + _("MSG_analyser_old_upload") + '</label><br>'
        + '<label><input id="script_G_on_upload" type="checkbox"> ' + _("MSG_analyser_on_upload") + '</label>'
        + '<textarea rows="10" id="script_G"># Python script</textarea>'
        + button('analyser_grade', _('MSG_analyser_gradeless')) + ' '
        + button('analyser_filtered', _('MSG_analyser_filtered')) + ' '
        + button('analyser_regrade', _('MSG_analyser_all'))
        + '<td>'
        + '<label><input id="script_U_upload" type="checkbox"> ' + _("MSG_analyser_upload") + '</label><br>'
        + '<label><input id="script_U_old_upload" type="checkbox"> ' + _("MSG_analyser_old_upload") + '</label><br>'
        + '<label><input id="script_U_on_upload" type="checkbox" checked> ' + _("MSG_analyser_on_upload") + '</label>'
        + '<textarea rows="10" id="script_U"># Your Python script</textarea>'
        + _("MSG_analyser_fast")
        + '</tr></table>'
        + '<p>'
        + '<button id="analyser_record" onclick="analyser_save(this)">' + _('LABEL_save') + '</button> '
        + (analyser_savable ? '' : '<span class="red">' + _("MSG_analyser_ro") + '<span>')
        + '<a href="'
        + add_ticket(year + "/" + semester + "/" + ue + '/analyser_zip/' + the_current_cell.data_col)
        + '">' + _("MSG_analyser_zip") + '</a>'
        , false);
    analyser_write_gui();
    if (!analyser_savable)
        for (var key in analyser_current_attrs)
            document.getElementById(key).disabled = 1;
}