/*
  TOMUSS: The Online Multi User Simple Spreadsheet
  Copyright (C) 2020 Thierry EXCOFFIER, Universite Claude Bernard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/
// 0: nobody, 1: student, 2: teacher, 3: both

annotator_options = {
    'colors': [
        ["#000", 3], ["#F00", 3], ["#0A0", 3], ["#00F", 3],
        ["#0CC", 0], ["#C0C", 0], ["#CC0", 0]],
    'color': [['#000', 1], ["#F00", 2]],
    'widths': [['1', 3], ['4', 3], ['8', 3], ['12', 3]],
    'width': [['1', 3]],
    'sizes': [['18', 3], ['24', 3], ['32', 3], ['48', 3]],
    'size': [['24', 3]],
    'modes': [
        ['Edit', 3, '➚'], ['Text', 3, '<b>T</b>'], ['Draw', 3, '✍'], ['Highlight', 3, '🖍'],
        ['-1', 2, '-1'], ['+1', 2, '+1'], ['-0.5', 2, '-½'], ['+0.5', 2, '+½'],
        ['-0.25', 0, '-¼'], ['+0.25', 0, '+¼'],
        ['😟', 0], ['😊', 0], ['👎', 0], ['👍', 0]],
    'mode': [['Text', 1], ["+1", 2]]
};

/*
'save'
'annotations': 'demo.txt',
*/

function annotator_unused_letter() {
    var used = annotator_configuration().images;
    for (var letter = 'A'; ; letter = String.fromCharCode(letter.charCodeAt(0) + 1))
        if (myindex(used, letter) == -1)
            return letter;
}

function annotator_config_options(column) {
    if (column.annotator == 1) {
        column.annotator = {};
        column.annotator.images = [];
        column.annotator.options = [{}, {}];
        for (var i in annotator_options) {
            var s = [], t = [];
            for (var j in annotator_options[i]) {
                var v = annotator_options[i][j];
                if (v[1] & 1)
                    s.push(v[0]);
                if (v[1] & 2)
                    t.push(v[0]);
            }
            column.annotator.options[0][i] = s.join(',');
            column.annotator.options[1][i] = t.join(',');
        }
    }
}

function year_sem_ue() {
    try {
        return DisplayGrades.ue.year + '/' + DisplayGrades.ue.semester + '/' + DisplayGrades.ue.ue;
    }
    catch (e) {
        return year + '/' + semester + '/' + ue;
    }
}

function annotator_pdf_get() {
    var line_ids = [];
    for (var i in filtered_lines)
        if(filtered_lines[i][0].value !== '')
            line_ids.push(filtered_lines[i].line_id);
    line_ids.push("tomuss.zip")
    window_open(add_ticket(year_sem_ue() + '/annotator_form/' + the_current_cell.column.the_id + '/' + line_ids.join('/')));
}

function annotator_config() {
    var column = the_current_cell.column;
    annotator_config_options(column);
    var t = [
        '<style>',
        'DIV.import_export.annotator { border: 2px solid black ; position: fixed; top: 10%; bottom:10%; z-index: 30 }',
        'TABLE.annotator { --size: 34px }',
        'TABLE.annotator TR { vertical-align: top }',
        'TABLE.annotator .annotator_image { display: inline-block; text-align: center}',
        'TABLE.annotator FORM { display: inline }',
        'TABLE.annotator FORM IFRAME { height: 1.5em ; width: 3em ; border: 0px }',
        'TABLE.annotator DIV.annotator_icons { line-height: 10px }', // Why not 30px ?
        'TABLE.annotator INPUT { width: auto; }',
        'TABLE.annotator DIV.i { width: var(--size) ; height: var(--size) ; display: inline-block;',
        '    overflow: hidden ; text-align: center; line-height: var(--size); user-select: none;',
        '    cursor: pointer; box-sizing: border-box}',
        'TABLE.annotator DIV.i SPAN { vertical-align: middle;}',
        'TABLE.annotator DIV.i:hover { border: 1px solid black }',
        'TABLE.annotator DIV.possible.selected { background:#FFF }',
        'TABLE.annotator DIV.possible { background:#DDD }',
        'TABLE.annotator TD:nth-child(1), TABLE.annotator TD:nth-child(2) { background: #BBB }',
        'TABLE.annotator DIV.impossible { background:#BBB }',
        'TABLE.annotator TH, TABLE.annotator TH BUTTON { font-size: 75%; padding: 0px }',
        'TABLE.annotator TH:nth-child(1) { width: calc(var(--size) * 2 + 1px) }',
        'TABLE.annotator TH:nth-child(2) { width: calc(var(--size) * 2 + 1px) }',
        'TABLE.annotator HR { margin: 0px }',
        '#annotator BUTTON { vertical-align: text-bottom }',
        '#annotator_save.blink { background-color: #8F8 ; }',
        '</style>',
        '<table id="annotator" class="annotator colored"><tr>',
        '<th>', _('TH_student'), '<br><button class="0">', _('Test'), '</button>',
        '<th>', _('COL_TITLE_teacher'), '<br><button class="1">', _('Test'), '</button>',
        '<td>', _('MSG_annotate_add_subject'),
        '<form method="POST" enctype="multipart/form-data" target="annotator_feedback" action="',
        add_ticket(year_sem_ue() + '/annotator_upload/' + column.the_id),
        '">',
        '<input type="file" name="data" accept="application/pdf" onchange="this.nextSibling.value=annotator_unused_letter();this.parentNode.submit()">',
        '<input type="hidden" name="letter">',
        '<iframe name="annotator_feedback"></iframe>',
        '</form>',
        '<button id="annotator_save">', _('LABEL_save'), '</button>',
        ' <button class="2">', _('Form'), '</button>',
        '</tr><tr>',
    ];
    function add(name, style, display) {
        var current = column.annotator.options[i][name];
        var possibles = column.annotator.options[i][name + 's'];
        var options = annotator_options[name + 's'];
        var cls;
        for (var j in options) {
            var value = options[j][0];
            if (possibles.indexOf(value) == -1)
                cls = 'impossible';
            else
                cls = 'possible';
            if (value == current)
                cls += ' selected';
            t.push('<div style="');
            t.push(style.replace(/{}/g, value));
            t.push('" value="');
            t.push(value);
            t.push('" class="i ');
            t.push(cls);
            t.push('"><span>');
            t.push(display.replace('{}', options[j][2] || value));
            t.push("</span></div>");
        }
    }
    for (var i = 0; i < 2; i++) {
        t.push('<td><div class="annotator_icons">');
        add('mode', "font-family:emoji", '{}');
        t.push('<hr>');
        add('color', "color:{}", '⬤');
        t.push('<hr>');
        add('width', "font-size:calc({}px * 1.5)", '⬤');
        t.push('<hr>');
        add('size', "font-size:calc({}px / 1.5)", 'T');
    }
    t.push('<td><div style="width:46em; margin-left: 1em">');
    t.push(_('MSG_annotate_doc'));
    t.push('</div></td>');
    t.push('</tr>');
    t.push('</table>');
    create_popup(
        'annotator',
        '',
        t.join(''),
        '',
        false);
    var a = document.getElementById('annotator');
    a.addEventListener("mousedown", annotator_click);
    a.addEventListener("dblclick", annotator_dblclick);
    for (var i in column.annotator.images)
        annotator_add(column.annotator.images[i]);
    annotator_configuration_is_modified();
}

function annotator_url(column, line, who, timestamp, merge) {
    if (line) {
        annotator_config_options(column);
        var options = column.annotator.options[who];
    }
    else
        var options = annotator_configuration(who).options[who == '2' ? 0 : who];
    var letter, line_id;
    if (line) {
        line_id = line.line_id;
        if (line[column.data_col].comment)
            letter = line[column.data_col].comment.substr(0, 1);
        else
            letter = '';
    } else {
        letter = column.annotator.images[0];
        line_id = my_identity;
        options['save'] = add_ticket(year_sem_ue() + '/annotator_record/'
            + column.the_id + '/' + line_id);
        options['annotations'] = add_ticket(year_sem_ue() + '/annotator_load/'
            + column.the_id + '/' + line_id);
    }
    if (column.annotator.images && column.annotator.images.length)
        options['image'] = annotator_image_url(column, letter, timestamp, line_id);
    else if (line)
        options['image'] = add_ticket(year_sem_ue()
            + '/upload_get/' + column.the_id + '/' + line_id + '/x.png');
    else
        options['image'] = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAAAAAA6fptVAAAACklEQVQI12P4DwABAQEAG7buVgAAAABJRU5ErkJggg";
    options['teacher'] = who;
    options['me'] = my_identity;
    options['origin'] = window.location.origin;
    if (merge)
        for (var i in merge)
            options[i] = merge[i];
    if (line && !line[column.data_col].modifiable(line, column))
        options['save'] = undefined;
    return '_FILES_/Annotator.html#' + JSON.stringify(options);
}

function annotator_image_url(column, letter, timestamp, line_id) {
    return add_ticket(year_sem_ue()
        + '/annotator_get/' + column.the_id
        + '/' + (letter === '' ? line_id : letter + '.png')
        + '/' + (timestamp || millisec()));
}

function annotator_close() {
    window.onmessage = function (event) {
        if (event.data != 'annotator ok')
            return;
        popup_close = annotator_open.old_close;
        the_current_cell.jump = annotator_open.old_jump;
        if (the_current_cell.tr)
            the_current_cell.tr.className = the_current_cell.tr.className.replace(' currentformline', '');
        popup_close();
        window.onmessage = function () { };
    };
    var iframe_annotate = document.getElementById('iframe_annotate');
    if (iframe_annotate.src.match(/%22save%22/))
        iframe_annotate.src = iframe_annotate.src.replace('%22save%22', '%22s%22'); // Ask save/cancel
    else
        window.onmessage({ data: 'annotator ok' });
}

function annotator_full_url(column, timestamp, error) {
    return annotator_url(column, the_current_cell.line, 1, timestamp,
        {
            'save': add_ticket(year_sem_ue() + '/annotator_record/'
                + column.the_id + '/' + the_current_cell.line_id),
            'annotations': add_ticket(year_sem_ue() + '/annotator_load/'
                + column.the_id + '/' + the_current_cell.line_id),
            'error': error
        });
}

function annotator_jump(lin, col, do_not_focus, line_id, data_col) {
    the_current_cell.tr.className = the_current_cell.tr.className.replace(' currentformline', '');
    col = the_current_cell.col;
    data_col = popup_column().data_col;
    annotator_open.old_jump(lin, col, do_not_focus, line_id, data_col);
    the_current_cell.tr.className += ' currentformline';
    var iframe = document.getElementById('iframe_annotate');
    var msg = annotator_annotable(popup_column(column));
    // Force ok/cancel confirm save question :
    iframe.src = annotator_full_url(popup_column(), annotator_open.timestamp, msg);
    annotator_open.key = url + '/' + year_sem_ue() + '/annotator_record/'
        + popup_column().the_id + '/' + line_id;
}

function annotator_annotable(column) {
    if (the_current_cell.line[column.data_col].comment === '') {
        if (column.annotator.images.length)
            return _('ALERT_annotate_subject').replace('{}', column.annotator.images);
        return _("MSG_annotate_no_file");
    }
    if (the_current_cell.td.className.match(/ suivi_modifiable/)) {
        return _("MSG_annotate_not_allowed");
    }
}

function annotator_open(_value, column) {
    annotator_config_options(column);
    create_popup(
        'annotator_grade',
        '',
        '<style>'
        + 'DIV.import_export.annotator_grade'
        + ' { border-left: 2px solid black ; left:auto; width: 1242px; max-width:80%; top:0px; bottom:0px; right: 2em;'
        + '   z-index: 30; overflow: hidden; }'
        + 'DIV.annotator_grade IFRAME { border: 0px ; width: calc(100% - 2em) ; height: 100%; }'
        + '</style>'
        + '<iframe id="iframe_annotate" src="'
        + encodeURI(annotator_full_url(column, 0, annotator_annotable(column)))
        + '"></iframe>'
        + '<p class="zoom_buttons" onclick="popup_zoom(-100)" style="top:3em">←→</p>'
        + '<p class="zoom_buttons" onclick="popup_zoom(100)" style="top: 6em">→←</p>',
        '',
        false);
    annotator_open.old_close = popup_close;
    annotator_open.old_jump = the_current_cell.jump.bind(the_current_cell);
    // XXX modified in annotator_jump
    annotator_open.key = url + '/' + year_sem_ue() + '/annotator_record/'
        + column.the_id + '/' + the_current_cell.line_id;
    annotator_open.timestamp = millisec();
    popup_close = annotator_close;
    the_current_cell.tr.className += ' currentformline';
    the_current_cell.jump = annotator_jump.bind(the_current_cell);
}

function annotator_configuration(who) {
    var annotator = document.getElementById('annotator');
    var student = annotator.rows[1].cells[0].firstChild.firstChild;
    var teacher = annotator.rows[1].cells[1].firstChild.firstChild;
    var data = JSON.parse(JSON.stringify(popup_column().annotator));
    var A = data.options;
    function test(name) {
        var options = annotator_options[name + 's'];
        var stu = [];
        var tea = [];
        for (var _i in options) {
            if (student.className.indexOf('impossible') == -1)
                stu.push(student.getAttribute('value'));
            if (student.className.indexOf('selected') != -1)
                A[0][name] = student.getAttribute('value');
            if (teacher.className.indexOf('impossible') == -1)
                tea.push(teacher.getAttribute('value'));
            if (teacher.className.indexOf('selected') != -1)
                A[1][name] = teacher.getAttribute('value');
            do {
                student = student.nextSibling;
                teacher = teacher.nextSibling;
            } while (student && student.tagName != 'DIV');
        }
        A[0][name + 's'] = stu.join(',');
        A[1][name + 's'] = tea.join(',');
    };
    if (who == 2) { // For forms
        A[0].modes = A[0].mode = 'Edit';
        A[0].colors = A[0].color = '#000';
        A[0].widths = A[0].width = '1';
        A[0].sizes = "18,24,32,48";
        A[0].size = '18';
    }
    else {
        test('mode');
        test('color');
        test('width');
        test('size');
    }
    var t = document.getElementById('annotator').rows[1].cells[2];
    data.images = [];
    for (var e = t.firstChild; e; e = e.nextSibling)
        if (e.lastChild.tagName == 'IMG')
            data.images.push(e.firstChild.textContent.substr(0, 1));
    data.images.sort();
    if (who == '2') {
        var cols = [];
        var cls = column_list_all();
        for (var i in cls)
            cols.push(columns[cls[i]].title);
        A[0].columns = A[1].columns = cols;
    }
    return data;
};

function annotator_configuration_is_modified() {
    var on_screen = annotator_configuration();
    var recorded = popup_column().annotator;
    var modified;
    if (on_screen.images.toString() != recorded.images.toString()) {
        modified = true;
    }
    for (var i = 0; i < 2; i++)
        for (var j in on_screen.options[i])
            if (on_screen.options[i][j] != recorded.options[i][j]) {
                modified = true;
            }
    document.getElementById("annotator_save").disabled = !modified;
    document.getElementById("annotator_save").className = modified ? "blink" : '';
    return modified;
}

function annotator_click(event) {
    var t = event.target;
    if (t.tagName == 'BUTTON') {
        if (t.className >= 0 && t.className <= 2)
            window_open(annotator_url(popup_column(), undefined, t.className));
        else {
            var data = annotator_configuration();
            column_attr_set(popup_column(), 'annotate', data, t, true);
            annotator_configuration_is_modified();
        }
        return;
    }
    while (t && t.tagName != 'DIV')
        t = t.parentNode;
    if (t) {
        if (event.target.className == 'annotator_delete')
            t.parentNode.removeChild(t);
        else
            if (t.className.indexOf('selected') == -1)
                t.className = t.className.replace('possible', 'impossible').replace('imim', '');
    }
    annotator_configuration_is_modified();
}

function annotator_dblclick(event) {
    var t = event.target;
    while (t && t.tagName != 'DIV')
        t = t.parentNode;
    if (!t)
        return;
    for (var o = t; o && o.tagName == 'DIV'; o = o.previousSibling)
        o.className = o.className.replace(' selected', '');
    for (var o = t; o && o.tagName == 'DIV'; o = o.nextSibling)
        o.className = o.className.replace(' selected', '');
    t.className = t.className.replace('impossible', 'possible') + ' selected';
    annotator_configuration_is_modified();
}

function annotator_add(image) {
    var t = document.getElementById('annotator').rows[1].cells[2];
    var div = document.createElement('DIV');
    var img = document.createElement('IMG');
    img.src = annotator_image_url(popup_column(), image);
    img.style.height = '550px';
    div.className = 'annotator_image';
    div.innerHTML = '<p>' + image + ' ' + '<span class="annotator_delete">🗑</span>' + '<br>';
    div.appendChild(img);
    if (t.lastChild && t.lastChild.lastChild.tagName != 'IMG')
        t.removeChild(t.lastChild);
    t.appendChild(div);
    annotator_configuration_is_modified();
}

function i_am_a_new_annotator(column) {
    for (var line_id in lines) {
        var cell = lines[line_id][column.data_col];
        if (cell.value !== '' && cell.comment.match(/^[a-z]*\/.*charset=/)) {
            Alert('ALERT_annotate_impossible');
            break;
        }
    }
    column_attr_set(column, 'visibility', 1);
    column_attr_set(column, 'modifiable', 2);
    column_attr_set(column, 'cell_writable', "= | @ | #[]");
    table_header_fill();
    if (column.annotate === 1)
        setTimeout(annotator_config, 100);
}

function annotator_only_for_teacher(column) {
    return !column.annotator
        || column.annotator == 1
        || !column.annotator.images
        || column.annotator.images.length == 0;
}

function popup_zoom(delta) {
    var e = popup_get_element().firstChild;
    var width = e.offsetWidth - delta;
    if (width < 100)
        width = 100;
    else if (width > 2000) // Should test height > 32000 (fabric.js bug)
        return;
    e.style.width = width + 'px';
}


function annotator_format_suivi() {
    var column = DisplayGrades.column;
    var modifiable = cell_modifiable_on_suivi();
    if (!modifiable && annotator_only_for_teacher(column)
        && !isNaN(DisplayGrades.cell.value))
        return '<a target="_blank" href="'
            + encodeURI(annotator_url(column, DisplayGrades.ue.line_real, is_a_teacher, undefined,
                {
                    'annotations': add_ticket(year_sem_ue() + '/annotator_load/'
                        + column.the_id + '/' + DisplayGrades.ue.line_id)
                }))
            + '">'
            + ((isNaN(DisplayGrades.cell.value) || DisplayGrades.cell.value === '' || modifiable)
                ? ''
                : note_format_suivi() + ' : ')
            + _("MSG_annotate_correction")
            + '</a>';

    if (column.modifiable === false) // Last grade list
        if (DisplayGrades.cell.value === '' || isNaN(DisplayGrades.cell.value))
            return '';
    if (annotator_only_for_teacher(column))
        return upload_format_suivi();
    var merge = {
        'annotations':
            add_ticket(year_sem_ue() + '/annotator_load/'
                + column.the_id + '/' + DisplayGrades.ue.line_id)
    };
    if (modifiable)
        merge['save'] = add_ticket(year_sem_ue() + '/annotator_record/'
            + column.the_id + '/' + DisplayGrades.ue.line_id);

    var graded = (
        column.annotator.options[0].modes
        + column.annotator.options[1].modes).match(/,[-+]/);

    var msg = ((isNaN(DisplayGrades.cell.value) || DisplayGrades.cell.value === ''
        || modifiable || !graded)
        ? ''
        : note_format_suivi() + ' : ')
        + (is_a_teacher ? _("MSG_annotate_grade") :
            (modifiable ? _("MSG_annotate_answer") : _("MSG_annotate_correction")
            ));
    var annotate_url = encodeURI(annotator_url(column, DisplayGrades.ue.line_real, is_a_teacher, undefined, merge));
    if (window_width() < window_height() && window_width() < 1600)
        return '<a target="_blank" href="' + annotate_url + '">' + msg + '</a>';
    else
        return '<a class="clickable" style="color:blue" onclick="'
            + encode_value("create_popup('annotate_suivi','', '"
                + '<style>DIV.import_export.annotate_suivi {position:fixed;top:0px;bottom:0px;max-width:85%;width:1242px;left:auto;right:0px}'
                + '.annotate_suivi IFRAME { width:calc(100% - 2em); height:100%; }</style>'
                + '<p class="zoom_buttons" onclick="popup_zoom(-100)" style="top:3em">←→</p>'
                + '<p class="zoom_buttons" onclick="popup_zoom(100)" style="top: 6em">→←</p>'
                + '<iframe id="iframe_annotate" src="' + annotate_url + "\"></iframe>','',false);"
                + 'annotator_open.old_close=popup_close;popup_close=annotator_close;'
                + 'annotator_open.key =' + js(url + '/' + year_sem_ue() + '/annotator_record/'
                    + column.the_id + '/' + DisplayGrades.ue.line_id)
            ) + '">' + msg + '</a>';
}
