/*
    TOMUSS: The Online Multi User Simple Spreadsheet
    Copyright (C) 2023 Thierry EXCOFFIER, Universite Claude Bernard

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

function Calendar(v, no_alert) {
    this.days = {};
    this.slots = {};
    this.day_slots = [];
    this.sorted_slots = [];
    this.start_minute = 24 * 60;
    this.end_minute = 0;
    var days = [];
    v = v.split(/[ \t\n]+/);
    for (var i in v) {
        if (v[i] === '')
            continue;
        var slot = v[i].split(':');
        if (slot.length != 2 || slot[0].length != 12 || isNaN(slot[0]) || isNaN(slot[1])) {
            this.error = _("ALERT_invalid_value") + v[i];
            if (!no_alert)
                alert(this.error);
            return;
        }
        var day = slot[0].substr(0, 8);
        if (this.days[day] === undefined) {
            var date = get_date_tomuss(day);
            this.days[day] = {
                name: date.formate('%a<br>%d %b'),
                slots: [],
                day: day
                // index will by added at the end
            };
            days.push(day);
        }
        var start_minute = 60 * Number(slot[0].substr(8, 2)) + Number(slot[0].substr(10));
        var duration = Number(slot[1]);
        this.slots[slot[0].substr(8)] = start_minute;
        day_slot = {
            index: i,
            start_label: slot[0].substr(8),
            start_minute: start_minute,
            duration: duration,
            day: this.days[day]
        };
        this.days[day].slots.push(day_slot);
        this.day_slots.push(day_slot);
        this.sorted_slots.push(day_slot);
        if (start_minute < this.start_minute)
            this.start_minute = start_minute;
        if (start_minute + duration > this.end_minute)
            this.end_minute = start_minute + duration;
    }
    days.sort();
    for (var i in days)
        this.days[days[i]].index = Number(i);
    this.error = '';

    function cmp_slots(a, b) {
        a = a.day.day + a.start_label;
        b = b.day.day + b.start_label;
        if (a < b) return -1;
        if (a > b) return 1;
        return 0;
    }
    this.sorted_slots.sort(cmp_slots);
}

Calendar.prototype.compute_stats = function (data_col) {
    this.stats = [];
    for (var j in this.day_slots)
        this.stats.push({});
    for (var i in filtered_lines) {
        var value = filtered_lines[i][data_col].value;
        for (var j in this.day_slots) {
            var char = value.substr(j, 1) || 'x';
            this.stats[j][char] = (this.stats[j][char] || 0) + 1;
        }
    }
};

Calendar.prototype.compute_export = function (data_col) {
    var content = ['<table><tr><td>'];
    for (var i in this.sorted_slots)
        content.push('<th>' + this.sorted_slots[i].day.day + this.sorted_slots[i].start_label);
    content.push('</tr><tr><td>');
    for (var i in this.sorted_slots)
        content.push('<th>' + this.sorted_slots[i].day.name + '<br>'
            + this.sorted_slots[i].start_label);
    content.push('</tr>');
    var translate = {
        'U': no, 'S': yes,
        'u': no.toLowerCase(), 's': yes.toLowerCase(),
        'x': ''
    }
    for (var i in filtered_lines) {
        content.push('<tr><th>' + filtered_lines[i][0].value);
        var value = filtered_lines[i][data_col].value;
        for (var j in this.sorted_slots)
            content.push('<td>' + translate[value.substr(this.sorted_slots[j].index, 1) || 'x']);
        content.push('</tr>');
    }
    content.push('</table>');
    return content.join('');
};

Calendar.prototype.html_content = function (selection, small, width, height,
    modifiable, stats) {
    var nr_columns = Object.keys(this.days).length;
    var header_height = small ? 0 : 27;
    var header_width = small ? 0 : 29;
    var start_minute = this.start_minute;
    var scale_x = (width - header_width) / nr_columns;
    var scale_y = (height - header_height) / (this.end_minute - this.start_minute);
    function div(char, x, y, height, text, modifiable) {
        content.push('<div class="slot slot' + char
            + '" '
            + ((modifiable && !small)
                ? 'onmousemove="calendar_action(the_event(event),' + slot.index + ',0)" '
                + 'onmousedown="calendar_action(the_event(event),' + slot.index + ',1)" '
                : '')
            + 'style="left:' + (x == -1 ? 0 : header_width + x * scale_x)
            + 'px;top:' + (y == -1 ? 0 : header_height + (y - start_minute) * scale_y)
            + 'px;height:' + scale_y * height
            + 'px;width:' + scale_x
            + 'px;' + ((modifiable || x == -1 || y == -1 || stats) ? '' : 'opacity:0.3;')
            + '">' + text + '</div>');
    }
    var content = [], text = '';
    if (!small)
        for (var i in this.slots)
            div('slotname', -1, this.slots[i], 60, i);
    for (var i in this.days) {
        var day = this.days[i];
        if (!small)
            div('dayname', day.index, -1, 60, day.name);
        for (var j in day.slots) {
            var slot = day.slots[j];
            var char = selection.substr(slot.index, 1) || 'x';
            if (stats && this.stats) {
                var counts = this.stats[slot.index];
                if (counts) {
                    text = '<span class="slots">' + ((counts['s'] || 0) + (counts['S'] || 0)) + '</span>';
                    text += '<span class="slotu">' + ((counts['u'] || 0) + (counts['U'] || 0)) + '</span>';
                }
                else
                    text = '';
            }
            div(char,
                day.index, slot.start_minute, slot.duration, text,
                modifiable && char == char.toLowerCase());
        }
    }
    return content.join('');
};

Calendar.prototype.html_div = function (selection, small, width, height, modifiable, stats) {
    return '<div ' + (small ? '' : 'id="the_calendar" ')
        + 'style="width:' + width + 'px;height:' + height
        + 'px;" class="calendar ' + (small ? 'calendar_small' : 'calendar_big')
        + '">' + this.html_content(selection, small, width, height, modifiable, stats) + '</div>';
};

Calendar.prototype.html = function (selection, small, width, height, modifiable, stats) {
    var div = document.createElement('DIV');
    div.innerHTML = this.html_content(selection, small, width, height, modifiable, stats);
    div.className = 'calendar ' + (small ? 'calendar_small' : 'calendar_big');
    div.style.width = width + 'px';
    div.style.height = height + 'px';
    return div;
};

function calendar_save_configuration(event) {
    column_attr_set(popup_column(), 'calendar',
        popup_value().join('\n'), event.target, true);
    table_fill(true, true);
}

function calendar_update(_event, stats) {
    var calendar = new Calendar(popup_text_area().value, true);
    if (stats)
        calendar.stats = stats;
    popup_text_area().style.background = calendar.error ? "#FDD" : "#FFF";
    if (calendar.error === '')
        document.getElementById('the_calendar').outerHTML =
            calendar.html_div('', false, 3 * window_width() / 4, window_height() / 4, false, true);
    document.getElementById('calendar_feedback').innerHTML = calendar.error;
}

function calendar_configure() {
    var column = the_current_cell.column;
    column.Calendar.compute_stats(column.data_col);
    create_popup('import_div calendar_popup', _('B_Calendar')
        + ' «' + html(column.title) + '» '
        + '<button onclick="calendar_save_configuration(the_event(event))">' + _('LABEL_save')
        + '</button><span id="calendar_feedback"></span>',
        _("TIP_column_attr_calendar__").replace(/\n/g, '<br>'),
        '<div id="the_calendar"></div>',
        column.calendar);
    popup_text_area().onkeyup = calendar_update;
    calendar_update(undefined, column.Calendar.stats);
}

function calendar_export() {
    var column = the_current_cell.column;
    create_popup('import_div calendar_popup', _('B_Calendar')
        + ' «' + html(column.title) + '» ',
        column.Calendar.compute_export(column.data_col), '', false
    );
}

function set_calendar(v, column, xattr) {
    var old_cal = column.calendar.replace(/[ \n\t]+/, ' ');
    var new_cal = v.replace(/[ \n\t]+/, ' ');
    if (xattr === false // Interactive
        && !column_empty_of_cells(column.data_col)
        && (new_cal.length < old_cal.length || new_cal.substr(0, old_cal.length) != old_cal)
    ) {
        Alert('ALERT_calendar_slots')
        return column.calendar;
    }
    column.Calendar = new Calendar(v);
    return v;
}

var calendar_state = 'x';

function calendar_action(event, index, action) {
    var value = calendar_current
        ? calendar_current.value
        : (the_current_cell.input.value === ''
            ? the_current_cell.column.empty_is
            : the_current_cell.input.value)
    while (value.length <= index)
        value += 'x';
    if (action == 1)
        calendar_state = { 'x': 's', 's': 'u', 'u': 'x' }[value.substr(index, 1) || 'x'];
    else if (!event.buttons)
        return;
    value = value.substr(0, index) + calendar_state + value.substr(index + 1);
    if (calendar_current) {
        calendar_current.value = value;
        document.getElementById('the_calendar').outerHTML = calendar_patch();
    }
    else {
        the_current_cell.input.value = value;
        the_current_cell.update_cell_headers();
    }
    stop_event(event);
}

function calendar_save(event) {
    _cell({ value: calendar_current.value, parentNode: event.target.parentNode },
        year + '/' + semester + '/' + calendar_current.ue.ue
        + '/cell/' + calendar_current.column.the_id
        + '/' + calendar_current.ue.line_id);
}

var calendar_current;

function calendar_patch() {
    return calendar_current.html_div(
        calendar_current.value, false,
        600, 300, calendar_current.modifiable);
}

function calendar_suivi(nr) {
    calendar_current = all_calendars[nr];
    create_popup('import_div calendar_div', _('B_Calendar')
        + ' «' + html(calendar_current.column.title) + '» '
        + '<div><button onclick="calendar_save(the_event(event))">' + _('LABEL_save') + '</button></div>',
        calendar_patch(), '', false);
}

var all_calendars = [];

function calendar_format_suivi() {
    var calendar = DisplayGrades.column.Calendar;
    calendar.column = DisplayGrades.column;
    calendar.value = DisplayGrades.value;
    calendar.ue = DisplayGrades.ue;
    calendar.modifiable = cell_modifiable_on_suivi();
    all_calendars.push(calendar);
    return DisplayGrades.column.Calendar.html_div(
        DisplayGrades.value, true, 100, 20)
        .replace('<div', '<div onclick="calendar_suivi('
            + (all_calendars.length - 1) + ')"');
}