#!/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2011-2016 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import inspect
from . import text
from .. import configuration
from .. import data
from .. import utilities

class Code_Etape(text.Text):
    human_priority = 12
    tip_cell = ""
    set_columns = 'set_columns'
    attributes_visible = ('columns',)
    type_type = 'people'

    def data_col(self, the_table, column):
        """Returns the data_col of the first column in columns"""
        try:
            id_column_title = column.get_columns()[0]
        except IndexError:
            return None
        return the_table.columns.data_col_from_title(id_column_title)

    def values(self, column, line_ids=None):
        data_col = self.data_col(column.table, column)
        if data_col is None:
            return []
        if line_ids is None:
            line_ids = list(column.table.lines.keys()) # pragma: no cover
        return [(line_id, column.table.lines[line_id][data_col].value)
                for line_id in line_ids
                ]

    def users(self, column, line_ids=None):
        values = [line_id_login
                  for line_id_login in self.values(column, line_ids)
                  if line_id_login[1] # No empty login
                 ]
        if values:
            users = configuration.Users(login  for line_id, login in values)
            for line_id, login in values:
                yield line_id, users[login]

    def get_all_values(self, column, line_ids=None):
        """Redefine this method to get the [line_id, value] list"""
        year = utilities.university_year(column.table.year, column.table.semester)
        for line_id, user in self.users(column, line_ids):
            yield line_id, ' '.join(user.get_etapes_list(year) or '')

    def get_one_value(self, student_id, column, line_id):
        """Deprecated, Define get_all_values"""
        return None # pragma: no cover

    def update_one(self, the_table, line_id, column):
        self.update_all(the_table, column, line_ids=(line_id,))

    def simulate_get_all_values(self, the_table, column, line_ids):
        data_col = self.data_col(the_table, column)
        if data_col is None:
            return
        for line_id in (line_ids or the_table.lines):
            v =  self.get_one_value(the_table.lines[line_id][data_col].value,
                                    column, line_id)
            if v is not None:
                yield (line_id, v)
        
    def update_all(self, the_table, column, attr=None, line_ids=None):
        if attr is not None and attr.name != 'columns' and attr.name != 'type':
            return
        if line_ids is None:
            line_ids = list(column.table.lines.keys())

        if 'get_all_values' in self.__class__.__dict__:
            values = self.get_all_values(column, line_ids)
        elif 'get_one_value' in self.__class__.__dict__:
            values = self.simulate_get_all_values(the_table, column, line_ids)
        else:
            raise ValueError("Missing method: get_all_values or get_one_value") # pragma: no cover


        line_id_done = set()
        data_col = the_table.column_inscrit()
        rw_page = None
        for line_id, value in values:
            line_id_done.add(line_id)
            the_table.lock()
            try:
                if (data_col
                    and the_table.lines[line_id][data_col].value) == 'non': # pragma: no cover
                    if rw_page is None:
                        rw_page = the_table.get_rw_page()
                    page = rw_page
                else:
                    page = the_table.pages[0]
                the_table.cell_change(page, column.the_id, line_id, value)
            finally:
                the_table.unlock()

        # Erase system defined cells without value
        for line_id in set(line_ids) - line_id_done:
            if (the_table.lines[line_id][column.data_col].author
                in (data.ro_user, data.rw_user, data.no_user)):
                the_table.lock()
                try:
                    the_table.cell_change(the_table.pages[0], column.the_id,
                                          line_id, '')
                finally:
                    the_table.unlock()
