// TOMUSS: The Online Multi User Simple Spreadsheet
// Copyright (C) 2023 Guillaume FOURCHE, Universite Claude Bernard
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact: Thierry.EXCOFFIER@univ-lyon1.fr

function CompetenceTable() {
    this.FUC_state = 0; // 0=None, 1=Formation 2=UE 3=Column
    this.do_update = false; // true if the table need to be update by a search
    this.catalog = undefined;
    this.update_id = undefined; // ID of the periodic popup update
    this.jump_old = undefined; // The normal value for Current.prototype.jump()
    this.popup_close_jump_old = undefined; // The normal value for popup_close()

    // HTML elements of the popup
    this.searchbar = undefined; // Search INPUT
    this.student_name = undefined;
    this.student_id = undefined;
};
CompetenceTable.prototype.try_open = function () {
    if (this.catalog)
        this.open();
    else {
        var xhr = new XMLHttpRequest();
        xhr.open("GET", add_ticket("competences"), true);
        var THIS = this;
        xhr.onload = function (_event) {
            THIS.catalog = new Catalog();
            THIS.catalog.parse_json(xhr.response);
            THIS.open();
        };
        xhr.send();
    }
};
CompetenceTable.prototype.update = function () {
    // Update popup content
    if (popup_is_open()) {
        if (this.do_update) {
            this.update_popup();
            this.do_update = false;
        }
    } else {
        clearInterval(this.update_id);
        this.update_id = undefined;
    }
};
CompetenceTable.prototype.open = function () {
    // Create a popup for the interactions with the catalog in the competences interface.
    var line = lines[the_current_cell.line_id];
    var observation_tip = _("TIP_comp_observations") + '<div class="color_observations">';
    var o_voc = this.catalog.vocabulary['o'];
    var color_observations = '';

    for (var index in o_voc) {
        var color = observation_color(index);
        color_observations += '.color_observations > LI:nth-child(' + (Number(index) + 1)
            + ') {color:' + color + '; margin-left: 2em;}'
            + '.color_observations > :nth-child(' + (Number(index) + 1)
            + ') INPUT{background-color:' + color + '; --form-control-color: ' + color + '}'
        observation_tip += '<LI style="list-style-type: square">' + html(o_voc[index]) + '</LI>';
    }
    observation_tip += '</div>';
    var head_popup = ['<style>',
        '.import_export.competences_content{ top: 198px; right: 17px; bottom: 0px; left: auto; width: 60vw; border: 1px solid #303030; padding: 0 }',
        'h2{ height: 30px; background-color: #a0a0ad; border-bottom: 1px solid #303030; overflow: hidden; display: flex; flex-direction: row; justify-content: space-between }',
        'DIV.help { display: block; background-color: black; overflow: visible; }',
        'p { min-height: 1.2em }',
        'DIV.import_export BUTTON.close { height: 30px; border: 1px solid #303030; cursor: pointer; }',
        'DIV.import_export BUTTON.close:hover { background: #f87c7c }',
        '#observation_table{ height: calc(100vh - 275px); overflow-y: auto }',

        '#observation_table .comp_line:nth-child(even) { background-color: #EEE }',
        '.comp_line{ min-height: 2em; position: relative; display: flex; align-items: center; overflow: hidden }',
        '.comp_line_absolute{ position: absolute; top: 0; height: 1.8em; width: 100%; box-sizing: border-box;',
        'display: flex; flex-direction: row; z-index: 1; overflow: hidden; white-space:nowrap;}',

        // Grades detailed :
        '.grades_container { max-width: calc(100% - 100px); margin-left: 15px }',
        '.grade_line { width: fit-content; max-width: 100%; display: flex; align-items: center; position: relative }',
        '.grade_line P { min-width: 1cm; margin: 0 35px 0 5px; color: black; font-size: 0.85em; white-space: nowrap; overflow: hidden }',
        '.desc_edit_btn { position: absolute; top: 1px; right: 10px; height: 20px; width: 20px; padding: 0; display: none }',
        '.grade_line:hover .desc_edit_btn {display: block}',

        '.toggle_observations{ position: static; display: flex }',
        'input[type="checkbox"] { appearance: none; cursor: pointer } input[disabled] { opacity: 0.2 }',
        'INPUT.grade_input { display: grid; place-content: center; margin: 0; width: 2em; min-width: 2em; height: 2em; border: 1px solid black; }',
        'INPUT.grade_input::before { content: ""; width: 1.95em; height: 1.95em; transform: scale(0);',
        'background-color: black; transition: 120ms transform ease-in-out; transform-origin: center;',
        'clip-path: polygon(0 5%, 5% 0, 50% 45%, 95% 0, 100% 5%, 55% 50%, 100% 95%, 95% 100%, 50% 55%, 5% 100%, 0 95%, 45% 50%)}', // cross
        'INPUT.grade_input:checked::before { transform: scale(1) }',
        '.toggle_FUC{ position: static; height: 100%; min-width: 90px; max-width: 90px; padding: 0 5px 0 5px; display: flex;',
        'align-items: center; font-size: 0.7em; text-align: center; border-right: 1px black solid }',
        'INPUT.FUC_input { width: 21px; height: 21px; margin: 3px; border: 2px solid #303030; background: #0000; }',

        '.line_desc { padding: 5px 0 5px 0px; cursor: pointer }',
        '.depth1 { background-color: #85C !important; border-top: 1px black solid }',
        '.depth2 { background-color: #AAF !important; border-top: 1px black solid }',
        '.depth3 { background-color: #CCF !important; border-top: 1px black solid }',
        '.depth1 .line_desc { font-weight: bold; font-size: larger; color: #FFF }',
        '.depth2 .line_desc { font-weight: bold; }',
        '.depth4 .line_desc { margin-left: 20px; color: #00F }',
        '.depth5 .line_desc { margin-left: 40px; color: #000 }',
        '.top_FUC{ width: 25px; height: 25px; background-color: white; border: 1px solid #303030; border-style: outset }',
        '.top_FUC[checked="true"]{ background-color: #00bfff; border-color: #4682b4; border-style: outset }',
        '.top_FUC:hover{ background-color: #87cefa; cursor: pointer }',
        '.top_FUC:active{ background-color: #1e90ff; border-style inset }',
        '.toggle_FUC INPUT:checked { background: #000 }',
        color_observations,

        // To add in file :
        '.toggle_FUC { min-height: 2em }',
        '.subcompetence-add {width: 25px; height: 25px; margin: 1px; padding: 0; font-size: 20px }',
        '.subcompetence-delete {width: 25px; height: 25px; margin: 1px; padding: 0; font-size: 20px }',
        '.title_edit_btn { height: 20px; width: 20px; margin: 4px; padding: 0; }',
        'P.editable{ padding: 3px 10px 3px 10px }',
        '#comp_searchbar { width: max-content; height: 1.5em; margin: 5px }',
        '#competences_head{ border-bottom: 1px solid #303030; background-color: #b0b0bd; display: flex; justify-content: space-between; overflow: hidden}',
        '</style>',

        // Head :
        '<div id="competences_content">',
        '<div id="competences_head">',
        '<div style="width:84px; padding: 2px; margin-right: 10px; display: flex; justify-content: space-between">',
        hidden_txt('<button class="top_FUC" onclick="competenceTable.toggle_top_FUC(this, 1);" instanttip="1">F</button>',
            _("TIP_comp_formation"), ' (-> ' + ue + ')'),
        hidden_txt('<button class="top_FUC" onclick="competenceTable.toggle_top_FUC(this, 2);" instanttip="1">U</button>',
            _("TIP_comp_ue"), ' (-> définies par le responsable de cette UE)'),
        hidden_txt('<button class="top_FUC" onclick="competenceTable.toggle_top_FUC(this, 3);" instanttip="1">C</button>',
            _("TIP_comp_column"), ' (-> définies par la colonne de cette cellule)'),
        '</div>',
        hidden_txt('<p instanttip="1" style="margin-top: 5px">' + _("MSG_comp_observations") + '</p>', observation_tip),
        hidden_txt('<input id="comp_searchbar" type="text" onkeyup="competenceTable.do_update = true" placeholder="' + _("MSG_comp_searchbar") + '"/>', _("TIP_comp_searchbar")),
        '<p style="max-height: 30px; width: 220px; margin: 0 5px 0 15px; font-size: 0.7em" >', _("MSG_competences_infos"), '</p>',
        '</div>',
        '<div id="observation_table" onclick="competenceTable.toggle(event)">',
        '</div>',
        '</div>'];

    var student_infos = the_current_cell.column.title
        + '<div id="comp_student_infos" style="padding-left: 30px; max-width: 60%">' + line[1].value + " " + line[2].value + '</div>'
        + '<p id="comp_student_id" style="text-align:right; margin-right: 35px; font-size: 0.5em">' + line[0].value + '</p>';
    create_popup('competences_content', student_infos, head_popup.join(''), '', false);
    this.searchbar = document.getElementById("comp_searchbar");
    this.student_name = document.getElementById('comp_student_infos');
    this.student_id = document.getElementById('comp_student_id');

    this.searchbar.focus();
    this.filters = document.getElementsByClassName("top_FUC");
    this.select_default_filter();
    if (this.FUC_state)
        this.filters[this.FUC_state - 1].setAttribute('checked', 'true');

    the_current_cell.tr.className += ' currentformline';
    the_current_cell.input.className += ' currentformline';

    this.jump_old = Current.prototype.jump.bind(the_current_cell);
    this.popup_close_old = popup_close;

    Current.prototype.jump = this.jump.bind(this);
    popup_close = this.close.bind(this);

    this.do_update = true;
    document.body.addEventListener("keyup", this.onkey);
    this.update_id = setInterval(this.update.bind(this), 100);
};
CompetenceTable.prototype.clear_current_line = function () {
    the_current_cell.tr.className = the_current_cell.tr.className
        .replace(/ *currentformline/, '');
};
CompetenceTable.prototype.close = function () {
    // Close the popup
    document.body.removeEventListener("keyup", this.onkey);
    this.clear_current_line();
    popup_close = this.popup_close_old;
    Current.prototype.jump = this.jump_old;
    popup_close();
};
CompetenceTable.prototype.jump = function (lin, col, do_not_focus, line_id, data_col) {
    // Switch the popup content to an other line
    if (!line_id)
        line_id = line_id_from_lin(lin);
    this.clear_current_line();
    var old_lin = the_current_cell.lin;
    this.jump_old(lin, col, do_not_focus, line_id, data_col);

    if (line_id && old_lin != lin) {
        var line = lines[line_id];
        this.do_update = true;
        this.student_name.innerHTML = line[1].value + " " + line[2].value;
        this.student_id.innerHTML = line[0].value;
    }
    the_current_cell.tr.className += ' currentformline';
    the_current_cell.input.className += ' currentformline';
    this.searchbar.focus();
};
CompetenceTable.prototype.onkey = function (event) {
    if (event.key == "ArrowUp" || event.key == "PageUp")
        the_current_cell.cursor_up();
    else if (event.key == "ArrowDown" || event.key == "PageDown")
        the_current_cell.cursor_down();
};
CompetenceTable.prototype.select_default_filter = function () {
    // Activate one of the category buttons depend to the most short competences list
    if (popup_column().competence.trim().length)
        this.FUC_state = 3;
    else if (table_attr.p_competence.keys.length
        || Object.keys(this.catalog.get_allowed_keys_ue(ue)).length)
        this.FUC_state = 2;
    else
        this.FUC_state = Object.keys(this.catalog.get_allowed_keys(ue)).length ? 1 : 0;
    for (var index in filters)
        filters[index].value = index == this.FUC_state - 1 ? "active" : "unactive";
};

CompetenceTable.prototype.subcompetence = function (key, desc = null) {
    /*
        Add subcompetence if key target competence,
        else delete targeted subcompetence if key contain "+"
    */
    var subcomps = table_attr.p_competence.subcompetences;

    if(key.indexOf("+") != -1) {
        // Delete
        var [cmp_key, index] = key.split("+");
        if (!subcomps[cmp_key] || subcomps[cmp_key].length <= index)
            return false;
        subcomps[cmp_key][index] = null;
        if(Number(index) == subcomps[cmp_key].length - 1)
            subcomps[cmp_key].splice(index, 1);

        // Clear table and column
        var table_keys = table_attr.p_competence.keys;
        var col_keys = popup_column().competence.split(/ +/);
        var tab_index = table_keys.indexOf(key);
        var col_index = col_keys.indexOf(key);
        if(tab_index !== -1) {
            table_keys.splice(tab_index, 1);
            update_table_competence("keys", table_keys);
        }
        if(col_index !== -1) {
            col_keys.splice(col_index, 1);
            column_attr_set(popup_column(), "competence", col_keys.join(" "));
        }
    } else {
        // Add
        if (!subcomps[key])
            subcomps[key] = [];
        var index = subcomps[key].length;
        subcomps[key].push(desc || _("MSG_new_subcompetence"));

        // Set Params
        var table_keys = table_attr.p_competence.keys;
        var col_keys = popup_column().competence.split(/ +/);
        if(table_keys.indexOf(key) !== -1) {
            table_keys.push(key + "+" + index);
            update_table_competence("keys", table_keys);
        }
        if(col_keys.indexOf(key) !== -1) {
            col_keys.push(key + "+" + index);
            column_attr_set(popup_column(), "competence", col_keys.join(" "));
        }
    }

    // Update key observations
    var lin = the_current_cell.line_id;
    var col = popup_column().data_col;
    var cell_keys = lines[lin][col].value
    if (cell_keys.indexOf(key + 'o') !== -1) {
        var comps = cell_keys.split(/ +/);
        var i = 0;
        while (comps[i].indexOf(key + 'o') === -1) i++;
        comps.splice(i, 1);
        cell_set_value_real(lin, col, comps.join(' ').trim());
        update_cell_at(lin, col);
        the_current_cell.update_cell_headers();
    }

    update_table_competence("subcompetences", subcomps);
    this.do_update = true;
}

CompetenceTable.prototype.edit_text_handler = function (event) {
    // The button need to be placed before a editable object
    var edit_obj = event.target;
    while (edit_obj.nodeName !== "BUTTON")
        edit_obj = edit_obj.parentElement;
    edit_obj = edit_obj.nextSibling;
    if (edit_obj.className.indexOf("editable") !== -1) {
        edit_obj.contentEditable = true;
        edit_obj.focus();
    }
}
CompetenceTable.prototype.grade_text_handler = function (event, default_value) {
    if (event.code === 'Enter' || event.type === "blur") {
        var in_text = event.target.innerText;
        if (in_text === "")
            event.target.innerText = default_value;
        else if (in_text !== default_value) {
            var cmp_key = get_parent_attribute(event, "competence");
            var word = key_last_word(cmp_key.split("+")[0]);
            var key = word.join("");
            var index = event.target.getAttribute("index");

            var grades_vocab = table_attr.p_competence.grades_vocab;
            if (!grades_vocab[key])
                grades_vocab[key] = [];
            while (grades_vocab[key].length - 1 < index)
                grades_vocab[key].push(null);

            grades_vocab[key][index] =
                in_text === competenceTable.catalog.vocabulary['o'][Number(index)] ?
                    null : in_text.split('\n').join('');

            update_table_competence('grades_vocab', grades_vocab);
        }
        event.target.contentEditable = false;
    }
}
CompetenceTable.prototype.subcomp_text_handler = function (event, default_value) {
    if (event.code === 'Enter' || event.type === "blur") {
        var in_text = event.target.innerText;
        if (in_text === "")
            event.target.innerText = default_value;
        else if (in_text !== default_value) {
            var [key, index] = get_parent_attribute(event, "competence").split("+");
            var subcomps = table_attr.p_competence.subcompetences;
            subcomps[key][index] = in_text.split("\n").join("");
            update_table_competence("subcompetences", subcomps);
        }
        event.target.contentEditable = false;
    }
}
CompetenceTable.prototype.switch_details = function (event) {
    if (!event)
        return this.update_popup();

    var key = get_parent_attribute(event, "competence");
    this.update_popup(key);
}

CompetenceTable.prototype.toggle_FUC = function (checkbox, competence, index) {
    /*  Desc:
            When a param box is checked,
            update table infos or column infos.

        Args:
            this checkbox
        Return: nothing
    */
    var value = index + 1;
    if (value == 1)
        return; // the formation can't be modified by this popup

    var key_list = value == 2 ? table_attr.p_competence.keys : popup_column().competence.split(" ");
    var key_index = key_list.indexOf(competence);
    if (checkbox.checked && key_index == -1)
        key_list.push(competence);
    else if (!checkbox.checked && key_index != -1)
        key_list.splice(key_index, 1);
    else
        return; // No change
    key_list = this.trim_bad_keys(key_list);
    if (value == 2) {
        update_table_competence('keys', key_list);
    } else
        column_attr_set(popup_column(), "competence", key_list.join(" "));
    this.do_update = true;
};
CompetenceTable.prototype.toggle_observation = function (checkbox, competence, index) {
    /*  Desc:
            When a observation box is checked,
            uncheck all other checkboxes on this line and save the observation in the cell

        Arg:    this checkbox
        Return: nothing
    */
    // Uncheck others boxes
    var toggles = checkbox;
    while (toggles.className.split(" ").indexOf("toggle_observations") === -1)
        toggles = toggles.parentNode;
    toggles = toggles.childNodes;
    for (var i = 0; i < toggles.length; i++) {
        var label = toggles[i].childNodes[1];
        if (label !== checkbox)
            label.checked = false;
    }
    // Update cell infos
    var lin = the_current_cell.line_id;
    var col = popup_column().data_col;
    var cell_keys = lines[lin][col].value.split(/ +/);
    var key_find = false;
    for (var cell_index in cell_keys) {
        if (cell_keys[cell_index].split('o')[0] == competence) {
            if (checkbox.checked)
                cell_keys[cell_index] = competence + 'o' + index;
            else
                cell_keys.splice(cell_index, 1);
            key_find = true;
            break;
        }
    }
    if (!key_find && checkbox.checked)
        cell_keys.push(competence + 'o' + index);
    cell_keys.sort();
    cell_set_value_real(lin, col, cell_keys.join(' ').trim());
    update_cell_at(lin, col);
    the_current_cell.update_cell_headers();
    the_current_cell.jump(the_current_cell.lin, col - 1);
};
CompetenceTable.prototype.toggle = function (event) {
    if (event.target.tagName != 'INPUT')
        return;
    if (event.target.disabled)
        return;

    var competence = get_parent_attribute(event, "competence");
    var toggle = event.target, active_child;
    while (toggle.className.split(" ").indexOf("toggle") == -1) {
        active_child = toggle;
        toggle = toggle.parentNode;
    }
    var local_child = toggle.firstChild;
    var index = 0;
    while (local_child != active_child) {
        local_child = local_child.nextSibling;
        index++;
    }
    if (toggle.className.split(" ").indexOf('toggle_FUC') != -1)
        this.toggle_FUC(event.target, competence, index);
    else if (toggle.className.split(" ").indexOf('toggle_observations') != -1)
        this.toggle_observation(event.target, competence, index);
    event.stopPropagation();
};
function competences_test_cell(value, _column) {
    /*  Desc:
            Verify observations keys inputed in a competences cell by the user

        Args:
            str:    text in the cell
        Return:
            str:    same texte without invalid observations
    */
    if (value === '')
        return '';
    var cell_keys = value.split(/ +/);
    var returned = [];
    var errors = [];
    for (var index in cell_keys) {
        var key_end = key_last_word(cell_keys[index]);
        var key = key_subtract_word(cell_keys[index], 'o' + key_end[1]);
        if (key_end[0] != 'o' || key_end[1] >= competenceTable.catalog.vocabulary['o'].length)
            errors.push(_('ALERT_invalid_value') + cell_keys[index] + _("ALERT_no_observation_key"));
        else if (cell_keys[index].indexOf("+") === -1
            && competenceTable.catalog.choices.indexOf(key) == -1)
            errors.push(_('ALERT_invalid_value') + key + _("ALERT_lost_key"));
        else if (returned.includes(key))
            errors.push(_('ALERT_invalid_value') + key + _("ALERT_double_key"));
        else
            returned.push([key, key_end[0] + key_end[1]]);
    }
    if (errors.length) {
        alert_append(errors.join('\n'));
        return;
    }
    for (var r_index in returned)
        returned[r_index] = returned[r_index].join('');
    return returned.join(' ');
}
CompetenceTable.prototype.trim_bad_keys = function (keys) {
    /*  Returns only the keys found in the catalog
 
        Args:
            str : a list of keys
        Return:
            str : the same text without invalid keys
    */
    var keys_verified = [];
    for (var index in keys) { // XXX should create:   this.catalog.key_is_valid()
        var key = key_split(keys[index]);
        var is_valid = true;
        for (key_index in key) {
            var word = key[key_index];
            if (word[1] == null || word[0] !== '+' &&
                (!this.catalog.vocabulary[word[0]] || this.catalog.vocabulary[word[0]].length <= word[1])
            )
                is_valid = false;
        }
        if (key.length == 0)
            is_valid = false;
        if (is_valid)
            keys_verified.push(keys[index]);
    }
    return keys_verified.sort();
};
//----- Table set functions -----------------------------------------------------------------------

CompetenceTable.prototype.line_of_title = function (key, fuc = null, active_fuc = null) {
    var splitted = key_split(key);
    var word = splitted.at(-1);

    // FUC
    var categories = "";
    if (fuc && active_fuc) {
        categories += '<div class="toggle_FUC toggle">';
        for (var i in fuc) {
            categories += '<input type="checkbox" class="FUC_input"';
            if (!active_fuc[i])
                categories += ' disabled';
            if (fuc[i])
                categories += ' checked';
            categories += '>';
        }
        categories += '</div>';
    }

    // Subcomps Button
    var style = "cursor: default; text-align: center; flex: 1;";
    var subcompetence_button = "";
    if (splitted.length >= 4) {
        subcompetence_button += '<div class="grades_container">'
            + '<button class="subcompetence-add"'
            + 'onclick="competenceTable.subcompetence(\'' + key + '\')">+</button>'
            + '</div>';
        style += "margin-right: 130px";
    } else
        style += "margin: 0 10% 0 10%";

    return ['<div class="comp_line depth' + splitted.length + '" competence="' + key + '">'
        + categories + subcompetence_button
        + '<p class="line_desc" style="' + style + '">'
        + (this.catalog.vocabulary[word[0]] ?
            this.catalog.vocabulary[word[0]][word[1]] : this.catalog.vocabulary_name[word[0]])
        + '</p></div>'].join("");
}
CompetenceTable.prototype.line_description = function (desc, detailed, is_subcomp = false) {
    var html = [];
    var style = "", click_evt = "";
    if (!detailed) {
        style = 'style="white-space: nowrap"';
        click_evt = 'event';
    }
    if (is_subcomp) {
        var input_handler = 'competenceTable.subcomp_text_handler(event, `' + desc + '`)';
        html.push('<button class="title_edit_btn" onclick="competenceTable.edit_text_handler(event)">✎</button>',
            '<p class="editable line_desc" ', style, 'onclick="competenceTable.switch_details(', click_evt, ')"',
            ' onKeyDown="', input_handler, '" onBlur="', input_handler, '">', desc, '</p>');
    } else
        html.push('<p class="line_desc" ', style,
            ' onclick="competenceTable.switch_details(', click_evt, ')">', desc, '</p>');
    return html.join("");
}
CompetenceTable.prototype.line_of_toggles = function (key, checkable_FUC, checked_FUC, observations, notation, detailed = false) {
    /*  Desc:
            Return the HTML for a table line.

        Args:
            str:        a choice key
            list[bool]: three booleans to define if F, U and C are checkable
            list[bool]: three booleans to define if F, U and C are auto-checked
            list[bool]: booleans define if observations are checkable
            int:        the notation for this line, -1 if no observation
            boolean:    true if the line is detailed
        Return:
            str: a html printable line
    */
    // observations -> null, true, intitulé d'observation

    var html = [];
    // Description
    var cmp_word = key_last_word(key);
    var desc;
    var is_subcomp = (cmp_word[0] === "+");
    if (is_subcomp)
        desc = table_attr.p_competence.subcompetences[key.split("+")[0]][cmp_word[1]];
    else if (this.catalog.vocabulary[cmp_word[0]])
        desc = this.catalog.vocabulary[cmp_word[0]][cmp_word[1]];
    else
        desc = this.catalog.vocabulary_name[cmp_word[0]];

    html.push('<div class="comp_line depth', key_split(key).length, '" competence="', key, '">');

    // FUC
    var FUC_form = detailed ?
        '<div class="toggle_FUC" style="flex-direction: column">'
        : '<div class="toggle_FUC toggle">';
    var FUC_names = ['formation', 'ue', 'column'];
    html.push(FUC_form);
    for (var i in checked_FUC) {
        if (detailed)
            html.push(_("TIP_comp_" + FUC_names[i]));
        html.push('<input type="checkbox" class="FUC_input"');
        if (!checkable_FUC[i])
            html.push(' disabled');
        if (checked_FUC[i])
            html.push(' checked');
        html.push('>');
    }
    html.push('</div>');

    var subcompetence_button = is_subcomp ?
        hidden_txt('<button class="subcompetence-delete"'
            + 'onclick="competenceTable.subcompetence(\'' + key + '\')">×</button>',
            "Retirez la sous compétence")
        : hidden_txt('<button class="subcompetence-add"'
            + 'onclick="competenceTable.subcompetence(\'' + key + '\')">+</button>',
            "Ajouter une sous compétence");

    // Grades
    var toggle_grade_class = 'class="toggle_observations color_observations toggle"'
    var grades_form = detailed ?
        '<div style="display: flex">' + subcompetence_button
        + '<p style="width: 133px; padding-top: 5px; text-align: center">' + _("MSG_comp_grades") + '</p>'
        + this.line_description(desc, detailed, is_subcomp) + '</div>'
        + '<div ' + toggle_grade_class + ' style="flex-direction: column">'
        : '<div ' + toggle_grade_class + '>';
    html.push('<div class="grades_container">', grades_form);
    for (var box in observations) {
        html.push('<div class="grade_line"> <input type="checkbox" class="grade_input"');
        if (observations[box]) {
            if (box == notation)
                html.push(' checked');
        } else
            html.push(' disabled');
        html.push('>');
        if (detailed) {
            var grade_desc = typeof observations[box] === 'string' ?
                observations[box] : this.catalog.vocabulary['o'][box];
            var input_handler = 'competenceTable.grade_text_handler(event, `' + grade_desc + '`)';
            html.push('<button class="desc_edit_btn" onclick="competenceTable.edit_text_handler(event)">✎</button>',
                '<p class="editable" index=', box, ' onKeyDown="', input_handler, '" onBlur="', input_handler, '">',
                grade_desc,
                '</p>');
        }
        html.push('</div>');
    }
    html.push('</div></div>'); // grades container closure

    if (!detailed)
        html.push(this.line_description(desc, detailed, is_subcomp));
    html.push('</div>') // line closure

    return html.join("");
};
CompetenceTable.prototype.table = function (competences, focused_key = null) {
    /*  Generate the HTML table of competences which are displayed in the "observation_table" div.
 
        Args:
            list[(str, list[str])]: (key, [observations "o0" to "o4"]) -> make one line per key
        Return:
            str: HTML for the table of competences and observations.
    */
    if (!table_attr.p_competence["subcompetences"])
        update_table_competence("subcompetences", {});
    if (!table_attr.p_competence["grades_vocab"])
        update_table_competence("grades_vocab", {});
    if(competences.length == 0)
        return '<h3 style="margin: 20px">' + _("MSG_no_competences") + '</h2>';

    var allowed_keys = this.catalog.get_allowed_keys(ue);
    var allowed_keys_ue = this.catalog.get_allowed_keys_ue(ue);
    var html = [];
    var cell_value = lines[the_current_cell.line_id][popup_column().data_col].value.split(' ');
    var observed = {};
    for (var i in cell_value) {
        var key_observation = cell_value[i].split('o');
        observed[key_observation[0]] = key_observation[1];
    }
    // XXX active_observation = [true, true, true, true, true]; // No need to loop
    var active_observation = [];
    var checkable_FUC = [false,
        (i_am_the_teacher || table_attr.masters.length == 0) && table_attr.modifiable,
        column_change_allowed(popup_column())];
    for (var _i in this.catalog.vocabulary['o'])
        active_observation.push(true);

    var table_comp = table_attr.p_competence
    for (var i in competences) {
        var line_key = competences[i][0].split('o')[0];
        var is_subcomp = line_key.indexOf('+') !== -1;
        if (line_key.length <= 1)
            continue;   // Stop if competences[i] is not defined

        // Observations

        // No more use 'o' vocabulary
        // var grades_voc = table_comp.grades_vocab[key_last_word(line_key.split("+")[0]).join("")];
        // competences[i][1] is no more useful
        // var observations = competences[i][1];
        // for (var j in active_observation)
        //     active_observation[j] = grades_voc && grades_voc[j] ?
        //         grades_voc[j] : !observations || observations.indexOf('o' + str(j)) != -1;

        var grades_voc = table_comp.grades_vocab[key_last_word(line_key.split("+")[0]).join("")];
        var observations = this.catalog.get_allowed_keys_observations(line_key);
        for (var j in active_observation)
            // An observation is possible if the title is not empty
            active_observation[j] = (grades_voc && grades_voc[j]) ?
                grades_voc[j] : observations[j];

        // Filters
        var checked_FUC = [
            allowed_keys[line_key.split('+')[0]] !== undefined,
            table_comp.keys.indexOf(line_key) != -1 || allowed_keys_ue[line_key] !== undefined,
            popup_column().competence.split(' ').includes(line_key)
        ];

        // is Title ?
        var next = competences[Number(i) + 1];
        next = next ? next[0] : '';
        var is_subcomp = line_key.indexOf('+') !== -1;
        var is_title = key_split(line_key).length < 4 || (!is_subcomp && next.indexOf('+') !== -1)
        if (is_title)
            html.push(this.line_of_title(line_key));
        else {
            var tmp = checkable_FUC[1];
            checkable_FUC[1] = allowed_keys_ue[line_key] === undefined;
            html.push(this.line_of_toggles(line_key, checkable_FUC, checked_FUC,
                active_observation, observed[line_key], line_key == focused_key));
            checkable_FUC[1] = tmp;
        }
    }
    if (!focused_key)
        html.push('<div style="height:157px"></div>');

    return html.join('');
};
CompetenceTable.prototype.sort_keys_by_vocabs = function (keys) {
    // Define what keys sort
    var key_length = Infinity;
    var shortest_keys = []
    for(var key of keys) {
        var splitted = key_split(key);
        if(splitted.length < key_length) {
            key_length = splitted.length;
            shortest_keys = [key];
        } else if(splitted.length == key_length)
            shortest_keys.push(key)
    }

    // Sorting list by vocabularies
    var descs = []
    for(var key of shortest_keys) {
        var [voc, index] = key_last_word(key);
        var desc = (voc === "+") ?
            table_attr.p_competence.subcompetences[key.split("+")[0]][index]
            : this.catalog.vocabulary[voc][index];

        descs.push(text_simplify(desc.slice(0, 5)));
    }
    var result = [];
    for(var i in descs) {
        var j = 0;
        while(j < result.length && descs[i] > descs[result[j]])
            j++;
        result.splice(j, 0, i);
    }
    for(var i in result)
        result[i] = shortest_keys[result[i]];

    // Add other keys with recursive call
    for(var i in shortest_keys) {
        var subkeys = [];
        var next_index = shortest_keys[Number(i)+1] ?
            keys.indexOf(shortest_keys[Number(i)+1]) : keys.length;
        for(var j = keys.indexOf(shortest_keys[i]) + 1; j < next_index; j++)
            subkeys.push(keys[j]);

        if(subkeys.length > 0) {
            subkeys = this.sort_keys_by_vocabs(subkeys);
            var index = result.indexOf(shortest_keys[i]) + 1;
            result = result.slice(0, index).concat(subkeys).concat(result.slice(index));
        }
    }
    return result;
};
CompetenceTable.prototype.update_popup = function (focused_key = null) {
    //  Update the table, using FUC toggles and searchbar.
    var observation_table = document.getElementById('observation_table');
    var allowed_keys = this.catalog.get_allowed_keys(ue); // XXX Unstable allowed keys
    // Get filtered table lines
    var competences = [];
    switch (this.FUC_state) {
        case 0:
            for (var comp of this.catalog.choices)
                if (comp[0] != 'o' && comp[0] != 'z')
                    competences.push(comp);
            var subcomps = table_attr.p_competence.subcompetences;
            var i = 0;
            while(i < competences.length) {
                var subcomp = subcomps[competences[i]];
                if(subcomp)
                    for(var j in subcomp)
                        if(subcomp[j])
                            competences.splice(Number(i) + Number(j) + 1, 0, competences[i] + '+' + j);
                i++;
            }
            break;
        case 1:
            competences = Object.keys(allowed_keys);
            var subcomps = table_attr.p_competence.subcompetences;
            var i = 0;
            while(i < competences.length) {
                var subcomp = subcomps[competences[i]];
                if(subcomp)
                    for(var j in subcomp)
                        if(subcomp[j])
                            competences.splice(Number(i) + Number(j) + 1, 0, competences[i] + '+' + j);
                i++;
            }
            break;
        default:
            var table_keys; // Keys are sorted
            if (this.FUC_state == 2) {
                var allowed_keys_ue = this.catalog.get_allowed_keys_ue(ue); // XXX Unstable allowed keys
                table_keys = table_attr.p_competence.keys.slice();
                for (var i in allowed_keys_ue)
                    table_keys.push(i);
            }
            else
                table_keys = popup_column().competence.length == 0 ? [] : popup_column().competence.split(' ');

            // Class By sections
            var prec_key;
            for (var i in table_keys) {
                var splited = key_split(table_keys[i]);
                for (var word = 0; word < splited.length - 1; word++)
                    if (!prec_key || prec_key[word].join('') != splited[word].join('')) {
                        var new_key = splited.slice(0, word + 1);
                        for (var j in new_key)
                            new_key[j] = new_key[j].join('');
                        new_key = new_key.join('');
                        competences.push(new_key);
                    }
                competences.push(table_keys[i]);
                prec_key = splited;
            }
            break;
    }

    // Sorting competences
    competences = this.sort_keys_by_vocabs(competences.sort());

    // Searchbar
    var searchbar = text_simplify(this.searchbar.value);
    var checkable_observations = [];
    if (searchbar === '')
        for (var competence of competences)
            checkable_observations.push([competence]);
    else {
        // Add competences only if they have all the searched words
        var inputed = searchbar.split(/ +/);
        var stack = [];
        for (var competence of competences) {
            while (stack.length && competence.indexOf(stack.at(-1)) == -1)
                stack.pop();
            stack.push(competence);
            var words = key_split(competence);
            var key = words.at(-1);
            var desc_split = text_simplify(this.catalog.vocabulary[key[0]][key[1]]).split(/ +/);
            if (all_texts_in_list(inputed, desc_split, 4)) {
                for (ancestor of stack)
                    checkable_observations.push([ancestor]);
                stack = [];
            }
        }
    }

    // Limited observations
    for (var index in checkable_observations)
        checkable_observations[index].push(allowed_keys[checkable_observations[index][0]]); // XXX allowed keys problem here
    observation_table.innerHTML = this.table(checkable_observations, focused_key);
};
CompetenceTable.prototype.toggle_top_FUC = function (button, output) {
    //4 possible values for filter: 0->all, 1-> formation only, 2-> UE only, 3->one competence only
    if (button.getAttribute('checked') == 'true') {
        button.setAttribute('checked', '');
        this.FUC_state = 0;
    } else {
        for (var filter of this.filters)
            filter.setAttribute('checked', '');
        button.setAttribute('checked', 'true');
        this.FUC_state = output;
    }
    this.do_update = true;
};

//-------------------------------------------------------------------------------------------------

function get_parent_attribute(event, attr_name) {
    var value = event.target;
    while (!value.getAttribute(attr_name))
        value = value.parentNode;
    value = value.getAttribute(attr_name);
    return value;
}

function update_table_competence(section, value) {
    if (!section)
        return table_attr_set('competence', value);

    var table_comp = table_attr.p_competence;
    table_comp[section] = value;
    return table_attr_set('competence', table_comp);
}

const OBSERVATION_COLORS = ['#7E7979', '#EB707F', '#EEE592', '#BAE291', '#84A661', '#3E601B'];
function observation_color(index) {
    return OBSERVATION_COLORS[index] || '#7E7979';
}
function competences_recap(cell_keys, catalog = null, textual = false) {
    // Return a HTML observations recap with colored squares
    var observations = '';
    var vocab;
    if (catalog)
        vocab = catalog.vocabulary;

    for (var i in cell_keys) {
        if (cell_keys[i] === '')
            continue;
        var comp = cell_keys[i].split("o");
        var color = observation_color(comp[1]);
        var is_subcomp = comp[0].indexOf("+") !== -1;
        var style = is_subcomp ? 'width: 3px' : '';

        if (!vocab || is_subcomp) {
            observations += '<span class="an_observation"'
                + 'style="background-color:' + color + ';' + style + '"></span>';
        } else {
            var last_word = key_last_word(comp[0]);
            var desc = vocab[last_word[0]][last_word[1]];
            var grade_desc = vocab["o"][comp[1]];

            observations += textual ?
                ['<div style="margin: 5px; display: flex; flex-direction: row">',
                    '<p style="white-space: normal; width: 250px; border: 3px solid ', color, '; padding: 3px">', grade_desc, '</p>',
                    '<p style="white-space: normal; flex: 1; margin: 6px;">', desc, '</p></div>'].join("")
                :
                hidden_txt('<span class="an_observation" style="background-color:'
                    + color + ';' + style + '"></span>', grade_desc + " - " + desc);
        }
    }
    return observations;
}
function competences_format_suivi() {
    // student_catalog defined in TMP/0/COMPETENCES/catalog
    return competences_recap(DisplayGrades.value.split(/ +/),
        student_catalog, display_update.top == 'Textual');
}

var competenceTable = new CompetenceTable();

function competences_open(value, _column) {
    competenceTable.try_open();
    return value;
}