/*
    TOMUSS: The Online Multi User Simple Spreadsheet
    Copyright (C) 2023 Thierry EXCOFFIER, Universite Claude Bernard

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

// TODO : if the choice order is not defined by the student,
//        then do not take a random choice order.

var dispatch_debug = 'x12209214';

function dispatcher_format_suivi() {
    var key = DisplayGrades.ue.year + '/' + DisplayGrades.ue.semester + '/' + DisplayGrades.ue.ue
        + '/' + DisplayGrades.column.the_id + '/' + DisplayGrades.ue.line_id;
    dispatcher_format_suivi.infos[key] = {
        'column': DisplayGrades.column,
        'value': DisplayGrades.cell.value,
        'modifiable': cell_modifiable_on_suivi(),
        'cellbox': DisplayGrades.cellbox,
        'html_object': DisplayGrades.html_object
    };
    return '<button onclick="dispatcher_open(\'' + key + '\')">'
        + _("COL_TITLE_your_choice") + '</button>';
}

dispatcher_format_suivi.infos = {};

function dispatcher_set_target(element) {
    dispatcher_format_suivi.moving.style.top = element.offsetTop - 2 + 'px';
    if (dispatcher_format_suivi.target === element)
        return;
    if (dispatcher_format_suivi.td === element.parentNode)
        dispatcher_format_suivi.target = element;
}
function dispatcher_dragstart(event) {
    dispatcher_format_suivi.moving = event.target;
    dispatcher_format_suivi.td = event.target.parentNode;
    dispatcher_format_suivi.td.parentNode.parentNode.className = 'drag';
    dispatcher_format_suivi.moving.className = 'dragged';
    event.stopPropagation();
    event.preventDefault();
}
function dispatcher_dragover(event) {
    if (!dispatcher_format_suivi.moving)
        return;
    if (!event.buttons && event.type != 'touchmove') {
        dispatcher_dragend();
        return;
    }
    var target;
    if (event.type == 'touchmove') {
        var x = event.touches[0].clientX;
        var y = event.touches[0].clientY;
        for (var i in dispatcher_format_suivi.td.childNodes) {
            var div = dispatcher_format_suivi.td.childNodes[i];
            if (div.getBoundingClientRect && div !== dispatcher_format_suivi.moving) {
                var rect = div.getBoundingClientRect();
                if (x > rect.left && x < rect.left + rect.width
                    && y > rect.top && y < rect.top + rect.height) {
                    target = div;
                    break;
                }
            }
        }
    } else
        target = event.target;

    if (target && target.parentNode == dispatcher_format_suivi.td)
        dispatcher_set_target(target);
    event.stopPropagation();
    event.preventDefault();
}
function dispatcher_dragend() {
    if (!dispatcher_format_suivi.moving)
        return;
    dispatcher_format_suivi.target = undefined;
    dispatcher_format_suivi.td.parentNode.parentNode.className = 'nodrag';
    dispatcher_format_suivi.moving.className = '';
    dispatcher_format_suivi.moving = undefined;
}
function dispatcher_dragdrop(event) {
    if (!dispatcher_format_suivi.moving)
        return;
    if (dispatcher_format_suivi.target
        && dispatcher_format_suivi.target !== dispatcher_format_suivi.moving
        && event.target.parentNode == dispatcher_format_suivi.td
    ) {
        dispatcher_format_suivi.td.removeChild(dispatcher_format_suivi.moving);
        dispatcher_format_suivi.td.insertBefore(dispatcher_format_suivi.moving,
            dispatcher_format_suivi.target);
    }
    dispatcher_dragend();
}
function dispatch_record(button) {
    var td = document.getElementById('dispatcher').rows[1].cells;
    var content = [];
    for (var i = 0; i < td.length; i++) {
        var items = td[i].childNodes;
        var line = []
        for (var j = 0; j < items.length - 1; j++)
            line.push(items[j].textContent);
        content.push(line.join(' '));
    }
    var where = dispatch_record.key.split('/');
    var infos = dispatcher_format_suivi.infos[dispatch_record.key];
    DisplayGrades.html_object = infos.html_object;
    DisplayGrades.cellbox = infos.cellbox;
    DisplayGrades.html_object.value = content.join('\n');
    DisplayGrades.column = infos.column;
    _cell(infos.html_object,
        where[0] + '/' + where[1] + '/' + where[2] + '/cell/' + where[3] + '/' + where[4]
    );
    popup_close();
}
function dispatcher_open(key) {
    var infos = dispatcher_format_suivi.infos[key];
    var config = JSON.parse(infos.column.dispatcher_config);
    dispatch_record.key = key;


    var content = [
        '<style>',
        'DIV.import_export { position: fixed; top: 5% }',
        '.dispatcher TD { vertical-align: top;}',
        '.dispatcher TD DIV { border-top: 0px solid #FFF0 ; font-size: 100%; padding-top: 1em; padding-bottom: 0em; transition: top 0.2s }',
        'TBODY.nodrag TD DIV:hover { font-weight: bold; }',
        '.dispatcher TR:first-child { height: 1.1em }',
        '.dispatcher { table-layout: fixed }',
        '.dispatcher TD { position: relative; }',
        '.dispatcher TD DIV.dragged { position: absolute; font-weight: bold; border: 0px; padding-top: 0px; pointer-events: none; }',
        //'.drag TD DIV { transition: border-top-width 0.3s; }',
        '.dispatcher TR TD DIV:last-child { width: 100%; height: 0.5em }',
        '.export_div BUTTON { text-align: right; font-size: 100% }',
        // modifiable ? '.nodrag TD DIV:hover { font-weight: bold}' : '',
        infos.modifiable ? '.dispatcher TD DIV { cursor: move }' : '',
        '</style>',
        infos.modifiable ? _("MSG_dispatcher_drag") : '',
        '<table id="dispatcher" class="colored dispatcher"><tbody class="nodrag"><tr>'
    ];
    var sorted = infos.value.split('\n');
    for (var i in config.choices)
        content.push('<th>' + html(config.choices[i].title));
    content.push('</tr><tr>');
    for (var i in config.choices) {
        content.push('<td>');
        var done = {};
        var todo = ((sorted[i] || '') + ' ' + config.choices[i].values).split(/ +/);
        for (var j in todo) {
            if (!done[todo[j]]) {
                content.push('<div>' + html(todo[j]) + '</div>');
                done[todo[j]] = true;
            }
        }
        content.push('<div onmousedown="event.preventDefault()"></div>');
    }
    content.push('</tr></table>');
    create_popup('export_div', html(infos.column.title)
        + (infos.modifiable
            ? ' <button onclick="dispatch_record(this)">' + _("LABEL_save") + '</button>'
            : ''
        ),
        content.join(''), '', false);
    var dispatcher = document.getElementById('dispatcher');
    var divs = dispatcher.getElementsByTagName('DIV');
    if (infos.modifiable)
        for (var i = 0; i < divs.length; i++) {
            divs[i].addEventListener("mousedown", dispatcher_dragstart);
            divs[i].addEventListener("touchstart", dispatcher_dragstart);
            divs[i].addEventListener("mouseenter", dispatcher_dragover);
            divs[i].addEventListener("touchmove", dispatcher_dragover);
            divs[i].addEventListener("mouseup", dispatcher_dragdrop);
            divs[i].addEventListener("touchend", dispatcher_dragdrop);
        }
    // To hide the last moved item
    // dispatcher.style.height = 'calc(' + dispatcher.offsetHeight + 'px + 44px)';
    dispatcher.rows[1].style.height = dispatcher.rows[1].offsetHeight + 'px';
}
function DispatchConfig() {
    // Get current configuration from current page content
    var tables = popup_get_element().getElementsByTagName('TABLE');
    this.columns = [];
    this.choices = [];
    var cols = tables[0].rows[1].cells;
    for (var i = 0; i < cols.length; i++) {
        cols[i].style.color = '#000';
        var values = cols[i].firstChild.value.trim().split(/ *\n[ \n]*/);
        var column = [];
        this.columns.push(column);
        for (var j in values) {
            var nr_slots = Number(values[j].replace(/ .*/, ''));
            if (isNaN(nr_slots)) {
                cols[i].style.color = '#F00';
                continue;
            }
            column.push([nr_slots, values[j].replace(/^[^ ]* /, '')]);
        }
    }
    var choices = tables[1].rows;
    for (var i = 1; i < choices.length; i++) {
        var values = choices[i].cells[2].firstChild.value.trim();
        if (values === '')
            continue;
        values = values.split(/ +/);
        var for_columns = {};
        for (var j in this.columns)
            for (var k in this.columns[j]) {
                for (var v in values)
                    if (this.columns[j][k][1].indexOf(values[v]) != -1) {
                        for_columns[j] = true;
                        break;
                    }
                if (for_columns[j])
                    break;
            }
        this.choices.push(
            {
                'title': choices[i].cells[1].firstChild.value.trim(),
                'values': values,
                'repetition': Number(choices[i].cells[3].firstChild.value),
                'weight': Number(choices[i].cells[4].firstChild.value),
                'for_columns': for_columns
            });
    }
}
DispatchConfig.prototype.record = function () {
    var config = { 'columns': [], 'choices': [] };
    for (var i in this.columns) {
        var values = [];
        for (var j in this.columns[i])
            values.push(this.columns[i][j][0] + ' ' + this.columns[i][j][1]);
        config.columns.push(values.join('\n'));
    }
    for (var i in this.choices)
        config.choices.push({
            'title': this.choices[i].title,
            'values': this.choices[i].values.join(' '),
            'repetition': this.choices[i].repetition,
            'weight': this.choices[i].weight
        });
    column_attr_set(
        popup_column(),
        'dispatcher_config',
        JSON.stringify(config),
        undefined, true);
    popup_close();
};

function dispatcher_change() {
    document.getElementById("start_stop").disabled = true;
}

function dispatcher_config() {
    if (!the_current_cell.column.columns.length) {
        Alert("MSG_analyser_no_columns");
        return;
    }
    var config = JSON.parse(the_current_cell.column.dispatcher_config);
    var content = [
        '<style>',
        '.import_div TEXTAREA { height: 22em; font-size: 100%; font-family: monospace,monospace }',
        'DIV.import_div { top: 5%; right: 5%; }',
        'E { font-family: emoji ; margin-left: -0.7em ; }',
        '</style>',
        _('MSG_dispatcher_columns'),
        '<table class="colored"><tr>'];
    for (var i in the_current_cell.column.average_from)
        content.push('<th>' + html(the_current_cell.column.average_from[i]));
    content.push('</tr><tr>');
    for (var i in the_current_cell.column.average_from)
        content.push('<td><textarea oninput="dispatcher_change()">' + (config.columns[i] || '') + '</textarea>');
    content.push('</tr></table>');
    content.push(_('MSG_dispatcher_choices'));
    content.push('<table class="colored"><tr><th><th>');
    content.push(_("MSG_dispatcher_title"));
    content.push('<th>');
    content.push(_("MSG_dispatcher_values"));
    content.push('<th>');
    content.push(_("BEFORE_column_attr_repetition"));
    content.push('<th>');
    content.push(_("BEFORE_column_attr_weight"));
    content.push('<th><button id="start_stop" onclick="dispatch_do()">');
    content.push(_("MSG_dispatcher_do"));
    content.push('</button> <span id="nr_tries">0</span> ');
    content.push(_("MSG_dispatcher_do_tries"));
    content.push('</tr>');
    config.choices.push({});
    for (var i in config.choices) {
        var choice = config.choices[i];
        content.push('<tr>');
        content.push('<th>');
        content.push(Number(i) + 1);
        content.push('<td>');
        content.push('<input value="' + encode_value(choice.title || '') + '" oninput="dispatcher_change()">');
        content.push('<td>');
        content.push('<input value="' + encode_value(choice.values || '') + '" oninput="dispatcher_change()">');
        content.push('<td style="width:1em">');
        content.push('<input value="' + encode_value((choice.repetition || '1').toString()) + '" oninput="dispatcher_change()">');
        content.push('<td style="width:2em">');
        content.push('<input value="' + encode_value((choice.weight || '1').toString()) + '" oninput="dispatcher_change()">');
        if (i == config.choices.length - 1)
            content.push('<th style="width:20em"><button id="do_fill" disabled onclick="dispatch_do_fill()">'
                + _("MSG_dispatcher_do_fill") + '</button>');
        else
            content.push('<td id="stats' + i + '" style="white-space:nowrap">');
        content.push('</tr>');
    }
    content.push('</table>');
    dispatch_do.nr_tries = 0;
    dispatch_do.badness = [];
    var bad = [];
    for (var i in config.choices)
        bad.push(1e50);
    for (var i in the_current_cell.column.average_from)
        dispatch_do.badness.push(bad);
    dispatch_do.student_lines = undefined;
    dispatch_do.reject = {};
    create_popup('import_div',
        html(the_current_cell.column.title)
        + ' <button onclick="(new DispatchConfig()).record()"'
        + (column_change_allowed(the_current_cell.column) ? '' : ' disabled')
        + '>'
        + _('LABEL_save') + '</button> '
        ,
        content.join(''), '', false);
}
function DStudent(line, choices) {
    // choices = [  [ [word, repetition], [word, repetition], ...], ...  ]
    this.line = line;
    this.choices = choices;
    this.badness = []; // [ [col1choice1, col1Choice2, ...], [col2choice1, col2choice2, ...], ...]
    this.results = [];
    this.login = this.line[0].value;
}
DStudent.prototype.toString = function () {
    var choices = [left_justify(this.login, 8), ' ', ' '];
    for (var i in this.choices) {
        for (var j in this.choices[i])
            choices.push(this.choices[i][j][0] + '[' + this.choices[i][j][1] + ']');
        choices.push('→' + left_justify(this.results[i], 9) + ' ');
    }
    return choices.join('').replace(/&nbsp;/g, ' ');
};
DStudent.prototype.select_value = function (column, values, choices) {
    var badness = [];
    this.badness.push(badness);
    for (var i in this.choices) { // Every criterion
        if (!choices[i].for_columns[column])
            continue; // Preference not usable for this column
        var choice = this.choices[i];
        var found = false;
        for (var j = 0; j < choice.length; j++) { // Every choice in the sorted preference
            var new_values = [];
            var text = choice[j][0];
            for (var v in values)
                if (values[v].indexOf(text) != -1 && !dispatch_do.reject[this.login + ' ' + column + ' ' + values[v]])
                    new_values.push(values[v]);
            if (new_values.length) {
                values = new_values; // Only the possible new values
                badness.push(j ? Math.pow(2, j - 1) : 0);
                found = true;
                if (--choice[j][1] <= 0) {
                    if (choice[j][1] < 0) {
                        // The previous possible choices were not possible.
                        // It stays only a choice yet taken.
                        badness[badness.length - 1] = 1000000; // XXX
                        for (var k = 0; k < column; k++)
                            if (this.results[k] && this.results[k].indexOf(text) != -1) {
                                // No more reuse this choice
                                dispatch_do.reject[this.login + ' ' + k + ' ' + this.results[k]] = true;
                                break;
                            }
                    }
                    // No more allowed : become the last choice
                    choice.push(choice[j]);
                    choice.splice(j, 1);
                }
                break;
            }
        }
        if (!found) {
            badness.push(1000000000); // XXX
            this.results[column] = undefined;
            return;
        }
    }
    if (this.login === dispatch_debug) {
        console.log("======================================");
        console.log(['columns', column]);
        console.log(['values', JSON.stringify(values)]);
        console.log(['badness', JSON.stringify(this.badness)]);
        for (var i in this.choices)
            console.log(["Choices", i, this.choices[i]]);
    }
    this.results[column] = values[0]; // Assume only 1 possible or 0 if impossible
};
function dispatcher_init_students(student_lines, datacol_choice, config) {
    var students = [];
    for (var i in student_lines) {
        var line = student_lines[i];
        if (line[0].value === '')
            continue;
        var choices = line[datacol_choice].value.split(/ *\n[\n ]*/);
        var clean = [];
        for (var j in config.choices) {
            var default_order = config.choices[j].values;
            default_order = default_order.slice();
            // Shuffle order for not answering students
            for (var n = 0; n < 3; n++) {
                var k = Math.round(Math.random() * (default_order.length - 1));
                var tmp = default_order[n];
                default_order[n] = default_order[k];
                default_order[k] = tmp;
            }
            if (line[0].value === dispatch_debug)
                console.log([config.choices[j].values, default_order]);
            // Search the good line of choices.
            // May not be the expected one if a criterium
            // was added after student answering
            var items;
            for (var k = 0; k < choices.length; k++) {
                items = (choices[k] || '').trim();
                if (items !== '') {
                    items = items.split(/ +/);
                    if (myindex(default_order, items[0]) != -1)
                        break; // Seems to match
                }
            }
            if (k === choices.length)
                items = [];
            // Add missing items
            for (var k in default_order)
                if (myindex(items, default_order[k]) == -1)
                    items.push(default_order[k]);
            if (line[0].value === dispatch_debug)
                console.log(JSON.stringify(items));
            items_with_priority = [];
            for (var k in items) {
                items_with_priority.push([
                    items[k],
                    config.choices[j].repetition]);
            }
            clean.push(items_with_priority);
        }
        students.push(new DStudent(line, clean));
    }
    return students;
}
function dispatcher_init_counters(values, datacol) {
    var free_slot = {};
    // Initialise start values
    for (var j in values)
        free_slot[values[j][1]] = values[j][0];
    // Decremente on each usage
    for (var lin in lines) {
        var value = lines[lin][datacol].value;
        if (value === '')
            continue;
        if (free_slot[value] !== undefined)
            free_slot[value]--;
    }
    var possibles = [];
    for (var j in free_slot)
        if (free_slot[j] > 0)
            possibles.push(j);

    return [free_slot, possibles];
}
function dispatch_do_real(student_lines) {
    var config = dispatch_do.config;
    var datacol_choice = popup_column().data_col;
    var datacol_to_fill = popup_column().average_columns;
    var students = dispatcher_init_students(student_lines, datacol_choice, config);
    var badness = [];

    for (var k in datacol_to_fill) {
        var tmp = [];
        for (var kk in config.choices)
            tmp.push(0);
        badness.push(tmp);
    }

    // Initalise student informations
    for (var i in datacol_to_fill) { // Each column to fill
        var datacol = datacol_to_fill[i];
        free_slot_and_possibles = dispatcher_init_counters(config.columns[i], datacol);
        var free_slot = free_slot_and_possibles[0];
        var values = free_slot_and_possibles[1];
        // The student order must not change, it's 'reject' job to check other answers.
        // if(i != 0)
        //     students.sort(function (a, b) { return cmp_badness(a.badness, b.badness); });
        // else
        //     students.sort(function () { return Math.random() - 0.5; });
        for (var j in students) {
            if (students[j].line[datacol].value !== '')
                continue;
            students[j].select_value(i, values, config.choices);
            for (var k in students[j].badness[i])
                badness[i][k] += students[j].badness[i][k];
            var value = students[j].results[i];
            free_slot[value]--;
            if (free_slot[value] == 0)
                values.splice(values.indexOf(value), 1);
        }
    }
    return [badness, students];
}

function cmp_badness(a, b) {
    var sum_a = sum_b = 0;
    for (var i in a[0]) {
        var weight = dispatch_do.config.choices[i].weight;
        for (var j in a) {
            sum_a += weight * a[j][i];
            sum_b += weight * b[j][i];
        }
    }
    if (sum_a < sum_b)
        return -1;
    if (sum_a > sum_b)
        return 1;
    return 0;
}

function dispatch_run() {
    if (!dispatch_do.student_lines)
        return; // Stopped
    dispatch_do.nr_tries++;
    if (dispatch_do.nr_tries % 50 == 0) {
        dispatch_do.reject = {};
        dispatch_do.student_lines.sort(function () { return Math.random() - 0.5; });
    }
    badness_students = dispatch_do_real(dispatch_do.student_lines);
    //console.log(dispatch_do.nr_tries + ' ' + Object.keys(dispatch_do.reject).length);
    if (cmp_badness(badness_students[0], dispatch_do.badness) === -1) {
        console.log(JSON.stringify(badness_students[0]));
        dispatch_do.badness = badness_students[0];
        dispatch_do.students = badness_students[1];
        // for (var i in dispatch_do.students)
        //     console.log(dispatch_do.students[i].toString());
        for (var i in dispatch_do.config.choices) {
            var message = ' ';
            var nr_impossible = 0, nr_duplicate = 0;
            var bad = {};
            for (var j in dispatch_do.students) {
                var badness = dispatch_do.students[j].badness;
                for (var k in badness) {
                    var nb = badness[k][i];
                    if (bad[nb] === undefined)
                        bad[nb] = 1;
                    else
                        bad[nb]++;
                }
            }
            for (var j in dispatch_do.badness) {
                var nb = dispatch_do.badness[j][i];
                if (nb > 1000000000)
                    nr_impossible += Math.round(nb / 1000000000);
                else if (nb > 1000000)
                    nr_duplicate += Math.round(nb / 1000000);
            }
            if (nr_impossible > 0)
                message += nr_impossible + '  <e>☠</e> ';
            if (nr_duplicate > 0)
                message += nr_duplicate + '  <e>👎</e> ';
            for (var j = 0; j < 100; j++)
                if (bad[j] !== undefined) {
                    message += ' ' + bad[j] + '  ';
                    if (j == 0)
                        message += '<e>😀</e>';
                    else {
                        if (j != 1)
                            var n = Math.floor(Math.log2(j)) + 1;
                        else
                            var n = 1;
                        for (var k = 0; k < n; k++)
                            message += '<e>😠</e>';
                    }
                }
            document.getElementById('stats' + i).innerHTML = message;
        }
        dispatch_do.message = message;
    }
    if (dispatch_do.nr_tries % 10 == 0)
        document.getElementById('nr_tries').innerHTML = dispatch_do.nr_tries;
    setTimeout(dispatch_run, 1);
}
function dispatch_do() {
    if (dispatch_do.student_lines) {
        for (var i in dispatch_do.student_lines) {
            var cell = dispatch_do.student_lines[i][popup_column().data_col];
            cell.value = cell._save;
        }
        dispatch_do.student_lines = undefined;
        dispatch_do.config = undefined;
        document.getElementById('start_stop').textContent = _("MSG_dispatcher_do");
        document.getElementById('do_fill').removeAttribute('disabled');
    }
    else {
        dispatch_do.student_lines = [];
        for (var i in filtered_lines) {
            var datacol_to_fill = popup_column().average_columns;
            for (var j in datacol_to_fill)
                if (filtered_lines[i][datacol_to_fill[j]].value === '') {
                    dispatch_do.student_lines.push(filtered_lines[i]);
                    // Save because choices are modified by dispatch_modify_random
                    filtered_lines[i][popup_column().data_col].save();
                    break;
                }
        }
        document.getElementById('start_stop').textContent = _("MSG_dispatcher_do_stop");
        document.getElementById('do_fill').setAttribute('disabled', 'true');
        dispatch_do.config = new DispatchConfig();
        dispatch_run();
    }
}
dispatch_do.student_lines = undefined;
function dispatch_random_init() {
    // Fill random choices for debugging purpose
    var config = new DispatchConfig();
    var data_col = popup_column().data_col;
    for (var i in filtered_lines) {
        var choices = [];
        for (var j in config.choices) {
            var items = config.choices[j].values.slice();
            items.sort(function () { return Math.random() - 0.5; });
            choices.push(items.join(' '));
        }
        filtered_lines[i][data_col].value = choices.join('\n');
    }
}
function dispatch_do_fill() {
    var datacols = popup_column().average_columns;
    alert_append_start();
    for (var i in dispatch_do.students) {
        var student = dispatch_do.students[i];
        var lin_id = login_to_line_id(student.login);
        for (var j in datacols)
            if (student.results[j] !== undefined)
                cell_set_value_real(lin_id, datacols[j], student.results[j]);
    }
    for (var j in datacols)
        columns[datacols[j]].need_update = true;
    columns[popup_column().data_col].need_update = true;
    alert_append_stop();
    popup_close();
    update_columns();
    table_init();
    table_fill(false, true);
}