/*
  TOMUSS: The Online Multi User Simple Spreadsheet
  Copyright (C) 2019 Thierry EXCOFFIER, Universite Claude Bernard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

mathjaxurl = "https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js?config=TeX-MML-AM_CHTML";

function Rand(seed) {
    this.seed = 1;
    for (var i = 0; i < seed.length; i++)
        this.seed = (2 * this.seed + seed.charCodeAt(i)) & 0x7fffffff;
    this.int();
}

Rand.prototype.int = function () {
    return this.seed = (1664525 * this.seed + 1013904223) & 0x7FFFFFFF;
};

function uncheck_if_checked(label) {
    // Use setTimeout to let the default behaviour some time to run
    var input = label.getElementsByTagName('INPUT')[0];
    if (input.checked && !input.disabled)
        setTimeout(function () { input.checked = false; }, 100);
    setTimeout(function () { mcq_from_element(input).update_html_student(); }, 200);
}

function mcq_use_bad_answer_value(x) {
    var p = x.option('progressive');
    return p != '±' && p != 'AMC';
};

/**************************************************************************/

function MCQ_Choice(args, parent, key) {
    this.choice = args[0];
    this.options = args[1];
    this.parent = parent;
    this.key = key;
    this.full_key = this.parent.full_key + '\001' + key;
    if (this.options === undefined)
        this.options = {};
}

MCQ_Choice.prototype.html_student = function (hide_answers) {
    var choice = this.choice;
    if (!this.option('html_choice'))
        choice = html(choice);
    choice = this.obfuscate(choice);
    var type;
    if (this.option('radio'))
        type = 'radio';
    else
        type = 'checkbox';
    var show_answers = this.parent.options.grade !== undefined && !hide_answers;
    var checked = this.parent.parent.checked[this.parent.key + '=' + this.key];
    var grade = show_answers && this.parent.options.grade[0][0];
    if (!show_answers
        && (this.parent.parent.options.gui == 'print'
            || this.parent.parent.options.gui == 'preview'
        ))
        checked = false;

    var expected = show_answers && (myindex(grade, this.key) != -1
        || myindex(grade, Number(this.key)) != -1);
    var color = (show_answers
        ? (!!expected == !!checked && checked ?
            'border:3px solid #0F0;' : '')
        +
        (!expected & checked ? 'border:1px solid #F00;' : '')
        : ''
    );
    return '<label onmouseup="uncheck_if_checked(this)">'
        + '<table class="mcqchoice">'
        + '<tr><td>'
        + '<input type="' + type
        + '" name="' + this.parent.full_key
        + '"' + (show_answers || !this.parent.parent.modifiable ? ' disabled' : '')
        + ' value="' + this.key + '"'
        + (checked ? ' checked' : '')
        + '>'
        + '<td style="'
        + (show_answers && !expected ? 'opacity: 0.3;' : '')
        + color
        + encode_value(this.option('choice_CSS') || '')
        + '">'
        + choice
        + (show_answers && (expected || checked)
            ? ' <span class="expected">'
            + (expected && checked
                ? _("LABEL_MCQ_expected_yes")
                : (expected && !checked
                    ? _("LABEL_MCQ_expected")
                    : _("LABEL_MCQ_expected_no")
                )
            ) + '</span>'
            : '')
        + '</td></table></label>\n';
};

MCQ_Choice.prototype.json = function () {
    if (Object.keys(this.options).length == 0)
        return [this.choice];
    return [this.choice, this.options];
};

MCQ_Choice.prototype.html_table = function (checked) {
    var answer = '';
    var key = this.parent.key + '=' + this.key;
    var active = '';
    if (mcq_filter_active(key))
        active = ' ×' + _('filtre.png');
    if (this.option('show_answers')) {
        var student = this.parent.parent.checked[key];
        if (student)
            if (!checked)
                answer = '#F00"';
            else
                answer = '#0F0"';
        else
            answer = '#DDD';
        answer = ' style="border: 2px solid ' + answer + '"';
    }
    return '<tr id="' + this.full_key + '" class="mcq_choice">'
        + '<td><span' + (this.previous ? '' : ' style="opacity: 0.3"')
        + '>↑</span>&nbsp;<span' + (this.next ? '' : ' style="opacity: 0.3"')
        + '>↓</span>'
        + '<td><input type="checkbox"' + (checked ? ' checked' : '')
        + '>'
        + '<td' + answer + '><input value="' + encode_value(this.choice)
        + '">'
        + (this.parent.parent.options.stats
            ? '<td><span onclick="mcq_filter(\'' + key + '\')">' + this.nr_checked + active
            : '')
        + '</tr>';
};

MCQ_Choice.prototype.option = function (name) {
    if (this.options[name] === undefined)
        return this.parent.option(name);
    return this.options[name];
};

MCQ_Choice.prototype.next_priority = function () {
    if (!this.next)
        return;
    if (this.next.next)
        this.options.priority = (this.next.options.priority
            + this.next.next.options.priority) / 2;
    else
        this.options.priority = this.next.options.priority + 1;
}

MCQ_Choice.prototype.previous_priority = function () {
    if (!this.previous)
        return;
    if (this.previous.previous)
        this.options.priority = (this.previous.options.priority
            + this.previous.previous.options.priority) / 2;
    else
        this.options.priority = this.previous.options.priority - 1;
}

/**************************************************************************/

function MCQ_Question(args, parent, key) {
    this.question = args[0];
    var choices = args[1];
    this.options = JSON.parse(JSON.stringify(args[2]));
    this.parent = parent;
    this.key = key;
    this.full_key = this.parent.key + '\001' + key;
    if (this.options == undefined)
        this.options = {};
    this.choices = {};
    for (var key in choices)
        this.choices[key] = new MCQ_Choice(choices[key], this, key);
    this.sorted_choices = sorted_dictionary(this.choices);
    var sc = this.sorted_choices;
    for (var i = 0; i < sc.length; i++) {
        sc[i].previous = i ? sc[i - 1] : undefined;
        sc[i].next = i != sc.length - 1 ? sc[i + 1] : undefined;
        sc[i].options.priority = Number(i);
    }
}

MCQ_Question.prototype.get_student_choices = function (rand) {
    return sorted_dictionary(
        this.choices,
        this.option('shuffle_choices') && rand)
};

function obfuscate(txt) {
    if (txt.length <= 4) // Do not touch 2 firsts and 2 lasts
        return txt;
    var rand = new Rand(txt + millisec());
    var t = [txt.substr(0, 2)];
    for (var i = 2; i < txt.length - 1; i++) {
        if (rand.int() % 3 == 0)
            t.push('<H>'
                + txt.substr(rand.int() % txt.length, 1).replace(/[&<> ]/, ' ')
                + '</H>');
        t.push(txt.substr(i, 1));
    }
    t.push(txt.substr(txt.length - 1));
    return t.join('');
}

MCQ_Question.prototype.obfuscate = function (txt) {
    if (this.option('obfuscate') != 2)
        return txt;
    if (!this.option('math') == 'no')
        return txt.replace(/(>|^|;)([^<>&;]*)(<|&|$)/g, obfuscate);
    if (txt.match(/\$\$/))
        return txt;
    return txt.replace(/(>|^|\\\)|\\\]|;)([^<>\\&;]*)(<|\\\(|\\\[|&|$)/g, obfuscate);
}
MCQ_Choice.prototype.obfuscate = MCQ_Question.prototype.obfuscate;

MCQ_Question.prototype.html_student = function (hide_answers) {
    var question = this.question;
    if (!this.option('html_question'))
        question = html(question).replace(/\n\n/g, '<br>');
    question = this.obfuscate(question);
    var use_bad = mcq_use_bad_answer_value(this);

    var s = ['<section class="mcqquestion" id="', this.full_key, '" style="',
        this.option('obfuscate') ? 'user-select: none;' : '',
        encode_value(this.option('question_CSS') || ''),
        '">',
        '<div role="heading">'];
    if (this.option('scoring_scale'))
        s.push(
            (this.option('good_answer') != this.parent.option('good_answer')
                ? '<span class="grading">' + _('MSG_mcq_good_answer') + ':' + _('MSG_mcq_good_answer_' + this.option('good_answer')) + '</span>' : ''),
            (this.option('no_answer') != this.parent.option('no_answer')
                ? '<span class="grading">' + _('MSG_mcq_no_answer') + ':' + _('MSG_mcq_no_answer_' + this.option('no_answer')) + '</span>' : ''),
            ((this.option('bad_answer') != this.parent.option('bad_answer') && use_bad)
                ? '<span class="grading">' + _('MSG_mcq_bad_answer') + ':' + _('MSG_mcq_bad_answer_' + this.option('bad_answer')) + '</span>' : ''),
            (this.option('progressive') != this.parent.option('progressive')
                ? '<span class="grading">' + hidden_txt(_('MSG_mcq_progressive') + ':' + _('MSG_mcq_progressive_' + this.option('progressive')),
                    _('MSG_mcq_progressive_' + this.option('progressive') + '_help')) + '</span>' : ''),
            hide_answers || !this.option('grade') ? '' : '<span class="question_grade">' + this.grade().toFixed(2) + '</span>'
        );
    s.push(question, '</div>');
    var choices = this.get_student_choices(this.parent.get_student_rand(undefined, this.key));
    for (var i in choices)
        s.push(choices[i].html_student(hide_answers));
    s.push('</section>\n');

    return s.join('');
};

MCQ_Question.prototype.html_table = function () {
    var stats = this.parent.options.stats;
    if (stats) {
        stats = ['<td><table class="stats colored"><tr><th>' + _("B_Note")];
        var grades = [];
        for (var k in this.grade_sum)
            grades.push(k);
        grades.sort(function (a, b) { return Number(a) - Number(b); });
        for (var k in grades)
            stats.push('<td>' + Number(grades[k]).toFixed(2).replace('.00', ''));
        stats.push('</tr><tr><th>' + _("B_s_nr_"));
        for (var k in grades)
            stats.push('<td>' + this.grade_sum[grades[k]]);
        stats.push('</tr></table>');
        stats = stats.join('');
    }
    else
        stats = '';
    var s = [
        '<table id="' + this.key + '" ',
        'onclick="mcq.update_question(this, event)" ',
        'onkeyup="mcq.update_question(this, event)" ',
        'onchange="mcq.update_question(this, event)" ',
        'ondrop="mcq.update_question(this, event)" ',
        'onblur="mcq.update_question(this, event)">',
        '<tr class="question_menu"><td colspan="3">',
        this.parent.options_html(this.parent.question_options, this.parent.options, this.options),
        '<td>', _("TITLE_table_attr_statistics"),
        '</tr>',
        '<tr class="question"><td><span',
        (this.previous ? '' : ' style="opacity: 0.3"'),
        '>↑</span>&nbsp;<span',
        (this.next ? '' : ' style="opacity: 0.3"'),
        '>↓</span>',
        '<td colspan="2"><textarea rows="',
        Math.max(this.question.split('\n').length + 1, 3), '">',
        html(this.question), '</textarea>',
        stats,
        '</tr>'
    ];
    var checked = this.get_checked();

    for (var i in this.sorted_choices)
        s.push(this.sorted_choices[i].html_table(checked[this.sorted_choices[i].key]));
    s.push('<tr><td><td colspan="2" style="text-align:left"><button');
    if (this.parent.freezed)
        s.push(' disabled');
    s.push('>+</button><td></tr></table>');
    return s.join('');
};

MCQ_Question.prototype.get_checked = function () {
    // Analyse only the first grading option
    var checked = {};
    if (this.options.grade && this.options.grade.length == 1)
        for (var j in this.options.grade[0][0])
            checked[this.options.grade[0][0][j]] = true;
    return checked;
};

MCQ_Question.prototype.multiple_choice = function () {
    var formula = this.option('grade');
    if (!formula || formula.length == 0)
        return false;
    return formula[0][0].length > 1;
};

MCQ_Question.prototype.grade_no_answer = function () {
    if (isNaN(this.option('no_answer')))
        return - Number(this.option('good_answer')) / (this.sorted_choices.length - 1);
    return Number(this.option('no_answer'));
};

MCQ_Question.prototype.grade = function () {
    var formula = this.option('grade');
    var ok, f0, f1, f2;
    var nr_checked = 0;
    var checked = {};
    for (var choice in this.choices) {
        if ((checked[choice] = this.parent.checked[this.key + '=' + choice]))
            nr_checked++;
    }
    if (nr_checked == 0)
        return this.grade_no_answer();

    for (var f in formula) {
        f0 = formula[f][0];
        ok = true;
        for (var require in f0)
            if (!checked[f0[require]]) { ok = false; break; }
        if (!ok)
            continue;
        f1 = formula[f][1];
        for (var reject in f1)
            if (checked[f1[reject]]) { ok = false; break; }
        if (ok) {
            f2 = formula[f][2];
            if (f2 !== undefined && f2 !== null)
                return f2;
            else
                return Number(this.option('good_answer'));
        }
    }
    if (formula.length && this.option('progressive') != 'bad') {
        var nr_checked_good = 0, nr_checked_bad = 0;
        f0 = formula[0][0];
        for (var require in f0)
            if (checked[f0[require]])
                nr_checked_good++;
        f1 = formula[0][1];
        for (var reject in f1)
            if (checked[f1[reject]])
                nr_checked_bad++;
        if (this.option('progressive') == '±' && nr_checked)
            return Number(this.option('good_answer'))
                * (nr_checked_good / f0.length - nr_checked_bad / f1.length);
        if (this.option('progressive') == 'AMC' && nr_checked) {
            var nr_unchecked_bad = f0.length - nr_checked_good;
            var nr_unchecked_good = f1.length - nr_checked_bad;
            return Number(this.option('good_answer'))
                * (nr_checked_good + nr_unchecked_good
                    - nr_checked_bad - nr_unchecked_bad) / this.sorted_choices.length;
        }
        if (nr_checked_good > 0 && nr_checked_bad == 0) {
            if (this.option('progressive') == 'good')
                return Number(this.option('good_answer'));
            if (this.option('progressive') == 'progressive')
                return nr_checked_good * Number(this.option('good_answer')) / f0.length;
            return this.grade_no_answer();
        }
    }
    if (isNaN(this.option('bad_answer')))
        return - Number(this.option('good_answer')) / (this.sorted_choices.length - 1);
    return Number(this.option('bad_answer'));
};

MCQ_Question.prototype.json = function () {
    var choices = {};
    for (choice in this.choices)
        if (this.choices[choice].choice !== '')
            choices[choice] = this.choices[choice].json();
    return [this.question, choices, this.options];
};

MCQ_Question.prototype.amc = function () {
    function multiline(txt) {
        return txt.trim().replace(/\n([^ ])/g, '\n $1');
    }
    function options(obj) {
        var opts = obj.options;
        var s = ['id=' + obj.key];
        for (var opt in opts) {
            if (opts[opt] === undefined)
                continue;
            if (opt == 'grade' || opt == 'priority' || opt == 'radio')
                continue;
            if (opt == 'no_answer')
                s.push('v=' + obj.parent.encode_amc(opts[opt]));
            else if (opt == 'bad_answer')
                s.push('m=' + obj.parent.encode_amc(opts[opt]));
            else if (opt == 'good_answer')
                s.push('b=' + obj.parent.encode_amc(opts[opt]));
            else if (opt == 'shuffle_choices' && !opts[opt])
                s.push('ordered');
            else
                s.push(opt + '=' + opts[opt]);
        }
        return s.join(',');
    }
    var checked = this.get_checked();
    var t = [this.options.radio === 0 ? '*' : '',
        '*[', options(this), '] ', multiline(this.question), '\n'];
    for (var choice in this.sorted_choices) {
        choice = this.sorted_choices[choice];
        if (choice.choice !== '')
            t.push((checked[choice.key] ? '+' : '-') + '[' + options(choice)
                + '] '
                + multiline(choice.choice)
                + '\n');
    }
    return t.join('');
};

MCQ_Question.prototype.add_choice = function () {
    var last_key = 0;
    for (choice in this.choices) {
        choice = Number(choice.substr(1));
        if (choice > last_key)
            last_key = choice;
    }
    var key = this.parent.get_new_id(this.choices);
    this.choices[key] = new MCQ_Choice([_("MSG_MCQ_choice"),
    { 'priority': this.sorted_choices[this.sorted_choices.length - 1].options.priority + 1 }],
        key);
};

MCQ_Question.prototype.option = MCQ_Choice.prototype.option;
MCQ_Question.prototype.next_priority = MCQ_Choice.prototype.next_priority;
MCQ_Question.prototype.previous_priority = MCQ_Choice.prototype.previous_priority;

/**************************************************************************/

function MCQ(args, initial_value, key) {
    this.mcq_options = [
        ['gui', ['beginner', 'amc', 'wooclap', 'expert', 'preview', 'print']],
        ['shuffle_questions', [0, 1]],
        ['same_questions', [0, 1]],
        ['show_answers', [0, 1]],
        ['math', ['no', 'mathjax']],
        ['scoring_scale', [0, 1]]
    ];
    this.question_options = [
        ['good_answer', [0, 0.5, 1, 1.5, 2, 2.5, 3, 4]],
        ['bad_answer', ['0', '-0.5', '-1', '-1/(#-1)']],
        ['no_answer', ['0', '-0.5', '-1', '-1/(#-1)']],
        ['progressive', ['bad', 'no', 'progressive', 'good', '±', 'AMC']],
        ['shuffle_choices', [0, 1]],
        ['html_question', [0, 1]],
        ['html_choice', [0, 1]],
        ['group', []],
        ['obfuscate', [0, 1, 2]]
    ];

    var questions = args[0];
    this.options = JSON.parse(JSON.stringify(args[1]));
    this.key = key;

    this.table_rand = new Rand(millisec().toString());
    if (key)
        this.rand = new Rand(this.key || ''); // Student

    this.update_checked(initial_value);
    MCQ.dict[this.key] = this;
    if (Object.keys(questions).length == 0) {
        questions = {
            'q1': [_("MSG_MCQ_question"), this.empty_choices(), { priority: 1, grade: [] }],
            'q2': [_("MSG_MCQ_question"), this.empty_choices(), { priority: 2, grade: [] }]
        }
    }
    var defaults = {
        'radio': 1, 'gui': 'beginner',
        'shuffle_questions': 1, 'shuffle_choices': 1,
        'same_questions': 0,
        'good_answer': 1, 'no_answer': 0, 'bad_answer': '-1/(#-1)',
        'progressive': 'bad', 'group': '', 'obfuscate': 1,
        'choice_CSS': '', 'question_CSS': '', 'scoring_scale': 1
    };
    if (this.options.same_questions === undefined)
        this.options.same_questions = this.options.shuffle_questions != 0 ? 0 : 1;
    for (var i in defaults)
        if (this.options[i] === undefined)
            this.options[i] = defaults[i];
    this.questions = {};
    for (var key in questions)
        this.questions[key] = new MCQ_Question(questions[key], this, key);
    this.sorted_questions = sorted_dictionary(this.questions);
    var sq = this.sorted_questions;
    for (var i = 0; i < sq.length; i++) {
        sq[i].previous = i ? sq[i - 1] : undefined;
        sq[i].next = i != sq.length - 1 ? sq[i + 1] : undefined;
        sq[i].options.priority = Number(i);
    }
}
MCQ.dict = {};

MCQ.prototype.multiple_choice = function () {
    for (var key in this.sorted_questions)
        if (this.sorted_questions[key].multiple_choice())
            return true;
};

MCQ.prototype.button_label = function () {
    return this.options.show_answers ? _("MSG_MCQ_expected") : _("MSG_MCQ_answer");
};

MCQ.prototype.update_checked = function (checked) {
    this.checked = {};
    if (checked && checked !== '') {
        checked = checked.split(/&/g);
        for (var i in checked)
            this.checked[checked[i]] = true;
    }
};

MCQ.prototype.option = function (name) {
    return this.options[name];
};

MCQ.prototype.student_select_group = function (questions) {
    var group, question;
    var counters = {};
    var selected_questions = [];
    for (var key in questions) {
        question = questions[key];
        if (!question._take)
            continue;
        group = question.options.group || '';
        if (counters[group] === undefined)
            counters[group] = 1;
        else
            counters[group]++;
        if (this.options.nr_question_in_group === undefined
            || this.options.nr_question_in_group[group] === undefined
            || counters[group] <= this.options.nr_question_in_group[group])
            selected_questions.push(question);
        else
            question._take = false;
    }
    return selected_questions;
};

MCQ.prototype.student_questions = function (rand) {
    var questions;
    for (var q in this.questions)
        this.questions[q]._take = true;
    if (this.options.same_questions || !this.options.shuffle_questions) {
        questions = sorted_dictionary(
            this.questions,
            this.options.same_questions
                ? new Rand(this.year + '/' + this.semester + '/' + this.ue + '/' + this.column.the_id)
                : rand
        );
        this.student_select_group(questions);
    }
    questions = sorted_dictionary(this.questions,
        this.options.shuffle_questions && rand);
    return this.student_select_group(questions);
};

MCQ.prototype.html_student = function (hide_answers) {
    var use_bad = mcq_use_bad_answer_value(this);
    var t = ['<div id="' + this.key + '" style="font-weight: normal"'
        + ' onclick="mcq_from_element(this).click_student(event)">',
    (this.column.comment ? '<p>' + html(this.column.comment) + '</p>' : '')];
    if (this.options.scoring_scale)
        t.push(
            _("MSG_MCQ_scoring"),
            '<ul>',
            '<li>' + _('MSG_mcq_good_answer') + ' : ' + _('MSG_mcq_good_answer_' + this.options.good_answer),
            '<li>' + _('MSG_mcq_no_answer') + ' : ' + _('MSG_mcq_no_answer_' + this.options.no_answer),
            use_bad
                ? '<li>' + _('MSG_mcq_bad_answer') + ' : ' + _('MSG_mcq_bad_answer_' + this.options.bad_answer)
                : '',
            '<li>' + _('MSG_mcq_progressive') + ' : ' + _('MSG_mcq_progressive_' + this.options.progressive + '_help'),
            '</ul><br>');
    var questions = this.student_questions(this.get_student_rand());
    for (var key in questions)
        t.push(questions[key].html_student(hide_answers));
    t.push('</div>');
    return t.join('\n');
};

MCQ.prototype.update_math = function () {
    try { MathJax.Hub.Queue(["Typeset", MathJax.Hub]); }
    catch (e) { try { this.window.MathJax.Hub.Startup.onload(); } catch (e2) { }; }
};

MCQ.prototype.update_html_student = function () {
    var i;

    if (this.options.math == 'mathjax') {
        var preview = document.getElementById("mcqpreview");
        var doc = preview ? preview.contentDocument : document;
        var script = doc.getElementById("mcqmath");
        if (script)
            this.update_math();
        else {
            if (preview)
                this.window = preview.contentWindow;
            else
                this.window = window;
            script = doc.createElement('SCRIPT');
            script.id = "mcqmath";
            script.async = '';
            script.src = mathjaxurl;
            script.onload = this.update_math.bind(this);
            doc.getElementsByTagName('HEAD')[0].appendChild(script);
        }
    }
    var questions = this.student_questions(this.get_student_rand());
    for (var key in questions) {
        var e = document.getElementById(questions[key].full_key);
        if (!e)
            return; // Popup not opened
        var checkboxes = e.getElementsByTagName('INPUT');
        for (i = 0; i < checkboxes.length; i++) {
            if (checkboxes[i].checked) {
                e.className = "mcqquestion";
                break;
            }
        }
        var tag = document.getElementById('TAG/' + e.id);
        var tagclass;
        if (i == checkboxes.length) {
            e.className = "mcqquestion unanswered";
            tagclass = 'mcqtag unanswered';
        }
        else
            tagclass = 'mcqtag';
        if (tag) // Safari bug
            tag.className = tagclass;
    }
};

MCQ.prototype.options_html = function (mcq_options, globals, locals) {
    var t = [], disabled;
    var global_option = globals === locals;
    for (var i in mcq_options) {
        var values = mcq_options[i][1];
        var name = mcq_options[i][0];
        var not_defined = (locals[name] === undefined || locals[name] === '') && !global_option;
        t.push('<div class="vertical');
        if (not_defined)
            t.push(' use_default');
        t.push('" x="' + locals[name]);
        t.push('">');
        t.push(_('MSG_mcq_' + name));
        t.push('<br>');
        if (values.length == 0) {
            t.push('<input class="group" get_completion_list="mcq_group_completion" name="');
            t.push(name);
            t.push('" value="');
            t.push(encode_value((locals[name] === '' || locals[name] === undefined
                ? globals[name] : locals[name]).toString()));
            t.push('" onfocus="element_focused=this" onblur="mcq.mcq_option_change(this)" onchange="mcq.mcq_option_change(this)"></div>');
            continue;
        }
        if (this.freezed && (name == 'shuffle_questions'
            || name == 'same_questions'
        ))
            disabled = ' disabled';
        else
            disabled = '';
        t.push('<select name="'
            + name + '"' + disabled
            + ' onchange="mcq.mcq_option_change(this)">');
        var first = true;
        for (var j in values) {
            var value = values[j];
            t.push('<option');
            if (locals[name] == value
                || global_option && first && globals[name] === undefined
            )
                t.push(' selected');
            t.push(' value="' + value + '">');
            t.push(_('MSG_mcq_' + name + '_' + value));
            t.push('</option>');
            first = false;
        }
        if (!global_option) {
            t.push('<optgroup label="⤒"></optgroup><option value=" undefined "');
            if (not_defined)
                t.push(' selected')
            t.push('>' + _('MSG_mcq_' + name + '_' + (globals[name] || values[0]))
                + '</option>');
        }
        t.push('</select>');
        t.push('</div>');
    }
    return t.join('');
}

MCQ.prototype.html_table = function () {
    this.freezed = this.column.is_visible();
    for (var i in lines)
        if (lines[i][this.column.data_col].comment !== '')
            this.freezed = true;

    var t = [
        '<style>',
        'DIV.import_export.mcq_editor { left:10%; right:2em; top:5%; bottom:5%; border: 2px solid #000; z-index:30 }',
        '#mcq_content > TABLE { width: 100% ; margin-bottom: 1em ; border-spacing: 0px }',
        '#mcq_content > TABLE[id] { border-spacing: 0.2em }',
        '#mcq_content TR.question_menu > TD { vertical-align: bottom }',
        '.mcq_choice TD:first-child, .mcq_choice TD:nth-child(2)',
        '   { width: 1% ; text-align: center ; white-space: pre; padding-right: 0.5em }',
        '#mcq_content TR.question TD { background: #EEE ; vertical-align: top }',
        '#mcq_content TD:nth-child(3) { vertical-align: top ; width: 85%; white-space: pre }',
        '#mcq_content TD:nth-child(4) { width: 14%; }',
        '#mcq_content TABLE.stats TD { text-align: right; padding: 0.2em; width: 3em }',
        '#mcq_content SPAN { color: #00F; cursor: pointer ; font-weight: normal }',
        '#mcq_content TABLE SELECT { width: initial }',
        '#mcq_content TEXTAREA { background: inherit ; border: 0px; box-sizing: border-box; width:100%; height: 99% }',
        '#mcq_content INPUT { vertical-align: middle ; background: initial;',
        '                     border: 0px;}',
        '#mcq_content INPUT:focus, #mcq_content TEXTAREA:focus { background: #FFF }',
        '#mcq_content INPUT, #mcq_content TEXTAREA {font-family: monospace; font-size: 120% }',
        '#mcq_content { position:absolute; overflow: auto;',
        '       top:9.5em ; left:0.3em ; right:0px ; bottom:0px ; }',
        '.vertical, #mcq_groups, #mcq_groups SELECT { display: inline-block ; font-size: 80%; text-align: center; vertical-align: bottom }',
        '#mcqpreview { box-sizing: border-box; width: 100%; height: 99% ;',
        'border: 0px; }',
        '.mcqhead > DIV { display: inline-block; vertical-align: top; position: absolute; z-index:1; overflow:hidden; }',
        '.mcqleft { background: #DDD ; padding-right: 2em; opacity: 0.9 ; left: 0.5em; right: 0px; white-space: pre }',
        '.mcqleft:hover { z-index: 2; }',
        '.mcqright { width: 20em; right: 2em ; min-height: 6.8em; height: 9.5em; text-align: right; background: #DDD ; opacity: 0.8 }',
        'DIV.use_default, DIV.use_default SELECT, DIV.use_default INPUT  { color: #AAA }',
        'DIV.use_default:hover, DIV.use_default:hover SELECT, DIV.use_default:hover INPUT { color: #000 }',
        '#mcq_content DIV.use_default INPUT { border: 1px solid #AAA; }',
        '#mcq_content DIV.use_default:hover INPUT { border: 1px solid #000; }',
        '#mcq_groups { overflow: auto; white-space: pre; width: 100%; text-align: left; margin-top: 0.2em }',
        '#mcq_content INPUT.group, INPUT.group { font-size: 150%; border: 1px solid black;',
        '    padding: 1.5px; width: 4em; background: #FFF;  }',
        "#mcq_histo { white-space: initial }",
        "#mcq_histo DIV { display: inline-block; width: 2em; position: relative; }",
        "#mcq_histo DIV SPAN { position: absolute; bottom: 0px; color: #888; font-size: 60% }",
        ".mcq_editor *[onclick] {user-select: initial;}",
        '</style>',
        '<div class="mcq">',
        '<div class="mcqhead">',
        '<div class="mcqleft">',
        this.options_html(this.mcq_options, this.options, this.options),
        '\n<div id="mcq_groups">init</div>\n',
        this.options_html(this.question_options, this.options, this.options),
        '<div id="mcq_histo"></div>',
        '</div><div class="mcqright">',
        '<button onclick="mcq.save(this)">', _("LABEL_mcq_save"), '</button>',
        '<button onclick="mcq.store_grades()">', _("LABEL_mcq_grade"), '</button><br>',
    ];
    if (!column_change_allowed(this.column))
        t.push('<div style="color:red">'
            + _("SELECT_table_modifiable_false")
            + '. '
            + (this.column.author != my_identity
                ? _("ALERT_columndelete_not_master_before")
                + this.column.author
                : '')
            + '</div>');
    if (!this.column.is_visible())
        t.push(_("SELECT_column_visibility_no"));
    else if (this.column.modifiable != 2)
        t.push(_("SELECT_column_modifiable_by_nobody"));
    else if (the_current_cell.cell.value != yes
        && myindex(this.column.cell_writable, yes) != -1)
        t.push(_("ALERT_change_to_MCQ").replace(/.*\n *\**/g, ''));
    t.push('</div></div>');
    t.push('<div id="mcq_content">');
    t.push(this.html_content());
    t.push('</div></div>');
    return t.join('');
};
/*
MCQ.prototype.incomplete_mcq = function()
{
  for( var group in this.groups)
    if ( this.options.nr_question_in_group[group] > 0
         && this.options.nr_question_in_group[group] < this.groups[group] )
      return true ;
} ;
*/
MCQ.prototype.contains_html = function () {
    for (var q in this.questions) {
        if (this.questions[q].option('html_question'))
            return true;
        for (var c in this.questions[q].choices)
            if (this.questions[q].choices[c].option('html_choice'))
                return true;
    }
};

MCQ.prototype.disable_html = function () {
    for (var q in this.questions) {
        this.questions[q].options.html_question = 0;
        for (var c in this.questions[q].choices)
            this.questions[q].choices[c].options.html_choice = 0;
    }
};

MCQ.prototype.export = function () {
    var csv = [];
    var csv_line = [_('COL_TITLE_0_0'), _('COL_TITLE_0_1'), _('COL_TITLE_0_2')];
    for (var i in this.questions)
        csv_line.push(this.questions[i].key,
            encode_lf_tab(this.questions[i].question));
    csv.push(csv_line.join('\t'));
    for (var i in filtered_lines) {
        var line = filtered_lines[i];
        if (line[0].value === '')
            continue;
        csv_line = [html(line[0].value), html(line[1].value), html(line[2].value)];
        this.update_checked(line[this.data_col].comment);
        for (var j in this.questions)
            this.questions[j].is_asked = false;
        var asked_questions = this.student_questions(this.get_student_rand(line));
        for (var j in asked_questions)
            asked_questions[j].is_asked = true;
        for (var j in this.questions) {
            var q = this.questions[j];
            var s = [];
            for (var choice in q.choices) {
                if (this.checked[q.key + '=' + choice])
                    s.push(choice);
            }
            csv_line.push(s.join('&'), q.is_asked ? q.grade() : '');
        }
        csv.push(csv_line.join('\t'));
    }
    return csv.join('\n');
};
MCQ.prototype.update_stat = function () {
    for (var i in this.questions) {
        this.questions[i].grade_sum = {};
        for (var choice in this.questions[i].choices)
            this.questions[i].choices[choice].nr_checked = 0;
    }
    this.options.multiple_choice = this.multiple_choice();
    this.options.stats = 0;
    var histo = new Stats(0, this.max_grade());
    for (var line in filtered_lines) {
        if (filtered_lines[line][0].value == '')
            continue;
        this.update_checked(filtered_lines[line][this.data_col].comment);
        for (var question_choice in this.checked) {
            question_choice = question_choice.split('=');
            var q = this.questions[question_choice[0]];
            if (q) {
                var c = q.choices[question_choice[1]];
                if (c) {
                    c.nr_checked++;
                    continue;
                }
            }
            console.log(filtered_lines[line][0].value + ' ' + question_choice);
        }
        var grade_sum = 0;
        var questions = this.student_questions(this.get_student_rand(filtered_lines[line]));
        for (var q in questions) {
            q = questions[q];
            var grade = q.grade();
            if (q.grade_sum[grade] === undefined)
                q.grade_sum[grade] = 0;
            q.grade_sum[grade] += 1;
            grade_sum += grade;
            if (grade != 0)
                this.options.stats = 1;
        }
        histo.add(grade_sum);
    }
    this.update_stat_group();
    setTimeout(function () {
        var e = document.getElementById('mcq_histo');
        var max = histo.histo_max();
        if (e && max > 1) {
            var t = [];
            var colors = ['FDD', 'FEE', 'FFF', 'EFE', 'DFD', 'CFC'];
            var colorsborder = ['F00', '0F0'];
            for (var i = 0; i < 20; i++)
                t.push('<div style="height:' + 1.7 * histo.histogram[i] / max + 'em;background-color:#'
                    + colors[Math.floor(i / 4)] + ';border-top:1px solid #'
                    + colorsborder[Math.floor(i / 10)] + '"><span>'
                    + i + '</span></div>');
            e.innerHTML = t.join('')
                + ' ' + _("Average") + '=' + (20 * histo.average() / histo.v_max).toFixed(2)
                + ' ' + _("Mediane") + '=' + (20 * histo.mediane() / histo.v_max).toFixed(2);
        }
    }, 1000);
};
MCQ.prototype.update_stat_group = function () {
    var groups = {};
    var grades = {};
    var problem = {};
    for (var i in this.questions) {
        var q = this.questions[i];
        var grp = q.options.group;
        if (grp === undefined)
            grp = this.options.group || '';
        if (!groups[grp]) {
            groups[grp] = 1;
            grades[grp] = q.option('good_answer');
        }
        else {
            groups[grp]++;
            if (grades[grp] != q.option('good_answer'))
                problem[grp] = true;
        }
    }
    setTimeout(function () {
        if (!document.getElementById('mcq_groups'))
            return;
        var s = [];
        if (this.options.nr_question_in_group === undefined)
            this.options.nr_question_in_group = {};
        for (var group in groups) {
            var nr = this.options.nr_question_in_group[group];
            var ss = [' ', html(group),
                '<select onchange="mcq.options.nr_question_in_group[',
                js2(group),
                ']=this.value===\'\'?undefined:Number(this.value);mcq.update_stat_group()"',
                problem[group] && nr && nr != groups[group] ? ' style="background:red": ' : '',
                this.freezed ? ' disabled' : '',
                '>'];
            var one_selected = false;
            var selected;
            for (var i = 0; i <= groups[group]; i++) {
                if (nr === i || i == groups[group] && !one_selected) {
                    selected = " selected";
                    one_selected = true;
                }
                else
                    selected = '';

                ss.push('<option value="' + (i === groups[group] ? '' : i) + '"' + selected + '>'
                    + i + '/' + groups[group] + '</option>');
            }
            ss.push('</select>');
            s.push(ss.join(''));
        }
        s.sort();
        document.getElementById('mcq_groups').innerHTML = _('MSG_mcq_groups_label') + ': ' + s.join('');
    }.bind(this), 2000);
    this.groups = groups;
};

MCQ_Choice.prototype.update_class = function (element) {
    element.style.color =
        (this.option('html_choice') && !valid_html(element.value))
            ? '#F00' : '#000';
};
MCQ_Question.prototype.update_class = function (element) {
    element.style.color =
        (this.option('html_question') && !valid_html(element.value))
            ? '#F00' : '#000';
};

MCQ.prototype.get_id = function (element) {
    while (!element.id)
        element = element.parentNode;
    return element.id;
};

MCQ.prototype.update_classes = function () {
    var content = document.getElementById('mcq_content');
    var inputs = content.getElementsByTagName('TEXTAREA');
    for (var i = 0; i < inputs.length; i++)
        this.questions[this.get_id(inputs[i])].update_class(inputs[i]);

    var inputs = content.getElementsByTagName('INPUT');
    for (var i = 0; i < inputs.length; i++) {
        var key = this.get_id(inputs[i]).split(/\001/);
        if (key.length == 2)
            this.questions[key[1]].choices[key[2]].update_class(inputs[i]);
    }
};

MCQ.prototype.autofocus = function () {
    var content = document.getElementById('mcq_content');
    if (!content)
        return;
    this.update_classes();
    var inputs = content.getElementsByTagName('TEXTAREA');
    for (var i = 0; i < inputs.length; i++)
        if (inputs[i].value == _('MSG_MCQ_question')) {
            inputs[i].focus();
            inputs[i].select();
            return;
        }
    var inputs = content.getElementsByTagName('INPUT');
    for (var i = 0; i < inputs.length; i++)
        if (inputs[i].value == _('MSG_MCQ_choice')) {
            inputs[i].focus();
            inputs[i].select();
            return;
        }
};

MCQ.prototype.iframe_content = function (to_print) {
    this.modifiable = this.column.modifiable != 0;
    return '<html><head>'
        + '<link rel="stylesheet" href="_FILES_/style.css" charset="UTF-8"/>'
        + '</head><body>'
        + '<div class="mcq">'
        + mcq_suivi_style
        + (to_print
            ? '<style>'
            + '.mcq_content { position: initial ; }'
            + '.mcq_buttons { display: none ; }'
            + '</style>'
            + '<script src="' + mathjaxurl + '"></script>'
            : '')
        + '<script>'
        + 'function uncheck_if_checked() { }'
        + 'function mcq_from_element() { return {click_student: function() {}} ; }'
        + 'var MCQ = { "undefined": { "terminate": function(){} } } ;'
        + '</script>'
        + (to_print ? '' : this.get_record_button_student())
        + this.get_content_student(!this.option('show_answers'), to_print)
        + '</div>';
};

MCQ.prototype.fill_iframe = function () {
    var mcqpreview = document.getElementById("mcqpreview").contentWindow.document;
    mcqpreview.write(this.iframe_content());
};

MCQ.prototype.encode_amc = function (v) {
    return v.toString().replace('1/(#-1)', '1/(N-1)');
};

MCQ.prototype.decode_amc = function (v) {
    v = v.toString().replace('1/(N-1)', '1/(#-1)');
    if (!isNaN(v) && v !== '')
        v = Number(v);
    return v;
};

MCQ.prototype.html_content = function () {
    this.update_stat();
    this.update_checked(this.cell.comment);
    var t = [];
    if (this.options.gui == 'expert') {
        t.push('<textarea>');
        t.push(JSON.stringify(this.json(), null, 2));
        t.push('</textarea>');
    }
    else if (this.options.gui == 'amc') {
        t.push('<textarea>');
        t.push('# AMC-TXT file\n');
        t.push('Lang: ' + languages[0].toUpperCase() + '\n');
        t.push('ShuffleQuestions: ' + this.options.shuffle_questions + '\n');
        t.push('DefaultScoringS:v=' + this.encode_amc(this.options.no_answer)
            + ',b=' + this.encode_amc(this.options.good_answer)
            + ',m=' + this.encode_amc(this.options.bad_answer)
            + ',e=0\n');
        t.push('\n');
        t.push(this.amc());
        t.push('</textarea>');
    }
    else if (this.options.gui == 'wooclap') {
        t.push('<textarea>');
        t.push(this.wooclap());
        t.push('</textarea>');
    }
    else if (this.options.gui == 'preview') {
        t.push('<iframe id="mcqpreview" src="javascript:window.parent.mcq.fill_iframe()">');
        t.push('</iframe>');
    }
    else if (this.options.gui == 'print') {
        var w = window_open('MCQ');
        w.document.open('text/html');
        w.document.write(this.iframe_content(true));
        w.document.close();
        t.push('Ok !');
    }
    else {
        for (var key in this.sorted_questions)
            t.push(this.sorted_questions[key].html_table());
        t.push('<table><tr><td><button ');
        if (mcq.column.visibility == 0)
            t.push('disabled');
        else
            t.push('onclick="mcq.add_question()"');
        t.push('>+</button><td><td></tr></table>');
        setTimeout("mcq.autofocus()", 100);
    }
    return t.join('');
};

MCQ.prototype.update_mcq_content = function () {
    document.getElementById('mcq_content').innerHTML = this.html_content();
}

MCQ.prototype.save = function (element) {
    if (this.options.gui != 'beginner') {
        alert(_("ALERT_MCQ_verify"));
        return;
    }
    var data = this.json();
    column_attr_set(this.column, 'mcq', data, element);
    MCQ.initial_value = JSON.stringify(data);
}

MCQ.prototype.get_new_id = function (yet_used) {
    var key;
    do
        key = 'c' + (100 + this.table_rand.int() % 900);
    while (yet_used[key] !== undefined);
    return key;
}

MCQ.prototype.get_new_ids = function (nr) {
    var yet_used = {}, new_ids = [];
    for (var i = 0; i < nr; i++) {
        var id = this.get_new_id(yet_used);
        new_ids.push(id);
        yet_used[id] = true;
    }
    return new_ids;
}

MCQ.prototype.parse_AMC = function (content) {
    // Parse AMC view
    var questions = {};
    content = content.split(/\n/);
    content.push('*');
    var question, priority = 0, choices, choice_priority, good, bad, choice_id;
    function amc_parse(THIS, txt) {
        var options = {};
        if (txt.substr(0, 1) == '*') {
            txt = txt.substr(1);
            options['radio'] = 0;
        }
        if (txt.substr(0, 1) != '[')
            return [options, txt];
        txt = txt.trim();
        var opts = txt.substr(1).replace(/].*/, '').split(/,/);
        for (var i in opts) {
            i = opts[i].split('=', 2);
            if (i.length != 2)
                continue;
            if (i[0] == 'v')
                options['no_answer'] = THIS.decode_amc(i[1]);
            else if (i[0] == 'm')
                options['bad_answer'] = THIS.decode_amc(i[1]);
            else if (i[0] == 'b')
                options['good_answer'] = THIS.decode_amc(i[1]);
            else if (i[0] == 'ordered')
                options['shuffle_choices'] = 0;
            else
                options[i[0]] = THIS.decode_amc(i[1]);
        }
        return [options, txt.replace(/^[^\]]*]/, '').trim()];
    }

    for (var i in content) {
        var first_char = content[i].substr(0, 1);
        if (first_char == '*') {
            priority++;
            if (question) { // Save the last question
                var key = question[0].id || 'q' + priority;
                delete question[0].id;
                while (questions[key] !== undefined)
                    key = 'q' + ++priority;
                if (choice_id !== undefined)
                    choices[choice_id][0] = choices[choice_id][0].replace(/\n*$/, '');
                if (question[0].radio === undefined)
                    question[0].radio = good.length == 1 ? 1 : 0;
                else
                    question[0].radio = (question[0].radio && good.length == 1) ? 1 : 0;
                question[0].grade = [[good, bad, undefined]];
                question[0].priority = priority;
                questions[key] = [question[1], choices, question[0]];
            }
            question = amc_parse(this, content[i].substr(1));
            choices = {};
            good = [];
            bad = [];
            choice_priority = 0;
            choice_id = undefined;
        }
        else if (first_char == '+' || first_char == '-') {
            choice = amc_parse(this, content[i].substr(1));
            choice_id = choice[0].id;
            if (choice_id == undefined || choices[choice_id] !== undefined)
                choice_id = this.get_new_id(choices);
            choices[choice_id] = [choice[1], { priority: choice_priority++ }];
            (first_char == '+' ? good : bad).push(choice_id);
        }
        else if (first_char == '#') {
            continue;
        }
        else if (question) {
            if (Object.keys(choices).length == 0)
                question[1] += '\n' + content[i];
            else
                choices[choice_id][0] += '\n' + content[i];
        }
    }
    return questions;
}


MCQ.prototype.mcq_option_change = function (element) {
    var value = isNaN(element.value) || element.value === ''
        ? element.value : Number(element.value);
    var question = this.questions[this.get_id(element)];
    if (question) {
        if (value === ' undefined ' || value === '') {
            value = undefined; // Use global option
            element.parentNode.className += ' use_default';
        }
        else
            element.parentNode.className = element.parentNode.className.replace(' use_default', '');
        question.options[element.name] = value;
        if (element.name == "good_answer" || element.name == "group")
            this.update_stat_group();
        return;
    }
    this.options[element.name] = value;
    var content = document.getElementById('mcq_content').firstChild;
    if (element.name == 'gui' && content.tagName == 'TEXTAREA') {
        content = content.value;
        if (content.trim().substr(0, 1) == '[') {
            // Parse expert view
            try {
                content = JSON.parse(content);
                content[1].gui = element.value;
                this.replace_global_mcq(content);
                return;
            }
            catch (e) {
                alert(e);
                element.value = 'expert';
                return;
            }
        }
        else if (content.trim().substr(0, 19) == 'Type\tTitle\tCorrect\t'
            || content.substr(0, 8) == '\tWOOCLAP') {
            var questions = {};
            content = content.split(/\n+/);
            for (var i = 1; i < content.length; i++)
                content[i] = content[i].split(/\t/);
            for (var i = 1; i < content.length; i++) {
                if (content[i][0] != 'MCQ')
                    continue;
                this.parse_wooclap(questions, content[i], content[i + 1]);
            }
            this.replace_global_mcq([questions, this.options]);
            this.options.gui = element.value;
            return;
        }
        else {
            try {
                var questions = this.parse_AMC(content);
            }
            catch (err) {
                alert(err);
                element.value = 'AMC';
            }
            this.options.gui = element.value;
            this.replace_global_mcq([questions, this.options]);
            return;
        }
    }
    this.update_mcq_content();
}

var valid_html_element = undefined;
function valid_html(txt) {
    if (!valid_html_element)
        valid_html_element = document.createElement('DIV');
    txt = txt.replace(/ /g, ' ');
    valid_html_element.innerHTML = txt;
    return valid_html_element.innerHTML == txt;
}

function mcq_group_completion(input) {
    var groups = [];
    for (var group in mcq.groups)
        if (group !== ''
            && group.substr(0, input.value.length).toLocaleLowerCase()
            == input.value.toLocaleLowerCase())
            groups.push([group, '', '', '', group]);
    groups.push([input.value === '' ? '' : ' ' + input.value, '', '', '', input.value]);
    groups.last = '';
    return groups;
}

MCQ.prototype.update_question = function (element, event, do_it) {
    if (!do_it && event.type == 'drop') {
        var THIS = this;
        // Want to get the text value after the drop
        setTimeout(function () { THIS.update_question(element, event, true); }, 0);
        return;
    }
    event = event || window.event;
    var target = event.target;
    if (target.tagName == 'SELECT') {
        if (event.type == 'change') {
            this.update_stat();
            this.update_mcq_content();
        }
        return;
    }

    if ((target.tagName == 'INPUT' && target.type == 'text'
        || target.tagName == 'TEXTAREA')
        && target !== this.focus_on
        && event && event.type == 'click' && event.button == 0) {
        if (target.value == _('MSG_MCQ_question')
            || target.value == _('MSG_MCQ_choice'))
            event.target.select();
        this.focus_on = target;
        return;
    }

    var content = target.textContent;
    if ((target.tagName == 'BUTTON' || target.tagName == 'SPAN')
        && event.type == 'click'
        && (content == '↓' || content == '↑' || content == '+')) {
        while (target.id === undefined || target.id === '')
            target = target.parentNode;
        var question = this.questions[target.id];
        if (question) {
            if (content == '↓')
                question.next_priority();
            else if (content == '↑')
                question.previous_priority();
            else if (content == '+')
                question.add_choice();
        }
        else {
            var keys = target.id.split('\001');
            question = this.questions[keys[1]];
            if (question) {
                var choice = question.choices[keys[2]];
                if (choice) {
                    if (content == '↓')
                        choice.next_priority();
                    else
                        choice.previous_priority();
                }
            }
        }
        this.replace_global_mcq(this.json());
        return;
    }
    this.update_question_real(element);
};

MCQ.prototype.update_question_real = function (element) {
    var q = this.questions[element.id];
    var ok = [];
    var bad = [];
    for (var r = 1; r < element.rows.length; r++) {
        var row = element.rows[r];
        if (r == 1) {
            q.question = row.cells[1].firstChild.value;
            q.update_class(row.cells[1].firstChild);
            continue;
        }
        var key = row.id.split(/\001/)[2];
        if (key === undefined)
            break;
        var choice = q.choices[key];
        choice.choice = row.cells[2].firstChild.value;
        choice.update_class(row.cells[2].firstChild);
        if (row.cells[1].firstChild.checked)
            ok.push(key);
        else
            bad.push(key);
    }
    if (ok.length == 1)
        q.options.radio = 1;
    else
        q.options.radio = 0;
    q.options.grade = [[ok, bad, undefined]];
};


MCQ.prototype.get_answers = function (element) {
    var checked = [];
    var buttons = element.getElementsByTagName('INPUT');
    for (var key = 0; key < buttons.length; key++) {
        var b = buttons[key];
        if (!b.checked)
            continue;
        checked.push([b.name.split(/\001/)[1] + '=' + b.value]);
    }
    checked.sort();
    return checked.join('&');
}

MCQ.prototype.grade = function (checked, rand) {
    var g = 0;
    this.update_checked(checked);
    var questions = this.student_questions(rand);
    for (var key in questions) {
        var grade = questions[key].grade();
        g += grade;
    }
    return g;
};

MCQ.prototype.json = function () {
    var questions = {};
    for (question in this.questions)
        if (this.questions[question].question !== '')
            questions[question] = this.questions[question].json();
    return [questions, this.options];
};

MCQ.prototype.amc = function () {
    var t = [];
    for (question in this.sorted_questions)
        if (this.sorted_questions[question].question !== '')
            t.push(this.sorted_questions[question].amc() + '\n');
    return t.join('');
};

MCQ.prototype.wooclap = function () {
    var t = ['Type', 'Title', 'Correct', 'Choice', 'Choice', 'Choice', 'Choice', 'Choice', 'Choice', 'Choice'];
    t = ['\tWOOCLAP\t', _('MSG_mcq_wooclap'), '\n', t.join('\t'), '\n'];
    for (question in this.sorted_questions)
        if (this.sorted_questions[question].question !== '')
            t.push(this.sorted_questions[question].wooclap() + '\n');
    return t.join('');
};

MCQ.prototype.empty_choices = function () {
    var nr = 2;
    var new_ids = this.get_new_ids(nr);
    var choices = {};
    for (var i = 0; i < nr; i++)
        choices[new_ids[i]] = [_("MSG_MCQ_choice"), { priority: i + 1 }];
    return choices;
}

MCQ.prototype.add_question = function () {
    var last_key = 0;
    for (question in this.questions) {
        question = Number(question.substr(1));
        if (question > last_key)
            last_key = question;
    }
    var key = 'q' + (last_key + 1);

    this.questions[key] =
        new MCQ_Question([_('MSG_MCQ_question'), this.empty_choices(),
        { 'priority': this.sorted_questions[this.sorted_questions.length - 1].options.priority + 1 }],
            this, key);
    this.replace_global_mcq(mcq.json());
    document.getElementById('popup_id').scrollTo(0, 10000000);
};

MCQ.prototype.max_grade = function () {
    var sum_max = 0;
    var questions = this.student_questions(new Rand('anyone'));
    for (var question in questions) {
        grades = questions[question].option('grade');
        var max = questions[question].option('good_answer');
        for (var grade in grades)
            max = Math.max(grades[grade][2] || 0, max);
        sum_max += max;
    }
    return sum_max;
};

MCQ.prototype.get_student_rand = function (line, more) {
    line = line || the_current_cell.line;
    more = more || '';
    var key = this.key || (this.year + '/' + this.semester + '/' + this.ue + '/'
        + this.column.the_id
        + '/' + the_login(line[0].value));
    return new Rand(key + more);
};

MCQ.prototype.store_grades = function () {
    var todo = [];
    column_attr_set(this.column, 'minmax', '[0;' + this.max_grade() + ']');
    for (var line in filtered_lines) {
        line = filtered_lines[line];
        if (line[0].value == '')
            continue;
        var rand = this.get_student_rand(line);
        try {
            var grade = this.grade(line[this.data_col].comment, rand);
            if (grade < 0)
                grade = 0;
            if (line[this.data_col].value === '' || line[this.data_col].value != grade)
                todo.push([line, grade.toString()]);
        }
        catch (e) {
        }
    }
    var changes = [];
    for (var i in todo)
        if (todo[i][0][this.data_col].value !== ''
            && todo[i][0][this.data_col].value != no)
            changes.push(todo[i][0][2].value + ' ' + todo[i][0][1].value + ' : '
                + todo[i][0][this.data_col].value + ' → ' + todo[i][1]);

    var message = '';
    if (changes.length)
        message += _("MSG_fill_room_simulation") + '\n' + changes.join('\n') + '\n\n';
    message += todo.length + ' ' + _("grades") + ' : ' + _("B_unsaved_save");
    if (!confirm(message))
        return;

    alert_append_start();
    for (var i in todo) {
        cell_set_value_real(todo[i][0].line_id, this.data_col, todo[i][1]);
        update_line(todo[i][0].line_id, this.data_col);
    }
    the_current_cell.do_update_column_headers = true;
    table_fill();
    table_header_fill();
    alert_append_stop();
};

MCQ.prototype.replace_global_mcq = function (content) {
    mcq = new MCQ(content);
    mcq.data_col = this.data_col;
    mcq.cell = this.cell;
    mcq.column = this.column;
    mcq.ue = this.ue;
    mcq.year = this.year;
    mcq.semester = this.semester;
    mcq.freezed = this.freezed;
    mcq.update_mcq_content();
};

MCQ.prototype.close_student = function () {
    if (this.modified) {
        if (confirm(_("LABEL_MCQ_save"))) {
            this.record_student(popup_get_element().getElementsByTagName('BUTTON')[0]);
            return;
        }
    }
    popup_close = MCQ.popup_close;
    popup_close();
};

MCQ.prototype.record_ok = function () {
    this.modified = false;
    this.record_button.setAttribute('disabled', true);
    var b = this.record_button.parentNode.getElementsByTagName('BUTTON');
    b[b.length - 1].removeAttribute('disabled');
};

MCQ.prototype.record_student = function (button) {
    if (this.modified) {
        var element = document.getElementById(this.key);
        this.cell.comment = this.get_answers(element);
        this.feedback = button.parentNode.firstChild;
        this.feedback.innerHTML = '<div><div>' + _("LABEL_MCQ_saved") + ':</div></div>';
        DisplayGrades.column = this.column;
        this.record_button = button;
        _cell(this.feedback.firstChild.firstChild,
            this.year + '/' + this.semester + '/' + this.url,
            this.cell.comment,
            this.record_ok.bind(this)
        );
    }
};

MCQ.prototype.terminate = function (button) {
    var feedback = button.parentNode.firstChild;
    feedback.innerHTML = '<div><div>' + _("LABEL_MCQ_terminated") + ':</div></div>';
    // Erase the YES
    feedback.firstChild.firstChild.value = no;
    DisplayGrades.column = this.column;
    _cell(feedback.firstChild.firstChild, this.year + '/' + this.semester + '/'
        + this.url.replace('/comment/', '/cell/'));
    var buttons = button.parentNode.getElementsByTagName('BUTTON');
    for (var i = 0; i < buttons.length; i++)
        buttons[i].setAttribute('disabled', true);
    var inputs = document.getElementById(this.key).getElementsByTagName('INPUT');
    for (var i = 0; i < inputs.length; i++)
        inputs[i].setAttribute('disabled', true);
    this.modifiable = false;

    var buttons = document.getElementsByTagName('BUTTON');
    for (var i = 0; i < buttons.length; i++)
        if (buttons[i].onclick
            && buttons[i].onclick.toString().indexOf(this.key) != -1)
            buttons[i].setAttribute('disabled', '1');
};

MCQ.prototype.click_student = function (event) {
    if (this.modifiable) {
        var element = event.target;
        var in_label = false;
        do {
            in_label = in_label || element.tagName == 'LABEL';
            var button = element.getElementsByTagName('BUTTON');
            if (button.length) {
                if (in_label) {
                    button[0].firstChild.innerHTML = _("LABEL_MCQ_save");
                    button[0].removeAttribute('disabled');
                    button[1].setAttribute('disabled', true);
                    this.modified = true;
                    if (this.feedback) {
                        this.feedback.firstChild.className = 'done';
                        this.feedback = undefined;
                    }
                }
                break;
            }
            element = element.parentNode;
        }
        while (element);
        localStorage[this.key] = this.get_answers(document.getElementById(this.key));
    }
};

MCQ.prototype.add_scrollbar_tags = function () {
    var element = document.getElementById(this.key).parentNode.parentNode;
    var t = [];
    var q, popup_height, scrolled_height, full_height, top;
    function percent(px) {
        return 100 * (scrolled_height * px / full_height + top) / popup_height;
    }
    for (var i in this.questions) {
        var key = this.questions[i].full_key;
        q = document.getElementById(key);
        if (!q)
            continue; // Hidden questions
        t.push('<div class="mcqtag" id="TAG/');
        t.push(key);
        t.push('" onmousedown="mcq_click_on_tag(event)" style="top:');
        if (!popup_height) {
            popup_height = q.parentNode.parentNode.parentNode.offsetHeight;
            scrolled_height = q.parentNode.parentNode.offsetHeight;
            full_height = q.parentNode.offsetHeight;
            top = q.parentNode.parentNode.offsetTop;
        }
        t.push(percent(q.offsetTop));
        t.push('%;height:');
        t.push(percent(q.offsetTop + q.offsetHeight) - percent(q.offsetTop));
        t.push('%"></div>');
    }
    element.innerHTML += t.join('');
};

MCQ.prototype.get_record_button_student = function () {
    var b = '<div class="mcq_buttons">';
    if (this.modifiable)
        b += '<div class="mcqfeedback"></div>'
            + '<button'
            + (this.modified ? '' : ' disabled')
            + ' class="mcq_record" '
            + 'style="border-radius:1em;'
            + '" onclick="MCQ.dict[\'' + this.key
            + '\'].record_student(this)"><span>'
            + _("LABEL_MCQ_save") + '<span></button>'
            + (this.textual ? ' ' + or_keyword() + ' ' : ' ')
            + '<button'
            + (this.modified ? ' disabled' : '')
            + ' class="mcq_terminate" '
            + 'style="border-radius:1em;'
            + '" onclick="MCQ.dict[\'' + this.key
            + '\'].terminate(this)"><span>'
            + _("LABEL_MCQ_terminate") + '<span></button>'
            ;
    else {
        b += '<p>';
        if (this.cell.value === '' || isNaN(this.cell.value))
            b += _('MSG_MCQ_not_open');
        else
            b += _('MSG_MCQ_done');
    }
    b += '</div>';
    return b;
};

MCQ.prototype.get_content_student = function (hide_answers, to_print) {
    if (!to_print)
        setTimeout(this.update_html_student.bind(this), 100);
    return '<div class="mcq_content">'
        + this.html_student(hide_answers)
        + (this.modifiable
            ? (this.textual
                ? this.get_record_button_student()
                : _('MSG_MCQ_end')
            )
            : '')
        + '</div>';
};

MCQ.prototype.get_button_student = function (in_last_grades_list) {
    var label, button;
    label = note_format_suivi();
    button = '';
    if (this.modifiable) {
        if (this.textual) {
            if (in_last_grades_list)
                return '';
            if (isNaN(DisplayGrades.cell.value))
                label = '';
            else
                label += '<br>'
            label += this.get_content_student();
        }
        else {
            label = '';
            button = _("MSG_MCQ_open");
        }
    }
    else {
        if (this.cell.value === no)
            label = _('MSG_MCQ_done');
        else if (this.cell.value === yes)
            label = _('MSG_MCQ_not_open');
        else if (this.cell.value === '')
            label = _('MSG_MCQ_not_open');
        else {
            if (this.textual) {
                if (!in_last_grades_list)
                    label += '<br>' + this.button_label() + this.get_content_student();
            }
            else if (this.options.show_answers)
                button = this.button_label();
        }
    }
    if (button !== '') {
        button = '<button onclick="mcq_open(\'' + this.key + '\')">'
            + button + '</button>';
        if (label !== '')
            label += '<br>';
    }
    return label + button;
};

MCQ_Question.prototype.wooclap = function () {
    function cleanup_options(key, options) {
        opt = JSON.parse(JSON.stringify(options));
        delete opt.priority;
        opt.key = key;
        opt = JSON.stringify(opt);
        if (opt === '{}')
            return '';
        return opt;
    }
    function wooclap(txt) {
        return txt.replace(/\n/g, '   ')
            .replace(/\\\(/g, ' $').replace(/\\\)/g, '$ ')
            .replace(/\\\[/g, '  $').replace(/\\\]/g, '$  ')
            .replace(/\$\$([^$]*)\$\$/g, '   $$$1$$   ');
    }

    var goods = [];
    if (this.options.grade)
        for (var good in this.options.grade[0][0]) {
            good = this.options.grade[0][0][good];
            for (var i in this.sorted_choices)
                if (this.sorted_choices[i].key == good) {
                    goods.push(Number(i) + 1);
                    break;
                }
        }
    var t = ['MCQ', wooclap(this.question), goods.join(' ')];
    for (var choice in this.sorted_choices)
        t.push(wooclap(this.sorted_choices[choice].choice));
    var tt = ['', cleanup_options(this.key, this.options), ''];
    for (var choice in this.sorted_choices) {
        choice = this.sorted_choices[choice];
        tt.push(cleanup_options(choice.key, choice.options));
    }
    return line1 = t.join('\t') + '\n' + tt.join('\t');
};

MCQ.prototype.parse_wooclap = function (questions, values, options) {
    function wooclap(txt) {
        return txt.replace(/   \$([^$]*)\$   /g, '$$$$$1$$$$')
            .replace(/  \$([^$]*)\$  /g, '\\[$1\\]')
            .replace(/ \$([^$]*)\$ /g, '\\($1\\)')
            .replace(/\$([^$]*)\$/g, '\\($1\\)')
            .replace(/   /g, '\n');
    }
    var choices = {}, choices_index = [''];
    var key, opt;
    for (var i = 3; i < values.length; i++) {
        if (values[i] === '')
            break;
        opt = {};
        if (options && options[0] === '' && options[i] !== '' && options[i]) {
            opt = JSON.parse(options[i]);
            key = opt.key;
            delete opt.key;
        }
        else {
            key = undefined;
        }
        opt.priority = i;
        if (!key)
            key = this.get_new_id(choices);
        choices[key] = [wooclap(values[i]), opt];
        choices_index.push(key)
    }
    if (options && options[0] === '' && options[1] !== '' && options[1]) {
        opt = JSON.parse(options[1]);
    }
    else {
        opt = {};
        var goods = [], bads = [];
        var g = values[2].split(/[ ,]/);
        for (var i = 1; i < choices_index.length; i++)
            if (g.indexOf(i.toString()) == -1)
                bads.push(choices_index[i]);
            else
                goods.push(choices_index[i]);
        opt.grade = [[goods, bads, null]];
    }
    key = opt.key;
    delete opt.key;
    opt.priority = Object.keys(questions).length;
    if (!key)
        key = 'q' + opt.priority;
    questions[key] = [wooclap(values[1]), choices, opt];
};

//**************************************************************************

function sorted_dictionary(dict, rand) {
    var list = [];
    for (var key in dict) {
        list.push(dict[key]);
        if (rand)
            dict[key]._priority = rand.int();
        else
            dict[key]._priority = dict[key].options.priority;
    }
    list.sort(function (a, b) { return a._priority - b._priority; });
    return list;
}

var mcq; // The last MCQ on popup: only for the table editor

//**************************************************************************
// TABLE

function mcq_close() {
    if (mcq.options.gui != 'beginner') {
        var save = mcq.options.gui;
        mcq.options.gui = 'beginner';
        if (MCQ.initial_value != JSON.stringify(mcq.json())) {
            mcq.options.gui = save;
            alert(_("ALERT_MCQ_verify"));
            return;
        }
    }
    if (MCQ.initial_value != JSON.stringify(mcq.json())
        && column_change_allowed(mcq.column)
        && confirm(_("LABEL_mcq_save")))
        mcq.save();
    popup_close = MCQ.popup_close;
    popup_close();
    the_current_cell.jump = MCQ.the_current_cell_jump.bind(the_current_cell);
}

function toggle_mcq(value) {
    mcq = new MCQ(the_current_cell.column.mcq, the_current_cell.cell.comment);
    mcq.column = the_current_cell.column;
    mcq.data_col = the_current_cell.data_col;
    mcq.cell = the_current_cell.cell;
    mcq.ue = ue;
    mcq.year = year;
    mcq.semester = semester;
    create_popup('mcq_editor', '', mcq.html_table(), '', false);

    if (MCQ.popup_close === undefined) {
        MCQ.popup_close = popup_close;
        MCQ.the_current_cell_jump = the_current_cell.jump.bind(the_current_cell);
    }
    popup_close = mcq_close;
    MCQ.initial_value = JSON.stringify(mcq.json());

    the_current_cell.jump = mcq_jump;

    return value;
}

function mcq_jump(lin, col, do_not_focus, line_id, data_col) {
    if (do_not_focus === undefined)
        return;
    MCQ.the_current_cell_jump(lin, col, do_not_focus, line_id, data_col);
    mcq.cell = the_current_cell.line[mcq.data_col];
    if (mcq.options.gui == 'beginner' || mcq.options.gui == 'preview')
        mcq.update_mcq_content();
}

function i_am_a_new_mcq(column) {
    column_attr_set(column, 'visibility', 1);
    column_attr_set(column, 'modifiable', 2);
    column_attr_set(column, 'cell_writable',
        "#[] = | #[] @ | #[] =" + no + " | !#[] =" + yes);
    table_header_fill();
}

//**************************************************************************
// SUIVI

var mcq_suivi_color = '444';
var mcq_suivi_style = '<style id="mcqstyle">' // Do not apply to the screen reader interface
    + 'DIV.import_export.mcq { left:150px; right:5%; top:6.5em; bottom:5%;'
    + '   border: 3px solid #' + mcq_suivi_color + '; position: fixed }'
    + 'DIV[role=heading] { background: #FFF }'
    + '.mcq_content { overflow-y: scroll; position: absolute ;'
    + '               top: 40px ; left: 0px; right: 0px; bottom: 40px ;'
    + '               padding-left: 0.5em; padding-right: 0.5em ; }'
    + '.mcq_content p:first-child { margin: 1em; font-weight: bold }'
    + '#popup_id { padding-top: 0px ; }'
    + '.mcqquestion { border: 2px solid #FFF ; border-left-width: 1em;'
    + '               margin-bottom:1em; transition: border-color 1s }'
    + '.mcqquestion.unanswered { border-color: #' + mcq_suivi_color + ' }'
    + '.mcqquestion div:first-child { padding-left: 0.5em }'
    + ".mcqquestion .grading, .mcqquestion .question_grade { margin-left:1em ; float: right; height: 1em }"
    + ".mcqquestion .grading { color: #FFF ; background: #888 }"
    + ".mcqquestion .question_grade { border: 1px solid #000 }"
    + '.mcqchoice { width: 100% }'
    + '.mcqchoice TD { cursor: pointer }'
    + '.mcqchoice TD:first-child { width: 2em; white-space: nowrap; text-align: right; margin-left: 0.2em; padding:1px }'
    + '.mcqchoice:hover { background: #EEE }'
    + '.mcqchoice .expected { float: right }'
    + '.mcq DIV.mcqfeedback IFRAME.feedback { width: 5em; height: 1.2em; background: red }'
    + '.mcq DIV.mcqfeedback > DIV { width: 25em ; transition: width 5s; overflow: hidden }'
    + '.mcq DIV.mcqfeedback > DIV.done { width: 0px; }'
    + '.mcq SECTION { margin-left: 0px }'
    + '.mcq H2, .mcq_buttons { position:absolute; left:0px; right:0px ;'
    + '                        height:38px ;}'
    + '.mcq H2 DIV { height: 100%;box-sizing: border-box; padding-left: 0.5em; padding-top: 4px }'
    + '.mcq H2 { border-bottom: 1px solid #' + mcq_suivi_color + ';}'
    + '.mcq_buttons { border-top: 1px solid #' + mcq_suivi_color
    + '; padding-right: 0.5em; padding-top: 4px ; box-sizing: border-box;  }'
    + '.mcq_buttons { bottom: 0px; right: 0px; text-align: right }'
    + '.mcq_buttons BUTTON { height: 30px; box-sizing: border-box; }'
    + '.mcq_buttons DIV { display: inline-block ; white-space: nowrap }'
    + 'BUTTON.mcq_terminate, BUTTON.mcq_terminate[disabled]:hover { background: #FCC }'
    + 'BUTTON.mcq_terminate:hover { background: #F88 }'
    + 'BUTTON.mcq_record, BUTTON.mcq_record[disabled]:hover  { background: #CFC }'
    + 'BUTTON.mcq_record:hover { background: #8F8 }'
    + '#popup_id .close { z-index: 100 }'
    + 'DIV.Picture DIV.tipped DIV.help { display: block }'
    + 'DIV.Picture DIV.tipped DIV.help IMG { height: auto; width: 150px }'
    + 'DIV.Picture DIV.tipped DIV.text IMG { display: none }'
    + '.mcqtag { position:absolute; right:0.6em; width:0.2em ; cursor: pointer;'
    + '          background: #FFF; transition: background 1s }'
    + '.mcqtag.unanswered { background: #000; }'
    + 'H { font-size: 0px; color:transparent; }'
    + '[onclick] { user-select: initial }'
    + '</style>';

function mcq_open(key) {
    var mcq = MCQ.dict[key];
    if (is_a_teacher && mcq.contains_html() && confirm(_('ALERT_MCQ_safe')))
        mcq.disable_html();

    var data = mcq.cell.comment;
    mcq.modified = false;
    if (localStorage[key] !== undefined && localStorage[key] != data
        && mcq.modifiable // Do not retrieve if closed
        && localStorage[key].length > data.length - 2) // XXX Not clean
        if (confirm(_("CONFIRM_MCQ_retrieve"))) {
            data = localStorage[key];
            mcq.modified = true;
        }
    mcq.update_checked(data);
    add_ticket_checker(the_body);
    window.scrollTo(0, 0);
    create_popup('mcq', mcq_suivi_style
        + '<div>«' + html(mcq.column.title) + '» : '
        + mcq.button_label() + '</div>'
        , mcq.get_record_button_student() + mcq.get_content_student(), '', false);
    if (MCQ.popup_close === undefined)
        MCQ.popup_close = popup_close;
    popup_close = mcq.close_student.bind(mcq);
    var images = document.getElementsByTagName('IMG');
    for (var i = 1; i < images.length; i++)
        if (images[i - 1].className == 'big' && images[i].className == 'small')
            images[i - 1].src = images[i].src.replace('-icon', '');
    mcq.add_scrollbar_tags();
}

function mcq_from_element(element) {
    while (element) {
        if (MCQ.dict[element.id])
            return MCQ.dict[element.id];
        element = element.parentNode;
    }
}

function mcq_filter_active(condition) {
    if (!mcq.column)
        return;
    return mcq.column.filter.match(RegExp(' #~' + condition + '$'));
}

function mcq_filter(condition) {
    var filter = mcq.column.filter.replace(/ *#~.*/, '');
    if (!mcq_filter_active(condition))
        filter += ' #~' + condition;
    filter_change_column(filter, mcq.column);
    table_fill(true, true, true);
}

function mcq_click_on_tag(event) {
    document.getElementById(the_event(event).target.id.substr(4)).scrollIntoView();
}

function mcq_format_suivi() {
    if (Object.keys(DisplayGrades.column.mcq[0]).length == 0)
        return _("MCQ is terminated");
    var key = DisplayGrades.ue.year + '/' + DisplayGrades.ue.semester + '/' + DisplayGrades.ue.ue + '/'
        + DisplayGrades.column.the_id + '/' + display_data['Login'];
    var mcq = MCQ.dict[key];
    if (!mcq) {
        mcq = new MCQ(DisplayGrades.column.mcq, DisplayGrades.cell.comment, key);
        mcq.cell = DisplayGrades.cell;
        mcq.column = DisplayGrades.column;
        mcq.ue = DisplayGrades.ue.ue;
        mcq.year = DisplayGrades.ue.year;
        mcq.semester = DisplayGrades.ue.semester;
        mcq.url = DisplayGrades.ue.ue
            + '/comment/' + DisplayGrades.column.the_id
            + '/' + DisplayGrades.ue.line_id;
        mcq.textual = display_update.top == 'Textual'; // Screen reader
        var save = DisplayGrades.column.modifiable;
        if (DisplayGrades.column.modifiable_backup !== undefined)
            DisplayGrades.column.modifiable = DisplayGrades.column.modifiable_backup;
        mcq.modifiable = cell_modifiable_on_suivi(); // 0 is not false
        if (DisplayGrades.column.modifiable)
            mcq.options.show_answers = false;
        DisplayGrades.column.modifiable = save;
        if (!isNaN(DisplayGrades.cell.value) && DisplayGrades.cell.value !== '')
            mcq.modifiable = false; // No more editable if graded
        if (DisplayGrades.cell.value === no)
            mcq.modifiable = false; // No more editable if terminated
    }
    return mcq.get_button_student(DisplayGrades.column.modifiable === false);
}

function test_mcq(value, column) {
    if (value !== '' && !isNaN(value))
        return test_note(value, column);
    return test_bool(value, column, true) || test_note(value, column);
}

function MCQ_export() {
    var mcq = new MCQ(the_current_cell.column.mcq);
    mcq.column = the_current_cell.column;
    mcq.data_col = the_current_cell.data_col;
    mcq.ue = ue;
    mcq.year = year;
    mcq.semester = semester;
    create_popup('mcq_export', html(the_current_cell.column.title),
        _("MSG_print_popup_title"), '', mcq.export());
}