#!/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2008,2010 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

from . import text

class Note(text.Text):
    attributes_visible = ('minmax', 'weight', 'rounding', 'repetition',
                          'url_import', "groupcolumn", "grade_type", "grade_session")
    human_priority = -10
    tip_filter = "TIP_filter_Note"
    tip_cell = "TIP_cell_Note"
    cell_test = "test_note"
    formatte = 'note_format'
    formatte_suivi = 'note_format_suivi'
 
    should_be_a_float = 1

    message = """<script>Write("MSG_Note_ABINJ");</script>"""

    def stat(self, column, value, cell, lines, grp_seq):
        """Returns a dict of various values, currently:
        'rank', 'rank_grp', 'average'
        The values are cached 60 seconds.
        """
        if getattr(column, 'floats', None) is None:
            column.floats = {}
        if '' not in column.floats:
            all_floats = column.cell_values(column.table.lines.values())
            column.floats[''] = all_floats
            if all_floats:
                column.floats['average'] = sum(all_floats) / len(all_floats)
                column.floats['mediane'] = all_floats[len(all_floats)//2]
        else:
            all_floats = column.floats['']

        if 'grp_seq' in column.floats:
            floats = column.floats[grp_seq] # pragma: no cover
        else:
            floats = column.floats[grp_seq] = column.cell_values(lines)

        d = {"nr": len(all_floats),
             "nr_in_grp": len(floats),
             }
        try:
            d["rank"] = all_floats.index(value)
        except ValueError:
            pass
        try:
            d["rank_grp"] = floats.index(value)
        except ValueError:
            pass
        if all_floats:
            d["average"] = column.floats['average']
            d["mediane"] = column.floats['mediane']
        return d

    def value_range(self, v_min, v_max):
        if v_min == 0:
            return '/%g' % v_max
        else:
            return '[%g;%g]' % (v_min, v_max)
