/*
  TOMUSS: The Online Multi User Simple Spreadsheet
  Copyright (C) 2010-2020 Thierry EXCOFFIER, Universite Claude Bernard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr
*/

var body = document.getElementsByTagName('BODY')[0];
var menutop = document.createElement('DIV')
menutop.className = 'menu';
menutop.innerHTML = '<p class="search"><b>Chercher le texte :</b><br>'
    + '<input id="input" onkeypress="key_pressed(event)" onkeydown="key_pressed(event)">'
    + '<br> </p>'
    + '<div id="menu"></div>'
    + '<div id="log"></div>'
    + '</div>';
body.insertBefore(menutop, body.firstChild);

var last_key_time = 0;
var display = false;
var input = document.getElementById('input');
var log = document.getElementById('log');
var divs = document.getElementsByTagName('DIV');
var menu = document.getElementById('menu');

function millisec() {
    var d = new Date();
    return d.getTime();
}

function window_height() {
    var height = window.innerHeight;
    if (height === undefined)
        return document.body.clientHeight;
    return height;
}


function diacritics_regex(s) {
    var diacritics = [
        "[aA\300-\306\340-\346]",
        "[eE\310-\313\350-\353]",
        "[iI\314-\317\354-\357]",
        "[oO\322-\330\362-\370]",
        "[uU\331-\334\371-\374]",
        "[nN\321\361]",
        "[cC\307\347]"
    ];

    for (var i = 0; i < diacritics.length; i++) {
        s = s.replace(RegExp(diacritics[i], "g"), diacritics[i]);
    }
    return s;
}

function key_pressed(event) {
    if (event === undefined)
        event = window.event;
    if (event.keyCode == 34 && event.type == 'keydown') {
        window.scrollBy(0, window_height());
        event.preventDefault();
    }
    if (event.keyCode == 33 && event.type == 'keydown') {
        window.scrollBy(0, -window_height());
        event.preventDefault();
    }
    display = true;
    last_key_time = millisec();

    if (input.value != input.old_value && input.value !== '') {
        setTimeout(function () {
            document.body.scrollTop = 0;
            filter();
        }, 100);
    }
}

var timestamp_click_menu = 0;

function go_hash(name, keep_filter) {
    if (!keep_filter) {
        input.value = '';
        filter();
        display = true;
    }

    setTimeout(function () {
        // window.location.hash = '' ; // Bug Chrome
        window.location.hash = name;
        set_current();
        input.focus();
        if (keep_filter)
            highlight();
    }, 1);
}

function go_clear(name) {
    input.value = '';
    window.location.hash = name;
    display = true;
}

function go(name) {
    if (name.substr(0, 1) == '#') {
        timestamp_click_menu = millisec();
        var m = menu.childNodes[Number(name.substr(2))];
        var keep_filter = m ? m.className.indexOf('unselected') == -1 : true;
        go_hash(name, keep_filter);
    }
    else {
        input.value = decodeURI(name);
        document.body.scrollTop = 0;
        input.focus();
        display = true;
    }
}

function set_current() {
    var m = RegExp('"' + window.location.hash + '"');
    for (var i in menu.childNodes)
        if (menu.childNodes[i].className !== undefined)
            if (menu.childNodes[i].href.match(m))
                menu.childNodes[i].className += ' current';
            else
                menu.childNodes[i].className = menu.childNodes[i].className.replace(/ current/g, '');

    update_scrollbar(true);
}

var last_scrolltop = -1;

function update_scrollbar(forced) {
    var child;

    if (forced != true && last_scrolltop == document.body.scrollTop)
        return;
    last_scrolltop = document.body.scrollTop;
    var first = true;
    for (var i in menu.childNodes) {
        var child = menu.childNodes[i];
        if (!child.getAttribute)
            continue;
        i = divs[Number(child.getAttribute('the_div'))];
        // Why the className is missing on first H1 ????
        if (child.className === undefined)
            child.className = '';

        var className = child.className.replace(/( onscreen)*/g, '');
        var scroll_allowed = millisec() - timestamp_click_menu > 500;
        if (i.offsetTop + i.offsetHeight > document.body.scrollTop &&
            i.offsetTop < document.body.scrollTop + window_height()) {
            className += ' onscreen';
            if (first && scroll_allowed) {
                child.scrollIntoView({ block: "center" });
                first = false;
            }
        }
        child.className = className;
    }
}

function insert_highlight(element, content, pattern) {
    element.innerHTML = content.replace(RegExp('(>[^<]*)(' + pattern + ')', 'mgi'),
        '$1<span class="filter">$2</span>');
}

function filter() {
    update_scrollbar(false);

    if (display == false) {
        if (input.value !== input.old_value
            && input.value !== ''
            && millisec() - last_key_time > 1000) {
            var i = document.createElement('IMG');
            i.src = 'log/help/' + encodeURI(input.value);
            log.appendChild(i);
            input.old_value = input.value;
        }
        return;
    }

    display = false;


    var v = input.value;

    if (v === '') {
        // Restore initial state
        for (var i in text_divs) {
            text_divs[i].style.display = '';
            if (menu.childNodes[i])
                menu.childNodes[i].className = 'unselected';
            text_divs[i].innerHTML = content_divs[i];
        }
        set_current();
        highlight();
        return;
    }
    // Protect special chars
    v = '\\b' + diacritics_regex(v.replace(/([*\\[.$+?()])/g, '\\$1'));
    var html;
    var selected = [''];
    var levels_menu = {};
    var levels_body = {};
    var my_level;
    // Change the content
    for (var i in text_divs) {
        html = content_divs[i];
        div = text_divs[i];
        for (var j = 0; j < div.children.length; j++)
            if (div.children[j].tagName.substr(0, 1) == 'H') {
                my_level = Number(div.children[j].tagName.substr(1, 1));
                levels_body[my_level] = i;
                break;
            }

        if (html.search(RegExp(v, 'gim')) == -1) {
            if (div.id != 'menu') {
                div.style.display = 'none';
                selected.push('unselected');
            }
        }
        else {
            selected.push('');
            div.style.display = '';
            for (var j = 2; j < my_level; j++) {
                selected[levels_menu[j] - 1] = '';
                text_divs[levels_body[j]].style.display = '';
            }
        }
        levels_menu[my_level] = selected.length;

        if (div.id != 'menu') {
            insert_highlight(div, html, v);
        }
    }
    // After because it was recreated
    var m;
    for (var i in menu.childNodes) {
        m = menu.childNodes[i];
        if (!m.getAttribute)
            continue;
        j = m.getAttribute('toc_number');
        if (j === undefined)
            continue;
        j = Number(j);
        m.className = class_divs[j] + ' ' + selected[j + 1];
        if (m.innerHTML !== undefined
            && m.innerHTML.search(RegExp(v, 'gim')) != -1)
            m.className += ' filter';
    }
    set_current();
    highlight();
}

function find_hash(h, fct) {
    if (h === '')
        return;
    var links = document.getElementsByTagName('A');
    for (var i = 0; i < links.length; i++)
        if (h == links[i].name)
            if (fct)
                fct(links[i]);
            else
                return links[i];
}

tips = document.getElementsByTagName('IMG');
for (var i in tips)
    if (tips[i].className == 'tip')
        tips[i].parentNode.setAttribute('tipped', "1");

var text_divs = [];
var content_divs = [];
var class_divs = [];

var hash_found = find_hash(window.location.hash.toString().substr(1));

// Search missing links

var all_names = {};
aa = document.getElementsByTagName('A');
for (var i in aa)
    if (aa[i].name)
        all_names[aa[i].name] = true;

var missing = {};
for (var i in aa)
    if (aa[i].href
        && aa[i].href.match(location.pathname)
    ) {
        if (!all_names[aa[i].href.split('#')[1]])
            missing[aa[i].href.split('#')[1]] = true;
        aa[i].href = "javascript:go_clear('#" + aa[i].href.split('#')[1] + "')";
    }
e = document.getElementsByTagName('H1')[0];
s = '';
for (var i in missing)
    if (i !== 'undefined')
        s += ' ' + i;
if (s !== '')
    e.innerHTML += '<br>' + s;

// Parse document content to create the Index of the menu

var menu_index = 0;
for (var i_div in divs) {
    if (i_div == 0)
        continue;
    var div = divs[i_div];
    if (div.parentNode && div.parentNode.className != 'top')
        continue;
    for (var ee in div.childNodes) {
        var e = div.childNodes[ee];
        if (e.tagName == 'A' && '#' + e.name == window.location.hash.toString())
            hash_found = true;
        if (e.tagName && e.tagName.length == 2) {
            text_divs.push(div);
            var m = document.createElement('A');
            if (div.firstElementChild.tagName == 'A'
                && div.firstElementChild.name !== '')
                m.href = 'javascript:go("#' + div.firstElementChild.name + '");';
            else
                m.href = 'javascript:go("#n' + class_divs.length + '");';
            m.innerHTML = e.innerHTML.replace(/.*>.*<.*/, '$1');
            div.toc_number = class_divs.length;
            m.setAttribute('toc_number', class_divs.length);
            m.setAttribute('the_div', i_div);
            if (e.tagName == 'H2' || e.tagName == 'H3'
                || (window.level4 && e.tagName == 'H4')) {
                if (menu_index++ % 2 == 0)
                    m.className = 'even';
                else
                    m.className = 'even'; // 'odd' for zebras
            }
            else
                m.style.display = 'none';
            m.className += ' ' + e.tagName;
            // XXX e.textContent & and " should be escaped
            e.innerHTML = '<a name="n' + class_divs.length + '">'
                + '<a name="' + e.textContent + '">'
                + e.innerHTML + '</a></a>';
            if ('#' + e.textContent == window.location.hash.toString()
                || "#n" + class_divs.length == window.location.hash.toString())
                hash_found = true;
            class_divs.push(m.className);
            content_divs.push(div.innerHTML);
            menu.appendChild(m);
            break;
        }
    }
}
text_divs.push(menu);
content_divs.push(menu.innerHTML);

if (window.location.hash !== '' && !hash_found) {
    input.value = decodeURI(window.location.hash.toString().substr(1));
}
display = true;

input.focus();

// setInterval(filter, 100);
document.onscroll = filter;

var last_current = [];
function highlight() {
    current = [];
    find_hash(location.hash.toString().substr(1),
        function (e) {
            e = e.parentNode;
            if (e.tagName.match(/H./))
                e = e.parentNode;
            current.push(e);
        });
    if (last_current[0] === current[0])
        return;
    for (var i in last_current)
        last_current[i].setAttribute('current', 'no');
    for (var i in current)
        current[i].setAttribute('current', "yes");
    last_current = current;
    if (current.length > 1)
        console.log(current.length + " balises avec le nom «"
            + location.hash + "»");
    if (current.length)
        current[0].scrollIntoView({ block: "start", XXXbehavior: "smooth" });
}

window.onhashchange = function () { display = true; highlight(); };

if (hash_found) {
    input.old_value = input.value;
}
else
    setTimeout("document.body.scrollTop = 0 ;", 200);

filter();
