/*
    TOMUSS: The Online Multi User Simple Spreadsheet
    Copyright (C) 2008-2015 Thierry EXCOFFIER, Universite Claude Bernard

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

// Constants
var vertical_scrollbar_width = 17;
var horizontal_scrollbar_height = 10;
var nr_headers = 2;
var bs = '<td>';
var maximum_url_length = 3000;
var is_a_teacher = false;
var periodic_work_period = 100; // In millisecs
// Work value
var element_focused;           // If undefined: it is the current_cell
var line_offset;		// The page being displayed
var column_offset;
var filters;			// The filters to apply to the lines
var nr_new_lines;		// Number of created lines
var nr_new_columns;		// Number of created columns
var nr_not_empty_lines;        // Number of non empty lines
var nr_not_fully_empty_lines;  // Number of non empty lines
var nr_filtered_not_empty_lines;        // Number of non empty lines
var nr_filtered_not_fully_empty_lines;  // Number of non empty lines
var sort_columns;		// Define the sort columns
var table;			// The table displayed on the screen
var tr_title;			// The header TR element for 'title'
var tr_filter;
var i_am_the_teacher;
var i_am_root;
var columns_filter;
var full_filter;
var line_filter;
var tr_classname;		// Column containing the className of the line
var popup_on_red_line;
var do_not_read_option;	// Option disabled for virtual tables
var the_current_cell;
var today;
var debug_window;
var filtered_lines;
var table_fill_do_not_focus;
var table_fill_force_current_cell_update;
var table_fill_hook;
var next_page_col;
var next_page_line;
var highlight_list;
var connection_state;
var auto_save_running;
var scrollbar_right;
var ask_login_list;
var first_day;
var last_day;
var current_window_width;
var current_window_height;
var table_info = []; // see middle.js
var last_user_interaction = 0; // setted with millisec()
var zebra_step;
var is_a_virtual_ue;
var shift_key_pressed = 0; // Number of shift key pressed
var set_the_tip_content; // If defined, then the tip content is this value

// HTML elements
var divtable;
var author;
var modification_date;
var server_log;
var the_body;
var nr_filtered_lines;
var the_comment;
var linefilter;
var horizontal_scrollbar;
var vertical_scrollbar;
var t_student_picture;
var t_student_firstname;
var t_student_surname;
var t_student_id;
var t_value;
var t_history;
var t_editor;
var t_date;
var t_author;
var t_menutop;
var thetable;

// Redefined if needed
var root;
var my_identity;
var days, days_full, months, months_full, ampms, ampms_full;
var contains_pm;
var css_themes = ["", "G", "A", "P", "D", "R", "BW"]; // In style.css
var rgpd_link;

// Give 1 if it is an ABJ/PPN value
var is_abjppn = {};
// Give 1 if it is an ABJ value
var is_abj = {};
// Give 1 if it is an ABI value
var is_abi = {};

function lib_init() {
    is_abjppn[abj] = 1;
    is_abjppn[abj_short] = 1;
    is_abjppn[ppn] = 1;
    is_abjppn[ppn_short] = 1;
    for (var i in allowed_grades)
        if (is_abjppn[allowed_grades[i][0]])
            is_abjppn[i] = 1;
    is_abj[abj] = 1;
    is_abj[abj_short] = 1;
    for (var i in allowed_grades)
        if (is_abj[allowed_grades[i][0]])
            is_abj[i] = 1;
    is_abi[abi] = 1;
    is_abi[abi_short] = 1;
    for (var i in allowed_grades)
        if (is_abi[allowed_grades[i][0]])
            is_abi[i] = 1;

    divtable = document.getElementById('divtable');
    author = document.getElementById('author');
    modification_date = document.getElementById('date');
    server_log = document.getElementById('log');
    the_body = document.getElementById('body');
    if (the_body)
        the_body.style.overflowX = 'hidden';
    nr_filtered_lines = document.getElementById('nr_filtered_lines');
    the_comment = document.getElementById('comment');
    linefilter = document.getElementById('linefilter');
    horizontal_scrollbar = document.getElementById('horizontal_scrollbar');
    vertical_scrollbar = document.getElementById('vertical_scrollbar');
    t_student_picture = document.getElementById('t_student_picture');
    t_student_firstname = document.getElementById('t_student_firstname');
    t_student_surname = document.getElementById('t_student_surname');
    t_student_id = document.getElementById('t_student_id');
    t_value = document.getElementById('t_value');
    t_history = document.getElementById('t_history');
    t_editor = document.getElementById('t_editor');
    t_date = document.getElementById('t_date');
    t_author = document.getElementById('t_author');
    t_menutop = document.getElementById('menutop');

    if (root === undefined)
        root = [];
    if (my_identity === undefined)
        my_identity = 'identity undefined';
    try {
        is_a_virtual_ue = ue === 'VIRTUALUE' || ue === '' || page_id <= 0;
    }
    catch (e) {
        is_a_virtual_ue = true;
    }
    line_offset = 0;// The page being displayed
    column_offset = 0;
    filters = [];// The filters to apply to the lines
    nr_new_lines = 0;// Number of created lines
    nr_new_columns = 0;// Number of created columns
    sort_columns = [];// Define the sort columns
    i_am_the_teacher = false;
    highlight_list = [];
    columns_filter = compile_filter_generic('');
    prst_is_input = true;
    popup_on_red_line = true;
    do_not_read_option = false; // Option disabled for virtual tables
    the_current_cell = new Current();
    connection_state = new Connection();
    auto_save_running = false;
    i_am_root |= myindex(root, my_identity) != -1;

    if (divtable) // In a table
    {
        compute_nr_cols();
        compute_nr_lines();
    }
    current_window_height = window_height();
    current_window_width = window_width();

    today = new Date();
    today = today.getFullYear() + two_digits(today.getMonth() + 1) +
        two_digits(today.getDate()) + '000000';

    _today = new Date();
    _today.setHours(0, 0, 0, 0);

    if (isNaN(first_day))
        first_day = 0;
    if (isNaN(last_day))
        last_day = 86400000 * 365 * 1000; // parse_date('31/12/2100').getTime() ;

    days = eval(_("MSG_days"));
    days_full = eval(_("MSG_days_full"));
    months = eval(_("MSG_months"));
    months_full = eval(_("MSG_months_full"));
    ampms = eval(_("MSG_ampms"));
    ampms_full = eval(_("MSG_ampms_full"));
    contains_pm = new RegExp('.*(' + ampms[1] + '|' + ampms[1].toLowerCase() + ').*');

    function update_tip() {
        if (!shift_key_pressed)
            return;
        show_the_tip(element_focused || the_current_cell.td || update_tip.td);
    }

    var body = document.getElementsByTagName('BODY')[0];
    if (!body)
        return; // nodejs
    body.addEventListener(
        'keydown',
        function (event) {
            shift_key_pressed += event.key == 'Shift';
            update_tip.td = event.target;
            if (shift_key_pressed)
                periodic_work_add(update_tip);
        }, { capture: true, passive: true });

    body.addEventListener(
        'keyup',
        function (event) {
            shift_key_pressed -= event.key == 'Shift';
            if (!event.shiftKey || shift_key_pressed < 0)
                shift_key_pressed = 0;
            update_tip.td = event.target;
            if (shift_key_pressed)
                periodic_work_add(update_tip);
            else
                hide_the_tip_real();
        }, { capture: true, passive: true });
    body.addEventListener('mouseover', hidden_over,
        { capture: true, passive: false });
    body.addEventListener('mouseout', hidden_out,
        { capture: true, passive: false });

}

function add_ticket(path, more) {
    if (ticket == 'none') // Shared suivi page
        return path;

    return (more === undefined ? url + '/' + path : path + '/' + more)
        + '?%E2%9C%80_________________________________________________________________'
        + ticket;
}

function add_ticket_checker(node) {
    if (ticket == 'none') // Shared suivi page
        return;
    if (window.check_ticket_time !== undefined
        && window.check_ticket_time > millisec() - 10000)
        return; // Wait 10 seconds before checking again
    window.check_ticket = undefined;
    window.check_ticket_time = millisec();
    var script = document.createElement('SCRIPT');
    script.src = add_ticket(url, 'check_ticket/' + millisec());
    node.appendChild(script);
    setTimeout(
        function () {
            if (window.check_ticket != 'True')
                Alert('MSG_bad_ticket');
        },
        10000); // Wait 10 seconds the server answer
}

function get_url_year_semester() {
    var t = window.location.pathname.split('/');
    for (var i in t)
        if (t[i] > 2000)
            return t[i] + '/' + t[Number(i) + 1];
}

function _d(_txt) {
}

function charsize() {
    return document.getElementById("charsize").offsetWidth;
}

function char_per_line() {
    return window_width() / charsize();
}

function compute_nr_cols() {
    table_attr.nr_columns = Math.floor(char_per_line() / 8.5);
    if (table_attr.nr_columns <= 0)
        // Needed for 'statistics_per_group' virtual table
        table_attr.nr_columns = 1;
}

var header_height;

function compute_header_height() {
    if (table) {
        document.body.style.overflow = 'hidden';
        header_height = findPosY(table.childNodes[0].childNodes[0]);
        var old = table_attr.nr_lines;
        compute_nr_lines();
        if (old !== table_attr.nr_lines) {
            table_init();
            table_fill(true, true, true);
        }
    }
}

function add_link_on_url(text) {
    return text.replace(/(https?:[-._a-zA-Z0-9/]*)/g, '<a target="_blank" href="$1">$1</a>');
}

function compute_nr_lines() {
    if (!header_height) {
        table_attr.nr_lines = zebra_step + 1;
        compute_nr_lines.do_compute_nr_lines = true;
        periodic_work_add(compute_header_height);
        return;
    }
    if (!compute_nr_lines.do_compute_nr_lines)
        return; // Value set by the preference.
    if (the_current_cell.input) {
        // Number of displayed lines on the screen
        var line_height;
        try {
            line_height = (table.childNodes[nr_headers + zebra_step].offsetTop
                - table.childNodes[nr_headers].offsetTop) / zebra_step;
        } catch (e) {
            // The table is too small
            line_height = (table.childNodes[nr_headers + 1].offsetTop
                - table.childNodes[nr_headers].offsetTop);
        };
        table_attr.nr_lines = (window_height() - header_height
            - horizontal_scrollbar.offsetHeight
            /* XXX Magic number for Chrome/Opera */
            - 15 // horizontal_scrollbar.offsetHeight ???
        ) / line_height;
        table_attr.nr_lines = Math.floor(table_attr.nr_lines) - nr_headers;
        compute_nr_lines.do_compute_nr_lines = false;
    }

    if (table_attr.nr_lines < 3)
        table_attr.nr_lines = 3;
}

/*
 * Standard Variable name used in all the code :
 * line_id  : index of the line in 'lines'
 * lin      : index of the line in 'table'
 * data_col : index of the column in 'lines[line_id]'
 * col      : index of the column in 'table[lin]'
 * column   : is columns[data_col]
 * line     : is lines[line_id]
 * tr       : is 'table[lin]'
 * td       : is 'table[lin][col]' attributes : line_id, data_col, lin, col
 * type     : a type of column
 * type_i   : type index
 * type_txt : textual type
 */

function data_col_from_col_id(col) {
    for (var i in columns)
        if (columns[i].the_id == col)
            return Number(i);
}

function data_col_from_col_title(title) {
    for (var i in columns)
        if (columns[i].title == title)
            return Number(i);
}

// Index in 'filtered_lines'
function lin_from_line_id(line_id) {
    var lin;

    lin = myindex(filtered_lines, lines[line_id]) - line_offset;
    if (lin < 0 || lin >= table_attr.nr_lines)
        return;
    return lin;
}

function td_from_line_id_data_col(line_id, data_col) {
    var col, lin;

    col = columns[data_col].col;
    if (col === undefined)
        return;

    lin = lin_from_line_id(line_id);
    if (lin === undefined)
        return;

    return table.childNodes[lin + nr_headers].childNodes[col];
}

function col_from_td(td) {
    return myindex(td.parentNode.childNodes, td);
}

function data_col_from_col(col) {
    return column_list(column_offset, col + 1)[col].data_col;
}

function data_col_from_td(td) {
    return data_col_from_col(col_from_td(td));
}

function column_from_td(td) {
    return columns[data_col_from_td(td)];
}

function lin_from_td(td) {
    return myindex(td.parentNode.parentNode.childNodes, td.parentNode);
}

function line_id_from_lin(lin) {
    var line = line_offset + lin - nr_headers;
    if (line >= filtered_lines.length)
        return;
    if (line < 0)
        return;
    return filtered_lines[line].line_id;
}

function line_id_from_td(td) {
    return line_id_from_lin(lin_from_td(td));
}

// The parameter can be an event or an HTML element
function the_td(event) {
    var td;
    if (event && event.tagName)
        td = event;
    else {
        event = the_event(event);
        if (event === undefined)
            return;
        td = event.target;
    }
    if (td.tagName == 'INPUT' || td.tagName == 'SELECT'
        || (td.tagName == 'IMG' && td.className != 'server')
        || td.tagName == 'A' || td.tagName == 'BUTTON')
        return td.parentNode;
    else {
        while (td.tagName != 'TD' && td.tagName != 'TH')
            td = td.parentNode;

        return td;
    }
}

function the_input(event) {
    return the_event(event).target;
}

function get_recorded_options() {
    try {
        return localStorage['/' + year + '/' + semester + '/' + ue + '/options'];
    } catch (e) {
        localStorage.clear();
    }
}

function set_recorded_options(x) {
    delete localStorage['/' + year + '/' + semester + '/' + ue + '/options'];
    if (x && x !== '')
        localStorage['/' + year + '/' + semester + '/' + ue + '/options'] = x;
}

function get_option(name, default_value, do_not_unescape) {
    var o;

    o = window.location.pathname.split('=' + name + '=');
    if (o.length == 1) {
        var h = get_recorded_options();
        if (h)
            o = h.split('=' + name + '=');
    }
    if (o.length == 1)
        return default_value;
    o = o[1].split('/')[0];
    if (!do_not_unescape)
        o = decodeURI(o);
    return o;
}

function record_options() {
    var h = window.location.toString().split('/' + ue + '/');
    if (h.length == 2)
        set_recorded_options(h[1]);
}

function previous_year_semester(year, semester) {
    var i = myindex(semesters, semester);
    if (i == -1)
        return [year - 1, semester];
    i = (i + semesters.length - 1) % semesters.length;
    if (i != semesters.length - 1)
        return [year, semesters[i]];
    else
        return [year - 1, semesters[i]];
}

function next_year_semester(year, semester) {
    year = Number(year);
    var i = myindex(semesters, semester);
    if (i == -1)
        return [year + 1, semester];
    i = (i + 1) % semesters.length;
    if (i != 0)
        return [year, semesters[i]];
    else
        return [year + 1, semesters[i]];
}

function next_year_semester_number(year, semester) {
    year = Number(year);
    semester += 1;
    if (semester == semesters.length) {
        semester = 0;
        year += 1;
    }
    return [year, semester];
}

/******************************************************************************
Function are launched on header events
******************************************************************************/

function filter_keyup(event, force) {
    var e = the_event(event);
    if (force || e.keyCode > 40 || e.keyCode == 8 || e.keyCode == 32)
        header_change_on_update(e, e.target, '');
    GUI.add("column_filter", event);
}

function empty_header(event) {
    event = the_event(event);
    var input = event.target;

    input.className = input.className.replace(/empty/g, '');
    input.onkeyup = filter_keyup;
    input.onblur = filter_unfocus;
}

function header_focus(t, event) {
    t = t.parentNode;
    the_current_cell.change();
    the_current_cell.jump(the_current_cell.lin, col_from_td(t), true);
    element_focused = t;
    empty_header(event);
}

function filter_unfocus(event) {
    element_focused = undefined;
    event = the_event(event);
    var input = event.target;
    if (input.value === '') {
        input.className += ' empty';
        input.onchange = function () { };
    }
}

function sort_columns_list() {
    var s = [];
    for (var c in sort_columns)
        if (sort_columns[c].dir > 0)
            s.push(sort_columns[c].data_col);
        else
            s.push(-sort_columns[c].data_col - 1);
    return s;
}

function sort_column_update_option() {
    change_option('sort', sort_columns_list().join('=') + '=');
}


/* The title is clicked */
function sort_column(data_col, force) {
    if (!force && periodic_work_in_queue(table_fill_do))
        return;

    if (data_col === undefined)
        data_col = the_current_cell.data_col;

    if (column_empty(data_col))
        return;

    line_offset = 0;

    if (sort_columns[0] !== undefined) {
        if (data_col == sort_columns[0].data_col) {
            sort_columns[0].dir = -sort_columns[0].dir;
            table_fill(true, true, true);
            sort_column_update_option();
            return;
        }
    }

    var t = [columns[data_col]];
    t[0].dir = 1;

    for (var i in sort_columns) {
        if (sort_columns[i] != t[0]) {
            t.push(sort_columns[i]);
        }
        if (t.length == 3) // Limit the number of sort columns
            break;
    }

    sort_columns = t;
    table_fill(true, true, true);
    sort_column_update_option();
}

function sort_column_by(data_col, what) {
    columns[data_col].sort_by = what;
    hide_the_tip_real(true);
    sort_column(data_col, true);
}

function get_tip_element() {
    var tip = document.getElementById('tip');

    if (!tip) {
        var body = document.getElementsByTagName('BODY')[0];
        tip = document.createElement('div');
        tip.id = 'tip';
        body.appendChild(tip);
        tip.highlighter = document.createElement('div');
        tip.highlighter.id = 'highlighter';
        body.appendChild(tip.highlighter);
    }
    return tip;
}

function header_title_click(t) {
    if (data_col_from_td(t.parentNode) === the_current_cell.data_col)
        return;
    last_user_interaction = millisec();
    if (element_focused && element_focused.onblur)
        element_focused.onblur({ target: element_focused });
    t = t.parentNode;
    the_current_cell.change();
    the_current_cell.jump(the_current_cell.lin, col_from_td(t), true);
}

function compute_rank(line_id, column) {
    var data_col = column.data_col;
    var the_value = a_float(lines[line_id][data_col].value);

    if (isNaN(the_value))
        return '&nbsp;';

    var v, rank = 1, nr = 0;
    for (var lin in lines) {
        v = lines[lin][data_col].value;
        if (v !== '') {
            nr++;
            if (v > the_value)
                rank++;
        }
    }
    return rank + '/' + nr;
}

function line_resume(line_id) {
    var s, column, h;
    h = Math.floor(window_height() - header_height);
    if (isNaN(h))
        h = window_height();
    s = '<div style="overflow:auto;max-height:' + h + 'px">'
        + '<table class="colored line_resume" style="max-width:'
        + Math.floor(window_width() * 0.75) + 'px">'
        + '<tr><th>' + _('TH_column')
        + '</th><th>' + _('TH_value')
        + '</th><th>' + _('TH_rank')
        + '</th><th>' + _('TH_comment')
        + '</th></tr>';
    for (var data_col in columns) {
        cell = lines[line_id][data_col];
        if (cell.value !== "" || cell.comment !== "") {
            column = columns[data_col];

            if (column.real_type.cell_compute)
                classe = 'ro';
            else
                classe = '';
            s += '<tr class="' + classe + '"><td style="text-align:right">' +
                html(columns[data_col].title) + '</td><td><div>' +
                (cell.value.toString().indexOf('\n') != -1 ? '<b>' : '') +
                cell.value_fixed().replace(/\n/g, '<br>') + '</div></td><td>' +
                compute_rank(line_id, column) + '</td><td>';
            if (cell.comment)
                s += '<div>' + (cell.comment.indexOf('\n') != -1 ? '<b>' : '')
                    + cell.comment_html() + '</div>';
            else
                s += '&nbsp;';
            s += '</td></tr>\n';
        }
    }
    s += '</table>';
    var x = table_attr.portails[lines[line_id][0].value];

    if (x === undefined)
        x = '';
    else {
        var xx = '';
        for (var i in x)
            xx += x[i] + '<br>';
        x = '<small>' + xx.substr(0, xx.length - 4) + '</small>';
    }
    x += '</div>';

    return s + x;
}

var the_current_line;
var instant_tip_display;

function td_from_table(td) {
    try {
        if (td === the_current_cell.input || td === the_current_cell.input_div)
            td = the_current_cell.td;
    }
    catch (err) {
        return; // Not in a table
    }
    for (var table = td; table; table = table.parentNode) {
        if (table.id === 'thetable')
            return td;
        if (table.tagName == 'TD' || table.tagName == 'TH')
            td = table;
    }
    if (table_forms_element)
        for (var table = td; table.parentNode; table = table.parentNode) {
            if (table.tagName == 'TR')
                return table;
        }
}

function show_the_tip_table_cell_real(data_col, line_id) {
    var line = lines[line_id];
    var column = columns[data_col];
    var type = column.real_type;
    if (line === undefined)
        return;
    var cell = line[data_col];
    var s;
    if (cell.modifiable(line, column) && type.tip_cell !== '')
        s = '<span class="title">' + ___(type.tip_cell) + '</span><br>';
    else
        s = '';
    if (cell.value)
        s += '<b>' + html(cell.value).replace(/\n/g, "<br>") + '</b><br>';
    if (cell.get_author() !== '')
        s += _("MSG_suivi_student_RSS_value_modified") + ' «' + cell.get_author() + '» ';
    if (cell.date)
        s += _("DisplayCellMTimeBefore") + ' «' + date(cell.date) + '»';
    s += '<br>' + _('Line') + ' ' + (1 + myindex(filtered_lines, line));

    var s_abjs = student_abjs(line[0].value);
    if (s_abjs !== "")
        s += s_abjs;
    return s;
}

function show_the_tip_table_cell(td) {
    td = td_from_table(td);
    if (!td)
        return;
    var data_col = data_col_from_td(td);
    var line_id = line_id_from_td(td);
    if (line_id === undefined)
        // Tips for the big table columns header: 'title' and 'filter'
        return _(columns[data_col].real_type[
            'tip_' + td.parentNode.className.split(' ')[0]]);
    // Tips for table cells
    return show_the_tip_table_cell_real(data_col, line_id);
}

function show_the_tip_table_form(td) {
    while (td && td.data_col === undefined)
        td = td.parentNode;
    if (td)
        return show_the_tip_table_cell_real(td.data_col, the_current_cell.line_id);
}

filter_ids = [
    'linefilter', 'columns_filter', 'fullfilter',
    't_column_cell_writable', 't_column_test_filter', 't_column_test_if',
    't_column_visibility_date',
    't_column_redtext', 't_column_greentext',
    't_column_red', 't_column_green'];

function show_the_tip_filter(td) {
    if (td.parentNode.parentNode.className.match(/\bfilter\b/)) {
        // In the INPUT
    }
    else if (td.parentNode.className.match(/\bfilter\b/))
        td = td.firstChild;
    else if (td.tagName != 'INPUT')
        return '';
    else if (td.id.substr(0, 13) == 'alert_filter_') {
        // Alert filter
    }
    else if (myindex(filter_ids, td.id) == -1)
        return '';
    else if (td.id == 'visibility_date' && user_date_to_date(value) != '9999')
        return ''; // Not a filter but a date
    else if (td.id.match(/_red|_green/) && !isNaN(td.value) && td.value == '')
        return ''; // Not a filter but a value

    var filter;
    if (td.id.substr(0, 2) == 't_')
        filter = compile_filter_generic(td.value, the_current_cell.column, true);
    else
        filter = compile_filter_generic(td.value);
    var value = filter.the_filter.editor();
    var errors = filter.get_filter_errors();
    if (errors)
        value += '<p class="attribute_error">' + errors + '</p>';
    return value;
}

function show_the_tip_picture(td) {
    if (td.id != 't_student_picture')
        return;
    return line_resume(the_current_cell.line_id);
}

function show_the_tip_average(td) {
    if (td.id != 't_column_stats')
        return;
    return current_histogram.nr + _("MSG_columnstats_values") + '<br>'
        + current_histogram.html_resume();
}

function show_the_tip_histogram(td) {
    td = td.parentNode;
    if (td.id != 't_column_histogram')
        return;
    var t = '';
    if (current_histogram.nr_nan())
        t += _("MSG_columnstats_empty") + ':' + current_histogram.nr_nan() + ' ';
    if (current_histogram.nr_ppn())
        t += _("MSG_columnstats_ppn") + ':' + current_histogram.nr_ppn() + ' ';
    if (current_histogram.nr_abi())
        t += _("MSG_columnstats_abi") + ':' + current_histogram.nr_abi() + ' ';
    if (current_histogram.nr_abj())
        t += _("MSG_columnstats_abj") + ':' + current_histogram.nr_abj() + ' ';
    if (current_histogram.nr_pre())
        t += _("MSG_columnstats_pre") + ':' + current_histogram.nr_pre() + ' ';
    if (current_histogram.nr_yes())
        t += _("MSG_columnstats_yes") + ':' + current_histogram.nr_yes() + ' ';
    if (current_histogram.nr_no())
        t += _("MSG_columnstats_no") + ':' + current_histogram.nr_no() + ' ';
    if (current_histogram.nr)
        t += _("MSG_columnstats_grade") + ':' + current_histogram.nr + ' ';
    return t;
}

function show_the_tip_generic(td) {
    var t = tip_top(td);
    if (t && t.childNodes.length && t.childNodes[0].innerHTML && t.tagName != 'TR')
        return '<div>' + t.childNodes[0].innerHTML + '</div>';
    return '';
}

function show_the_tip_input(td) {
    // td.selectedText
    // (td.firstChild ? td.firstChild.value : 'BUG')
    if (table_forms_element)
        return '';
    if (window.url_suivi)
        return ''; // No tip on INPUT on suivi
    var value;
    if (td.options) {
        if (td.options[td.selectedIndex])
            value = td.options[td.selectedIndex].textContent;
        else
            return '';
    }
    else
        value = td.value;
    if (value)
        return '<hr><div style="font-weight:bold">' + html(value) + '</div>';
    return '';
}

function show_the_tip_debug(td) {
    if (!preferences.debug_table)
        return '';
    if (!filtered_lines)
        return '';
    var lin_id = line_id_from_td(td);
    var data_col = data_col_from_td(td);
    var s = td.tagName + '#' + td.id + ' focused='
        + (element_focused ? element_focused.tagName + '#' + element_focused.id : '')
        + (lin_id !== undefined && data_col !== undefined
            ? '<br>'
            + 'line_id=' + lin_id
            + ' col_id[' + data_col + ']='
            + columns[data_col].the_id
            : '') + '<br>';

    for (var i in filters)
        s += filters[i][0] + '<br>';

    return s;
}

function show_the_tip_headers(td) {
    if (td.id === '' || td.id === undefined)
        return;
    var what;
    if (td.id.indexOf('t_column') == 0)
        what = 'column';
    else if (td.id.indexOf('t_table') == 0)
        what = 'table';
    else
        return;
    var attr = td.id.replace(/t_column_|t_table_attr_/, '');

    var tip_base = 'TIP_' + what + '_attr_' + attr;
    tip_content = _(tip_base);
    tip_exists = tip_base != tip_content;
    if (!tip_exists && what == 'column' && the_current_cell.column.type) {
        tip_id = tip_base + '__' + the_current_cell.column.type;
        tip_content = _(tip_id);
        tip_exists = tip_id != tip_content;
        if (!tip_exists) {
            tip_id = tip_base + '__';
            tip_content = _(tip_id);
            tip_exists = tip_id != tip_content;
        }
    }

    if (td.id == 't_column_type')
        tip_content += '<hr>' + _('H_' + the_current_cell.column.type);

    if (tip_exists || td.tagName != 'A')
        return tip_content;

    if (td.className.indexOf('linkstroked') == -1)
        tip_id = tip_base + '__1';
    else
        tip_id = tip_base + '__0';
    tip_content = _(tip_id);
    if (tip_id != tip_content)
        return tip_content;
}

function column_menu_focus(THIS, data_col) {
    element_focused = THIS;
    element_focused.data_col = data_col;
    element_focused.initial_value = element_focused.value;
    set_the_tip_content = function () { return ''; };
}

function column_menu_blur(show_feedback) {
    set_the_tip_content = undefined;
    if (element_focused.value == element_focused.initial_value)
        return;
    var column = columns[element_focused.data_col]
    var attr = element_focused.getAttribute('attr');
    switch (attr) {
        case 'course_dates':
            column_attr_set(column, attr,
                element_focused.value, show_feedback ? element_focused.parentNode : undefined);
            var formated = course_dates_formatter(undefined, column.course_dates);
            var header_input = document.getElementById('t_column_course_dates');
            header_input.value = element_focused.initial_value = element_focused.value = formated;
            header_input.className = formated === '' ? 'empty' : '';
            break;
        case 'visibility':
        case 'modifiable':
            column_attr_set(column, attr, element_focused.value);
            element_focused.initial_value = element_focused.value;
            the_current_cell.update_table_headers();
            the_current_cell.do_update_column_headers = true;
            table_header_fill();
    }
}

function column_menu_keydown(event) {
    event = the_event(event);
    if (event.keyCode == 13) {
        column_menu_blur(true);
        stop_event(event);
    }
}

function show_the_tip_column_menu(td) {
    if (td.getAttribute('instanttip') != "1")
        return;
    td = td_from_table(td);
    if (!td)
        return;
    if (myindex(td.parentNode.className, 'column_title') == -1)
        return;
    var what = ['LABEL_sort_value', 'LABEL_sort_date', 'LABEL_sort_author',
        'LABEL_sort_comment', 'LABEL_sort_shuffle'];
    var data_col = data_col_from_td(td);
    var column = columns[data_col];
    if (column.is_computed()) {
        what.push('LABEL_sort_%ABJ');
        what.push('LABEL_sort_stddev');
    }
    var s = [];
    for (var i in what) {
        i = what[i];
        var v = '<li><a style="color:#00F" onmousedown="sort_column_by('
            + data_col + ",'" + i + '\')">' + _(i) + '</a>';
        if (column.sort_by == i)
            v = '<b>' + v + '</b>';
        s.push(v);
    }
    var more = _("BEFORE_column_attr_course_dates") + '<br>';
    var formated = course_dates_formatter(undefined, column.course_dates);
    if (column_change_allowed(column)) {
        more += '<div><input onfocus="column_menu_focus(this,' + data_col
            + ')" value="' + encode_value(formated) + '"'
            + ' onblur="column_menu_blur()" attr="course_dates"'
            + ' instanttip="keep" onkeydown="column_menu_keydown(event)"></div>'
            + '<SELECT style="width:15em" attr="visibility"'
            + ' onfocus="column_menu_focus(this,' + data_col
            + ')" instanttip="keep" onblur="column_menu_blur()">'
            + document.getElementById('t_column_visibility').innerHTML
                .replace(/disabled=""/g, '')
                .replace(RegExp('value="' + column.visibility + '"'), '$0 selected')
            + '</SELECT><br>'
            // copy/paste
            + '<SELECT style="width:15em" attr="modifiable"'
            + ' onfocus="column_menu_focus(this,' + data_col
            + ')" instanttip="keep" onblur="column_menu_blur()">'
            + document.getElementById('t_column_modifiable').innerHTML
                .replace(/disabled=""/g, '')
                .replace(RegExp('value="' + column.modifiable + '"'), '$0 selected')
            + '</SELECT>'
            ;
    }
    else
        more += html(formated + ' ') + '<br>';

    return _('LABEL_sort_by')
        + '<ul style="padding-left:1.5em">'
        + s.join('') + '</ul>'
        + more + '</div>';
}

function debug_label(label, txt) {
    if (preferences.debug_table && txt && txt !== '')
        txt = '<div style="background:#000;color:#FFF;font-size:60%">' + label + '</div>' + txt;
    return txt;
}

function show_the_tip(td) {
    var td_in_table = td_from_table(td);
    if (popup_is_open()) {
        var popup = document.getElementById('popup_id');
        for (var element = td; element !== popup; element = element.parentNode)
            if (!element)
                return; // No tip on element outside of popup
    }
    if (td_in_table && td_in_table.parentNode !== the_current_line) {
        remove_highlight();
        the_current_line = td_in_table.parentNode;
        td_in_table.parentNode.className += ' highlight_current';
    }
    if (td.tagName == 'B')
        td = td.parentNode;

    var tip = get_tip_element();
    if (td === tip)
        return;
    if (td.getAttribute('instanttip') == 'keep')
        return;

    if (!(instant_tip_display
        || td.getAttribute('instanttip')
        || tip.tip_target && tip.tip_target.getAttribute('instanttip')
        || shift_key_pressed
        || set_the_tip_content)
        || td.offsetWidth > window_width() / 2 && td.offsetHeight > 100 // too big !!
    ) {
        hide_the_tip_real(true);
        return;
    }
    if (body_on_mouse_up_doing)
        return;
    if (set_the_tip_content)
        s = set_the_tip_content(tip);
    else
        s = debug_label('generic', show_the_tip_generic(td))
            + (
                debug_label('picture', show_the_tip_picture(td))
                || debug_label('average', show_the_tip_average(td))
                || debug_label('histogram', show_the_tip_histogram(td))
                || debug_label('headers', show_the_tip_headers(td))
                || debug_label('column_menu', show_the_tip_column_menu(td))
                || debug_label('table_cell', show_the_tip_table_cell(td))
                || debug_label('table_form', show_the_tip_table_form(td))
                || '')
            + debug_label('filter', show_the_tip_filter(td))
            + debug_label('input', show_the_tip_input(td))
            + (tip.tip_target && tip.tip_target.getAttribute('instanttip')
                ? ''
                : debug_label('debug', show_the_tip_debug(td)));
    if (s === '')
        return;
    blur_focus_in_tip();
    if (!set_the_tip_content)
        // Replacement to load big images only on display 'suivi_student.js'
        tip.innerHTML = s.replace(' newsrc="', ' src="');
    tip.tip_target = td;
    set_element_relative_position(
        tip_top(td) || (td.className == 'icon_hidden' ? td.parentNode.parentNode : td), tip);
    tip.style.visibility = "visible";
    tip.className = tip.className.replace(/tip_fade[^ ]*/, "tip_fade_in");
    tip.style.pointerEvents = td.getAttribute('instanttip') == 1 || set_the_tip_content ? 'all' : 'none';
}

function on_mouse_down(event) {
    last_user_interaction = millisec();
    var td = the_td(event);
    column_from_td(td).real_type.onmousedown(event);
    return false;
}

function wheel(event) {
    if (the_body.offsetHeight > window_height())
        return;
    if (popup_is_open())
        return;
    if (table_forms_element)
        return;
    if (element_focused && element_focused.tagName == 'TEXTAREA')
        return;
    var e = the_event(event);
    if (e.ctrlKey || e.altKey || e.metaKey || e.shiftKey)
        return;
    var where = e.target;
    while (where.tagName != 'BODY') {
        if (where.scrollHeight > where.offsetHeight)
            return; // Use normal scroll
        where = where.parentNode;
    }
    var old_line_offset = line_offset;
    var old_lin = the_current_cell.lin;
    if (e.wheelDelta < 0) {
        next_page(undefined, zebra_step);
        if (old_line_offset != line_offset)
            the_current_cell.lin = Math.max(nr_headers + preferences.one_line_more,
                old_lin - zebra_step);
    }
    else {
        previous_page(undefined, zebra_step);
        if (old_line_offset != line_offset)
            the_current_cell.lin = Math.min(nr_headers + table_attr.nr_lines
                - 1 - preferences.one_line_more,
                old_lin + zebra_step);
    }
    stop_event(event);
}

// Helper functions

function column_parse_attr(attr, value, column, xcolumn_attr) {
    // xcolumn_attr :
    //    undefined : initialisation in a new column with default values
    //    false     : user interaction
    //    0         : initialisation of the attributes in an existing column 
    //    true      : value received from another user
    return column_attributes[attr].check_and_set(value, column, xcolumn_attr);
}

function column_modifiable_attr(attr, column) {
    if (!column_attributes[attr])
        return false;

    if (column_attributes[attr].visible_for.length)
        return myindex(column_attributes[attr].visible_for, column.type) >= 0;
    else
        return true;
}

/******************************************************************************
Table initialization. Only done once
******************************************************************************/

// 'set' each of the column attributes
function init_column(column) {
    column.real_type = type_title_to_type(column.type);

    for (var attr in column_attributes)
        column_parse_attr(attr,
            column_attributes[attr].formatter(column, column[attr]),
            column,
            // XXX: 0, 1, false and true are all their meanings
            column_modifiable_attr(attr, column) ? 0 : true
        );
}

var use_touch = true;

function table_move(event) {
    if (!thetable)
        return;
    event = the_event(event);
    if (body_on_mouse_up_doing == "column_drag") {
        if (event.target.tagName != 'TH' && event.target.tagName != 'TD')
            return;
        var datacol = data_col_from_td(event.target);
        if (thetable.column_drag == datacol)
            return;
        var cols = column_list();
        for (var i in cols)
            if (cols[i].data_col == datacol)
                break;
        var drag_position = columns[thetable.column_drag].position;
        var position = columns[datacol].position;
        var previous_position = cols[Number(i) - 1]
            ? cols[Number(i) - 1].position
            : position - 1;
        var next_position = cols[Number(i) + 1].position;
        var new_pos = drag_position > position
            ? (previous_position + position) / 2
            : (position + next_position) / 2;
        //console.log(thetable.column_drag + '(' + drag_position + ') → '
        //    + datacol + '(' + previous_position + ' ' + position + ' ' + next_position + ') ' + new_pos);
        if (new_pos != drag_position) {
            columns[thetable.column_drag].position = new_pos;
            table_fill_real();
            table_header_fill();
        }
        return;
    }

    var d = (thetable.start_drag_y - event.y)
        / the_current_cell.input.offsetHeight;
    line_offset = thetable.start_line_offset + Math.floor(d);
    if (line_offset < 0)
        line_offset = 0;

    d = (thetable.start_drag_x - event.x) / the_current_cell.input.offsetWidth;
    column_offset = thetable.start_column_offset + Math.floor(d);
    if (column_offset + table_attr.nr_columns
        > columns.length - nr_new_columns)
        column_offset = columns.length - table_attr.nr_columns - nr_new_columns;
    if (column_offset < 0)
        column_offset = 0;
    change_option('column_offset', column_offset ? column_offset : '');


    if (thetable.last_column_offset != column_offset
        || thetable.last_line_offset != line_offset)
        table_fill(undefined, thetable.last_column_offset != column_offset,
            false, true);

    thetable.last_column_offset = column_offset;
    thetable.last_line_offset = line_offset;
}

function start_table_drag(event) {
    the_current_cell.change();
    event = the_event(event);
    if (event.target.tagName == "TH") {
        thetable.column_drag = data_col_from_td(event.target);
        body_on_mouse_up_doing = "column_drag";
        thetable.column_initial_pos = columns[thetable.column_drag].position;
    }
    else if (event.target.tagName == "TD") {
        thetable.start_line_offset = line_offset;
        thetable.start_column_offset = column_offset;
        thetable.start_drag_y = event.y;
        thetable.start_drag_x = event.x;
        body_on_mouse_up_doing = "table_drag";
    }
    else
        return;

    set_body_onmouseup(); // ??? Why not working in HTML TAG
    window.onmousemove = table_move;
    stop_event(event);
}

function do_touchstart(event) {
    do_touchstart.touch_device = true;
    if (!thetable)
        return;
    the_current_cell.change();
    event = the_event(event);
    if (event.one_finger) {
        thetable.start_line_offset = line_offset;
        thetable.start_column_offset = column_offset;
        thetable.start_drag_y = event.y;
        thetable.start_drag_x = event.x;
    }
    else
        thetable.start_line_offset = undefined;
}

function do_touchmove(event) {
    var e = the_event(event);
    if (e.one_finger) {
        if (thetable.start_line_offset !== undefined) {
            table_move(event);
            stop_event(event);
        }
    }
    else
        thetable.start_line_offset = undefined; // No more panning
}



// table innerHTML is not supported by anyone

var colgroup;

function table_init() {
    if (table_init.running) // FireFox multithreaded JS
        return;
    table_init.running = true;

    // Create the table
    // The first child is HOVER
    while (divtable.childNodes[1])
        divtable.removeChild(divtable.childNodes[1]);
    thetable = document.createElement('table');
    thetable.className = "colored";
    if (preferences.unmodifiable == 1)
        thetable.className += " leaves";
    thetable.id = 'thetable';
    thetable.style.tableLayout = "fixed";

    if (vertical_scrollbar) {
        var m, sb_width = vertical_scrollbar_width + 3;
        var table_width = (divtable.offsetWidth - sb_width).toFixed(0);
        var pos = findPos(divtable);
        vertical_scrollbar.style.top = pos[1] + vertical_scrollbar_width + 'px';
        if (scrollbar_right)
            m = "0px " + sb_width + "px 0em 1px";
        else
            m = "0px 0px 0px " + sb_width + 'px';
        thetable.style.margin = m;
        thetable.style.width = table_width + 'px';
        // Use timeout because of a firefox3 bug...
        setTimeout(function () { thetable.style.width = table_width + 'px'; }, 100);
    }

    divtable.appendChild(thetable);

    _d('table created\n');

    colgroup = document.createElement('COLGROUP');
    thetable.appendChild(colgroup);
    for (var i = 0; i < table_attr.nr_columns; i++)
        colgroup.appendChild(document.createElement('COL'));


    // Create the table
    table = document.createElement('tbody');
    table.id = 'table';
    thetable.appendChild(table);
    thetable.onmousedown = start_table_drag;
    if (use_touch)
        try {
            table.addEventListener("touchstart", do_touchstart, false);
            table.addEventListener("touchmove", do_touchmove, false);
        } catch (e) { };

    // Header lines

    tr_title = document.createElement('tr');
    tr_title.className = 'column_title';
    var th = document.createElement('th');
    th.innerHTML = '<div onclick="header_title_click(this);">'
        + '<span>&nbsp;</span>'
        + '<div instanttip="1" onclick="header_title_click(this.parentNode);sort_column();GUI.add(\'column_sort\',\'\',the_current_cell.column.title)"></div></div>';
    for (var i = 0; i < table_attr.nr_columns; i++) {
        var th2 = th.cloneNode(true);
        tr_title.appendChild(th2);
        th2.type = tr_title.className;
        th2.tip_align = "right_if_small";
    }
    table.appendChild(tr_title);

    th.innerHTML = '<INPUT TYPE="TEXT" placeholder="' + _("filtre.png")
        + '" onfocus="header_focus(this,event)" onblur="element_focused=undefined" oninput="filter_keyup(event, true)">';
    tr_filter = document.createElement('tr');
    tr_filter.className = 'filter';
    for (var i = 0; i < table_attr.nr_columns; i++) {
        var th2 = th.cloneNode(true);
        th2.onclick = empty_header;
        th2.onpaste = header_paste;
        tr_filter.appendChild(th2);
        th2.type = tr_filter.className;
    }
    table.appendChild(tr_filter);

    // Content lines
    var td = document.createElement('td');
    td.innerHTML = '&nbsp;';
    var tr = document.createElement('tr');
    for (var i = 0; i < table_attr.nr_columns; i++)
        tr.appendChild(td.cloneNode(true));

    for (var i = 0; i < table_attr.nr_lines; i++) {
        var t = tr.cloneNode(true);
        if (i % zebra_step === 0)
            t.zebra = 'separator ';
        else
            t.zebra = '';
        table.appendChild(t);
    }

    _d('headers inited\n');

    for (var lin = 0; lin < table_attr.nr_lines + nr_headers; lin++) {
        tr = table.childNodes[lin];

        for (var col = 0; col < table_attr.nr_columns; col++) {
            td = tr.childNodes[col];
            if (lin >= nr_headers)
                td.onmousedown = on_mouse_down;
        }
    }
    table_init.running = false;
}

/******************************************************************************
Update one line of the table
******************************************************************************/

function update_line(line_id, data_col) {
    var line = lines[line_id];
    if (line_empty(line))
        return;

    var column = columns[data_col];
    column.need_update = true;
    update_columns(line);

    if (table === undefined)
        return;

    var lin = lin_from_line_id(line_id);
    if (lin === undefined)
        return;

    var tr = table.childNodes[lin + nr_headers];

    for (var data_col2 in columns) {
        column = columns[data_col2];
        if (column.col === undefined)
            continue;
        if (column.real_type.cell_compute === undefined
            && column.red.indexOf('[') == -1
            && column.green.indexOf('[') == -1
            && column.redtext.indexOf('[') == -1
            && column.greentext.indexOf('[') == -1
            && column.visibility_date.indexOf('[') == -1
            && column.cell_writable.indexOf('[') == -1
        )
            continue;
        update_cell(tr.childNodes[column.col], line[data_col2], column,
            the_student_abjs[line[0].value], line);
    }
}

function update_cell_at(line_id, data_col) {
    if (table === undefined)
        return;
    var lin = lin_from_line_id(line_id);
    if (lin === undefined)
        return;
    var col = columns[data_col].col;
    if (col === undefined)
        return;
    var tr = table.childNodes[lin + nr_headers];
    update_cell(tr.childNodes[col], lines[line_id][data_col],
        columns[data_col], undefined, lines[line_id]);
}

/******************************************************************************
Update the header of the table
******************************************************************************/

function set_columns_filter(h) {
    var cf = document.getElementById('columns_filter');
    if (h === '')
        cf.className = 'empty';
    else
        cf.className = '';
    if (cf.value != h)
        cf.value = h;
    columns_filter = compile_filter_generic(h);
    try {
        columns_filter(undefined, C());
    }
    catch (e) {
        columns_filter = compile_filter_generic('=');
        columns_filter.errors = "BUG";
    }
    if (columns_filter.errors)
        cf.className = 'attribute_error';
}


function columns_filter_change(v) {
    if (columns_filter.filter == v.value)
        return;

    set_columns_filter(v.value);

    column_offset = 0;
    table_fill(true, true, true);

    change_option('columns_filter', encode_uri_option(columns_filter.filter));
    change_option('column_offset');
}

function column_list_full_filter_hide(column, data_col) {
    if (column.column_list_full_filter !== undefined)
        return column.column_list_full_filter;

    column.column_list_full_filter = false;
    if (full_filter && !column.is_empty) {
        for (var lin in filtered_lines) {
            if (full_filter(filtered_lines[lin], filtered_lines[lin][data_col], my_identity))
                return false;
        }
        column.column_list_full_filter = true;
        return true;
    }
}

function column_list(col_offset, number_of_cols) {
    if (col_offset === undefined)
        col_offset = column_offset;
    if (number_of_cols === undefined)
        number_of_cols = table_attr.nr_columns;

    var freezed = [];
    var cl = [];
    for (var data_col in columns) {
        var column = columns[data_col];

        if ((column.freezed == 'F' || column.filter_freeze)
            && column.hidden != 1) {
            freezed.push(column);
            continue;
        }
        if (column.hidden == 1)
            continue;
        if (column.private.length && column.author != my_identity
            && myindex(column.private, my_identity) == -1
            && zebra_step // Not the suivi page
        )
            continue;
        var v = C(column.title, column.author, '20080101', column.comment);
        if (!columns_filter(undefined, v, my_identity) && !column.is_empty)
            continue;
        if (column_list_full_filter_hide(column, data_col))
            continue;
        cl.push(column);
    }

    freezed.sort(function (a, b) { return a.position - b.position; });
    cl.sort(function (a, b) { return a.position - b.position; });

    if (number_of_cols > freezed.length)
        return freezed.concat(cl.slice(col_offset,
            col_offset + number_of_cols - freezed.length));
    return freezed;
}

/* This function is used by CSV and print
   because there is no scroll and no empty columns
*/

function column_list_all() {
    var cl = column_list(0, columns.length);
    var cl2 = [];
    for (var c in cl) {
        var data_col = cl[c].data_col;
        if (column_empty(data_col))
            continue;
        cl2.push(data_col);
    }
    return cl2;
}

function next_column_from_data_col(data_col) {
    var cl = column_list_all();
    for (var c in cl) {
        if (cl[c] == data_col) {
            return cl[Number(c) + 1];
        }
    }
}


function nr_freezed() {
    var i;
    i = 0;
    for (var data_col in columns)
        if (columns[data_col].freezed == 'F')
            i++;
    return i;
}

function update_horizontal_scrollbar_cursor() {
    var sorted = sort_columns[0].data_col;
    //  var sorted = sort_columns[0].data_col ;
    for (var cc in horizontal_scrollbar.childNodes) {
        var c = horizontal_scrollbar.childNodes[cc];
        if (c && c.className) {
            var classe = c.className.replace(/ (cursor|sorted|filtered)/g, '');
            if (c.data_col == the_current_cell.data_col)
                classe += ' cursor';
            if (sorted == c.data_col)
                classe += ' sorted';
            if (columns[c.data_col].filter !== '')
                classe += ' filtered';
            c.className = classe;
        }
    }
}

// IE bug for BODY, must use document for this event
function set_body_onmouseup() {
    GUI.add(body_on_mouse_up_doing, '', 'start');

    if (window.onmouseupold === undefined)
        window.onmouseupold = window.onmouseup;
    window.onmouseup = body_on_mouse_up;

    /* // Does not work for Chrome to detect the cursor moving outside window
    window.onmouseout = function(event) {
      if ( event.target.tagName == 'TABLE' )
        {
      window.onmouseout = function() {} ;
              body_on_mouse_up(event) ;
        }
    } ;
    */
}

function move_horizontal_scrollbar_begin(event) {
    if (element_focused && element_focused.onblur)
        element_focused.onblur(event);
    the_current_cell.focus(); // Take focus to do the necessary 'blurs'
    var col = the_event(event).target.col;
    page_horizontal(0, col);
    body_on_mouse_up_doing = "horizontal_scrollbar_drag";
    set_body_onmouseup(); // ??? Why not working in HTML TAG
    window.onmousemove = function (event) {
        var x = the_event(event).x;
        var b;
        for (var a in horizontal_scrollbar.childNodes) {
            a = horizontal_scrollbar.childNodes[a];
            if (a.style && x < a.xcoord) {
                if (b)
                    page_horizontal(0, b.col);
                else
                    page_horizontal(0, a.col);
                break;
            }
            b = a;
        }
    };
    stop_event(the_event(event));
}

function move_horizontal_scrollbar_move(event) {
    if (body_on_mouse_up_doing == "horizontal_scrollbar_drag")
        page_horizontal(0, the_event(event).target.col);
}


function update_horizontal_scrollbar(cls) {
    if (horizontal_scrollbar === undefined)
        return;

    if (cls === undefined)
        cls = column_list();

    var width = 0;
    var cls_all = column_list(0, columns.length);
    for (var col in cls_all) {
        if (cls_all[col].is_empty)
            continue;
        width += cls_all[col].width;
    }
    var pos = 0;
    var dx;
    var hwidth;
    if (vertical_scrollbar) {
        if (scrollbar_right)
            dx = horizontal_scrollbar_height;
        else
            dx = vertical_scrollbar_width + 2;
        hwidth = window_width() - dx - horizontal_scrollbar_height - 2;
    }
    else {
        dx = horizontal_scrollbar_height;
        hwidth = window_width() - 2 * horizontal_scrollbar_height;
    }
    var left = horizontal_scrollbar.parentNode.childNodes[0];
    var right = horizontal_scrollbar.parentNode.childNodes[2];

    left.style.height = horizontal_scrollbar_height + 2 + 'px';
    right.style.height = horizontal_scrollbar_height + 2 + 'px';

    left.style.left = dx - horizontal_scrollbar_height + 'px';
    right.style.left = dx + hwidth + 'px';

    while (horizontal_scrollbar.firstChild)
        horizontal_scrollbar.removeChild(horizontal_scrollbar.firstChild);

    for (var col in cls_all) {
        if (cls_all[col].is_empty)
            continue;
        var a = document.createElement('a');
        a.className = 'column_invisible';
        for (var c in cls)
            if (cls[c] == cls_all[col]) {
                a.className = 'column_visible';
                break;
            }
        a.xcoord = (dx + ((hwidth * pos) / width)).toFixed(0);
        a.style.left = a.xcoord + 'px';
        a.style.width = (hwidth * cls_all[col].width / width).toFixed(0) + 'px';
        a.innerHTML = cls_all[col].title;
        a.data_col = cls_all[col].data_col;
        a.col = col;
        a.href = "javascript:page_horizontal(0," + col + ");";

        a.onmousedown = move_horizontal_scrollbar_begin;

        horizontal_scrollbar.appendChild(a);
        pos += cls_all[col].width;
    }
    update_horizontal_scrollbar_cursor();
}



function sb_height() {
    return divtable.offsetHeight - 2 * vertical_scrollbar_width;
}

function sb_line_to_pixel(line) {
    var height = filtered_lines.length;
    if (height === 0)
        return vertical_scrollbar_width;
    else
        return (sb_height() * line) / height + vertical_scrollbar_width;
}

function sb_pixel_to_line(pixel, dont_cut) {
    var line;

    line = (filtered_lines.length * (pixel - vertical_scrollbar_width)) / sb_height();
    line = Number(line.toFixed(0));
    if (dont_cut === undefined)
        if (line >= filtered_lines.length - table_attr.nr_lines)
            line = filtered_lines.length - table_attr.nr_lines + 1;
    if (line < 0)
        line = 0;
    return line;
}

function update_vertical_scrollbar_cursor_real() {
    var line = myindex(filtered_lines, the_current_cell.line);
    if (line == -1) {
        vertical_scrollbar.childNodes[3].style.height = 0;
    }
    else {
        var p1 = sb_line_to_pixel(line);
        var p2 = sb_line_to_pixel(line + 1);

        vertical_scrollbar.childNodes[3].style.top = p1.toFixed(0) + 'px';
        height = p2 - p1;
        if (height <= 2)
            height = 2;
        vertical_scrollbar.childNodes[3].style.height = height.toFixed(0) + 'px';
        //debug(vertical_scrollbar.childNodes[3].style,undefined,undefined,true);
    }
}

function update_vertical_scrollbar_cursor() {
    periodic_work_add(update_vertical_scrollbar_cursor_real);
}


function update_vertical_scrollbar_position_real() {
    if (!vertical_scrollbar)
        return;
    var p = vertical_scrollbar.childNodes[0];
    var height = filtered_lines.length;
    if (height === 0) {
        p.style.top = sb_line_to_pixel(0) + 'px';
        p.style.height = sb_height() + 'px';
    }
    else {
        p.style.top = sb_line_to_pixel(line_offset) + 'px';
        p.style.height = sb_line_to_pixel(line_offset + table_attr.nr_lines)
            - sb_line_to_pixel(line_offset) + 'px';
    }
    update_vertical_scrollbar_cursor_real();
}

function update_vertical_scrollbar_position() {
    periodic_work_add(update_vertical_scrollbar_position_real);
    periodic_work_add(update_vertical_scrollbar_cursor_real);
}

var body_on_mouse_up_doing;

// Return true if the event should not be taken into account
function body_on_mouse_up(event) {
    if (body_on_mouse_up_doing) {
        GUI.add(body_on_mouse_up_doing, '', 'stop');
        var was_doing = body_on_mouse_up_doing;
        window.onmouseup = window.onmouseupold;
        window.onmousemove = function () { };

        body_on_mouse_up_doing = undefined;
        if (was_doing == 'table_drag') {
            if (thetable.start_line_offset == line_offset)
                return false; // jump of the cursor
        }
        else if (was_doing == 'column_drag') {
            var column = columns[thetable.column_drag];
            if (thetable.column_initial_pos != column.position)
                column_attr_set(column, 'position', column.position);
        }
        stop_event(the_event(event));
        return true;
    }
}


function move_vertical_scrollbar_begin(event) {
    the_current_cell.change();
    body_on_mouse_up_doing = "vertical_scrollbar_drag";
    set_body_onmouseup(); // ??? Why not working in HTML TAG
    window.onmousemove = move_scrollbar;
    move_scrollbar(event);
    stop_event(the_event(event));
}

function move_scrollbar(event) {
    event = the_event(event);
    var y;
    y = event.y - vertical_scrollbar.offsetTop;
    if (y < vertical_scrollbar_width)
        return;
    if (y > divtable.offsetHeight - vertical_scrollbar_width)
        return;
    var old_line_offset = line_offset;
    line_offset = sb_pixel_to_line(y, true) - Math.floor(table_attr.nr_lines / 2);
    if (line_offset < 0)
        line_offset = 0;
    the_current_cell.lin += old_line_offset - line_offset;
    the_current_cell.lin = Math.max(nr_headers,
        Math.min(nr_headers + table_attr.nr_lines,
            the_current_cell.lin));
    table_fill();
    stop_event(event);
}

function update_vertical_scrollbar_real() {
    if (!vertical_scrollbar)
        return;

    vertical_scrollbar.onmousedown = move_vertical_scrollbar_begin;
    vertical_scrollbar.style.height = divtable.offsetHeight + 'px';
    vertical_scrollbar.style.width = vertical_scrollbar_width + 'px';
    if (scrollbar_right)
        vertical_scrollbar.style.right = 0;
    else
        vertical_scrollbar.style.left = 0;

    vertical_scrollbar.style.top = divtable.offsetTop + 'px';

    if (sort_columns.length === 0)
        return;

    var last = '';
    var data_col = sort_columns[0].data_col;
    var v, vv, v_upper;
    var y, last_y = -100;
    s = '<span class="position">&nbsp;</span><img src="_FILES_/up.gif" onclick="javascript:previous_page();"><img src="_FILES_/down.gif" onclick="javascript:next_page();"><span class="cursor"></span>';


    if (preferences.v_scrollbar_nr
        && columns[data_col].sort_by == 'LABEL_sort_value')
        for (var i in filtered_lines) {
            v = filtered_lines[i][data_col].value.toString().substr(0, preferences.v_scrollbar_nr);
            v_upper = v.toUpperCase();
            if (v_upper != last) {
                y = sb_line_to_pixel(i);
                if (y - last_y < 10)
                    continue;
                if (last.substr(0, 1) != v_upper.substr(0, 1))
                    vv = '<b class="letter">' + v.substr(0, 1) + '</b>' + v.substr(1);
                else
                    vv = '<b class="hide">' + v.substr(0, 1) + '</b>' + v.substr(1);
                last = v_upper;
                // Was (y-6) ???
                s += '<span style="top:' + (y).toFixed(0) + 'px">' + vv + '</span>';
                last_y = y;
            }
        }

    vertical_scrollbar.innerHTML = s + '';
    vertical_scrollbar.childNodes[1].style.width = vertical_scrollbar_width + "px";
    vertical_scrollbar.childNodes[2].style.width = vertical_scrollbar_width + "px";
    vertical_scrollbar.childNodes[2].style.top = divtable.offsetHeight - vertical_scrollbar_width + 'px';

    vertical_scrollbar.childNodes[3].style.width = vertical_scrollbar_width + 'px';
    update_vertical_scrollbar_position_real();
}

function update_vertical_scrollbar() {
    periodic_work_add(update_vertical_scrollbar_real);
    periodic_work_remove(update_vertical_scrollbar_cursor_real);
    periodic_work_remove(update_vertical_scrollbar_position_real);
}

function table_header_fill() {
    periodic_work_add(table_header_fill_real);
}

function update_nr_columns() {
    if (table_attr.autowidth === 0) {
        return;
    }
    var cls = column_list(undefined, 9999);
    var cs = charsize();
    var width = 0;
    for (var col = 0; col < cls.length; col++) {
        width += 1 + cs * Math.max(2, cls[col].compute_char_width());
        if (width > window_width() || (col > 1 && cls[col - 2].is_empty)) {
            if (table_attr.nr_columns != col) {
                table_attr.nr_columns = col;
                table_init();
                table_fill(false, true, false, true);
                update_horizontal_scrollbar();
                setTimeout("the_current_cell.update()", 100);
            }
            return true;
        }
    }
}

function table_header_fill_real() {
    add_empty_columns();
    var cls = column_list();
    var w;
    the_current_cell.update_column_headers();
    update_horizontal_scrollbar(cls);

    for (var data_col in columns)
        columns[data_col].col = undefined;

    var cs = charsize();
    if (table_attr.autowidth !== 0) {
        table.className = 'autowidth';
        for (var col = 0; col < table_attr.nr_columns; col++) {
            colgroup.childNodes[col].width = 1 + cs * Math.max(2, cls[col].compute_char_width());
        }
    }
    else {
        // This loop is not with the other in order to speed up display.
        // So the table is not displayed with all the possible columns width.
        var width = 0;
        for (var col = 0; col < table_attr.nr_columns; col++) {
            width += cls[col].width + 1;
        }

        for (var col = 0; col < table_attr.nr_columns; col++) {
            w = ((window_width() * cls[col].width) / width - 8).toFixed(0);
            if (w <= cs)
                w = cs;
            colgroup.childNodes[col].width = w;
        }
    }

    for (var col = 0; col < table_attr.nr_columns; col++)
        colgroup.childNodes[col].className = 'col_id_' + cls[col].the_id;


    for (var col = 0; col < table_attr.nr_columns; col++) {
        var className;
        var column = cls[col];
        if (column.author != my_identity || !table_attr.modifiable)
            className = 'ro';
        else
            className = '';
        if (column.is_visible()) {
            if (column.modifiable == 2)
                className += ' modifiable_by_student';
            else if (column.visibility == 3 || column.visibility == 4)
                className += ' public_display';
        }
        else
            className += ' hidden_to_student';
        if (column.private.length != 0)
            className += ' private_column';

        var td_title = tr_title.childNodes[col];
        var td_filter = tr_filter.childNodes[col];

        column.col = col;
        var title = td_title.childNodes[0];

        // td_title.data_col = td_filter.data_col = column.data_col ;

        title.firstChild.innerHTML = html(column.title);

        td_filter.childNodes[0].value = column.filter;
        if (column.filter === '')
            td_filter.childNodes[0].className = 'empty';
        else
            td_filter.childNodes[0].className = '';
        if (column.filter_error)
            td_filter.childNodes[0].className += ' attribute_error';
        if (column.freezed !== '')
            td_filter.childNodes[0].className += ' freezed';

        td_title.className = className;

        var icon = title.childNodes[1];
        icon.innerHTML = "&nbsp;▲";
        icon.className = "icon_hidden";
        for (var i = 0; i < 2; i++) {
            if (column != sort_columns[i])
                continue;
            var triangle;
            if (i == 0) {
                td_title.className += ' sorted';
                triangle = ["▼", "", "▲"];
            }
            else
                triangle = ["▽", "", "△"];
            icon.innerHTML = '&nbsp;' + triangle[column.dir + 1];
            icon.className = "";
            break;
        }
    }

    // XXX If updated, the value being edited may be erased
    if (!the_current_cell.focused)
        the_current_cell.update(true);
}

/******************************************************************************
Filter and sort the lines of data
******************************************************************************/
function get_filtered_lines() {
    var f = [], empty, ok;

    if (filters.length === 0) {
        for (var line in lines) {
            empty = line_empty(lines[line]);
            if (table_attr.hide_empty && empty)
                continue;
            if (empty !== true)
                f.push(lines[line]);  // Not empty on screen AND history
        }
        return f;
    }

    for (var line in lines) {
        line = lines[line];
        var ok = true;
        for (var filter in filters) {
            filter = filters[filter];
            if (!filter[0](line, line[filter[1]], my_identity)) {
                ok = false;
                break;
            }
        }
        if (!ok)
            continue;
        f.push(line);
    }
    return f;
}

/*
 * Search in all the table
 */


function full_filter_change(value) {
    if ((full_filter ? full_filter.filter : "") == value.value)
        return;

    for (var data_col in columns)
        columns[data_col].column_list_full_filter = undefined;

    if (value.value === '') {
        // value.className = 'empty' ;
        full_filter = undefined;
    }
    else {
        value.className = '';
        full_filter = compile_filter_generic(value.value);
        if (full_filter.errors)
            value.className = "attribute_error";
        else
            value.className = value.className.replace("attribute_error", "");
    }
    column_offset = 0;
    line_offset = 0;
    table_fill_hook = function () {
        the_current_cell.jump(nr_headers,
            the_current_cell.col,
            true);
    };
    table_fill(true, true, true);

    change_option('full_filter', encode_uri_option(value.value))
    change_option('column_offset');
}


function change_option(option, value) {
    if (column_get_option_running)
        return;
    if (!window.history.replaceState)
        return;
    // Remove ticket
    var loc = window.location.toString().split("?")[0];
    // Remove old option value
    loc = loc.replace(RegExp('/=' + option + '=[^/]*'), '');
    // Remove trailing /
    loc = loc.replace(RegExp('/+$'), '');
    if (value)
        loc += '/=' + option + '=' + value;
    // Replace state because Undo this way has yet to be done
    if (table_attr.bookmark)
        // Changing history 100 times in 30 seconds raise an exception
        try { window.history.replaceState('_a_', '_t_', loc); } catch (e) { };
}

var line_filter_change_value;

function line_filter_change_real() {
    var value = line_filter_change_value;
    if (!value)
        return;
    line_filter_change_value = undefined;
    var old_value = line_filter ? line_filter.filter : "";

    if (old_value == value.value)
        return;

    if (value.value === '') {
        // value.className = 'empty' ;
        line_filter = undefined;
    }
    else {
        value.className = '';
        line_filter = compile_filter_generic(value.value);
        if (line_filter.errors)
            value.className = "attribute_error";
        else
            value.className = value.className.replace("attribute_error", "");
    }

    line_offset = 0;
    the_current_cell.jump(nr_headers, the_current_cell.col, true);
    table_fill(true, true, true);
    update_histogram(true);

    change_option('line_filter', encode_uri_option(value.value));
}

function line_filter_change(value) {
    line_filter_change_value = value;
    periodic_work_add(line_filter_change_real);
}

function sort_lines23(a, b) {
    var c, cc, va, vb;
    if (a.empty && !b.empty)
        return 1;
    if (b.empty && !a.empty)
        return -1;
    for (var c in sort_columns) {
        c = sort_columns[c];
        cc = c.data_col;
        va = a[cc]._key;
        vb = b[cc]._key;
        if (va > vb)
            return c.dir;
        if (va < vb)
            return -c.dir;
    }
    return 0;
}

function sort_lines3() {
    var v, cell;
    for (var i in filtered_lines) {
        filtered_lines[i].empty = line_empty(filtered_lines[i]);
        for (var c in sort_columns) {
            c = sort_columns[c];
            cell = filtered_lines[i][c.data_col];

            switch (c.sort_by) {
                case undefined:
                case 'LABEL_sort_value':
                    cell._key = undefined;
                    v = cell.key(c.empty_is);
                    break;
                case 'LABEL_sort_author':
                    v = cell.author.replace(".", "<br>");
                    break;
                case 'LABEL_sort_date':
                    v = cell.date.substr(0, 4) + ' '
                        + cell.date.substr(4, 2) + ' '
                        + cell.date.substr(6, 2) + '<br>'
                        + cell.date.substr(8, 2) + ':'
                        + cell.date.substr(10, 2) + ':'
                        + cell.date.substr(12);
                    break;
                case 'LABEL_sort_comment':
                    v = html(cell.comment).replace('\n', '<br>');
                    break;
                case 'LABEL_sort_%ABJ':
                    var o = { test_filter_filter: function (_line, x) { return is_abjppn[x.value]; } };
                    v = compute_weighted_percent_(c.data_col, filtered_lines[i], o);
                    v = '  ' + (100 * v).toFixed(1);
                    v = v.substr(v.length - 6);
                    break;
                case 'LABEL_sort_stddev':
                    var s = new Stats();
                    function add_grade(line, x) {
                        if (isNaN(x.value) || x.value === '')
                            return;
                        var col = columns[myindex(line, x)];
                        s.add((x.value - col.min) / (col.max - col.min));
                    };
                    var o = { test_filter_filter: add_grade };
                    compute_weighted_percent_(c.data_col, filtered_lines[i], o);
                    v = s.standard_deviation();
                    if (isNaN(v))
                        v = 0;
                    v = '            ' + (c.min + (c.max - c.min) * v).toFixed(2);
                    v = v.substr(v.length - 7);
                    break;
                case 'LABEL_sort_shuffle':
                    if (cell.value == '')
                        v = c.dir > 0 ? 1 : 0;
                    else
                        v = Math.random();
                    break;
            }
            cell._key = v;
        }
    }

    filtered_lines.sort(sort_lines23);
}

/******************************************************************************
Update the content of the table
******************************************************************************/

// Key value: return value of 'line_empty()'
var count_empty = { false: 1, 1: 0, true: 0 };
var count_fully_empty = { false: 1, 1: 1, true: 0 };

function update_nr_empty(empty_before, empty_after, filtered) {
    var ne = count_empty[empty_after] - count_empty[empty_before];
    var nfe = count_fully_empty[empty_after] - count_fully_empty[empty_before];
    nr_not_empty_lines += ne;
    nr_not_fully_empty_lines += nfe;

    if (filtered) {
        nr_filtered_not_empty_lines += ne;
        nr_filtered_not_fully_empty_lines += nfe;
    }
    if (ne || nfe)
        periodic_work_add(update_line_counts);
}

function update_line_counts() {
    if (nr_filtered_lines
        && Number(nr_filtered_lines.innerHTML) != nr_filtered_not_empty_lines) {
        var empty = nr_filtered_not_fully_empty_lines
            - nr_filtered_not_empty_lines;
        nr_filtered_lines.innerHTML = nr_filtered_not_empty_lines
            + (empty ? " (+" + empty + '∅)' : "");
        highlight_add(nr_filtered_lines);
    }
    if (document.getElementById('nr_not_empty_lines'))
        document.getElementById('nr_not_empty_lines').innerHTML
            = nr_not_empty_lines;
}

function update_filtered_lines() {
    var d1 = millisec();
    filtered_lines = get_filtered_lines();

    if (full_filter !== undefined) {
        var cls = column_list_all();
        var f = [];
        for (var line in filtered_lines) {
            line = filtered_lines[line];
            for (var column in cls)
                if (full_filter(line, line[cls[column]], my_identity)) {
                    f.push(line);
                    break;
                }
        }
        filtered_lines = f;
    }

    if (line_filter !== undefined) {
        var cls = column_list_all();
        var f = [];
        for (var line in filtered_lines) {
            line = filtered_lines[line];
            for (var column in cls)
                if (line_filter(line, line[cls[column]], my_identity)) {
                    f.push(line);
                    break;
                }
        }
        filtered_lines = f;
    }

    for (var line in lines)
        lines[line].is_filtered = false;

    nr_filtered_not_empty_lines = 0;
    nr_filtered_not_fully_empty_lines = 0;
    for (var line in filtered_lines) {
        filtered_lines[line].is_filtered = true;
        switch (line_empty(filtered_lines[line])) {
            case false: nr_filtered_not_empty_lines++;       // Fall thru
            case 1: nr_filtered_not_fully_empty_lines++;
        }
    }

    update_line_menu();

    var d2 = millisec();
    if (sort_columns.length !== 0) {
        sort_lines3();
    }
    var d3 = millisec();
    _d('Filter time: ' + (d2 - d1) + 'ms, Sort time: ' + (d3 - d2) + 'ms');

    update_line_counts();
    update_vertical_scrollbar();
}

function line_fill(line, write, cls, empty_column) {
    if (cls === undefined)
        cls = column_list();
    if (empty_column === undefined)
        empty_column = add_empty_columns();

    var the_line = filtered_lines[line];

    var tr = table.childNodes[write];
    var abj = the_student_abjs[the_line[0].value];
    if (tr_classname !== undefined) {
        tr.className = tr.zebra + the_line[tr_classname].value;
    }
    else
        tr.className = tr.zebra;
    var data_col, td;
    tr = tr.childNodes;
    for (var col = 0; col < table_attr.nr_columns; col++) {
        data_col = cls[col].data_col;
        td = tr[col];

        if (data_col >= empty_column) {
            td.className = 'empty';
            td.innerHTML = ' ';
        }
        else
            update_cell(td, the_line[data_col], cls[col], abj, the_line);
    }
}

var table_fill_force_update_filtered_lines;

function table_fill_do() {
    if (table_fill_force_update_filtered_lines)
        update_filtered_lines();

    table_fill_real();

    if (table_fill_hook) {
        table_fill_hook();
        table_fill_hook = undefined;
    }
    // Do not update while the cell is being edited
    if (!the_current_cell.focused || table_fill_force_current_cell_update) {
        the_current_cell.update(table_fill_do_not_focus);
        // Timeout because the cell must be repositionned after
        // The table column resize in case of horizontal scroll with
        // variable size columns.
        setTimeout("the_current_cell.update(" + table_fill_do_not_focus + ");",
            periodic_work_period + 1); // XXX *2 in place of +1 ???
    }
}

function manage_window_resize_event() {
    if (do_touchstart.touch_device)
        return true;
    var width = window_width(), height = window_height();

    if (current_window_width != width) {
        if (table_attr.default_nr_columns == 0
            && get_option('nr_cols', '0') == '0') {
            compute_nr_cols();
        }
        update_column_menu();
        update_histogram(true);
    }
    if (current_window_height != height) {
        compute_nr_lines.do_compute_nr_lines = true;
        compute_nr_lines();
        update_line_menu();
    }
    if (current_window_width != width || current_window_height != height) {
        the_current_cell.input.blur();
        table_init();
        table_fill(false, true, true, true);
        current_window_width = width;
        current_window_height = height;
    }
    if (!preferences.compact && width < 800) {
        preferences.compact = 1;
        set_body_theme(semester);
        preferences.compact = 0;
    }
    else
        set_body_theme(semester);
    return true;
}

var display_tips_saved;

var last_login_sent = 0;
function send_login_to_server() {
    if (millisec() - last_login_sent < 500)
        return true; // Wait a moment after last key stroke
    login_list(replaceDiacritics(ask_login_list),
        [['', '', _('loading_logins_before') + ask_login_list
            + _('loading_logins_after')]]);
    var s = document.createElement('script');
    s.src = add_ticket('login_list/'
        + encode_uri(replaceDiacritics(ask_login_list)));
    the_body.appendChild(s);
    return; // Stop sending
}

function login_list_ask(input) {
    last_login_sent = millisec();
    if (input.initial_value != input.value
        && input.value.length > 1
        && input.value != ask_login_list
        && input.value.toString().search('[.]$') == -1
    ) {
        ask_login_list = input.value;
        periodic_work_add(send_login_to_server);
    }
    return true;
}

function login_list_hide() {
    set_the_tip_content = undefined;
    body_on_mouse_up_doing = undefined;
    hide_the_tip_real();
}

function login_list_select(event) {
    event = the_event(event);
    var t = event.target;
    if (t.tagName == 'OPTION')
        t = t.parentNode;
    var s = t.options[t.selectedIndex].value;
    if (s !== '') {
        if (t.my_target.value != t.my_target_value) {
            console.log('bad value current=' + t.my_target.value + ' expected=' + t.my_target_value);
            console.log(t.my_target);
        }
        t.my_target.value = s;
        t.my_target.scrollLeft = 9999;
        setTimeout(function () {
            t.my_target.focus_from_select = true;
            t.my_target.focus();
            set_selection(t.my_target, s.length, s.length);
            t.my_target.focus_from_select = false;
        }, 100);
    }
    ask_login_list = s;
    login_list_hide();
}

function login_list(name, x, current_value) {
    // x contains:
    //   [ ["id (value)", "firstname", "surname", "grp", "real_value"], ...]
    if (name != replaceDiacritics(ask_login_list))
        return;

    if (x.length == 0)
        x = [['', '', _("ALERT_unknown_user")]];
    hide_the_tip_real();

    var nr = Math.floor(table_attr.nr_lines / 2);
    if (x.length < nr)
        nr = x.length;
    if (nr < 2)
        nr = 2;
    var s = '<select tabindex="-1" class="login_list" size="' + nr + '" style="width:100%">';

    var w = 0;
    for (var i in x)
        if (x[i][0].length > w)
            w = x[i][0].length;

    if (current_value === undefined)
        current_value = x[0][0];

    var autoselect;
    for (var ii in x)
        if (x[ii][0].substr(0, current_value.length) == current_value) {
            autoselect = ii;
            break;
        }
    if (autoselect === undefined)
        for (var ii in x)
            if (x[ii][0].toLowerCase() == current_value.toLowerCase()) {
                autoselect = ii;
                break;
            }
    if (autoselect === undefined)
        autoselect = 0;

    x.sort(function (a, b) {
        if (a[0] < b[0]) return -1;
        if (a[0] > b[0]) return 1;
        if (a[1] < b[1]) return -1;
        if (a[1] > b[1]) return 1;
        if (a[2] < b[2]) return -1;
        if (a[2] > b[2]) return 1;
        return 0
    });

    for (var ii in x) {
        var i = x[ii];
        var cn = '';
        if (i[3]) {
            cn = i[3].replace(/OU=/g, '');
            cn = cn.split(',');
            cn = cn.slice(1, cn.length - 2);
            cn = '<i><small>' + cn.toString() + '</small></i>';
        }
        s += '<option value="' + encode_value(i[4] ? i[4] : i[0]) + '"'
            + (ii == autoselect ? ' selected' : '') + '>'
            + left_justify(i[0], w).replace('&nbsp;', ' ')
            + '&nbsp;' + i[1] + ' ' + i[2] + ' ' + cn + '</option>';
    }
    s += '</select>';
    body_on_mouse_up_doing = undefined;
    set_the_tip_content = function (tip) {
        tip.innerHTML = s;
        var select = tip.firstChild;
        select.onchange = login_list_select;
        select.onclick = login_list_select;
        select.my_target = element_focused || the_current_cell.input;
        select.my_target_value = select.my_target.value;
        body_on_mouse_up_doing = 'login_list'; // do not hide tip
        if (element_focused && !element_focused.blur_patched) {
            element_focused.blur_patched = true;
            element_focused.setAttribute('onblur',
                'if(element_focused!==this){return;}if(body_on_mouse_up_doing == "login_list") {if ( event.relatedTarget === tip.firstChild ) {return ;};login_list_hide();};' + element_focused.getAttribute('onblur'));
        }
        return s;
    };
    show_the_tip(element_focused || the_current_cell.td);
}

function table_fill(do_not_focus, display_headers, compute_filtered_lines,
    force_current_cell_update) {
    if (table === undefined)
        return;
    if (table_forms_element)
        display_headers = false;
    table_fill_do_not_focus = do_not_focus;
    table_fill_force_current_cell_update = force_current_cell_update;
    table_fill_force_update_filtered_lines = compute_filtered_lines;
    periodic_work_add(table_fill_do);
    if (display_headers)
        table_header_fill();
}

function table_fill_real() {
    var write = nr_headers;
    var td;
    var empty_column = add_empty_columns();
    var cls = column_list();
    if (cls.length < table_attr.nr_columns)
        report_fake_error('table_fill_real cls=' + cls.length + ' < nr=' + table_attr.nr_columns
            + ', offset=' + column_offset + ' columns=' + columns.length
            + ', notEmpty=' + first_column_not_empty());
    var d1 = millisec();
    for (var line = line_offset; line < filtered_lines.length; line++) {
        line_fill(line, write, cls, empty_column);
        write++;
        if (write == table_attr.nr_lines + nr_headers)
            break;
    }
    // lines of tables after the end of the data, the empty lines
    while (write < table_attr.nr_lines + nr_headers) {
        var tr = table.childNodes[write];
        tr.className = tr.zebra;
        for (var col = 0; col < table_attr.nr_columns; col++) {
            td = tr.childNodes[col];
            td.innerHTML = " "; // Unsecable space
            td.className = 'rw empty';
        }
        write++;
    }
    update_vertical_scrollbar_position();
    _d(', table_fill_real: ' + (millisec() - d1) + 'ms');
    if (the_current_cell.focused)
        the_current_cell.update(); // 2010-10-06
}

/******************************************************************************
Table utilities
******************************************************************************/

// true : empty and no history, 1 : empty and an history
function line_empty(line) {
    for (var i in columns)
        if (line[i].is_not_empty())
            return false;
    for (var i in columns)
        if (line[i].history.length)
            return 1;
    return true;
}

function column_empty(column) {
    return !!columns[column].is_empty;
}

function column_empty_of_cells(column) {
    for (var i in lines) {
        if (lines[i][column].is_not_empty())
            return false;
    }
    return true;
}

function column_empty_of_user_cells(column) {
    var c;
    for (var i in lines) {
        c = lines[i][column];
        if (c.is_not_empty() && c.author !== "*" && c.author !== '?' && c.author !== "")
            return false;
    }
    return true;
}

function first_column_not_empty() { // Right to left
    for (var i = columns.length - 1; i >= 0; i--)
        if (!column_empty(i))
            break;
    return i;
}

function add_empty_column(keep_data) {
    var data_col = columns.length;

    if (!keep_data)
        for (var line in lines)
            lines[line][data_col] = C();

    var position = 0;
    for (var i in columns)
        if (columns[i].position > position)
            position = columns[i].position;

    var d = -1
    if (tr_classname !== undefined)
        while (columns[d + 1] && columns[d + 1].author == '*')
            d++;

    while (data_col_from_col_id(page_id + '_' + nr_new_columns) !== undefined)
        nr_new_columns++; // Problem raised by history removal

    var column = Col({
        the_id: page_id + '_' + nr_new_columns,
        the_local_id: nr_new_columns.toString(),
        data_col: columns.length,
        is_empty: keep_data === undefined,
        filter: ""
    });

    column.real_type = type_title_to_type(column_attributes['type'].default_value);
    var value;
    for (var attr in column_attributes) {
        switch (attr) {
            case 'title': value = default_title + (columns.length - d); break;
            case 'position': value = position + 1; break;
            case 'author': value = my_identity; break;
            default: value = column_attributes[attr].default_value; break;
        }
        column[attr] = column_parse_attr(attr, value, column);
    }
    columns[columns.length] = column;
    nr_new_columns++;
}

/*****************************************************************************/

function Column(attrs) {
    for (var attr in attrs)
        this[attr] = attrs[attr];
    if (this.minmax === undefined && this.type == 'Nmbr') {
        // Old table
        if (this.columns === undefined)
            this.minmax = '[0;1]';
        else
            this.minmax = '[0;' + Math.max(1, this.columns.split(/ +/).length) + ']';
        this.rounding = 1;
    }
    for (var attr in column_attributes)
        if (this[attr] === undefined)
            this[attr] = column_attributes[attr].default_value;
    this.sort_by = 'LABEL_sort_value';
    this.table = table_attr;
}

function Col(attrs) {
    return new Column(attrs);
}

Column.prototype.is_computed = function () {
    return this.real_type.cell_compute !== undefined && !this.incomplete;
};

Column.prototype.cell_is_modifiable = function () {
    return this.real_type.cell_is_modifiable;
};

Column.prototype.is_visible = function (line, cell) {
    if (this.title.substr(0, 1) == '.')
        return false;
    if (this.visibility == 0 || this.visibility == 6) {
        if (this.visibility_date != '') {
            if (this.visibility_date_filter) {
                if (line)
                    return this.visibility_date_filter(line, cell, line[0].value)
                return true; // not computable
            }
            if (this.visibility_date > get_date().formate('%Y%m%d'))
                return false;
        }
        return true;
    }
    return this.visibility == 3 || this.visibility == 4;
};

Column.prototype.all_cells_are_empty = function () {
    for (var lin in lines)
        if (lines[lin][this.data_col].value !== '')
            return false;
    return true;
};

Column.prototype.contain_mails = function (allow_multiple) {
    if (this.all_cells_are_empty())
        return false;
    var mail = "[-'_.a-zA-Z0-9]*@[-'_.a-zA-Z0-9]*";
    var test = mail;
    if (allow_multiple)
        test += "( +" + mail + ")*";
    test = RegExp("^( *$|(mailto:)?" + test + ")$");
    for (var lin in lines) {
        var value = lines[lin][this.data_col].value;
        if (!value.match)
            return false; // An int
        if (!test.exec(value))
            return false;
    }
    return true;
};

Column.prototype.compute_char_width = function () {
    if (this.is_empty)
        return 4;
    var t = [];
    for (var lin_id in lines)
        t.push(lines[lin_id][this.data_col].value.toString().replace(/[a-z][a-z][a-z]/g, '00').length);
    if (t.length == 0)
        return 1;
    t.sort(function (a, b) { return a - b; });
    return t[Math.floor(0.95 * t.length).toString()];
};


Column.prototype.repetition_allow_this_value = function (value, for_line) {
    var n;
    var verify_lines;

    if (value === '' || !this.real_repetition)
        return 0;

    if (this.real_repetition > 0)
        verify_lines = lines;
    else {
        verify_lines = [];
        var grp = for_line[3].value; // XXX should seach column name
        var seq = for_line[4].value;
        for (var line in lines) {
            line = lines[line];
            if (line[3].value == grp && line[4].value == seq)
                verify_lines.push(line);
        }
    }
    var col = data_col_from_col_title(this.groupcolumn);
    if (col === undefined) {
        n = 0;
        for (var line in verify_lines)
            if (verify_lines[line][this.data_col].value == value)
                n++;
    }
    else {
        var groups = {};
        for (var line in verify_lines)
            if (verify_lines[line][this.data_col].value == value)
                groups[verify_lines[line][col].value] = true;
        n = dict_size(groups);
    }
    if (n >= Math.abs(this.real_repetition))
        return n;
    return 0;
};

/*****************************************************************************/

function add_empty_columns() {
    var not_empty = first_column_not_empty();
    var nr_empty_columns = columns.length - not_empty - 1;

    /* There is a +5 because a user may hide empty columns.
       It will make 'column_list' function have problems
       because there is missing empty columns.
       So, we add some more.
    */
    for (var i = 0; i < table_attr.nr_columns - nr_empty_columns + 5; i++)
        add_empty_column();

    return not_empty + 1;
}

function search_column_in_columns(column, title) {
    if (column.title == title)
        return column;
    if (!column_modifiable_attr('columns', column))
        return;
    for (var i in column.average_columns) {
        var c = search_column_in_columns(columns[column.average_columns[i]],
            title);
        if (c)
            return c;
    }
}


/******************************************************************************
Cursor movement
******************************************************************************/

function need_to_save_change() {
    return !element_focused || element_focused.id != 'linefilter';
}


function next_page(next_cell, dy) {
    if (need_to_save_change())
        the_current_cell.change();

    if (filtered_lines !== undefined
        && line_offset + table_attr.nr_lines > nr_filtered_not_fully_empty_lines + 1)
        return true;

    if (dy === undefined)
        dy = Number((table_attr.nr_lines * preferences.page_step).toFixed(0));

    if (next_cell === true) {
        table_fill_hook = function () {
            cell_goto(table.childNodes[nr_headers + table_attr.nr_lines - dy - preferences.one_line_more].childNodes[the_current_cell.col]);
        };
    }

    line_offset += dy;
    the_current_cell.lin = Math.max(nr_headers, the_current_cell.lin - dy);
    if (preferences.one_line_more)
        the_current_cell.lin++;
    table_fill(true);
    return true;
}

function previous_page(previous_cell, dy) {
    if (need_to_save_change())
        the_current_cell.change();
    if (dy === undefined)
        dy = Number((table_attr.nr_lines * preferences.page_step).toFixed(0));
    if (previous_cell === true) {
        table_fill_hook = function () {
            cell_goto(table.childNodes[nr_headers + dy - 1 + preferences.one_line_more].childNodes[the_current_cell.col]);
        };
    }
    dy = Math.min(line_offset, dy);
    line_offset -= dy;
    the_current_cell.lin = Math.min(nr_headers + table_attr.nr_lines - 1,
        the_current_cell.lin + dy);
    if (dy && preferences.one_line_more)
        the_current_cell.lin--;
    table_fill(true);
    return true;
}

function first_page() {
    if (need_to_save_change())
        the_current_cell.change();
    line_offset = 0;
    the_current_cell.lin = nr_headers;
    table_fill(false);
    return true;
}

function last_page() {
    if (need_to_save_change())
        the_current_cell.change();
    var nr_lines = Math.min(nr_filtered_not_fully_empty_lines, filtered_lines.length);
    line_offset = nr_lines - table_attr.nr_lines + 1;
    if (line_offset < 0) {
        line_offset = 0;
        the_current_cell.lin = nr_lines + nr_headers - 1;
    }
    else {
        the_current_cell.lin = table_attr.nr_lines - 1;
    }
    table_fill(false);
    return true;
}

function table_fill_hook_horizontal() {
    var tr = table.childNodes[next_page_line];
    var col = next_page_col - column_offset;
    if (col < 0)
        cell_goto(tr.childNodes[0], true);
    else if (col < table_attr.nr_columns)
        cell_goto(tr.childNodes[col], true);
    else
        cell_goto(tr.childNodes[table_attr.nr_columns - 1], true);
}

/*
 * Autofreeze columns with a filter
 */
function autofreeze() {
    if (preferences.filter_freezed) {
        for (var data_col in columns) {
            var column = columns[data_col];
            if (column.filter !== '' && column.filter !== undefined)
                column.filter_freeze = true;
            else
                column.filter_freeze = false;
        }
    }
}
/*
 * If 'col' is defined : then it is the required column (centered)
 * Else 'direction' is a delta
 */
function page_horizontal(direction, col, do_not_focus) {
    var cls = column_list_all();
    if (column_offset + direction >= cls.length)
        return;
    if (!do_not_focus)
        the_current_cell.change();

    autofreeze();
    if (col === undefined) {
        col = myindex(cls, the_current_cell.data_col) +
            (direction > 0 ? 1 : -1);
        if (direction > 0 && col === 0) {
            // alert("À quoi cela vous sert d'aller à droite, le tableau est vide !") ;
            return;
        }
        column_offset += direction;
    }
    else {
        column_offset = col - Math.floor((table_attr.nr_columns + nr_freezed()) / 2);
    }

    next_page_col = col;
    next_page_line = the_current_cell.lin;

    if (column_offset + table_attr.nr_columns > columns.length)
        column_offset = columns.length - table_attr.nr_columns;
    if (column_offset < 0)
        column_offset = 0;

    table_fill_hook = table_fill_hook_horizontal;

    if (!update_nr_columns())
        table_fill(do_not_focus, true, false, true);

    periodic_work_do();

    change_option('column_offset', column_offset ? column_offset : '');

}

function next_page_horizontal(full_page) {
    var n = table_attr.nr_columns - nr_freezed();
    page_horizontal(full_page ? n : Math.floor(n / 2));
}

function previous_page_horizontal(full_page) {
    var n = table_attr.nr_columns - nr_freezed();
    page_horizontal(full_page ? -n : -Math.floor(n / 2));
}


/******************************************************************************
Cursor movement
******************************************************************************/

function cell_get_value_real(line_id, data_col) {
    return columns[data_col].real_type.formatte(lines[line_id][data_col].value,
        columns[data_col]);
}

function cell_class(column, line, cell) {
    var className = '';

    if (column.green_filter(line, cell, my_identity))
        className += ' color_green';
    if (column.red_filter(line, cell, my_identity))
        className += ' color_red';
    if (column.greentext_filter(line, cell, my_identity))
        className += ' greentext';
    if (column.redtext_filter(line, cell, my_identity))
        className += ' redtext';
    return className;
}

function there_is_an_abj(cell, column, abj) {
    if (!abj || abj[0].length == 0)
        return;
    if (column.parsed_course_dates) {
        var t, first, last;
        for (var a in abj[0]) {
            a = abj[0][a];
            if (!abj_is_fine(a))
                continue;
            first = parse_date(a[0]).getTime();
            last = parse_date(a[1]).getTime();
            for (var date in column.parsed_course_dates) {
                t = column.parsed_course_dates[date];
                if (t >= first && t <= last)
                    return 'is_not_an_abi';
            }
        }
    }
    else {
        var d = new Date(cell.date.substr(0, 4),
            cell.date.substr(4, 2) - 1,
            cell.date.substr(6, 2));
        d = d.getTime();
        for (var a in abj[0]) {
            a = abj[0][a];
            if (!abj_is_fine(a))
                continue;
            if (parse_date(a[0]).getTime() <= d
                && d < parse_date(a[1]).getTime() + 86400000 * 1)
                return 'is_an_abj';
        }
    }
}


function update_cell(td, cell, column, abj_list, line) {

    var v = cell.value;
    var className = cell_class(column, line, cell);
    if (className.indexOf('text') == -1)
        if (cell.is_mine() && column.real_type.cell_is_modifiable)
            className += ' rw';
        else
            className += ' ro';
    if (cell.comment !== '')
        className += ' comment';
    if (cell.date >= today)
        className += ' today';
    if (v === '' && column.empty_is && line.empty == false) {
        className += ' default';
        v = column.empty_is;
    }
    if (v.toFixed) {
        className += ' number';
        v = column.real_type.formatte(v, column);
    }
    if (full_filter && full_filter(line, cell, my_identity))
        className += ' filtered';
    else if (line_filter && line_filter(line, cell, my_identity))
        className += ' filtered';
    is_a_teacher = 0;
    if (column.visibility_date_filter)
        if (!column.visibility_date_filter(line, cell, the_login(line[0].value)))
            className += ' suivi_hidden';
    if (column.modifiable == 2 && column.is_visible(line, cell) && column.cell_writable_filter
        && table_attr.modifiable && table_attr.official_ue)
        if (column.cell_writable_filter(line, cell, the_login(line[0].value)))
            className += ' suivi_modifiable';
    is_a_teacher = 1;

    // XXX : This does not work if there are no courses dates
    // because the ABJ modify the ABI date
    if (is_abj[v]) {
        var c = there_is_an_abj(cell, column, abj_list);
        if (!c)
            className = className.replace(' default', '') + ' is_not_an_abj';
    }
    else if (is_abi[v]) {
        var c = there_is_an_abj(cell, column, abj_list);
        if (c)
            className = className.replace(' default', '') + ' ' + c;
    }
    if (the_student_abjs[line[0].value])
        className += ' some_abj';
    if (cell.author.length != 1 && column.is_computed() && cell.date !== '')
        className += ' override';
    if (line[0].value === '') // Remove coloring but not comment indicator
        if (className.indexOf(' comment') == -1)
            className = '';
        else
            className = 'comment';
    td.className = className;

    if (column.type == 'URL' && (column.urlimg & 2))
        td.innerHTML = '<img class="urlimg" onerror="this.style.display=\'none\'" src="' + encode_value(column.url_base + v) + '">';
    else if (column.type == 'Calendar') {
        td.innerHTML = "";
        td.appendChild(column.Calendar.html(
            v.toString(), true, td.offsetWidth, td.offsetHeight - 2, className.indexOf(' ro') == -1));
    }
    else if (column.type == 'Competences' && v.length > 0) {
        td.innerHTML = competences_recap(v.split(' '));
    }
    else {
        if (td.firstChild.tagName)
            td.innerHTML = " ";
        if (v === '')
            td.lastChild.nodeValue = ' '; // If empty : zebra are not displayed
        else
            td.lastChild.nodeValue = v.toString();
    }
    if (column.sort_by != 'LABEL_sort_value' && column.sort_by != 'LABEL_sort_shuffle'
        && cell._key !== undefined) {
        var vv = td.firstChild;
        if (vv.tagName != 'SPAN') {
            vv = document.createElement('SPAN');
            vv.className = "sorted_value";
            if (cell.value.toFixed) {
                vv.style.left = '0px';
                vv.style.textAlign = 'left';
            }
            else {
                vv.style.right = '0px';
                vv.style.textAlign = 'right';
            }
            td.insertBefore(vv, td.firstChild);
        }
        vv.innerHTML = cell._key;
    }
    else
        if (td.firstChild.className == 'sorted_value')
            td.removeChild(td.firstChild);
    return v;
}

function column_change_allowed_text(column) {
    if (!table_attr.modifiable)
        return _("ERROR_table_read_only");
    if (column.title === '')
        return true;
    if (column.author == '*')
        return _("ERROR_column_system_defined");
    if (column.author == my_identity)
        return true;
    if (column.author == '')
        return true;
    if (i_am_the_teacher)
        return true;
    if (myindex(minors, column.author) != -1)
        return true;
    return _("ERROR_value_defined_by_another_user");
}

function column_change_allowed(column) {
    return column_change_allowed_text(column) === true;
}

// Indicate that 'line_id' will be filled
function add_a_new_line(line_id, hide_if_created) {
    var reuse_old_empty_line;

    if (line_id === undefined) {
        for (var line_id in lines) {
            var line = lines[line_id];
            if (!line.is_filtered && line_empty(line) === true) {
                reuse_old_empty_line = true;
                break;
            }
        }
        if (!reuse_old_empty_line) {
            while (lines[page_id + '_' + nr_new_lines] !== undefined)
                nr_new_lines++; // Problem raised by history removal

            line_id = page_id + '_' + nr_new_lines;
            nr_new_lines++;
        }
    }
    else {
        if (lines[line_id])
            return;
    }

    if (!reuse_old_empty_line) {
        // Create a new line

        var line = [];
        for (var c in columns)
            line[c] = C();
        line.line_id = line_id;
        lines[line_id] = line;
    }

    if (hide_if_created && (filters.length !== 0 || full_filter || line_filter))
        return;

    filtered_lines.push(line);
    line.is_filtered = true;

    /* Update screen table with the new id */
    var lin = filtered_lines.length - 1 - line_offset;
    if (lin >= 0 && lin < table_attr.nr_lines) {
        line_fill(filtered_lines.length - 1, lin + nr_headers);
    }
    update_vertical_scrollbar();
    return line_id;
}

function create_column(column, title) {
    if (column.is_empty && column.the_local_id !== undefined) {
        if (title === undefined)
            title = column.title;
        else
            column.title = title;
        column.is_empty = false;
        append_image(undefined, 'column_attr_title/' + page_id + '_'
            + column.the_local_id + '/' + encode_uri(title));
        column.the_local_id = undefined;
        update_horizontal_scrollbar();
        return true;
    }
}

// self is not included in the returned lines
function lines_of_the_group(column, line_id) {
    var g = [];
    if (column.groupcolumn === '')
        return g;
    var col = data_col_from_col_title(column.groupcolumn);
    if (!col)
        return g;
    var group = lines[line_id][col].value.toString();
    if (group !== '')
        for (var line_key in lines) {
            if (line_key == line_id)
                continue; // Itself
            if (lines[line_key][col].value.toString() === group)
                g.push(lines[line_key]);
        }
    return g;
}

function cell_set_value_real(line_id, data_col, value, td) {
    var cell = lines[line_id][data_col];
    var column = columns[data_col];

    if (!modification_allowed_on_this_line(line_id, data_col, value)) {
        GUI.add("cell_change_error", undefined, "not_allowed");
        return;
    }

    // toString is used because '' != '0' and '00' != '000'
    // === is not used because 5.1 == "5.1"
    if (value.toString() == lines[line_id][data_col].value.toString())
        return;

    if (!cell.modifiable(lines[line_id], column))
        return;

    if (column.is_empty && columns_filter.filter !== '') {
        Alert("ERROR_column_creation");
        return;
    }
    if (column.is_empty && column.data_col > 0
        && columns[column.data_col - 1].is_empty)
        alert_append(_("ERROR_column_left_to_right"));

    var orig_value = value;
    if (value.replace)
        value = encode_lf_tab(value);
    value = column.real_type.cell_test(value, column);
    if (value === undefined) {
        if (!column.is_empty)
            return;
        if (!confirm(_("ALERT_to_text")))
            return;
        column_attr_set(column, "type", "Text", undefined, true);
        the_current_cell.do_update_column_headers = true;
        return cell_set_value_real(line_id, data_col, orig_value, td);
    }
    else if (value.replace)
        value = decode_lf_tab(value);

    // Used as a group column
    for (var i in columns)
        if (columns[i].groupcolumn == column.title && column.title !== '') {
            var e = [], quoi;
            var group = lines[line_id][column.data_col].value.toString();
            var value_group = lines[line_id][columns[i].data_col].value;
            for (var line_key in lines) {
                if (line_key == line_id)
                    continue;
                quoi = '';
                if (lines[line_key][columns[i].data_col].value !== ''
                    && group !== ''
                    && lines[line_key][column.data_col].value.toString()
                    == group)
                    quoi = '-';
                if (lines[line_key][columns[i].data_col].value !== ''
                    && lines[line_key][columns[i].data_col].value !== value_group
                    && value !== ''
                    && lines[line_key][column.data_col].value.toString()
                    == value.toString())
                    quoi = '+';
                if (quoi)
                    e.push(_('MSG_columngroup' + quoi)
                        + ' ' + lines[line_key][0].value + ' '
                        + lines[line_key][1].value + ' '
                        + lines[line_key][2].value + ' '
                        + columns[i].title
                        + (quoi == '+' ?
                            ' : ' + lines[line_key][columns[i].data_col].value
                            + '≠' + value_group : '')
                        + '\n');
            }
            if (e.length) {
                e.sort();
                alert_append(_("ALERT_columngroup_change") + '\n\n'
                    + lines[line_id][column.data_col].value
                    + '→'
                    + value + '\n\n'
                    + e.join(""));
            }
        }

    create_column(columns[data_col]);
    add_a_new_line(line_id);
    var empty_before = line_empty(lines[line_id]);
    cell.set_value(value);
    update_nr_empty(empty_before, line_empty(lines[line_id]),
        lines[line_id].is_filtered);

    var v;
    if (td !== undefined)
        v = update_cell(td, cell, column, undefined, lines[line_id]);

    /* Create cell */
    append_image(td, 'cell_change/' + column.the_id + '/' +
        line_id + "/" + encode_uri(cell.value)
    );

    if (value !== '') {
        column.is_empty = false;
        lines[line_id].empty = false;
    }

    update_histogram(true); // XXX

    if (empty_before && td) // Fill empty cells with empty is
        line_fill(myindex(filtered_lines, lines[line_id]), td.parentNode.rowIndex);

    var g = lines_of_the_group(column, line_id);
    if (g.length) {
        for (var line in g) {
            line = g[line];
            var cell = line[column.data_col];
            cell.set_value(value);
            td = td_from_line_id_data_col(line.line_id, column.data_col);
            if (td !== undefined)
                update_cell(td, cell, column, undefined, line);
        }
    }
    return v;
}

function cell_set_value(td, value, line_id, data_col) {
    if (value === undefined)
        // Next/Prev page if there is not a cell selected (Prst)
        return cell_get_value_real(line_id, data_col);

    var v = cell_set_value_real(line_id, data_col, value, td);
    if (v !== undefined)
        return v;
    return cell_get_value_real(line_id, data_col);
}

/*REDEFINE
  If LDAP students login and real students ID are not equals
  then a translation must be done.
  This function translate the login to a student number.
*/
function login_to_id(login) {
    return login;
}

/*REDEFINE
  If LDAP students login and real students ID are not equals
  then a translation must be done.
  This function translate the student number to a login
*/
function the_login(login) {
    return login;
}

function abj_is_fine(abj) {
    if (parse_date(abj[0]).getTime() > last_day)
        return 0;
    if (parse_date(abj[1]).getTime() < first_day)
        return false;
    return true;
}

function student_abjs(login) {
    var abjs_da = the_student_abjs[login];

    if (abjs_da === undefined)
        return '';

    var abjs = [];
    var s = '';

    for (var i in abjs_da[0]) {
        i = abjs_da[0][i];
        if (abj_is_fine(i))
            abjs.push(i);
    }

    if (abjs.length) {
        s += '<TABLE class="display_abjs colored">';
        s += '<TR><TH COLSPAN="4">' + _("TH_ABJ_list") + '</TH></TR>';
        s += '<TR><TH>' + _('TH_begin')
            + '</TH><TH>' + _('TH_end')
            + '</TH><TH>' + _('TH_length') + '</TH><TH></TH></TR>';
        var lines = [];
        for (var abj in abjs) {
            var start = abjs[abj][0].split('/');
            var line = '<TR><!--';
            line += start[2].substr(0, 4) + two_digits(start[1]) + two_digits(start[0]);
            line += '-->'
            if (abjs[abj][2].substr(0, 13) == '{{{MESSAGE}}}') {
                line += '<TD COLSPAN="4">' + nice_date(abjs[abj][0])
                    + ' <span style="background: #F00; color: #FFF">'
                    + html(abjs[abj][2].replace('{{{MESSAGE}}}', ''))
                    + '</span>'
                    ;
            }
            else {
                var d = (0.5 + (parse_date(abjs[abj][1]).getTime()
                    - parse_date(abjs[abj][0]).getTime()) / (1000 * 86400));
                var x = new RegExp('[' + ampms[0] + ampms[1] + ']');
                if (d == 0.5)
                    line += '<TD COLSPAN="2">' + nice_date(abjs[abj][0]);
                else if (abjs[abj][0].replace(x, '')
                    == abjs[abj][1].replace(x, ''))
                    line += '<TD COLSPAN="2">' + nice_date(abjs[abj][0]).replace(/ [^ ]*$/, '');
                else
                    line += '<TR><TD>' + nice_date(abjs[abj][0]) +
                        '</TD><TD>' + nice_date(abjs[abj][1]);

                line += '</TD><TD>' + d.toFixed(1) +
                    '</TD><TD>' + html(abjs[abj][2]) +
                    '</TD>';
            }
            line += '</TR>';
            lines.push(line);
        }
        lines.sort();
        s += lines.join('') + '</TABLE>';
    }

    var das = abjs_da[1];
    if (das.length) {
        var others = [];
        for (var da in das) {
            if (das[da][0] == ue)
                s += '<p class="da">' + _("MSG_DA") + ' ' + das[da][1] + ' <em>'
                    + html(das[da][2]).replace(/\n/g, '<br>') + '</em></p>';
            else
                others.push(das[da][0]);
        }
        if (others.length)
            s += '<hr><p class="da">' + _("TITLE_abjtt_da")
                + ' : ' + others.join(' ') + '</p>';
    }
    if (abjs_da[2])
        s += '<p class="tierstemps"><b>' + _('MSG_TT')
            + '</b> :<br>' + html(abjs_da[2]).replace(/\n/g, '<br>');

    return s;
}

function set_element_relative_position(anchor, element) {
    if (anchor.offsetWidth == 0)
        // Contains an element with absolute position (History)
        try { anchor = anchor.children[1].children[0] || anchor; }
        catch (err) { }

    var pos = findPos(anchor);

    if (pos[1] + anchor.offsetHeight + element.offsetHeight > window_height())
        element.style.top = pos[1] - element.offsetHeight - 1 + 'px';
    else
        element.style.top = pos[1] + anchor.offsetHeight - 1 + 'px';

    var left;
    var align = anchor.tip_align;
    if (align == 'right_if_small')
        if (element.offsetWidth > anchor.offsetWidth)
            align = 'center';
        else
            align = 'right';

    switch (align) {
        case 'center':
            left = Math.max(pos[0] + anchor.offsetWidth - element.offsetWidth / 2, 0);
            break;
        case 'right':
            left = Math.max(pos[0] + anchor.offsetWidth - element.offsetWidth, 0);
            break;
        default:
            left = pos[0];
    }
    element.style.left = Math.min(window_width() - element.offsetWidth, left) + 'px';

    tip.highlighter.style.display = 'block';
    tip.highlighter.style.left = pos[0] + 'px';
    tip.highlighter.style.top = pos[1] + 'px';
    tip.highlighter.style.width = anchor.offsetWidth + 'px';
    tip.highlighter.style.height = anchor.offsetHeight + 'px';
}

function highlight_effect() {
    var t = [];
    for (var o in highlight_list) {
        o = highlight_list[o];
        o.className = o.className
            .replace(/ *highlight8/, '')
            .replace(/ *highlight7/, ' highlight8')
            .replace(/ *highlight6/, ' highlight7')
            .replace(/ *highlight5/, ' highlight6')
            .replace(/ *highlight4/, ' highlight5')
            .replace(/ *highlight3/, ' highlight4')
            .replace(/ *highlight2/, ' highlight3')
            .replace(/ *highlight1/, ' highlight2')
            ;
        if (o.className.search('highlight') != -1)
            t.push(o);
    }
    highlight_list = t;
    if (highlight_list.length)
        return true;
}

function highlight_add(element) {
    // The spaces broke the filter input with empty message ??? XXX WHY
    // element.className += (element.className + ' highlight1')
    //  .replace('empty','').replace(/^ */,'').replace(/ *$/,'') ;
    // With this code, highlight of 'freezed' link does not happen.

    element.className = 'highlight1';
    if (myindex(highlight_list, element) == -1) {
        highlight_list.push(element);
        periodic_work_add(highlight_effect);
    }
}

function update_value_and_tip(o, value) {
    if (!o) {
        alert(value);
        return;
    }
    if (o.tagName == 'SELECT')
        return;

    var v = html(value.toString()) + '&nbsp;';
    if (o.innerHTML != v) {
        highlight_add(o);
        if (o.tagName != 'INPUT')
            o.innerHTML = v;
    }
}

function update_input(element, value, empty) {
    if (!element)
        return;
    if (element_focused == element)
        return; // Do not update value if the input has the focus

    if (empty) {
        element.className = ' empty';
        element.value = '';
    }
    else {
        value = encode_lf_tab(value.toString());
        if (element.value != value) {
            element.value = value;
            highlight_add(element);
        }
    }
    element.theoldvalue = element.value;
}

function cell_goto(td, do_not_focus) {
    var lin = lin_from_td(td);
    var col = col_from_td(td);
    var data_col = data_col_from_col(col);
    var line_id = line_id_from_lin(lin);

    if (do_not_focus !== true && element_focused) {
        //element_focused.blur();// To save values being edited before cell change.
        //if (element_focused && element_focused.onblur)
        //    element_focused.onblur({target: element_focused}); // Buggy Browser
    }

    if (the_current_cell.td != td && do_not_focus !== true)
        the_current_cell.input.selectionEnd = 0; // For Opera
    the_current_cell.jump(lin, col, do_not_focus, line_id, data_col);
}

/* Communication to the server */

function Connection() {
    this.state = 'closed';
    this.t_authenticate = document.getElementById('authenticate');
    this.connection_state = document.getElementById('connection_state');
    // Page are unloaded every 10 minutes server side.
    // So ping every 5 minutes to keep it loaded even if not focused
    if (this.connection_state) // In the table editor
        setInterval(this.ping.bind(this), 5 * 60 * 1000);
}

Connection.prototype.ping = function () {
    if (this.state !== "closed")
        return;
    this.connect();
    setTimeout(this.close.bind(this), 100);
};

Connection.prototype.close = function () {
    if (this.state == "closed")
        return;
    if (this.xmlhttp) {
        this.xmlhttp.abort();
        this.xmlhttp = undefined;
    }
    // '+=' does not work in my Firefox
    this.connection_state.innerHTML = this.connection_state.innerHTML + '...';
    this.state = "closed";
};

Connection.prototype.click_to_revalidate_ticket = function () {
    this.close();
    this.state = 'revalidate';
    var validate = url + '/allow/' + ticket + '/' + millisec();
    this.t_authenticate.innerHTML =
        "<h2>" + _("MSG_session_expired") + "</h2>"
        + _("MSG_session_reconnect")
        + ' <a href="' + validate + '" target="_blank">'
        + _("MSG_session_link") + '</a>';
    this.t_authenticate.style.width = "80%";
    this.t_authenticate.style.height = "80%";
    this.t_authenticate.style.top = "10%";
    this.t_authenticate.style.left = "10%";
    var c = this;
    this.t_authenticate.onmouseup = function () {
        c.t_authenticate.style.width =
            c.t_authenticate.style.height =
            c.t_authenticate.style.left = "auto";
        c.t_authenticate.style.top = c.t_authenticate.style.right = "0px";
    };
    this.t_authenticate.style.display = 'block';
    this.connection_state.innerHTML = _('MSG_unconnected');
};

function click_to_revalidate_ticket() {
    connection_state.click_to_revalidate_ticket();
}
Connection.prototype.need_connection = function () {
    if (is_a_virtual_ue)
        return;
    if (page_id === '')
        return;
    if (this.state == 'revalidate')
        return;
    if (this.state == 'opened')
        return;
    return true;
};

Connection.prototype.connect = function () {
    if (!this.need_connection())
        return;

    if (!this.connection_state)
        report_fake_error('connect ' + document.getElementById('connection_state'));

    var connection = add_ticket(year
        + '/' + semester + '/' + ue + '/' + page_id + '/0'
        + '.' + page_index + '.' + table_creation_date + '.' + table_load_time
        + '.' + (this.last_sync / 1000).toFixed(0));

    if (this.xmlhttp)
        this.xmlhttp.abort();
    this.xmlhttp = new XMLHttpRequest();
    this.xmlhttp.nb_read = 0;
    this.xmlhttp.js_buffer = '';
    this.xmlhttp.connection = this;
    this.xmlhttp.open("GET", connection, true);
    this.xmlhttp.setRequestHeader("If-Modified-Since",
        "Thu, 1 Jan 1970 00:00:00 GMT");
    this.xmlhttp.setRequestHeader("Cache-Control", "no-cache");

    // Must be defined last: resetted by .open()
    this.xmlhttp.onreadystatechange = function () {
        if (this.readyState >= 3 && this.status == 200) {
            if (this.responseText == '<html>ticket</html>') {
                this.connection.click_to_revalidate_ticket();
                return;
            }
            if (this.connection.state != 'opened') {
                this.connection.state = 'opened';
                this.connection.connection_state.innerHTML = _('MSG_connected');
                this.connection.t_authenticate.style.display = 'none';
                requester.try_send();
            }
            var t;
            t = this.responseText.substr(this.nb_read);
            this.js_buffer += t;
            this.nb_read += t.length;
            // Evaluate only complete <script>....</script> sequence
            t = this.js_buffer.split("</script>");
            var to_eval = '';
            for (var i in t) {
                if (i != t.length - 1)
                    to_eval += t[i].split('<script>')[1] + ';';
                else
                    this.js_buffer = t[i];
            }
            eval(to_eval.replace(this.clean_js, '//'));
            // Will not work if the network is cut while transfering
            // and the page is unloaded on the server and
            // nothing was modified server side after this time.
            this.connection.last_sync = millisec();
            // If the buffer is really too big: create a new one
            if (this.nb_read > 10000000) {
                this.connection.close();
                this.connection.connect();
            }
        }
    };
    this.xmlhttp.addEventListener('error', function () {
        if (this.connection.state != 'closed') {
            this.connection_state.innerHTML += 'Connection Lost';
            this.connection.close();
            this.connection.connect();
        }
    });

    this.xmlhttp.send();
}

var do_reload_when_all_saved = false;

function restore_unsaved_forgot() {
    localStorage['/' + year + '/' + semester + '/' + ue] = '';
    index = localStorage['index'];
    localStorage['index'] = index.replace(RegExp('\n/' + year + '/' + semester + '/' + ue,
        'g'), '');
    popup_close();
}

function restore_unsaved_do_save() {
    for (var i in restore_unsaved.t_splited)
        requester.push(restore_unsaved.t_splited[i]);
    do_reload_when_all_saved = true;
    if (GUI) {
        GUI.add('restore_unsaved', '', 'save');
        GUI.save();
    }
    create_popup('import_div', _('MSG_currently_saving'), '', '', false);
}

function display_requests(t_splited, history) {
    var message = '';
    for (var i in t_splited) {
        var line = t_splited[i].split('/');
        if (line[0] == 'cell_change' || line[0] == 'comment_change') {
            var data_col = data_col_from_col_id(line[1]);
            var line_id = line[2];
            line[3] = decode_uri(line[3]);
            if (data_col !== undefined && lines[line_id] !== undefined) {
                if (!history && lines[line_id][data_col].value == line[3])
                    continue; // Unchanged value
                message += '<tr><td>' + html(lines[line_id][0].value)
                    + '<td>' + html(lines[line_id][1].value
                        + ' ' + lines[line_id][2].value)
                    + '<td>' + html(columns[data_col].title)
                    + '<td>'
                    + (line[0] == 'comment_change'
                        ? '<b>' + _("F_cell_comment") + '</b>: '
                        : '')
                    + html(line[3]) + '</tr>';
                continue;
            }
        }
        if (line[0] == 'column_attr_mcq') {
            var data_col = data_col_from_col_id(line[1]);
            message += '<tr><td colspan="2">' + _('B_MCQ')
                + '<td>' + html(columns[data_col].title)
                + '<td>' + html(line[2]) + '</tr>';
            continue;
        }
        var attr = line[0];
        var search = [
            'TIP_' + attr + '__', 'BEFORE_' + attr, 'TIP_' + attr, 'TITLE_' + attr,
            'TIP_table_attr_' + attr, 'MSG_' + attr];
        var action = '';
        for (var j in search)
            if (_(search[j]) != search[j]) {
                action = _(search[j]);
                break;
            }
        if (action) {
            line.splice(0, 1);
            if (attr.indexOf("column_") == 0) {
                var data_col = data_col_from_col_id(line[0]);
                var title = data_col > 0 ? columns[data_col].title : line[0];
                line.splice(0, 1);
                message += '<tr><td colspan="2">' + action.split('<br>')[0]
                    + '<td>' + html(title)
                    + '<td colspan="3">' + html(decode_uri(line.join(' - ')))
                    + '</tr>';
            }
            else
                message += '<tr><td colspan="2">' + action.split('<br>')[0]
                    + '<td colspan="2">' + html(decode_uri(line.join(' - ')))
                    + '</tr>';
        }
        else
            message += '<tr><td>BUG:' + attr + '</tr>';
    }
    return message;
}

function display_requests_add_tag(message, history) {
    return '<div style="overflow:auto;'
        + (history ? 'min-width:60em;max-height:25vh' : 'height:10.5em')
        + '">'
        + '<table class="colored"><tr><th>'
        + _('COL_TITLE_ID')
        + '<th>' + _('COL_TITLE_firstname')
        + ' ' + _('COL_TITLE_surname')
        + '<th>' + _('TH_column')
        + '<th>'
        + (history ? _('TH_value') : _('TH_unsaved_value'))
        + '</tr>'
        + message
        + '</table></div>';
}

function restore_unsaved() {
    if (GUI)
        GUI.add('restore_unsaved', '', 'localStorage=' + !!localStorage);
    if (!localStorage)
        return;
    var t = localStorage['/' + year + '/' + semester + '/' + ue];
    if (!t)
        return;
    var t_splited = t.split('\n');
    var message = display_requests(t_splited);
    if (message == '') {
        restore_unsaved_forgot();
        return;
    }
    restore_unsaved.t_splited = t_splited;
    create_popup('restoring_data', _('ASK_restore'),
        '',
        display_requests_add_tag(message)
        + '<button onclick="restore_unsaved_forgot()">'
        + _('B_unsaved_forgot') + '</button> '
        + '<button onclick="restore_unsaved_do_save()">'
        + _('B_unsaved_save') + '</button> '
        + '<button onclick="popup_close()">'
        + _('B_unsaved_cancel') + '</button>',
        false);
}

/*
 ****************************************************************************
 * Management of periodic work.
 * Once added, the function is called every 0.1 seconds until it returns false
 * 'add' can be called from a periodic function, in this case the function
 * may be called more than one in a period.
 * When a function is added to the list, it goes to the end,
 * so it is processed after the others.
 * The same function can not be added twice
 ****************************************************************************
 */

var periodic_work_functions = [];
var periodic_work_id;

function periodic_work_add_once(table, item) // Do not use this
{
    var i = myindex(table, item);
    if (i == -1)
        table.push(item);
    else {
        table.splice(i, 1);
        table.push(item);
    }
}

function periodic_work_in_queue(f) // The function is in the queue
{
    return myindex(periodic_work_functions, f) != -1;
}

function periodic_work_add(f) {
    periodic_work_add_once(periodic_work_functions, f);
    if (periodic_work_id === undefined)
        periodic_work_id = setInterval(periodic_work_do,
            periodic_work_period);
}

function periodic_work_remove(f) {
    var i = myindex(periodic_work_functions, f);
    if (i != -1)
        periodic_work_functions.splice(i, 1);
}

function periodic_work_do() {
    var f, to_do;
    var to_continue = [];
    while (periodic_work_functions.length) {
        to_do = periodic_work_functions;
        periodic_work_functions = [];
        for (f in to_do) {
            f = to_do[f];
            if (f())
                periodic_work_add_once(to_continue, f);
        }
    }
    periodic_work_functions = to_continue;
    if (to_continue.length == 0) {
        clearInterval(periodic_work_id);
        periodic_work_id = undefined;
    }
}

function RequesterSet() {
    this.requests = [];
}
RequesterSet.prototype.push = function (request) {
    this.requests.push(request);
};
RequesterSet.prototype.merge = function (requester_set) {
    this.requests = this.requests.concat(requester_set.requests);
};

function Requester() {
    this.sent = undefined;
    this.to_send = new RequesterSet();
    this.sender = this.try_send.bind(this);
    this.retry_time = 1000;
    this.history = [];
    this.log('init');
}

Requester.prototype.there_is_unsaved_data = function () {
    return this.sent || this.to_send.requests.length;
}

Requester.prototype.log = function (txt) {
    return;
    var text = txt + " sent=";
    if (this.sent)
        text += this.sent.timestamp + '/' + JSON.stringify(this.sent.requests);
    else
        text += 'NONE';
    text += ' to_send=' + JSON.stringify(this.to_send.requests);
    text += ' history=' + JSON.stringify(this.history);
    console.log(text);
}

Requester.prototype.status = function () {
    var history_size = 10;
    if (!this.feedback)
        this.feedback = document.getElementById('requester_feedback');

    var text = [], more;
    for (var i in this.history) {
        if (this.history[i][0].indexOf('bad.png') != -1)
            more = _("TIP_red_square");
        else if (this.history[i][0].indexOf('bug.png') != -1)
            more = _("ERROR_server_bug");
        else
            more = _("TIP_green_square");
        text.push(hidden_txt('<var instanttip="1" style="opacity:'
            + (0.1 + 0.9 * (history_size - this.history.length + Number(i)) / (history_size - 1))
            + '">' + this.history[i][0] + this.history[i][1].length + '</var>',
            more
            + display_requests_add_tag(display_requests(this.history[i][1], true), true)));
    }
    if (this.sent)
        text.push(hidden_txt(
            '<b style="background: #FFA500;" instanttip="1">⏱'
            + this.sent.requests.length + '</b>',
            display_requests_add_tag(display_requests(this.sent.requests, true), true)));
    if (this.to_send.requests.length)
        text.push(hidden_txt(
            '<var instanttip="1">🅿️' + this.to_send.requests.length + '</var>',
            display_requests_add_tag(display_requests(this.to_send.requests, true), true)));

    this.feedback.innerHTML = text.join(' ');
    this.history.splice(0, Math.max(0, this.history.length - history_size));
};

Requester.prototype.push = function (request) {
    this.log('push start');
    this.last_push = millisec();
    this.to_send.push(request);
    this.status();
    this.log('push end');
    periodic_work_add(this.sender);
};

Requester.prototype.try_send = function () {
    this.log('trysend start');
    if (!table_attr.autosave)
        return true;
    if (millisec() - this.last_push < 100)
        return true; // May be a second change will quickly come
    if (this.sent && millisec() - this.sent.timestamp < 5000)
        return true; // Wait server answer before requesting again
    if (!this.there_is_unsaved_data())
        return;
    if (this.sent === undefined) {
        this.to_send.timestamp = millisec();
        this.sent = this.to_send;
    } else {
        this.sent.timestamp = millisec();
        this.sent.merge(this.to_send);
    }
    this.to_send = new RequesterSet();
    this.send_to_server();
}

Requester.prototype.send_event = function (event) {
    this.log('send_event ' + event.target.response);
    if (!event.target.response)
        return;
    if (event.target.response.length != 2) {
        if (event.target.response == 'bad.png') {
            // Alert("TIP_red_square");
            this.history.push(['<img class="server" src="_FILES_/bad.png">', this.sent.requests]);
        }
        else if (event.target.response == 'bug.png') {
            Alert("ERROR_server_bug");
            this.history.push(['<img class="server" src="_FILES_/bug.png">', this.sent.requests]);
        }
        else if (event.target.response == 'ticket') {
            click_to_revalidate_ticket();
            return;
        }
        this.sent = undefined;
        this.status();
        return;
    }
    if (!this.sent || event.target.response[0] != this.sent.timestamp)
        return;
    this.history.push(['<img class="server" src="_FILES_/ok.png">',
        this.sent.requests]);
    this.sent = undefined;
    this.status();
    page_index = event.target.response[1];
    this.retry_time = 1000;
    if (do_reload_when_all_saved) {
        restore_unsaved_forgot();
        window.location = window.location;
        do_reload_when_all_saved = false;
    }
};

Requester.prototype.send_event_error = function (_event) {
    this.log('sendtoserver error');
    this.status();
    setTimeout(this.send_to_server.bind(this), this.retry_time);
    this.retry_time *= 1.1;
};

Requester.prototype.send_to_server = function () {
    if (!this.sent)
        return; // Answer yet received
    this.log('sendtoserver');
    this.status();
    var xhr = new XMLHttpRequest();
    xhr.addEventListener('readystatechange', this.send_event.bind(this), false);
    xhr.addEventListener('error', this.send_event_error.bind(this), false);
    xhr.responseType = 'json';
    xhr.open("POST", add_ticket('pageaction'), true);
    var formData = new FormData();
    formData.append('content', JSON.stringify(
        [this.sent.timestamp, year, semester, ue, page_id, this.sent.requests]));
    xhr.send(formData);
};

// XXX Does not work if there is 2 browser page open in the table.
Requester.prototype.store_unsaved = function () {
    if (!table_attr.autosave)
        return;
    if (!localStorage) {
        Alert("ERROR_save_to_localstorage_failed");
        return;
    }
    var s = this.sent ? this.sent.requests.concat(this.to_send.requests) : this.to_send.requests;
    if (s.length == 0)
        return;
    var key = '/' + year + '/' + semester + '/' + ue;
    if (localStorage[key])
        localStorage[key] += '\n' + s.join('\n');
    else {
        localStorage[key] = s.join('\n');
        index = localStorage['index'];
        if (!index)
            index = '';
        index += '\n' + key;
        localStorage['index'] = index;
    }
};


requester = new Requester();

function append_image(td, text, force) {
    if (!table_attr.modifiable && !force)
        return;
    if (is_a_virtual_ue)
        return;
    requester.push(text);
}

function login_to_line_id(login) {
    if (login_to_line_id.dict === undefined) {
        login_to_line_id.dict = {};
        for (var line_id in lines)
            login_to_line_id.dict[login_to_id(lines[line_id][0].value)] = line_id;
    }
    return login_to_line_id.dict[login_to_id(login)];
}


/* Communication from the server */
function Xcell_change(col, line_id, value, date, identity, history) {
    var data_col = data_col_from_col_id(col);
    if (data_col === undefined) {
        console.log("Unexpected Xcell_change");
        return;
    }
    add_a_new_line(line_id, true);

    var cell = lines[line_id][data_col];
    var empty_before = line_empty(lines[line_id]);

    cell.set_value_real(value);
    cell.author = identity;
    cell.date = date;
    cell.history = history;
    update_nr_empty(empty_before, line_empty(lines[line_id]),
        lines[line_id].is_filtered);

    var td = td_from_line_id_data_col(line_id, data_col);

    if (td !== undefined) {
        update_cell(td, cell, columns[data_col], undefined, lines[line_id]);
        if (td == the_current_cell.td) {
            the_current_cell.update_cell_headers();
            the_current_cell.jump(the_current_cell.lin,
                the_current_cell.col);
        }
    }
    update_line(line_id, data_col);
}

function Xcomment_change(identity, col, line_id, value, history) {
    var data_col = data_col_from_col_id(col);
    add_a_new_line(line_id);
    var cell = lines[line_id][data_col];

    cell.set_comment(value, history);
    cell.author = identity;

    var td = td_from_line_id_data_col(line_id, data_col);
    if (td !== undefined) {
        update_cell(td, cell, columns[data_col], undefined, lines[line_id]);
        if (cell === the_current_cell.cell)
            the_current_cell.update_cell_headers();
    }
}

function Xcolumn_delete(page, col) {
    var data_col = data_col_from_col_id(col);
    if (data_col === undefined) {
        console.log("Xcolumn_delete unexpected");
        return; // Should never be  here
    }
    for (line_id in lines)
        lines[line_id].splice(data_col, 1);
    columns.splice(data_col, 1);
    for (data_col in columns)
        columns[data_col].data_col = Number(data_col);
    if (the_current_cell.col == col)
        the_current_cell.col = columns[0].the_id;
    the_current_cell.update();

    if (page != ' ')
        Alert("MSG_refresh");
    the_current_cell.do_update_column_headers = true;
    the_current_cell.update_headers();
    table_fill(true, true, true, true);
}

function Xcolumn_attr(attr, col, value) {
    var data_col = data_col_from_col_id(col);
    var column;
    if (data_col === undefined) {
        data_col = add_empty_columns();
        column = columns[data_col];
        column.the_id = col;
        column.local_id = undefined;
        column.is_empty = false;
        table_fill();
    }
    else
        column = columns[data_col];
    column[attr] = column_parse_attr(attr, value, column, true);
    attr_update_user_interface(column_attributes[attr], column, true);
}

function Xtable_attr(attr, value) {
    table_attr[attr] = table_attributes[attr].formatter(value, true);
    the_current_cell.update_table_headers();
}

function update_table_size() {
    // In order to force Gecko to update table size
    var tr = table.childNodes[table_attr.nr_lines + nr_headers - 1];

    table.removeChild(tr);
    table.appendChild(tr);
}

function stop_event(event) {
    if (!event)
        return;
    if (event.real_event)
        event = event.real_event;
    if (event.stopPropagation)
        event.stopPropagation(true);
    if (event.preventDefault)
        event.preventDefault(true);
    else {
        event.returnValue = false;
        try { event.keyCode = 0; } catch (e) { };
    }

    event.cancelBubble = true;
}

// Set comment

function comment_change(line_id, data_col, comment, td) {
    create_column(columns[data_col]);
    add_a_new_line(line_id);

    var ok = lines[line_id][data_col].changeable(lines[line_id],
        columns[data_col]);
    if (ok !== true) {
        alert_append(ok);
        return;
    }

    lines[line_id][data_col].set_comment(comment);
    var col_id = columns[data_col].the_id;
    append_image(td, 'comment_change/' + col_id + '/' +
        line_id + '/' + encode_uri(comment));
    if (td)
        update_cell(td, lines[line_id][data_col], columns[data_col], undefined,
            lines[line_id]);
}

function comment_on_change() {
    var input = the_comment;

    if (the_comment === undefined)
        return;

    var value = decode_lf_tab(input.value);

    if (lines[the_current_cell.line_id][the_current_cell.data_col].comment == value)
        return;

    if (!the_current_cell.cell.modifiable(lines[the_current_cell.line_id],
        the_current_cell.column)) {
        Alert("ERROR_value_not_modifiable");
        return;
    }

    the_current_cell.td.className += ' comment';
    comment_change(the_current_cell.line_id, the_current_cell.data_col,
        value, the_current_cell.td);
}

function the_filters() {
    var s = "";
    var column;

    for (var data_col in columns) {
        column = columns[data_col];
        if (column.filter !== '')
            s += _("MSG_filter_on") + ' « ' + column.title + ' » : « '
                + column.filter + ' »\n';
    }
    return s;
}

function printable_introduction() {
    var problems = [];

    if (tr_classname !== undefined)
        if (sort_columns[0].data_col != 2)
            problems.push(_("WARN_not_name_sorted"));

    if (tr_classname !== undefined && popup_on_red_line) {
        var nb = 0;
        for (var i in filtered_lines) {
            line = filtered_lines[i];
            if (line[0].value == '' && line[1].value == '')
                continue;
            if (line[tr_classname].value === 'non')
                nb++;
        }
        if (nb)
            problems.push(nb + ' ' + _("WARN_students_not_registered"));
    }

    return '<p class="hidden_on_paper printable_introduction">'
        + _("MSG_not_printed") + "<br>"
        + _("MSG_sorted_by") + " «<b>" + sort_columns[0].title + '</b>» '
        + (sort_columns[1]
            ? _("MSG_sorted_by_more") + ' «<b>' + sort_columns[1].title + '</b>»<br>'
            : ''
        )
        + html(the_filters()).replace(/\n/g, '<br>')
        + '<span style="background:#F00;color:white">'
        + problems.join('<br>') + '</span>';
}

function display_on_signature_table(line) {
    if (line[0].value == '' && line[1].value == '')
        return false; // Empty line

    if (tr_classname === undefined)
        return true; // Not concept of registered student

    if (line[tr_classname].value != 'non')
        return true; // The student is registered in the UE

    if (popup_on_red_line)
        return false; // Do not display unregistered student.

    return true;
}

function lines_in_javascript() {
    var s = [], t, x, i;
    for (i in filtered_lines) {
        line = filtered_lines[i];
        if (display_on_signature_table(line)) {
            t = [];
            for (var data_col in columns)
                if (!columns[data_col].is_empty)
                    t.push(line[data_col].get_data());

            x = the_student_abjs[line[0].value];
            if (x && x[2])
                t.push(js('<li>' + x[2].substr(0, x[2].length - 1).replace(/\n/g, '<li>')));

            s.push('[' + t.join(',') + ']');
        }
    }
    return '[\n' + s.join(',\n') + ']';
}

function columns_in_javascript() {
    var s = [], p, column, all_cls;
    all_cls = column_list_all();
    for (var i in all_cls)
        columns[all_cls[i]].ordered_index = i;

    for (var data_col in columns) {
        column = columns[data_col];
        if (column.is_empty)
            continue;
        p = [];

        for (var attr in column_attributes) {
            var value = column[attr];
            p.push('"' + attr + '":'
                + (typeof value == 'number' || typeof value == 'boolean' ? value : js(column[attr])));
        }

        p.push("green_filter:" + column.green_filter);
        p.push("red_filter:" + column.red_filter);
        p.push("greentext_filter:" + column.greentext_filter);
        p.push("redtext_filter:" + column.redtext_filter);
        if (isNaN(column.red) || column.red === '')
            p.push("color_red:" + js(column.red));
        else
            p.push("color_red:" + column.red);
        if (isNaN(column.green) || column.green === '')
            p.push("color_green:" + js(column.green));
        else
            p.push("color_green:" + column.green);

        if (isNaN(column.redtext) || column.redtext === '')
            p.push("color_redtext:" + js(column.redtext));
        else
            p.push("color_redtext:" + column.redtext);
        if (isNaN(column.greentext) || column.greentext === '')
            p.push("color_greentext:" + js(column.greentext));
        else
            p.push("color_greentext:" + column.greentext);
        p.push("min:" + column.min);
        p.push("max:" + column.max);
        p.push("ordered_index:" + column.ordered_index);
        p.push("rounding:" + js(column.rounding));
        p.push("data_col:" + data_col);
        s.push('Col({' + p.join(',\n') + '})');
    }
    return '[\n' + s.join(',\n') + ']';
}

function button_toggle(dictionnary, data_col, tag, event) {
    if (dictionnary[data_col]) {
        delete dictionnary[data_col];
        tag.className = tag.className.replace(/ toggled/g, '');
    }
    else {
        tag.className += ' toggled';
        dictionnary[data_col] = true;
    }
    do_printable_display = true;
    if (event)
        stop_event(the_event(event));
}

function toggle_button(text, dictionnary, name, help) {
    if (help)
        text = hidden_txt(text, help);
    var toggled = eval(dictionnary)[name] ? " toggled" : "";
    var a = '<span class="button_toggle' + toggled
        + '" id="' + dictionnary + '.' + name
        + '" onclick="button_toggle('
        + dictionnary + ",'" + name + "',this,event)\">"
        + text + '</span>';
    return a;
}

function set_radio(variable, selected) {
    var children = document.getElementById('id_' + variable).childNodes;

    for (var i = 0; i < children.length; i++)
        if (children[i].textContent == _(selected)) {
            children[i].onclick();
            return;
        }
    button_not_found;
}

function radio_buttons(variable, values, selected, action) {
    var value, the_class, tip, v;
    var s = [];

    if (variable.indexOf('.') == -1)
        s.push('<script>' + variable + ' = ' + js(selected) + ';</script>');

    if (action === undefined)
        action = "do_printable_display=true";
    s.push('<var id="id_' + variable + '">');
    for (var i in values) {
        value = values[i];

        if (value.sort) {
            tip = value[1];
            value = value[0];
        }
        else
            tip = '';


        if (value == selected)
            the_class = 'toggled';
        else
            the_class = '';
        v = '<span class="button_toggle ' + the_class
            + '" onclick="' + variable + "=" + js2(value)
            + "; radio_clean(this);this.className += ' toggled' ;"
            + action + ';stop_event(the_event(event))">' +
            (tip ? hidden_txt(value, tip) : value) + '</span>';
        s.push(v);
    }
    s.push('</var>');
    return s.join('\n');
}

function radio_clean(t) {
    for (t = t.parentNode.firstChild; t; t = t.nextSibling)
        if (t.tagName == 'SPAN')
            t.className = t.className.replace(/ toggled/, '');
}

function compute_grouped_by_sorted(grouped_by) {
    var cols = column_list_all();
    var s = [];
    for (var i in cols) {
        var data_col = cols[i];
        if (grouped_by[data_col])
            s.push(data_col);
    }
    return s;
}

function compute_groups_key_function(grouped_by) {
    var cols = compute_grouped_by_sorted(grouped_by);
    var s = [];
    for (var i in cols)
        s.push('line[' + cols[i] + '].value');
    if (s.length == 0) {
        function _grp_key_() { return ''; };
        return _grp_key_;
    }
    return eval('function _grp_key_(line) { return ' + s.join('+"\001"+') + ';}; _grp_key_');
}

/* XXX Deprecated because slow and the columns are not in the good order */
function compute_groups_key(grouped_by, line) {
    var s = [];
    for (var data_col in grouped_by)
        if (grouped_by[data_col])
            s.push(line[data_col].value);
    return s.join('\001');
}

function compute_groups_values(grouped_by) {
    var g = {};
    var get_key = compute_groups_key_function(grouped_by);
    for (var line_id in lines)
        g[get_key(lines[line_id])] = true;
    tabl = [];
    for (var gg in g)
        tabl.push(gg);
    tabl.sort();
    return tabl;
}

function goto_resume() {
    window_open(add_ticket(year + '/' + semester + '/' + ue + '/resume'));
}

function html_begin_head() {
    var a = '{';
    for (var i in table_attr) {
        var value = table_attr[i];
        if (table_attributes[i])
            value = table_attributes[i].formatter(value);
        a += i + ':' + JSON.stringify(value) + ',';
    }
    a = a.substr(0, a.length - 1) + '}';

    var languages = '';
    var all = preferences['language'].split(',');
    for (var i in all)
        languages += '<script onload="this.onloadDone=true;" src="_FILES_/'
            + all[i] + '.js" charset="UTF-8"></script>';

    return [
        '<html><head>',
        '<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />',
        '<meta charset="utf-8">',
        '<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">',
        '<link rel="stylesheet" href="_FILES_/style.css" type="text/css">',
        '<script src="_FILES_/utilities.js" onload="this.onloadDone=true;" charset="UTF-8">',
        '</script>',
        '<script src="_FILES_/middle.js" onload="this.onloadDone=true;" charset="UTF-8">',
        '</script>',
        '<script src="_FILES_/lib.js" onload="this.onloadDone=true;" charset="UTF-8"></script>',
        '<script src="_FILES_/types.js" onload="this.onloadDone=true;" charset="UTF-8"></script>',
        '<script src="_FILES_/abj.js" onload="this.onloadDone=true;" charset="UTF-8"></script>',
        '<style id="computed_style"></style>',
        '<script>var translations = {} ; </script>',
        languages,
        table_headers,
        '<script>',
        'page_id = "" ;',
        'url = "' + url.split('/=')[0] + '" ;',
        'my_identity = "' + my_identity + '" ;',
        'year = "' + year + '" ;',
        'semester = "' + semester + '" ;',
        'ticket = "' + ticket + '" ;',
        'upload_max = "' + upload_max + '" ;',
        'ampms = ' + JSON.stringify(ampms) + ' ;',
        'ampms_full = ' + JSON.stringify(ampms_full) + ' ;',
        'days = ' + JSON.stringify(days) + ' ;',
        'days_full = ' + JSON.stringify(days_full) + ' ;',
        'months = ' + JSON.stringify(months) + ' ;',
        'months_full = ' + JSON.stringify(months_full) + ' ;',
        'ue = "VIRTUALUE" ;',
        'real_ue = "' + ue + '" ;',
        'root = [];',
        'suivi = "' + suivi + '";',
        'version = "' + version + '" ;',
        'preferences = ' + JSON.stringify(preferences) + ';',
        'rounding_default = ' + rounding_default + ';',
        'rounding_min = ' + rounding_min + ';',
        'columns = [] ;',
        'lines = {} ;',
        'adeweb = {} ;', // XXX should not be here (LOCAL/spiral.py)
        // "table_headers = " + JSON.stringify(table_headers) + ";",
        'table_attr = ' + a + ';',
        'function columns_filter() { return true ; }',
        'all_the_semesters = ' + js(all_the_semesters) + ' ;',
        wait_scripts, // The function definition
        '</script>',
        '<title>' + ue + ' ' + year + ' ' + semester + '</title>',
        '</head>',
        '<body class="' + the_body.className + '">'
    ].join('\n');
}

function notes_columns() {
    var cols = [];

    var cls = column_list_all();

    for (var data_col in cls) {
        data_col = cls[data_col];
        if (columns[data_col].real_type.should_be_a_float
            && !columns[data_col].is_empty) {
            var a = {
                min: 1000000, max: -1000000, sum: 0, nr: 0,
                nr_nan: 0, nr_pre: 0, nr_abi: 0, nr_abj: 0, nr_ppn: 0,
                sum2: 0
            };
            a.data_col = Number(data_col);
            cols.push(a);
        }
    }
    return cols;
}

// XXX yet done somewhere else
function student_search(id) {
    for (var line_id in lines)
        if (lines[line_id][0].value == id)
            return line_id;

    id = login_to_id(id);
    for (var line_id in lines)
        if (lines[line_id][0].value == id)
            return line_id;
}

function remove_highlight() {
    if (the_current_line) {
        var name = the_current_line.className.replace('highlight_current', ' ');
        // The spaces broke the filter input : WHY XXX !!!
        name = name.replace(/^ */, '').replace(/ *$/, '');
        the_current_line.className = name;
        the_current_line = undefined;
    }
}

function blur_focus_in_tip() {
    for (var focus = element_focused; focus; focus = focus.parentNode) {
        if (focus === tip) {
            element_focused.blur();
            element_focused = undefined;
            return;
        }
    }
}

function hide_the_tip_real(force) {
    var tip = get_tip_element();
    if (!tip.tip_target)
        return;
    if (!force && tip.innerHTML.indexOf('overflow:') != -1)
        return; // To let the user scroll
    if (!force && tip.tip_target.getAttribute('instanttip'))
        return;
    if (set_the_tip_content)
        return;
    tip.style.visibility = "hidden";
    tip.className = "tip_fade_out";
    tip.tip_target = undefined;
    tip.highlighter.style.display = 'none';
}

function update_popup_on_red_line() {
    var e = document.getElementById('popup_on_red_line');

    if (e)
        if (popup_on_red_line)
            e.lastChild.className = '';
        else
            e.lastChild.className = 'stroked';
}

function change_popup_on_red_line(event) {
    popup_on_red_line = !popup_on_red_line;
    update_popup_on_red_line();
    table_attr_set('popup_on_red_line', popup_on_red_line, the_event.target);
}


function update_mail(login, mail) {
    table_attr.mails[login] = mail;
}

function update_portail(login, portail) {
    table_attr.portails[login] = portail;
}

function change_size(dx, dy) {
    table_attr.nr_columns += dx;
    table_attr.nr_lines += dy;
    table_init();
    table_fill(false, true, false, true);
}

function change_table_size(select) {
    var i = select.childNodes[select.selectedIndex].innerHTML;
    i = Math.floor(i.split(' ')[0]); // Remove text after number
    if (select.id == 't_table_attr_nr_columns') {
        table_attr.nr_columns = i;
        column_offset = 0;
        change_option('column_offset');
        if (the_current_cell.col >= table_attr.nr_columns)
            the_current_cell.col = table_attr.nr_columns - 1;
    }
    else {
        table_attr.nr_lines = i;
        line_offset = 0;
    }
    table_init();
    table_fill(false, true, false, true);
    update_vertical_scrollbar();
    setTimeout("the_current_cell.update()", 100);
}


function nice_scale_from(min, v, max, bigger) {
    var t = [];
    var i, j;

    for (i = min; i <= bigger; i = Math.ceil(1.3 * i))
        t.push(i);
    if (myindex(t, v) == -1)
        t.push(v);
    if (myindex(t, max) == -1)
        t.push(max);
    for (i = 1; i <= 10; i *= Math.ceil(1.3)) {
        j = v + i;
        if (myindex(t, j) == -1)
            t.push(j);
        j = v - i;
        if (j > min && myindex(t, j) == -1)
            t.push(j);
    }
    t.sort(function (x, y) { return x - y; });
    t[myindex(t, max)] = max + ' (' + _('MSG_all') + ')';
    return t;
}

function update_a_menu(min, current, all, max, select) {
    if (!select)
        return;

    var i, i_striped;
    var sel;
    var t = nice_scale_from(min, current, all, max);

    for (var ii in t) {
        var created = false;
        var option = select.childNodes[ii];
        if (option === undefined) {
            option = document.createElement('option');
            created = true;
        }
        i = t[ii];
        i_striped = i.toString().replace(' (' + _('MSG_all') + ')', '');
        if (i_striped == current)
            sel = ii;
        option.innerHTML = i;
        option.value = i_striped;
        if (created)
            select.appendChild(option);
    }
    select.selectedIndex = sel;
}

function get_theme(theme) {
    return css_themes[Math.max(myindex(css_themes, theme), 1)];
}

// Used only for table and abj pages
function set_body_theme(the_semester) {
    var theme = preferences.theme === ''
        ? the_semester.substr(0, 1) // A or P or T
        : preferences.theme;
    theme = get_theme(theme);
    the_body.className = the_body.className.replace(/ theme[^ ]*/, '')
        .replace(/ compact_[01]/, '')
        + " theme" + theme + ' compact_' + preferences.compact;
}

function initialise_columns() {
    for (var data_col in columns) {
        columns[data_col].filter = '';
        columns[data_col].data_col = Number(data_col);
        if (columns[data_col].freezed == 'C') {
            tr_classname = data_col;
        }
    }
    if (tr_classname === undefined) {
        default_title = _("DEFAULT_title");
    }
    for (var data_col in columns) {
        init_column(columns[data_col]);
        columns[data_col].need_update = true;
    }
    update_columns();
}

function runlog(the_columns, the_lines) {
    column_get_option_running = true;
    columns = the_columns;
    lines = the_lines;

    if (Number(preferences.zebra_step) > 0)
        zebra_step = Number(preferences.zebra_step);
    else
        zebra_step = 3;

    table_attr['competence'] = table_attributes['competence'].formatter(table_attr['competence']);

    lib_init();

    scrollbar_right = test_bool(preferences.scrollbar_right) == yes;
    if (test_bool(preferences.invert_name) == yes
        && columns.length > 2 && columns[2].title == COL_TITLE_0_2) {
        columns[2].position = columns[1].position - 0.1;
    }
    if (test_bool(preferences.invert_grpseq) == yes
        && columns.length > 4 && columns[4].title == COL_TITLE_0_4) {
        columns[4].position = columns[3].position - 0.1;
    }

    if (table_attr.default_nr_columns)
        table_attr.nr_columns = table_attr.default_nr_columns;
    if (test_bool(preferences.v_scrollbar) == no)
        vertical_scrollbar = undefined;
    popup_on_red_line = table_attr.popup_on_red_line;

    nr_not_empty_lines = 0;
    nr_not_fully_empty_lines = 0;
    for (var line_id in lines) {
        lines[line_id].line_id = line_id;
        switch (line_empty(lines[line_id])) {
            case false: nr_not_empty_lines++;       // Fall thru
            case 1: nr_not_fully_empty_lines++;
        }
    }

    initialise_columns();
    // Default : Name sort

    add_empty_columns();
    if (table_attr.default_sort_column[0] != undefined) {
        // default_sort column is a list
        sort_columns = [];
        for (var i in table_attr.default_sort_column) {
            sort_columns.push(columns[table_attr.default_sort_column[i]]);
            sort_columns[i].dir = 1;
        }
    }
    else if (columns.length > table_attr.default_sort_column) {
        // DEPRECATED, only here for old tables
        if (columns.length > 1) {
            sort_columns = [columns[table_attr.default_sort_column], columns[1]];
            columns[1].dir = 1;
        }
        else
            sort_columns = [columns[table_attr.default_sort_column]];
        columns[table_attr.default_sort_column].dir = 1;
    }
    else {
        table_attr.default_sort_column = 0;
        sort_columns = [columns[0]];
    }

    if (!do_not_read_option) {
        try {
            get_all_options(); // Defined in by tablebookmark ATTRIBUTE
        }
        catch (err) {
        }
    }
    update_line_menu();
    update_column_menu();
    update_popup_on_red_line();
    update_filtered_lines();
    try { table_fill_hook = template_init; } catch (e) { }

    set_body_theme(semester);

    if (!is_a_virtual_ue)
        document.write('<img width="1" height="1" src="'
            + add_ticket(year + '/' + semester + '/' + ue + '/' +
                page_id + "/end_of_load")
            + '" style="position:absolute;left:0;top:0">');

    // This function is used when we want to replace the current window
    // content by the popup content.
    // It is NEEDED because some browser open popup UNDER the current window
    function replace_window_content(new_one) {
        for (var i in the_body.childNodes)
            if (the_body.childNodes[i].style)
                the_body.childNodes[i].style.display = 'none';
        the_body.onunload = '';
        the_body.onkeydown = '';
        new_one();
    }
    if (get_option('print-table', 'a') !== 'a') {
        replace_window_content(function () { print_selection(undefined, undefined, '_self') });
        return;
    }
    if (get_option('signatures-page', 'a') !== 'a') {
        replace_window_content(function () { print_selection(undefined, 1, '_self') });
        return;
    }
    if (get_option('facebook', 'a') !== 'a') {
        replace_window_content(function () {
            tablefacebook('_self',
                get_option('facebook',
                    undefined))
        });
        return;
    }
    if (get_option('graph', 'a') !== 'a') {
        var cols = get_option('graph').split(/=/);
        setTimeout(
            function () {
                the_current_cell.data_col_previous_previous = cols[0] || 0;
                the_current_cell.data_col_previous = cols[1] || 0;
                the_current_cell.data_col_current = cols[2] || 0;
                column_graph_zoom();
            }, 500);
    }
    if (table_forms_element || get_option('tableforms', 'a') !== 'a') {
        setTimeout(table_forms, 1500);
    }


    if (preferences.interface == 'L') {
        dispatch('init');
        return;
    }

    if (get_option('bigfont', 'a') !== 'a') {
        the_body.style.fontSize = '100%';
        document.getElementById('template_style').textContent +=
            'TABLE#thetable TD { height: 3em ; vertical-align: middle }';
    }

    /*
     * The normal interface (not linear)
     */

    table_init();
    autofreeze();
    table_fill(true, true, true);

    if (window.addEventListener)
        /** DOMMouseScroll is for mozilla. */
        window.addEventListener('DOMMouseScroll', wheel, false);
    /** IE/Opera. */
    window.onmousewheel = document.onmousewheel = wheel;
    window.onblur = connection_state.close.bind(connection_state);
    window.onfocus = connection_state.connect.bind(connection_state);

    if (window.attachEvent) {
        // IE does not launch resize event if the window is loading
        periodic_work_add(manage_window_resize_event);
    }
    else
        window.onresize = manage_window_resize_event;
    manage_window_resize_event();

    // Firefox bug : the page refresh reload the old iframe, not the new one
    // setTimeout(reconnect, 10) ;

    the_current_cell.jump(nr_headers, 0, true);
    the_current_cell.update_table_headers();

    // The restore popup must not be erased by the table filling
    var old_table_fill_hook = table_fill_hook;
    function ask_for_restore() {
        if (old_table_fill_hook)
            old_table_fill_hook();
        restore_unsaved();
    }
    table_fill_hook = ask_for_restore;

    document.getElementById("linefilter").focus();
    column_get_option_running = false;

    if (table_attr.code === '' && table_attr.modifiable !== 0
        && i_am_the_teacher && nr_not_empty_lines === 0
        && millisec() - get_date_tomuss(table_creation_date).getTime() < 86400000
        && table_attr.contains_users
    )
        send_invitation();

    analyse_table();
    setTimeout(connection_state.connect.bind(connection_state), 500);
}

function analyse_table() {
    slash_in_title();
    grade_type_in_average();
}

function slash_in_title() {
    var messages = [];
    var bad = RegExp("[^0-9]/ *([0-9]+)");
    for (var data_col in columns) {
        var found = bad.exec(columns[data_col].title);
        if (found)
            messages.push(
                _("MSG_slash_in_title")
                    .replace(/{title}/g, html(columns[data_col].title))
                    .replace(/{max}/g, found[1])
                    .replace(/{type}/g, columns[data_col].type)
            );
    }
    set_message('slash_in_title', 2, '<br>'.join(messages));
}

function grade_type_in_average() {
    var messages = [];
    for (var data_col in columns)
        if (columns[data_col].type == "Moy")
            for (var i in columns[data_col].average_columns) {
                var col = columns[columns[data_col].average_columns[i]];
                if (col.grade_type > 0) {
                    messages.push(_("MSG_grade_type_in_average")
                        .replace(/{title}/g, html(col.title))
                        .replace(/{doc}/g, url + '/doc_table.html#T_Ue_Grade'));
                }
            }
    set_message('grade_type_in_average', 1, '<br>'.join(messages));
}

function user_is_doing_nothing() {
    if (millisec() - last_user_interaction > 60000) // 60 seconds
        return true;
    return false;
}

/*REDEFINE
*/
function do_change_abjs(m) {
    the_student_abjs = m;
}

var the_student_abjs = {};
function change_abjs(m) {
    do_change_abjs(m);
    if (user_is_doing_nothing())
        table_fill(true);
}

function the_full_login_list(login, results, add) {
    if (!document.getElementById('students_list')) {
        // We are in a table, not the home page
        login_list(login, results);
    }
    else
        full_login_list(login, results, add); // Defined in home2.js
}

function set_updating(bool) {
    var u = document.getElementById('updating');
    if (u)
        if (bool)
            u.style.display = 'inline';
        else
            u.style.display = 'none';
}

window.onerror = function (message, url_error, lineNumber, _colNumber, error) {
    if (lineNumber == 0)
        return false; // Error not in a TOMUSS script
    window.onerror = function () { return false; }; // Only first error
    var i = document.createElement('IMG');
    var now = new Date();
    var user;
    try { user = window.username || window.my_identity; }
    catch (e) { user = window.my_identity; }
    function clean(url) {
        return url.replaceAll(RegExp('https?://[^/]*[^:]*/([^/:?]+)([?][^:]*)*', 'g'), '$1');
    }

    i.width = i.height = 1;
    i.src = url + '/log/javascript_errors/'
        + encode_uri(JSON.stringify([
            now, message, clean(url_error), lineNumber, window.location.pathname, user,
            navigator.platform + '/' + navigator.appName
            + '/' + navigator.appVersion + '/' + navigator.product,
            navigator.userAgent, clean(error.stack.toString())]))
    if (!window.server_log)
        server_log = document.getElementsByTagName('BODY')[0];
    server_log.appendChild(i);
    return false;
};

function report_fake_error(message) {
    window.onerror(message, window.location.toString(), 99999, 99999, new Error('fake'));
}

function desync() {
    if (!popup_is_open() && !requester.there_is_unsaved_data()) {
        Alert('ALERT_desync');
        window.location.reload();
    }
    else
        setTimeout(desync, 1000);
}

// XXX COPY/PASTE in the end of new_page.py
window.Xcell_change = Xcell_change;
window.Xcomment_change = Xcomment_change;
window.Xcolumn_delete = Xcolumn_delete;
window.Xcolumn_attr = Xcolumn_attr;
window.Xtable_attr = Xtable_attr;
window.change_abjs = change_abjs;
window.update_mail = update_mail;
window.update_portail = update_portail;
window.login_list = login_list;
window.click_to_revalidate_ticket = click_to_revalidate_ticket;
window.set_updating = set_updating;
window.desync = desync;
