"""
Your local configuration of TOMUSS goes here.

You can redefine any method, function or variable in the other modules.

This file contains the most important configuration options
that can't be modified interactively in the configuration table.
"""

import socket
import time
from .. import configuration
from .. import servers
from .. import entity

# Define a super user login list before activating the authentication
# So you will be able to edit configuration table.
configuration.root = ("super.user",)


# How to authenticate users with a CAS provider
configuration.cas = "https://cas.domain.org/cas"

# True to use OpenID
if False: # pylint: disable=using-constant-test
    # The identity is the mail address
    # Modify the ACLS table to give the right to connect to users
    # (for roots users, do it before switching to OpenID)

    # Google
    configuration.cas = 'https://www.google.com/accounts/o8/id'
    # Yahoo
    configuration.cas = 'https://me.yahoo.com'
    configuration.Authenticator = configuration.authenticators.OpenID

# To see how to modify the student lists and other values: see regtestpatch.py


# Add a cached attribute for User
def birth(items):
    """Get the birth date from item.ID_ or item.student_id or ..."""
    for item in items:
        item.birth = 2021
birth.ttl = 7 * 86400 # update once a week (default is 30000 seconds)
configuration.Users.add_attribute(birth)

# Add a function to User (the self.birth will be automaticaly vectorized)
def age(self):
    """Get the age"""
    return time.localtime()[0] - self.birth
entity.UsersItem.age = age


# List of 'suivi' servers to launch.
# Add a server for each semester of the current year.
# You must redefine this to add old years

configuration.suivi = servers.Suivi(https=False)
YEAR = time.localtime()[0]
for semester in configuration.semesters:
    configuration.suivi.add(
        YEAR, semester,
        # The user visible URL for the 'suivi' server
        socket.gethostbyname(socket.gethostname()) + ':8889',
        8889)
