#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2008 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

import json
from .. import utilities
from .. import plugin
from .. import document
from .. import configuration

def cell_change(server, comment=False):
    err = document.get_cell_from_table(server)
    if isinstance(err, str):
        return 'bad.png'
    table, page, column, lin = err
    value = server.the_path[2]
    try:
        if column.type.name == 'MCQ' or column.type.name == 'Notation': # XXX really not clean
            if not comment and value != configuration.no:
                # The only possible value is NO
                return 'bad.png'
            if column.type.name == 'Notation':
                answers = json.loads(value)
                for v in answers.values():
                    if not v.startswith('?/'):
                        return 'bad.png'
        else:
            if comment:
                # Comment change is only allowed for MCQ and Notation column
                return 'bad.png'
        table.lock()
        try:
            try:
                if comment:
                    return table.comment_change(page, column.the_id, lin, value)
                else:
                    return table.cell_change(page, column.the_id, lin, value)
            except ValueError: # pragma: no cover
                return 'bad.png'
        finally:
            table.unlock()
    finally:
        table.do_not_unload_remove('cell_change')

def cell(server, comment=False):
    """Modify an unique cell in a table. It is used by the 'suivi' page"""

    r = cell_change(server, comment)

    if r == 'bad.png':
        server.the_file.write(server._("ERROR_write_fail"))
    elif r == 'ok.png':
        server.the_file.write('OK!')
    else:
        server.the_file.write(server._("TIP_violet_square")) # pragma: no cover


plugin.Plugin('cell', '/{Y}/{S}/{U}/cell/{*}', function=cell,
              priority = -10 # Before student_redirection
              )

plugin.Plugin('comment', '/{Y}/{S}/{U}/comment/{*}',
              function=lambda server: cell(server, comment=True),
              documentation = "Change the cell comment",
              priority = -10 # Before student_redirection
              )
