#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2008-2014 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import os
import time
import collections
import gc
import html
import math
import sys
from .. import objgraph
from .. import plugin
from .. import utilities
from .. import configuration
from .. import document
from .. import entity
from .. import plugins

def gc_top(server):
    "Display a clickable list of Python classes and their number of instance."
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
        server.the_file.write(server._("MSG_evaluate"))
        return
    
    server.the_file.write('<META NAME="ROBOTS" CONTENT="NOINDEX, NOFOLLOW">\n')
    server.the_file.write('<pre>')
    server.the_file.write('gc.garbage=%s\n' % gc.garbage)
    # leaking = objgraph.get_leaking_objects()
    # server.the_file.write('leaking=%s\n' % len(leaking))
    server.the_file.write('\n')
    gc.collect()
    for name, nr in objgraph.most_common_types(limit=100):
        server.the_file.write('%6d <a href="%s/=%s/type/%s">%s</a>\n' % (
            nr, utilities.StaticFile._url_, server.ticket.ticket, name, name))
    server.the_file.write('</pre>')
    # server.the_file.write('Leak :<pre>')
    # t = ['%s\n' % html.escape(repr(o)[:80])
    #      for o in leaking
    #      ]
    # t.sort()
    # server.the_file.write(''.join(t) + '</pre>')

def check(path, o):
    s = []
    first = path[0]
    rest = path[1:]
    for child in gc.get_referents(o):
        if type(child).__name__ == first:
            if len(path) == 1:
                s.append(child)
            else:
                s += check(rest, child) # pragma: no cover
    return s

last = None
    
def gc_type(server):
    """Display a clickable list of the instance of the classes specified"""
    global last
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
        server.the_file.write(server._("MSG_evaluate"))
        return
    gc.collect()
    objects = gc.get_objects()
    server.the_file.write('<h1> ? → ' + '→'.join(server.the_path) + '</h1>')
    server.the_file.write('<pre>')
    what = collections.defaultdict(dict)
    for i in objects:
        for j in check(server.the_path, i):
            what[type(i).__name__][id(j)] = j
    s = sorted(what, key=lambda x: len(what[x]))
    s.reverse()
    for k in s:
        server.the_file.write('%s(<a href="%s/=%s/type/%s">%s</a>)\n' % (
                k,
                utilities.StaticFile._url_,
                server.ticket.ticket,
                k + '/' + '/'.join(server.the_path),
                len(what[k])))
        if last: # pragma: no cover
            for i in what[k]:
                if i in last:
                    continue
                server.the_file.write('        <b><a href="%s/=%s/object/%s">%s</a></b>\n' % (
                        utilities.StaticFile._url_,
                        server.ticket.ticket,
                        i,
                        html.escape(repr(what[k][i])[:100])))

        for n, i in enumerate(what[k]):
            server.the_file.write('        <a href="%s/=%s/object/%s">%s</a>\n' % (
                utilities.StaticFile._url_,
                server.ticket.ticket,
                i,
                html.escape(repr(what[k][i])[:100])))
            if n == 50:
                break
        server.the_file.write('\n')
                    
    server.the_file.write('</pre>')
    last = set(id(i) for i in objects)

def gc_object(server):
    """Display the graph of instances using or used by the object"""
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
        server.the_file.write(server._("MSG_evaluate"))
        return
    i = objgraph.at(int(server.the_path[0], 0))
    gc.collect()
    filename = os.path.join('TMP', 'objects.png')
    objgraph.show_backrefs(i, max_depth=5, too_many=10, filename=filename)
    server.the_file.write(utilities.read_file(filename, "bytes"))

plugin.Plugin('gctop'   , '/gc'        , group='roots', function=gc_top,
              launch_thread=True,
              link=plugin.Link(where='debug', html_class='verysafe')
              )
plugin.Plugin('gctype'  , '/type/{*}'  , group='roots',
              function=gc_type,
              launch_thread=True,
              )
plugin.Plugin('gcobject', '/object/{*}', group='roots', function=gc_object,
              launch_thread=True,
              mimetype = 'image/png')

def histogram(values):
    if len(values) == 0:
        return ''
    bin_size = 300 # 5 minute
    value_max = max(values)
    nr_bins = value_max // bin_size
    counters = [0] * (nr_bins+1)
    for v in values:
        counters[v//bin_size] += 1

    while counters and counters[-1] == 0:
        del counters[-1] # pragma: no cover

    if len(counters) == 0:
        return '' # pragma: no cover

    counter_max = math.log(max(counters)+1)

    s = '<table width="100%" style="table-layout:fixed"><colgroup>'
    s += '<col with="*">' * len(counters) + '</colgroup>'
    s += '<tr style="vertical-align:bottom">'
    for v in counters:
        s += '<td><img src="../ok.png" style="width:100%%" height="%dpx">' % int(50*math.log(v+1)/counter_max)
    s += '</tr><tr>'
    i = 0
    for v in counters:
        s += '<td style="text-align:center">%d<br><small><small>%s' % (int(i), v)
        i += bin_size / 60.
    s += '</tr></table>'
    return s

def caches(server):
    """Display the caches statistics"""
    s = ['<title>' + server._("MSG_cache_title")
         + '</title><table border="1"><tr><th>' + server._("TH_cache_name")
         + '<th>' + server._("TH_cache_what")
         + '<th>' + server._("TH_cache_type")
         + '<th>' + server._("TH_cache_age")
         + '<th>' + server._("TH_cache_maxage")
         + '<th>' + server._("TH_cache_content")]
    now = int(time.time())
    for cache in utilities.caches:
        if  cache.the_type == 'add_a_cache0':
            nr_items = 1
            size = server._("MSG_cache_size") % sys.getsizeof(cache.cache[0])
            since = now - int(cache.cache[1])
        else:
            nr_items = len(cache.cache)
            size = ''
            sinces = list([now - int(c[1]) for c in cache.cache.values()])
            since = histogram(sinces)
        if cache.__doc__ is None:
            doc = '???'
        else:
            doc = cache.__doc__
        
        s.append('<tr><td>%s<br><span style="font-size:60%%">%s</span><td>%s<td>%s<td width="50%%">%s<td>%s<td>%s items %s' % (
            cache.fct.__name__.replace('_',' '),
            cache.fct.__module__.replace('.', ' '),
            html.escape(doc),
            cache.the_type.replace('_',' '),
            since, cache.timeout, nr_items, size))
    s.append('</table>')
    server.the_file.write('\n'.join(s))

plugin.Plugin('caches'   , '/caches'   , group='roots', function=caches,
              link=plugin.Link(where='debug', html_class='verysafe')
              )

def locks(server):
    """Displays all the lock states"""
    server.the_file.write('<title>' + server._("MSG_locks_title")
                          + '</title><pre>'+ utilities.lock_state() + '</pre>')
    for t in document.tables_values():
        if t.the_lock.locked():
            server.the_file.write('<br>LOCKED: ' + str(t) + '<br>') # pragma: no cover

plugin.Plugin('locks'   , '/locks'   , group='roots', function=locks,
              link=plugin.Link(where='debug', html_class='verysafe')
              )

def threads(server):
    """Displays the running thread"""
    server.the_file.write('<title>' + server._("MSG_threads_title")
                          + '</title>'
                          + '<pre>'
                          + html.escape(utilities.all_the_stacks())
                          + '</pre>')

plugin.Plugin('threads'   , '/threads'   , group='roots', function=threads,
              link=plugin.Link(where='debug', html_class='verysafe')
              )

def gcbig(server):
    "Display big Python objects"
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
        server.the_file.write(server._("MSG_evaluate"))
        return
    
    server.the_file.write('<META NAME="ROBOTS" CONTENT="NOINDEX, NOFOLLOW">\n')
    server.the_file.write('<pre>')
    gc.collect()
    sizes = [[] for dummy in range(32)]
    log2 = math.log(2)
    for o in gc.get_objects():
        try:
            sizes[ int(math.log(sys.getsizeof(o)+1)/log2) ].append(o)
        except: # pragma: no cover
            pass
    total = 0
    for n, objects in enumerate(sizes):
        size = sum(sys.getsizeof(o) for o in objects)
        total += size
        server.the_file.write('%d-%d: %d objects (%d bytes)\n' % (
                2**n, 2**(n+1)-1, len(objects), size))
        if len(objects) < 10:
            for o in objects:
                server.the_file.write('   <a target="_blank" href="object/%s">%s</a>\n' % (
                        server.ticket.url(str(id(o))),
                        html.escape(repr(o)[:200])))
        
        
    server.the_file.write('</pre>')
    server.the_file.write('Total: %d bytes\n' % total)
    


plugin.Plugin('gcbig'   , '/gcbig'   , group='roots', function=gcbig,
              launch_thread=True,
              link=plugin.Link(where='debug', html_class='verysafe')
              )

def memory_size_checker(server):
    """Display thread list each time the memory footprint change"""
    import resource
    mem = 0
    server.the_file.write('<pre>')
    i = 0
    while True:
        if i % 100 == 0:
            # Keep browser connection open
            server.the_file.write(' ')
            server.the_file.flush()
        i += 1
        time.sleep(0.01)
        current = resource.getrusage(resource.RUSAGE_SELF)[2]
        if current == mem:
            continue
        try:
            server.the_file.write(
                '\n\n%d → %d\n\n' % (mem, current)
                + '\n'.join(t.stack()
                          for t in utilities.thread_list
                      )
                + '\n'*2 + '<hr>\n'
            )
            server.the_file.flush()
        except: # pragma: no cover
            break
        mem = current

plugin.Plugin('memory_size_checker', '/memory_size_checker',
              group='roots', function=memory_size_checker, launch_thread=True,
              link=plugin.Link(where='debug', html_class='verysafe'),
              unsafe=False
              )

def atomic_checker(server):
    """Compute the size of atomic Python type : integer, string"""
    atomic_types = (str, int, int, float)
    gc.collect()
    objs = gc.get_objects()
    atomics = set()
    for o in objs:
        # Do not iterate over files...
        if isinstance(o, (dict, list, tuple, set)):
            for a in o:
                if isinstance(a, atomic_types):
                    atomics.add(a)
        if isinstance(o, dict):
            for a in o.values():
                if isinstance(a, atomic_types):
                    atomics.add(a)
    t = collections.defaultdict(lambda: [0,0])
    for a in atomics:
        stat = t[type(a)]
        stat[0] += 1
        stat[1] += sys.getsizeof(a)

    for a_type, stat in t.items():
        server.the_file.write("%d %s %d bytes<br>\n" %(stat[0],
                                                       html.escape(str(a_type)),
                                                       stat[1]))

plugin.Plugin('atomic_checker', '/atomic_checker',
              group='roots', function=atomic_checker, launch_thread=True,
              link=plugin.Link(where='debug', html_class='verysafe')
              )


def print_cycles(obj, done, path, server):
    """
    Give the list of objects to analyse
    """
    if id(obj) in done:
        return # Cycle yet found # pragma: no cover
    if id(obj) in path: # pragma: no cover
        texts = []
        in_the_cycle = False
        for an_id in path[1:]:
            texts.append(html.escape(str(type(objgraph.at(an_id)))) + str(an_id))
            texts.append(' ')
            if an_id == id(obj):
                texts[-2] = '<a target="_blank" href="%s">%s</a>' % (
                    server.ticket.url('object/' + str(an_id)), texts[-2])
                in_the_cycle = True
            if in_the_cycle:
                done.add(an_id)
        server.the_file.write(''.join(texts) + '<br>')
        return
    path.append(id(obj))
    if isinstance(obj, (list, tuple, set, frozenset)):
        for item in obj:
            print_cycles(item, done, path, server)
    elif isinstance(obj, dict):
        for key, value in obj.items():
            print_cycles(key, done, path, server)
            print_cycles(value, done, path, server)
    elif hasattr(obj, '__dict__'):
        for value in obj.__dict__.values():
            print_cycles(value, done, path, server)
    elif '.' not in obj.__class__.__name__:
        pass
    else: # pragma: no cover
        for attr in dir(obj):
            print_cycles(getattr(obj, attr, None), done, path, server)
    path.pop()

def find_cycles(server):
    """Display the memory cycles"""
    server.the_file.write("<h1>Search Garbage</h1>")
    gc.set_debug(gc.DEBUG_SAVEALL)
    gc.collect(2)
    check_objs = gc.garbage
    gc.set_debug(0)
    for i in check_objs:
        server.the_file.write(html.escape(str(type(i))) + '<br>\n')
    check_objs.sort(key=lambda i: -len(str(type(i))))
    server.the_file.write("<h1>Start searching cycles</h1>")
    print_cycles(check_objs, set(), [], server)
    server.the_file.write("<h1>Clean</h1>")
    check_objs.clear()
    gc.collect(2)
    server.the_file.write("<h1>The End</h1>")

plugin.Plugin('find_cycles', '/find_cycles',
              group='roots', function=find_cycles, launch_thread=True,
              link=plugin.Link(text="Find circular references",
                               help="For debugging",
                               where='debug',
                               html_class='verysafe')
              )

def find_garbage(server):
    """Display garbage"""
    server.the_file.write("<h1>Search Garbage</h1>")
    gc.set_debug(gc.DEBUG_SAVEALL)
    gc.collect(2)
    check_objs = gc.garbage
    gc.set_debug(0)
    nr_displayed = 0
    for i in check_objs: # pragma: no cover
        server.the_file.write('<b><a target="_blank" href="%s/=%s/object/%s">%s</a></b><br>\n' % (
            utilities.StaticFile._url_, # pylint: disable=protected-access
            server.ticket.ticket,
            id(i),
            html.escape(repr(i)[:100])))
        nr_displayed += 1
        if nr_displayed == 1000:
            break
    server.the_file.write("<h1>Press Escape to free objects</h1>")
    if configuration.real_regtest:
        return
    try: # pragma: no cover
        while True:
            server.the_file.write(".")
            server.the_file.flush()
            time.sleep(1)
    except BrokenPipeError: # pragma: no cover
        check_objs.clear()
        gc.garbage.clear()
        gc.collect(2)

plugin.Plugin('find_garbage', '/find_garbage',
              group='roots', function=find_garbage, launch_thread=True,
              link=plugin.Link(text="Find garbage",
                               help="For debugging",
                               where='debug',
                               html_class='verysafe')
              )



def users(server):
    """Display all the UsersItem in memory"""
    url = configuration.server_url + '/users/'
    if server.the_path:
        if server.the_path[0] == '_doc_':
            server.the_file.write(plugins.entity_doc(entity.Users))
            return
        if server.the_path[0] == '_cleanup_':
            for _ in configuration.Users.cleanup(unused=int(server.the_path[1])):
                server.the_file.write('* ')
            server.the_file.write('<p>THE END')
            return
        if server.the_path[0] == '_erase_':
            for identifier in tuple(configuration.Users.instances):
                if not configuration.read_only and identifier:
                    try:
                        os.unlink(os.path.join('TMP', 'ENTITY', identifier[:3], identifier))
                        server.the_file.write('* ')
                    except FileNotFoundError: # pragma: no cover
                        server.the_file.write('@ ')
                configuration.Users.instances.pop(identifier)
            server.the_file.write('<p>THE END')
            return
        clear = server.the_path[0] == '_clear_'
        update = server.the_path[0].lower() == '_update_'
        force_update = server.the_path[0] == '_UPDATE_'
        if update or clear:
            server.the_path.pop(0)
        todo = configuration.Users(server.the_path)
        if update:
            if force_update:
                # All attributes even without useless
                attrs_to_update = todo.get_attributes_name()
            else:
                attrs_to_update = set()
                for user in todo:
                    attrs_to_update.update(user.attributes_)
            todo.get(*attrs_to_update)
        if clear:
            for item in todo:
                item.pop()
        for item in todo:
            server.the_file.write(
                '''<h1>{} (<a href="{}">update</a>,
                <a href="{}">UPDATE</a>,
                <a target="_blank" href="{}">doc</a>,
                <a target="_blank" href="{}">clear</a>)
                </h1>\n{}\n'''.format(
                    item.ID_,
                    server.ticket.url(url + '_update_/{}'.format(utilities.safe(item.ID_))),
                    server.ticket.url(url + '_UPDATE_/{}'.format(utilities.safe(item.ID_))),
                    server.ticket.url(url + '_doc_'),
                    server.ticket.url(url + '_clear_/{}'.format(utilities.safe(item.ID_))),
                    item.backtrace_html()))
            table = ['<table>']
            for k, fct in item.__class__.__dict__.items():
                if k.startswith('_'):
                    continue
                if not hasattr(fct, '__code__'):
                    continue # pragma: no cover
                if fct.__code__.co_argcount != 1:
                    continue
                table.append('<tr><th>{}<td>{}</tr>'.format(k, html.escape(repr(fct(item)))))
            table.append('</table>')
            server.the_file.write('\n'.join(table))


    else:
        server.the_file.write('<p><a href="{}">Cleanup 5m</a>, <a href="{}">Cleanup 1m</a>, <a href="{}">Erase all</a>,<p>'.format(
            server.ticket.url(url + '_cleanup_/300'),
            server.ticket.url(url + '_cleanup_/60'),
            server.ticket.url(url + '_erase_'),
            ))
        for key in sorted(entity.Users.instances):
            server.the_file.write('<a target="_blank" href="{}">{}</a> '.format(
                server.ticket.url(url + utilities.safe(key)), key))


plugin.Plugin('users', '/users/{*}',
              group='roots', function=users, launch_thread=True,
              link=plugin.Link(text="All Users object",
                               help="For debugging",
                               url='/users',
                               where='debug',
                               html_class='verysafe')
              )
