#!/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2015 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

import json
import collections
from .. import plugin
from .. import document

def get_columns(server):
    """Return the column definition of the table"""
    t = document.table(server.the_year, server.the_semester, server.the_ue,
                       create=False)
    if t:
        grp_seq = collections.defaultdict(int)
        grp_col = t.columns.get_grp()
        seq_col = t.columns.get_seq()
        if grp_col and seq_col:
            if t.columns[seq_col].hidden:
                for line in t.lines.values():
                    if line[0].value:
                        grp_seq[line[grp_col].value] += 1
            else:
                for line in t.lines.values():
                    if line[0].value:
                        if line[seq_col].value:
                            grp_seq[line[seq_col].value + '\001' + line[grp_col].value] += 1
                        else:
                            grp_seq[line[grp_col].value] += 1
        server.the_file.write(t.columns.js(hide=False)
                              + 'the_columns(columns,%s,%s);'
                              % (json.dumps(grp_seq),
                                 int(t.columns[seq_col].hidden) if seq_col else 0))
    else:
        server.the_file.write('the_columns([], []);')

plugin.Plugin('get_columns', '/{Y}/{S}/{U}/get_columns',
              function=get_columns,
              group='staff',
              mimetype = 'application/x-javascript; charset=UTF-8',
              unsafe=False,
              launch_thread=True
              )
