/*
  TOMUSS: The Online Multi User Simple Spreadsheet
  Copyright (C) 2015-2022 Thierry EXCOFFIER, Universite Claude Bernard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Contact: Thierry.EXCOFFIER@univ-lyon1.fr
*/

function hide_rightclip() {
}

function parse_options(value) {
    var parsed = {};
    if (value) {
        var options = value.split(",");
        for (var i in options) {
            var option = options[i].split(":");
            parsed[option[0]] = option[1];
        }
    }
    return parsed;
}

function get_hash() {
    var h = location.href.split("#")[1] || "";
    try { h = decodeURIComponent(h); }
    catch (e) { }
    return parse_options(h);
}

function get_option(name, default_value) {
    try {
        return get_hash()[name]
            || (localStorage && parse_options(localStorage['homepage'])[name])
            || default_value;
    } catch (e) { return get_hash()[name] || default_value; };
}

function options_string(options) {
    var t = [];
    for (var i in options)
        t.push(i + ":" + options[i]);
    return t.join(',');
}

function set_option(name, value, only_session) {
    var h = get_hash();
    h[name] = value;
    window.location.hash = encodeURIComponent(options_string(h));

    if (localStorage && !only_session) {
        try {
            var h = parse_options(localStorage['homepage']);
            h[name] = value;
            localStorage['homepage'] = options_string(h);
        }
        catch (e) {
            alert('LocalStorage NOT WORKING: ASK HELP TO FIX YOUR BROWSER\n\n'
                + navigator.userAgent + ':\n\n' + e);
        }
    }
}


function DisplayHomeTitle(_node) {
    return 'TOMUSS';
}
DisplayHomeTitle.need_node = [];

var suivi = {};
function DisplayHomeSemesters(node) {
    var current = get_option("year_semester", '');
    if (current === '') {
        for (var i = node.data.length - 1; i >= 0; i--) {
            current = node.data[i];
            if (current[2] == year && current[3] == semester)
                break;
        }
        if (i == -1)
            current = year_semester(); // No matching suivi server
        else
            current = current[2] + '/' + current[3];
    }
    var s = [];
    for (var i in node.data) {
        var ys = node.data[i][2] + '/' + node.data[i][3];
        s.push("<option" + (ys == current ? ' selected' : '')
            + ">" + ys + '</option>');
        suivi[ys] = node.data[i][0];
    }
    s.reverse();
    return '<select id="s" onchange="'
        + 'set_option(\'year_semester\',this.value,true);'
        + 'display_update_real()">'
        + '<option value="2008/Test">' + _('2008/Test') + '</option>'
        + s.join('')
        + '</select>';
}

function year_semester() {
    var s = document.getElementById("s");
    if (s)
        return s.value;
    // The semester menu list is not yet on screen: take the last one
    var hs = display_data['HomeSemesters'];
    if (hs && hs[hs.length - 1]) {
        hs = hs[hs.length - 1];
        return hs[2] + '/' + hs[3];
    }
    return '2008/Test';
}

function current_year_semester() {
    var s = display_data["HomeSemesters"];
    if (s) {
        s = s[s.length - 1];
        return s[2] + '/' + s[3];
    }
    return '2008/Test';
}

function first_university_year_semester(ys) {
    if (!ys)
        ys = year_semester();
    ys = ys.split('/');
    var the_year = ys[0];
    var the_semester = ys[1];
    // Search first semester
    while (semesters_year[myindex(semesters, the_semester)] != 0) {
        ys = previous_year_semester(the_year, the_semester);
        if (the_year < 2000)
            return ['?', '?'];
        the_year = ys[0];
        the_semester = ys[1];
    }
    return [the_year, the_semester];
}

function url_ue_wo_ticket(code, more) {
    if (code.indexOf('/') == -1) {
        if (all_ues[code] && all_ues[code].login && code.indexOf("-") == -1)
            code = 'UE-' + code;
        code = year_semester() + '/' + code;
    }
    if (more) {
        if (more.indexOf('/') != 0)
            code += '/';
        code += more;
    }
    return code;
}

function url_ue(code, more) {
    return add_ticket(url_ue_wo_ticket(code, more));
}

function goto_url(path) {
    if (path === goto_url.last_url && (millisec() - goto_url.last_time) < 1000) {
        Alert("ALERT_no_double_clic");
        return;
    }
    goto_url.last_url = path;
    goto_url.last_time = millisec();

    window.open(path);
}

function goto_url_grp_seq(path) {
    goto_url(add_ticket(path + grp_seq_filter()));
}

function go(x) {
    goto_url(add_ticket(year_semester() + "/" + x));
}

function suivi_url(x) {
    var s = suivi[test_bool(preferences.current_suivi) == yes
        ? current_year_semester()
        : year_semester()
    ];
    if (s === undefined)
        s = suivi[current_year_semester()];
    return add_ticket(s, x);
}

function go_suivi(x) {
    window.open(suivi_url(x));
}

function open_ue(t, code) {
    while (!t.className.match(/\bue_line\b/))
        t = t.parentNode;
    if (t.childNodes[t.childNodes.length - 1].className.match('HomeUEMenu')) {
        t.onmouseleave();
        return;
    }
    popup_close();
    t.className += " ue_line_hover";
    var m = document.createElement("DIV");
    open_ue.code = code;
    m.innerHTML = display_display(display_definition["HomeUEMenu"]);
    m = m.firstChild;
    t.onmouseleave = function () {
        selected_grp_seq = {};
        try {
            t.removeChild(m);
            t.className = t.className.replace(" ue_line_hover", "");
        }
        catch (e) { };
        t.onmouseleave = "";
    };
    t.appendChild(m);
}

function box_button(txt, url, grp_seq) {
    if (grp_seq === undefined)
        grp_seq = '';
    return ['<div>' + txt.replace(/([-.@])/g, "$1 ") + '</div>', [], [],
    'onclick="goto_url' + grp_seq + "('" + url + '\')"'];
}

function box_button_grp_seq(txt, url) {
    return box_button(txt, url, '_grp_seq');
}

function DisplayHomeUEOpen(_node) {
    return box_button_grp_seq(_("B_home_edit"), url_ue_wo_ticket(open_ue.code));
}
DisplayHomeUEOpen.need_node = [];
function DisplayHomeUEOpenRO(_node) {
    return box_button_grp_seq(_("B_home_display"),
        url_ue_wo_ticket(open_ue.code, '=read-only='));
}
DisplayHomeUEOpenRO.need_node = [];
function DisplayHomeUESignature(_node) {
    return box_button_grp_seq(_("B_home_signature"),
        url_ue_wo_ticket(open_ue.code, '=signatures-page=/=read-only='));
}
DisplayHomeUESignature.need_node = [];
function DisplayHomeUEPrint(_node) {
    return box_button_grp_seq(_("B_home_export_print"),
        url_ue_wo_ticket(open_ue.code, '=print-table=/=read-only='));
}
DisplayHomeUEPrint.need_node = [];

function do_extension(code) {
    if (code.indexOf('/') == -1)
        code = first_university_year_semester().join('/') + '/' + code;
    create_popup('import_popup',
        _("ALERT_home_extend_before") + code + '<br>' + _("ALERT_home_extend_after"),
        '<iframe width="100%" src="'
        + add_ticket(code + '/extension') + '"></iframe>',
        '', false);
}
function undo_extension(code) {
    if (code.indexOf('/') == -1)
        code = first_university_year_semester().join('/') + '/' + code;
    create_popup('import_popup',
        _("ALERT_home_unextend_before") + code + '<br>' + _("ALERT_home_unextend_after"),
        '<iframe width="100%" src="'
        + add_ticket(code + '/undo_extension') + '"></iframe>',
        '', false);
}
function DisplayHomeUEUnsemestrialize(_node) {
    if (!open_ue.code || first_university_year_semester()[0] == '?')
        return ''
    var path = open_ue.code.split('/');
    if (path.length == 1 || myindex(semesters, path[1]) != -1)
        // No semester or a recognized semester
        return '<div onclick="do_extension(\'' + open_ue.code
            + '\')">' + _("B_home_unsemestrialize") + '</div>';
    return '';
}
DisplayHomeUEUnsemestrialize.need_node = [];

function DisplayHomeUESemestrialize(_node) {
    if (!open_ue.code || first_university_year_semester()[0] == '?')
        return ''
    var path = open_ue.code.split('/');
    if (path.length == 1 || myindex(semesters, path[1]) != -1)
        // No semester or a recognized semester
        return '<div onclick="undo_extension(\'' + open_ue.code
            + '\')">' + _("B_home_semestrialize") + '</div>';
    return '';
}
DisplayHomeUESemestrialize.need_node = [];

function get_grp_seq(all_if_none) {
    try {
        var grpseqs = [];
        var all = [];
        for (var i in selected_grp_seq) {
            var grpseq = i.split('\001');
            var seq = grpseq[0];
            var grp = grpseq[1];
            if (grpseq.length == 1) {
                grp = seq;
                if (hide_seq)
                    seq = undefined;
                else
                    seq = '';
            }
            if (selected_grp_seq[i])
                grpseqs.push([grp, seq]);
            all.push([grp, seq]);
        }
        if (all_if_none && grpseqs.length == 0)
            return all;
        return grpseqs;
    }
    catch (e) { return [] };
}

function grp_seq_filter() {
    var grpseqs = get_grp_seq();
    if (grpseqs.length === 0)
        return '';
    var filter = [];
    for (var i in grpseqs) {
        var grp = encode_uri_option('=' + grpseqs[i][0].replace(/ /g, '\\ '));
        if (grpseqs[i][1] !== undefined)
            filter.push(grp + ' [Seq]' + encode_uri_option('=' + grpseqs[i][1]));
        else
            filter.push(grp);
    }
    return '/=filters=0_3:' + filter.join('|') + '=';
}

function prst_column() {
    return selected_col.split(/--/)[1];
}
function prst_datacol() {
    return selected_col.split(/--/)[2];
}

function do_presence() {
    goto_url(url_ue(open_ue.code, grp_seq_filter() + '/=facebook=' + prst_column()));
}

function prst_qrcode() {
    goto_url(url_ue(open_ue.code, '/qrcode_new/' + prst_datacol()
        + '/grpseq/' + JSON.stringify(get_grp_seq())));
}

function prst_qrcode_etape_go() {
    var input = document.getElementById("prst_qrcode_etape").value.split(/[\/ :]+/);
    var d = new Date(input[2], input[1] - 1, input[0], input[3], input[4]);
    var now = new Date();
    if (d.getTime() > now.getTime() + 3600 * 1000) {
        alert(_("ALERT_invalid_value") + d);
        return;
    }
    var inputs = popup_get_element().getElementsByTagName('INPUT');
    var duration;
    for (var i in inputs)
        if (inputs[i].checked) {
            duration = inputs[i].value;
            break;
        }
    if (!duration) {
        alert(_("Teachers_miss_duration") + '1h 1h30...');
        return;
    }
    goto_url(url_ue(open_ue.code, '/qrcode_new/'
        + '-' + d.formate('%Y%m%d%H%M') + duration
        + '/grpseq/' + JSON.stringify(prst_qrcode_etape.grpseqs)));
    popup_close();
}

function prst_qrcode_etape() {
    var d = new Date();
    d.setMinutes(Math.round(d.getMinutes() / 15) * 15);
    var groups = get_grp_seq();
    var text = [];
    if (groups.length) {
        for (var i in groups) {
            if (groups[i][1] === undefined)
                groups[i][1] = '';
            text.push(groups[i][0] + groups[i][1]);
        }
    }
    else
        text.push(_("MSG_home_~any~"));
    prst_qrcode_etape.grpseqs = groups;
    create_popup('import_popup',
        '<style>'
        + '.import_popup LABEL { border: 1px solid #000; margin:0.3em }'
        + '.import_popup, BUTTON { font-size: 150% }'
        + '</style>'
        + _("MSG_prst_qrcode_etape").replace(/<br>/g, ' '),
        ' <br>' + open_ue.code + ' ' + html(get_info(open_ue.code).intitule || '?') + '<br><br>'
        + _("COL_TITLE_messages_group") + ' : ' + text.join('+') + '<br><br>'
        + _("COL_TITLE_when") + ' : <input id="prst_qrcode_etape" value="'
        + d.formate('%d/%m/%Y %H:%M') + '" style="font-size:100%"><br><br>'
        + _("P_new_slot_duration") + ' : '
        + '<label><input type="radio" name="duration" value="60"> 1h</label>'
        + '<label><input type="radio" name="duration" value="90"> 1h30</label>'
        + '<label><input type="radio" name="duration" value="120"> 2h</label>'
        + '<label><input type="radio" name="duration" value="150"> 2h30</label>'
        + '<label><input type="radio" name="duration" value="180"> 3h</label>'
        + '<label><input type="radio" name="duration" value="210"> 3h30</label>'
        + '<label><input type="radio" name="duration" value="240"> 4h</label>'
        + '<br><br><button onclick="prst_qrcode_etape_go()">GO !</button>',
        '', false);
}


function prst_print(one_per_grp) {
    var w = window_open(url + '/ok.png', true);
    var qrcode_id = millisec();
    var title = '<br>' + year_semester() + '/' + open_ue.code + '<br>'
        + _("TAB_column") + ': «'
        + selected_col.split('<b>')[1].split('</b>')[0] + '»';
    w.document.open('text/html');
    w.document.write(
        '<style>'
        + 'BODY { font-size: 200% }'
        + 'IMG { image-rendering: pixelated; width: calc(min(100vw, 100vh) - 4em); display: block }'
        + (one_per_grp
            ? '@media print { IMG { width: 100%; } }'
            : '@media print { IMG { width: calc(min(100vw, 100vh) / 2 - 2em); } }')
        + '</style>');
    if (one_per_grp) {
        // Group wth /
        var grp_seq = get_grp_seq(true);
        var qrcode_id = millisec();
        var equal = one_per_grp == 1 ? '==' : '=';
        var more;
        for (var i in grp_seq) {
            var title_grp = '<br>' + _('MSG_prst_print_per_grp' + (one_per_grp == 1 ? '' : '2'))
                + title + '<br>';
            if (grp_seq[i][1] === undefined)
                more = '';
            else {
                more = 'Seq' + equal + grp_seq[i][1] + ' ';
                title_grp += _("COL_COMMENT_0_4") + ': <b>' + grp_seq[i][1] + '</b><br>';
            }
            title_grp += _("COL_COMMENT_0_3") + ': <b>' + grp_seq[i][0] + '</b>';
            w.document.write(
                (i == 0 ? '<p>' : '<p style="page-break-before: always">')
                + '<b>' + _('Start') + '</b>' + title_grp
                + '<img src="'
                + url_ue(open_ue.code, '/get_pair/' + prst_datacol() + '/1/' + qrcode_id
                    + '/' + more + 'Grp' + equal + encode_uri_option(grp_seq[i][0].replace(/ /g, ' ')))
                + '"><p style="page-break-before: always">'
                + '<b>' + _('Stop') + '</b>' + title_grp
                + '<img src="'
                + url_ue(open_ue.code, '/get_pair/' + prst_datacol() + '/0/' + qrcode_id)
                + '">');
            qrcode_id++;
        }
    }
    else
        w.document.write(
            _('Start') + title
            + '<img src="'
            + url_ue(open_ue.code, '/get_pair/' + prst_datacol() + '/1/' + qrcode_id)
            + '"><br><br><br><br>'
            + _('Stop') + title
            + '<img src="'
            + url_ue(open_ue.code, '/get_pair/' + prst_datacol() + '/0/' + qrcode_id)
            + '">');
    w.document.close();
}

function update_type_css(col_type, event) {
    selected_type = col_type;
    var buttons = document.getElementById("TYPE_buttons");
    var children = buttons.children;
    for (var i = 0; i < children.length; i++)
        children[i].className =
            children[i].getAttribute('what') == col_type ? "selected_column" : "unselected_column";
    var cols = buttons.nextElementSibling.nextElementSibling.children;
    for (var i = 0; i < cols.length; i++)
        cols[i].style.display = cols[i].firstChild.getAttribute('what') == col_type
            ? 'inline-block' : 'none';

    function disable(button_id) {
        var button = document.getElementById(button_id);
        if (prst) {
            button.style.opacity = 1;
            button.style.pointerEvents = 'initial';
        }
        else {
            button.style.opacity = 0.3;
            button.style.pointerEvents = 'none';
        }

    }
    var prst = selected_col.indexOf('what="Prst"') != -1;
    disable("do_presence");
    disable("prst_qrcode");
    disable("prst_print");
    disable("prst_print_per_grp");
    disable("prst_print_per_grp2");

    stop_event(event);
}

function update_action_buttons() {
    update_type_css(selected_type);
}


function grp_seq_change(buttons) {
    for (var i = 0; i < buttons.childNodes.length; i++)
        if (buttons.childNodes[i].className == "button_toggle")
            return;
    for (var i = 0; i < buttons.childNodes.length; i++)
        buttons.childNodes[i].className = "button_toggle";
}

var table_attr = {};
var selected_grp_seq;
var selected_col;
var selected_type;
var hide_seq;
var columns_of_ue;
function the_columns(columns, grp_seq, hidden_seq) {
    hide_seq = hidden_seq;
    var options = [];
    var nb = 0;
    for (var i in grp_seq) {
        options.push(i + '<br>' + grp_seq[i]);
        nb += grp_seq[i];
    }
    options.sort();
    var types = {};
    // options.push(_('MSG_home_~any~') + '<br>' + nb);
    for (var i in columns)
        columns[i].data_col = i;
    columns.sort(function (a, b) { return a.position - b.position; });
    columns_of_ue = columns;
    var col_buttons = [];
    var last_prst = undefined;
    for (var i in columns) {
        types[columns[i].type] = true;
        var dates = columns[i].course_dates.split(/  */);
        var lines = [
            '<!--' + columns[i].the_id + '--' + columns[i].data_col + '-->'
            + '<b>' + html(columns[i].title) + '</b>'];
        var in_the_past = true;
        for (var j in dates) {
            if (dates[j] === '')
                in_the_past = false;
            else {
                if (dates[j].substr(0, 6) >= today)
                    in_the_past = false;
                lines.push(get_date_tomuss_short(dates[j]));
            }
        }
        if (columns[i].comment !== '')
            lines.push(html(columns[i].comment));
        col_buttons.push('<div class="Display'
            + (in_the_past ? ' in_the_past' : '')
            + '" what="' + columns[i].type
            + '">' + lines.join("<br>") + '</div>');
        if (columns[i].type == 'Prst')
            last_prst = col_buttons[col_buttons.length - 1];
    }
    var sorted_types = [];
    for (var i in types)
        sorted_types.push([_('B_' + i), i]);
    sorted_types.sort();
    var t = [];
    for (var i in sorted_types)
        t.push('<button onclick="update_type_css(\'' + sorted_types[i][1]
            + '\', event)" what="' + sorted_types[i][1] + '">'
            + (sorted_types[i][0].indexOf(' ') != -1
                ? sorted_types[i][0].replace(' ', '<br>')
                : sorted_types[i][0] + '<br> ') + '</button>');
    t = '<div id="TYPE_buttons">' + t.join('') + '</div>';
    var e = document.getElementById("the_columns");
    if (e) {
        selected_col = last_prst || col_buttons[col_buttons.length - 1];
        if (!selected_col) {
            e.innerHTML = '';
            return;
        }
        var toggles = '';
        selected_grp_seq = {};
        for (var i in options)
            selected_grp_seq[options[i].split('<')[0]] = false;
        // selected_grp_seq[_('MSG_home_~any~')] = true;
        for (var i in options) {
            toggles += toggle_button(options[i], 'selected_grp_seq',
                options[i].split('<')[0]);
        }
        toggles = toggles.replace(/"button_toggle"/g, '"button_toggle toggled"')
        e.firstChild.innerHTML = _("MSG_home_presence") + '<div onmousedown="grp_seq_change(this)">' + toggles + '</div>';
        e.childNodes[1].innerHTML = t + radio_buttons('selected_col', col_buttons,
            selected_col, 'update_action_buttons()');
        e.className = "some_columns";

        update_type_css(last_prst === undefined ? 'Text' : 'Prst');
    }
}

function prst_open(read_only) {
    if (read_only)
        read_only = '/=read-only=';
    else
        read_only = '';
    var selected = selected_col.split('<b>')[1].split('</b>')[0];
    for (var i in columns_of_ue)
        if (html(columns_of_ue[i].title) == selected) {
            var cols = ['=Seq | =Grp'];
            i = Number(i);
            for (var j = Math.max(0, i - 1); j < Math.min(i + 2, columns_of_ue.length); j++)
                cols.push('=' + encode_uri_option(columns_of_ue[j].title));
            goto_url_grp_seq(url_ue_wo_ticket(open_ue.code)
                + '/=columns_filter=' + cols.join(' | ')
                + (DisplayHomeColumns.large_screen ? '' : '/=bigfont=')
                + read_only
            );
            return;
        }
    goto_url_grp_seq(url_ue_wo_ticket(open_ue.code) + read_only);
}
function DisplayHomeUEMenuColumns(_node) {
    var s = document.createElement("SCRIPT");
    s.src = url_ue(open_ue.code, 'get_columns');
    the_body.appendChild(s);
    return '<div id="the_columns"><div class="grps">' + _("MSG_home_wait")
        + '</div><div class="select_columns"></div>'
        + '<div class="HomeUEMenuActions" style="padding-bottom:2em">'
        + '<div class="Display" id="do_presence" onclick="do_presence()"><div>'
        + _('MSG_prst_phone') + '</div></div> '
        + '<div class="Display" id="prst_qrcode" onclick="prst_qrcode()"><div>'
        + _('MSG_prst_qrcode') + '</div></div> '
        + '<div class="Display" id="prst_qrcode" onclick="prst_qrcode_etape()"><div>'
        + _('MSG_prst_qrcode_etape') + '</div></div> '
        + '<div class="Display" id="prst_print" onclick="prst_print()"><div>'
        + _('MSG_prst_print') + '</div></div> '
        + '<div class="Display" id="prst_print_per_grp" onclick="prst_print(1)"><div>'
        + _('MSG_prst_print_per_grp') + '</div></div> '
        + '<div class="Display" id="prst_print_per_grp2" onclick="prst_print(2)"><div>'
        + _('MSG_prst_print_per_grp2') + '</div></div> '
        + '<div class="Display" onclick="prst_open(0)" style="font-weight:bold"><div>'
        + _("MSG_prst_table") + '</div></div> '
        + '<div class="Display" onclick="prst_open(1)"><div>'
        + _("MSG_prst_table_ro") + '</div></div>'
        + '</div>';
}
DisplayHomeUEMenuColumns.need_node = [];


function do_close_ue(code) {
    create_popup('import_popup',
        _("B_home_close_pages"),
        '<iframe width="100%" src="'
        + url_ue(code, 'page_unload') + '"></iframe>',
        '', false);
}

function DisplayHomeUEClose(_node) {
    if (i_am_root)
        return '<div onclick="do_close_ue(\'' + open_ue.code + '\')">'
            + _("B_home_close_pages") + '</div>';
    return '';
}
DisplayHomeUEClose.need_node = [];

function DisplayHomeLogout(_node) {
    return '<a href="' + add_ticket('logout') + '">' + _("LABEL_logout") + '</a>';
}
DisplayHomeLogout.need_node = [];


function DisplayHomeLogin(_node) {
    return username;
}
DisplayHomeLogin.need_node = [];


function DisplayHomeFeed(_node) {
    return '<a target="_blank" href="' + url
        + '/news.xml"><img style="border:0px;" src="_FILES_/feed.png"></a>';
}
DisplayHomeFeed.need_node = [];


function home_help_popup() {
    if (popup_is_open()) {
        popup_close();
        return;
    }
    create_popup('top_right', 'TOMUSS <span class="copyright">'
        + tomuss_version + '</span>',
        _("MSG_home_welcome")
        + '<p>'
        + '<a href="_FILES_/doc_table.html" target="_blank">'
        + _("LABEL_documentation") + '</a>'
        + '<p>'
        + '<a href="mailto:' + admin + '">'
        + _("MSG_suivi_student_mail_link") + '</a>'
        , '', false);
}

function DisplayHomeHelp(_node) {
    return '<a href="javascript:home_help_popup()">' + _("TAB_?") + '</a>';
}
DisplayHomeHelp.need_node = [];

function language_change(t) {
    function change_language() {
        languages = [preferences.language];
        display_update_real();
    }

    preference_change(t, "language=" + preferences.language);
    if (translations[preferences.language] === undefined) {
        var t = document.createElement("SCRIPT");
        t.src = url_files + '/' + preferences.language + ".js"
        t.onload = change_language;
        the_body.appendChild(t);
    }
    else
        change_language();
}
function DisplayHomePreferencesLanguages(_node) {
    return show_preferences_language();
}
DisplayHomePreferencesLanguages.need_node = [];

function change_text_size() {
    set_option('text_size', preferences.text_size);
    the_body.style.fontSize = preferences.text_size;
}

function DisplayHomePreferencesSize(_node) {
    return radio_buttons('preferences.text_size', ['70%', '100%', '150%', '200%', '300%'],
        preferences.text_size,
        "change_text_size()")
        + _("MSG_text_size");
    ;
}
DisplayHomePreferencesSize.need_node = [];

function current_suivi_change(t) {
    preference_change(t, "current_suivi=" + (preferences.current_suivi == yes ? 1 : 0));
    display_update_real();
}
function DisplayHomePreferencesYearSemester(_node) {
    return radio_buttons('preferences.current_suivi', [no, yes],
        test_bool(preferences.current_suivi),
        "current_suivi_change(this)")
        + _("Preferences_current_suivi");
}
DisplayHomePreferencesYearSemester.need_node = [];

function home_3scrollbar_change(t) {
    preference_change(t, "home_3scrollbar="
        + (preferences.home_3scrollbar == yes ? 1 : 0));
    display_update_real();
}
function DisplayHomePreferences3ScrollBars(_node) {
    return '';
    /*
    return radio_buttons('preferences.home_3scrollbar', [no, yes],
                     test_bool(preferences.home_3scrollbar),
                         "home_3scrollbar_change(this)")
      + _("Preferences_home_3scrollbar") ;
    */
}
DisplayHomePreferences3ScrollBars.need_node = [];

function forget_input_change(t) {
    preference_change(t, "forget_input=" + (preferences.forget_input == yes ? 1 : 0));
}
function DisplayHomePreferencesForgetInput(_node) {
    return radio_buttons('preferences.forget_input', [no, yes],
        test_bool(preferences.forget_input),
        "forget_input_change(this)")
        + _("Preferences_forget_input");
}
DisplayHomePreferencesForgetInput.need_node = [];

function compressed_change(t) {
    preference_change(t, "compressed=" + (preferences.compressed == yes ? 1 : 0));
}

function DisplayHomePreferencesCompressed(_node) {
    return radio_buttons('preferences.compressed', [no, yes],
        test_bool(preferences.compressed),
        "compressed_change(this)")
        + _("Preferences_compressed");
}
DisplayHomePreferencesCompressed.need_node = [];

function current_debug_change(t) {
    preference_change(t, "debug_home=" + (preferences.debug_home == yes ? 1 : 0));
    display_update_real();
}
function DisplayHomePreferencesDebug(_node) {
    if (!i_am_root)
        return '';
    return radio_buttons('preferences.debug_home', [no, yes],
        test_bool(preferences.debug_home),
        "current_debug_change(this)")
        + _("Preference_debug_suivi");
}
DisplayHomePreferencesDebug.need_node = [];

function theme_change(theme, t) {
    preference_change(t, "theme=" + theme);
    popup_close();
    home_preferences_popup();
}

DisplayHomePreferencesThemes.need_node = [];

function home_preferences_popup() {
    if (popup_is_open()) {
        popup_close();
        return;
    }

    var s = [_('MSG_home_preferences'),
        '<div class="Preferences">'];
    var children = display_definition['HomePreferences'].children;
    for (var i in children)
        s.push(display_display(children[i]));
    s.push('</div>');

    create_popup('top_right', 'TOMUSS <span class="copyright">'
        + tomuss_version + '</span>',
        s.join(''), '', false);
}

var preferences;

function DisplayHomePreferences(node) {
    display_do_debug = node.data['debug_home'];
    preferences = node.data;
    languages = [preferences.language];
    if (!preferences.text_size)
        preferences.text_size = get_option('text_size', '100%');
    the_body.style.fontSize = preferences.text_size;
    search_ue_change.last_value = (preferences.forget_input
        ? ''
        : get_option('ue', '')
    );

    return '<a href="javascript:home_preferences_popup()">≡</a>';
}

// The message may start with some 3 hexdigit colors:
//    #backgroundColor #ForegroundColor #BorderColor
// #F00 #FFF to have white on a red background
function DisplayHomeMessage(node) {
    if (node.data !== '') {
        var d = node.data.match(RegExp("((#[0-9A-F]{3}  *)*)(.*)"));
        var cols = d[1] ? d[1].split(/  */) : [];
        return [d[3], [], ["background:" + (cols[0] || "#FFF"),
        "color:" + (cols[1] || "#000"),
        "border: 4px solid " + (cols[2] || "#000")
        ], ''];
    }
    return '';
}

function em_size_in_pixel() {
    var o = document.createElement('DIV');
    o.style.width = '1em';
    the_body.appendChild(o);
    var w = o.offsetWidth;
    the_body.removeChild(o);
    return w;
}

function DisplayHomeColumns(node) {
    if (!DisplayHomeColumns.init_done) {
        DisplayHomeColumns.init_done = true;
        the_body.onresize = function () {
            if (!do_touchstart.touch) // No resize on tablet and phone
                display_update_real();
        };

        the_body.addEventListener("touchstart", do_touchstart, false);
    }

    display_do_debug = preferences.debug_home;
    i_am_root = myindex(root, username) != -1; // XXX yet in lib.js
    var s = ['<style>',
        '.HomeUE .ue_line_hover {width:', window_width() - 40, 'px ;}',
        '.HomeStudents .ue_line_hover{width:', window_width() / 2 - 20, 'px ;}',
        'DIV.tabs .ue_line_hover {width:', window_width() - 30, 'px ;}',
        'DIV.tabs .ue_title, DIV.tabs .ue_more { width:', window_width() - 165, 'px ;}',
        '.ue_title, .ue_more {width:', window_width() - 170, 'px;}',
        '.HomeStudents .ue_title, .HomeStudents .ue_more {width:',
        window_width() / 2 - 120, 'px;}',
        '</style>',
        '<title>', _("MSG_home_title"), '</title>'
    ];
    DisplayHomeColumns.large_screen = 70 * em_size_in_pixel() < window_width();
    if (DisplayHomeColumns.large_screen) {
        s.push('<table><tbody><tr><td width="50%">' + _("TH_home_ue")
            + '<td width="25%">' + _("TH_home_students")
            + '<td width="25%">' + _('TH_home_right') + '</tr><tr>'
        );
        for (var i in node.children) {
            s.push("<td>");
            s.push(display_display(node.children[i]));
        }
        s.push("</tbody></table>");
    }
    else {
        s.push(create_tabs('home3', [
            [_("TH_home_ue"), display_display(node.children[0])],
            [_("TH_home_students"), display_display(node.children[1])],
            [_("TH_home_right"), display_display(node.children[2])]
        ]));
        DisplayHomeColumns.tab = selected_tab('home3') || DisplayHomeColumns.tab;
        if (!DisplayHomeColumns.tab)
            DisplayHomeColumns.tab = _("TH_home_ue");
        setTimeout("select_tab('home3'," + js(DisplayHomeColumns.tab) + ")", 0);
    }
    return s.join('');
}
DisplayHomeColumns.need_node = [];

function _UE(name, responsable, intitule, parcours, code, login,
    nr_students_ue, nr_students_ec, planning, tt, credit, old_names) {
    this.name = name;
    if (parcours === undefined) {
        parcours = '';
        code = '';
        nr_students_ue = 1;
        nr_students_ec = 0;
    }
    this.responsable = responsable;
    this.intitule = intitule;
    this.parcours = parcours;
    this.code_ade = code;
    this.login = login;
    this.nr_students_ue = nr_students_ue;
    this.nr_students_ec = nr_students_ec;
    if (tt === undefined)
        tt = 0;
    this.tt = tt;
    this.credit = credit;
    this.old_names = old_names;
    this.planning = planning;
}

function UE(name, responsable, intitule, parcours, code, login,
    nr_students_ue, nr_students_ec, planning, tt, credit, old_names) {
    return new _UE(name, responsable, intitule, parcours,
        code, login, nr_students_ue, nr_students_ec, planning, tt,
        credit, old_names);
}

function get_info(code) {
    var info;
    try { info = all_ues[code]; }
    catch (e) { }
    if (!info)
        info = all_ues[code.split('-')[1]];
    if (!info)
        info = { name: code };
    return info;
}

function fast_tip(txt, help) {
    return '<div class="fast_tip">' + txt + '<div>' + help + '</div></div>';
}

function code_clean_up(code) {
    return code.replace(/<!--/g, "").replace(/-->/g, "").replace(/<.?small>/g, "");
}

function code_create(year_semester_ue) {
    if (!year_semester_ue[3])
        year_semester_ue.push(year_semester_ue.join('/'));
    return year_semester_ue[3];
}

function ue_set_favorite(event, code) {
    event = the_event(event);
    var t = event.target;
    var img = document.createElement('IMG');
    var bookmarked = get_info(code).is_bookmarked;
    var code_ue = code_clean_up(code);
    if (bookmarked)
        code_ue = bookmarked;
    else if (code_ue.indexOf('/') == -1)
        code_ue = year_semester() + '/' + code_ue;
    img.src = add_ticket(code_ue + '/bookmark');
    t.appendChild(img);

    if (bookmarked) {
        get_info(code).is_bookmarked = false;
        for (var i in display_data['HomeUEBookmarked'])
            if (display_data['HomeUEBookmarked'][i][3] == bookmarked) {
                display_data['HomeUEBookmarked'].splice(Number(i), 1);
                break;
            }
    }
    else {
        display_data['HomeUEBookmarked'].push(code_ue.split('/'));
    }
    create_ue_lists.done = false;
    create_ue_lists();
    display_update_real();
    stop_event(event);
}

function student_set_favorite(event, code, favorite) {
    favorite = !favorite;
    event = the_event(event);
    var t = event.target;
    var img = document.createElement('IMG');
    img.src = add_ticket('favorite_student/' + code);
    t.appendChild(img);
    if (!favorite) {
        for (var i in display_data['HomeStudentFavorites'])
            if (display_data['HomeStudentFavorites'][i][0] == code) {
                display_data['HomeStudentFavorites'].splice(Number(i), 1);
                break;
            }
        is_a_favorite[code] = false;
    }
    else {
        display_data['HomeStudentFavorites'].push(students_info[code]);
    }
    display_update_real();
    stop_event(event);
}

function action_set_favorite(code, favorite) {
    is_a_favorite_action[code] = !favorite;
    try {
        localStorage["favorite_actions"] = JSON.stringify(is_a_favorite_action);
    } catch (e) { };
    display_update_real();
}

function get_ue_table(t) {
    while (!t.firstChild || !t.firstChild.getAttribute
        || !t.firstChild.getAttribute("alt"))
        t = t.parentNode;
    return t;
}


function get_ue_table_title(t) {
    return get_ue_table(t).firstChild.getAttribute("alt");
}

function select_ue_sort(t) {
    set_option('S' + get_ue_table_title(t), t.value);
    display_update_real();
}

function select_code_filter(t) {
    set_option('F' + get_ue_table_title(t), t.value, true);
    display_update_real();
}

function select_open_close(t, event) {
    set_option('B' + get_ue_table_title(t),
        t.innerHTML == icon_close ? icon_open : icon_close);
    var title = t.parentNode.parentNode.parentNode;
    title.className = title.className.replace(/ is_(open|close)/, '');
    if (t.innerHTML == icon_close) {
        title.className += " is_close";
        t.innerHTML = icon_open;
        t.nextSibling.innerHTML = _("TIP_open");
        t = get_ue_table(t);
        while (t.childNodes[1])
            t.removeChild(t.childNodes[1]);
    }
    else {
        title.className += " is_close";
        if (get_ue_table(t).parentNode.id) {
            update_job.todo[get_ue_table(t).parentNode.id] = true;
            periodic_work_add(update_job);
        }
        else
            display_update_real();
    }
    stop_event(event);
}


var ue_sorters = {
    'A': function (a, b) { return a > b ? 1 : (a == b ? 0 : -1); },
    'α': function (a, b) {
        a = get_info(a).intitule || '';
        b = get_info(b).intitule || '';
        return a > b ? 1 : (a == b ? 0 : -1);
    },
    '☆': function (a, b) {
        a = get_info(a).nr_access || 0;
        b = get_info(b).nr_access || 0;
        return b - a;
    },
    '★': function (a, b) {
        return !get_info(a).is_bookmarked - !get_info(b).is_bookmarked;
    },
    '#': function (a, b) {
        a = get_info(a).nr_students_ue || 0;
        b = get_info(b).nr_students_ue || 0;
        return b - a;
    },
    'surname+firstname': function (a, b) {
        a = a[2] + ' ' + a[1];
        b = b[2] + ' ' + b[1];
        return a > b ? 1 : (a == b ? 0 : -1);
    }
};

var icon_open = '▶';
var icon_close = '▼';

function search_infos(node_name, id) {
    var table = display_data[node_name];
    id = the_login(id);
    for (var i in table)
        if (the_login(table[i][0]) == id)
            return table[i];
}

function update_note(id, value) {
    var where = ['HomeStudentStudents', 'HomeStudentTeacher',
        'HomeStudentFavorites', 'HomeStudentRefered'];
    for (var i in where) {
        var infos = search_infos(where[i], id);
        if (infos) {
            if (infos[8] === undefined)
                infos[8] = {};
            infos[8]['note'] = value;
        }
    }
}

function add_note(element) {
    var ue_line = element.parentNode.parentNode.parentNode;
    ue_line.className += ' with_note';
    element.nextSibling.focus();
    update_note(ue_line.getAttribute('ondblclick').split("'")[1], ' ');
}

function record_note(element) {
    var ue_line = element.parentNode.parentNode.parentNode;
    var id = ue_line.getAttribute('ondblclick').split("'")[1];
    element.style.background = 'url(' + add_ticket('record_note/' + id + '/' + encode_uri(element.value)) + ') #F00';
    update_note(id, element.value);
}

function display_ues(title, tip, codes, options) {
    var order = get_option('S' + title, options.default_order);
    var opened = get_option('B' + title, icon_close) != icon_open;
    var s = ['<div class="ue_list border">'];
    var length_index;
    if (options.hide_open_close)
        opened = true;
    if (!options.hide_title) {
        s.push('<div class="table_title ' + (opened ? 'is_open' : 'is_close')
            + '" alt=' + js2(title)
            + (options.hide_sort
                && !options.code_filter
                && !options.hide_open_close
                ? ' onclick="select_open_close(this.getElementsByTagName(\'SPAN\')[0],event)"'
                : ''
            )
            + '>'
            + (options.before_title || '')
            + (tip ? hidden_txt(_(title), tip) : _(title)));
        if (codes.length) {
            s.push(' <small class="ue_icons" style="font-size:80%">(');
            length_index = s.length; // Updated at the end
            s.push(codes.length);
            s.push(')</small>');
        }
        if (display_data['HomePreferences']
            && display_data['HomePreferences']['debug_home'])
            s.push('<small>' + display_ues.nb++ + '</small>');
        if (options.title_second_line)
            s.push("<br>" + options.title_second_line);
        if (!options.hide_open_close)
            s.push('<div class="open_close ue_icons">'
                + fast_tip('<span onclick="select_open_close(this,event)">'
                    + (opened ? icon_close : icon_open)
                    + '</span>',
                    opened ? _("TIP_close") : _("TIP_open")
                )
                + '</div>'
            );
        s.push('<div class="select_ue_sort">');
        var current;
        if (options.code_filter) {
            var t = [];
            for (var i in options.code_filter)
                t.push(i);
            t.sort();

            var date = new Date();
            current = t[t.length - 1];
            if (!isNaN(current) && current > date.getFullYear())
                current = date.getFullYear().toString();

            current = get_option('F' + title, current);
            s.push('<select onchange="select_code_filter(this)">');
            for (var i in t)
                s.push('<option' + (t[i] == current ? ' selected' : '')
                    + ' value="' + t[i] + '">'
                    + (_('MSG_home_' + t[i]) == 'MSG_home_' + t[i]
                        ? t[i] : _('MSG_home_' + t[i]))
                    + '</option>');
            s.push('</select>');
        }
        else if (!options.hide_sort) {
            s.push('<select onchange="select_ue_sort(this)">');
            for (var i in ue_sorters)
                if (i.length == 1)
                    s.push('<option' + (i == order ? ' selected' : '')
                        + ' value="' + i + '">'
                        + _('sort_' + i) + '</option>');
            s.push("</select>");
        }
        s.push('</div></div>');
    }
    var favorite;
    if (!opened)
        codes = [];
    if (!options.do_not_sort)
        codes.sort(ue_sorters[order]);
    var nr_displayed = 0, classes;
    var display_student_icon = options.students && (options.code_filter
        || codes.length < 40);
    var nr_max = get_option('C' + title, options.nr_max);
    if (codes.length == 0
        && opened
        && options.students
        && !options.hide_open_close
        && !update_students_real.pending
        && document.getElementById('students_list')
        && document.getElementById('students_list').value.length > 1
    )
        s.push(_("MSG_home_nothing"));
    for (var ue in codes) {
        if (nr_displayed == nr_max) {
            s.push('<b onclick="set_option('
                + js2('C' + title) + ',' + nr_max * 4
                + ',true);display_update_real()">'
                + _("MSG_see_following") + '</b>');
            break;
        }
        ue = codes[ue];
        var info = undefined;
        if (options.students) {
            classes = ' ' + (ue[4] ? 'first_registration' : '') + ' ' + ue[6]
                + ' ' + (ue[5] ? 'tt' : '') + ' ';
            if (current && current != '~any~' && classes.indexOf(' ' + current + ' ') == -1)
                continue
            if (is_a_favorite[ue[0]])
                favorite = '★';
            else
                favorite = '☆';
            favorite = '<tt onclick="student_set_favorite(event,' + js2(ue[0]) + ','
                + is_a_favorite[ue[0]] + ')" class="icon">' + favorite + '</tt>';
        }
        else if (options.actions) {
            if (options.favorite_toggle) {
                if (is_a_favorite_action[ue[3]])
                    favorite = '★';
                else
                    favorite = '☆';
                favorite = '<tt onclick="action_set_favorite(' + js2(ue[3])
                    + ',' + is_a_favorite_action[ue[3]] + ')" class="icon">'
                    + favorite + '</tt>';
            }
            else
                favorite = '';
        }
        else {
            if (current && code_clean_up(ue).substr(0, current.length) != current)
                continue;
            info = get_info(ue);
            if (info.is_bookmarked)
                favorite = '★';
            else
                favorite = '☆';
            if (info.is_master_of && favorite == '☆')
                favorite = ''; // ♚
            else
                favorite = '<tt onclick="ue_set_favorite(event,' + js2(ue)
                    + ')" class="icon">' + favorite + '</tt>';
        }
        var favorite_tipped =
            fast_tip(favorite,
                (myindex(favorite, '★') != -1
                    ? _("B_home_remove_bookmark")
                    : _("B_home_bookmark"))
                + (info && info.nr_access
                    ? '<br>' + _("MSG_home_nr_view_before")
                    + info.nr_access
                    + _("MSG_home_nr_view_after")
                    : ''));
        if (options.students)
            s.push(
                '<div class="ue_line border ' + ((ue[8] && ue[8]['note'] || '') == '' ? '' : 'with_note ')
                + classes + '" onclick="open_student(this,'
                + js2(ue[0]) + ')" ondblclick="go_suivi(' + js2(ue[0]) + ')">'
                + '<div class="ue_right"><div class="ue_title">'
                + string_highlight(html(ue[2] + ' ' + title_case(ue[1])),
                    ask_login_list)
                + ' <span class="edit" onclick="add_note(this)">✎</span>'
                + '<input value="' + encode_value(ue[8] && ue[8]['note'] || '') + '" onchange="record_note(this)">'
                + '</div>'
                + '<div class="ue_more">'
                + (students_info[ue[0]][7]
                    ? fast_tip(students_info[ue[0]][7], _("MSG_suivi_referent_is"))
                    : ' ')
                + '</div>'
                + '</div><div class="ue_code">'
                + string_highlight(ue[0], ask_login_list)
                + '</div><div class="ue_icons">'
                + favorite_tipped
                + ' ' + (display_student_icon && suivi[year_semester()]
                    ? fast_tip('<img src="' + add_ticket(suivi[year_semester()], '_' + ue[0])
                        + '">',
                        _("TIP_home_squares"))
                    : '')
                + ' ' + (ue[5]
                    ? fast_tip('<tt class="icon">♿</tt>',
                        _("MSG_print_tt_title"))
                    : '')
                + '</div></div>'
            );
        else if (options.actions) {
            var el, eld, help, link = ue;
            var el, eld, help;
            if (link[4] === '' || link[3].toUpperCase().indexOf('INPUT') != -1) {
                el = '';
                eld = '';
            }
            else {
                el = '<a class="' + link[2] + '" href="' + link[4]
                    + '" target="' + link[5] + '">';
                eld = '</a>';
            }
            help = link[6];
            if (display_do_debug) {
                help += '<br>PLUGIN:' + link[7]
                    + '<br>FILE:' + link[8]
                    + '<br>PRIORITY:' + link[1];
            }
            s.push('<div style="width:100%">'
                + favorite_tipped
                + '<img class="safety" src="_FILES_/' + link[2] + '.png">'
                + el
                + hidden_txt(link[3], help)
                + eld
                + '</div>'
            );
        }
        else {
            var code = string_highlight(ue, search_ue_list.txt);
            if (code.match('/'))
                code = code.replace(RegExp("([^<])(/[^/]*)$"),
                    '$1<span class="path_code">$2</span>');
            s.push('<div class="ue_line" onclick="open_ue(this,' + js2(ue)
                + ')" ondblclick="goto_url_grp_seq(' + js2(url_ue_wo_ticket(ue)) + ')">'
                + '<div class="ue_right"><div class="ue_title">'
                + (info.nr_students_ue ? '<b>' : '')
                + string_highlight(html(info.intitule || ''),
                    search_ue_list.txt)
                + (info.credit > 0 ? ' <span class="ue_credits">'
                    + info.credit + ' ' + _("MSG_home_credit") + '</span>' : '')
                + (info.nr_students_ue ? '</b>' : '')
                + '</div><div class="ue_more">'
                + string_highlight(html(info.responsable
                    ? info.responsable.join(', ')
                    : ''), search_ue_list.txt) + ' '
                + (info.old_names
                    ? '<span>' + fast_tip(string_highlight(html(info.old_names.join(' ')),
                        search_ue_list.txt),
                        _("MSG_home_old_codes")) + '</span>'
                    : '')
                + '</div></div><div class="ue_code">'
                + code
                + '</div><div class="ue_icons">'
                + favorite_tipped
                + ' ' + (info.tt
                    ? fast_tip('<tt class="icon">♿</tt>',
                        info.tt + ' ' + _("MSG_print_tt_title"))
                    : '')
                + ' ' + (info.nr_students_ue
                    ? ' ' + fast_tip(info.nr_students_ue,
                        info.nr_students_ue
                        + ' ' + _("TH_home_students")
                    )
                    : '')
                + '</div></div>'
            );
        }
        nr_displayed++;
    }
    if (length_index !== undefined && nr_displayed != s[length_index])
        s[length_index] = nr_displayed + '/' + s[length_index];
    s.push("</div>");
    return s.join('');
}
display_ues.nb = 0;

// Merge all information from Favorites, Bookmark and MasterOf
// into 'all_ues' and create the lists to display.
// create_ue_lists.all_ues[ue] contains the original UE list
function create_ue_lists() {
    if (create_ue_lists.done
        && create_ue_lists.favorite_list.length != 0)
        return;
    create_ue_lists.favorite_list = [];
    create_ue_lists.master_of_list = [];
    create_ue_lists.acceded_list = [];
    create_ue_lists.unsaved_list = [];
    create_ue_lists.max = 0;
    try { all_ues[0]; }
    catch (e) {
        setTimeout(create_ue_lists, 100);
        return;
    }
    if (!create_ue_lists.yet_done) {
        create_ue_lists.all_ues = {};
        for (var ue in all_ues)
            create_ue_lists.all_ues[ue] = all_ues[ue];
    }

    var ue_first = [];
    for (var code in display_data['HomeUENrAccess'])
        if (code.substr(0, 3) == 'UE-')
            ue_first.push(code);
    for (var code in display_data['HomeUENrAccess'])
        if (code.substr(0, 3) != 'UE-')
            ue_first.push(code);
    var is_bookmarked = {};
    for (var code in display_data['HomeUEBookmarked']) {
        code = display_data['HomeUEBookmarked'][code][2];
        is_bookmarked[code] = true;
        if (get_info(code).intitule)
            create_ue_lists.favorite_list.push(code);
    }

    for (var code in ue_first) {
        code = ue_first[code];
        if (code == 'undefined')
            continue;
        var nr = display_data['HomeUENrAccess'][code];
        create_ue_lists.acceded_list.push(code);
        create_ue_lists.max = Math.max(create_ue_lists.max, nr);

        if (!create_ue_lists.yet_done) {
            all_ues[code] = get_info(code);
            if (all_ues[code].nr_access === undefined)
                all_ues[code].nr_access = 0;
            all_ues[code].nr_access += nr;
        }
    }
    create_ue_lists.max = Math.round(create_ue_lists.max);

    function add_to(year_semester_ue, list, force_add) {
        var code = code_create(year_semester_ue);
        if (get_info(year_semester_ue[2]).nr_students_ue !== undefined) {
            if (force_add)
                list.push(code);
            return {};
        }
        var ue = all_ues[code] = get_info(code);
        ue.intitule = year_semester_ue[2];
        list.push(code);
        return ue;
    }

    for (var i in display_data['HomeUEBookmarked']) {
        var year_semester_ue = display_data['HomeUEBookmarked'][i];
        add_to(year_semester_ue, create_ue_lists.master_of_list);
        get_info(year_semester_ue[2]).is_bookmarked = year_semester_ue[3];
        get_info(year_semester_ue[3]).is_bookmarked = year_semester_ue[3];
    }

    for (var i in display_data['HomeUEMasterOf']) {
        var ue = add_to(display_data['HomeUEMasterOf'][i],
            create_ue_lists.master_of_list);
        ue.is_master_of = true;
        if (ue.is_bookmarked)
            create_ue_lists.master_of_list.pop();
    }

    create_ue_lists.ues_teacher = [];
    for (var ue in create_ue_lists.all_ues) {
        if (myindex(all_ues[ue].login, username) == -1)
            continue;
        if (ue.match(/-/)) // ue.substr(0, 6) == 'etape-' )
            create_ue_lists.ues_teacher.push(ue);
        else
            create_ue_lists.ues_teacher.push('UE-' + ue);
    }

    // List of unsaved tables
    try {
        var index = localStorage && localStorage['index'];
        if (index && index !== '') {
            index = index.substr(1).split('\n');
            for (var i in index) {
                add_to(index[i].substr(1).split('/'),
                    create_ue_lists.unsaved_list, true);
            }
        }
    }
    catch (e) { };

    create_ue_lists.done = true;
    create_ue_lists.yet_done = true;
}

function storageEventHandler(_event) {
    create_ue_lists.done = false;
    update_job.todo['HomeUEUnsaved'] = true;
    periodic_work_add(update_job);
}

function DisplayHomeUEUnsaved(_node) {
    if (window.localStorage && !DisplayHomeUEUnsaved.done) {
        window.addEventListener('storage', storageEventHandler, false);
        DisplayHomeUEUnsaved.done = true;
    }
    create_ue_lists();
    if (create_ue_lists.unsaved_list.length == 0)
        return [' ', [], [], 'id="HomeUEUnsaved"'];
    return [display_ues("TH_home_unsaved_tables",
        _("TIP_home_unsaved_tables"),
        create_ue_lists.unsaved_list,
        {
            hide_open_close: true, hide_sort: true,
            default_order: "A"
        }),
    [], [], 'id="HomeUEUnsaved"'];
}
DisplayHomeUEUnsaved.need_node = ['HomeUEBookmarked', 'HomeUENrAccess',
    'HomeUEMasterOf', 'HomeSemesters',
    'HomePreferences'];

function DisplayHomeUENrAccess(_node) {
    create_ue_lists();
    if (create_ue_lists.favorite_list.length == 0)
        return '';
    return [display_ues("TH_home_bookmark_ue",
        _("TIP_home_bookmark_ue_before") + '★',
        create_ue_lists.favorite_list,
        { default_order: '★' }),
    [], [], 'id="HomeUENrAccess"'];
}
DisplayHomeUENrAccess.need_node = DisplayHomeUEUnsaved.need_node;

function DisplayHomeUEAcceded(_node) {
    create_ue_lists();
    return [display_ues("TH_home_acceded_ue", "", create_ue_lists.acceded_list,
        { default_order: '☆', nr_max: 5 }),
    [], [], 'id="HomeUEAcceded"'];
}
DisplayHomeUEAcceded.need_node = DisplayHomeUENrAccess.need_node;

function DisplayHomeUEBookmarked(_node) {
    create_ue_lists();
    if (create_ue_lists.master_of_list.length == 0)
        return '';
    years = {};
    for (var i in create_ue_lists.master_of_list)
        years[code_clean_up(create_ue_lists.master_of_list[i]).split('/')[0]] = true;
    return [display_ues("TH_home_master_of", _("TIP_home_master_of"),
        create_ue_lists.master_of_list,
        { default_order: 'A', code_filter: years }),
    [], [], 'id="HomeUEBookmarked"'];
}
DisplayHomeUEBookmarked.need_node = DisplayHomeUENrAccess.need_node;

function DisplayHomeUETeacher(_node) {
    create_ue_lists();
    if (create_ue_lists.ues_teacher.length == 0)
        return '';
    return [display_ues("TH_home_ue_master", _("TIP_home_ue_master"),
        create_ue_lists.ues_teacher,
        { default_order: 'A' }),
    [], [], 'id="HomeUETeacher"'];
}
DisplayHomeUETeacher.need_node = DisplayHomeUENrAccess.need_node;

function DisplayHomeUEMasterOf(_node) {
    return '';
}

function string_contains(string, value) {
    if (string)
        return replaceDiacritics(string.toUpperCase()).indexOf(value) != -1;
}

function string_highlight(string, txt) {
    if (txt == '')
        return string;
    var u = replaceDiacritics(string.toUpperCase());
    i = u.indexOf(txt);
    if (i == -1)
        return string;
    return string.substr(0, i)
        + '<span class="highlight">' + string.substr(i, txt.length)
        + '</span>'
        + string_highlight(string.substr(i + txt.length));
}

function search_ue_list(txt) {
    txt = replaceDiacritics(txt.toUpperCase());
    search_ue_list.txt = html(txt);
    if (txt == '')
        return '';
    var ues = [], done = {};
    for (var ue in all_ues) {
        ue = all_ues[ue];
        if (done[ue.name])
            continue;
        done[ue.name] = true;
        if (string_contains(ue.intitule, txt)
            || string_contains(ue.name, txt)
            || string_contains('UE-' + ue.name, txt)
            || (ue.responsable
                && string_contains(ue.responsable.join(', '), txt)
            )
            || (ue.old_names
                && string_contains(ue.old_names.join(' '), txt)
            )
        )
            if (ue.nr_students_ue != undefined
                && !ue.name.match(/-/)) // 'etape-'
                ues.push('UE-' + ue.name);
            else
                ues.push(ue.name);
    }
    if (ues.length == 0)
        return '<b>' + _("MSG_home_nothing") + '</b>';
    return display_ues("TH_home_search", '', ues,
        { nr_max: 100, hide_title: true, default_order: "A" });
}

function update_block_content(name) {
    var e = document.getElementById(name);
    if (!e)
        return;
    var node = display_definition[name];
    node.data = display_data[name.replace('Display', '')];
    var html = node.fct(node);
    if (html instanceof Array)
        html = html[0];
    e.innerHTML = html;
}

function update_job() {
    for (var block in update_job.todo) {
        update_block_content(block);
        delete update_job.todo[block];
        return true;
    }
    return false;
}
update_job.todo = [];

function search_ue_change(t) {
    if (t.value == search_ue_change.last_value)
        return;
    search_ue_change.last_value = t.value;
    set_option('ue', t.value);
    document.getElementById("ue_search_result").innerHTML =
        search_ue_list(t.value);

    for (var i in display_definition['HomeUE'].children)
        update_job.todo[display_definition['HomeUE'].children[i].name] = true;
    periodic_work_add(update_job)
}

function HomeReset(event) {
    var input = the_event(event).target.parentNode.getElementsByTagName('INPUT')[0];
    input.value = '';
    input.onpaste();
    input.focus();
}

function DisplayHomeUE(_node) {
    var s = [display_ues("TH_home_search", _("TIP_home_search_ue"), [],
        { default_order: 'A', hide_open_close: true })
    ];
    s[0] = s[0].substr(0, s[0].length - 6); // Remove last </div>
    s.push('<div class="HomeReset" onclick="HomeReset(event)">×</div>');
    s.push('<div class="search_box">');
    s.push(hidden_txt('<input id="search_ue" onkeyup="search_ue_change(this)" onpaste="t=this;setTimeout(function(){search_ue_change(t);},100)" value="'
        + encode_value(search_ue_change.last_value) + '">',
        _("TIP_home_search_ue")));
    s.push('</div></div>');
    s.push('<div id="ue_search_result" class="ue_list border">');
    if (search_ue_change.last_value)
        s.push(search_ue_list(search_ue_change.last_value));
    s.push('</div>');
    var children = display_definition['HomeUE'].children;
    function auto_select() {
        try { document.getElementById('search_ue').select(); }
        catch (e) { setTimeout(auto_select, 100); }
    }
    setTimeout(auto_select, 100);
    for (var i in children)
        s.push(display_display(children[i]));
    return s.join('');
}
DisplayHomeUE.need_node = ['HomePreferences'];

function university_year() {
    var ys = year_semester().split('/');
    var i = myindex(semesters, ys[1]);
    if (i == -1)
        return Number(ys[0]);
    return Number(ys[0]) + semesters_year[i];
}

var is_a_favorite = {};
var is_a_favorite_action = {};
var is_a_refered = {};
var students_info = {};

function follow_notebook_link(t, path) {
    var title = get_ue_table(t).firstChild.innerHTML;
    var filter = '';
    if (title.match('\\([0-9]+/[0-9]+\\)')) {
        var students = get_student_list(t);
        var s = [];
        for (var i in students)
            s.push(students_info[students[i]][0]);
        filter = '/=filters=' + '0_0:' + s.join('|');
    }
    window.open(path + filter);
}

function student_notepad_link(path) {
    return fast_tip('<a onclick="follow_notebook_link(this,\''
        + path + '\')">'
        + _("TH_home_notepad") + '</a>',
        _("TIP_home_notepad"));
}

function get_student_list(t) {
    while (t.className.substr(0, 7) != 'ue_list')
        t = t.parentNode;
    var s = [];
    for (var i = 1; i < t.childNodes.length; i++)
        s.push(t.childNodes[i].childNodes[1].innerHTML);
    return s;
}

function send_mails(t) {
    var students = get_student_list(t);
    var s = [];
    for (var i in students)
        s.push(students_info[students[i]][3]);
    my_mailto(s.join(','), true);
}
function student_mails_link() {
    return fast_tip('<a onclick="send_mails(this)">'
        + _("TH_home_mail") + '</a>', _("TIP_home_mail"));
}

function go_suivi_students(t) {
    go_suivi(get_student_list(t).join(','));
}
function student_suivi_link() {
    return fast_tip('<a onclick="go_suivi_students(this)" target="_blank">'
        + _("TH_home_suivi") + '</a>', _("TIP_home_suivi"));
}

function go_import_list_do() {
    var values = popup_text_area().value.split(/[ \t\n,;.:]+/);

    create_popup('import_list',
        _("TH_home_import_refered"),
        '<iframe width="100%" src="' + add_ticket('referent_get/'
            + values.join('/')) + '">' + '</iframe>',
        _("MSG_home_reload"),
        false);
}

function go_import_list() {
    create_popup('import_list',
        _("TH_home_import_refered"), _("MSG_home_import_refered"),
        _("MSG_home_import_refered_after")
        + '<BUTTON OnClick="go_import_list_do();">'
        + _("B_home_import_refered")
        + '</BUTTON>.',
        '');
}

function student_import_link() {
    return fast_tip('<a href="javascript:go_import_list()">'
        + _("TH_home_import") + '</a>', _("TIP_home_import"));
}

function DisplayHomeStudentPicture(_node) {
    return fast_tip('<img src="' + student_picture_url(open_student.code) + '">',
        '<img src="' + student_picture_url(open_student.code) + '">'
    );
}
DisplayHomeStudentPicture.need_node = [];

function DisplayHomeStudentMail(_node) {
    var mail = students_info[open_student.code][3];
    return '<a href="mailto:' + mail + '">' + mail + '</a>';
}
DisplayHomeStudentMail.need_node = [];

function DisplayHomeStudentSuivi(_node) {
    return box_button(_("MSG_home_suivi"), suivi_url(open_student.code));
}
DisplayHomeStudentSuivi.need_node = [];

function DisplayHomeStudentBilan(_node) {
    if (i_am_a_referent)
        return box_button(_("B_home_bilan_tomuss"),
            add_ticket('bilan/' + open_student.code));
    return '';
}
DisplayHomeStudentBilan.need_node = [];

function DisplayHomeStudentTimeline(_node) {
    if (i_am_a_referent)
        return box_button(_("Timeline"),
            suivi_url('timeline/' + open_student.code));
    return '';
}
DisplayHomeStudentTimeline.need_node = [];

function DisplayHomeStudentGet(_node) {
    if (display_definition["HomeStudentMenu"].table == "MSG_home_staff")
        return '';
    if (i_am_a_referent && !is_a_refered[open_student.code])
        return '<div onclick="javascript:catch_this_student('
            + js2(open_student.code) + ')">'
            + _("MSG_bilan_take_student").split("\n")[0] + '</div>';
    return '';
}
DisplayHomeStudentGet.need_node = [];

function open_student(t, code) {
    display_definition["HomeStudentMenu"].table = get_ue_table_title(t);
    while (!t.className.match(/\bue_line\b/))
        t = t.parentNode;
    if (t.childNodes[t.childNodes.length - 1].className.match('HomeStudentMenu')) {
        t.onmouseleave();
        return;
    }
    t.className += " ue_line_hover";
    var m = document.createElement("DIV");
    open_student.code = code;
    m.innerHTML = display_display(display_definition["HomeStudentMenu"]);
    m = m.firstChild;
    t.onmouseleave = function () {
        try {
            t.removeChild(m);
            t.className = t.className.replace(" ue_line_hover", "");
        }
        catch (e) { };
        t.onmouseleave = "";
    };
    t.appendChild(m);
}

function get_possible_filters(node, dict) {
    var filter = { 'tt': true, 'first_registration': true, '~any~': true };
    for (var i in node.data) {
        dict[node.data[i][0]] = true;
        students_info[node.data[i][0]] = node.data[i];
        if (node.data[i][6]) {
            var t = node.data[i][6].split(/  */);
            if (t[0] !== '')
                for (var j in t)
                    filter[t[j]] = true;
        }
    }
    return filter;
}

function DisplayHomeStudentFavorites(node) {
    if (node.data.length == 0)
        return '';
    var notebook = add_ticket(university_year()
        + "/Favoris/"
        + username.replace(/\./g, "__")
    );
    return [display_ues(_("TH_home_bookmark_student"), '', node.data,
        {
            students: true, hide_sort: true,
            default_order: 'surname+firstname',
            code_filter: get_possible_filters(node, is_a_favorite),
            title_second_line: student_notepad_link(notebook)
                + '/' + student_mails_link()
                + '/' + student_suivi_link()
        }),
    [], [], 'id="HomeStudentFavorites"'];
}

function current_semester() {
    return year_semester().split('/')[1];
}

function DisplayHomeStudentRefered(node) {
    if (node.data.length == 0)
        return '';

    var notebook = add_ticket(university_year()
        + "/Referents/"
        + username.replace(/\./g, "__")
        + (current_semester() != first_university_year_semester()[1]
            ? '/=column_offset=5'
            : '')
    );

    return [display_ues(_("TH_home_refered_student"), '', node.data,
        {
            students: true, hide_sort: true,
            code_filter: get_possible_filters(node, is_a_refered),
            default_order: 'surname+firstname',
            title_second_line:
                '<small>'
                + student_notepad_link(notebook)
                + '/' + student_mails_link()
                + '/' + student_suivi_link()
                + '/' + student_import_link()
                + '</small>'
        }),
    [], [], 'id="HomeStudentRefered"'];
}

// This function may be called by 'login_list' plugin.
function full_login_list(login, results, add) {
    if (full_login_list.cache[login] === undefined)
        full_login_list.cache[login] = { 'student': [], 'teacher': [] };

    full_login_list.cache[login][add] = results;

    for (var i in results) {
        var i = results[i];
        if (students_info[i[0]] === undefined)
            students_info[i[0]] = i;
    }

    if (login != ask_login_list)
        return;

    if (full_login_list.style)
        full_login_list.style.display = "none";

    if (add == 'student')
        update_job.todo['HomeStudentStudents'] = true;
    else
        update_job.todo['HomeStudentTeachers'] = true;
    periodic_work_add(update_job);
}
full_login_list.cache = {};

function update_students_real(t) {
    update_students_real.pending = false;
    if (ask_login_list.length <= 1)
        return;
    if (t)
        full_login_list.style = t.parentNode.parentNode.parentNode.lastChild.style;
    if (full_login_list.style)
        full_login_list.style.display = 'block';
    var s = document.createElement('SCRIPT');
    s.src = add_ticket('login_list/' + encode_uri(ask_login_list));
    document.getElementsByTagName('BODY')[0].appendChild(s);
}

function search_student_change(t, event) {
    ask_login_list = replaceDiacritics(
        t.value.trim().replace(/\t/g, ' ')).toUpperCase();
    set_option('student', ask_login_list);
    if (ask_login_list.length >= 2
        && full_login_list.cache[ask_login_list] === undefined
    ) {
        if (t.parentNode.lastChild.style)
            t.parentNode.lastChild.style.display = "block";
        if (!update_students_real.pending) {
            update_students_real.pending = true;
            setTimeout(function () { update_students_real(t); }, 1000);
        }
    }
    update_job.todo['HomeStudentStudents'] = true;
    update_job.todo['HomeStudentTeachers'] = true;
    update_job.todo['HomeStudentFavorites'] = true;
    update_job.todo['HomeStudentRefered'] = true;
    periodic_work_add(update_job);
    if (event
        && event.keyCode == 13
        && full_login_list.cache[ask_login_list]
        && full_login_list.cache[ask_login_list].student.length
        + full_login_list.cache[ask_login_list].teacher.length == 1
    ) {
        var login;
        if (full_login_list.cache[ask_login_list].student.length == 1)
            login = full_login_list.cache[ask_login_list].student[0][0];
        else
            login = full_login_list.cache[ask_login_list].teacher[0][0];
        go_suivi(login);
    }
}

function DisplayHomeStudentStudents(_node) {
    var s = [];
    try {
        s = full_login_list.cache[ask_login_list].student;
    } catch (e) { };
    return [display_ues("MSG_home_students", "", s,
        {
            students: true, hide_sort: true,
            default_order: 'surname+firstname',
            //  code_filter: get_possible_filters(s)
        }),
    [], [], 'id="HomeStudentStudents"'];
}
DisplayHomeStudentStudents.need_node = [];

function DisplayHomeStudentTeachers(_node) {
    var s = [];
    try {
        s = full_login_list.cache[ask_login_list].teacher;
    } catch (e) { };
    return [display_ues("MSG_home_staff", "", s,
        {
            students: true, hide_sort: true,
            default_order: "surname+firstname",
        }),
    [], [], 'id="HomeStudentTeachers"'];
}
DisplayHomeStudentTeachers.need_node = [];

function DisplayHomeStudents(_node) {
    if (ask_login_list === undefined) {
        ask_login_list = (preferences.forget_input
            ? ""
            : get_option('student', '')
        );
        update_students_real();
    }

    var s = [display_ues("TH_home_search", _("TIP_home_search_student"), [],
        {
            students: true, default_order: 'A', hide_sort: true, hide_open_close: true,
            default_order: "surname+firstname"
        })
    ];
    s[0] = s[0].substr(0, s[0].length - 6); // Remove last </div>
    s.push('<div class="HomeReset" onclick="HomeReset(event)">×</div>');
    s.push('<div class="search_box">');
    s.push(hidden_txt('<input id="students_list" onkeyup="search_student_change(this, the_event(event))" onpaste="t=this;setTimeout(function(){search_student_change(t);},100)" value="'
        + encode_value(ask_login_list && ask_login_list.toLowerCase() || '')
        + '">', _("TIP_home_search_student")));
    s.push('<div style="display:none">' + _("MSG_home_searching")
        + '</div></div></div>');
    var children = display_definition['HomeStudents'].children;
    for (var i in children)
        s.push(display_display(children[i]));
    return s.join('');
}
DisplayHomeStudents.need_node = ['HomePreferences'];


function do_action(action, html_class, help) {
    if (html_class == 'veryunsafe')
        if (!confirm(help + '\n\n' + _("ALERT_are_you_sure2")))
            return;
    if (action.substr(0, 1) == '/')
        goto_url(url + action);
    else
        goto_url(add_ticket(action));
}

function go_year(x) {
    goto_url(add_ticket(university_year() + "/" + x));
}

function go_year_after(x) {
    goto_url(add_ticket(x + '/' + the_year()));
}

function the_year() {
    return Number(year_semester().split('/')[0]);
}

function DisplayHomeActions(node) {
    var links = node.data;
    var boxes = {}, link_name, favorites = [];
    var nb_actions = 0;
    try {
        is_a_favorite_action = JSON.parse(localStorage["favorite_actions"] || "{}");
    } catch (e) { };
    for (var i in links) {
        if (links[i][7] === '')
            links[i][7] = links[i][4]; // Translation ID is the URL
        if (links[i][3] === '')
            links[i][3] = _('LINK_' + links[i][7]);
        if (links[i][6] === '') {
            links[i][6] = _('HELP_' + links[i][7]);
            if (links[i][6] == 'HELP_' + links[i][7])
                links[i][6] = '';
        }
        if (links[i][4].substr(0, 1) == '/')
            links[i][4] = "javascript:do_action('" + links[i][4].substr(1)
                + "','" + links[i][2] + "'," + js2(links[i][6]) + ")";
        link_name = links[i][0];
        nb_actions++;
        if (boxes[link_name])
            boxes[link_name].push(links[i]);
        else
            boxes[link_name] = [links[i]];
        if (is_a_favorite_action[links[i][3]])
            favorites.push(links[i]);
    }
    var sorted_boxes = [];
    for (var box_name in boxes) {
        var box_title = _('BOX_' + box_name);
        if (box_title === 'BOX_' + box_name)
            box_title = box_name;
        sorted_boxes.push([box_title, box_name]);
    }
    sorted_boxes.sort();
    var t = [];
    if (favorites.length)
        t.push(display_ues('', '', favorites, {
            actions: true, hide_sort: true, do_not_sort: true,
            favorite_toggle: true
        }));
    for (var box_name in sorted_boxes) {
        box_name = sorted_boxes[box_name];
        box = boxes[box_name[1]];
        box.sort(
            function (x, y) {
                if (x[1] != y[1])
                    return x[1] - y[1];
                if (x[3] < y[3])
                    return -1;
                else
                    return 1;
            });
        t.push(display_ues(box_name[0], '', box, {
            actions: true, hide_sort: true, do_not_sort: true,
            favorite_toggle: links.length > 30
        }));
    }

    var children = display_definition['HomeActions'].children;
    for (var i in children)
        t.push(display_display(children[i]));
    return t.join('');
}

function do_touchstart() {
    do_touchstart.touch = true;
}

function initialize_home3() {
    lib_init();
}
