#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2010-2013 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import os
import time
from .. import plugin
from .. import utilities
from .. import configuration

last = 0

def log(server):
    """Store a text in a log file"""
    global last
    if time.time() - last < 0.1  and  not configuration.real_regtest:
        return # pragma: no cover
    last = time.time()
    
    if server.the_path[0] not in ('help', 'javascript_errors'):
        return
    text = '/'.join(server.the_path[1:]).replace('\n','\\n'
    ).replace('\r', '\\r') + '\n'
    # XXX Not process safe
    f = open(os.path.join('LOGS', server.the_path[0]), 'a', encoding = "utf-8")
    f.write(text[:2000].strip() + '\n')
    f.close()
    

plugin.Plugin('log', '/log/{*}',
              function = log,
              authenticated = False,
              group='staff',
              )


def gui_record(server):
    """Store a text in a log file"""
    posted_data = server.uploaded
    if posted_data is None:
        return # pragma: no cover

    table = posted_data.getfirst('table')
    if not server.table_access_allowed(table):
        return
    start = posted_data.getfirst('start')
    data = posted_data.getfirst('data')
    try:
        utilities.literal_eval(data)
        utilities.literal_eval(start)
    except:
        utilities.send_backtrace(data, "GUI record")
        return
    with open(os.path.join('LOGS', 'GUI_record', server.ticket.user_name), "a") as f:
        f.write('R({},{},{})\n'.format(utilities.js(table), utilities.js(start), data))

plugin.Plugin('gui_record', '/gui_record',
              function = gui_record, no_output = True,
              group='', upload_max_size = 100000,
              priority=-7, # Before student_redirect
              launch_thread=True,
              )

utilities.mkpath(os.path.join('LOGS', 'GUI_record'))

