#!/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2010 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import json
from .. import plugin
from .. import inscrits
from .. import configuration
from .. import utilities

def send(server, values, what):
    server.the_file.write('the_full_login_list('
                          + utilities.js(server.the_path[0])
                          + ',' + values + ', %s);' % utilities.js(what))

def login_list_of(text, base, notes):
    """Retrieve information about any person, not only teachers"""
    a = inscrits.L_slow.firstname_or_surname_to_logins(
        text,
        attributes=[configuration.attr_login,
                    configuration.attr_surname,
                    configuration.attr_firstname,
                    configuration.attr_mail,
                    ],
        base = base
        )

    r = []
    no_note = None
    for login, surname, firstname, mail  in a:
        if surname is None or firstname is None:
            continue # pragma: no cover
        login = login.lower()
        firstname = firstname.title()
        surname = surname.upper()
        r.append((login, firstname, surname, mail, None, None, None, None,
                  notes.get(login, no_note)))
    return json.dumps(r)

def login_list(server):
    """Return the list of teacher and students matching the search query"""
    notes = server.ticket.user.manage_key("notes")
    if notes:
        notes = eval(notes)
    else:
        notes = {}
    send(server,
         login_list_of(server.the_path[0], configuration.cn_students, notes),
         'student')
    send(server,
         login_list_of(server.the_path[0], configuration.cn_teachers, notes),
         'teacher')

plugin.Plugin('login_list', '/login_list/{*}',
              function=login_list,
              group='staff',
              mimetype = 'application/x-javascript; charset=UTF-8',
              launch_thread=True, unsafe=False,
              priority = -10 # Before student_redirection
              )
