#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2014 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

import time
import sys
import os
import signal
import subprocess
from .. import plugin
from .. import utilities
from .. import document
from .. import configuration

def restart_tomuss(server, start=True):
    "Restart or stop TOMUSS when it is unused"
    wait = 1 # Tables must be unused for this time in minutes
    upload = 5 # No upload for this number of minutes
    if server is None: # pragma: no cover
        _ = utilities._
        def w(txt):
            print(txt)
    else:
        _ = server._
        def w(txt):
            server.the_file.write(txt)
            if txt[-1] != ' ':
                server.the_file.write('<br>\n')
            server.the_file.flush()
        if len(server.the_path) >= 1:
            wait = float(server.the_path[0])
        if len(server.the_path) >= 2:
            upload = float(server.the_path[1])
    w(_("HELP_restart_tomuss"))
    while True:
        w(time.ctime() + ' ')
        now = time.time()
        for t in document.tables_values():
            mtime = now - t.mtime
            if  mtime < wait*60: # pragma: no cover
                w(_("MSG_restart_access") % wait
                  + ' <small>(%s mtime=%d seconds)</small>' % (t, mtime))
                break
        else:
            time_since_last_upload = now - configuration.time_of_last_upload
            if time_since_last_upload < upload*60:
                w(_("B_Upload")
                  + ' <small>%ds &lt; %ds</small>' % (
                           time_since_last_upload, upload*60)) # pragma: no cover
            elif utilities.important_job_running() and not configuration.real_regtest:
                w(str(utilities.current_jobs)) # pragma: no cover
            else:
                # Restart TOMUSS
                w('GO '*10)
                utilities.warn(repr(sys.argv))
                if start:
                    subprocess.Popen(
                        sys.argv + configuration.removed_args, close_fds=True)
                utilities.on_kill.received = 42
                if configuration.real_regtest:
                    os.kill(os.getpid(), signal.SIGTERM)
                else:
                    os.kill(os.getpid(), signal.SIGKILL) # pragma: no cover
                return # Never here
        time.sleep(wait*6) # pragma: no cover

configuration.restart_tomuss = restart_tomuss

plugin.Plugin('restart_tomuss', '/restart_tomuss/{*}', group='roots',
              function=restart_tomuss, launch_thread=True,
              link=plugin.Link(where='debug', html_class='unsafe', url="/restart_tomuss")
              )

plugin.Plugin('stop_tomuss', '/stop_tomuss/{*}', group='roots',
              function=lambda server: restart_tomuss(server, start=False),
              launch_thread=True,
              link=plugin.Link(where='debug', html_class='veryunsafe', url="/stop_tomuss")
              )
