#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2013,2018 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

"""
Save user preferences for suivi
"""

import os
import time
from .. import plugin
from .. import utilities
from .. import files
from .. import configuration

def save_preferences(server):
    """Set the user preferences"""
    user = server.ticket.user
    d = user.preferences
    for item in server.the_path:
        item = item.split('=')
        if len(item) != 2:
            continue
        key, new_value = item
        assert utilities.safe(key) == key
        old_value = d.get(key, None)
        if new_value.isdigit():
            new_value = int(new_value)
        if old_value != new_value:
            d[key] = new_value
            msgid = 'Preference_' + key
            text = server._(msgid)
            if msgid == text:
                text = server._('Preferences_' + key)
            utilities.manage_key(
                'LOGINS',
                os.path.join(user.ID_, 'preferences.log'),
                content = repr((time.strftime('%Y%m%d%H%M%S'),
                                key, new_value, text)) + '\n',
                append=True
                )
            try:
                configuration.preferences_change(user.ID_, key,old_value, new_value)
            except: # pragma: no cover
                utilities.send_backtrace(repr((user.ID_, key,old_value, new_value)),
                                         "BUG preferences_change")
    utilities.display_preferences_set(user, d)
    server.the_file.write(files.files['ok.png'].bytes())

    # XXX For old files from TOMUSS before 5.3.2
    user.manage_key('private', delete=True)

plugin.Plugin('save_preferences', '/save_preferences/{*}', priority=-11,
              mimetype="image/png", group="",
              function=save_preferences)

def ask_share(server):
    user = configuration.User(server.something)
    prefs = user.preferences
    key = 'share_with_' + server.ticket.user_name
    message = utilities._('ask_share')
    body = message.format(server.ticket.user_name, configuration.server_url)
    if prefs.get(key, None) != -1:
        prefs[key] = -1
        utilities.send_mail_in_background(user.mail, body.split('\n')[0], body)
    utilities.display_preferences_set(user, prefs)
    server.the_file.write(server._("MSG_send_alert") + '<pre>' + body + '</pre>')

plugin.Plugin('ask_share', '/ask_share/{?}',
              function=ask_share)
