#!/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2010-2012 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import html
import re
from .. import plugin
from .. import utilities
from .. import configuration

def get_mail(users, txt):
    if '@' in txt and txt not in users:
        return re.split(" +", txt)
    mail = users[txt].mail
    if mail:
        return [mail]

def send_mail(server):
    """Send personnalized mails"""
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
        server.the_file.write(server._("MSG_evaluate"))
        return
    server.the_file.write('<title>{} TOMUSS</title>'.format(server._("TH_home_mail")))
    progress_bar = utilities.ProgressBar(
        server, message = '<h1>' + server._("MSG_abj_wait") + '</h1>')
    
    data = server.uploaded
    if data is None: # pragma: no cover
        server.the_file.write("BUG")
        return

    noreply = data.getfirst('noreply')
    subject = data.getfirst('subject')
    message = data.getfirst('message')
    sender = data.getfirst('sender')
    recipients = data.getfirst('recipients').split("\001")
    titles = data.getfirst('titles').split("\001")
    try:
        cc = data.getfirst('cc').split("\001")
    except AttributeError:
        cc = None
    try:
        archive_receiver = data.getfirst('archive').split("\001")
    except AttributeError:
        archive_receiver = None
    users = [recipient.split("\002")[0]
             for recipient in recipients + (cc or []) + (archive_receiver or []) + [server.ticket.user_name]
             # If there is no '@' it is an user ID.
             if '@' not in recipient.split("\002")[0]
             # 999@mail.org is not a mail but an user ID (shibboleth)
             or recipient.split("\002")[0].split('@')[0].isdigit()]
    users = configuration.Users(users)
    frome = users[server.ticket.user_name].mail
    if frome is None or frome == '': # pragma: no cover
        server.the_file.write('<b style="color:#F00">'
                              + server._("MSG_send_mail_impossible")
                              + '</b>'
                              + '<pre><hr><b>' + html.escape(subject)
                              + '</b><hr>\n'
                              + html.escape(message) + '<hr>'
                              + '\n'.join(
                                  user.mail
                                  for user in users
                                  if user.ID_ != server.ticket.user_name
                              )
                              + '</pre>')
        return
    if not sender:
        sender = frome
    progress_bar.append_to_message(server._("MSG_send_mail_start")
                                   % len(recipients))

    bad_mails = []
    good_mails = []
    to_send = set() # To remove duplicate mails
    for nb, recipient in enumerate(recipients):
        recipient = recipient.split("\002")
        m = get_mail(users, recipient[0])
        if m is None: # pragma: no cover
            bad_mails.append(recipient[0])
            continue
        content = message
        the_subject = subject
        for title, value in zip(titles, recipient[1:]):
            if '[' not in title: # XXX Needed when upgrading from <6.1.13
                title = '[' + title + ']'
            content = content.replace(title, value)
            the_subject = the_subject.replace(title, value)

        # print( m, the_subject, content, frome)
        carbon_copy = ()
        if cc:
            cc[nb] = get_mail(users, cc[nb])
            if cc[nb]:
                carbon_copy = tuple(cc[nb])
        to_send.add((tuple(m), the_subject, content, carbon_copy))
        progress_bar.update(nb, len(recipients))
    progress_bar.hide()

    try:
        reply =  get_mail(users, data.getfirst('reply'))[0]
    except (AttributeError, TypeError):
        reply = frome
    if noreply:
        reply = 'noreply.' + frome + '.please'

    for m, the_subject, content, carbon_copy in to_send:
        utilities.send_mail_in_background(m, the_subject, content, sender,
                                          show_to = True, cc = carbon_copy, reply_to=reply)
        good_mails.append(m)

    archive = message + '\n' + '='*79 + '\n'
    filters = data.getfirst('filters') or ''
    archive += filters + '\n' + '='*79 + '\n'
    archive += server._("MSG_send_mail_done") + ' ' + str(len(good_mails)) + ' @mails\n'
    if bad_mails:
        archive += str(len(bad_mails)) + ' ' + server._("MSG_mail_unknow") + '\n'
    archive += '='*79 + '\n'
    if bad_mails: # pragma: no cover
        archive += (server._("MSG_send_mail_error") + '\n'
                    + '\n'.join(bad_mails) + '\n'
                    )
    archive += (server._("MSG_send_mail_done") + '\n'
                + '\n'.join(' '.join(i) for i in good_mails) + '\n'
                )
    if cc:
        archive += (server._("CC:") + '\n'
                + '\n'.join(' '.join(i)
                            for i in cc
                            if i
                ) + '\n'
                )
    if archive_receiver != ['None']:
        if archive_receiver:
            for i, mail in enumerate(archive_receiver):
                if '@' not in mail:
                    the_mail = get_mail(users, mail)
                    if the_mail:
                        archive_receiver[i] = the_mail[0]
                    else:
                        bad_mails.append(mail) # pragma: no cover
            archive_receiver.append(frome)
        else:
            archive_receiver = [frome]
        utilities.send_mail_in_background(archive_receiver,
                                        server._("MSG_mail_archive")
                                        + ' ' + subject,
                                        archive, frome, show_to=True)
    server.the_file.write('<p>')
    if bad_mails: # pragma: no cover
        server.the_file.write(server._("MSG_send_mail_error")
                              + ', '.join(bad_mails) + '\n')
    progress_bar = utilities.ProgressBar(
        server, '<p>' + server._("MSG_send_mail_close"), show_numbers=False)
    progress_bar.wait_mail_sent()

plugin.Plugin('send_mail', '/send_mail', function=send_mail,
              upload_max_size = 1000000,
              group='staff', launch_thread=True)
