#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2008,2020 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

from .. import plugin
from .. import configuration

def redirect_headers(server):
    """If it seems like a table link, keep it to highlight the table
       on the 'suivi' page"""
    if (len(server.the_path) == 3
            and server.the_path[0].isdigit()
            and server.the_path[1] in configuration.semesters
       ):
        year, semester, ue = server.the_path # pragma: no cover
    else:
        year, semester = configuration.year_semester
        ue = ''
    url = configuration.suivi.url(year, semester) + '/' + ue
    return (('Location', server.ticket.url(url)),)

plugin.Plugin('student_redirect', '/{*}', group='!staff', password_ok=None,
              response=307,
              headers=redirect_headers,
              documentation="Redirect the student on the 'suivi'.",
              priority=-5,
              unsafe=False,
             )
