"""
Shares are stored in LOGINS/.../share in a dict with the share ID as key.
The share itself is defined by a dict of attributes.
"""


import os
import time
import codecs

from TOMUSS import configuration
from TOMUSS import plugin
from TOMUSS import files
from TOMUSS import display
from TOMUSS import utilities
from TOMUSS import ticket

D = display.Display

def load_share(user):
    share = user.manage_key('share')
    if share:
        share = utilities.literal_eval(share)
        t = time.time()
        expired = []
        share_duration = 86400 * configuration.share_duration
        for key, value in share.items():
            if t - value['time'] > share_duration:
                expired.append(key)
        if expired:
            for key in expired:
                del share[key]
            if not configuration.read_only:
                user.manage_key('share', content=utilities.stable_repr(share))
        return share
    else:
        return {}

def display_share(server):
    return load_share(server.ticket.user)

D('ShareButton', 'Preferences', 1, data=display_share)


files.add('PLUGINS', 'suivi_share.js')
files.append('suivi_student.js', "share", files.files['suivi_share.js'])

def share_store(server):
    share = display_share(server)
    action = server.the_path[0]
    key = server.the_path[1]
    if action == 'del':
        share.pop(key)
        message = "MSG_shared_deleted"
    elif action == 'add':
        share[key] = {'photo': int(server.the_path[2]),
                      'upload': int(server.the_path[3]),
                      'comment': server.the_path[4],
                      'time': int(time.time())}
        message = "MSG_shared_created"
    else:
        server.the_file.write('???')
        return

    server.ticket.user.manage_key('share', content=utilities.stable_repr(share))
    server.the_file.write('<b style="font-family: sans-serif">{}</b>'.format(server._(message)))
    return

plugin.Plugin('share_store', '/share_store/{*}', function=share_store, group='',
              priority=-10, launch_thread=True)

def display_names(server):
    from TOMUSS.PLUGINS import suivi_student
    return suivi_student.display_names(server)
def display_grades(server):
    from TOMUSS.PLUGINS import suivi_student
    g = suivi_student.display_grades(server)
    for table in g[0]:
        new_columns = []
        nr_deleted = 0
        line = table['line']
        for i, column in enumerate(table['columns']):
            if column.get('modifiable', 0) == 2:
                # For example an MCQ. So do not leak informations
                del line[i - nr_deleted]
                nr_deleted += 1
                continue
            column['modifiable'] = 0
            column['cell_writable'] = '=¤£§'
            column['author'] = '⛔'
            new_columns.append(column)
        table['columns'] = new_columns
        for i, cell in enumerate(line):
            if len(cell) < 3 or len(cell[1]) <= 1:
                continue
            line[i] = list(cell)
            line[i][1] = '⛔'
    return g
def display_semesters(server):
    from TOMUSS.PLUGINS import suivi_student
    return suivi_student.display_semesters(server)


D('Share'         , []        ,0, js='Vertical')
D('ShareTop'      , 'Share'   ,1, js='Horizontal')
D('SharePicture'  , 'ShareTop',0)
D('ShareLogin'    , 'ShareTop',1, js='Vertical', data=lambda s: s.suivi_login)
D('ShareNames'    , 'ShareTop',2, data=display_names)
D('ShareDelete'   , 'ShareTop',3)
D('SharePreamble' , 'Share'   ,2, js='Preamble')
D('ShareSemesters', 'Share'   ,3, data=display_semesters)
D('ShareUETree'   , 'Share'   ,4, data=configuration.display_tree)
D('ShareGrades'   , 'Share'   ,5, data=display_grades)


def not_shared(server):
    server.send_response(200)
    server.send_header('Content-Type', 'text/html;charset=UTF-8')
    server.end_headers()
    server.the_file.write(utilities._('MSG_not_shared').encode('utf-8'))

def shared(server):
    from TOMUSS.PLUGINS import suivi_student
    student = utilities.safe(server.the_path[0])
    user = configuration.User(student)
    share = load_share(user)
    if server.the_path[1] not in share:
        not_shared(server)
        return
    share = share[server.the_path[1]]
    server.suivi_login = student
    server.is_a_student = True
    server.ticket = ticket.Anonymous()
    server.year, server.semester = configuration.year_semester
    if (len(server.the_path) >= 4 and server.the_path[2] == 'upload'):
        if share['upload']:
            server.the_year = int(server.the_path[3])
            server.the_semester = utilities.safe(server.the_path[4])
            server.the_ue = utilities.safe(server.the_path[5])
            server.the_path = server.the_path[6:]
            server.ticket.user_name = student
            server.ticket.ticket = 'shared'
            from TOMUSS.COLUMN_TYPES import upload
            upload.upload_get(server)
        else:
            not_shared(server)
        return
    if (len(server.the_path) >= 4
        and server.the_path[3] in ('picture', 'picture-icon')
        and server.the_path[-1].startswith(student)
        ):
        if share['photo']:
            server.something = server.the_path[-1]
            from TOMUSS.PLUGINS import picture
            server.send_response(200)
            server.send_header('Content-Type', 'image/jpeg')
            server.end_headers()
            server.the_file = server.wfile
            picture.picture(server, '' if server.the_path[3] == 'picture' else '-')
        else:
            server.send_response(200)
            server.send_header('Content-Type', 'image/png')
            server.end_headers()
            server.wfile.write(files.files['tip.png'].bytes())
        return
    if len(server.the_path) == 3 and server.the_path[2] == 'del':
        server.ticket.user_name = student
        server.ticket.user = user
        server.the_path = ['del', server.the_path[1]]
        server.send_response(200)
        server.send_header('Content-Type', 'text/html;charset=UTF-8')
        server.end_headers()
        server.the_file = codecs.getwriter("utf-8")(server.wfile)
        share_store(server)
        return
    if len(server.the_path) >= 3:
        server.year = int(server.the_path[2])
    if len(server.the_path) >= 4 and server.the_path[3] in configuration.semesters:
        server.semester = server.the_path[3]
    server.send_response(200)
    server.send_header('Content-Type', 'text/html;charset=UTF-8')
    server.end_headers()
    server.the_file = codecs.getwriter("utf-8")(server.wfile)
    suivi_student.suivi_headers(server, is_student=True)
    server.suivi = user
    display.data_to_display(server, 'Share')

plugin.Plugin('shared', '/shared/{*}', function=shared,
              authenticated=False, priority=-10, launch_thread=True,
              mimetype=None)

