""" Competences Catalog functions and classes Regular Tests """

def error_msg(func_name, value, expected):
    raise ValueError("Function "+func_name+"() do not work,\nHave returned : "
                     +str(value)+"\n-----\nExpected : "+str(expected)+"\n========================")

# ----- Utils functions ----------------------------------------------------------------------------
def comp_regtest_utils():
    """ Desc:
            Test utils functions of the competences file

        Return:
            bool: True if all passed
    """
    # key_split() test
    for key, expected in [
        ['b9dc0', [['b', 9], ['d', None], ['c', 0]]],
        ['b9dc0+2', [['b', 9], ['d', None], ['c', 0], ['+', 2]]],
        ['B9DC0', [['B', 9], ['D', None], ['C', 0]]],
        ['b9e6c7', [['b', 9], ['e', 6], ['c', 7]]],
        ['m', [['m', None]]],
        ['b987D8c44', [['b', 987], ['D', 8], ['c', 44]]],
        ['95c5o76', [['c', 5], ['o', 76]]],
        ['', []]
        ]:
        if key_split(key) != expected:
            error_msg("key_split", key_split(key), expected)
    # filter_split() test
    for filter, expected in [
        ['m0pbo', [('m', [0]), ('p', []), ('b', []), ('o', [])]],
        ['m0p1,2b', [('m', [0]), ('p', [1, 2]), ('b', [])]],
        ['m1pbe0', [('m', [1]), ('p', []), ('b', []), ('e', [0])]],
        ['m', [('m', [])]],
        ['m0b0cf1,2,3', [('m', [0]), ('b', [0]), ('c', []), ('f', [1, 2, 3])]],
        ['', []]
        ]:
        if filter_split(filter) != expected:
            error_msg("filter_split", filter_split(filter), expected)
    # key_match_filter(key, filter) test
    for key, filter, expected in [
        [[('m', 1), ('b', 0), ('c', 0), ('f', 2), ('e', 5)], [('m', [1]), ('b', []), ('c', [0]), ('f', [0, 2, 3])], True],
        [[('m', 0), ('b', 5)], [('m', [1]), ('b', [])], False],
        [[('m', 2), ('c', 6)], [('m', [0, 2, 5]), ('c', [])], True],
        [[('m', 4), ('c', 6)], [('m', [0, 2, 5]), ('c', [])], False]
        ]:
        if key_match_filter(key, filter) != expected:
            error_msg("key_match_filter", key_match_filter(key, filter), expected)
    # key_last_word() test
    for key, expected in [
        ['b9dc0', ['c', 0]], ['B9DC0', ['C', 0]],
        ['b9e6c7Gz', ['z', None]], ['m', ['m', None]],
        ['b987D8c44', ['c', 44]], ['95c5d76', ['d', 76]],
        ['m0b2c3e4+23', ['+', 23]], ['', []]
        ]:
        if key_last_word(key) != expected:
            error_msg("key_last_word", key_last_word(key), expected)
    # key_subtract_word() test
    for key, word, expected in [
        ['b9d5c0', 'c0', 'b9d5'],
        ['B9DC0', 'D', 'B9C0'],
        ['b9e6c7d55+28', 'c7d55', 'b9e6+28'],
        ['m', 'm', ''], ['b987D8c44', 'b987', 'D8c44'],
        ['95c5a76', '95c5a76', ''], ['', 'dfazd', '']
        ]:
        if key_subtract_word(key, word) != expected:
            error_msg("key_subtract_word", key_subtract_word(key, word), expected)
    # choices_filter() test
    for choices, key, expected, simplified in [
        [['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1'], 'c0', ['m0b0c0', 'm0b0c0f0', 'm0b0c0f1'], False],
        [['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1'], 'm0', ['b0', 'b0c0', 'b0c0f0', 'b0c0f1'], True],
        [['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1'], 'f0c0m0', ['m0b0c0f0'], False],
        [['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1'], 'c0b0', ['m0', 'm0f0', 'm0f1'], True],
        [['m', 'm0b0', 'mb0c0'], 'm', ['m', 'm0b0', 'mb0c0'], False],
        [['m', 'm0b0', 'mb0c0'], '', ['m', 'm0b0', 'mb0c0'], False],
        [['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1'], 'b0c1', [], False],
        [[], 'm0', [], False]
        ]:
        choice_filtered = choices_filter(choices, key, simplified)
        if choice_filtered != expected:                 error_msg("choices_filter", choice_filtered, expected)
    # choices_sum() test
    sum_choices = ['m0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1', 'm0b0c0s0', 'm0b0c0f2', 'm0b0c0f3', 'm0b0c0f4', 'm0b0c0f5', 'm0b0c0e0',
                   'm0b0c0e1', 'm0b0c1', 'm0b0c1s1', 'm0b0c1s2', 'm0b0c1f6', 'm0b0c1e2', 'm0b0c1e3', 'm0b0c2', 'm0b0c2s4', 'm0b0c2f15',
                   'm0b0c2e4', 'm0b0c2e5', 'm0b0c3', 'm0b0c3f19', 'm0b0c3f20', 'm0b0c3s1', 'm0b0c4s9', 'm0b0c4s10', 'm0b0c4f35']
    sum_expected = {'m0b0': 29, 'm0b0c0': 10, 'm0b0c0f0': 1, 'm0b0c0f1': 1, 'm0b0c0s0': 1, 'm0b0c0f2': 1, 'm0b0c0f3': 1, 'm0b0c0f4': 1,
                    'm0b0c0f5': 1, 'm0b0c0e0': 1, 'm0b0c0e1': 1, 'm0b0c1': 6, 'm0b0c1s1': 1, 'm0b0c1s2': 1, 'm0b0c1f6': 1, 'm0b0c1e2': 1,
                    'm0b0c1e3': 1, 'm0b0c2': 5, 'm0b0c2s4': 1, 'm0b0c2f15': 1, 'm0b0c2e4': 1, 'm0b0c2e5': 1, 'm0b0c3': 4, 'm0b0c3f19': 1,
                    'm0b0c3f20': 1, 'm0b0c3s1': 1, 'm0b0c4s9': 1, 'm0b0c4s10': 1, 'm0b0c4f35': 1}
    if choices_sum(sum_choices) != sum_expected:
        error_msg("choices_sum", choices_sum(sum_choices), expected)
    # string_distance() test
    for text1, text2, expected in [
        ["Concevoir une interface homme-machine, en faire la maquette et en réaliser le prototype",
         "Concevoir une interface homme-machine, en faire la", 37],
        ["Concevoir et réaliser un logiciel", "Allons enfants de la patriiiie !", 27],
        ["Concevoir et réaliser un logiciel", "Conçèvoir, et, reäliser, ùn, laugiciel", 11],
        ["Définir les Spécifications Techniques de Besoin (STB)", "Défizdnir les Spécifiartons Tazdenityques ddze Besoin (STaz)", 15],
        ["", "######", 6],
        ["Analyser pour préciser le besoin client et produire un cahier des charges",
         "Analyse besoin client, cahier des charges", 33],
        ["Identifier les données utiles au fonctionnement d’un logiciel",
         "fonctionement logiciel, données", 47],
        ["Identifier, analyser le besoin d’un client et spécifier un logiciel permettant d’y répondre",
         "fonctionement logiciel, données", 70]
        ]:
        if string_distance(text1, text2) != expected:
            error_msg("string_distance", string_distance(text1, text2), expected)
    # extract_rating() test
    for desc, expected in [
        ['Non acquis (0/20)', 0],
        ['Pas satisfaisant (10/40)', 0.25],
        ['Dans la moyenne (12/20)', 0.6],
        ['Pas mal (15/20)', 0.75],
        ['Non évalué (NE)', -1],
        ['Ceci n\'est pas une note', -2],
        ['Ceci n\'est pas une note (f/4)', -2],
        ]:
        if extract_rating(desc) != expected:
            error_msg("extract_rating", extract_rating(desc), expected)
    # get_tree_of() test
    for observations, expected in [
        [   [ "m0b0c0f0o2", "m0b0c0f1o3", "m0b0c0f2o4", "m0b0c0f3o4", "m0b0c0f3o3", "m0b0c0f4o5"],
            {'m0': {'o': 0, 'b0': {'o': 0, 'c0':
                {'o': 0, 'f0': {'o': 2}, 'f1': {'o': 3}, 'f2': {'o': 4}, 'f3': {'o': 4}, 'f4': {'o': 5}}}}, 'o': 0}
        ]
    ] :
        if get_tree_of(observations) != expected:
            error_msg("get_tree_of", get_tree_of(observations), expected)
    # make_array_tree() test
    for dico, name, expected in [
        [   {"o": 0, "f0":{"o": 2}, "f1": {"o": 4}}, "m0",
            ['m0', 3, ['f0', 2], ['f1', 4]]
        ],
        [   {"o": 0, "b0":{"o": 0, "c0": {"o": 3}}}, None,
            [ 3, ['b0', 3, ['c0', 3]]]
        ]
    ] :
        if make_array_tree(dico, name) != expected :
            error_msg("make_array_tree", make_array_tree(dico, name), expected)
    # get_arr_tree_leaves() test
    for tree, has_name, expected in [
        [["val1", 2, ["val2", 0, ["val3", 2]], ["val4", 3]], True, [['val3', 2], ['val4', 3]]],
        [[2, ["a", 0, ["b", 2]], ["c", 3, ["d", 3], ["e", 4]]], False, [["b", 2], ["d", 3], ["e", 4]]],
        [[1], False, [[1]]]
    ] :
        if get_arr_tree_leaves(tree, has_name) != expected:
            error_msg("get_arr_tree_leaves", get_arr_tree_leaves(tree, has_name), expected)

    return True

# ----- CATALOG TESTS ------------------------------------------------------------------------------
def comp_regtest_catalog():
    """ Desc:
            Test all functions of the catalog class in the competences file

        Return:
            bool: True if all passed
    """
# ----- Initialisation functions -------------------------------------------------------------------

    text = """m0\tDescription m0
m0b0	Concevoir et réaliser un logiciel
m0b0c0f4	Concevoir une interface homme-machine, en faire la maquette et en réaliser le prototype
m2b2c3\t
m0b0c0	Identifier, analyser le besoin d’un client et spécifier un logiciel permettant d’y répondre
m0b0c0f0	Analyser pour préciser le besoin client et produire un cahier des charges
o1\tNon acquis (0/20)
o0\tNon évalué (NE)
m0b0c0o4\tTrès bien (20/20)
o2\tPas satisfaisant (12/40)
m0b0c0f2o3\tPas mal (14/20)
m0b0c0f1	Identifier les données utiles au fonctionnement d’un logiciel
^INF.*\tm0bf2,3o0,1,3
^MAT.*\tm0o3 f3
^\tm1pbeO
#m\tDescription m
#c\tDescription c
#\tzoùzoù
\tDescription perdue
m0b0c0f0	Analyser pour préciser le besoin client et produire un cahier des charges
m0b0c0f2	Utiliser des notations formelles pour produire des spécifications non-ambiguës
m0b0c0f3	Définir les Spécifications Techniques de Besoin (STB)
#ff\t
:m0p3	{"method": "average", "fail": ["o1"], "possible_results": ["m2b2c3o2", "m0b0c0f0o1", "m0b0o0"]}
:m0	{"method": "median", "fail": ["o0"], "possible_results": ["m2b2c3o2", "m0b0c0f2o3", "m0o0"]}
:m0b0c0\t{"method": "average", "fail": [], "possible_results": []}
:	{"method": "median"}"""
    expected_catalog = {
        'vocabulary':{'m': ['Description m0'],
                      'c': ['Identifier, analyser le besoin d’un client et spécifier un logiciel permettant d’y répondre'],
                      'b': ['Concevoir et réaliser un logiciel'],
                      'f': ['Analyser pour préciser le besoin client et produire un cahier des charges',
                            'Identifier les données utiles au fonctionnement d’un logiciel',
                            'Utiliser des notations formelles pour produire des spécifications non-ambiguës',
                            'Définir les Spécifications Techniques de Besoin (STB)',
                            'Concevoir une interface homme-machine, en faire la maquette et en réaliser le prototype'],
                      'o': ['Non évalué (NE)', 'Non acquis (0/20)', 'Pas satisfaisant (12/40)', 'Pas mal (14/20)', 'Très bien (20/20)']},
        'choices': ['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1', 'm0b0c0f2', 'm0b0c0f2o3', 'm0b0c0f3', 'm0b0c0f4', 'm0b0c0o4', 'o0', 'o1', 'o2'],
        'vocabulary_name': {'m': 'Description m', 'c': 'Description c'},
        'trees': {'INF.*': ['m0bf2,3o0,1,3'], 'MAT.*': ['m0o3', 'f3']},
        'trees_ue': {},
        'merges': {'m0p3': {"method": "average", "fail": ["o1"], "possible_results": ["m2b2c3o2", "m0b0c0f0o1", "m0b0o0"]},
                   'm0': {"method": "median", "fail": ["o0"], "possible_results": ["m2b2c3o2", "m0b0c0f2o3", "m0o0"]},
                   'm0b0c0': {"method": "average", "fail": [], "possible_results": []}},
        'errors': [],
        'observations': {}
    }
    json = dumpJSON(expected_catalog)

    # parse_lines() test
    lines = [line.split('\t') for line in text.split('\n')]
    catalog = Catalog()
    catalog.parse_lines(lines)
    for data, expected in [
        [catalog.vocabulary, 'vocabulary'],
        [catalog.choices, 'choices'],
        [catalog.vocabulary_name, 'vocabulary_name'],
        [catalog.trees, 'trees'],
        [catalog.merges, 'merges']
        ]:
        if str(data) != str(expected_catalog[expected]): #error_msg("parse_lines", extract_rating(desc), expected)
            raise ValueError(
                   "Function parse_lines() do not work, a field is not correctly transmitted !\n"
                   +"Found    : " + str(data) + "\nExpected : " + str(expected_catalog[expected]))

    # parse_json() test
    json_catalog = Catalog()
    json_catalog.parse_json(json)
    json_catalog.errors = []
    if json_catalog.get_json() != json:                 error_msg("parse_json", json_catalog.get_json(), json)

# ----- Get functions ------------------------------------------------------------------------------
    # get_formation_vocabs() test
    for formation, expected in [[
        ['m', 0],
        ['m0', 'm0b0', 'm0b0c0', 'm0b0c0f0', 'm0b0c0f1', 'm0b0c0f2', 'm0b0c0f2o3', 'm0b0c0f3', 'm0b0c0f4', 'm0b0c0o4']
    ]] :
        if catalog.get_formation_vocabs(formation) != expected:
            error_msg("get_formation_vocabs", catalog.get_formation_vocabs(formation), expected)
    # get_key() test
    for key, expected in [
        ['m0b0', [['m', 0, "Description m0"], ['b', 0, "Concevoir et réaliser un logiciel"]]],
        ['', []],
        ['m0b0c0', [['m', 0, "Description m0"],
                    ['b', 0, "Concevoir et réaliser un logiciel"],
                    ['c', 0, "Identifier, analyser le besoin d’un client et spécifier un logiciel permettant d’y répondre"]]],
        ['b0c0f3', [['b', 0, "Concevoir et réaliser un logiciel"],
                    ['c', 0, "Identifier, analyser le besoin d’un client et spécifier un logiciel permettant d’y répondre"],
                    ['f', 3, "Définir les Spécifications Techniques de Besoin (STB)"]]],
        ['m1', []],             #crash test
        ['m0b0cf1', []],        #crash test
        ]:
        try:
            if catalog.get_key(key) != expected:        error_msg("get_key", catalog.get_key(key), expected)
        except:
            if key not in ['m1', 'm0b0cf1']:
                raise ValueError("Function get_key() has crashed for value " + key)
    # filter_is_valid() test
    for filter, expected in [
        ['m0pbo', False], ['m0p1,2bo', False],
        ['m0bc0o1,2,4', True], ['m', True],
        ['m0b0cf1,2,3o', True], ['o2,3', True]
        ]:
        if catalog.filter_is_valid(filter_split(filter)) != expected:
            raise ValueError("Function filter_is_valid() do not work, " + filter +" -> "
                             + str(catalog.filter_is_valid(filter_split(filter))) + "\nExpected : " + str(expected) + " !")
    # get_json() test
    if json_catalog.get_json() != json:                 error_msg("get_json", catalog.get_json(), json)
    # get_allowed_keys() test
    for reg_exp, expected in [
        ["INF1010", {'m0b0c0f2': ['o0', 'o1', 'o3'], 'm0b0c0f3': ['o0', 'o1', 'o3']}],
        ["MAT1520", {'m0': ['o3'], 'm0b0': ['o3'], 'm0b0c0': ['o3'], 'm0b0c0f0': ['o3'],
                   'm0b0c0f1': ['o3'], 'm0b0c0f2': ['o3'], 'm0b0c0f3': ['o0', 'o1', 'o2', 'o3', 'o4'], 'm0b0c0f4': ['o3']}]
        ]:
        if catalog.get_allowed_keys(reg_exp) != expected: error_msg("get_allowed_keys", catalog.get_allowed_keys(reg_exp), expected)
    # get_lines(self, key_filter) test
    for key, expected in [
        ['f0', [['m0b0c0f0', catalog.vocabulary['f'][0]]]],
        ['c0', [['m0b0c0', catalog.vocabulary['c'][0]],
                ['m0b0c0f0', catalog.vocabulary['f'][0]],
                ['m0b0c0f1', catalog.vocabulary['f'][1]],
                ['m0b0c0f2', catalog.vocabulary['f'][2]],
                ['m0b0c0f2o3', catalog.vocabulary['o'][3]],
                ['m0b0c0f3', catalog.vocabulary['f'][3]],
                ['m0b0c0f4', catalog.vocabulary['f'][4]],
                ['m0b0c0o4', catalog.vocabulary['o'][4]]]],
        ['f3c0m0', [['m0b0c0f3', catalog.vocabulary['f'][3]]]],
        ['', [['m0', catalog.vocabulary['m'][0]],
              ['m0b0', catalog.vocabulary['b'][0]],
              ['m0b0c0', catalog.vocabulary['c'][0]],
              ['m0b0c0f0', catalog.vocabulary['f'][0]],
              ['m0b0c0f1', catalog.vocabulary['f'][1]],
              ['m0b0c0f2', catalog.vocabulary['f'][2]],
              ['m0b0c0f2o3', catalog.vocabulary['o'][3]],
              ['m0b0c0f3', catalog.vocabulary['f'][3]],
              ['m0b0c0f4', catalog.vocabulary['f'][4]],
              ['m0b0c0o4', catalog.vocabulary['o'][4]],
              ['o0', catalog.vocabulary['o'][0]],
              ['o1', catalog.vocabulary['o'][1]],
              ['o2', catalog.vocabulary['o'][2]]]],
        ['b0c1', []],
        ]:
        if catalog.get_lines(key) != expected:          error_msg("get_lines", catalog.get_lines(key), expected)
    # index_rate() test
    for choices, rating, expected in [
        [catalog.merges['m0p3']['possible_results'], 0.1, 1],
        [catalog.merges['m0']['possible_results'], -1, 0],
        [catalog.merges['m0b0c0']['possible_results'], 0.75, 0]
        ]:
        if catalog.index_rate(choices, rating) != expected:
            error_msg(f"index_rate({choices},{rating})", catalog.index_rate(choices, rating), expected)
    # get_completion() test
    for key, inputed, expected in [
        ['m', "descrip", [[10008, 0, "Description m0"]]],
        ['b', "logiciel réalisé", [[10023, 0, "Concevoir et réaliser un logiciel"]]],
        ['c', "analyse besoin client", [[10070, 0, "Identifier, analyser le besoin d’un client et spécifier un logiciel permettant d’y répondre"]]],
        ['f', "interface homme machine",
         [[10042, 3, 'Définir les Spécifications Techniques de Besoin (STB)' ],
          [10048, 1, 'Identifier les données utiles au fonctionnement d’un logiciel'],
          [10060, 0, 'Analyser pour préciser le besoin client et produire un cahier des charges'],
          [10065, 4, 'Concevoir une interface homme-machine, en faire la maquette et en réaliser le prototype'],
          [10066, 2, 'Utiliser des notations formelles pour produire des spécifications non-ambiguës']]],
        ['f', "analyse de données et fonctionement logicielle",
         [[10026, 1, 'Identifier les données utiles au fonctionnement d’un logiciel'],
          [10043, 3, 'Définir les Spécifications Techniques de Besoin (STB)' ],
          [10049, 0, 'Analyser pour préciser le besoin client et produire un cahier des charges'],
          [10058, 2, 'Utiliser des notations formelles pour produire des spécifications non-ambiguës'],
          [10068, 4, 'Concevoir une interface homme-machine, en faire la maquette et en réaliser le prototype']]],
        ['f', "Utiliaser des notatiozans formelles pouazr produire des zspécificatieons non-ambiguës, Définazzir les Spéecifizca",
         [[10035, 2, 'Utiliser des notations formelles pour produire des spécifications non-ambiguës'],
          [10082, 0, 'Analyser pour préciser le besoin client et produire un cahier des charges'],
          [10084, 1, 'Identifier les données utiles au fonctionnement d’un logiciel'],
          [10084, 3, 'Définir les Spécifications Techniques de Besoin (STB)' ],
          [10085, 4, 'Concevoir une interface homme-machine, en faire la maquette et en réaliser le prototype']]],
        ['', "fzefzefzefzaef", []]
        ]:
        if catalog.get_completion(key, inputed) != expected: error_msg("get_completion", catalog.get_completion(key, inputed), expected)
    # aggregate() test
    for dico, param, expected in [
        [{'m0b0c0o4': 3, 'm0b0c0o0': 1, 'm0b0c0o1': 1, 'm0b0c0f0o3': 1, 'm0b0c0f1o2': 2, 'm0b0c0f2o3': 1, 'm0b0c0f3o1': 2},
         {'method': 'average', 'fail': [], 'possible_results': ['o0', 'o1', 'o2', 'o3', 'o4']}, 'm0b0c0o3'],
        [{'m0b0f0o0': 3, 'm0b0f0o0': 1, 'm0b0c0f0o0': 1},
         {'method': 'average', 'fail': ["o1"], 'possible_results': ['o0', 'o1', 'o2', 'o3', 'o4']}, 'm0b0f0o0'],
        [{'m0b0f0o4': 1, 'm0f1o4': 2, 'm0f2o4': 1, 'm0f3o1': 3},
         {'method': 'median', 'fail': ["o0"], 'possible_results': ['o0', 'o1', 'o2', 'o3', 'o4']}, 'm0o4'],
        [['m0b0c1o2', 'm0b0o1', 'm0b0c0o4'],
         {'method': 'average', 'fail': ["o0", "o1"], 'possible_results': ['o0', 'o1', 'o2', 'o3', 'o4']}, 'm0b0o1'],
        [{}, {'method': '', 'fail': [], 'possible_results': ['o0', 'o1', 'o2', 'o3', 'o4']}, 'o0'],
        ]:
        aggregation = catalog.aggregate(dico, param)
        if aggregation != expected:
            error_msg(f"aggregate({dico},{param})", aggregation, expected)

# ----- Set functions ------------------------------------------------------------------------------

    # choices_add() test
    for key in ['a0a0a0a0a0', 's4h5e3b2', 'h4h4h4h4h4', 'g6v2y0j0q6o4']:
        catalog.choices_add(key)
        if key not in catalog.choices:
            raise ValueError("Function choices_add() do not work, key " + str(key) + " not added")
    if catalog.choices_add('s4h5e3b2'): # crash test
        raise ValueError("Function choices_add() trying to add key that already in choices (return True instead of False)")
    # choices_del() test
    for key in ['a0a0a0a0a0', 's4h5e3b2', 'h4h4h4h4h4', 'g6v2y0j0q6o4']:
        catalog.choices_del(key)
        if key in catalog.choices:
            raise ValueError("Function choices_del() do not work, key "
                            + str(key) + " not deleted")
    if catalog.choices_del('not_a_key'):            # crash test
        raise ValueError("Function choices_del() trying to delete key that not existing")
    # add_description() test
    for key, desc in [['m', "desc1"], ['m', "description2"], ['b', "desc1"], ['o', "Un peut moins nul (3/20)"]]:
        catalog.add_description(key, desc)
        if desc not in catalog.vocabulary[key]:
            raise ValueError("Function add_description() do not work, key "
                             + str(desc) + " not added in vocabulary " + str(key) + " !")
    for key, desc, expect in [['m', "", -3], ['b', "desc1", -1], ['o', 'ouibonsoir', -2]]:
        result = catalog.add_description(key, desc)
        if result != expect:
            raise ValueError(
                'Function add_description('+key+','+desc+')→'+str(result)+' in place of '+str(expect))
    # replace_description() test
    for key, ind, desc in [['m', 0, "descA"], ['b', 0, "descB"], ['f', 3, "descC"]]:
        catalog.replace_description(key, ind, desc)
        if catalog.vocabulary[key][ind] != desc:
            raise ValueError("Function replace_description() don't work, description"
                             + desc + "not replaced !")
    result = catalog.replace_description('m', 1, "descA")
    if result != -1:
        raise ValueError("Function replace_description() returns " + str(result))
    # update_vocabulary() test
    for key, desc, expected in [
        ['a', 'description de A', 1],
        ['m', 'description de M modifiée', 0],
        ['z', '', -3],
        ['', 'description de rien', -3],
        ['toto', 'zéro plus zéro', -3]
        ]:
        result = catalog.update_vocabulary(key, desc)
        if result != expected:
            raise ValueError("update_vocabulary(" + key + "," + desc + ") → "
                             + str(result) + ' != ' + str(expected))

    return True

comp_regtest_utils()
comp_regtest_catalog()
print("competences regtest are fine")
