#!/usr/bin/python3
# -*- coding: utf-8 -*-
# TOMUSS: The Online Multi User Simple Spreadsheet
# Copyright (C) 2013-2014 Thierry EXCOFFIER, Universite Claude Bernard
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contact: Thierry.EXCOFFIER@univ-lyon1.fr

debug = True

def checkDate(value, expected):
    if user_date_to_date(value) != expected:
        print(("user_date_to_date(",value,')=', user_date_to_date(value)))
        print(('Expected:', expected))
        raise ValueError('regression_test_failed')

minors = []

def localtime():
    return [2014, 5, 15, 12]

def millisec():
    return 1400148000 * 1000

def _(txt):
    return '%' + txt + '%'

def dateRegtest():
    if ','.join(REsplit("[ab]", "1a2b3")) != '1,2,3':
        regression_test_failed

    checkDate('1/1/2014', '20140101')
    checkDate('1', '20140501')
    checkDate('16', '20140516')
    checkDate('16/1', '20140116')
    checkDate('16 18', '2014051618')
    checkDate('16 18:1', '201405161801')
    checkDate('16/1 18:1:1', '20140116180101')
    checkDate('1h', '20140515110000')
    checkDate('0.5h', '20140515113000')
    checkDate('1d', '20140514120000')
    checkDate('24h', '20140514120000')
    checkDate('1w', '20140508120000')
    checkDate('1m', '20140415120000')
    checkDate('5j', '20140510120000')
    checkDate('/1/2014', '9999')


def filterRegtest():
    bugs = []
    def bug(message, txt, username="", column_type=""):
        print((message + ' for ' + txt))
        print((Filter(txt, column_type).js()))
        bugs.append(1)
    class Cell:
        def __init__(self, value, author, date, history, comment):
            self.value = value
            self.author = author
            self.date = date
            self.history = history
            self.comment = comment
        def js(self):
            return ('Cell(' + self.value + ',' + self.author + ','
                   + self.date + ',' + self.history + ',' + self.comment
                    + ')')

    def F(x, y):
        x = Filter(x, y)
        if x:
            x.js()
        return x

    tst = "joHé n<>Ê#@?&=→/\\\\"
    c = Cell(4, tst, "20140510181920", "14", "")
    line = []
    # Simple filter that should return True.
    # They are also tested prefixed by !
    for tst in ['', '<5', '<=4', '>3', '=4', '>=4', '@j', '@<k', ':<15', ':~4',
                '<12', '<4.1', '=4.0', '<4,1', '<14,1', '=4,0',
                '?=10/05/2014', '?=10/05', '?=10',
                '?>9', '?<1/6', '?>=10', '?>ad',
                '?>10/05/2014\\ 18:18', '?>10/05/2014\\ 18:19:19',
                '?>10/05/2014_17',
                '?<5j', '?<114h', '?>4j',
                '#=', ":",
                '@~=', '@~\\&=', "@~\\ n", "@~\\ ", "@~H",
                "@~h", "@~e", "@~é", "@~ê", "@~E", "@~→/",
                "#=", "=4 ", "4 ", "4.", "4.0", "@~\\\\\\\\",
                "@~\\ ", "@>é", "@<ô"
                ]:
        if not F(tst, "Note").evaluate(line, c, 'nobody'):
            bug("BUG1", tst)
        if not F(tst, "").evaluate(line, c, 'nobody'):
            bug("BUG1.0", tst)
        if not F(' ' + tst + ' ', "").evaluate(line, c, 'nobody'):
            bug("BUG1.1", ' ' + tst + ' ')
        tst = '!' + tst
        if F(tst, "").evaluate(line, c, 'nobody'):
            bug("BUG2", tst)
    # Simple filter that should return False.
    # They are also tested prefixed by !
    for tst in ['<4', '<=3', '>4', '=5', '>=5', '@o', '@<j', ':>3', ':~3',
                '?=9/05/2014', '?=9/05', '?=9',
                '?>10', '?>=31', '@>k', "#",
                '?>10/05/2014-18:19', '?>10/05/2014\\ 18:19:20',
                '?>10/05/2014_18',
                '?', '?<=4j',
                'undefined',
                "@~è", contextual_case_sensitive and "@~J" or 'NO',
                "@>ô",
                "@~=/", # Test if → is lost
                "\\ ",
                "=#[]", # undefined behavior
                ">[", # Unclosed
                "!:[]=",
                ]:
        if F(tst, "").evaluate(line, c, 'nobody'):
            bug("BUG3", tst)
        if F(tst+' ', "").evaluate(line, c, 'nobody'):
            bug("BUG3space", tst)
        tst = '!' + tst
        if not F(tst, "").evaluate(line, c, 'nobody'):
            bug("BUG4", tst)
        if not F(tst + ' ', "").evaluate(line, c, 'nobody'):
            bug("BUG4space", tst)

    # Complex filter that should return True.
    for tst in ["@j&:14", "@j &:14", "@j & :14", "@j& :14", "@j :14",
                "4 :14", "4  :14",
                "@j & :14 & =4", "@j :14 =4", "@j   :14   =4",
                "=4|=5", "=4 | =5", "=5 | =4", "=5 OR =4", "=5 OR !=6",
                "@~&=4", '!a !b', '!a 4', '4 !a',
                "=5 OR =4 OR =3", "=4 OR =3 OR =5", "=3 OR =5 OR =4",
                ">3 <5",
                ">3 <5 <9",
                ">3 <5 <9 OR >9",
                ">3 <5 <9 OR >9 >9",
                ">9 >9 OR >3 <5 <9",
                "!#[]",
                ]:
        tst = replace_all(tst, 'OR', or_keyword())
        if not F(tst, "").evaluate(line, c, 'nobody'):
            bug("BUG5", tst)
        if not F(tst + ' ', "").evaluate(line, c, 'nobody'):
            bug("BUG5space", tst)
    # Complex filter that should return False.
    for tst in ['@j & :13', '@k & :14', '@j :13', '@k :14', '@k   :14',
                '@~&=', "@~ n", 'b !4', '!4 b',
                 "#[]>5",
                ]:
        if F(tst, "").evaluate(line, c, 'nobody'):
            bug("BUG6", tst)
        if F(tst+' ', "").evaluate(line, c, 'nobody'):
            bug("BUG6space", tst)
        if tst == "#[]>5":
            if F(tst, "").get_errors() != "«>#[] » → « #[] »<br>\n":
                raise ValueError("get_error")

    # Special cases
    for cell_filters_result in [
        [[''   , "john", "", "", ""], ["@"    , "xx"  , ""], False],
        [[''   , "john", "", "", ""], ["@"    , "john", ""], True ],
        [[''   , "john", "", "", ""], ["@"    , "joh" , ""], False],
        [['jo' , ""    , "", "", ""], ["=@[]" , "jo"  , ""], True],
        [['jo' , ""    , "", "", ""], ["=@[]o", "j"  , ""], True],
        [['jo' , ""    , "", "", ""], ["@[]"  , "jo"  , ""], True],
        [['jo2', ""    , "", "", ""], ["@[]"  , "jo"  , ""], True],
        [[''   , ""    , "", "", "jo"], ["#=@[]", "jo"  , ""], True],
        [[''   , ""    , "", "", "jo"], ["#=@[]", "jh"  , ""], False],
        [[''   , ""    , "", "", "joh"], ["#@[]", "jo"  , ""], True],
        [[''   , ""    , "", "", "joh"], ["#@[]", "jh"  , ""], False],
        [['j'  , ""    , "", "", ""], ["~@[]" , "jo"  , ""], False],
        [['j'  , ""    , "", "", ""], ["@[]"  , "jo"  , ""], False],
        [['5,3', ""    , "", "", ""], ["<5.2" , ""    , ""], False],
        [['5,3', ""    , "", "", ""], ["<5.31", ""    , ""], True ],
        [['5.3', ""    , "", "", ""], ["<5,31", ""    , ""], True ],
        [['5.3', ""    , "", "", ""], ["<5,2" , ""    , ""], False],
        [['5/' , ""    , "", "", ""], ["<51"  , ""    , ""], False],
        [['5a' , ""    , "", "", ""], ["<51"  , ""    , ""], False],
        [['a'  , ""    , "", "", ""], [">10"  , ""    , ""], False],
        [[''   , ""    , "", "", ""], ["<4"   , ""    , ""], True],
        [[''   , ""    , "", "", ""], [">=0"  , ""    , ""], True],
        [[''   , ""    , "", "", ""], ["=0"   , ""    , ""], True],
        [[''   , ""    , "", "", ""], ["<0"   , ""    , ""], False],
        [[''   , ""    , "", "", ""], [">0"   , ""    , ""], False],
        [[''   , ""    , "", "", ""], ["0"    , ""    , ""], False],
        [[''   , ""    , "", "", ""], ["="    , ""    , ""], True],
        [['\n' , ""    , "", "", ""], ["="    , ""    , ""], False],
        [['a b', ""    , "", "", ""], ["a b"  , ""    , ""], True],
        [['a'  , ""    , "", "", ""], ["a | b", ""    , ""], True],
        [[''   , ""    , "", "", ""], ["x"    , ""    , "Date"], False],
        [[''   , ""    , "", "", ""], ["xd"   , ""    , "Date"], False],
        [['ab' , ""    , "", "", ""], ["=a"   , ""    , ""], False],
        [['a'  , ""    , "", "", ""], ["=a"   , ""    , ""], True],
        [[''   , ""    , "", "","x"], ["#=x"  , ""    , ""], True],
        [['0'  , ""    , "", "", ""], ["="    , ""    , ""], False],
        [['#'  , ""    , "", "", ""], ["\\#"  , ""    , ""], True],
        [['!'  , ""    , "", "", ""], ["\\!"  , ""    , ""], True],
        [[5.3  , ""    , "", "", ""], ["<5,2" , ""    , ""], False],
        [[5.1  , ""    , "", "", ""], ["<5,2" , ""    , ""], True],
        [[5.1  , ""    , "", "", ""], ["5"    , ""    , ""], True],
        [[42   , ""    , "", "", ""], ["=@[]o", "j"  , ""], False],
        [['01' , ""    , "", "", ""], ["01"  , ""    , ""], True],
        [['012', ""    , "", "", ""], ["01"  , ""    , ""], True],
        [['012', ""    , "", "", ""], ["1"   , ""    , ""], False],
        [['012', ""    , "", "", ""], ["0"   , ""    , ""], True],
        [['012', ""    , "", "", ""], [">1"  , ""    , ""], True],
        [['012', ""    , "", "", ""], ["=12" , ""    , ""], True],
        ]:
        cell, filters, result = cell_filters_result
        if F(filters[0], filters[2]
                  ).evaluate(line, Cell(cell[0], cell[1], cell[2], cell[3], cell[4]
                              ), filters[1]) != result:
            bug("BUG7", filters[0], username=filters[1])
        if F('!'+filters[0], filters[2]
                  ).evaluate(line, Cell(cell[0], cell[1], cell[2], cell[3], cell[4]
                              ), filters[1]) == result:
            print((cell, filters))
            bug("BUG8", '!'+filters[0], username=filters[1])

    # Date
    c = Cell('10/5/2014 18:19:20', "", "", "", "")
    for tst in ['<16/5/2014', '<16/5', '<16', "=10", "=10/5", "=10/5/2014",
                '<10/5/2014\\ 19', '<=10/5/2014-18:19','<10/5/2014-18:19:21']:
        if not F(tst, "Date").evaluate(line, c, 'nobody'):
            bug("BUG10", tst, username="", column_type="Date")
    for tst in ['<9/5/2014', '<9/5', '<9',
                '<10/5/2014\\ 18', '<10/5/2014_18:19','<10/5/2014-18:19:20']:
        if F(tst, "Date").evaluate(line, c, 'nobody'):
            bug("BUG11", tst, username="", column_type="Date")

    # Double spaces
    c = Cell('a  b', "", "", "", "")
    for tst in ['a ', 'a  b  ', 'a  ~b  ', '~\\ ', "~\\ \\ "]:
        if F(tst, "").evaluate(line, c, 'nobody') is not True:
            bug("BUG12", tst, username="", column_type="Text")

    # Other column on operator left
    columns_set([Column({'title': 'A'}),
                 Column({'title': 'B'}),
                 Column({'title': 'C'}),
                 Column({'title': 'D', 'type':'Date'}),
                 Column({'title': 'E'}),
                 Column({'title': 'F'}),
                 Column({'title': 'G'}),
                 Column({'title': 'H'}),
                 Column({'title': 'I'}),
             ])
    line = [
        Cell('Va'                ,"Aa","20140510181920","8,8","Ca"),
        Cell('9,9'               ,"Ab","20140510181930","Va" ,"Cb"),
        Cell('10'                ,"Ac","20140510181940","8.8","Cc"),
        Cell('10/5/2014 18:19:35',"Ad","20140510181950",""   ,"Cd"),
        Cell(8.9                 ,"Ae","20140510181940","He" ,"Ce"),
        Cell("9a"                ,"Af","20140510181940","Hf" ,"Cf"),
        Cell(11                  ,"Ag","20140515120000","Aa" ,"15/5/2014 13"),
        Cell(0                   ,"Ah","20140515120000","Hh" ,"Cee"),
        Cell('10/5/2014 18:19:35',"Ai","20140510181950",""   ,"Ci"),
    ]
    for tst in ['9', '[A]=Va', '[C]=10', '[A]V', '[A]<Vb', '[A]~a', '![A]>Va',
                '[C]1', '[A] [C]', '9 [A]',
                '#[A]=Ca', '#[A]C', '!#[A]>D', '[B]<10',
                '[A]=Va [B]=9,9 @[C]Ac ?[B]=10/5/2014 !?[B]=11/5/2014',
                ":=[A]", ":<[C]", ":~[A]", "<=[B]", "<[C]",
                "?<[D]", "?[A]<[D]", "?[C]>[D]",
                "?[B]>?[A]", "?[C]>?[B]", "!?[C]<?[A]", "?>?[A]", "?<?[C]",
                ":[A]=:[C]", ":<:[A]",
                "[D]=[D]", ":[D]=:[D]", "#[D]=#[D]", "?[D]=?[D]", "@[D]=@[D]",
                "[D]>=[D]",":[D]>=:[D]","#[D]>=#[D]","?[D]>=?[D]","@[D]>=@[D]",
                "[D]<=[D]",":[D]<=:[D]","#[D]<=#[D]","?[D]<=?[D]","@[D]<=@[D]",
                "[D]~[D]", ":[D]~:[D]", "#[D]~#[D]", "?[D]~?[D]", "@[D]~@[D]",
                "@[C]>@[B] [C]>[B] [A]=:[B]",
                "[E]<[G]", "[E]<[B]",
                "[E]>[F]", # XXX Not nice because 9a is smaller than 8
                "@[A]=Aa", ":[G]=@[A]", "@[]=Ad", "@[]=@[D]", "@[]=ad",
                "?[]=15/5/2014-12", "?[]=?[G]", "?[G]=?[]", "[D]<?[]",
                "#[G]>?[]", "#[F]>?[]",
                '!?[]>10∕10∕2021', # This is not a real '/' but U+2215
                "[H]=:[D]", "[H]0.", "#[H]#[E]",
                ">[F]", "!>\\[F]",
                "@>[F]", "!@<\\[A]", "@>\\[A]",
                "[D]>9", "![D]>10", "[D]>=10", "[D]>=2/4",
                "![I]>9", "![I]>10", "![I]>=10", "![I]>=2/4", "[I]>/"
            ]:
        if F(tst, "Text").evaluate(line, line[1], 'Ad') is not True:
            bug("BUG13", tst, username="Ad", column_type="Text")
        if tst != '<[C]' and F(tst, "Date").evaluate(line, line[1], 'Ad') is not True:
            bug("BUG13bis", tst, username="Ad", column_type="Date")

    # Check other_data_col
    for tst in [
            ["A", []],
            ["[B]", [1]],
            ["?[C]", [2]],
            [">[D]", [3]],
            ["?[] @[]", []],
            ["[E]>[F]", [4,5]],
            ["[G] | [H]", [6,7]],
            ["[H] [G] | ?[B]<?[A]", [0,1,6,7]],
            ]:
        computed = F(tst[0], "").other_data_col()
        expected = tst[1]
        error = len(computed) != len(expected)
        for i in range(len(computed)):
            if computed[i] != expected[i]:
                error = True
        if error:
            print(computed)
            print(expected)
            bug("other_data_col", tst[0])

    # Check editor
    for tst in [
            ['', '%F_true%'],
            ['!', '%F_false%'],
            ['A B', '%F_cell_value% %F_start% «a b»'],
            ['>1 <2 | >=4 <=5', '%F_cell_value% %F_>% «1»</div>%TIP_TABLECOPY_AND%<div>%F_cell_value% %F_<% «2»</div></div>%OR%<div><div>%F_cell_value% %F_>=% «4»</div>%TIP_TABLECOPY_AND%<div>%F_cell_value% %F_<=% «5»'],
            ['~A', '%F_cell_value% %F_~% «a»'],
            ['=A', '%F_cell_value% %F_=% «a»'],
            ['!A', '%F_cell_value% %!F_start% «a»'],
            ['@', '%F_cell_author% %F_%is%% %F_login%'],
            ['@=', '%F_cell_author% %F_is_empty%'],
            ['@[]~d', '%F_login% %F_~% «d»'],
            ['@>d', '%F_cell_author% %F_>% «d»'],
            ['@[B]=d', '%F_col_author% «B» %F_=% «d»'],
            ['@[B]>d', '%F_col_author% «B» %F_>% «d»'],
            ['?', '%F_cell_date% %F_=% Thu 15/05/2014'],
            ['?<3j', '%F_cell_date% %F_?>% 3 %F_j%'],
            ['?[]>1/1/2020', '%F_date% %F_>% Wed 01/01/2020'],
            ['?[B]=01/01/2020-7', '%F_col_date% «B» %F_=% Wed 01/01/2020 07'],
            ['?[B]=01/01/2020-7:8', '%F_col_date% «B» %F_=% Wed 01/01/2020 07:08'],
            ['?[B]=01/01/2020-7:8:9', '%F_col_date% «B» %F_=% Wed 01/01/2020 07:08:09'],
            ['#X', '%F_cell_comment% %F_start% «x»'],
            ['#=X', '%F_cell_comment% %F_=% «x»'],
            ['#[]', '%F_teacher%'],
            ['#[B]>5', '%F_col_comment% «B» %F_>% «5»'],
            ['@<#[B]', '%F_cell_author% %F_<% %F_col_comment% «B»'],
            ['#[B]>@[C]', '%F_col_comment% «B» %F_>% %F_col_author% «C»'],
            ['#[B]@[C]', '%F_col_comment% «B» %F_start% %F_col_author% «C»'],
            ['!=a', '%F_cell_value% %!F_=% «a»'],
            ['!=', '%F_cell_value% %!F_is_empty%'],
            ['[B]', '%F_col_value% «B» %F_start% %F_empty%: <b class="attribute_error">%F_true%</b>'],
            ['![B]', '%F_col_value% «B» %!F_start% %F_empty%: <b class="attribute_error">%F_false%</b>'],
            ['!>5', '%F_!% %F_cell_value% %F_>% «5»'],
            ['!#[]', '%F_!% %F_teacher%'],
            ['!:a', '%F_cell_history% %!F_start% «a»'],
            ['!!a', '%F_cell_value% %!F_start% «!a»'],
            ['!~a', '%F_cell_value% %!F_~% «a»'],
            ['!~', '%F_cell_value% %!F_~% %F_empty%: <b class="attribute_error">%F_false%</b>'],
            ['~', '%F_cell_value% %F_~% %F_empty%: <b class="attribute_error">%F_true%</b>'],
        ]:
        computed = F(tst[0], "").editor()[37:-18].replace('.0', '')
        if tst[1] != computed:
            print("FILTER:", tst[0])
            print("COMPUTED", computed)
            print("EXPECTED", tst[1])
            bug('Filter explanation', tst[0])


    columns_set([Column({'title': 'A'}), Column({'title': 'B', 'type': 'Date'})])
    line = [Cell(42, '', '20211201141516', '', ''), Cell('22/02/2022', '', '20130101020304', '', '')]
    for condition, expected in [
        ['=42', [['0000', False, '9999', False]]],
        ['4', [['0000', False, '9999', False]]],
        ['=41 | =42', [['0000', False, '9999', False]]],
        ['=42 | =41', [['0000', False, '9999', False]]],
        ['=41', []],
        ['=41 =42', []],
        ['=42 =41', []],
        ['!=42', []],
        ['?[]<1/1/2022', [['0000', False, '20220101', False]]],
        ['?[]<=1/1/2022', [['0000', False, '20220101', True]]],
        ['?[]>1/1/2022', [['20220101', False, '9999', False]]],
        ['?[]>=1/1/2022', [['20220101', True, '9999', False]]],
        ['?[]=1/1/2022', [['20220101', True, '20220101', True]]],
        ['!?[]<1/1/2022', [['20220101', True, '9999', False]]],
        ['!?[]<=1/1/2022', [['20220101', False, '9999', False]]],
        ['!?[]>1/1/2022', [['0000', False, '20220101', True]]],
        ['!?[]>=1/1/2022', [['0000', False, '20220101', False]]],
        ['!?[]=1/1/2022', [['0000', False, '20220101', False], ['20220101', False, '9999', False]]],
        ['?[]>1/1/2022 ?[]<3/1/2022', [['20220101', False, '20220103', False]]],
        ['?[]>1/1/2022 ?[]>3/1/2022', [['20220103', False, '9999', False]]],
        ['?[]<1/1/2022 ?[]<3/1/2022', [['0000', False, '20220101', False]]],
        ['?[]>=3/1/2022 ?[]<=3/1/2022', [['20220103', True, '20220103', True]]],
        ['?[]>=3/1/2022 ?[]<3/1/2022', []],
        ['?[]>3/1/2022 ?[]<=3/1/2022', []],
        ['?[]<1/1/2022 | ?[]<3/1/2022', [['0000', False, '20220103', False]]],
        ['?[]<3/1/2022 | ?[]<1/1/2022', [['0000', False, '20220103', False]]],
        ['?[]<3/1/2022 | ?[]>4/1/2022', [['0000', False, '20220103', False], ['20220104', False, '9999', False]]],
        ['?[]>4/1/2022 | ?[]<3/1/2022', [['0000', False, '20220103', False], ['20220104', False, '9999', False]]],
        ['?[]>4/1/2022 | ?[]<4/1/2022', [['0000', False, '20220104', False], ['20220104', False, '9999', False]]],
        ['?[]>=4/1/2022 | ?[]<=4/1/2022', [['0000', False, '9999', False]]],
        ['?[]>=4/1/2022 | ?[]<4/1/2022', [['0000', False, '9999', False]]],
        ['?[]>2 ?[]<5 | ?[]>3 ?[]<=5', [['20140502', False, '20140505', True]]],
        ['?[]<=5 ?[]<5', [['0000', False, '20140505', False]]],
        ['?[]<5 ?[]<=5', [['0000', False, '20140505', False]]],
        ['?[]>1 ?[]<=5 ?[]<5', [['20140501', False, '20140505', False]]],
        ['?[]>1 ?[]<5 ?[]<=5', [['20140501', False, '20140505', False]]],
        ['?[]<=5 ?[]<5 ?[]>1', [['20140501', False, '20140505', False]]],
        ['?[]<5 ?[]<=5 ?[]>1', [['20140501', False, '20140505', False]]],
        ['?[]>5 ?[]>=5', [['20140505', False, '9999', False]]],
        ['?[]>=5 ?[]>5', [['20140505', False, '9999', False]]],
        ['?[]>5 ?[]>=5 ?[]<9', [['20140505', False, '20140509', False]]],
        ['?[]>=5 ?[]>5 ?[]<9', [['20140505', False, '20140509', False]]],
        ['?[]<9 ?[]>5 ?[]>=5', [['20140505', False, '20140509', False]]],
        ['?[]<9 ?[]>=5 ?[]>5', [['20140505', False, '20140509', False]]],
        ['?[]>4/1/2022 ?[]<9/1/2022 ?[]>5/1/2022 ?[]<8/1/2022', [['20220105', False, '20220108', False]]],
        ['?[]>5/1/2022 ?[]<8/1/2022 ?[]>4/1/2022 ?[]<9/1/2022', [['20220105', False, '20220108', False]]],
        ['?[]>=5/1/2022 ?[]<=8/1/2022 ?[]>5/1/2022 ?[]<8/1/2022', [['20220105', False, '20220108', False]]],
        ['?[]>[B]', [['20220222', False, '9999', False]]],
        ['?<01/12/2021-14:15:17', [['0000', False, '9999', False]]],
        ['?<01/12/2021', []],
        ['?<1j', [['0000', False, '20211202141516', False]]],
        ['?[B]<3j', [['0000', False, '20130104020304', False]]],
        ['[B]<3j', [['0000', False, '20220225000000', False]]],
        ['[B]>3j', [['20220225000000', False, '9999', False]]],
        ['[B]>0j', [['20220222000000', False, '9999', False]]],
        ['[B]>?[]', [['0000', False, '20220222', False]]],
        ['>10/10/2030', [['0000', False, '9999', False]]], # Because '42' > '10...'
        ['<10/10/2030', []],
        ['?<?[B]', []],
        ['?>?[B]', [['0000', False, '9999', False]]],
        ['>?[]', [['0000', False, '20140542', False]]], # Yes: May the 42 !!!
        ['?[]>27', [['20140527', False, '9999', False]]],
        ['?[]=1', [['20140501', True, '20140501', True]]],
        ['?[]=1/1', [['20140101', True, '20140101', True]]],
        ['?[B]=1', []],
        ['?[B]=1/1/2013', [['0000', False, '9999', False]]],
        ['', [['0000', False, '9999', False]]],
        ['!', []],
    ]:
        computed = F(condition, '').dates(line, line[0], '').dates
        if computed != expected:
            print("CONDITION:", condition)
            print("DATES COMPUTED", computed)
            print("DATES EXPECTED", expected)
            print("JS", F(condition, '').js())
            bug('Filter dates', condition)

    for condition, expected in [
        ['?[]>1/1/2022', 'Sun 02/01/2022 00:00:00 → '],
        ['?[]>=1/1/2022', 'Sat 01/01/2022 00:00:00 → '],
        ['?[]<1/1/2022', ' → Sat 01/01/2022 00:00:00'],
        ['?[]<=1/1/2022', ' → Sun 02/01/2022 00:00:00'],
        ['?[]>=1/1/2022-12 ?[]<1/1/2022-13', 'Sat 01/01/2022 12:00:00 → Sat 01/01/2022 13:00:00'],
        ['?[]>1/1/2022-12 ?[]<=1/1/2022-13', 'Sat 01/01/2022 13:00:00 → Sat 01/01/2022 14:00:00'],
        ['?[]>1/1/2022-12:00', 'Sat 01/01/2022 12:01:00 → '],
    ]:
        computed = F(condition, '').dates(line, line[0], '')
        if computed.forever():
            bug("forever", condition)
        computed = computed.__str__()
        if computed != expected:
            print("CONDITION:", condition)
            print("DATES COMPUTED", computed)
            print("DATES EXPECTED", expected)
            bug('Filter display dates', condition)
    if F('?[]<1 | ?[]>2', '').dates(line, line[0], '').forever():
        bug("forever", "1")
    if not F('=42', '').dates(line, line[0], '').forever():
        bug("forever", "2")

    if len(bugs):
        regression_test_failed

            

if str(5) != '5': # For translation to JS source
    raise ValueError('float2str_bug')

if js_str('\\"\'\\') != '"\\\\\\"\'\\\\"':
    raise ValueError('js_str_bug1')

copy_to_tomuss_python()
dateRegtest()
filterRegtest()
print('Filter regtest are fine')
