#!/usr/bin/python3
# -*- coding: utf-8 -*-
# TOMUSS: The Online Multi User Simple Spreadsheet
# Copyright (C) 2021 Thierry EXCOFFIER, Universite Claude Bernard
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contact: Thierry.EXCOFFIER@univ-lyon1.fr

"""
"""

def compute_normalize_key(selectors, line):
    key = ''
    for selector in selectors:
        key += ',' + str(line[selector].value)
    return key

def compute_normalize(data_col, line, _username):
    column_out = columns[data_col]
    if len(column_out.average_columns) == 0:
        return ''
    selectors = column_out.average_columns[1:]
    column_in = column_out.average_columns[0]
    if columns[column_in].visibility == 5 and python_mode: # Compute without the grade for the suivi
        return ''

    if (not getattr(column_out, 'normalize_cache_time', None)
        or column_out.normalize_cache_time < millisec() - 100):
        # Compute all values
        try:
            the_lines = lines # Browser side
            if not the_lines or isinstance(the_lines, Array): # pragma: no cover
                # 'print' or 'stat' page: do not recompute normalize
                return line[data_col].value
        except: # pylint: disable=bare-except
            the_lines = column_out.table.lines # Server side
        grades = {}
        for line_id in the_lines:
            a_line = the_lines[line_id]
            value = a_line[column_in].value
            if str(value) == '':
                continue
            value = to_float_or_nan(value)
            if isNaN(value):
                continue
            key = compute_normalize_key(selectors, a_line)
            if key not in grades:
                grades[key] = []
            grades[key].append(value)
        column_out.normalize_average = {}
        column_out.normalize_stddev = {}
        for key in grades:
            values = grades[key]
            column_out.normalize_average[key] = sum(values) / len(values)
            column_out.normalize_stddev[key] = (
                sum([i*i for i in values]) / len(values)
                - column_out.normalize_average[key] * column_out.normalize_average[key]
            ) ** 0.5
        column_out.normalize_cache_time = millisec()

    value = line[column_in].value
    if str(value) == '':
        return ''
    value = to_float_or_nan(value)
    if isNaN(value):
        return line[column_in].value
    key = compute_normalize_key(selectors, line)
    stddev = column_out.normalize_stddev[key]
    if not stddev:
        return nan # stddev is 0
    goals = REsplit(' +', column_out.normalize)
    average_goal = float(goals[0])
    stddev_goal = float(goals[1])
    value = clamp(
        (value - column_out.normalize_average[key]) * stddev_goal / stddev
        + average_goal,
        column_out)
    return do_round(value, column_out.round_by, column_out.table.rounding,
                    column_out.old_function)
