#!/usr/bin/python3
# -*- coding: utf-8 -*-
# TOMUSS: The Online Multi User Simple Spreadsheet
# Copyright (C) 2013-2014 Thierry EXCOFFIER, Universite Claude Bernard
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contact: Thierry.EXCOFFIER@univ-lyon1.fr

python_mode = False

# For RapydScript:

if "console" not in window:
    window.console = console

try:
    str(5)
except:
    def str(x): return "" + x

try:
    "".join([])
except:
    def __join__(t):
        return t.join(this)
    String.prototype.join = __join__

try:
    "".strip()
except:
    String.prototype.strip = String.prototype.trim

try:
    "".lstrip()
except:
    if String.prototype.trimLeft:
        String.prototype.lstrip = String.prototype.trimLeft
    else:
        def __trimLeft__():
            i = 0
            while this[i] == ' ':
                i += 1
            return this[i:]
        String.prototype.lstrip = __trimLeft__

try:
    "".startswith("")
except:
    def __startswith__(t):
        return this.substr(0, len(t)) == t
    String.prototype.startswith = __startswith__

try:
    "".find("")
except:
    String.prototype.find = String.prototype.indexOf

try:
    "".lower()
except:
    String.prototype.lower = String.prototype.toLowerCase

try:
    "".upper()
except:
    String.prototype.upper = String.prototype.toUpperCase

o = Object

try:
    [].append(0)
except:
    o.defineProperty(Array.prototype, 'append' ,
                     {'enumerable': False,'value': Array.prototype.push}) ;

try:
    [].insert(0, 0, 0)
except:
    def _array_insert_(index, data):
        this.splice(index, 0, data)
    o.defineProperty(Array.prototype, 'insert' ,
                     {'enumerable': False,'value': _array_insert_}) ;

def python_pop(array, i):
    array.splice(i, 1)

def replace_all(txt, regexp, value):
    regexp = regexp.replace(RegExp("\\\\", "g"), "\\\\")
    return txt.replace(RegExp(regexp, "g"), value)

def js_str(txt):
    return JSON.stringify(txt)

def date_add(date, seconds):
    date.setTime(date.getTime() + 1000 * seconds)

##############################################################################
#
# Not easily writable in Python
#
##############################################################################


flat_map = '\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0¡¢£¤¥¦§¨©ª«¬­®¯°±²³´µ¶·¸¹º»¼½¾¿AAAAAAACEEEEIIIIDNOOOOOOOUUUUYÞßaaaaaaaceeeeiiiiðnooooooouuuuyþy'

def char_flat(c):
    return flat_map.substr(c.charCodeAt(0),1)

def flat(txt):
    return txt.replace(RegExp('[\\x80-\\xFF]', 'g'), char_flat)

unicode = str

def localtime():
    year_month_day = new(Date)
    year_month_day = [year_month_day.getFullYear(),
                      year_month_day.getMonth() + 1,
                      year_month_day.getDate()]
    return year_month_day

def seconds_to_date(seconds=None):
    d = new(Date)
    if seconds:
        d.setTime(seconds * 1000)
    else:
        d.setTime(millisec())
    return (d.getFullYear() + two_digits(d.getMonth()+1)
            + two_digits(d.getDate()) + two_digits(d.getHours())
            + two_digits(d.getMinutes()) + two_digits(d.getSeconds()) )

def date_to_seconds(date):
    return get_date_tomuss(date).getTime() / 1000

def REsplit(expreg, txt):
    return txt.split(RegExp(expreg))

def to_float_or_nan(txt):
    try:
        return Number.prototype.constructor(txt.replace(',', '.'))
    except:
        return Number.prototype.constructor(txt)

def to_float_or_small(txt):
    try:
        n = Number.prototype.constructor(txt.replace(',', '.'))
    except:
        n = txt
    if isNaN(n):
        return -1e50
    return n

# '' is 0
def to_float(txt):
    n = to_float_or_nan(txt)
    if isNaN(n):
        raise ValueError("not float")
    return n

def as_fixed(t):
    if t == '':
        return t
    v = Number(t)
    if isNaN(v):
        return t
    else:
        if v >= 1 and t[0] == '0':
            return t
        return v.toFixed(3)

rint = Math.round
ceil = Math.ceil
floor = Math.floor

def _array_extend_(tab2):
    for elem in tab2:
        this.push(elem)
o.defineProperty(Array.prototype, 'extend' ,
                 {'enumerable': False,'value': _array_extend_})

def regexp_match(regexp, text):
        return RegExp(regexp).test(text)

def copy_to_tomuss_python():
    pass

def major_of(login):
    return []

parseJSON = JSON.parse
dumpJSON = JSON.stringify

def isdigit():
    return this.match(RegExp('^[0-9]+$'))
String.prototype.isdigit = isdigit
