#!/usr/bin/python3
# -*- coding: utf-8 -*-
# TOMUSS: The Online Multi User Simple Spreadsheet
# Copyright (C) 2024 Thierry EXCOFFIER, Universite Claude Bernard
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contact: Thierry.EXCOFFIER@univ-lyon1.fr

"""
2000000000 is here because JavaScript sort strings
"""

def compute_all_ranks(column, lines):
    """Compute the reverse sorted list of cells FLOAT values"""
    data_col = column.data_col
    cells = []
    for lin_id in lines:
        line = lines[lin_id]
        value = line[data_col].get_value(column)
        if value == '':
            continue
        float_value = to_float_or_nan(value)
        if isNaN(float_value):
            continue
        cells.append(2000000000 + float_value)
    cells.sort()
    i = len(cells)
    ranks = {}
    for value in cells:
        ranks[value] = i
        i -= 1
    return ranks

def compute_rank(data_col, line, _username):
    """Use 'floats' the same way than Note.stat()"""
    column = columns[data_col]
    if len(column.average_columns) != 1:
        return ''
    column = columns[column.average_columns[0]]
    try:
        the_lines = lines # Browser side
    except: # pylint: disable=bare-except
        the_lines = column.table.lines # Server side
    floats = getattr(column, 'floats', None)
    if not floats:
        floats = column.floats = {}
    if 'ranks' not in floats:
        floats['ranks'] = compute_all_ranks(column, the_lines)
    value = line[column.data_col].get_value(column)
    if value == '':
        return ''
    value = to_float_or_nan(value)
    if isNaN(value):
        return ''
    return floats['ranks'][2000000000 + value]
