#!/usr/bin/python3

"""
Verify if the analyser works.
"""

import os
import sys
import subprocess
import io
import zipfile
import tarfile
import csv
import shutil
import xlsxwriter

EXPECTED = r'''
{
 "competences": "[\"c000 1Q1 Answer Nval01 somewhere\", \"c001 1Q1 Answer Nval somewhere\", \"c002 1Q10 Number of 0 in answers\", \"c003 1Q1 Upload is fine\", \"c004 1B1 Upload is tar\", \"c005 1Q1 Upload is zip\", \"c006 1Q1 Archive content\", \"c007 1Q3 XLSX\", \"c008 1Q1 New upload is the same than old\"]",
 "students": {
  "s0": "{\"c000\": \"1/1 Fine\", \"c001\": \"0.5/1 \", \"c002\": \"3/10 Nice\", \"c003\": \"1/1 \", \"c004\": \"0/1 \", \"c005\": \"1/1 \", \"c006\": \"0/1 \", \"c007\": \"3/3 \", \"c008\": \"1/1 \", \"message_green\": \"green\", \"message_red\": \"redred\"}",
  "s1": "{\"c000\": \"0/1 \", \"c001\": \"0.5/1 \", \"c002\": \"1/10 \", \"c003\": \"0/1 \", \"c004\": \"0/1 \", \"c005\": \"1/1 \", \"c006\": \"1.11/1 Nval10\", \"c007\": \"0/3 \", \"c008\": \"0/1 \", \"message_red\": \"redred\"}",
  "s2": "{\"c000\": \"0/1 \", \"c001\": \"0.5/1 \", \"c002\": \"1/10 \", \"c003\": \"0/1 \", \"c004\": \"1/1 \", \"c005\": \"0/1 \", \"c006\": \"1.11/1 Nval21\", \"c007\": \"0/3 \", \"c008\": \"0/1 \", \"message_red\": \"redred\"}"
 }
}

'''.strip()

def add_file(tar, name, content):
    """Add a file content into the tar"""
    with open('xxx.xxx', 'wb') as file:
        file.write(content)
    tar.add('xxx.xxx', name)

def create_zip(to_analyse): # pylint: disable=too-many-locals
    """Create the ZIP content"""
    csv_file = io.StringIO()
    writer = csv.writer(csv_file)
    def get_id(upload_id=[0]): # pylint: disable=dangerous-default-value
        upload_id[0] += 1
        return str(upload_id[0])

    with open('SCRIPTS/analyser_grading.py', 'rb') as file:
        to_analyse.writestr('YourScript.py', file.read())

    for student in range(3):
        line = ['s' + str(student)]
        for column in range(2):
            new_upload = old_upload = ''
            add = f'{student}{column}'
            if column == 0 and student == 0:
                new_upload = get_id()
                to_analyse.writestr(new_upload, b'FileContent')
            if column == 0 and student == 1:
                new_upload = get_id()
                string = io.BytesIO()
                a_zip = zipfile.ZipFile(string, mode="w")
                a_zip.writestr('file1', 'content1')
                a_zip.writestr('file2', 'content2')
                a_zip.writestr('dir1/file3', 'content3')
                a_zip.close()
                to_analyse.writestr(new_upload, string.getvalue())
            if column == 1 and student == 2:
                new_upload = get_id()
                string = io.BytesIO()
                a_tar = tarfile.TarFile(fileobj=string, mode="w")
                add_file(a_tar, 'file1', b'content1')
                add_file(a_tar, 'file2', b'content2')
                add_file(a_tar, 'dir1/file3', b'content3')
                a_tar.close()
                to_analyse.writestr(new_upload, string.getvalue())
            if column == 1 and student == 0:
                new_upload = old_upload = get_id()
                string = io.BytesIO()
                workbook = xlsxwriter.Workbook(string)
                worksheet = workbook.add_worksheet("sheetname")
                worksheet.write('A1', 'topleft')
                workbook.close()
                to_analyse.writestr(new_upload, string.getvalue())
            line.extend([
                f'Nval{add}', f'Naut{add}', f'Ncom{add}', f'Ndat{add}', new_upload,
                f'Oval{add}', f'Oaut{add}', f'Ocom{add}', f'Odat{add}', old_upload
            ])
        writer.writerow(line)
    to_analyse.writestr('students.csv', csv_file.getvalue().encode('utf-8'))
    to_analyse.close()

def not_ok(computed: str) -> bool:
    """Verify that the computed value is the expected one"""
    for line_a, line_b in zip(computed.strip().split('\n'), EXPECTED.split('\n')):
        if line_a.strip() != line_b.strip():
            print('A', line_a)
            print('B', line_b)
            print(f'computed:\n{computed}\nexpected:\n{EXPECTED}')
            return True
    return False

def start_process():
    """Run the process"""
    zip_out, zip_in = os.pipe()
    analyser = subprocess.Popen(['SCRIPTS/analyser.py', str(zip_out)],
                                stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                pass_fds=(zip_out,),)
    return analyser, os.fdopen(zip_in, "wb")

def send_dir():
    """Create the ZIP directory and analyse it"""
    print("===== Analyse a directory =====")
    with open('analyser.zip', 'wb') as file:
        to_analyse = zipfile.ZipFile(file, mode="w", compression=zipfile.ZIP_STORED,
                                     allowZip64=True)
        create_zip(to_analyse)
    os.mkdir('XXX-analyser')
    os.chdir('XXX-analyser')
    subprocess.Popen(['unzip', '../analyser.zip']).wait()
    analyser = subprocess.Popen(['../SCRIPTS/analyser.py'],
                                 stdout=subprocess.PIPE)
    computed = analyser.stdout.read().decode('utf-8')
    if not_ok(computed):
        sys.exit(1)
    analyser.wait()
    os.chdir('..')
    shutil.rmtree('XXX-analyser')
    print("OK")

send_dir()
print("===== Regtests are fine =====")
