#!/usr/bin/python3

"""
This script display/fix any student registration done on a fixed date :

SCRIPTS/bad_registration.py 'Y2020/SAutomne/UE-*L.py' '2020101[56]'

If the 'fix' parameter is added, the file is fixed in order
to remove the bad lines.

BEWARE :

  * This procedure is done without any lock on the files.
    The critical section exists but is very small (do you know Murphy ?)
    Use at your own risk.

  * It is your responsability to delete '~' backup files that may contains
    data wrote while in the critical section.
"""

import re
import sys
import os
import glob

remove_after = sys.argv[2]
try:
    fix = sys.argv[3] == 'fix'
except IndexError:
    fix = False

no = re.compile(r"^cell_change\(0,'0_0','([0-9_]*)','[0-9]+',\"" + remove_after)
change = re.compile(r"^cell_change\(([0-9]+),'[^']*','([0-9_]*)'")


for filename in glob.glob('DB/{}'.format(sys.argv[1])):
    if os.path.islink(filename):
        continue
    mtime = os.path.getmtime(filename)
    with open(filename, 'r') as f:
        lines = f.readlines()
    to_delete = set()
    pages = []
    for line in lines:
        m = no.findall(line)
        if m:
            to_delete.add(m[0])
        elif line.startswith('new_page('):
            pages.append(line.split("'")[3])
    if len(to_delete) > 5: # Only if there are many students
        do_not_remove = set()
        for line in lines:
            m = change.findall(line)
            if m and m[0][1] in to_delete:
                user = pages[int(m[0][0])]
                if len(user) > 1: # User modification on the student
                    do_not_remove.add(m[0][1])

        clean = []
        removed = []
        for line in lines:
            m = change.findall(line)
            if m and m[0][1] in to_delete and m[0][1] not in do_not_remove:
                removed.append(line)
            else:
                clean.append(line)
        print(filename, len(to_delete), len(removed), len(removed)/len(to_delete))
        if fix:
            with open(filename+'.new', 'w') as f:
                f.writelines(clean)
            print("FIXED")
            # Hope there is no access between the next two lines...
            os.rename(filename, filename+'~')
            os.rename(filename+'.new', filename)

            if os.path.getmtime(filename+'~') != mtime:
                print("DATA HAS BEEN LOST IN ", filename+'~')
                print("PROCESSING STOPPED")
                sys.exit(1)
