#!/usr/bin/python3

"""
Search bad semesters

   SCRIPTS/bad_semesters.py

You will be asked to move to Trash the bad semesters
"""
# pylint: disable=import-error,wrong-import-position

import glob
import time
import os
import re
import itertools
import random
import sys

fast_tomuss_init = True # pylint: disable=invalid-name
import tomuss_init # pylint: disable=unused-import
from TOMUSS import configuration # pylint: disable=wrong-import-order

MAX_YEAR = time.localtime().tm_year + 1
TRASH_DIR = time.strftime('Trash/%Y%m%d')

def get_templates():
    """Returns the list of table template"""
    templates = set(
        template.split('/')[-1][:-3]
        for template in itertools.chain(glob.glob('TEMPLATES/[A-Z]*.py'),
                                        glob.glob('LOCAL/LOCAL_TEMPLATES/[A-Z]*.py')
                                        )
    )
    templates.update(configuration.semesters)
    templates.add('Public')
    return templates

def content(path):
    """File list"""
    return [i for i in os.listdir(path) if i != '__init__.py']

def bad_semester(path, templates=get_templates()): # pylint: disable=dangerous-default-value
    """Returns True on strange semester name"""
    _, year, semester, _ = path.split('/')
    if year in ('CLOSED', 'LOGINS', 'RSSLOGINS'):
        return False
    if int(year[1:]) > MAX_YEAR:
        return ">MAX_YEAR"
    if semester[1:] in templates:
        files = content(path)
        if files:
            return False
        return 'empty'
    return '?'

def delete(path):
    """Move path to Trash"""
    os.rename(path, TRASH_DIR + '/' + path.replace('/', '___'))

def check():
    """Check all semesters"""
    if not os.path.exists(TRASH_DIR):
        os.mkdir(TRASH_DIR)
    for path in glob.glob('DB/*/*/'):
        error = bad_semester(path)
        if error == 'empty':
            delete(path)
            continue
        if error:
            print(path, error)
            nr_files = 0
            for i in content(path):
                with open(path + '/' + i, 'r', encoding='utf-8') as file:
                    lines = [line.split(",")[-2]
                             for line in file.readlines()
                             if 'cell_change' in line and 'cell_change(0' not in line
                            ]
                if not lines:
                    delete(path + '/' + i)
                    continue
                print('    ', len(lines), path + i)
                kind = set(re.sub("'[-a-zA-Z.]+\\.[-a-zA-Z.]+'", 'L', re.sub("['\"][0-9_]+['\"]", '#', i)) for i in lines)
                kind = list(kind)
                random.shuffle(kind)
                for line in kind:
                    print('\t', line)
                print("Delete (y or n) ?", path + i)
                if sys.stdin.readline().strip() == 'y':
                    delete(path + '/' + i)
                    continue
                nr_files += 1

            if nr_files == 0:
                delete(path)

check()
