#!/usr/bin/python3

"""
This script display any student unregistration done after the date :

SCRIPTS/bad_unregistration.py 2019 Printemps '*' 20190605 [fix]

If the 'fix' parameter is added, the file is fixed in order
to remove the bad lines.

BEWARE :

  * This procedure is done without any lock on the files.
    The critical section exists but is very small (do you know Murphy ?)
    Use at your own risk.

  * It is your responsability to delete '~' backup files that may contains
    data wrote while in the critical section.
"""

import re
import sys
import os
import glob

remove_after = sys.argv[4]
try:
    fix = sys.argv[5] == 'fix'
except IndexError:
    fix = False

no = re.compile(r"^cell_change\(0,'0_5','([0-9_]*)','non',\"([0-9]*)")

for filename in glob.glob('DB/Y{}/S{}/{}.py'.format(sys.argv[1], sys.argv[2], sys.argv[3])):
    mtime = os.path.getmtime(filename)
    with open(filename, 'r') as f:
        lines = []
        deleted = []
        state = -1
        for line in f:
            m = no.findall(line)
            if m:
                student, date = m[0]
                if date > remove_after:
                    state = 0
                    deleted.append(line)
                    continue
                else:
                    state = -1
            if state != -1:
                if re.match(r"^cell_change\([0-9]*,'0_{}','{}',"
                            .format(state, student), line):
                    state += 1
                    deleted.append(line)
                    continue
                if re.match(r"^cell_change\([0-9]*,'0_{}','{}',"
                            .format(state+1, student), line):
                    state += 2 # The student ID was recorded by a teacher
                    deleted.append(line)
                    continue
                else:
                    if state == 0:
                        # Undelete the line, no a classic student unregistration
                        lines.append(deleted.pop())
                state = -1
            lines.append(line)
    if deleted:
        print(filename)
        print(''.join(deleted))
        if fix:
            with open(filename+'.new', 'w') as f:
                f.writelines(lines)
            print("FIXED")
            # Hope there is no access between the next two lines...
            os.rename(filename, filename+'~')
            os.rename(filename+'.new', filename)

            if os.path.getmtime(filename+'~') != mtime:
                print("DATA HAS BEEN LOST IN ", filename+'~')
                print("PROCESSING STOPPED")
                sys.exit(1)

