#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
Dump the CSV tables content :
    SCRIPTS/check_file.py 2020/Automne/UE-BIO2007L ...
or
    SCRIPTS/check_file.py Y2020/SAutomne/UE-BIO*.py

Options :

  --cols=ID,Surname,Firstname   indicates a subset of columns to display

  --prefix                      insert the table name to each line

  --hide-empty                  hide the lines without student ID

  --count=regexp                display only the number of matching lines
                                This option is usable multiple times.
"""

import csv
import sys
import re
import tomuss_init
from TOMUSS import document

cols = ''
prefix = False
hide_empty = False
count = []
while True:
    if sys.argv[1].startswith('--cols='):
        cols = sys.argv[1][7:].split(",")
    elif sys.argv[1].startswith('--prefix'):
        prefix = True
    elif sys.argv[1].startswith('--hide-empty'):
        hide_empty = True
    elif sys.argv[1].startswith('--count='):
        count.append(sys.argv[1][8:])
    else:
        print(sys.argv[1])
        break
    sys.argv.pop(1)


for name in sys.argv[1:]:
    if not prefix and not count:
        print("\n# {}".format(name))
    year, semester, ue = name.split('/')
    if year[0] == 'Y':
        year = year[1:]
        semester = semester[1:]
        ue = ue[:-3]
    t = document.table(int(year), semester, ue, ro=True, create=False)
    if t:
        if cols:
            datacols = [t.columns.from_title(n).data_col for n in cols]
        else:
            datacols = range(len(t.columns))
        if count:
            counters = [0]*len(count)
        else:
            w = csv.writer(sys.stdout)
        for line in t.lines.values():
            if hide_empty and line[0].value == '':
                continue
            line = tuple(str(line[x].value) for x in datacols)
            if prefix:
                line = (name,) + line
            if count:
                line = ','.join(line)
                for i, regexp in enumerate(count):
                    if re.match(regexp, line):
                        counters[i] += 1
            else:
                w.writerow(line)
        if count:
            for regexp, n in zip(count, counters):
                name += ' ' + '{}:{}'.format(regexp, n)
            print(name)
    else:
        print("#Unreadable")


