#!/usr/bin/python3

"""
Collapse not necessary changes in the columns Grp and Seq
in the given table file names.

The pattern of values A B A become A by removing the last 2 changes

The goal is to remove fake history created by a bug.

"""

import fcntl
import os
import re
import collections
import sys

def collapse(f):
    lines = []
    usages = collections.defaultdict(list)
    
    for i, line in enumerate(f):
        m = re.match("^cell_change\\(([0-9]+),'(0_[34])','([0-9_]+)','([^']*)',",
                    line)
        if m:
            page, col_id, line_id, value = m.groups()
            key = (page, col_id, line_id)
            usage  = usages[key]
            usage.append([i, value])
            if len(usage) > 2 and usage[-3][1] == usage[-1][1]:
                # The pattern A B A => A
                lines[usage[-2][0]] = ''
                usage.pop()
                usage.pop()
                line = ''
        lines.append(line)

    return [line for line in lines if line != '']
    
def update_file(filename):
    with open(filename, 'r+') as f:
        fileno = f.fileno()
        fcntl.lockf(fileno, fcntl.LOCK_EX)
        try:
            c = collapse(f)
            with open(filename + '.new', 'w') as g:
                g.write(''.join(c))
            os.rename(filename, filename + '~')
            os.rename(filename + '.new', filename)
        finally:
            fcntl.lockf(fileno, fcntl.LOCK_UN)

def main():
    for filename in sys.argv[1:]:
        update_file(filename)

main()



