#!/usr/bin/python3
# -*- coding: utf-8

import collections
import re
import sys
import getopt
import json

class Opt:
    def __init__(self):
        self.depth = 6
        self.stop = "User"
        self.minimum = 2
        for name, value in getopt.getopt(sys.argv[1:], "",
                                         ["depth=", "stop=", "minimum="])[0]:
            setattr(self, name.strip('-'), value)
        s = sys.argv[0]
        for name, value in self.__dict__.items():
            s += " --" + name + "=" + str(value)
        self.minimum = int(self.minimum)
        self.depth = int(self.depth)
        print("   * depth   : maximum depth of the tree")
        print("   * stop    : tree is pruned under this attribute")
        print("   * minimum : hide items if less than this value")
        print()
        
opt = Opt()

names = ['Error', 'Script', 'Line', 'URL', 'User', 'Browser', 'Agent', 'Stack']


f = open("LOGS/javascript_errors", "r", encoding = "utf-8")
lines = []
for line in f:
    try:
        lines.append(json.loads(line)[1:])
    except (SyntaxError, json.decoder.JSONDecodeError):
        print(line)
f.close()
lines = [line
	for line in lines
	if len(line) >= len(names)
	]

browser = names.index("Browser")
for line in lines:
    line[browser] = re.sub(" *[0-9._]{3,99} *", "€", line[browser])

script = names.index("Script")
for line in lines:
    line[script] = re.sub(r"/files/[0-9]*\.[0-9]*\.[0-9][@0-9]*", "/FILES", line[script])


def remove_cols(lines, cols):
    cols = set(cols)
    return [
        tuple(v
         for (i, v) in enumerate(line)
         if i not in cols
         )
        for line in lines
        ]

WIDTH = 80

def most_common_recursive(lines, cols, depth=0):
    if depth == opt.depth:
        return
    if len(lines) < opt.minimum:
        return
    nr_lines = len(lines)

    def display(text):
        if '\n' in text:
            for i in text.split('\n'):
                display(i)
        else:
            before = ''
            while text:
                print('   |'*depth + before + text[:WIDTH])
                text = text[WIDTH:]
                before = '        '

    while lines:
        max_val = (0,'',-1)
        for i, _col in enumerate(cols):
            counter = collections.defaultdict(int)            
            for line in lines:
                counter[line[i]] += 1
            m = max((v, str(k), i)
                    for k, v in counter.items()
                )
            if m > max_val:
                max_val = m

        repetition, value, column = max_val

        if repetition < nr_lines // 20 and depth != 0:
            break
        # v = str(value, "UTF-8", "replace")
        display(f"{repetition}*{cols[column]}={value}".replace('\n', '\n        '))

        selected = []
        unselected = []
        for line in lines:
            if str(line[column]) == value:
                selected.append(line)
            else:
                unselected.append(line)

        if cols[column] != opt.stop:
            most_common_recursive(remove_cols(selected, [column]),
                                  cols[:column] + cols[column+1:],
                                  depth+1
                          )
        lines = unselected

most_common_recursive(lines, names)
