"""
A debug scanner.

Scanners whose class names start by 'Debug' are only launched
if specified on the command line:

   SCRIPTS/scanner.py 'Y2023/S*' Debug

This example scanner prints the averages using non standard values.

You can create other debugging scanner by using
class names starting by Debug.
"""

from TOMUSS import configuration
from TOMUSS.SCRIPTS import scanner_plugin

class Debug(scanner_plugin.Plugin): # pragma: no cover
    """Put your debug script.
    The class name must start by Debug.
    """

    def start(self):
        """Init"""

    def stop(self):
        """End"""

    def table(self, table):
        """Search problems"""
        texts = (configuration.pre, configuration.pre_short,
                 configuration.abi, configuration.abi_short,
                 configuration.abj, configuration.abj_short,
                 configuration.tnr, configuration.tnr_short,
                 configuration.ppn, configuration.ppn_short,
                 '')
        for column in table.columns:
            if column.type.name != 'Moy':
                continue
            if column.abj_is == 0:
                continue
            if column.best == '0' and column.worst == '0':
                continue
            data_cols = [table.columns.from_title(title).data_col
                         for title in column.get_columns()
                        ]
            for line in table.lines.values():
                for data_col in data_cols:
                    value = line[data_col].value
                    if isinstance(value, float) or value in texts:
                        continue
                    print()
                    print('===============', column, repr(value))
                    print()
                    return

    def record(self):
        """Nothing to record"""
        return {}

    def merge(self, data): # pylint: disable=too-many-locals,too-many-branches
        """Nothing to merge"""
