"""
Scan LOGS/SUIVI*/YYYY.connections

The computed dict is : User → Date

Store merge results in: TMP/lastlog.py
"""

import glob
import collections
import os
from TOMUSS.SCRIPTS import scanner_plugin
from TOMUSS import utilities

class LastLog(scanner_plugin.Plugin):
    """Last login on administrative year (not university one)"""
    lastlog = None
    not_on_tables = True
    only_real_file = False
    only_official_ue = False
    only_suivi_semester = False
    year_done = {} # Year → the Semester done
    merge_dict = collections.defaultdict(str)

    def start(self):
        # Compute it only once
        if self.year in self.year_done:
            self.state_filename = None # do not record
            return
        self.year_done[self.year] = self.semester

        # Do not store this information in a semester
        semester = os.path.sep + 'S' + self.semester + os.path.sep
        new_semester = os.path.sep + 'S*' + os.path.sep
        self.state_filename = self.state_filename.replace(semester, new_semester)
        utilities.mkpath(os.path.sep.join(self.state_filename.split(os.path.sep)[:-1]),
                         create_init=False)

        lastlog = collections.defaultdict(str)
        for log in glob.glob(os.path.join('LOGS', 'SUIVI*', f'{self.year}.connections')):
            with open(log, 'r') as file:
                for line in file:
                    if ' ' in line:
                        date, login = line.strip().split(' ', 1)
                        if date > lastlog[login]:
                            lastlog[login] = date
        self.lastlog = lastlog

    def stop(self):
        self.lastlog = None

    def record(self):
        if self.year_done[self.year] == self.semester:
            yield from self.lastlog.items()

    def merge(self, data):
        for (login, date), _, _, _ in data:
            if date > self.merge_dict[login]:
                self.merge_dict[login] = date

    def done(self):
        utilities.write_file(os.path.join('TMP', 'lastlog.py'), repr(dict(self.merge_dict)))
