"""
Table master scanner

The computed dict is : Teacher → List[ue]

Store in LOGINS/.../master_of


The merging must be done only in case of problem.
The 'master_of' must be always good if TOMUSS is stopped safely.
So the merging is not done except if 'Master' is on the command line

So the merging must never be done.
Except when many tables are created or destroyed on the filesystem
without using the TOMUSS interface.

To update the database:

   SCRIPTS/scanner.py 'S*/Y*' Master
"""

import collections
from TOMUSS.SCRIPTS import scanner_plugin
from TOMUSS import configuration
from TOMUSS import utilities

class Master(scanner_plugin.Plugin):
    """The tables were the teacher is master"""
    only_official_ue = False
    only_suivi_semester = False
    merge_manually = True
    teachers = None # type: Dict[str, List[str]]

    def start(self):
        self.teachers = collections.defaultdict(list)
    def stop(self):
        self.teachers = None
    def table(self, table):
        if table.semester in configuration.master_of_exceptions:
            return
        for login in table.masters:
            self.teachers[login].append(table.ue)
    def record(self):
        return self.teachers.items()
    def merge(self, data):
        tables = []
        login = '' # To please pylint
        for (login, ues), year, semester, _ in data:
            for code_ue in ues:
                tables.append((year, semester, code_ue))
        user = configuration.User(login)
        if user.valid_ID_:
            print('\r' + login, end='         ')
            user.manage_key('master_of', content=utilities.stable_repr(tables))
        user.forget() # To not eat memory (not reused)
