"""
New student scanner.

The computed dict is : teacher → List[message]

The merging send the mails
"""

import collections
import time
from TOMUSS.SCRIPTS import scanner_plugin
from TOMUSS import configuration
from TOMUSS import utilities
from TOMUSS import data

NO_MAIL_IF_MORE_THAN = 0.9 # new students
MAIL_FROM = configuration.maintainer
FORMAT = '{:10} {:<40} {:8} {:<8} {}'.format
HEADER = FORMAT(utilities._("COL_TITLE_0_0"),
                utilities._("COL_TITLE_0_2") + ' ' + utilities._("COL_TITLE_0_1"),
                utilities._("COL_TITLE_0_3"),
                utilities._("COL_TITLE_0_4"),
                ''
               )

def formatte(user, firstname, surname, grp, seq):
    """Return the tabuled student information string"""
    if user.surname not in ('Inconnu', '', None):
        surname = user.surname
    if user.firstname not in ('Inconnu', '', None):
        firstname = user.firstname
    return FORMAT(user.student_id, surname + ' ' + firstname.title(),
                  grp, seq, ' '.join(user.etapes_list) if user.etapes_list else '')

class NewStudent(scanner_plugin.Plugin):
    """New student list"""
    only_current_year = True
    one_merge_per_day = True
    teachers = None

    def start(self):
        self.teachers = collections.defaultdict(list)

    def stop(self):
        self.teachers = None

    def table(self, table):
        if self.start_time == 0: # pragma: no cover
            return # Do nothing the first time
        added = []
        removed = []
        date = time.strftime('%Y%m%d%H%M%S', time.localtime(self.start_time))

        col_inscrit = table.column_inscrit()
        for line in table.lines.values():
            if line[0].date < date:
                continue
            student_id = line[0].value
            if student_id:
                author = line[0].author
                if author == data.ro_user:
                    added.append((student_id,
                                  line[1].value,
                                  line[2].value,
                                  line[3].value,
                                  line[4].value))
                elif author == data.no_user:
                    if col_inscrit is None or line[col_inscrit].value == 'non':
                        removed.append((student_id,
                                        line[1].value,
                                        line[2].value,
                                        line[3].value,
                                        line[4].value)) # pragma: no cover
            else:
                removed.append((line[0].previous_value(),
                                line[1].previous_value(),
                                line[2].previous_value(),
                                line[3].previous_value(),
                                line[4].previous_value()))
        users = configuration.Users(infos[0] for infos in added)
        added = [formatte(users[student_id], fn, sn, grp, seq)
                 for student_id, fn, sn, grp, seq in added
                 if users[student_id].valid_ID_
                ]
        users = configuration.Users(infos[0] for infos in removed)
        removed = [formatte(users[student_id], fn, sn, grp, seq)
                   for student_id, fn, sn, grp, seq in removed
                   if users[student_id].valid_ID_
                  ]

        if not added and not removed:
            return # None change

        limit = NO_MAIL_IF_MORE_THAN * len(table.lines)
        if  len(added) > limit or len(removed) > limit:
            return # To much change

        # 10 because {:10} in the header
        added.sort(key=lambda x: x[10:])
        removed.sort(key=lambda x: x[10:])

        message = f"""
{'='*79}
{table.year}/{table.semester}/{table.ue} {table.table_title}
{', '.join(table.masters)}
{'-'*79}
{HEADER}
{'-'*79}
"""
        if added:
            message += utilities._("mail_on_new_student+") + '\n' + '\n'.join(added) + '\n'
        if removed:
            message += '\n' + utilities._("mail_on_new_student-") + '\n' + '\n'.join(removed)
        for teacher in table.masters:
            self.teachers[teacher].append(message)

    def record(self):
        return self.teachers.items()

    def merge(self, datas):
        teachers = collections.defaultdict(list)
        for (teacher, messages), _year, _semester, _ in datas:
            teachers[teacher].extend(messages)

        users = configuration.Users(teachers)
        for teacher, messages in teachers.items():
            mail = users[teacher].mail
            if mail:
                utilities.send_mail(
                    mail, utilities._("MSG_auto_update_done"), '\n\n'.join(messages),
                    show_to=True)
