"""
Search special dates for today.

The computed dict is : Teacher → List[Tuple[code_ue, column_title, cell_writable]]

The merging send the mails only once a day.
"""

import collections
import re
import time
import html
from TOMUSS import configuration
from TOMUSS import utilities
from TOMUSS.SCRIPTS import scanner_plugin

class Special(scanner_plugin.Plugin):
    """Get all the column cell_writable with the today date"""
    only_current_year = True
    one_merge_per_day = True
    tables = None
    today = None
    def start(self):
        year, month, day = [str(i) for i in time.localtime()[:3]]
        if day < '10':
            day = '0?' + day # pragma: no cover
        if month < '10':
            month = '0?' + month # pragma: no cover
        self.today = re.compile(rf"\b({day}/{month}/{year})\b")
        self.tables = collections.defaultdict(list)

    def stop(self):
        self.tables = None

    def table(self, table):
        for column in table.columns:
            if self.today.search(column.cell_writable): # pragma: no cover
                self.tables[column.author].append((table.ue, column.title, column.cell_writable))

    def record(self):
        return self.tables.items()

    def merge(self, data): # pragma: no cover # Because no special dates
        self.start() # Compute self.today
        teachers = collections.defaultdict(list)
        for (teacher, tables), year, semester, _ in data:
            for code_ue, column_title, cell_writable in tables:
                semester = html.escape(semester)
                column_title = html.escape(column_title)
                cell_writable = self.today.sub(r'<b>\1</b>', html.escape(cell_writable))
                teachers[teacher].append(
                    f'<a href="{configuration.server_url}/{year}/{semester}/{code_ue}>'
                    f'{year}/{semester}/{html.escape(code_ue)}</a> '
                    f'<b>{column_title}</b> {cell_writable}')
        for teacher, lines in teachers.items():
            user = configuration.User(teacher)
            mail = user.mail or configuration.maintainer
            lines.sort()
            message = [
                "<html>",
                utilities._("MSG_hello").format(teacher, mail),
                "<p>",
                utilities._("MSG_special_date"),
                '<pre>']
            message.extend(lines)
            message.append("</pre></html>")
            message = '\n'.join(message)
            utilities.send_mail(
                mail, utilities._("MSG_special_date_subject"), message)
            user.forget() # To not eat memory (not reused)
