"""
Table scanner to compute statistics

The computed dict is : Table → Dict[attribute, value]

"""

from TOMUSS.SCRIPTS import scanner_plugin
from TOMUSS import tablestat

class Table(scanner_plugin.Plugin):
    """Manage uploads cleanup"""
    tables = None # type: Dict[List[str]]]
    only_real_file = True
    merge_manually = True
    def start(self):
        self.tables = {}
    def stop(self):
        self.tables = None

    def table(self, table):
        self.tables[table.ue] = stats = tablestat.TableStat(table.ue)
        stats.nr_cols = len(table.columns)
        stats.nr_pages = len(table.pages)
        stats.nr_lines = len(table.lines)
        stats.empty = table.empty()
        stats.problem_in_column_name = table.problem_in_column_name()
        stats.modifiable = table.modifiable
        stats.grade_without_date = []
        col_inscrit = table.column_inscrit()
        for column in table.columns:
            if (column.type.name in self.grading_types
                    and column.weight[0] not in '+-'
                    and column.course_dates == ''
               ):
                stats.grade_without_date.append(column.title)
        for line in table.lines.values():
            if col_inscrit is not None:
                if line[col_inscrit].value != 'non':
                    stats.nr_inscrits += 1
                else:
                    stats.nr_not_inscrits += 1
                if len(line) > 4:
                    group_and_seq = str(line[3].value) +'/'+ str(line[4].value)
                    stats.group_and_seq.add(group_and_seq)
            for cell in line:
                stats.update(cell)

    def record(self):
        for values in self.tables.values():
            if not values.teachers:
                values.teachers = () # XXX No empty set (literal_eval bug)
            if not values.group_and_seq:
                values.group_and_seq = () # XXX No empty set (literal_eval bug)
        keys = ('nr_cols', 'nr_pages', 'nr_lines', 'empty', 'problem_in_column_name',
                'modifiable',
                'nr_inscrits', 'nr_not_inscrits', 'date_min', 'date_max',
                'nr_comments', 'teachers', 'nr', 'group_and_seq', 'grade_without_date')
        return ((code_ue, {key: getattr(stats, key) for key in keys})
                for code_ue, stats in self.tables.items())
