"""
Cell input scanner.

The computed dict is : User → Dict[table, nr_input]

Store results in: LOGINS/.../tables
"""

import collections
from TOMUSS import configuration
from TOMUSS import utilities
from TOMUSS.SCRIPTS import scanner_plugin

class Tables(scanner_plugin.Plugin):
    """Statistics on the grades entered by the teachers"""
    only_official_ue = False    # Not invisible tables
    only_suivi_semester = False # Not table not on the suivi page
    teachers = None

    def start(self):
        self.teachers = collections.defaultdict(lambda: collections.defaultdict(int))

    def stop(self):
        self.teachers = None

    def table(self, table):
        code_ue = table.ue
        teachers = self.teachers
        for line in table.lines.values():
            for cell in line:
                if len(cell.author) > 1:
                    teachers[cell.author][code_ue] += 1
        for column in table.columns:
            if len(column.author) > 1:
                teachers[column.author][code_ue] += 1

    def record(self):
        for code_ue, stats in self.teachers.items():
            yield code_ue, dict(stats)

    def merge(self, data):
        resume = collections.defaultdict(int)
        login = '' # To please pylint
        for (login, stats), year, semester, _ in data:
            year = int(year)
            for code_ue, nr_input in stats.items():
                resume[year, semester, code_ue] = nr_input
        user = configuration.User(login)
        user.manage_key('tables', content=utilities.stable_repr(resume))
        user.forget() # To not eat memory (not reused)
