"""
Table scanner to compute statistics

The computed dict is : Teacher → Dict[attribute, value]

"""

from TOMUSS.SCRIPTS import scanner_plugin
from TOMUSS import tablestat

class Teacher(scanner_plugin.Plugin):
    """Teacher stats"""
    teachers = None # type: Dict[List[str]]]
    only_real_file = True
    only_official_ue = False
    only_suivi_semester = False
    merge_manually = True
    def start(self):
        self.teachers = {}
    def stop(self):
        self.teachers = None

    def table(self, table):
        teachers = self.teachers
        for column in table.columns:
            user_name = column.author
            if user_name not in teachers:
                teachers[user_name] = tablestat.TableStat(user_name)
            teachers[user_name].nr_cols += 1
        for page in table.pages:
            if page.user_name not in teachers:
                teachers[page.user_name] = tablestat.TableStat(page.user_name)
            teachers[page.user_name].nr_pages += 1
        for line in table.lines.values():
            for cell in line:
                if cell.author not in teachers:
                    teachers[cell.author] = tablestat.TableStat(cell.author)
                teachers[cell.author].update(cell)

    def record(self):
        keys = ('nr_cols', 'nr_pages', 'date_min', 'date_max', 'nr_comments', 'nr')
        return ((code_ue, {key: getattr(stats, key) for key in keys})
                for code_ue, stats in self.teachers.items())
