"""
Compute statistiques per column type

The computed dict is : Type → int
"""

import collections
import os
from TOMUSS.SCRIPTS import scanner_plugin

class Type(scanner_plugin.Plugin):
    """Column type statistics"""
    only_official_ue = False
    only_suivi_semester = False
    types = None
    types_per_year = {}

    def start(self):
        self.types = collections.defaultdict(int)

    def stop(self):
        self.types = None

    def table(self, table):
        for datacol, column in enumerate(table.columns):
            if len(column.author) < 2:
                continue
            for line in table.lines.values():
                if line[datacol].value:
                    break
            else:
                continue # Empty column
            self.types[column.type.name] += 1

    def record(self):
        return self.types.items()

    def merge(self, data):
        per_year = collections.defaultdict(int)
        column_type = None # to please pylint
        for (column_type, nr_use), year, _semester, _ in data:
            per_year[year] += nr_use
        self.types_per_year[column_type] = per_year

    def finalize(self):
        years = set()
        for per_year in self.types_per_year.values():
            years.update(per_year)
        years = sorted(years)
        formater = '{}' + ',{}' * len(years)
        texts = []
        def add(key, data):
            texts.append(formater.format(key, *data))
        add('Type', years)
        add('----', ('----',) * len(years))
        for column_type in sorted(self.types_per_year,
                                  key=lambda x: -max(self.types_per_year[x].values())):
            per_year = self.types_per_year[column_type]
            add(column_type, [per_year.get(year, '') for year in years])
        txt = '\n'.join(texts)
        filename = os.path.join('TMP', 'xxx.type_usage.csv')
        with open(filename, 'w') as file:
            file.write(txt)
        print("Result in", filename)
        
