"""
Upload scanner

The computed dict is : Table → (seconds, mail_to, mail_subject, mail_content)
where 'seconds' is the time where the mail was sent
to indicate the future destruction of uploads (in current time + a constant).

The merging really destroy the uploads.
"""

import glob
import time
import os
import shutil
import html
from TOMUSS.SCRIPTS import scanner_plugin
from TOMUSS import configuration
from TOMUSS import utilities

KEEP = 2 # university year
DELETE_AFTER = 60 # days
UPLOAD_DIR = 'UPLOAD'

class Upload(scanner_plugin.Plugin):
    """Manage uploads cleanup"""
    only_official_ue = False
    only_suivi_semester = False
    merge_manually = True
    tables = None # type: Dict[str, Tuple[secs, mail_to, mail_subject, mail_content]]
    keep_after = None
    delete_after = None

    # XXX To remove in the future
    deprecated_file = deprecated_state = ()

    def start(self):
        """Retrieve previous start to get timestamps"""
        self.tables = self.get_state()
        self.keep_after = time.time() - KEEP * 86400 * 365
        self.delete_after = time.time() - DELETE_AFTER * 86400

        # XXX To remove in the future
        self.deprecated_file = os.path.join('TMP', 'clean_up_upload')
        if not self.tables and os.path.exists(self.deprecated_file): # pragma: no cover
            self.deprecated_state = utilities.literal_eval(
                utilities.read_file(self.deprecated_file))
    def stop(self):
        self.tables = None
    def send_mail_now(self, table):
        """Really send the mail now (when adding table, not when merging)"""
        y_s_ue = str(table)
        mail_content = [
            '<html>'
            '<head>'
            '<style>'
            'TABLE TD, TABLE TH, TABLE { border: 1px solid #888; vertical-align: top; }'
            'TABLE { border-spacing: 0px }'
            '</style>'
            '</head>',
            utilities._("MSG_clean_up_uploads").format(configuration.server_url,
                                                       y_s_ue, y_s_ue, DELETE_AFTER),
            '<table>'
            ]
        mails = set()
        for col in table.columns:
            if col.type.name == 'Upload': # pragma: no cover
                mail_content.append(
                    '<tr><th>{}<td>{}<td>{}<td><a href="{}">ZIP</a></tr>'.format(
                        html.escape(col.title),
                        html.escape(col.comment),
                        html.escape(col.author),
                        "{}/{}/upload_zip/{}/*/{}.zip".format(
                            configuration.server_url, y_s_ue, col.the_id,
                            utilities.safe(utilities.flat(y_s_ue + '/' + col.title)))
                        ))
                mails.add(col.author)
        mail_content.append('</table></html>')
        if mails: # pragma: no cover
            mails.update(table.masters)
            users = configuration.Users(mails)
            mail_to = [user.mail for user in users if user.mail]
        else:
            mail_to = configuration.maintainer
        mail_subject = y_s_ue + " " + utilities._('B_Upload')
        mail_content = '\n'.join(mail_content)
        utilities.send_mail(mail_to, mail_subject, mail_content, show_to=True)
        self.tables[table.ue] = (time.time(), mail_to, mail_subject, mail_content)

    def table(self, table):
        # XXX To remove in the future
        if str(table) in self.deprecated_state: # pragma: no cover
            self.tables[table.ue] = (self.deprecated_state[str(table)], '', '')

        name = os.path.join(UPLOAD_DIR, str(table.year), table.semester, table.ue)
        if table.ue in self.tables:
            # Mail yet sent
            if table.ue and self.tables[table.ue][0] < self.delete_after: # pragma: no cover
                # Sent a long time ago
                print("Delete:", name)
                del self.tables[table.ue]
                shutil.rmtree(name)
            return
        if not os.path.exists(name):
            return
        last = max(os.path.getmtime(os.path.join(name, col))
                   for col in os.listdir(name) + ['.'])
        if last < self.keep_after:
            self.send_mail_now(table)

    def tables_to_analyse(self):
        keep_after = time.time() - KEEP * 86400 * 365
        for name in glob.glob(os.path.join(UPLOAD_DIR, '*', '*', '*')):
            if os.path.getmtime(name) < keep_after:
                _, year, semester, code_ue = name.split(os.path.sep)
                if year.isdigit():
                    yield ('Y' + year, 'S' + semester, code_ue + '.py')

    def record(self):
        return self.tables.items()

    # XXX To remove in the future
    def done(self):
        if os.path.exists(self.deprecated_file): # pragma: no cover
            os.rename(self.deprecated_file, self.deprecated_file + '~')

    def merge(self, _data):
        return # pragma: no cover
