"""
Visibility scanner.

Resume per UE/columns : (title, course date, visibility date, input dates, column type)

The computed dict is : UE → List[Tuple[exam_date, visibility_date,List[grade_date]]

Store results in: TMP/xxx_visibilities.json
"""

import collections
from TOMUSS.SCRIPTS import scanner_plugin

class Visibility(scanner_plugin.Plugin):
    """Statistics on the grade visibility"""
    tables = None

    def start(self):
        self.tables = collections.defaultdict(list)

    def stop(self):
        self.tables = None

    def table(self, table):
        for column in table.columns:
            if column.type.name not in self.grading_types:
                continue
            if column.weight[0] in '+-':
                continue # pragma: no cover
            if column.visible(hide=1, line=None, login=None):
                visibility_date = max(
                    getattr(column, 'visibility_date__mtime', ''),
                    getattr(column, 'visibility__mtime', ''),
                    column.visibility_date + '000000' if column.visibility_date.isdigit() else '')
                if visibility_date == '000000' or not visibility_date.isdigit():
                    visibility_date = ''
            else:
                visibility_date = '0'
            input_dates = {}
            visibility = (column.title, column.course_dates, visibility_date,
                          input_dates, column.type.name)

            for login in table.the_keys():
                for line in table.get_lines(login):
                    cell = line[column.data_col]
                    if cell.date:
                        date = cell.date[:8]
                        if date in input_dates:
                            input_dates[date] += 1
                        else:
                            input_dates[date] = 1
            if input_dates or visibility[1]:
                # Only store if there is exam date or grades
                self.tables[table.ue].append(visibility)

    def record(self):
        return self.tables.items()

    def merge(self, data):
        pass
