// TOMUSS: The Online Multi User Simple Spreadsheet
// Copyright (C) 2023 Aurelyen MOREL, Universite Claude Bernard
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact: Thierry.EXCOFFIER@univ-lyon1.fr

function EditCatalog() {
    this.originalJumpFunct = Current.prototype.jump; //We put the function in a variable to store it
    this.originalAllowFunct = modification_allowed_on_this_line;
    this.running = false;
    this.update_filter = false;
    this.update_id = undefined;
}

EditCatalog.prototype.init = function () {
    this.reparse();
    this.btnPlus = document.createElement("BUTTON");
    this.btnPlus.id = "button_plus_bot";
    this.btnPlus.onclick = this.button_plus_bot_click.bind(this);
    document.body.appendChild(this.btnPlus);
};

EditCatalog.prototype.button_plus_bot_click = function (event) {
    switch (event.target.className) {
        case 'button_plus_bot': this.inser_popup(event); break;
        default: bug();
    }
};

EditCatalog.prototype.cell_set_value = function (lin, col, value) {
    this.running = true;
    cell_set_value_real(lin, col, value);
    this.running = false;
};

EditCatalog.prototype.reparse = function () {
    var dataTable = [];
    for (var i in lines)
        dataTable.push([lines[i][0].value, lines[i][1].value, lines[i][2].value, lines[i][3].value, lines[i][4].value, lines[i][5].value]);
    this.catalog = new Catalog();
    this.catalog.parse_lines(dataTable);
};

EditCatalog.prototype.button_insert_line_aggregation = function () {
    var lin = popup_line().line_id;
    var checkedFail = document.querySelectorAll(
        '#id_attribut_fail_list input[type="checkbox"]:checked');
    var listObservationFail = [];
    for (var i = 0; i < checkedFail.length; i++)
        listObservationFail.push(checkedFail[i].value);

    var checkedResult = document.querySelectorAll(
        '#id_attribut_result_list input[type="checkbox"]:checked');
    var listObservationResult = [];
    for (var i = 0; i < checkedResult.length; i++)
        listObservationResult.push(checkedResult[i].value);

    var desc = JSON.stringify({
        "method": document.getElementById("id_method_choice_select").value,
        "fail": listObservationFail,
        "possible_results": listObservationResult
    });

    this.cell_set_value(lin, 4, desc);
    table_fill(true, true, true);
    this.reparse();
    popup_close();
};

EditCatalog.prototype.add_aggregation_popup = function () {

    create_popup("button_add_aggregation", _("MSG_add_aggregation_popup"),
        "<STYLE>" +
        ".button_add_aggregation {top: 10%;max-height: 80%;}" +
        "</STYLE>" +
        "<br><br>" +
        "<p>" + _("MSG_aggregation_popup_explication") + "<p>" +
        "<br>" +
        '<div id="div_method_choice">' +
        "<p>" + _("MSG_method_choice") + "<p>" +
        '<div id="div_method_choice_select">' +
        '<select id="id_method_choice_select"></select>' +
        "</div>" +
        "</div>" +
        "<br>" +
        '<div id="id_attribut_choice">' +
        "<p>" + _("MSG_attribut_choice") + "</p>" +
        '<div id="id_attribut_content">' +
        '<div id="id_attribut_fail">' +
        '<p class="list_title_aggregation">' + _("MSG_list_fail_title") + '</p>' +
        '<div id="id_attribut_fail_list">' +
        '<ul id="list_attribut_fail" style="list-style-type:none;"></ul>' +
        "</div>" +
        "</div>" +
        '<div id="id_attribut_result">' +
        '<p class="list_title_aggregation">' + _("MSG_list_result_title") + '</p>' +
        '<div id="id_attribut_result_list">' +
        '<ul id="list_attribut_result" style="list-style-type:none;"></ul>' +
        "</div>" +
        "</div>" +
        "</div>" +
        "</div>" +
        "<br>" +
        '<button id="button_insert_line_aggregation">' + _("MSG_popup_save_button") + '</button>' +
        "<br>",
        "", false);
    var select_method = document.getElementById("id_method_choice_select");

    var method_max = document.createElement("OPTION");
    method_max.textContent = "Maximum";
    method_max.value = "maximum";

    var method_median = document.createElement("OPTION");
    method_median.textContent = "Median";
    method_median.value = "median";

    var method_avg = document.createElement("OPTION");
    method_avg.textContent = "Average";
    method_avg.value = "average";

    select_method.appendChild(method_max);
    select_method.appendChild(method_median);
    select_method.appendChild(method_avg);

    var listFail = document.getElementById("list_attribut_fail");
    var listResult = document.getElementById("list_attribut_result");
    var listObservation = this.catalog.vocabulary.o || [];

    for (var i = 0; i < listObservation.length; i++) {
        var option = listObservation[i];

        var liFail = document.createElement("li");
        var checkboxFail = document.createElement("input");
        checkboxFail.type = "checkbox";
        checkboxFail.value = "o" + i;
        checkboxFail.className = "class_checkbox_fail";

        var labelFail = document.createElement("label");
        labelFail.appendChild(checkboxFail);
        labelFail.appendChild(document.createTextNode(option));

        liFail.appendChild(labelFail);
        listFail.appendChild(liFail);

        var liResult = document.createElement("li");
        var checkboxResult = document.createElement("input");
        checkboxResult.type = "checkbox";
        checkboxResult.value = "o" + i;
        checkboxResult.className = "class_checkbox_result";

        var labelResult = document.createElement("label");
        labelResult.appendChild(checkboxResult);
        labelResult.appendChild(document.createTextNode(option));

        liResult.appendChild(labelResult);
        listResult.appendChild(liResult);
    }
    this.fill_popup_aggregation(popup_line()[0].value);
    document.getElementById("button_insert_line_aggregation").onclick =
        this.button_insert_line_aggregation.bind(this);
};

EditCatalog.prototype.fill_popup_aggregation = function (keys) {
    var currentMerge = this.catalog.merges[keys]
        || { method: 'maximum', 'fail': [], 'possible_results': [] };

    document.getElementById("id_method_choice_select").value = currentMerge.method;
    var listFail = currentMerge.fail;
    var checkBoxFail = document.getElementById("id_attribut_fail_list").querySelectorAll(".class_checkbox_fail");
    for (var i = 0; i < checkBoxFail.length; i++)
        checkBoxFail[i].checked = listFail.includes(checkBoxFail[i].value);

    var listResult = currentMerge.possible_results;
    var checkBoxResult = document.getElementById("id_attribut_result_list").querySelectorAll(".class_checkbox_result");
    for (var i = 0; i < checkBoxResult.length; i++)
        checkBoxResult[i].checked = listResult.includes(checkBoxResult[i].value);
};

EditCatalog.prototype.get_sorted_vocabularies = function () {
    var listTrie = [];
    for (var i in this.catalog.vocabulary_name)
        listTrie.push(i);
    listTrie.sort();
    return listTrie;
};

EditCatalog.prototype.get_index_of_vocabulary = function (voc) {
    return myindex(this.get_sorted_vocabularies(), voc);
};

EditCatalog.prototype.generate_key_discipline = function (divContainer) {
    var keyList = [];
    var vocElem = divContainer.querySelectorAll(".div_voc");
    for (var i = 0; i < vocElem.length - 1; i++) {
        var wordElem = vocElem[i].querySelectorAll(".div_select_word");
        var selectVocElem = vocElem[i].querySelector("#add_voc_select");
        var selectedVocOption = selectVocElem.options[selectVocElem.selectedIndex];
        keyList.push(selectedVocOption.getAttribute("vocabulary"));

        var listWord = [];
        for (var j = 0; j < wordElem.length - 1; j++) {
            var selectWordElem = wordElem[j].querySelector("#add_word_select")
            var selectedWordOption = selectWordElem.options[selectWordElem.selectedIndex];
            listWord.push(selectedWordOption.getAttribute("descMotIndex"));
        }
        keyList.push(listWord);
    }
    return keyList.join("");
};

EditCatalog.prototype.buttons_option_show_errors_popup = function () {
    create_popup("button_show_errors", _("MSG_popup_show_errors"),
        "<br><br>" +
        '<p>' + _("MSG_nb_errors") + this.catalog.errors.length + '</p>' +
        "<br><br>" +
        "<p>" + _("MSG_errors_details") + "</p>" +
        "<br><br>" +
        '<ul id="errorList"></ul>' +
        "<br>",
        "", false);
    var errorListElement = document.getElementById("errorList");
    for (var i in this.catalog.errors) {
        var error = this.catalog.errors[i];
        var errorMessage = error[0] + " " + error[1];
        var listItemElement = document.createElement("LI");
        listItemElement.textContent = errorMessage;
        errorListElement.appendChild(listItemElement);
    }
};

EditCatalog.prototype.inser_popup = function () {
    create_popup("button_plus_top", _("MSG_insert_line_popup"),
        "<br><br>" +
        "<p>" + _("MSG_voc_choice") +
        ' «' + html(the_current_cell.line[1].value) + "»</p>" +
        "<br>" +
        '<select id="vocabulary_select" onchange="editCatalog.add_new_voc()"></select>' +
        '<div id="add_voc"></div>' +
        "<br><br>" +
        '<p for="recherche">Description :</p>' +
        "<br>" +
        '<input type="text" id="recherche" style="width: 80%;" list="suggestions"' +
        'oninput="editCatalog.update_filter = true">' +
        '<datalist id="suggestions"></datalist>' +
        "<br><br>" +
        '<button id="button_inser">' +
        _("B_insert_line") + "</button>",
        "", false);
    document.getElementById("recherche").setAttribute("placeholder", _("MSG_new_desc"));
    this.fill_select_vocabulary();
    this.add_new_voc();
    this.update_id = setInterval(this.try_suggest.bind(this), 100);
    document.getElementById("button_inser").onclick = function () {
        editCatalog.add_line(document.getElementById("recherche").value);
    }
};

EditCatalog.prototype.fill_select_vocabulary = function () {
    var key = key_split(popup_line()[0].value);
    var vocList = {};
    for (var j = 0; j < key.length; j++)
        vocList[key[j][0]] = true;

    var select = document.getElementById("vocabulary_select");
    select.innerHTML = "";

    var listTrie = this.get_sorted_vocabularies();
    for (var voc in listTrie) {
        var vocabulary = document.createElement("OPTION");
        vocabulary.textContent = listTrie[voc] + ", " + this.catalog.vocabulary_name[listTrie[voc]];
        vocabulary.vocabulary_key = listTrie[voc];
        vocabulary.value = listTrie[voc];
        if (vocList[listTrie[voc]]
            || listTrie[voc] === "o" && popup_line()[0].value !== "")
            vocabulary.setAttribute("disabled", "true");
        select.appendChild(vocabulary);
    }
    var vocabulary = document.createElement("OPTION");
    vocabulary.textContent = _("MSG_add_new_voc");
    vocabulary.id = "opt_add_new_voc";
    select.appendChild(vocabulary);
};

EditCatalog.prototype.add_new_voc = function () {
    var voca_selected = document.getElementById("vocabulary_select").value;

    if (voca_selected === _("MSG_add_new_voc")) {
        var inputVoc = document.createElement("SELECT");
        inputVoc.id = "input_new_voc";
        for (var i = 'a'; i <= 'z'; i = String.fromCharCode(i.charCodeAt() + 1)) {
            var option = document.createElement("OPTION");
            option.value = i;
            option.text = i;
            if (this.catalog.vocabulary_name[i]
                || i === "o" && popup_line()[0].value !== ""
                || i === "z") {
                option.setAttribute("disabled", "true");
            }
            inputVoc.appendChild(option);
        }
        document.getElementById("add_voc").appendChild(inputVoc);

        var inputVocDesc = document.createElement("INPUT");
        inputVocDesc.id = "input_new_desc_voc";
        inputVocDesc.setAttribute("type", "text");
        inputVocDesc.setAttribute("placeholder", _("MSG_desc_new_voc"));
        inputVocDesc.setAttribute("size", "40");
        inputVocDesc.setAttribute("oninput", "editCatalog.button_new_voc()");
        document.getElementById("add_voc").appendChild(inputVocDesc);
    } else if (document.getElementById("input_new_voc")) {
        document.getElementById("input_new_voc").remove();
        document.getElementById("input_new_desc_voc").remove();
        if (document.getElementById("button_save_voc")) {
            document.getElementById("button_save_voc").remove();
        }
    }
};

EditCatalog.prototype.button_new_voc = function () {
    if (document.getElementById("button_save_voc"))
        document.getElementById("button_save_voc").textContent = _("B_save_new_voc");
    else {
        var saveBtn = document.createElement("BUTTON");
        saveBtn.id = "button_save_voc";
        saveBtn.textContent = _("B_save_new_voc");
        saveBtn.onclick = this.save_new_voc.bind(this);
        document.getElementById("add_voc").appendChild(saveBtn);
    }
};

EditCatalog.prototype.save_new_voc = function () {
    var voc = document.getElementById("input_new_voc").value;
    var vocDesc = document.getElementById("input_new_desc_voc").value;
    var error = this.catalog.update_vocabulary(voc, vocDesc);

    if (error > 0) {
        var lin = add_a_new_line();
        this.cell_set_value(lin, 0, "#" + voc);
        this.cell_set_value(lin, 1, vocDesc);
    } else if (error === 0) {
        for (var i in lines)
            if (lines[i][0].value === "#" + voc)
                this.cell_set_value(i, 1, vocDesc);
    }
    else {
        alert(_("MSG_comp_error" + error));
        return;
    }
    document.getElementById("input_new_voc").remove();
    document.getElementById("input_new_desc_voc").remove();
    document.getElementById("button_save_voc").remove();
    this.fill_select_vocabulary();
    document.getElementById("vocabulary_select").value = voc;
};

EditCatalog.prototype.desc_in_catalog = function (voc, desc) {
    return this.catalog.vocabulary[voc].includes(desc);
};

EditCatalog.prototype.add_line = function (desc) {
    var voc_element = document.getElementById("vocabulary_select");
    var voc;
    var index;
    var lin;

    if (voc_element.value === _("MSG_add_new_voc"))
        voc = document.getElementById("input_new_voc").value;
    else
        voc = voc_element.options[voc_element.selectedIndex].vocabulary_key;

    if (this.desc_in_catalog(voc, desc) === false)
        index = this.catalog.add_description(voc, desc);
    else {
        if (document.getElementsByClassName("option_suggestion")[0] !== undefined)
            index = document.getElementsByClassName("option_suggestion")[0].description_index;
        else
            index = 0;
    }
    if (index < 0) {
        alert(_('MSG_comp_error' + index));
        return 0;
    }
    if (popup_line()[0].value[0] === "#")
        var key = voc + index;
    else
        var key = popup_line()[0].value + voc + index;

    for (var i in lines) {
        if (key === lines[i][0].value) {
            alert(_("MSG_error_key_alr_exists"));
            return 0;
        }
    }
    lin = add_a_new_line();
    this.cell_set_value(lin, 0, key);
    this.cell_set_value(lin, 1, desc);
    this.catalog.choices_add(key);
    table_fill(true, true, true);
    this.z0_delete(voc + index);
    this.reparse();
    popup_close();
};

EditCatalog.prototype.z0_delete = function (voc) {
    var key = "z0" + voc;
    for (var i in lines) {
        if (key === lines[i][0].value) {
            this.cell_set_value(i, 0, "");
            this.cell_set_value(i, 1, "");
        }
    }
};

EditCatalog.prototype.try_suggest = function () {
    if (popup_is_open()) {
        if (this.update_filter) {
            this.desc_suggestion();
            this.update_filter = false;
        }
    } else {
        clearInterval(this.update_id);
        this.update_id = undefined;
    }
}
EditCatalog.prototype.desc_suggestion = function () {
    var voc_element = document.getElementById("vocabulary_select");
    var voc = voc_element.options[voc_element.selectedIndex].vocabulary_key;
    var datalist = document.getElementById("suggestions");
    var completion = this.catalog.get_completion(voc, document.getElementById("recherche").value);

    datalist.innerHTML = "";
    for (var i in completion) {
        var option = document.createElement("OPTION");
        option.className = "option_suggestion";
        option.value = completion[i][2];
        option.description_index = completion[i][1];
        datalist.appendChild(option);
    }
};

EditCatalog.prototype.delete_line = function (key, lin) {
    // XXX To check
    var count = 0;
    var lastKey = key_last_word(key).join("");
    for (var i in lines) {
        var currentKey = key_last_word(lines[i][0].value).join("");
        if (lastKey === currentKey)
            count += 1;
    }
    if (count === 1) {
        this.catalog.choices_del(key);
        lastKey = "z0" + lastKey;
        this.cell_set_value(lin, 0, lastKey);

    } else {
        this.catalog.choices_del(key);
        this.cell_set_value(lin, 0, "");
    }
    table_fill(true, true, true);
    this.reparse();
    return count > 1;
};

EditCatalog.prototype.update_desc = function (key, desc) {
    var word = key_last_word(key);
    key = word.join("");
    index = this.catalog.replace_description(word[0], word[1], desc);
    if (index < 0) {
        alert(_("MSG_comp_error" + index));
        return;
    }
    for (var i in lines) {
        var currentWord = key_last_word(lines[i][0].value);
        var currentKey = currentWord.join("");
        if (key === currentKey)
            this.cell_set_value(i, 1, desc);
    }
    table_fill(true, true, true);
    this.reparse();
};

function competences_update_student_information(_line) {
    if (!t_student_picture.parentNode)
        return;

    var parent_attributs = t_student_picture.parentNode.attributes;
    t_student_picture.parentNode.removeAttribute(parent_attributs[1].name);
    t_student_picture.parentNode.removeAttribute(parent_attributs[0].name);
    t_student_picture.parentNode.innerHTML =
        '<div style="font-size:60%;width:20em;display: grid;">' +
        '<button id="button_show_errors" class="button_options" onclick="editCatalog.buttons_option_show_errors_popup()">' +
        _("B_show_errors") + '</button></div>';

    var button_show_errors = document.getElementById("button_show_errors");
    if (editCatalog.catalog.errors.length === 0) {
        button_show_errors.setAttribute("disabled", "true");
        button_show_errors.style.cursor = "default";
    } else
        button_show_errors.style.color = "red";
};

function competences_modification_allowed_on_this_line(data_lin, data_col, value) {
    if (editCatalog.running || alert_merged !== false)
        return true;
    if (data_col === 0)
        return false; // Never allow key modification
    if (data_col > 1) {
        // Formation and UE
        setTimeout(editCatalog.reparse.bind(editCatalog), 100);
        return true;
    }
    var key = lines[data_lin][0].value;
    switch (key[0]) {
        case undefined: return false; // No key on the line
        case 'z': break;
        case 'o':
            if (value === '')
                return false; // Not allowed to delete an observation
            break;
        case '#':
            var error = editCatalog.catalog.update_vocabulary(key[1], value);
            if (error >= 0)
                return true; // allowed for desc of a vocabulary
            alert(_("MSG_comp_error" + error));
            return false;
    }
    // if (data_lin !== the_current_cell.line.line_id) return true; // When a z0 key needs to be deleted after a line is added
    var allow_erase = true;
    editCatalog.running = true;
    if (value === "")
        allow_erase = editCatalog.delete_line(lines[data_lin][0].value, data_lin);
    else
        editCatalog.update_desc(lines[data_lin][0].value, value);
    editCatalog.running = false;
    return allow_erase;
};

const COMPETENCE_CLASSES = {
    'z': 'button_plus_hide',
    '#': 'button_plus_hide'
};
function competences_jump(lin, col, do_not_focus, line_id, data_col) {
    editCatalog.originalJumpFunct.call(this, lin, col, do_not_focus, line_id, data_col);
    if (this.auto_select_tab)
        select_tab('cellule', _('TAB_cell'));
    editCatalog.btnPlus.style.left = table.rows[0].cells[0].offsetWidth - editCatalog.btnPlus.offsetWidth - 5 + 'px';
    editCatalog.btnPlus.className = COMPETENCE_CLASSES[the_current_cell.line[0].value[0]] || 'button_plus_bot';
    editCatalog.btnPlus.style.top =
        the_current_cell.input.parentNode.offsetTop +
        (editCatalog.btnPlus.className == 'button_plus_bot'
            ? the_current_cell.input.offsetHeight - editCatalog.btnPlus.clientHeight / 2
            : the_current_cell.input.offsetHeight / 3) + 'px';
    if (data_col === 4)
        editCatalog.add_aggregation_popup();
};

var editCatalog = new EditCatalog();

function template_init() {
    editCatalog.init();
    modification_allowed_on_this_line = competences_modification_allowed_on_this_line;
    update_student_information = competences_update_student_information;
    Current.prototype.jump = competences_jump;
}
