#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2020 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

from .. import configuration
from .. import display
from .. import utilities

i_am_unique = (0, 'Dossiers')

columns = {
    'k': {'type': 'Text',
          'title': utilities._("Name"),
          'width': 10, 'position': 10
         },
    'v': {'type': 'Note',
          'title': utilities._("SELECT_hiddens"),
          'red': '!=1',
          'width': 1, 'minmax': '[0;1]', 'rounding': 1, 'position': 5
         },
    }

def create(table):
    ro = table.get_ro_page()
    table.get_a_root_page()
    table.table_attr(ro, 'masters', list(configuration.root))
    table.table_attr(ro, 'default_nr_columns', 2)
    table.table_attr(ro, 'default_sort_column', [0,1])
    table.update_columns(columns, ro)

def onload(table):
    table.contains_users = 0
    if not configuration.read_only:
        rw = table.pages[1]
        table.lock()
        try:
            for d in display.display_dict:
                if d not in table.lines:
                    table.cell_change(rw, 'k', d, d)
                    table.cell_change(rw, 'v', d, 1)
        finally:
            table.unlock()
    for lin_id, line in table.lines.items():
        if lin_id in display.display_dict:
            display.display_dict[lin_id].enabled = line[1].value == 1

def cell_change(table, page, col, lin_id, value, _date):
    """Update all the link attributes"""
    if page.page_id < 2:
        return
    if col == 'v':
        display.display_dict[lin_id].enabled = value == 1
        display.init()
        table.unload_on_suivi()
