#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2010-2012 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

from .. import configuration
from .. import plugin
from .. import plugins
from .. import utilities
from .. import sender
from .. import data

i_am_unique = (0, 'Dossiers')

def update_column(table):
    p = table.pages[0]
    for name, width in (
            ('plugin', 4), ('info', 10), ('link', 3), ('suivi', 1), ('invited', 20)):
        table.column_attr(p, name, 'type', 'Text')
        table.column_attr(p, name, 'width', width)
        table.column_comment(p, name, utilities._('COL_TITLE_cp_' + name))
    table.table_attr(p, 'default_nr_columns', 5)

def create(table):
    p = table.get_a_root_page()
    table.table_attr(p, 'masters', list(configuration.root))
    update_column(table)
    check(table, from_create=True)

def init(table):
    table.default_sort_column = [2,0]
    table.private = 1
    table.contains_users = 0
    table.do_not_unload_add('*config_plugin')

def check(table, from_create=False):
    if not from_create:
        table.lock()

    d =  {None: '',True: configuration.yes,False:configuration.no}

    try:
        tomuss = set(p.name for p in plugin.plugins)
        suivi = set(p.name for p in plugins.suivi_plugins)
        done = set()
        for p in plugin.plugins + plugins.suivi_plugins:
            done.add(p.name)
            if p.name not in table.lines:
                table.cell_change(table.pages[1], 'invited', p.name,
                                  '("grp:' + p.group + '",)')
            table.cell_change(table.pages[0], 'plugin', p.name, p.name)
            if p.documentation:
                table.cell_change(table.pages[0], 'info'  , p.name,
                                  p.documentation)
            if p.link:
                table.cell_change(table.pages[0], 'link', p.name, p.link.where)

            is_in = ''
            if p.name in tomuss:
                is_in += 'T'
            if p.name in suivi:
                is_in += 'S'
                        
            table.cell_change(table.pages[0], 'suivi', p.name, is_in)

        # Make writable old plugins
        for p, line in table.lines.items():
            if p in done:
                continue
            for column in table.columns: # pragma: no cover
                if line[column.data_col].author != data.ro_user:
                    continue
                table.cell_change(table.pages[1], column.the_id, p,
                                  line[column.data_col].value,
                                  force_update=True)
    finally:
        if not from_create:
            table.unlock()

def onload(table):
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
         # To close a Security Hole
        for p in plugin.plugins + plugins.suivi_plugins:
            p.invited = ('grp:' + p.group,)
        return
    c = table.columns.from_id('invited').data_col
    for p in plugin.plugins + plugins.suivi_plugins:
        try:
            value = tuple(table.get_lines(p.name))[0][c].value or '()'
        except IndexError: # pragma: no cover
            # Was not in the table
            value = '("grp:' + p.group + '",)'
        try:
            p.invited = utilities.literal_eval(value)
        except: # pragma: no cover
            utilities.send_backtrace('config_plugin: ' + str(p), exception=False)

def cell_change(table, page, col, lin, value, dummy_date):
    if configuration.regtest and not configuration.real_regtest: # pragma: no cover
        return # Security Hole
    if col != 'invited':
        return

    plugin_name = table.lines[lin][0].value
    for p in plugin.plugins + plugins.suivi_plugins:
        if plugin_name != p.name:
            continue
        if value.strip() == '':
            p.invited = ()
            continue
        try:
            p.invited = utilities.literal_eval(value)
        except:
            sender.append(page.browser_file,
                          '<script>alert("Error!");</script>')
            utilities.send_backtrace('config_plugin')
            raise

    if page.page_id > 1:
        table.unload_on_suivi()
