#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2018 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

import html
from .. import data
from .. import utilities
from .. import configuration

i_am_unique = (0, 'Dossiers')

default_priority = 10
default_nr_items = 2

columns = {
    '0': {'type': 'Note', 'freezed': 'F', "width": 1,
          'empty_is': default_priority,
          'title': utilities._("COL_TITLE_rss_priority"),
          'comment': utilities._("COL_COMMENT_rss_priority"),
          },
    '1': {'type': 'Note', 'freezed': 'F', "width": 1,
          'empty_is': default_nr_items, 'minmax': '[-10;10]',
          'title': utilities._("COL_TITLE_rss_nr"),
          'comment': utilities._("COL_COMMENT_rss_nr"),
          },
    '2': {'type': 'Text', 'freezed': 'F', "width": 4,
          'title': utilities._("COL_TITLE_rss_group"),
          'comment': utilities._("COL_COMMENT_rss_group"),
          },
    '3': {'type': 'Text', "width": 8,
          'title': utilities._("COL_TITLE_rss_title"),
          'comment': utilities._("COL_COMMENT_rss_title"),
          },
    '4': {'type': 'Text', "width": 8,
          'title': utilities._("COL_TITLE_rss_url"),
          'comment': utilities._("COL_COMMENT_rss_url"),
          },
    }

def create(table):
    ro = table.get_a_root_page()
    p = table.get_nobody_page()
    table.table_attr(ro, 'masters', list(configuration.root))
    table.table_attr(ro, 'default_nr_columns', 5)
    table.table_attr(ro, 'default_sort_column', [0,2])
    table.table_attr(ro, 'table_title',
                     utilities._('LINK_/0/Dossiers/config_suivi_rss'))
    table.update_columns(columns, p)

@utilities.add_a_cache
def parse_rss(url_nr):
    url, nr = url_nr
    try:
        c = utilities.read_url(url)
    except IOError:
        return [['Bug!', '', '']]
    c = c.split('<item>')[1:]
    data= []
    if nr > 0:
        c = c[-nr:]
    else:
        c = c[:-nr]
    for item in c:
        titre = html.unescape(item.split('<title>')[1].split('<')[0])
        if '<description>' in item:
            description = html.unescape(item.split('<description>')[1].split('<')[0])
        else:
            description = '' # pragma: no cover
        link = item.split('<link>')[1].split('<')[0]
        data.append((titre, description, link))
    return data

class SuiviRSS:
    def __init__(self, table):
        self.table = table
    def get_suivi_rss(self, login):
        self.table.update()
        html = []
        for line in self.table.lines.values():
            group = line[2].value
            if group and line[1].value != 0:
                if configuration.is_member_of(login, group):
                    priority = line[0].value or default_priority
                    title = line[3].value
                    url = line[4].value
                    nr_items = int(line[1].value or default_nr_items)
                    html.append([priority, title, parse_rss((url, nr_items))])
        html.sort()
        return html

def onload(table):
    table.contains_users = 0
    configuration.get_suivi_rss = SuiviRSS(table).get_suivi_rss
