#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2009-2013 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

from .. import utilities
from .. import configuration
from .. import sender
from .. import files

i_am_unique = (0, 'Dossiers')

def create(table):
    p = table.get_ro_page()
    _ = utilities._
    table.table_attr(p, 'masters', list(configuration.root))
    table.column_change(p,'0_0',_("COL_TITLE_ct_variable"),'Text','','','F',0,2)
    table.column_change(p,'0_1',_("COL_TITLE_ct_what")  ,'Text','','','F',0,10)
    table.column_change(p,'0_2',_("COL_TITLE_ct_value") ,'Text','','','F',0,10 )
    table.table_attr(p, 'default_nr_columns', 3)
    table.table_attr(p, 'default_sort_column', [1])
    table.table_attr(p, 'private', 1)
    table.get_a_root_page()

variable_list = [
    "do_not_display",
    'message',
    "year_semester",
    "year_semester_next",
    "ue_not_per_semester",
    "master_of_exceptions",
    "allow_student_list_update",
    "allow_student_removal",
    'maintainer',
    'smtpserver',
    'abj_sender',
    'ldap_server',
    'ldap_server_login',
    "ldap_server_password",
    "ldap_server_port",
    "ldap_encoding",
    "attr_login",
    "attr_login_alt",
    "attr_mail",
    "attr_surname",
    "attr_firstname",
    "attr_default_password",
    "ou_top",
    "ou_students",
    "cn_students",
    "cn_teachers",
    "ou_groups",
    "ou_ue_contains",
    "ou_ue_starts",
    "ou_ue_starts2",
    "ou_portail_contains",
    "banned_ip",
    "root",
    "teachers",
    "teacher_if_login_contains" ,
    "administratives",
    "abj_masters",
    "referents",
    "invited_teachers",
    "invited_administratives",
    "invited_abj_masters",
    'students_check_interval',
    'maximum_out_of_date',
    'maxage',
    "ticket_time_to_live",
    "unload_interval",
    "not_teachers",
    "login_not_teacher",
    "logo",
    "suivi_display_more_ue",
    "language",
    "suivi_student_message",
    "tt_masters",
    "year_semester_modifiable",
    "year_semester_update_student_list",
    "suivi_student_allow_private",
    "max_visibility_date",
    "gui_record",
    "time_between_mails",
    "grp_modifiable",
    "advertising",
    "removal_allowed",
    "upload_max",
    "single_logout",
    "special_days",
    "allowed_grades",
    "allow_picture_upload",
    "show_teacher_pictures",
    "allow_teacher_picture_upload",
    "private_suivi",
    "forbidden_names",
    "rounding_default", "rounding_avg", "rounding_min",
    "clamav_server", "clamav_port",
    "backtrace_times_span", "backtrace_times_number", "backtrace_times_url",
    "backtrace_times_stop",
    "display_abj_comment",
    'hide_empty_table',
    'share_duration',
    'display_sequence_column',
    'intro_message',
    'rgpd_link',
    'exceptions',
    'abi2abj',
    ]

deprecated = set(('root', 'invited_teachers', 'invited_administratives',
                  'invited_abj_masters', 'tt_masters', 'teachers',
                  'administratives', 'abj_masters',
                  'referents', 'ldap_reconnect', 'abinj', "authenticate_iframe"))

variables = {}

def check(table):
    p_ro = table.pages[0]
    p_rw = table.pages[1]

    table.lock()
    try:
        for variable, comment in variables.items():
            if variable not in table.lines:
                if variable in deprecated:
                    continue
                else:
                    # Do not change user entered value
                    # Do not change default value after the first time.
                    v = configuration.__dict__[variable]
                    if isinstance(v, list) or isinstance(v, tuple):
                        if len(v) == 0:
                            v = '()'
                        else:
                            v = '('+','.join([utilities.js(i) for i in v])+',)'
                    elif isinstance(v, (bool, dict)):
                        v = repr(v)
                    table.cell_change(p_ro,'0_0', variable,variable)
                    table.cell_change(p_rw, '0_2', variable, v)
            if variable in deprecated and variable in table.lines:
                table.cell_change(p_ro, '0_1', variable,
                                  utilities._("COL_TITLE_ct_deprecated")) # pragma: no cover
            else:
                table.cell_change(p_ro, '0_1', variable, comment)
    finally:
        table.unlock()

def set_value(variable, value):
    current = configuration.__dict__[variable]

    if isinstance(current, tuple) or isinstance(current, list) \
             or isinstance(current, bool) or isinstance(current, dict):
        value = utilities.literal_eval(value)
        if not isinstance(value, type(current)): # pragma: no cover
            utilities.warn("VARIABLE=%s CURRENT=%s NEW=%s" %
                 (variable, current, value), what="error")
            raise ValueError("Big issue")
    elif isinstance(current, float):
        value = float(value)
    elif isinstance(current, int):
        value = int(value)
    configuration.__dict__[variable] = value
    if variable == 'logo':
        files.files['style.css'].replace("config_table", '_LOGO_', value)

def init(table):
    table.contains_users = 0
    table.do_not_unload_add('*config_table')

    for v in variable_list:
        variables[v] = utilities._("config_table_" + v)
    variables.update(configuration.local_options)
    
def onload(table):
    """Copy table content into configuration"""

    if len(table.lines) == 0:
        return
    for variable in variables:
        if variable in table.lines: # do not create it by reading it
            set_value(variable, table.lines[variable][2].value)


def cell_change(table, page, col, lin, value, _date):
    if page.page_id <= 1:
        return
    if col != '0_2':
        return # pragma: no cover
    try:
        set_value(lin, value)
    except:
        import sys
        sender.append(page.browser_file,
                      '<script>alert(%s);</script>' %
                      utilities.js(str(sys.exc_info()[0])))
        raise
    configuration.config_acls_clear_cache()
    table.unload_on_suivi(clear='ACLS')
