#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    Copyright (C) 2010-2012 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import sys
import os
import atexit
import gc
import json

package_name = "TOMUSS" # Choose your package name (for absolute import)

for i in range(10):
    try:
        caller_globals = sys._getframe(i).f_globals
        if caller_globals.get('__package__', '?') is None:
            # Needed when launched by python3-coverage
            break
    except ValueError: # pragma: no cover
        break



# Only the __main__ must become a package, not the other modules.
if caller_globals["__name__"] in ('__main__', 'coverage.cmdline'):
    # Search package root: its container directory does not have an __init__.py
    current_dir = __file__.split(os.path.sep)
    if len(current_dir) == 1:
        current_dir = os.getcwd().split(os.path.sep) # pragma: no cover
    else:
        # Remove filename
        current_dir.pop()
    local_name = []
    while os.path.exists(os.path.sep.join(current_dir + ['__init__.py'])):
        local_name.insert(0, current_dir.pop())
    # Import the package root (else relative imports fails)
    sys.path.insert(0, os.path.sep.join(current_dir))
    try:
        sys.modules[package_name] = __import__(local_name[0])
    except ImportError: # pragma: no cover
        # Here if the directory name contains dots
        try:
            import importlib
            open_file, file_name, description = importlib.find_module(local_name[0])
            sys.modules[package_name] = importlib.load_module(local_name[0],
                                                              open_file, file_name,
                                                              description)
        except AttributeError:
            import imp
            open_file, file_name, description = imp.find_module(local_name[0])
            sys.modules[package_name] = imp.load_module(local_name[0],
                                                        open_file, file_name,
                                                        description)

    sys.modules[package_name].__name__ = 'TOMUSS'
    # The python path can be reset to the initial value
    sys.path.pop(0)

    if 'nowarn' in sys.argv:
        from TOMUSS import utilities
        def log_nothing(*_args, **_kargs):
            return
        utilities.warn = log_nothing
        sys.argv.remove('nowarn')

    # The local top level directory is renamed to the desired package name.
    current_dir.append(local_name[0])
    local_name[0] = package_name

    # Set the good package name (eg. TOMUSS or TOMUSS.YYY or TOMUSS.YYY.TTT)
    __package__ = caller_globals["__package__"] = '.'.join(local_name)

    if len(local_name) != 1:
        # To fix absolute import: load the directory containing the script
        __import__('.'.join(local_name))

    # Some old table data file contains 'from data import *'
    # in place of 'from TOMUSS.data import *'
    # This link fixes the problem
    for i in ('data', 'abj', 'teacher'):
        try:
            __import__(package_name + '.' + i)
            sys.modules[i] = sys.modules[package_name + '.' + i]
        except ImportError: # pragma: no cover
            pass

    from TOMUSS import configuration
    from TOMUSS import entity

    for arg in ('regtest', 'regtest_sync', 'regtest_load_local', 'blur', 'regtest_bug1',
                'search_leak'):
        if arg in sys.argv:
            setattr(configuration, arg, True)
            sys.argv.remove(arg)
            configuration.removed_args.append(arg)
        else:
            setattr(configuration, arg, False)

    configuration.Users = entity.Users
    configuration.User = entity.User

    def terminate_init():
        # Not in a TOMUSS server but in a script in a direct subdirectory
        os.chdir(os.path.sep.join(current_dir))
        entity.init('tomuss.py' not in sys.argv[0])
        sys.stderr.write("*"*50 + '\n')
        sys.stderr.write("BEWARE The current directory is now %s\n"
                         % os.getcwd())
        sys.stderr.write("*"*50 + '\n')
        sys.stderr.write("regtest={} regtest_real={}".format(
            configuration.regtest, configuration.real_regtest))
        if 'REGTEST_SERVER' in __package__ or 'PYTHON_JS' in __package__:
            configuration.regtest = True
        if configuration.regtest:
            from TOMUSS import regtestpatch
            regtestpatch.do_patch()

        configuration.terminate()
        sys.stderr.write('DB=%s\n' % configuration.db)

        from TOMUSS import plugins
        plugins.load_types()

        from TOMUSS import column
        column.initialize()

        from TOMUSS import utilities
        utilities.init(launch_threads=False)

        from TOMUSS import document
        document.table(0, 'Dossiers', 'config_table', None, None)

    if ('.' in __package__
        and not sys.modules["__main__"].__dict__.get("fast_tomuss_init", 0)
        ):
        terminate_init()
    else:
        entity.init('tomuss.py' not in sys.argv[0])
        if not configuration.real_regtest:
            atexit.register(os._exit, 0) # Do not run gc.collect on exit # pragma: cover # pylint: disable=protected-access
        gc.collect(2)
        gc.set_threshold(0) # Disable garbage collection because they are no circular reference

        dumps = json.dumps
        def new_dumps(*args, **kargs):
            "Fix a leak in simplejson package"
            result = dumps(*args, **kargs)
            gc.collect(0)
            return result
        json.dumps = new_dumps
