#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2008-2019 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import time
import os
import glob
from . import utilities
from . import configuration

files = {}

def update_one_static_file():
    for f in utilities.failsafe_tuple(files.values):
        try:
            yield f.update_content()
        except FileNotFoundError:
            pass # Some not yet created files as stats pictures
    yield update_version()

def update_static_files():
    while not hasattr(configuration, 'initialisations_done'):
        time.sleep(0.1)
    t = time.time() # Relaunch update if modifications after this time
    for _ in update_one_static_file():
        pass
    return t

def update_version():
    configuration.version = '{}@{}'.format(
        configuration.version_real,
        int(max(os.path.getmtime(f.name) if os.path.exists(f.name) else 0
                for f in utilities.failsafe_tuple(files.values)
               )))
    configuration.url_files = configuration.server_url + '/files/' + configuration.version

def add(*args, cached=True):
    filename = args[-1]
    old = files.get(filename, None)
    files[filename] = utilities.StaticFile(os.path.join(*args), cached=cached)
    if old is not None:
        files[filename].append_text = old.append_text
        files[filename].replace_text = old.replace_text
        files[filename].clear_cache()
    utilities.start_job(update_static_files, 1)
    return files[filename]

def append(filename, key, content):
    """'append' can by called before 'add'"""
    if filename not in files:
        files[filename] = utilities.StaticFile(os.path.join("FILES",
                                                            "bad.png"))
    files[filename].append(key, content)
    utilities.start_job(update_static_files, 1)

for name in (
    'style.css', 'display.css', 'display.js',
    'favicon.ico', 'logo.png',
    'header.png',
    'verysafe.png', 'safe.png', 'unsafe.png', 'veryunsafe.png',
    'tip.png', 'feed.png', 'news.xml', 'butterflynet.png',
    'ok.png', 'bad.png', 'bug.png', 'abjus.png',
    'weight.png',
    'close.png', 'cell_comment.png',
    'up.gif', 'down.gif', 'next.gif', 'prev.gif',
    'tt.png',
    'lib.js', 'abj.js','types.js','utilities.js',
    'middle.js',
    'abj.html',
    'robots.txt',
    'doc_table.html', 'documentation.js', 'documentation.css',
    'stats.html',
    'howto.html',
    'printemps.png', 'automne.png'
    ):
    add('FILES', name)

for name in (
    'all_ues.js',
    'premier_cours.html',
    ):
    add('TMP', name)

for name in glob.glob('FILES/doc-*.png'):
    name = name.split('/')[-1]
    add('FILES', name, cached=False)

files['lib.js'].append('files.py',
                       utilities.wait_scripts()
                       + '\nvar semesters = '
                       + utilities.js(configuration.semesters) + ';\n'
                       + '\nvar semesters_year = '
                       + utilities.js(configuration.semesters_year) + ';\n'
                       + '\nvar semesters_months = '
                       + utilities.js(configuration.semesters_months) + ';\n'
                       + '\nvar semesters_color = '
                       + utilities.js(configuration.semesters_color) + ';\n'
                       )

add('PYTHON_JS', 'tomuss_python.js')
files['utilities.js'].append('files.py', files['tomuss_python.js'])
