#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet)
#    Copyright (C) 2008-2016 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

"""
Modify the application in order to support regression tests.
"""

import sys
from . import configuration
from . import inscrits
from . import referent
from . import utilities

if "protect_do_not_display" in sys.argv: # pragma: no cover
    configuration.do_not_display = ('debug', 'auth', 'table', 'ldap', 'plugin',
                                    'check', 'lang', 'DNU', 'info')

def etapes_list(items, args=('etapes_list',)):
    """Etapes list"""
    attr = ' '.join(args)
    for i in items:
        setattr(i, attr, ('etape-' + i.ID_,))

def firstname(items):
    """Get surname and firstname"""
    for i in items:
        if i.ID_ == 'toto':
            i.firstname = 'firstnametoto'
            i.surname = 'surnametoto'
            i.mail = 'mail@toto'
        elif i.ID_ == '10800000':
            i.firstname = 'Jacques'
            i.surname = 'MARTIN'
            i.mail = 'jacques@martin'
        elif i.ID_ in inscrits.demo_animaux:
            i.firstname, i.surname, i.mail = inscrits.demo_animaux[i.ID_][1:4]
        elif i.ID_ == '00000000':
            pass # No information about this one
        else:
            i.firstname = i.ID_ + 'Firstname'
            i.surname = i.ID_ + 'Surname'
            i.mail = 'mail@mail.fr'

def ues_short(items):
    """The UE codes"""
    for i in items:
        if i.ID_ == '10800000':
            i.ues_short = ['UE-INF20UE2L', 'UE-INF20UE2', 'UE-UE1']
        else:
            i.ues_short = []

def member_of_list(items):
    """The list of the groups the user is a member of"""
    for i in items:
        if i.ID_ == 'user.2':
            i.member_of_list = (configuration.teachers[0], )
        elif i.ID_ == 'user.3':
            i.member_of_list = (configuration.teachers[0],
                                configuration.not_teachers[0])
        else:
            i.member_of_list = ()


def do_patch():
    if configuration.regtest:
        configuration.db = 'DBregtest'
        configuration.allow_student_list_update = True
        configuration.year_semester_modifiable = ([9999, 'Test'],)
        configuration.Users.add_attribute(etapes_list)
        configuration.Users.add_attribute(firstname)
        configuration.Users.add_attribute(ues_short)
        configuration.Users.add_attribute(member_of_list)
        configuration.Users.surname = configuration.Users.firstname
        configuration.Users.mail = configuration.Users.firstname
        class LDAP_regtest(inscrits.Empty):
            def students(self, ue, year=None, semester=None, table=None):
                if ue == 'UE-update' or ue == 'UE-update2':
                    with open("xxx.students", "r") as f:
                        for line in f:
                            yield line.strip().split(' ')
                    return
                if ue == 'UE-INF20UE2' or ue == 'UE-XXX9999L' or ue == 'UE-UE1':
                    yield ('10800000', 'Jacques', 'MARTIN', 'jacques@martin',
                           'A', '1')
                if len(ue) == 3 and ue.startswith('UE') and ue[-1].isdigit():
                    for i in list(inscrits.demo_animaux.values()):
                        yield i
                return

            def query_logins(self, logins, attributes, only_first_value=True):
                r = []
                for login in logins:
                    q = self.query_login(login, attributes)
                    if only_first_value:
                        r.append([login]+[q[i][0] for i in attributes[1:]])
                    else:
                        r.append(q) # pragma: no cover
                return r

            def firstname_or_surname_to_logins(self, name, base=None,
                                               attributes=None):
                if attributes == None: # pragma: no cover
                    attributes = [configuration.attr_login,
                                  configuration.attr_surname,
                                  configuration.attr_firstname]
                all = []
                name = name.lower()
                for i, infos in inscrits.demo_animaux.items():
                    if (i.lower() == name
                        or infos[1].lower().startswith(name)
                        or infos[2].lower().startswith(name)
                        ):
                        r = []
                        for a in attributes:
                            if a == configuration.attr_login:
                                r.append(i)
                            elif a == configuration.attr_surname:
                                r.append(infos[2])
                            elif a == configuration.attr_firstname:
                                r.append(infos[1])
                            elif a == configuration.attr_mail:
                                r.append(infos[3])
                            else:
                                r.append(name + '.' + a)
                        all.append(r)
                return all


        inscrits.LDAP = LDAP_regtest
        utilities.warn('Inscrit patched')

        from . import authenticators
        configuration.Authenticator = authenticators.RegTest


    from TOMUSS.REGTEST_SERVER import variables

    # AFTER : because it creates an LDAP connection
    configuration.terminate()

    if not configuration.regtest:
        return # pragma: no cover

    configuration.use_cookie = False
    configuration.display_sequence_column = '.*'

    if "protect_do_not_display" not in sys.argv:
        configuration.do_not_display = ()

    def student_list(f, portails, not_in):
        students = configuration.Users(('10800000', '10800001','10801111','10802222'))
        dummy = students['10800001'].licence
        students['10800001'].discipline = set(('DISCIPLINE',))
        return students

    referent.student_list = student_list

    from .TEMPLATES import Printemps
    if configuration.real_regtest:
        from .TEMPLATES import _ucbl_
        def cell_change(table, page, col, lin, value, date):
            if table.ue in ('UE-INF11UE2', 'UE-INF20UE2', 'UE-numbers',
                    'UE-importzip', 'UE-annotate', 'UE-getref'):
                return 0
            return _ucbl_.cell_change(table, page, col, lin, value, date)
        Printemps.cell_change = cell_change
        def stupid_password(login, _passwords):
            return login == 'ue5.master'
        utilities.stupid_password = stupid_password
        def need_a_charte(login):
            return login.endswith('0000')
        referent.need_a_charte = need_a_charte
        configuration.maximum_out_of_date = 0
        configuration.rgpd_link = 'regtest_rgpd_link'
        configuration.allowed_grades = {
            'DIS': [utilities._('abj_short'), "_dis_"],
            'ABS': [utilities._('abi_short'), "_abs_", 1]}
    else:
        Printemps.cell_change = lambda table,page,col,lin,value,date:0

    def is_an_official_ue(code):
        return int('-' in code)
    configuration.is_an_official_ue = is_an_official_ue

