#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2009-2013 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

# All data structure are lock protected.

import time
import socket
from . import utilities

class Closed:
    closed = True

class File(object):
    nr_active_thread = 0
    to_send = {}

    def __init__(self, f, txt, keep_open, index, page):
        assert(append.the_lock.locked())
        self.file = f
        File.to_send[f] = self
        self.send = [txt]
        self.keep_open = keep_open
        self.in_processing = False
        self.index = index
        self.page = page
        
    def append(self, txt, keep_open, index):
        assert(append.the_lock.locked())
        if self.keep_open is False:
            raise ValueError('Append to a closed file') # pragma: no cover
        self.send.append(txt)
        if index is not None:
            self.index = index
        self.keep_open = keep_open

    def delete(self):
        del File.to_send[self.file]
        if not File.to_send:
            utilities.important_job_remove("sender") # In a locked state
        File.nr_active_thread -= 1

    def send_text(self):
        """Return True if a text is sent"""
        assert(self.in_processing)
        append.the_lock.acquire()
        try:
            if len(self.send) == 0:
                # Remove this sender object, no more useful
                self.delete()
                return False
            to_send = self.send
            self.send = []
            keep_open = self.keep_open
            index = self.index
        finally:
            append.the_lock.release()
        if index is not None:
            to_send.append(('<script>window.page_index=%d</script>\n' % index))
        if isinstance(to_send[0], str) :
            txt = ''.join(to_send)
        else :
            txt = b''.join(to_send)
        try:
            self.file.write(txt)

            if keep_open:
                self.file.flush()
            else:
                self.file.close()
        except:
            if not self.file.closed:
                try:
                    self.file.close()
                except: # pragma: no cover
                    utilities.send_backtrace("Can not mark the file as closed"
                                             + str(self.file))
            if self.page is not None:
                self.page.browser_file = Closed()

            append.the_lock.acquire()            
            self.delete()
            append.the_lock.release()
            return False

        return True

def send_thread(verbose=False):
    while True:
        time.sleep(0.1)
        while File.to_send:
            try:
                try:
                    # Get a file to write into
                    append.the_lock.acquire()
                    for f in File.to_send.values():
                        if not f.in_processing:
                                f.in_processing = True
                                File.nr_active_thread += 1
                                break
                    else:
                        continue # No work to do
                finally:
                    append.the_lock.release()

                # While there is content to send: continue
                while f.send_text():
                    pass
                # Now the File queue is empty, it self destroyed
            except socket.error: # pragma: no cover
                # A write error on a socket
                utilities.warn('Write error on %s closed=%s' % (
                    f.file, f.file.closed))

@utilities.add_a_lock
def append(f, txt, keep_open=True, index=None, page=None):
    # if not txt.startswith('GIF'): utilities.warn('%s %s %s' % (f, txt[0:20], keep_open))
    if f is None:
        utilities.warn('f is None: ' + txt)
        return
    # xxx.write('*' + txt + '\n')
    try:
        File.to_send[f].append(txt, keep_open, index)
    except KeyError:
        utilities.important_job_add("sender")
        File(f, txt, keep_open, index, page)
