#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2008-2011 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr

import tomuss_init
import time
import sys
import os
import glob
import http.server
from . import abj
from . import configuration
from . import utilities
from . import document
from . import files
from . import ticket
from . import sender

if False:
    from . import deadlock
    deadlock.start_check_deadlock()

warn = utilities.warn

running = True
current_time = time.time()

class MyRequestBroker(utilities.FakeRequestHandler):
    background_clean = utilities.background_clean()
    check_down_connections = document.check_down_connections()

    def send_file(self, name):
        # print self.headers
        if name not in files.files: # pragma: no cover
            return
        s = files.files[name]
        # print("*"*99, s.name, s.encoding, type(s.content), type(s.gzipped))
        if s.encoding == 'bytes':
            content = s.get_content()
        else:
            content = s.get_zipped()
        self.send_response(200)
        if s.encoding != 'bytes':
            self.send_header('Content-Encoding', 'gzip')
        if s.name and len(content) != 0:
            self.send_header('Content-Length', len(content))
        self.send_header('Content-Type', s.mimetype)
        if 'UNCACHED' in name:
            self.send_header('Cache-Control', 'no-cache')
        else:
            self.send_header('Cache-Control',
                             'max-age=%d' % configuration.maxage)
        self.end_headers()
        sender.append(self.wfile, content, keep_open=False)
        self.do_not_close_connection()

    def log_time(self, action, start_time=None):
        if start_time is None:
            start_time = self.start_time
        now = time.time()
        logs.write('%f %g %s\n' % (start_time, now - start_time, action))
        logs.flush()

    def do_GET_real(self):
        next(self.background_clean)
        next(self.check_down_connections)

        if self.path.startswith('/PROFILE_THIS_URL/'):
            self.path = self.path.replace('/PROFILE_THIS_URL/', '/')
            self.do_profile = True
        else:
            self.do_profile = False

        # APACHE make a mess with %2F %3F and others
        # so we encode them with $2F $3F...
        self.path = self.path.replace("$", "%")

        if ticket.client_ip(self) in configuration.banned_ip: # pragma: no cover
            self.send_file('bad.png')
            self.log_time('static-file')
            return

        if (self.path.startswith('/files/')
            or self.path != '/' and self.path[1:] in files.files
            ):
            if self.path.startswith('/files/0/') and '/.' not in self.path:
                # Only here if there is no web server frontend
                if self.path not in files.files:
                    name = self.path.replace('/files/0/', 'TMP/0/')
                    files.files[self.path] = utilities.StaticFile(name)
                self.send_file(self.path)
                self.log_time('static-file0')
                return
            name = self.path.split("/")[-1]
            if name in files.files:
                self.send_file(name)
                self.log_time('static-file')
            return

        if configuration.regtest and self.path == '/stop':
            global running
            self.send_response(200)
            self.send_header('Cache-Control', 'no-cache')
            self.send_header('Cache-Control', 'no-store')
            self.send_header('Content-Type', 'text/plain')
            self.end_headers()
            self.wfile.write(b'stopped')
            warn('stopped')
            running = False
            return

        the_ticket, self.the_path = ticket.get_ticket_string(self)
        self.ticket = ticket.get_ticket_objet(the_ticket, self)
        warn('ticket=%s' % str(self.ticket)[:-1])
        warn('the_path=%s' % str(self.the_path))
        self.the_file = self.wfile
        if plugin.dispatch_request(self, manage_error=False) is None:
            # An unauthenticated dispatch was done
            return

        # Don't want to be blocked by authentication
        authentication.ok(self)

    def do_GET_real_real_safe(self):
        """Called by  authentication.ok"""
        plugin.dispatch_request(self)
            
    def do_GET(self):
        global current_time
        self.start_time = time.time()
        self.wfile.write = self.wfile._sock.sendall
        utilities.important_job_add("do_GET")
        try:
            self.do_GET_real()
        except: # pragma: no cover
            if '/picture/' in self.path:
                return # To common error to be logged
            user_name = self.__dict__.get('ticket', '')
            if not user_name:
                user_name = ''
            if not isinstance(user_name, str):
                user_name = user_name.user_name

            utilities.send_backtrace('',
                                     subject = 'BUG TOMUSS ('
                                     + self.path + ') ' + user_name
                                     )
            try:
                if 'plugin' in self.__dict__:
                    if self.plugin.mimetype == 'image/png':
                        self.wfile.write(files.files['bug.png'].bytes())
                    else:
                        self.wfile.write(b'There is a bug')
                else:
                    self.send_file('bug.png')
                self.wfile.close()
            except:
                pass
        utilities.important_job_remove("do_GET")

    def do_OPTIONS(self): # pragma: no cover
        self.send_response(200)
        self.allow_cross_origin()
        self.end_headers()

if __name__ == "__main__":
    if 'daemon' in sys.argv:
        utilities.start_as_daemon(os.path.join("LOGS", "TOMUSS")) # pragma: no cover
    utilities.display_stack_on_kill()
    from . import regtestpatch
    regtestpatch.do_patch()
    from . import plugin # AFTER from . import regtestpatch???
    from . import plugins
    plugins.load_types()
    document.table(0, 'Dossiers', 'config_table', None, None)
    document.table(0, 'Dossiers', 'config_acls', None, None)

    if 'recompute_the_ue_list' in sys.argv:
        from . import teacher
        print(teacher.all_ues(compute=True))
        sys.exit(0)

    configuration.read_only = None # Indicate that it is not a script
    utilities.init(launch_threads=False)

    from . import authentication
    authentication.run_authentication()

    import socket
    for i in range(5):
        try:
            server = utilities.HTTPServer(("0.0.0.0",
                                           configuration.server_port),
                                          MyRequestBroker)
            break
        except socket.error: # pragma: no cover
            warn('Socket port used: %s' % configuration.server_port)
            time.sleep(1)
    else:
        raise IOError("Can not start TOMUSS") # pragma: no cover
    warn('Database:' + configuration.db)
    warn("Server Ready on:" + configuration.server_url)

    utilities.mkpath(os.path.join("LOGS", "TOMUSS"))
    logs = open(os.path.join("LOGS", "TOMUSS",
                             str(time.localtime()[0]) +".times"), "a")

    plugins.plugins_tomuss()
    document.table(0, 'Dossiers', 'config_cron', None, None)
    document.table(0, 'Dossiers', 'config_plugin', None, None)
    plugins.generate_data_files()
    document.start_threads()

    authentication.authentication_redirect = configuration.server_url
    utilities.StaticFile._url_ = authentication.authentication_redirect

    utilities.start_new_thread_immortal(sender.send_thread, (True,))

    for i in range(3):
        utilities.start_new_thread_immortal(sender.send_thread, ())

    document.table(0, 'Dossiers', 'config_home', None, None)
    document.table(0, 'Dossiers', 'config_cache', None, None)
    document.table(0, 'Dossiers', 'config_login', None, None)
    document.table(0, 'Dossiers', 'config_room', None, None)
    from . import display
    display.init()
    document.table(0, 'Dossiers', 'config_display', None, None)
    configuration.initialisations_done = True

    # While there is an updating table, there is many messages
    # Wait the end of the flow
    while utilities.filename_to_bufferize is not None:
        time.sleep(0.1) # pragma: no cover
    print('\n\n\n' +'*'*78)
    print(utilities._("MSG_tomuss_start"
                      ) % (configuration.server_url + '/=super.user'))
    if configuration.regtest:
        regtest = 'regtest'
    else:
        regtest = '' # pragma: no cover
    for url, port, year, semester, host in configuration.suivi.servers():
        print('\t./suivi.py %d %s %d' % (year, semester, port), regtest)
    print('*'*78 + '\n\n')

    if 'profile' in sys.argv: # pragma: no cover
        # To profile a single URL and not the full server.
        # Put PROFILE_THIS_URL just before the ticket:
        #    http://192.168.0.1:8888/PROFILE_THIS_URL/=ticket/stats
        import cProfile
        try:
            cProfile.run("while running: server.handle_request()", "xxx.prof")
        except KeyboardInterrupt:
            import pstats
            ps = pstats.Stats('xxx.prof')
            ps.strip_dirs().sort_stats('cumulative').print_stats()
    else:
        while running:
            server.handle_request()
        
    utilities.stop_threads()
