#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    TOMUSS: The Online Multi User Simple Spreadsheet
#    Copyright (C) 2009-2019 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@univ-lyon1.fr

# pylint: disable=consider-using-f-string,superfluous-parens,line-too-long,invalid-name

import sys
import os
import shutil
import http.client
import time
import json
import traceback
import re
import io
import zipfile
import glob

sys.argv.append("real_regtest")
sys.argv.append("protect_do_not_display")

import tests_config
tests_config.init(time.localtime()[0])

import tomuss_init
from .. import configuration
from TOMUSS import utilities
from TOMUSS.ATTRIBUTES import tableinvitation
from .server import Server, ServerSuivi, check
sys.argv.remove("real_regtest")
sys.argv.remove("protect_do_not_display")

configuration.do_not_display = ('debug', 'auth', 'table', 'ldap', 'plugin',
                                'check', 'lang', 'DNU', 'info')

configuration.regtest = True

root     = configuration.root[0]
abj      = configuration.invited_abj_masters[1]
invited  = configuration.invited_teachers[0]
assert( root != abj )



configuration.language = 'en'

ok_png_bin = utilities.read_file('FILES/ok.png','bytes')
ok_png = 'ok.png'
bad_png_bin = utilities.read_file('FILES/bad.png','bytes')
bad_png = 'bad.png'
bug_png_bin = utilities.read_file('FILES/bug.png','bytes')
bug_png = 'bug.png'
unauthorized_html = utilities._("MSG_new_page_unauthorized")
not_in_demo_mode = utilities._("MSG_evaluate")
deletion_done = utilities._("MSG_delete_this_table_done")

year = configuration.year_semester[0]
semester = configuration.year_semester[1]
uyear = utilities.university_year()
ys = '%d/%s' % (year, semester)
ys_old = '%d/%s' % (year-1, semester)

abj_date_old      = '1/1/%d' % (uyear-2)
abj_date_previous = '1/1/%d' % (uyear-1)
abj_date_current  = '1/1/%d' % (uyear)
abj_date_next     = '31/12/%d' % (uyear+1)

now_plus_32_days = time.strftime('%Y%m%d',time.localtime(time.time()+86400*63))
now_plus_30_days = time.strftime('%Y%m%d',time.localtime(time.time()+86400*30))

green = 'OK!'
red1 = "You are not allowed to modify this value."
red = "Impossible to record this value. Reload the page"

print(ys, uyear)

_ = utilities._


names = open('xxx.names','w', encoding = "utf-8")

def do(t):
    names.write(' ' + t)
    names.flush()
    c = len(sys.argv) == 1 or t in sys.argv or do.found
    if c:
        if '+' in sys.argv:
            do.found = True
        do.current = t
        print(t)
    return c
do.found = False

def save_sec():
    save_sec.time = int(time.time())
def wait_sec():
    while int(time.time()) == save_sec.time:
        time.sleep(0.1)

def replace_dates(d):
    return re.sub('''["']20[0-9]{12}["']''', '"DATE"', d)


def retry(action, test):
    for i in range(20):
        v = action()
        if test(v):
            return v
        time.sleep(1)
    print(v)
    bug

def sign(url, year_semester=None):
    if year_semester is None:
        year_semester = ys
    url = year_semester + '/' + url
    checksum = tableinvitation.checksum_short('/'.join(url.split('/')[-7:]).replace('%20', ' '))
    return url + '/' + checksum

def remove_abjs():
    for abj_file in glob.glob('DBregtest/LOGINS/*/*/abj_*'):
        os.unlink(abj_file)

if len(sys.argv) == 1:
    print('You can indicate in parameters the tests you want to do')

c = ''
s = ''
ss = ''

done = {}

def create_tt():
    if 'create_tt' in done:
        return
    done['create_tt'] = True
    tt = '=' + abj + '/%d/Dossiers/tt' % uyear

    global c
    c = s.url(tt)
    assert_col({"the_id":"0_8","type":"Date","author":"*","position":3,
                "title":_("TH_begin"),"width":6}, c)

    c = s.url(tt + '/1/0/column_attr_title/col_0/COL_TITLE')
    assert( c == ok_png)
    c = s.url(tt + '/1/1/column_attr_comment/col_0/COL_COMMENT')
    assert( c == ok_png)
    c = s.url(tt + '/1/2/cell_change/0_0/0_0/10800000')
    assert( c == ok_png)
    c = s.url(tt + '/1/3/cell_change/col_0/0_0/CELL_VALUE')
    assert( c == ok_png)
    c = s.url(tt + '/1/4/comment_change/col_0/0_0/CELL_COMMENT')
    assert( c == ok_png)
    c = s.url(tt + '/1/5/cell_change/0_8/0_0/01$2F09$2F%s' % uyear)
    assert( c == ok_png)
    c = s.url(tt + '/1/6/cell_change/0_9/0_0/01$2F09$2F%s' % (uyear+2))
    assert( c == ok_png)

def create_u2():
    if 'create_u2' in done:
        return
    done['create_u2'] = True

    global c

    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/0/column_attr_title/col_0/TITLE0')
    assert(c == ok_png)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/1/cell_change/col_0/0_0/11.11') # A note
    assert(c == ok_png)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/2/cell_change/0_0/0_0/10800000') # Student ID
    assert(c == bad_png)
    check('Y%d/S%s/UE-INF20UE2.py' % (year, semester),
          cell_required = (0,'0_5','0_0','ok'))

    # Add bad student
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/3/cell_change/col_0/0_1/22.22') # A note
    assert(c == ok_png)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/4/cell_change/0_0/0_1/10800001') # Student ID
    assert(c == ok_png)
    check('Y%d/S%s/UE-INF20UE2.py' % (year, semester),
          cell_required = (0,'0_5','0_1','non'))

    # Add table comment
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/5/table_attr_comment/_TABLE_COMMENT_')
    assert(c == ok_png)

    # Add a master
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/6/table_attr_masters/ue1.master')
    assert(c == ok_png)

    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/7/column_attr_type/col_1/Text') # A text
    assert(c == ok_png)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/8/cell_change/col_1/0_1/ABIGAT') # A text
    assert(c == ok_png)

    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/9/comment_change/col_1/0_1/CELL_COMMENT')
    assert(c == ok_png)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/10/column_attr_comment/col_1/COLUMN_COMMENT')
    assert(c == ok_png)
    # c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
    #           '/1/11/cell_change/0_0/0_3/A') # TD Grp A
    # assert(c == ok_png)
    # c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
    #           '/1/12/cell_change/0_1/0_3/B') # TD Grp B
    # assert(c == ok_png)


    # Verify number of loads
    c = utilities.manage_key('LOGINS', os.path.join(root, 'pages'))
    assert("'UE-INF20UE2':" in c)
    c = utilities.manage_key('LOGINS', '')
    assert(c == False)
    c = utilities.manage_key('LOGINS', os.path.join(root, 'xxx'), content='txt')
    assert(c == 'txt' or c == '')
    t = utilities.key_mtime('LOGINS', os.path.join(root, 'xxx'))
    time.sleep(0.01)
    c = utilities.manage_key('LOGINS', os.path.join(root, 'xxx'), content="txt", touch=True)
    assert(c == 'txt')
    assert(utilities.key_mtime('LOGINS', os.path.join(root, 'xxx')) > t)
    utilities.manage_key('LOGINS', os.path.join(root, 'xxx'), content="nothing42")
    c = utilities.manage_key('LOGINS', os.path.join(root, 'xxx'), delete=True)
    assert(c is None)
    c = utilities.manage_key('LOGINS', os.path.join(root, 'xxx'))
    assert(c == False)

    # Change student id (for mail on new student)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/11/cell_change/0_0/0_2/10800010') # Student ID
    assert(c == ok_png)
    c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/1/12/cell_change/0_0/0_2/10800011') # Student ID
    assert(c == ok_png)


def create_referents():
    if 'create_referents' in done:
        return
    done['create_referents'] = True
    global c
    c = s.url('=' + abj + '/%s/referents_students' % ys)
    c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/2/0/cell_change/a/line_0/toto')
    assert(c == ok_png)
    c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/2/1/cell_change/a/line_1/a_referent')
    assert(c == ok_png)

    # Create a column in referents table (a a line)
    c = s.url('=a_referent/referent_get/10900000/')
    assert(_("MSG_referent_get_done") in c)
    assert('10900000' in c)

    c = s.url('=a_referent/referent_get/10900000')
    assert('you are yet' in c)

    c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/2/2/cell_change/0/line_0/10800000')
    assert(c == ok_png)

    c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/2/3/cell_change/b/line_0/DISCIPLINE')
    assert(c == ok_png)



def assert_col(attrs, lines, contains="", startswith="", endswith=""):
    s = []
    for line in lines.split('\n'):
        if 'Col(' in line:
            s.append(line)
        if '"the_id": "%s"' % attrs['the_id'] not in line:
            continue
        if contains not in line:
            continue
        if not line.startswith(startswith):
            continue
        if not line.endswith(endswith):
            continue
        for attr, value in attrs.items():
            assert('"%s": %s' % (attr, json.dumps(value)) in line)
        return True
    else:
        raise ValueError('c=%s s=%s e=%s\nNot found: %s\nIn :\n%s'
                         % (contains, startswith, endswith,
                            attrs, '\n'.join(s)))

def tests():
    global c, s, ss, done
    done = {}
    ss = ServerSuivi(coverage=coverage, search_leak=search_leak)
    s = Server(coverage=coverage, search_leak=search_leak)
    s.start()
    c = s.url('=' + root + '/0/Dossiers/config_suivi_rss')
    if do('badurl'):
        c = s.url('=' + root + '/BADURL')
        assert(c == 'bad_url')
        c = s.url('='+root+'/evaluate/5-2')
        assert(c == '3')
    if do('homeroot'):
        c = s.url('=' + root)
        assert("javascript:goto_url(add_ticket('clean/'+year_semester()))" in c)
    if do('homeabj'):
        c = s.url('=' + abj)
        assert("javascript:go('abj')" in c)
    if do('stat'):
        c = s.url('=' + root + '/stat')
        assert("<table border>" in c)
    if do('profiling'):
        c = s.url('=' + root + '/profiling/%d' % year )
        assert("initialize()" in c)
    #if do('gc'):
    #    c = s.url('=' + root + '/gc')
    #    assert(not_in_demo_mode in c)

    if do('preferences'):
        c = s.url('=' + abj).split('preferences=')[-1]
        c = c.split(';')[0]
        assert('"language":"' in c)

        c = s.url('=' + abj + '/save_preferences/debug_home=1')
        assert(c == ok_png_bin)
        s.url('=' + abj + '/save_preferences/debug_home=1=1') # coverage

        c = utilities.read_file('DBregtest/LOGINS/%s/%s/preferences' % (
            abj[:3], abj))
        assert("'debug_home':1," in c)

        c = s.url('=' + abj).split('HomePreferences')[1]
        assert('"debug_home": 1' in c)

    if do('backtrace_url'):
        c = s.url('=' + root + '/0/Dossiers/config_table')
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('='+root+'/0/Dossiers/config_table/%s/0/cell_change/0_2/backtrace_times_url/file:$2Fbin$2Fsh' % page)
        assert(c == ok_png)

        # For coverage
        c = s.url('='+root+'/0/Dossiers/config_table/%s/1/cell_change/0_2/hide_empty_table/.*a.*' % page)
        assert(c == ok_png)

    if do('config_message'):
        c = s.url('=' + root + '/0/Dossiers/config_messages')
        assert('The HTML messages to display' in c)
        c = s.url('=' + root + '/0/Dossiers/config_messages/2/0/cell_change/a/L1/0')
        assert(c == ok_png)

    if do('emptydossier'):
        c = s.url('=' + abj + '/%d/Dossiers/emptydossier' % uyear)

        lines_id = []
        nr_columns = 0
        nr_pages = 2
        nr_cells = 0
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              dump=False)

        # Create cell in non existent column
        c = s.url('='+abj+'/%d/Dossiers/emptydossier' % uyear +
                  '/1/0/cell_change/col_0/line_0/_VALUE_')
        assert(c == bug_png)
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              dump=False)

        # Create column
        c = s.url('='+abj+'/%d/Dossiers/emptydossier' % uyear +
                  '/1/1/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        nr_columns += 1
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              column_required =('title', 1, 'col_0', 'TITLE0'),
              dump=False)

        # Create cell
        c = s.url('='+abj+'/%d/Dossiers/emptydossier' % uyear +
                  '/1/2/cell_change/col_0/line_0/_VALUE_')
        assert(c == ok_png)
        lines_id = ['line_0']
        nr_cells += 1
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = (1,'col_0','line_0','_VALUE_'),
              dump=False)

        # Change comment on cell
        c = s.url('='+abj+'/%d/Dossiers/emptydossier' % uyear +
                  '/1/3/comment_change/col_0/line_0/_COMMENT_')
        nr_cells += 1
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              cell_required = (1,'col_0','line_0','_COMMENT_'),
              nr_cells = nr_cells,
              dump=False)

        # Change cell value
        c = s.url('='+abj+'/%d/Dossiers/emptydossier' % uyear +
                  '/1/4/cell_change/col_0/line_0/_VALUE_2_')
        assert(c == ok_png)
        lines_id = ['line_0']
        nr_cells += 1
        
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = (1,'col_0','line_0','_VALUE_2_'),
              dump=False)

        # Check HTML
        c = s.url('=' + abj + '/%d/Dossiers/emptydossier' % uyear)
        nr_pages += 1
        check('Y%d/SDossiers/emptydossier.py' % uyear,
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              dump=False)
        assert_col({"the_id":"col_0","type":"Note","author":abj,
                   "position":0,"title":"TITLE0"}, c)
        assert('P("line_0",[C("_VALUE_2_","%s","' % abj in c)
        assert('","_COMMENT_",[["_COMMENT_","' in c) # History

    if do('badsyntax'):
        c = s.url('=' + abj + '/%d/Dossiers/badsyntax' % uyear)
        c = s.url('=' + abj + '/%d/Dossiers/badsyntax' % uyear +
                  '/1/0/cell_change/col_0/line_0/_VALUE_')
        assert(c == bug_png)
        c = s.url('=' + abj + '/%d/Dossiers/badsyntax' % uyear +
                  '/1/1/cell_chane/col_0/line_0/_VALUE_')
        assert(c == bug_png)
        c = s.url('=' + abj + '/%d/Dossiers/badsyntax' % uyear +
                  '/1/0/dsfsdffds')
        assert(c == bug_png) # ok for old requests

    if do('master'):
        c = s.url('=' + root + '/%d/Dossiers/master' % uyear)
        c = s.url('=' + root + '/%d/Dossiers/master' % uyear +
                  '/1/0/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        c = s.url('=' + root + '/%d/Dossiers/master' % uyear +
                  '/1/1/cell_change/col_0/line_0/_VALUE_')
        assert(c == ok_png)
        lines_id = ['line_0']
        nr_columns = 1
        nr_pages = 2
        nr_cells = 1
        cell_required = (1,'col_0','line_0','_VALUE_')
        check('Y%d/SDossiers/master.py' % uyear,
              masters_expected = [root], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = s.url('=' + abj + '/%d/Dossiers/master' % uyear)
        assert(unauthorized_html in c)

        c = s.url('=' + root + '/%d/Dossiers/master' % uyear +
                  '/1/2/table_attr_masters/super.user%20' + abj)
        check('Y%d/SDossiers/master.py' % uyear,
              masters_expected = [root,abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = utilities.manage_key('LOGINS', os.path.join(root, 'master_of'))
        assert("('%d', 'Dossiers', 'master')" % uyear in c)
        assert("('0', 'Dossiers', 'config_table')" in c)

        c = s.url('=' + abj + '/%d/Dossiers/master' % uyear)
        assert("masters:['%s', '%s']" % (root, abj) in c)
        nr_pages += 1
        check('Y%d/SDossiers/master.py' % uyear,
              masters_expected = [root,abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = s.url('=' + abj + '/%d/Dossiers/master' % uyear +
                  '/2/0/cell_change/col_0/line_0/_VALUE_2_')
        assert(c == ok_png)
        nr_cells += 1
        cell_required = (2,'col_0','line_0','_VALUE_2_')
        check('Y%d/SDossiers/master.py' % uyear,
              masters_expected = [root,abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = s.url('=' + root + '/%d/Dossiers/master' % uyear +
                  '/1/3/table_attr_masters/' + root)
        check('Y%d/SDossiers/master.py' % uyear,
              masters_expected = [root], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = s.url('=' + abj + '/%d/Dossiers/master' % uyear)
        assert(unauthorized_html in c)

        c = s.url('=' + abj + '/%d/Dossiers/master' % uyear +
                  '/2/1/cell_change/col_0/line_0/_VALUE3_')
        # It is ok because we are the cell author.
        # But it could be 'bad' because we are not a master in 'Dossiers'
        assert(c == ok_png)
        nr_cells += 1
        cell_required = (2,'col_0','line_0','_VALUE3_')
        check('Y%d/SDossiers/master.py' % uyear,
              masters_expected = [root], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

    if do('badpage'):
        c = s.url('=' + abj + '/%d/Dossiers/badpage' % uyear +
                  '/1/0/column_change/col_0/TITLE0/Note/[0;20]/1//1')
        # The answer may be different if the user is yet
        # authenticated or not
        assert(c == "<type 'exceptions.IndexError'>\nlist index out of range"
               or c == bug_png)

        c = s.url('=' + abj + '/%d/Dossiers/badpage' % uyear)
        c = s.url('=' + abj + '/%d/Dossiers/badpage' % uyear +
                  '/99/0/column_change/col_0/TITLE0/Note/[0;20]/1//1')
        assert(c == bug_png)

        # Take the page of somebody else
        c = s.url('=' + root + '/%d/Dossiers/badpage' % uyear +
                  '/0/0/column_change/col_0/TITLE0/Note/[0;20]/1//1')
        assert( c == bug_png or 'Cheater' in c)

    if do('cellprotect'):
        c = s.url('=' + root + '/9999/Test/cellprotect')
        c = s.url('=' + root + '/9999/Test/cellprotect' +
                  '/1/0/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        c = s.url('=' + root + '/9999/Test/cellprotect' +
                  '/1/1/cell_change/col_0/line_0/_VALUE_1_')
        assert(c == ok_png)
        lines_id = ['line_0']
        nr_columns = 7
        nr_pages = 2
        nr_cells = 1
        check('Y9999/STest/cellprotect.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              dump=False)
        c = s.url('=' + abj + '/9999/Test/cellprotect')
        assert('P("line_0",[C(),C(),C(),C(),C(),C()' in c)
        assert('),C("_VALUE_1_","%s","' % root in c)
        c = s.url('=' + abj + '/9999/Test/cellprotect' +
                  '/2/0/cell_change/col_0/line_0/_VALUE_2_')
        assert(c == bad_png)
        c = s.url('=' + root + '/9999/Test/cellprotect' +
                  '/1/2/cell_change/col_0/line_0/_VALUE_2_')
        assert(c == ok_png)
        nr_cells = 2
        nr_pages += 1
        check('Y9999/STest/cellprotect.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              dump=False)
        
        c = s.url('=' + abj + '/9999/Test/cellprotect' +
                  '/2/1/comment_change/col_0/line_0/_COMMENT_')
        assert(c == bad_png)
        # nr_cells += 1
        cell_required = (2,'col_0','line_0','_COMMENT_')
        cell_required = (1, 'col_0', 'line_0', '_VALUE_2_')
        check('Y9999/STest/cellprotect.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = s.url('=' + abj + '/9999/Test/cellprotect')
        assert('P("line_0",[C(),C(),C(),C(),C(),C()' in c)
        assert('),C("_VALUE_2_","%s","' % root in c)
        assert('_COMMENT_' not in c)


    if do('badchars'):
        c = s.url('=' + root + '/9999/Test/badchars')
        c = s.url('=' + root + '/9999/Test/badchars' +
                  '/1/0/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        c = s.url('=' + root + '/9999/Test/badchars' +
                  '/1/1/cell_change/col_0/line_0/~`!@%23$24%25^%26*()%2B[]{}|\\;:\'"$2F$3F.,$3F')
        assert(c == ok_png)
        lines_id = ['line_0']
        nr_columns = 7
        nr_pages = 2
        nr_cells = 1
        cell_required = (1,'col_0','line_0','~`!@#$%^&*()+[]{}|\\;:\'"/?.,?')
        check('Y9999/STest/badchars.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)

        c = s.url('=' + root + '/9999/Test/badchars/=option=1=/=xxx=yyy=')
        assert('page_id = "2" ;' in c)

    if do('addfirstmaster'):
        c = s.url('=' + abj + '/9999/Test/addfirstmaster')
        c = s.url('=' + abj + '/9999/Test/addfirstmaster' +
                  '/1/0/table_attr_masters/' + abj)
        assert(c == ok_png)
        check('Y9999/STest/addfirstmaster.py', masters_expected = [abj])
        c = s.url('=' + abj + '/9999/Test/addfirstmaster' +
                  '/1/1/table_attr_masters/' + abj + '%20' + root)
        assert(c == ok_png)
        check('Y9999/STest/addfirstmaster.py', masters_expected = [abj,root])

    if do('badaddmaster'):
        c = s.url('=' + root + '/9999/Test/badaddmaster')
        c = s.url('=' + root + '/9999/Test/badaddmaster' +
                  '/1/0/table_attr_masters/' + root)
        assert(c == ok_png)
        c = s.url('=' + abj + '/9999/Test/badaddmaster')
        assert("masters:['%s']" % root in c)
        c = s.url('=' + abj + '/9999/Test/badaddmaster' +
                  '/2/0/table_attr_masters/' + abj)
        assert(c == bad_png)

    if do('masterpower'):
        c = s.url('=' + root + '/9999/Test/masterpower')
        c = s.url('=' + root + '/9999/Test/masterpower' +
                  '/1/0/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        c = s.url('=' + root + '/9999/Test/masterpower' +
                  '/1/1/cell_change/col_0/line_0/_VALUE_')
        assert(c == ok_png)

        c = s.url('=' + abj + '/9999/Test/masterpower')
        c = s.url('=' + abj + '/9999/Test/masterpower' +
                  '/2/0/table_attr_masters/' + abj)
        assert(c == ok_png)
        c = s.url('=' + abj + '/9999/Test/masterpower' +
                  '/2/1/cell_change/col_0/line_0/_VALUE_2_')
        assert(c == ok_png)
        lines_id = ['line_0']
        nr_columns = 7
        nr_pages = 3
        nr_cells = 2
        cell_required = (2,'col_0','line_0','_VALUE_2_')
        check('Y9999/STest/masterpower.py',
              masters_expected = [abj], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              cell_required = cell_required,
              dump=False)
        
    if do('comments'):
        c = s.url('=' + root + '/9999/Test/comments')
        c = s.url('=' + root + '/9999/Test/comments' +
                  '/1/0/column_attr_comment/col_0/_COMMENT_')
        assert(c == ok_png)
        c = s.url('=' + root + '/9999/Test/comments' +
                  '/1/1/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        lines_id = []
        nr_columns = 7
        nr_pages = 2
        nr_cells = 0
        column_required = ('comment', 1, 'col_0', '_COMMENT_')
        check('Y9999/STest/comments.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              column_required = column_required,
              dump=False)

        c = s.url('=' + abj + '/9999/Test/comments')
        c = s.url('=' + abj + '/9999/Test/comments' +
                  '/2/0/column_attr_comment/col_0/_COMMENT2_')
        assert(c == bad_png)
        
        column_required = ('comment', 1, 'col_0', '_COMMENT_')
        nr_pages += 1
        # nr_columns += 1
        check('Y9999/STest/comments.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              column_required = column_required,
              dump=False)

        c = s.url('=' + abj + '/9999/Test/comments' +
                  '/2/1/table_attr_comment/_COMMENT3_')
        assert(c == ok_png)
        column_required = (2, '_COMMENT3_')
        check('Y9999/STest/comments.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              column_required = column_required,
              dump=False)
        
        c = s.url('=' + root + '/9999/Test/comments' +
                  '/1/2/table_attr_comment/_COMMENT4_')
        assert(c == ok_png)
        column_required = (1, '_COMMENT4_')
        check('Y9999/STest/comments.py',
              masters_expected = [], nr_pages = nr_pages,
              nr_columns = nr_columns, lines_id = lines_id,
              nr_cells = nr_cells,
              column_required = column_required,
              dump=False)

        c = s.url('=' + root + '/9999/Test/comments')
        assert('comment:"_COMMENT4_"' in c)
        assert_col({"the_id":"col_0","type":"Note","author":root,
                   "comment":"_COMMENT_","position":6,"title":"TITLE0"}, c)

    if do('ue2'):
        # See 'tomuss.py' for more information on this case
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        assert('P("0_0",[C("10800000","*","' in c)
        assert('),C("Jacques","*","' in c)
        assert('),C("MARTIN","*","' in c)

    if do('tablecopy'):
        create_u2()

        # Copy only columns
        c = s.url('=' + root + '/%s/UE-INF20UE2/tablecopy/0/Dossiers/columns'
                  % ys)
        assert('OK' in c)
        c = s.url('=ue1.master')
        assert('["0", "Dossiers", "UE-INF20UE2"]' in c)
        c = s.url('=' + root + '/0/Dossiers/UE-INF20UE2')
        assert('TITLE0' in c)
        assert('_TABLE_COMMENT_' in c)
        assert('MARTIN' not in c)

        # Copy content without history
        c = s.url('=' + root + '/%s/UE-INF20UE2/tablecopy/0/Dossiers/content'
                  % ys)
        assert('OK' not in c) # Overwrite not allowed
        c = s.url('=ue1.master/0/Dossiers/UE-INF20UE2/delete_this_table')
        assert(deletion_done in c)
        c = s.url('=' + root + '/%s/UE-INF20UE2/tablecopy/0/Dossiers/content'
                  % ys)
        assert('OK' in c)
        c = s.url('=' + root + '/0/Dossiers/UE-INF20UE2')
        assert('MARTIN' in c)

        # Copy history
        c = s.url('=' + root + '/%s/UE-INF20UE2/tablecopy/0/Dossiers/history'
                  % ys)
        assert('OK' not in c) # Overwrite not allowed
        c = s.url('=ue1.master/0/Dossiers/UE-INF20UE2/delete_this_table')
        c = s.url('=' + root + '/%s/UE-INF20UE2/tablecopy/0/Dossiers/history'
                  % ys)
        assert('OK' in c)
        a = utilities.read_file('DBregtest/Y0/SDossiers/UE-INF20UE2.py')
        b = utilities.read_file('DBregtest/Y%d/S%s/UE-INF20UE2.py' % (
            year, semester))
        assert( a == b )

    if do('tablecopyuploads'):
        c = s.url('=%s/%s/UE-copyupload' % (abj, ys))
        c = s.url('=%s/%s/UE-copyupload/1/0/column_attr_type/A/Upload' % (abj, ys))
        assert( c == ok_png) 
        c = s.url('=%s/%s/UE-copyupload/1/1/cell_change/0_0/L1/18000000' % (abj, ys))
        assert( c == ok_png)
        c = s.post('=%s/%s/UE-copyupload/upload_post/A/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), )
               )
        assert('Uploaded file size in bytes: 16' in c)

        # Copy uploads
        c = s.url('=%s/%s/UE-copyupload/tablecopy/0/Dossiers/upload' % (abj, ys))
        assert('OK' in c)
        a = utilities.read_file('DBregtest/Y0/SDossiers/UE-copyupload.py')
        b = utilities.read_file('DBregtest/Y%d/S%s/UE-copyupload.py' % (
            year, semester))
        assert( a == b )
        c = utilities.read_file('UPLOAD/0/Dossiers/UE-copyupload/A/L1', encoding='bytes')
        assert( c == b"the file content")

    if do('abj'):
        c = s.url('=' + abj + '/%s/abj' % ys)
        assert('abj.js' in c)
        c = s.url('=' + abj + '/%s/abj/0/10800000/display' % ys)
        # The student is not in any UE
        assert("ues_without_da(['UE-INF20UE2', 'UE-INF20UE2L', 'UE-UE1'])" in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/add_da/UE-UE1/1/2/3/com1' % ys)
        assert("display_da([['UE-UE1','1/2/3','%s',\"com1\"]])" % abj in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/add_da/UE-UE1/1/2/3/com2' % ys)
        assert("display_da([['UE-UE1','1/2/3','%s',\"com1\"]])" % abj in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/add_da/UE-INF20UE2/1/1/%d/com2' % (ys,year))
        assert("window.parent.ues_without_da(['UE-INF20UE2L'])" in c)
        assert("display_da([['UE-UE1','1/2/3','%s',\"com1\"],['UE-INF20UE2','1/1/%d','%s',\"com2\"]])" % (abj, year, abj) in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/rem_da/UE-UE1' % ys)
        assert("display_da([['UE-INF20UE2','1/1/%d','%s',\"com2\"]])" % (year, abj) in c)


        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/M/%s/A/com4' % (
            ys, abj_date_old, abj_date_previous))
        assert("display_da([['UE-INF20UE2','1/1/%d','%s',\"com2\"]])" % (year, abj) in c)
        assert("display_abjs([['%sM','%sA','%s',\"com4\",[]]])" % (
            abj_date_old, abj_date_previous, abj) in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/M/%s/A/com5' % (
            ys, abj_date_current, abj_date_next))
        assert("display_da([['UE-INF20UE2','1/1/%d','%s',\"com2\"]])" % (year, abj) in c)
        assert("display_abjs([['%sM','%sA','%s',\"com4\",[]],['%sM','%sA','%s',\"com5\",[]]])" % (
            abj_date_old, abj_date_previous, abj,
            abj_date_current, abj_date_next, abj) in c)

        # For coverage: add a duplicate
        s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/M/%s/A/com5' % (
            ys, abj_date_current, abj_date_next))
        # Add a bad date
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/X/%s/A/com5' % (
            ys, abj_date_current, abj_date_next))
        assert('There is a bug' in c)
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/4%s/M/%s/A/com5' % (
            ys, abj_date_current, abj_date_next))
        assert('There is a bug' in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/delabj/%s/M/%s/A' % (
            ys, abj_date_old, abj_date_previous))
        assert("display_da([['UE-INF20UE2','1/1/%d','%s',\"com2\"]])" % (year, abj) in c)
        assert("display_abjs([['%sM','%sA','%s',\"com5\",[]]])" % (
            abj_date_current, abj_date_next, abj) in c)

        c = s.url('=' + abj + '/%s/abj/alpha.xls' % ys)
        assert('<tbody id="t"><tr><td>10800000firstname</td><td>10800000surname</td><td>10800000</td><td>ABJ</td><td>%sM</td><td>%sA</td></tr>\n<tr><td>10800000firstname</td><td>10800000surname</td><td>10800000</td><td>DAS</td><td>UE-INF20UE2</td><td>1/1/%d</td></tr>\n</tbody></table>' % (
            abj_date_current, abj_date_next, year))

        # No messages sent because there is no UE master mail
        c = s.url('=' + abj + '/%s/abj/list_mail' % ys)
        assert('UE-INF20UE2L: ' in c)
        assert('UE-INF20UE2: ' in c)
        assert(_("MSG_no_master") in c)
        assert(_("MSG_abj_send") % 1 in c)

        create_u2()

        # See 'tomuss.py' for more information on this case
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        assert('P("0_0",[C("10800000","*","' in c)
        assert('),C("Jacques","*","' in c)
        assert('),C("MARTIN","*","' in c)
        assert('),C(11.11,"%s","' % root in c)
        assert('"10800000":"jacques@martin"' in c)
        assert('change_abjs({"10800000":[[["%sM","%sA","com5"]],[["UE-INF20UE2","1/1/%d","com2"]],""]});' % (
            abj_date_current, abj_date_next, year) in c)

        c = s.url('=' + root + '/%s/UE-INF20UE2/resume' % ys)
        assert(_("TH_ABJ_list") in c)
        assert('10800000 MARTIN Jacques' in c)
        # assert('Du %s matin au %s apr' % (
        #    abj_date_current, abj_date_next) in c)
        assert(str(abj_date_current) in c)
        assert(str(abj_date_next) in c)
        # assert('avec une DA' in c)
        # assert('partir du %s' % year in c)

        s.stop()
        s.restart()

        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        assert('P("0_0",[C("10800000","*","' in c)
        assert('),C("Jacques","*","' in c)
        assert('),C("MARTIN","*","' in c)
        assert('),C(11.11,"%s","' % root in c)
        assert('"10800000":"jacques@martin"' in c)
        assert('change_abjs({"10800000":[[["%sM","%sA","com5"]],[["UE-INF20UE2","1/1/%d","com2"]],""]});' % (
            abj_date_current, abj_date_next, year) in c)

        c = s.url('=' + abj + '/%s/abj/0/10800000/add_da/UE-INF20UE2L/1/1/%d/com42' % (ys,year))
        assert("document.write(window.parent.display_da([['UE-INF20UE2','1/1/{0}','{1}',\"com2\"],['UE-INF20UE2L','1/1/{0}','{1}',\"com42\"]]));window.parent.ues_without_da(['UE-UE1']);".format(year, abj) in c)
        c = s.url('=' + abj + '/%s/abj/0/10800000/add_da/UE-UE1/1/1/%d/com42' % (ys,year))
        assert('window.parent.ues_without_da([])' in c)

        # One day
        d = abj_date_next.replace('/12/', '/05/')
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/M/%s/M/com6' % (
            ys, d, d))
        assert('"com6",[]]]))' in c)
        d = abj_date_next.replace('/12/', '/04/')
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/M/%s/A/com7' % (
            ys, d, d))
        assert('com7",[]]]))' in c)

        d = abj_date_next.replace('/12/', '/03/')
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/%s/M/%s/A/{{{MESSAGE}}}comX' % (
            ys, d, d))
        assert('comX",[]]]))' in c)

        # pruned date
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/1/1/2001/M/2/1/2001/A/com8' % ys)
        assert('"com8",[]]])' in c)
        c = s.url('=' + abj + '/%s/abj/0/10800000/addabj/1/1/2101/M/2/1/2101/A/com9' % ys)
        assert('"com9",[]]])' in c)

        s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        c = s.url('=' + root + '/%s/UE-INF20UE2/resume' % ys)
        assert(_("TH_ABJ_list") in c)
        assert('10800000 MARTIN Jacques' in c)
        assert('JLV of' in c)
        assert(str(abj_date_current) in c)
        assert(str(abj_date_next) in c)
        assert('10800000 MARTIN Jacques Since 1/1' in c)


    if do('extension'):
        if semester == 'Printemps':
            c = s.url('=%s/%s/emptytable' % (abj, ys))
            c = s.url('=%s/%s/emptytable' % (abj, uyear))
            c = s.url('=%s/%s/emptytable/1/0/table_attr_masters/%s' % (abj, ys, abj))
            assert( c == ok_png)
            c = s.url('=%s/%s/emptytable/1/1/cell_change/0_0/L1/18000001' % (abj, ys))
            assert( c == ok_png)

            utilities.mkpath('DBregtest/Y%s/SAutomne' % uyear)
            c = utilities.read_file('DBregtest/Y%s/S%s/emptytable.py' % (year, semester))
            utilities.write_file('DBregtest/Y%s/SAutomne/extension.py' % uyear, c)

            # For coverage
            s.url('=%s/%s/Automne/extension' % (abj, uyear))

            c = s.url('=%s/%s/Automne/extension/undo_extension' % (abj, uyear))
            assert(c.count('Create the table') == 1)
            assert(c.count('is yet semestrialized') == 1)

            # s.url('=%s/%s/Printemps/extension' % (abj, uyear+1))

        c = s.url('=%s/%s/Automne/extension/extension' % (abj, uyear))
        assert( _("MSG_extension_ok").split('\n')[1] in c)
        c = s.url('=%s/%s/Automne/extension/extension' % (abj, uyear))
        assert('The table is yet unsemestrialized' in c)
        c = s.url('=%s/%s/Automne/NOextension/extension' % (abj, uyear))
        assert('impossible because the table does not exist' in c)

        c = s.url('=%s/%s/Printemps/extension' % (abj, uyear+1))
        assert('semester = "Automne" ;' in c)

        c = s.url('=%s/%s/extension2' % (abj, ys))
        c = s.url('=%s/%s/extension2/extension' % (abj, ys))
        assert('you are not a table master' in c)

        c = s.url('=%s/%s/Automne/extension/undo_extension' % (invited, uyear))
        assert('you are not a table master' in c)

        c = s.url('=%s/%s/Automne/extension/undo_extension' % (abj, uyear))
        assert('is yet semestrialized<br>' in c)
        assert('Create the table !<br>' in c)

        c = s.url('=%s/%s/Automne/extension/undo_extension' % (abj, uyear))
        assert(c.count('is yet semestrialized<br>') == 2)

        c = s.url('=%s/%s/Automne/extension/extension' % (abj, uyear))
        assert( _("MSG_extension_ok").split('\n')[1] in c)

    if do('tt'):
        create_tt()

        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        assert('COL_COMMENT: CELL VALUE (CELL COMMENT)' in c)

        s.stop()
        s.restart()

        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        assert('COL_COMMENT: CELL VALUE (CELL COMMENT)' in c)

        c = s.url('=' + root + '/%s/UE-INF20UE2/resume' % ys)
        assert('- COL_COMMENT: CELL VALUE (CELL COMMENT)' in c)

        ss.start()
        c = ss.url('=%s/%s/18000000' % (abj, ys))
        c = ss.url('=%s/%s/18000000' % ('ue3.master', ys))

    if do('delcol'):
        c = s.url('=' + abj + '/%d/Dossiers/delcol' % uyear)
        c = s.url('='+abj+'/%d/Dossiers/delcol' % uyear +
                  '/1/0/column_attr_title/col_0/TITLE0')
        assert( c == ok_png)
        c = s.url('='+abj+'/%d/Dossiers/delcol' % uyear +
                  '/1/1/cell_change/col_0/line_0/_VALUE_')
        assert(c == ok_png)
        c = s.url('='+abj+'/%d/Dossiers/delcol' % uyear +
                  '/1/2/column_delete/col_0')
        assert(c == bad_png)
        c = s.url('='+abj+'/%d/Dossiers/delcol' % uyear +
                  '/1/3/cell_change/col_0/line_0/')
        assert(c == ok_png)
        c = s.url('='+abj+'/%d/Dossiers/delcol' % uyear +
                  '/1/4/column_delete/col_0')
        assert(c == ok_png)
        c = s.url('='+abj+'/%d/Dossiers/delcol' % uyear +
                  '/1/5/column_attr_title/col_1/TITLE1')
        assert( c == ok_png)

        s.stop()
        s.restart()

        c = s.url('=' + abj + '/%d/Dossiers/delcol' % uyear)
        assert_col({"the_id":"col_1","type":"Note","author":abj,
                   "position":0,"title":"TITLE1"}, c,
                   )

    if do('referents'):
        create_referents()

    if do('rss'):
        create_u2()
        save_sec()
        ss.start()
        c = ss.url('%s/rss/10800000' % ys)
        assert(_("MSG_suivi_student_RSS_forbiden_title") in c)

        c = ss.url('=%s/%s/%%2010800000' % (root, ys))
        assert('<iframe' not in c)
        key = utilities.manage_key('LOGINS', '10800001/rsskey')
        assert(key is False)

        ss.url('=10800000/%s' % ys)
        s.url('=10800000/rsskey')
        key = utilities.manage_key('LOGINS', '10800000/rsskey')
        assert(key is not False)
        c = ss.url('%s/rss/%s' % (ys, key))
        assert('<title>UE-INF20UE2 : TITLE0 : 11.11/20</title>' in c)

        c2 = ss.url('%s/rss/%s' % (ys, key))
        assert(c.split('</lastBuildDate>')[1] ==
               c2.split('</lastBuildDate>')[1])

        wait_sec()
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
                  '/%s/0/column_attr_visibility_date/col_0/%s'
                  % (page, now_plus_30_days))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
                  '/%s/1/column_attr_visibility_date/col_0/'
                  % (page,))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/2/column_attr_minmax/col_0/[-20;20]' % (root, ys, page))
        assert(c == ok_png)

        c = ss.url('=' + root + '/%s/unload/%s' % (ys, 'UE-INF20UE2'))
        assert(c == '')
        c = ss.url('%s/rss/%s' % (ys, key))
        assert('<title>UE-INF20UE2 : TITLE0 : 11.11[-20;20]</title>' in c)
        assert(c.split('</lastBuildDate>')[1] !=
               c2.split('</lastBuildDate>')[1]) # RSS want the most recent date

    if do('rss2'):
        s.url('=%s/%s/UE-rss2' % (root, ys))
        c = s.url('=%s/%s/UE-rss2/1/0/column_attr_comment/COCO2/COCO2' % (root, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-rss2/1/1/cell_change/0_0/L1/18765432' % (root, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-rss2/1/2/cell_change/COCO2/L1/4.321' % (root, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-rss2/1/3/comment_change/COCO2/L1/5.432' % (root, ys))
        assert(c == ok_png)

        ss.start()
        c = s.url('=18765432/rsskey')
        key = utilities.manage_key('LOGINS', '18765432/rsskey')

        c = ss.url('%s/rss/%s' % (ys, key))
        assert('COCO2' in c)
        assert('4.321' in c)
        assert('5.432' in c)

        c = s.url('=%s/%s/UE-rss2/1/4/column_attr_visibility/COCO2/1' % (root, ys))
        assert(c == ok_png)
        c = retry(lambda: ss.url('%s/rss/%s' % (ys, key)),
            lambda c: 'COCO' not in c)
        assert('4.321' not in c)
        assert('5.432' not in c)

    if do('suivi'):
        create_tt()
        create_referents()
        create_u2()
        ss.start()
        c = ss.url('=' + abj + '/%s/10800000' % ys)
        assert('"ue": "UE-INF20UE2"' in c)
        assert_col({"the_id":"col_0","type":"Note","author":root,
                   "position":6,"title":"TITLE0"}, c)
        assert( c.count('start_display()') == 1)
        c = ss.url('=' + abj + '/%s/10800000,10800001' % ys)
        assert( c.count('start_display()') == 2)

        assert(', [11.11, "%s", "' % root in c)
        assert('"rank": 1' in c)
        assert('"rank_grp": 0' in c)
        assert('"nr_in_grp": 1' in c)
        assert('"nr": 2' in c)
        assert('"average": 16.665' in c)
        assert('"mediane": 11.11' in c)

        c = s.url('=' + root + '/%s/UE-XXX9999L' % ys)
        c = s.url('=' + root + '/%s/UE-XXX9999L' % ys +
                  '/1/0/column_attr_type/col_0/Bool')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-XXX9999L' % ys +
                  '/1/1/column_attr_title/col_0/TITLE0')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-XXX9999L' % ys +
                  '/1/2/column_attr_modifiable/col_0/1')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-XXX9999L' % ys +
                  '/1/3/column_attr_modifiable/col_0/a')
        assert(c == bad_png)

        c = retry(lambda: ss.url('=' + abj + '/%s/10800000' % ys),
                  lambda c: '"ue": "UE-INF20UE2"' in c and '"ue": "UE-XXX9999L"' in c)

        assert('toto' in c)

        c = s.url('=' + abj + '/%s/UE-XXX9999L/cell/col_0/0_0/%s' % (
                ys, _("yes")))
        assert(green in c)

        c = s.url('=' + abj + '/%s/UE-XXX9999L/comment/col_0/0_0/%s' % (
                ys, _("yes")))
        assert(red in c)

        c = ss.url('=' + abj + '/%s/unload/UE-XXX9999L' % ys)
        assert(c == '')

        c = ss.url('=' + abj + '/%s/10800000' % ys)
        assert('"ue": "UE-INF20UE2"' in c)
        assert('"ue": "UE-XXX9999L"' in c)

        c = ss.url('=' + abj + '/%s/%s/10800007' % (year-1, semester)) # coverage referent_old
        assert('<br class="noprint">' in c)

        c = s.url('=' + root + '/%s/UE-XXX9999L/cell/col_0/0_0/%s' % (
                ys, _("no")))
        assert(red in c)

    if do('suivi-computed'):
        s.url('=%s/%s/UE-Computed' % (abj, ys))
        c = s.url('=%s/%s/UE-Computed/1/0/cell_change/0_0/L1/10800000' % (abj, ys))
        assert( c == ok_png)
        time.sleep(1) # So the average to not overwrite 12.43 with NaN
        c = s.url('=%s/%s/UE-Computed/1/1/column_attr_title/A/A' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/2/column_attr_title/B/B' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/3/cell_change/A/L1/12.34' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/4/cell_change/B/L1/12.43' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/5/column_attr_type/A/Moy' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/6/column_attr_columns/A/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/7/column_attr_type/B/Moy' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/8/column_attr_columns/B/Firstname' % (abj, ys))
        assert( c == ok_png)
        ss.start()
        c = ss.url('=%s/%s/10800000' % (abj, ys))
        assert('[10800000.0, "*"' in c)
        assert('[12.43, "tt.master"' in c) # Keep grade because average is NaN
        c = s.url('=%s/%s/UE-Computed/1/9/column_attr_type/A/Note' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-Computed/1/10/column_attr_type/B/Note' % (abj, ys))
        assert( c == ok_png)
        c = ss.url('=%s/%s/10800000' % (abj, ys))
        assert('[10800000.0, "*"' not in c)
        assert('[NaN, "?"' not in c) # Really can't fail
        assert('[12.34, "tt.master"' in c) # Thats the real point: the value is restored
        assert('[12.43, "tt.master"' in c)

    if do('private'):
        c = s.url('=' + root + '/%s/UE-INF11UE2' % ys)
        c = s.url('=' + root + '/%s/UE-INF11UE2/1/0/table_attr_masters/' % ys + root)
        assert( c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF11UE2/1/1/table_attr_private/1' % ys)
        assert( c == ok_png)
        c = s.url('=' + abj + '/%s/UE-INF11UE2' % ys)
        assert(_("MSG_new_page_unauthorized") in c)
        c = s.url('=' + root + '/%s/UE-INF11UE2/1/2/cell_change/0_0/0_0/10800000' % ys)
        time.sleep(0.1)
        c = s.url('=' + root + '/%s/UE-INF11UE2/1/3/cell_change/0_3/0_0/X'% ys)
        assert( c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF11UE2' % ys)
        assert( ('C("X","%s"' % root) in c)
        ss.start()
        c = ss.url('%s/rss2/UE-INF11UE2' % ys)
        assert(_("MSG_suivi_student_RSS_table").split("<br>")[1] % 2
               in c)
        c = ss.url('=' + abj + '/%s/10800000' % ys)
        assert('[], [], ["X", "%s"' % root in c)

        c = ss.url('%s/rss2/badUE-INF11UE2' % ys)
        assert("</channel>" not in c)

    if do('resume'):
        create_u2()
        ss.start()
        c = ss.url('=' + root + '/%s/resume/UE-INF20UE2/UE-INF20UE2/BADUE-INF20UE2' % ys)
        # assert('lines = {"0": [C("10800001"),C(),C(),C(1),C(1),C()],\n"1": [C("10800000"),C("Jacques"),C("MARTIN"),C(1),C(1),C()]}' in c)
        assert(': [C("10800001"),C("10800001Firstname"),C("10800001Surname"),C(2),C(2),C()]' in c)
        assert(': [C("10800000"),C("Jacques"),C("MARTIN"),C(1),C(1),C()]' in c)

    if do('delete_this_table'):
        create_u2()
        c = s.url('=' + abj + '/%s/UE-INF20UE2/delete_this_table' % ys)
        assert(_("MSG_extension_not_master") in c)
        c = s.url('=ue1.master/%s/UE-INF20UE2/delete_this_table' % ys)
        assert(_("MSG_delete_this_table_done") in c)
        assert(check('%s/UE-INF20UE2.py', exists=False))
        assert(check('%s/UE-INF20UE2.pyc', exists=False))

    if do('empty_is'):
        c = s.url('=' + root + '/%s/UE-INF20UE3' % ys)
        c = s.url('=' + root + '/%s/UE-INF20UE3/1/0/column_attr_title/col_0/TITLE0' % ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-INF20UE3' % ys)
        c = s.url('=' + abj + '/%s/UE-INF20UE3/2/0/column_attr_title/col_1/TITLE1' % ys)
        assert(c == ok_png)

        c = s.url('=' + abj + '/%s/UE-INF20UE3/2/1/column_attr_title/col_0/TITLE2' % ys)
        assert(c == bad_png)

        c = s.url('=' + abj + '/%s/UE-INF20UE3/2/2/column_attr_empty_is/col_1/EMPTY1' % ys)
        assert(c == ok_png)

        c = s.url('=' + abj + '/%s/UE-INF20UE3/2/3/column_attr_empty_is/col_0/EMPTY0' % ys)
        assert(c == bad_png)

    if do('visibility'):
        s.url('=' + root + '/%s/UE-INF20UE9' % ys)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/0/column_attr_visibility_date/col_0/99' % ys)
        assert(c == bug_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/1/column_attr_title/col_0/TITLE99' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/2/column_attr_visibility_date/col_0/99' % ys)
        assert(c == bug_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/3/column_attr_visibility_date/col_0/%s' % (ys, now_plus_32_days))
        assert(c == bad_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/4/column_attr_visibility_date/col_0/%s' % (ys, now_plus_30_days))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/5/cell_change/0_0/lin_0/10123456' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/6/cell_change/col_0/lin_0/7.77' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/7/column_attr_title/col_1/TITLE88' % ys)
        assert(c == ok_png)
        now = time.strftime('%Y%m%d')
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/8/column_attr_visibility_date/col_1/%s' % (ys, now))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/9/cell_change/col_1/lin_0/1.234' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF20UE9/1/10/cell_change/col_1/lin_0/8.88' % ys)
        assert(c == ok_png)
        ss.start()
        c = ss.url('=' + abj + '/%s/10123456' % ys)
        assert('"ue": "UE-INF20UE9"' in c)
        assert('7.77' in c)
        assert('8.88' in c)
        # This history is visible
        # assert('1.234' in c) # No more True

        # The student the rank 7.77 is not in the file (not visible)
        c = ss.url('=' + abj + '/%s/%%2010123456' % ys)
        assert('7.77' not in c)
        assert('8.88' in c)
        # This history is not visible
        assert('1.234' not in c)

        c = s.url('=' + root + '/%s/UE-INF20UE9/1/11/column_attr_visibility_date/col_1/20000101' % ys)
        assert(c == bad_png) # past date

    if do('visibility2'):
        s.url('=' + root + '/%s/UE-INF22UE9' % ys)
        c = s.url('='+root+'/%s/UE-INF22UE9/1/0/column_attr_visibility/col_0/1' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-INF22UE9/1/1/cell_change/0_0/lin_0/10045678' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-INF22UE9/1/2/cell_change/col_0/lin_0/12.31' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-INF22UE9/1/3/column_attr_visibility/col_1/2' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-INF22UE9/1/4/cell_change/col_1/lin_0/19.62' % ys)
        assert(c == ok_png)
        
        ss.start()
        retry(lambda: ss.url('=' + abj + '/%s/10045678' % ys),
              lambda c: '"ue": "UE-INF22UE9"' in c and '12.31' in c and '19.62' not in c
             )
        c = ss.url('=' + abj + '/%s/%%2010045678' % ys)
        assert('12.31' not in c)
        assert('19.62' not in c)
        
    if do('import'):
        s.url('=' + root + '/%s/UE-INF21UE9' % ys)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/0/column_attr_title/col_0/T1' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/1/column_attr_title/col_1/T2' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/2/cell_change/0_0/0_0/10123456' % ys)
        assert(c == ok_png)
        f = open('xxx.csv', 'w', encoding = "utf-8")
        f.write('qqqq,qqq,qqqq\n10123456,4.4444,XXXXYYYY\nttt,yyy,uuu,uu')
        f.close()
        csv = ('file://%s/xxx.csv' % os.getcwd()).replace('/','$2F')
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/3/column_attr_comment/col_0/xxIMPORT(%s$232)yy' % (ys, csv))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/4/column_attr_comment/col_1/xxIMPORT(%s$233)yy' % (ys, csv))
        assert(c == ok_png)
        ss.start()
        c = retry(lambda: ss.url('=' + abj + '/%s/10123456' % ys), lambda c: 'xxyy' in c)
        assert('4.4444' in c)
        assert('XXXXYYYY' in c)

        # Change values
        f = open('xxx.csv', 'w', encoding = "utf-8")
        f.write('qqqq,qqq,qqqq\n10123456,val=é,val=è\r\nttt,yyy,uuu,uu\n1,2,3')
        f.close()
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/5/column_attr_comment/col_0/xxIMPORT()yy' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/6/column_attr_comment/col_1/xxIMPORT()yy' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/7/column_attr_comment/col_0/xxIMPORT(%s$232)yy' % (ys, csv))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/8/column_attr_comment/col_1/xxIMPORT(%s$233)yy' % (ys, csv))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9' % ys)
        assert('4.4444' in c)
        assert('XXXXYYYY' in c)
        assert('val=é' in c)
        assert('val=è' in c)
        f = open('xxx.csv', 'w', encoding = "utf-8")
        f.write('qqqq,qqq,qqqq\r10123456,val2=\xe9,val2=\xe8\rttt,yyy,uuu,uu\r1,2,3')
        f.close()
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/9/column_attr_comment/col_0/xxIMPORT()yy' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/10/column_attr_comment/col_1/xxIMPORT()yy' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/11/column_attr_comment/col_0/xxIMPORT(%s$232)yy' % (ys, csv))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9/1/12/column_attr_comment/col_1/xxIMPORT(%s$233)yy' % (ys, csv))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-INF21UE9' % ys)
        assert('4.4444' in c)
        assert('XXXXYYYY' in c)
        assert('val=é' in c)
        assert('val=è' in c)
        assert('val2=é' in c)
        assert('val2=è' in c)

    if do('past'):
        # Normal user can't modify the past
        utilities.mkpath('DBregtest/Y%d/SAutomne' % (year-1))
        utilities.write_file('DBregtest/Y%d/SAutomne/UE-pastue.py'
                                  % (year-1),
                             """# -*- coding: utf8 -*-
from data import *
new_page('' ,'*', '', '', None)
""")    
        c = s.url('=' + abj + '/%d/Automne/UE-pastue' % (year-1))
        assert('columns = [' in c)
        assert('modifiable:0' in c)
        c = s.url('=' + abj + '/%d/Automne/UE-pastue/1/0/column_attr_comment/col_0/test' % (year-1))
        assert(c == bad_png)
        c = s.url('=' + abj + '/%d/Automne/UE-pastue/1/1/table_attr_modifiable/1' % (year-1))
        assert(c == bad_png)

        # root can modify table attribute in the past.
        s.url('=' + root + '/%d/Automne/UE-pastue' % (year-1))
        c = s.url('=' + root + '/%d/Automne/UE-pastue/2/0/table_attr_masters/%s' % (year-1, abj))
        assert(c == bad_png)
        c = s.url('=' + root + '/%d/Automne/UE-pastue/2/1/table_attr_modifiable/1' % (year-1))
        assert(c == ok_png)
        c = s.url('=' + root + '/%d/Automne/UE-pastue/2/2/table_attr_masters/%s' % (year-1, abj))
        assert(c == ok_png)
        c = s.url('=' + root + '/%d/Automne/UE-pastue/2/3/table_attr_modifiable/0' % (year-1))
        assert(c == ok_png)

        # abj can now modify the table attributes (not values)
        c = s.url('=' + abj + '/%d/Automne/UE-pastue/1/2/column_attr_comment/col_0/test2' % (year-1))
        assert(c == bad_png)
        c = s.url('=' + abj + '/%d/Automne/UE-pastue/1/3/table_attr_comment/col_0/cococo' % (year-1))
        assert(c == bad_png)
        c = s.url('=' + abj + '/%d/Automne/UE-pastue/1/4/table_attr_modifiable/1' % (year-1))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%d/Automne/UE-pastue/1/5/column_attr_comment/col_0/test3' % (year-1))
        assert(c == ok_png)

        # Can't create an UE in the past
        s.url('=' + root + '/%d/Automne/UE-pastue2' % (year-1))
        assert(c == ok_png)
        assert(os.path.exists('DBregtest/Y%d/SAutomne/UE-pastue.py' % (year-1)))
        assert(not os.path.exists('DBregtest/Y%d/SAutomne/UE-pastue2.py' % (year-1)))
                             
    if do('modifiable'):
        c = s.url('=' + abj +'/%s/UE-modif' % ys)
        assert('modifiable:1' in c)
        c = s.url('=' + abj +'/%s/UE-modif/1/0/table_attr_modifiable/0' % ys)
        assert(c == bad_png)
        c = s.url('=' + abj +'/%s/UE-modif/1/1/table_attr_masters/%s'%(ys,abj))
        assert(c == ok_png)
        c = s.url('=' + abj +'/%s/UE-modif/1/2/table_attr_modifiable/0' % ys)
        assert(c == ok_png)
        check('Y%d/S%s/UE-modif.py' % (year, semester), nr_pages = 2)

        c = s.url('=' + abj +'/%s/UE-modif' % ys)
        assert('modifiable:0' in c)
        c = s.url('=' + abj +'/%s/UE-modif' % ys)
        assert('modifiable:0' in c)
        c = s.url('=' + abj +'/%s/UE-modif/3/0/table_attr_masters/'%(ys))
        assert(c == bad_png)
        check('Y%d/S%s/UE-modif.py' % (year, semester), nr_pages = 2)
        c = s.url('=' + abj +'/%s/UE-modif/3/1/table_attr_modifiable/1'%(ys))
        assert(c == ok_png)
        check('Y%d/S%s/UE-modif.py' % (year, semester), nr_pages = 4)

    if do('lostpage'):
        c = s.url('=' + abj +'/%s/UE-lost' % ys)
        assert('modifiable:1' in c)
        c = s.url('=' + abj +'/%s/UE-lost/1/0/table_attr_masters/%s'%(ys,abj))
        assert(c == ok_png)
        c = s.url('=' + abj +'/%s/UE-lost/1/1/table_attr_modifiable/0' % ys)
        assert(c == ok_png)
        check('Y%d/S%s/UE-lost.py' % (year, semester), nr_pages = 2)
        c = s.url('=' + abj +'/%s/UE-lost' % ys)
        assert('modifiable:0' in c)
        check('Y%d/S%s/UE-lost.py' % (year, semester), nr_pages = 2)
        s.stop()
        s.restart()
        check('Y%d/S%s/UE-lost.py' % (year, semester), nr_pages = 2)
        c = s.url('=' + abj +'/%s/UE-lost/0/0' % ys)
        assert('click_to_revalidate_ticket' in c)
        c = s.url('=' + abj +'/%s/UE-lost/2/0' % ys)
        assert('window.parent.Xcell_change' in c)
        
    if do('template_reload'):
        f = open('TEMPLATES/xxx_regtest.py', 'w', encoding = "utf-8")
        f.write('''
from data import ro_user
def content(table):
    return "XXX_REGTEST1"
def create(table):
    table.new_page("", ro_user, "", "")
''')
        f.close()
        t = time.time()
        c = s.url('=' + abj +'/%s/xxx_regtest-1' % ys)
        assert("XXX_REGTEST1" in c)
        while time.time() - t < 1:
            pass
        f = open('TEMPLATES/xxx_regtest.py', 'a', encoding = "utf-8")
        f.write('def content(table): return "XXX_REGTEST2"\n')
        f.close()
        c = s.url('=' + abj +'/%s/xxx_regtest-3' % ys)
        assert("XXX_REGTEST2" in c)
        from ..TEMPLATES import xxx_regtest
        os.unlink(xxx_regtest.__cached__)

    if do('code_etape'):
        c = s.url('=' + abj +'/%s/UE-etape' % ys)
        assert('modifiable:1' in c)
        c = s.url('='+abj+'/%s/UE-etape/1/0/column_attr_type/A/Code_Etape'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/1/cell_change/0_0/L1/10800000' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/2/cell_change/0_0/L2/10800001' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/3/column_attr_columns/A/ID' % ys)
        assert(c == ok_png)

        c = s.url('=' + abj +'/%s/UE-etape' % ys)
        assert('etape-10800000' in c)
        assert('etape-10800001' in c)
        c = s.url('='+abj+'/%s/UE-etape/1/4/cell_change/0_0/L3/10800002' % ys)
        assert(c == ok_png)

        c = s.url('=' + abj +'/%s/UE-etape' % ys)
        assert('etape-10800002' in c)

        c = s.url('='+abj+'/%s/UE-etape/1/5/table_attr_masters/%s' % (ys,abj))
        assert(c == ok_png)

        utilities.mkpath('DBregtest/Y%d/S%s' % (year-1, semester))
        os.rename('DBregtest/Y%d/S%s/UE-etape.py' % (year, semester),
        'DBregtest/Y%d/S%s/UE-etape.py' % (year-1, semester))
        
        c = s.url('=' + abj +'/%s/UE-etape' % ys_old)
        assert('modifiable:0' in c)
        c = s.url('=' + abj +'/%s/UE-etape/1/6' % ys_old)
        assert('set_updating(1);' in c)
        c = s.url('='+abj+'/%s/UE-etape/1/6/table_attr_modifiable/1' % ys_old)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/7/cell_change/0_0/L4/10800003' % ys_old)
        assert(c == ok_png)
        c = retry(lambda: s.url('=' + abj +'/%s/UE-etape' % ys_old), lambda c: 'etape-10800003' in c)

        # Test status
        c = s.url('='+abj+'/%s/UE-etape/1/8/column_attr_columns/S/ID' % ys_old)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/9/column_attr_type/S/Status'% ys_old)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/10/update_content/column' % ys_old)
        assert(c == ok_png)
        c = retry(lambda: s.url('=' + abj +'/%s/UE-etape' % ys_old), lambda c: '"?"' in c)

        # Test civilite (for coverage)
        c = s.url('='+abj+'/%s/UE-etape/1/11/column_attr_columns/S/ID' % ys_old)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/12/column_attr_type/S/Civilite'% ys_old)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-etape/1/13/update_content/column' % ys_old)
        assert(c == ok_png)
        c = retry(lambda: s.url('=' + abj +'/%s/UE-etape' % ys_old), lambda c: '"?"' in c)

    if do('enumeration'):
        c = s.url('=' + abj +'/%s/UE-enum' % ys)
        assert('modifiable:1' in c)
        c = s.url('='+abj+'/%s/UE-enum/1/0/column_attr_type/A/Enumeration'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-enum/1/1/column_attr_columns/A/X%%20Y'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj +'/%s/UE-enum' % ys)
        assert('"columns": "X Y"' in c)

    if do('regtest-bug1'):
       while True:
           c = s.url('='+root+'/0/Dossiers/config_table')
           if '"unload_interval' in c:
              break

       conf = os.path.join('DBregtest','Y0','SDossiers', 'config_table.py')
       ticket_ttl = 1
       c = s.url('='+root+'/0/Dossiers/config_cron')
       assert('page_id = "2"' in c)
       c = s.url('='+root+'/0/Dossiers/config_cron/2/0/cell_change/b/2/0.0001')
       assert(c == ok_png)
       c = s.url('='+root+'/0/Dossiers/config_cron/2/1/cell_change/b/L99/1')
       assert(c == ok_png)
       utilities.write_file(conf + '.old', utilities.read_file(conf))
       utilities.append_file(conf,'''
cell_change(1,'0_2','unload_interval',2,"")
cell_change(1,'0_2','ticket_time_to_live','%d',"")
''' % ticket_ttl)
       s.stop()
       for t in glob.glob(os.path.join('TMP','TICKETS','*')):
           os.unlink(t)
       s.restart(more=['regtest_bug1'])

       c = s.url('='+root+'/0/Dossiers/config_table')
       assert('C(2,"super.user","","",[[600,"' in c) # History

       c = s.url('='+abj+'/%d/Dossiers/regtest-bug1' % uyear)
       assert('runlog' in c)

       # TOMUSS assumes that this page come from the future so it must
       # be from a read-only table (not the case here).
       # So it redirect browser in order to reload the table (read-only)
       c = s.url('='+abj+'/%d/Dossiers/regtest-bug1/10/0' % uyear)
       assert('window.parent.location' in c)
       c = s.url('='+abj+'/%d/Dossiers/regtest-bug1/0/0' % uyear)

       # We are not allowed to see an old page from an other navigator
       assert('window.parent.click_to_revalidate_ticket' in c)

       # This the good page content
       c = s.url('='+abj+'/%d/Dossiers/regtest-bug1/1/0' % uyear,
                 display_log_if_error=False, timeout=1)
       # Because page load does not end
       assert('***TIMEOUT***' in c)
       assert('window.parent.Xcell_change' in c)
       # The frame connection is now broken.
       # Wait the page unloading
       # while True:
       #    c = s.url('='+root+'/stat', silent=True)
       #    if 'regtest-bug1' not in c:
       #        # Wait more to be sure
       #        time.sleep(1)
       #        break
       # print()

       # Create a column
       c = s.url('='+root+'/%d/Dossiers/regtest-bug1/1/0/column_attr_title/col_0/TITLE0' % uyear)
       assert( c == bug_png )

       # Make the ticket last longer because if it is short regtest may fail
       utilities.append_file(conf,
       'cell_change(1,"0_2","ticket_time_to_live","1984","")\n')
       c = s.url('=' + root + '/0/Dossiers/config_table/page_unload')
       assert(_("MSG_page_unload_after") in c)
       c = s.url('=' + root + '/0/Dossiers/config_table')
       assert( 'C("1984","super.user"' in c )

       # The browser attempt to reconnect
       c = s.url('='+abj+'/%d/Dossiers/regtest-bug1/1/0' % uyear,
                 display_log_if_error=False)
       # Because page load does not end
       # assert('***TIMEOUT***' in c)
       assert('window.parent.Xcell_change' in c)

       # The ticket is now valid
       c = s.url('='+abj+'/%d/Dossiers/regtest-bug1/1/0/column_attr_title/col_0/TITLE0' % uyear)
       assert(c == ok_png)

       utilities.write_file(conf, utilities.read_file(conf + '.old'))

       s.stop()
       s.restart()

    if do('modifiable-column'):
        c = s.url('=' + abj +'/%s/UE-modifcol' % ys)
        # Modifiable by teacher
        c = s.url('='+abj+'/%s/UE-modifcol/1/0/column_attr_type/A/Bool'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-modifcol/1/1/column_attr_modifiable/A/1'%ys)
        assert(c == ok_png)
        # Modifiable by student
        c = s.url('='+abj+'/%s/UE-modifcol/1/2/column_attr_type/B/Bool'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-modifcol/1/3/column_attr_modifiable/B/2'%ys)
        assert(c == ok_png)
        # Modifiable by nobody
        c = s.url('='+abj+'/%s/UE-modifcol/1/4/column_attr_type/C/Bool'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-modifcol/1/5/column_attr_modifiable/C/0'%ys)
        assert(c == ok_png)
        # Add 2 students
        c = s.url('=' + abj + '/%s/UE-modifcol' % ys
                  + '/1/6/cell_change/0_0/0_0/10800000')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-modifcol' % ys
                  + '/1/7/cell_change/0_0/0_1/10800001')
        assert(c == ok_png)

        ss.start()
        retry(lambda: ss.url('=' + abj + '/%s/10800001' % ys), lambda c: 'UE-modifcol' in c)
        # assert('UE-modifcol/cell/A/0_1' in c)
        # assert('UE-modifcol/cell/B/0_1' in c)
        # assert('UE-modifcol/cell/C/0_1' not in c)

        c = ss.url('=10800001/%s/' % ys)
        assert('UE-modifcol' in c)
        # assert('UE-modifcol/cell/A/0_1' not in c)
        # assert('UE-modifcol/cell/B/0_1' in c)
        # assert('UE-modifcol/cell/C/0_1' not in c)

        # Not modifiable by student
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/C/0_1/' + _("no"))
        assert(red in c)
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/A/0_1/' + _("no"))
        assert(red in c)

        # For the teacher
        c = s.url('=' + abj + '/%s/UE-modifcol' % ys + '/cell/C/0_1/'+_("yes"))
        assert(green in c)
        # assert('not allowed' in c)
        c = s.url('=' + abj + '/%s/UE-modifcol' % ys + '/cell/A/0_1/'+_("yes"))
        assert(green in c)
        c = s.url('=' + abj + '/%s/UE-modifcol' % ys + '/cell/B/0_1/'+_("yes"))
        assert(green in c)

        # The student can't modify a teacher note
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/B/0_1/' + _("no"))
        assert(red in c)
        # The student can't modify another student note
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/B/0_0/' + _("no"))
        assert(red in c)

        # Teacher erase the note
        c = s.url('=' + abj + '/%s/UE-modifcol' % ys + '/cell/B/0_1/')
        assert(green in c)
        # The student can now modify its note
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/B/0_1/' + _("no"))
        assert(green in c)
        # The student can change its note
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/B/0_1/' + _("yes"))
        assert(green in c)

        # Try to send garbage
        c = s.url('=10800001/%s/UE-modifcol' % ys + '/cell/B/0_1/GOOD_OR_BAD')
        assert(green in c) # No check on cell values : good or bad ?

        c = retry(lambda: ss.url('=10800001/%s/' % ys), lambda c: 'GOOD_OR_BAD' in c)
        assert('<h1>Javascript is needed for this application</h1>' in c)

        
    if do('repetition'):
        c = s.url('=' + abj +'/%s/UE-repetition' % ys)
        c =s.url('='+abj+'/%s/UE-repetition/1/0/column_attr_repetition/A/1'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/1/cell_change/A/L1/10' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/2/cell_change/A/L2/10' % ys)
        assert(c == bad_png)

        c =s.url('='+abj+'/%s/UE-repetition/1/3/column_attr_repetition/B/1'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/4/column_attr_type/B/Enumeration' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/5/column_attr_enumeration/B/QQQQ%%20WWWW' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/6/cell_change/B/L2/QQQQ' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/7/column_attr_title/B/EEE' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/8/cell_change/0_0/L3/10800001' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/9/cell_change/A/L3/20' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/10/column_attr_modifiable/B/2' % ys)
        assert(c == ok_png)

        # Test repetition within grp/seq

        time.sleep(0.1)
        c =s.url('='+abj+'/%s/UE-repetition/1/11/column_attr_repetition/C/-1'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/12/cell_change/0_3/L3/grpA'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/13/cell_change/0_4/L3/seqA'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/14/cell_change/0_3/L2/grpB'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/15/cell_change/0_4/L2/seqA'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/16/cell_change/0_3/L1/grpA'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/17/cell_change/0_4/L1/seqB'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/18/column_attr_type/C/Enumeration'%ys)
        c =s.url('='+abj+'/%s/UE-repetition/1/19/column_attr_enumeration/C/EEEE%%20RRRR'%ys)
        c =s.url('='+abj+'/%s/UE-repetition/1/20/cell_change/C/L1/EEEE'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/21/cell_change/C/L2/EEEE'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/22/column_attr_modifiable/C/2' % ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/23/cell_change/0_3/L4/grpA'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/24/cell_change/0_4/L4/seqA'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repetition/1/25/cell_change/0_0/L5/10800002' % ys)
        assert(c == ok_png)
        time.sleep(0.1)
        c =s.url('='+abj+'/%s/UE-repetition/1/26/cell_change/0_3/L5/grpA'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/27/cell_change/0_4/L5/seqB'%ys)
        assert(c == ok_png)

        ss.start()
        c = ss.url('=10800001/%s/' % ys)
        assert( '"enumeration": ["", ["WWWW", 1]]' in c)
        assert( '"enumeration": ["", ["EEEE", 1], ["RRRR", 1]]' in c)

        c = ss.url('=10800002/%s/' % ys)
        assert( '"enumeration": ["", ["WWWW", 1]]' in c)
        assert( '"enumeration": ["", ["RRRR", 1]]' in c)

        c =s.url('=10800001/%s/UE-repetition/cell/C/L3/EEEE' % ys)
        assert(green in c)

        c = s.url('=10800001/%s/UE-repetition/cell/C/_/EEEE' % ys)
        assert(green in c)

        c = s.url('=1080BAD1/%s/UE-repetition/cell/C/_/EEEE' % ys)
        assert(red in c)

        c =s.url('=10800002/%s/UE-repetition/cell/C/L5/EEEE' % ys)
        assert(red in c)

        c =s.url('='+abj+'/%s/UE-repetition/1/28/cell_change/C/L4/EEEE'%ys)
        assert(c == bad_png)
        c =s.url('='+abj+'/%s/UE-repetition/1/29/column_attr_repetition/C/a'%ys)
        assert(c == bad_png)

    if do('acls'):
        acls = '=' + root + '/0/Dossiers/config_acls/'
        ss.start()
        # User not in LDAP
        c = s.url('=user.1/%s/UE-acls' % ys)
        assert(c.startswith('<script>window.location = "http://'))
        c = s.url(acls)
        assert('grp:referents' in c)
        c = s.url('=user.1/%s/UE-acls' % ys)
        assert(c.startswith('<script>window.location = "http://'))
        c = s.url(acls + '2/0/cell_change/a/L1/user.1')
        assert(c == ok_png)
        c = s.url(acls + '2/1/cell_change/b/L1/referents')
        assert(c == ok_png)
        c = s.url('=user.1/%s/UE-acls' % ys)
        assert('runlog(columns, lines)' in c)
        retry(lambda: ss.url('=%s/%s/user.2' % (root, ys)),
              lambda c: '"teachers": true' in c)
        retry(lambda: ss.url('=user.1/%s/user.2' % ys),
              lambda c: 'is_a_teacher = 1' in c and 'initialize_suivi_real()' in c)
        
        # User in LDAP
        c = s.url('=user.2/%s/UE-acls' % ys)
        assert('runlog(columns, lines)' in c)

        # User in LDAP but not teacher
        c = s.url('=user.3/%s/UE-acls' % ys)
        assert(c.startswith('<script>window.location = "http://'))

        # Put user.3 in acls using 'table:0/Dossiers/acls' selector

        c = s.url('=' + root + '/0/Dossiers/acls')
        assert('runlog' in c)

        c = s.url(acls)
        assert('user.1' in c)
        c = s.url(acls + '3/0/cell_change/a/L2/table:0$2FDossiers$2Facls')
        assert(c == ok_png)
        c = s.url(acls + '3/1/cell_change/b/L2/staff')
        assert(c == ok_png)
        check('Y0/SDossiers/config_acls.py',
              cell_required = (3, 'a', 'L2', 'table:0/Dossiers/acls'),
              dump=False)

        c = s.url('=user.3/%s/UE-acls' % ys)
        assert('runlog(columns, lines)' not in c)

        c = s.url('='+root+'/0/Dossiers/acls/1/0/column_attr_title/col_id/X')
        assert(c == ok_png)
        c = s.url('='+root+'/0/Dossiers/acls/1/1/cell_change/col_id/0_0/user.3')
        assert(c == ok_png)

        retry(lambda: s.url('=user.3/%s/UE-acls' % ys),
              lambda c: 'runlog(columns, lines)' in c)

        # For coverage
        c = s.url(acls + '3/2/cell_change/b/L3/tt_masters')
        assert(c == ok_png)
        c = s.url(acls + '3/3/cell_change/a/L3/' + abj)
        assert(c == ok_png)

        c = s.url(acls + '3/4/cell_change/b/L4/staff')
        assert(c == ok_png)
        c = s.url(acls + '3/5/cell_change/a/L4/python:not%20login[0].isdigit()')
        assert(c == ok_png)

        c = s.url(acls + '3/6/cell_change/b/L4/tt_masters')
        assert(c == ok_png)
        c = s.url(acls + '3/7/cell_change/a/L4/python:Tru')
        assert(c == ok_png)

    if do('owner'):
        c = s.url('=' + abj +'/%s/UE-owner' % ys)
        assert( 'runlog' in c )
        c = s.url('='+abj+'/%s/UE-owner/1/0/column_attr_type/A/Surname' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-owner/1/1/column_attr_columns/A/ID' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-owner/1/2/cell_change/0_0/a/10800001' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-owner/1/3/cell_change/A/a/FOO' % ys)
        assert( c == bad_png )  # XXX
        c = s.url('='+abj+'/%s/UE-owner/1/4/column_attr_type/A/Text' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-owner/1/5/cell_change/A/a/FOO' % ys)
        assert( c == ok_png )

    if do('signature'):
        ss.start()
        c = ss.url('=' + abj +'/%s/10800011' % ys)
        assert( '["Signature", ""]' in c )
        c = ss.url('=10800011/%s' % ys)
        assert( 'LABEL_signature_new' not in c )
        c = s.url('=' + abj
                  + '/signature_new/10800011/1/sig_message{{{sig_button}}}')

        assert(utilities._("MSG_saved") in c)
        c = ss.url('=10800011/%s' % ys)
        assert('AskQuestion' in c)
        assert('tt.masterSurname' in c)
        assert('sig_message' in c)
        assert('\\x3Esig_button<' in c)
        c = s.url('=10800011/signature/0/sig_button')
        assert( c == ok_png_bin )
        c = s.url('=10800011/signature/0/sig_button') # for coverage only
        assert( c == ok_png_bin )
        c = ss.url('=10800011/%s/signatures/10800012' % ys)
        assert(utilities._('TITLE_signatures') in c)
        assert('sig_message' in c)
        assert(')sig_button<' in c)

        c = ss.url('=%s/%s/signatures/10800012' % (abj, ys))
        assert(utilities._('TITLE_signatures') in c)

        c = s.url('=10800011/signature/-1/dontpanic')
        assert( c == ok_png_bin )
        c = ss.url('=%s/%s/signatures/10800012' % (abj, ys))
        assert('dontpanic' not in c)
        c = utilities.read_file('DBregtest/LOGINS/108/10800011/signatures')
        assert(c.split('\n')[-2].startswith("ask('file:suivi_student_charte.html','do_nothing','',"))


    if do('extract'):
        c = s.url('=' + abj + '/%s/UE-extract' % ys)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/0/column_attr_title/col_0/a')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/1/column_attr_title/col_1/b')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/2/cell_change/0_0/L1/10800099')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/3/cell_change/col_0/L1/1')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/4/cell_change/col_1/L1/2')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/5/column_attr_title/col_2/Moy')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/6/column_attr_type/col_2/Moy')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/7/column_attr_columns/col_2/a$20b')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-extract' % ys +
                  '/1/8/cell_change/0_0/L2/')
        assert(c == ok_png)
        ss.start()
        c = ss.url('=' + root +'/%s/extract/UE-extract:Moy' % ys)
        assert('<td>10800099<td>1.5' in c)
        c = ss.url('=' + root +'/%s/extract/UE-badextract:Moy' % ys)
        assert('Unknown table' in c)
        c = ss.url('=' + root +'/%s/extract/UE-extract:badcol' % ys)
        assert("I can't find the column" in c)
        c = ss.url('=ue1.master/0/Dossiers/extract/config_table:Comment')
        assert('You are not allowed to read' in c)
        
        c = ss.url('=' + root +'/%s/fusion/UE-extract:Moy' % ys)
        assert('<tr><td>10800099<td>10800099Firstname<td>10800099Surname<td>UE-extract<td>1.5' in c)
        c = ss.url('=' + root +'/%s/fusion/UE-badextract:Moy' % ys)
        assert('Unknown table' in c)
        c = ss.url('=' + root +'/%s/fusion/UE-extract:badcol' % ys)
        assert("I can't find the column" in c)
        c = ss.url('=ue1.master/0/Dossiers/fusion/config_table:Comment')
        assert('You are not allowed to read' in c)

        c = ss.url('=' + root +'/%s/fusion_inscrit_author/UE-extract:a' % ys)
        assert('<tr><td>10800099<td>10800099Firstname<td>10800099Surname<td>UE-extract<td><td>1.0<td>%s</tr>' % abj in c)

        c = ss.url('=' + root +'/%s/fusion/UE-extract' % ys)
        assert('<tr><td>10800099<td>10800099Firstname<td>10800099Surname<td>UE-extract<td>1.0<td>a<td>2.0<td>b<td>1.5000000000000002<td>Moy</tr>' in c)

        c = ss.url('=' + root +'/%s/fusion/UE-extract:a:b' % ys)
        assert('<tr><td>10800099<td>10800099Firstname<td>10800099Surname<td>UE-extract<td>1.0<td>a<td>2.0<td>b</tr>' in c)

    if do('cell_writable'):
        c = s.url('=' + root + '/%s/UE-cell_writable' % ys)
        c = s.url('=' + root + '/%s/UE-cell_writable' % ys +
                  '/1/0/column_attr_title/col_0/a')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-cell_writable' % ys +
                  '/1/1/cell_change/0_0/L1/10800099')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-cell_writable' % ys +
                  '/1/2/cell_change/col_0/L1/1')
        assert(c == ok_png)
        
        c = s.url('=' + abj + '/%s/UE-cell_writable' % ys)
        c = s.url('=' + abj + '/%s/UE-cell_writable' % ys +
                  '/2/0/cell_change/col_0/L1/2')
        assert(c == bad_png)

        c = s.url('=' + root + '/%s/UE-cell_writable' % ys +
                  '/1/3/column_attr_cell_writable/col_0/=1')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-cell_writable' % ys +
                  '/2/1/cell_change/col_0/L1/2')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-cell_writable' % ys +
                  '/2/2/cell_change/col_0/L1/3')
        assert(c == bad_png)

        c = s.url('=' + root + '/%s/UE-cell_writable' % ys +
                  '/1/4/column_attr_cell_writable/col_0/@' + abj)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-cell_writable' % ys +
                  '/2/3/cell_change/col_0/L1/4')
        assert(c == ok_png)

    if do('groupcolumn'):
        
        s.url('=' + root + '/%s/UE-groupcolumn' % ys)
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/0/column_attr_title/C0/C0')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/1/column_attr_title/C1/C1')
        assert(c == ok_png)


        # another user set a cell
        s.url('=' + abj + '/%s/UE-groupcolumn' % ys)
        c = s.url('=' + abj + '/%s/UE-groupcolumn' % ys +
                  '/2/0/cell_change/C1/L12/16.61')

        
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/2/column_attr_groupcolumn/C1/C0')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/3/column_attr_groupcolumn/C1/C0')
        assert(c == ok_png)
        n = 4
        for student, group in (
                ('10800111', '1'), # 4
                ('10800112', '1'), # 6
                ('10800113', '2'), # 8
                ('10800114', '2'), # 10
                ('10800115', '2'), # 12
                ('10800116', ''),  # 14
                ('10800117', ''),  # 16
                ):
            c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                      '/1/%s/cell_change/0_0/L%d/%s' % (n, n, student))
            assert(c == ok_png)
            c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                      '/1/%d/cell_change/C0/L%d/%s' % (n+1,n, group))
            assert(c == ok_png)
            n += 2

        # First user modify values
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/%d/cell_change/C1/L4/13.31' % n)
        assert(c == ok_png)
        n += 1
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/%d/cell_change/C1/L8/14.41' % n)
        assert(c == ok_png)
        n += 1
        c = s.url('=' + root + '/%s/UE-groupcolumn' % ys +
                  '/1/%d/cell_change/C1/L14/15.51' % n)
        assert(c == ok_png)

        for cell in (
                (1, 'C1', 'L4', 13.31),
                (1, 'C1', 'L6', 13.31),
                (1, 'C1', 'L8', 14.41),
                (1, 'C1', 'L10', 14.41),
                (2, 'C1', 'L12', 16.61), # untouched
                (1, 'C1', 'L14', 15.51),
                ):
            check('Y%d/S%s/UE-groupcolumn' % (year, semester) + '.py',
                  cell_required = cell, dump=False)
        
    if do('upload'):
        try:
            os.unlink('UPLOAD/%s/UP-upload/A/0_0' % ys)
        except OSError:
            pass
        c = s.url('=' + abj +'/%s/UE-upload' % ys)
        # Modifiable by student
        c = s.url('='+abj+'/%s/UE-upload/1/0/column_attr_type/A/Upload'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload/1/1/column_attr_modifiable/A/2'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload/1/2/column_attr_cell_writable/A/=0.016|='%ys)
        assert(c == ok_png)
        # Add student
        c = s.url('=' + abj + '/%s/UE-upload' % ys
                  + '/1/3/cell_change/0_0/0_0/10800000')
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload/1/4/column_attr_title/B/A.log'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload/1/5/column_attr_title/A/A'%ys)
        assert(c == ok_png)

        ss.start()
        retry(lambda: ss.url('=' + abj + '/%s/10800000' % ys), lambda c: 'UE-upload' in c)
        s.url('=10800000/bad_url') # Create the ticket on GET not POST
        c = s.post('=10800000/%s/UE-upload/upload_post/A/0_0' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), )
               )
        assert('Uploaded file size in bytes: 16' in c)
        assert('Uploaded file type: text/plain; charset=us-ascii' in c)
        assert('>foo.txt<' in c)
        # assert('No virus found.' in c)
        c = utilities.read_file('UPLOAD/%s/UE-upload/A/0_0' % ys,
                                encoding="bytes")
        assert(c == b"the file content")

        c = s.post('=10800000/%s/UE-upload/upload_post/A/0_0' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content 2"), )
               )
        assert('Uploaded file size in bytes: 18' in c)
        c = utilities.read_file('UPLOAD/%s/UE-upload/A/0_0' % ys,
                                encoding="bytes")
        assert(c == b"the file content 2")

        c = utilities.read_file('UPLOAD/%s/UE-upload/A/0_0~' % ys,
                                encoding="bytes")
        assert(c == b"the file content")

        c = s.url('=' + abj +'/%s/UE-upload' % ys)
        assert('C(0.018,"10800000",' in c)
        assert(',"text/plain; charset=us-ascii foo.txt",' in c)
        c = s.post('=10800000/%s/UE-upload/upload_post/A/0_0' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content 3"), )
               )
        assert(utilities._("ERROR_value_not_modifiable") in c)

        c = s.url('='+abj+'/%s/UE-upload/1/6/column_attr_cell_writable/A/?>1h'%ys)
        assert(c == ok_png)
        c = s.post('=10800000/%s/UE-upload/upload_post/A/0_0' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content 4"), )
               )
        assert(utilities._("ERROR_value_not_modifiable") in c)

        c = s.url('='+abj+'/%s/UE-upload/1/7/column_attr_cell_writable/A/?<0.01h'%ys)
        assert(c == ok_png)
        c = s.post('=10800000/%s/UE-upload/upload_post/A/0_0' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT",
                              b"the\rfile\ncontent 5 \201\002"), )
               )
        assert('Your file has been successfully sent.' in c)
        c = utilities.read_file('UPLOAD/%s/UE-upload/A/0_0' % ys,
                                 encoding="bytes" )
        assert(c == b"the\rfile\ncontent 5 \201\002")

        # Check upload log
        c = s.url('='+abj+'/%s/UE-upload/upload_get/A/0_0/foo.txt'%ys)
        assert("the\rfile\ncontent 5" in c)
        c = s.url('='+abj+'/%s/UE-upload/upload_get/A/0_0~/foo.txt'%ys)
        assert(c == 'the file content 2')

        c = s.url('=' + abj +'/%s/UE-upload' % ys)
        assert('text/plain; charset=us-ascii foo.txt' in c)
        assert('C("' + abj + ' 1","?"' in c)

        s.url('='+root+'/%s/UE-upload/upload_get/A/0_0/foo.txt'%ys)
        s.url('='+abj+'/%s/UE-upload/upload_get/A/0_0/foo.txt'%ys)
        c = s.url('=' + abj +'/%s/UE-upload' % ys)
        assert(abj + " 2" in c)

        s.url('=' + abj + '/%s/UE-upload' % ys + '/1/8/cell_change/0_0/L2/10800001')
        c = s.url('='+abj+'/%s/UE-upload/upload_get/A/L2/foo.txt'%ys)
        assert("Can't find this file." in c)

    if do('start_job'):
        c = [[],[],[]]
        t = time.time()
        def f1(c=c):
            c[0].append(time.time()-t)
            time.sleep(0.1)
            return time.time() # Tell that all data before this time is processed
        def f2(c=c):
            c[1].append(time.time()-t)
            time.sleep(0.2)
            return time.time() # Tell that all data before this time is processed
        def f3(c=c):
            c[2].append(time.time()-t)
            time.sleep(0.239)
            return time.time() # Tell that all data before this time is processed
        while time.time() < t + 1:
            utilities.start_job(f1, 0.6, important="f1")
            utilities.start_job(f2, 0.3, important="f2")
            utilities.start_job(f3, 0, important="f3")
            time.sleep(0.001)
        assert(len(c[0]) == 1 and len(c[1]) == 2 and len(c[2]) == 5)
        while len(utilities.current_jobs) != 0:
            time.sleep(0.1)
        assert(len(c[0]) == 2 and len(c[1]) == 2 and len(c[2]) == 5)

    if do('repet-grp'):
        c = s.url('=' + abj +'/%s/UE-repet-grp' % ys)
        c =s.url('='+abj+'/%s/UE-repet-grp/1/0/column_attr_repetition/COL/1'%ys)
        assert(c == ok_png)
        c =s.url('='+abj+'/%s/UE-repet-grp/1/1/column_attr_groupcolumn/COL/Grp'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/2/cell_change/0_0/0_0/10800001')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/4/cell_change/0_0/0_1/10800002')
        assert(c == ok_png)
        time.sleep(0.1) # update student list (erase group)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/5/cell_change/0_3/0_1/A')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/3/cell_change/0_3/0_0/A')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/6/cell_change/COL/0_0/7')
        assert(c == ok_png)
        check('Y%d/S%s/UE-repet-grp.py' % (year, semester),
              cell_required = (1, 'COL', '0_1', 7.))
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/7/cell_change/0_0/0_2/10800003')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/8/cell_change/0_0/0_3/10800004')
        assert(c == ok_png)
        time.sleep(0.1) # update student list (erase group)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/9/cell_change/0_3/0_2/B')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/10/cell_change/0_3/0_3/B')
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-repet-grp/1/11/column_attr_repetition/COL/2'
                  % ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-repet-grp' % ys +
                  '/1/12/cell_change/COL/0_3/7')
        assert(c == ok_png)
        check('Y%d/S%s/UE-repet-grp.py' % (year, semester),
              cell_required = (1, 'COL', '0_2', 7.))

    if do('nmbr'):
        # Check Nmbr with 0 columns
        c = s.url('=' + abj +'/%s/UE-nmbr' % ys)
        c = s.url('='+abj+'/%s/UE-nmbr/1/0/column_attr_title/A/A' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-nmbr/1/1/column_attr_type/A/Nmbr' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-nmbr/1/2/cell_change/0_0/0_0/10800100' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-nmbr/1/3/column_attr_title/B/B' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-nmbr/1/4/column_attr_type/B/Note' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-nmbr/1/5/cell_change/B/0_0/11.11' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-nmbr/1/6/column_attr_type/C/Moy' % ys)
        assert( c == ok_png )
        c = s.url('='+abj+'/%s/UE-nmbr/1/7/column_attr_columns/C/A%%20B' % ys)
        assert( c == ok_png )
        ss.start()
        c = ss.url('=' + root + '/10800100')
        assert('"columns": "A B"' in c)
        assert(' 11.11,' in c)
        assert(' 5.555,' in c)

    if do('js-api'):
        c = s.url('=' + abj +'/%s/UE-js' % ys)
        data = []
        for script in c.split('<script ')[1:]:
            url = script.split(':%d/' % configuration.server_port
                           )[1].split('"')[0]
            data.append(s.url(url, read_bytes=True))
        utilities.write_file("xxx.js.gz", b''.join(data), encoding="bytes")
        data = []
        for script in c.split('<script>')[1:]:
            script = script.replace("<!--", "")
            script = script.split("</script>")[0].replace("-->", "")
            script = script.replace("\ninitialize()", "\n")
            data.append(script)
        utilities.write_file("xxx.js", '\n'.join(data))
        f = os.popen("""
        (
         cat REGTEST_SERVER/preamble.js xxx.js
         zcat xxx.js.gz
         echo '
         year = 2020;'
         cat REGTEST_SERVER/tests.js
        ) >xxx_full.js
        (node xxx_full.js || nodejs xxx_full.js) 2>&1 3>&1
        """, "r")
        for c in f.readlines():
            print("NODEJS:" + c.strip())
            if 'tests are fine' in c:
                break
        else:
            js_tests_failed
        f.close()

    if do('import2'):
        s.url('=' + abj + '/%s/UE-imp' % ys)
        c = s.url('=' + abj + '/%s/UE-imp/1/0/column_attr_title/C/N' % ys)
        assert( c == ok_png )
        c = s.url('=' + abj + '/%s/UE-imp/1/1/column_attr_empty_is/C/!Def!'%ys)
        assert( c == ok_png )
        c = s.url('=' + abj + '/%s/UE-imp/1/2/cell_change/0_0/0_0/p0800001'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp/1/3/cell_change/0_0/0_1/p0800002'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp/1/4/comment_change/C/0_1/!com!'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp/1/5/cell_change/0_0/Lempty/'%ys)
        assert(c == ok_png)

        s.url('=' + abj + '/%s/UE-imp2' % ys)
        c = s.url('=' + abj +'/%s/UE-imp2/1/0/cell_change/0_0/0_0/p0800001'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp2/1/1/column_attr_title/C/E' % ys)
        assert( c == ok_png )
        u = ('UE-imp/N').replace('/', '$2F')
        c = s.url('=' + abj + ('/%s/UE-imp2/1/2/column_attr_url_import/C/'%ys)
                  + u)
        assert( c == ok_png )
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert('C("!Def!","*","' in c)

        c = s.url('=' + abj +'/%s/UE-imp2/1/3/cell_change/0_0/0_1/p0800002'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert(c.count("!Def!") == 2)
        assert('!com!' not in c)

        u = ('#UE-imp/N').replace('/', '$2F').replace('#', '$23')
        c = s.url('=' + abj + ('/%s/UE-imp2/1/4/column_attr_url_import/C/'%ys)
                  + u)
        assert( c == ok_png )
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert('!com!' in c)

        # For coverage

        u = ('UE-imp/badcolumn').replace('/', '$2F')
        c = s.url('=' + abj + ('/%s/UE-imp2/1/5/column_attr_url_import/C/'%ys)
                  + u)
        assert(c == ok_png)

        u = ('badyear/%s/UE-imp/N' % semester).replace('/', '$2F')
        c = s.url('=' + abj + ('/%s/UE-imp2/1/6/column_attr_url_import/C/'%ys)
                  + u)
        assert(c == ok_png)

        u = '1900/badsemester/badtable/N'.replace('/', '$2F')
        c = s.url('=' + abj + ('/%s/UE-imp2/1/7/column_attr_url_import/C/'%ys)
                  + u)
        assert(c == ok_png)

        # Extract local column
        c = s.url('=' + abj + ('/%s/UE-imp2/1/8/column_attr_title/P/P'%ys))
        assert(c == ok_png)
        c = s.url('=' + abj + ('/%s/UE-imp2/1/9/column_attr_url_import/C/P'%ys))
        assert(c == ok_png)
        c = s.url('=' + abj +'/%s/UE-imp2/1/10/cell_change/P/0_0/12.23'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert(c.count('12.23') == 2)
        c = s.url('=' + abj +'/%s/UE-imp2/1/11/cell_change/P/0_0/13.33'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert(c.count('13.33') == 2)
        # Extract private column
        c = s.url('=' + abj + ('/%s/UE-imp2/1/12/column_attr_private/P/bigbro'%ys))
        assert(c == bad_png)
        c = s.url('=' + abj + ('/%s/UE-imp2/1/13/column_attr_private/P/%s'%(ys, abj)))
        assert(c == ok_png)
        c = s.url('=' + abj +'/%s/UE-imp2/1/14/cell_change/P/0_0/14.44'%ys)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert(c.count('14.44') == 1)

        # Duplicate import
        u = ('UE-imp/N').replace('/', '$2F')
        c = s.url('=' + abj + ('/%s/UE-imp2/1/15/column_attr_url_import/C/'%ys)
                  + u + '%20' + u)
        c = s.url('=' + abj + '/%s/UE-imp2' % ys)
        assert('"!Def!(UE-imp/N) !Def!(UE-imp/N)"' in c)

        # import bad protocol
        c = s.url('=' + abj + '/%s/UE-imp2/1/16/column_attr_url_import/C/f$3A'%ys)
        assert(c == ok_png)

        ss.start()
        c = ss.url('=ue1.master/%s/extract/UE-imp2:P' % ys)
        assert('You are not allowed to read' in c)
        c = ss.url('=%s/%s/extract/UE-imp2:P' % (abj, ys))
        assert('<td>p0800001<td>14.44</tr>' in c)


    if do('average_change'):
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/0/column_attr_title/col_0/a')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/1/column_attr_type/col_1/Moy')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/2/column_attr_title/col_1/Moy')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/3/column_attr_columns/col_1/a')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/4/cell_change/0_0/lin_0/a_login')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/5/cell_change/col_1/lin_0/16.43')
        assert(c == bad_png) # Average not changeable because not master
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/6/table_attr_masters/' + abj)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-avg_chg' % ys)
        c = s.url('=' + abj + '/%s/UE-avg_chg' % ys +
                  '/2/0/cell_change/col_1/lin_0/16.43')
        assert(c == ok_png) # Average changeable because master
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys)
        assert("16.43" in c)

        # If a value is changed: modified average is not erased server
        # side (except if it is needed)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/7/cell_change/col_0/lin_0/11.11')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys)
        assert("11.11" in c)
        assert("16.43" in c)

        # No more possible to modify the average if it is a number
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/8/cell_change/col_1/lin_0/17.87')
        assert(c == bad_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys)
        assert("17.87" not in c)

        # Erase value: average is modifiable by master
        time.sleep(1)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/9/cell_change/col_0/lin_0/ABJUS')
        assert(c == ok_png)

        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/10/cell_change/col_1/lin_0/9.99')
        assert(c == bad_png)
        c = s.url('=' + abj + '/%s/UE-avg_chg' % ys +
                  '/2/1/cell_change/col_1/lin_0/9.99')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys)
        assert('C("ABJUS"' in c)
        date = c.split('C(9.99,"' + abj + '","')[1][:14]

        # Must be possible to set the same value
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys +
                  '/1/11/cell_change/col_0/lin_0/')
        assert(c == ok_png)
        time.sleep(1)
        c = s.url('=' + abj + '/%s/UE-avg_chg' % ys +
                  '/2/2/cell_change/col_1/lin_0/9.99')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-avg_chg' % ys)
        date2 = c.split('C(9.99,"' + abj + '","')[1][:14]
        assert(date2 > date)

    if do('variable'):
        for k, v in tests_config.vars.items():
            c = s.url('get_var/' + k)
            assert(utilities.literal_eval(c) == v[1])
        c = s.url('=' + root + '/0/Variables/_variables')
        c = re.sub('"[0-9]+"', '""', c)
        assert( 'C("int","?",""),C("","?","")' in c)
        assert( 'C("tuple","?",""),C("(8, 9)","?","")' in c)
        assert( 'C("str","?",""),C("\'7\'","?","")' in c)
        assert( 'C("list","?",""),C("[10, 11]","?","")' in c)
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/0/cell_change/2/test_int/2017')
        assert(c == ok_png)
        c = s.url('get_var/test_int')
        assert(c == "2017")
        ss.start()
        c = ss.url('get_var/test_int')
        assert(c == '2017')
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/1/cell_change/2/test_int/a')
        assert(c == bug_png)
        c = s.url('get_var/test_int')
        assert(c == "2017")
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/2/cell_change/2/test_list/[0,1]')
        assert(c == ok_png)
        c = s.url('get_var/test_list')
        assert(c == "[0, 1]")
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/3/cell_change/2/test_list/')
        assert(c == bug_png)
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/4/cell_change/3/test_list/')
        assert(c == bug_png)
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/5/cell_change/1/test_list/')
        assert(c == ok_png) # the change is not recorded
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/6/cell_change/2/test_int/5.5')
        assert(c == bug_png)
        c = s.url('=' + root +
                  '/0/Variables/_variables/2/7/cell_change/9/test_list/5.5')
        assert(c == bug_png)

        c = s.url('=' + root +
                  '/0/Variables/_variables/2/8/cell_change/2/test_int/2021')
        assert(c == ok_png)
        retry(lambda: ss.url('get_var/test_int'), lambda c: c == '2021')

        c = ss.url('=%s/%s/evaluate/__import__("TOMUSS.document").document.table(0,"Variables","_variables")' % (root, ys))
        assert('TOMUSS.document.Table object at' in c)
        c = ss.url('robots.txt')
        assert('User-agent:' in c)
        assert('Disallow:' in c)

    if do('public'):
        s.url('=' + abj + '/%s/UE-public' % ys)
        c = s.url('='+abj+'/%s/UE-public/1/0/column_attr_title/C/ColT' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-public/1/1/cell_change/C/L/12.34' % ys)
        assert(c == ok_png)
        ss.start()
        c = ss.url('/public/%s/UE-public' % ys)
        assert(c == '')
        c = s.url('='+abj+'/%s/UE-public/1/2/column_attr_visibility/C/3' % ys)
        assert(c == ok_png)
        c = ss.url('=' + abj + '/%s/unload/UE-public' % ys)
        assert(c == '')
        c = ss.url('public/%s/UE-public' % ys)
        cols = utilities.literal_eval(c.split("var columns = ")[1].split(";")[0])
        assert(len(cols) == 1)
        col = cols[0]
        assert(col["author"] == abj)
        assert(col["title"] == "ColT")
        assert(col["visibility"] == 3)
        lines = utilities.literal_eval(c.split("var lines = ")[1].split(";")[0])
        assert(len(lines) == 1)
        line = lines['L']
        assert(len(line) == 1)
        cell = line[0]
        assert(len(cell) == 3)
        assert(cell[0] == 12.34)
        assert(cell[1] == abj)

        c = s.url('='+abj+'/%s/UE-public/1/3/column_attr_visibility/D/3' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-public/1/4/column_attr_cell_writable/D/' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-public/1/5/column_attr_modifiable/D/2' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-public/1/6/cell_change/D/L/6.78' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-public/1/7/column_attr_type/D/Upload' % ys)
        assert(c == ok_png)
        c = ss.url('=' + abj + '/%s/unload/UE-public' % ys)
        assert(c == '')
        c = ss.url('public/%s/UE-public' % ys)
        cols = utilities.literal_eval(c.split("var columns = ")[1].split(";")[0])
        assert(len(cols) == 2)
        assert("modifiable" not in cols[1])
        lines = utilities.literal_eval(c.split("var lines = ")[1].split(";")[0])
        assert(len(lines) == 1)
        line = lines['L']
        assert(len(line) == 2)
        cell = line[1]
        assert(len(cell) == 3)
        assert(cell[0] == 6.78)
        assert(cell[1] == abj)

        c = ss.url('public/%s/bugUE-public' % ys)
        assert(c == '')

        c = s.post('=' + abj + '/%s/UE-public/upload_post/D/L' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), )
               )
        assert('Your file has been successfully sent.' in c)

        c = s.url('=' + abj + '/%s/UE-public/upload_get/D/L' % ys)
        assert(c == "the file content")
        # c = s.url('%s/UE-public/upload_get/D/L' % ys)
        # assert('You are here because you followed a direct link inside TOMUSS'
        #        in c)
        c = s.url('%s/UE-public/upload_get_public/D/L' % ys)
        assert(c == "the file content")
        c = s.url('=garbage/%s/UE-public/upload_get_public/D/L' % ys)
        assert(c == "the file content")

        # Add a public with login
        c = s.url('='+abj+'/%s/UE-public/1/8/column_attr_visibility/E/4' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-public/1/9/cell_change/E/L/13.57' % ys)
        assert(c == ok_png)
        c = ss.url('=' + abj + '/%s/unload/UE-public' % ys)
        c = ss.url('public/%s/UE-public' % ys)
        assert('13.57' not in c)
        c = ss.url('=john.doe/public_login/%s/UE-public' % ys)
        assert('13.57' in c)

    if do('column_private'):
        s.url('=' + root + '/%s/UE-private' % ys)
        c = s.url('='+root+'/%s/UE-private/1/0/column_attr_title/C/ColP' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-private/1/1/cell_change/C/L/12.37' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-private/1/2/column_attr_private/C/%s'
         % (ys, invited))
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-private/1/3/cell_change/0_0/L/10800001' % ys)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-private' % ys)
        assert('12.37' in c)
        c = s.url('=' + invited + '/%s/UE-private' % ys)
        assert('12.37' in c)
        c = s.url('=' + abj + '/%s/UE-private' % ys)
        assert('12.37' not in c)

        ss.start()
        c = ss.url('=10800001/%s/' % ys)
        assert('[12.37' in c)
        c = ss.url('=' + root + '/%s/10800001' % ys)
        assert('"\\u26d4"' in c)
        c = ss.url('=' + abj + '/%s/10800001' % ys)
        assert('[12.37' not in c)
        assert('"\\u26d4"' in c)
        c = ss.url('=' + invited + '/%s/10800001' % ys)
        assert('[12.37' not in c)
        assert('"\\u26d4"' in c)
        c = ss.url('=' + root + '/%s/%%2010800001' % ys)
        assert('[12.37' not in c)
        assert('"\\u26d4"' in c)

    if do('modifiable2'):
        # Set to not modifiable
        c = s.url('=' + abj +'/%s/UE-modif2' % ys)
        assert('modifiable:1' in c)
        c = s.url('=' + abj +'/%s/UE-modif2/1/0/table_attr_masters/%s'%(ys,abj))
        assert(c == ok_png)
        c = s.url('=' + abj +'/%s/UE-modif2/1/1/table_attr_modifiable/0' % ys)
        assert(c == ok_png)
        # Open a new page
        c = s.url('=' + abj +'/%s/UE-modif2' % ys)
        assert('modifiable:0' in c)
        # Open a new page and allow modification
        c = s.url('=' + abj +'/%s/UE-modif2' % ys)
        assert('modifiable:0' in c)
        c = s.url('=' + abj +'/%s/UE-modif2/1/2/table_attr_modifiable/1' % ys)
        assert(c == ok_png)
        # Reload page
        c = s.url('=' + root +'/%s/UE-modif2/page_unload' % ys)
        assert('Unload done' in c)
        # Page should be loadable
        c = s.url('=' + abj +'/%s/UE-modif2' % ys)
        assert('page_id = "4"' in c)

    if do('updatecontent'):
        students = ['1234560{0} FN{0} SN{0} M{0}@x.y A {0}'.format(i)
                    for i in range(7)]
        utilities.write_file('xxx.students', '\n'.join(students[0:4]))
        c = s.url('='+root+'/%s/UE-update' % ys)
        assert('("FN0"' in c)
        assert(c.count('P(') == 4)

        utilities.write_file('xxx.students', '\n'.join(students[4:6]))
        c = s.url('='+root+'/%s/UE-update' % ys)
        assert('("FN0"' in c)
        assert('("FN4"' not in c)
        assert(c.count('P(') == 4)
        c = s.url('='+root+'/%s/UE-update/1/0/update_content/column/mail/abj' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-update' % ys)
        assert('("FN0"' in c)
        assert('("FN4"' not in c)
        assert(c.count('P(') == 4)
        c = s.url('='+root+'/%s/UE-update/1/1/update_content/add' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-update' % ys)
        assert('("FN0"' in c)
        assert('("FN4"' in c)
        assert(c.count('P(') == 6)
        c = s.url('='+root+'/%s/UE-update/3/0/update_content/remove' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-update' % ys)
        assert('("FN4"' in c)
        assert('("FN0"' not in c)
        assert('[["FN0","' in c) # history
        assert(c.count('P(') == 6)

        utilities.write_file('xxx.students', '\n'.join(students[:6]))
        c = s.url('='+root+'/%s/UE-update/1/2/update_content/add' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-update' % ys)
        assert(c.count('P(') == 10)
        for i in range(6):
                assert('("FN{}"'.format(i) in c)

        utilities.write_file('xxx.students', students[6])
        # Should fail if '/update_content/remove' but it doesn't because
        # 'remove' do its job, and the 'add' do not wait one hour to work
        # because the regression tests are running.
        c = s.url('='+root+'/%s/UE-update/1/3/update_content/add/remove' % ys)
        assert(c == ok_png)
        c = s.url('='+root+'/%s/UE-update' % ys)
        for i in range(6):
                assert('("FN{}"'.format(i) not in c)
        assert('("FN6"' in c)
        assert(c.count('P(') == 11)

        c = s.url('='+root+'/%s/UE-update/1/4/table_attr_masters/%s' % (ys, abj))
        assert(c == ok_png)

        # Merge student list with itself to get full coverage
        c = s.url('='+abj+'/%s/UE-update' % ys)
        c = s.url('='+abj+'/%s/UE-update/8/0/update_content/merge_students=UE-update' % ys)
        assert(c == ok_png)

    if do('writable'):
        s.url('=' + abj + '/%s/UE-wc' % ys)
        s.url('='+abj+'/%s/UE-wc/1/0/column_attr_title/Ca/Ca' % ys)
        s.url('='+abj+'/%s/UE-wc/1/1/cell_change/Ca/L/12.34' % ys)
        s.url('='+abj+'/%s/UE-wc/1/2/column_attr_title/Cb/Cb' % ys)
        s.url('='+abj+'/%s/UE-wc/1/3/cell_change/Cb/L/12.35' % ys)
        s.url('='+abj+'/%s/UE-wc/1/4/column_attr_title/Cc/X' % ys)
        s.url('='+abj+'/%s/UE-wc/1/5/column_attr_modifiable/Cc/2' % ys)
        c = s.url('='+abj+'/%s/UE-wc/1/6/column_attr_cell_writable/Cc/[Ca]<[Cb]' % ys)
        assert(c == ok_png)
        c = s.url('='+invited+'/%s/UE-wc/cell/Cc/L/11.11' % ys)
        assert(green in c)
        c = s.url('='+abj+'/%s/UE-wc/1/7/column_attr_cell_writable/Cc/[Ca]>[Cb]' % ys)
        assert(c == ok_png)
        c = s.url('='+invited+'/%s/UE-wc/cell/Cc/L/7.77' % ys)
        assert(red in c)
        # Swap titles
        s.url('='+abj+'/%s/UE-wc/1/8/column_attr_title/Cb/Caa' % ys)
        s.url('='+abj+'/%s/UE-wc/1/9/column_attr_title/Ca/Cb' % ys)
        s.url('='+abj+'/%s/UE-wc/1/10/column_attr_title/Cb/Ca' % ys)
        c = s.url('='+invited+'/%s/UE-wc/cell/Cc/L/6.66' % ys)
        assert(green in c)
        # Use connected name
        s.url('='+abj+'/%s/UE-wc/1/11/column_attr_cell_writable/Cc/@[]=toto' % ys)
        c = s.url('='+invited+'/%s/UE-wc/cell/Cc/L/5.55' % ys)
        assert(red in c)
        s.url('='+abj+'/%s/UE-wc/1/12/column_attr_cell_writable/Cc/@[]=%s' % (ys, invited))
        c = s.url('='+invited+'/%s/UE-wc/cell/Cc/L/4.44' % ys)
        assert(green in c)

    if do('is-a-teacher'):
        s.url('=' + abj + '/%s/UE-iat' % ys)
        s.url('='+abj+'/%s/UE-iat/1/0/column_attr_title/Ca/Ca' % ys)
        s.url('='+abj+'/%s/UE-iat/1/1/column_attr_cell_writable/Ca/$23[]' % ys)
        c = s.url('='+abj+'/%s/UE-iat/cell/Ca/L/3.33' % ys)
        assert(green not in c)
        s.url('='+abj+'/%s/UE-iat/1/2/cell_change/0_0/L/11111111' % ys)
        c = s.url('='+abj+'/%s/UE-iat/cell/Ca/L/3.33' % ys)
        assert(green in c)
        # assert(red in c)
        s.url('='+abj+'/%s/UE-iat/1/3/column_attr_modifiable/Ca/1' % ys)
        c = s.url('='+abj+'/%s/UE-iat/cell/Ca/L/3.33' % ys)
        assert(green in c)
        c = s.url('='+abj+'/%s/UE-iat/cell/Ca/L/4.44' % ys)
        assert(green in c)
        c = s.url('=a_student/%s/UE-iat/cell/Ca/L/5.55' % ys)
        assert(red in c)

    if do('avg-change'):
        s.url('=' + abj + '/%s/UE-fc' % ys)
        s.url('='+abj+'/%s/UE-fc/1/0/column_attr_title/Ca/Ca' % ys)
        s.url('='+abj+'/%s/UE-fc/1/1/column_attr_title/Cb/Cb' % ys)
        s.url('='+abj+'/%s/UE-fc/1/2/column_attr_title/Cc/Cc' % ys)
        s.url('='+abj+'/%s/UE-fc/1/3/column_attr_type/Cc/Moy' % ys)
        s.url('='+abj+'/%s/UE-fc/1/4/column_attr_title/Cd/Cd' % ys)
        s.url('='+abj+'/%s/UE-fc/1/5/column_attr_type/Cd/Moy' % ys)
        s.url('='+abj+'/%s/UE-fc/1/6/column_attr_columns/Cc/Cb' % ys)
        s.url('='+abj+'/%s/UE-fc/1/7/column_attr_columns/Cd/Cc' % ys)
        s.url('='+abj+'/%s/UE-fc/1/8/cell_change/0_0/L1/L1' % ys)
        s.url('='+abj+'/%s/UE-fc/1/9/cell_change/0_0/L2/L2' % ys)
        s.url('='+abj+'/%s/UE-fc/1/10/cell_change/0_0/L3/L3' % ys)
        c = s.url('='+abj+'/%s/UE-fc/1/11/column_attr_title/Cd/Cd' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-fc/1/12/cell_change/Cc/L1/11.1' % ys)
        assert(c == bad_png)

        s.url('='+root+'/%s/UE-fc' % ys)
        s.url('='+root+'/%s/UE-fc/2/0/table_attr_masters/'%ys + abj)

        c = s.url('='+abj+'/%s/UE-fc/1/13/cell_change/Cc/L1/11.1' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-fc/1/14/cell_change/Cb/L2/11.2' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-fc/1/15/cell_change/Cd/L2/11.3' % ys)
        assert(c == bad_png) # Computed
        c = s.url('='+abj+'/%s/UE-fc/1/16/cell_change/Cc/L2/11.4' % ys)
        assert(c == bad_png) # Computed
        c = s.url('='+abj+'/%s/UE-fc/1/17/column_delete/Ca' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-fc/1/18/cell_change/Cd/L3/11.5' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-fc/1/19/cell_change/Cc/L3/11.6' % ys)
        assert(c == ok_png)

    if do('invisible-writable'):
        s.url('='+abj+'/%s/UE-iw' % ys)
        s.url('='+abj+'/%s/UE-iw/1/0/column_attr_title/Ca/.Ca' % ys)
        s.url('='+abj+'/%s/UE-iw/1/1/column_attr_modifiable/Ca/2' % ys)
        c = s.url('='+abj+'/%s/UE-iw/1/2/cell_change/0_0/L0/10800001' % ys)
        assert(c == ok_png)
        c = s.url('=10800001/%s/UE-iw/cell/Ca/L0/1' % ys)
        assert(red in c)

    if do('ask-share'):
        c = s.url('=' + abj + '/ask_share/10800001')
        assert(utilities._("ask_share").split('\n')[0].format(abj) in c)
        c = utilities.read_file('DBregtest/LOGINS/108/10800001/preferences')
        assert(c == "{\n'share_with_tt.master':-1,\n}")

    if do('referentupdate'):
        create_referents()

        c = s.url('=' + abj + '/%s/referents_students' % ys)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/0/cell_change/a/Ldup1/ue2.master' % (page,))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/1/cell_change/a/Ldup2/ue2.master' % (page,))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/2/cell_change/0/Ldup1/10801111' % (page,))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/3/cell_change/0/Ldup2/10802222' % (page,))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/4/column_attr_type/1/Text' % (page,))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/5/cell_change/1/Ldup2/10803333' % (page,))
        assert(c == ok_png)


        c = s.url('=' + root + '/referents_safe')
        assert('Remove student: 10900000' in c)
        assert('Add student: 10800001' in c)
        c = s.url('=' + root + '/referents_safe_R')
        assert('Remove student: 10900000' in c)

        c = s.url('=' + root + '/referents')
        c = c.split('referents_students.py\n', 1)[1]
        d = """<h1>4 students</h1>
<h1>Remove multiple instances of students or teachers</h1>
10900000 not in the student list (a_referent) ('etape-10900000',) REMOVED<br>
Remove teacher: ue2.master<br>
<h1>Students in need of a teacher</h1>
10800001 10800001Firstname 10800001Surname mail@mail.fr ues=[] disciplines={'DISCIPLINE'}<br>
10802222 10802222Firstname 10802222Surname mail@mail.fr ues=[] disciplines=()<br>
<h1>Teacher list sorted by number of students</h1>
<li> a_referent  (0.000000) []
<li> samaccountname  (0.000000) []
<li> toto DISCIPLINE (1.000000) ['10800000']
<li> ue2.master  (3.000000) ['10801111', '10802222', '10803333']
<h1>Affectations</h1>
<li><b>toto[1]</b> ({'DISCIPLINE'}): 10800001 {'DISCIPLINE'}
<li> NO POSSIBLE TEACHER for 10802222 10802222Firstname 10802222Surname mail@mail.fr ues=[] disciplines=()
<h1>Resume</h1>
<li> a_referent<br>
Remove student: 10900000 10900000Firstname 10900000Surname
<li> toto<br>
Add student: 10800001 10800001Firstname 10800001Surname mail@mail.fr ues=[] disciplines={'DISCIPLINE'}
"""
        assert(len(c) == len(d))
        if c != d:
                for x, y in zip(c.split('\n'), d.split('\n')):
                        if x == y:
                                print(x)
                        else:
                                print('*****', x)
                                print('*****', y)
        assert(c == d)

        if ss.started:
                ss.stop()
                ss.restart()
        else:
                ss.start()
        c = ss.url('=' + root + '/referents.csv')
        assert(sorted(c.strip().split('\r\n')) == [
            ';10800000;MARTIN;Jacques;surnametoto;Firstnametoto;mail@toto',
            ';10800001;10800001Surname;10800001Firstname;surnametoto;Firstnametoto;mail@toto',
            ';10801111;10801111Surname;10801111Firstname;ue2.masterSurname;Ue2.Masterfirstname;mail@mail.fr',
            ';10802222;10802222Surname;10802222Firstname;ue2.masterSurname;Ue2.Masterfirstname;mail@mail.fr'])

        c = ss.url('=' + root + '/referents.html')
        assert('<td>10800000</td><td>MARTIN</td><td>Jacques</td>' in c)

        c = ss.url('=' + root + '/toto')
        assert("""[
  "Students",
  [""" in c)
        assert("""
   [
    "10800001",
    "10800001Firstname",
    "10800001Surname",
    "mail@mail.fr"
   ]""" in c)
        assert("""
   [
    "10800000",
    "Jacques",
    "MARTIN",
    "jacques@martin"
   ]""" in c)

        c = s.url('=' + root + '/referents_R')
        assert(c.endswith('''<h1>Resume</h1>
<li> ue2.master<br>
Remove student: 10803333 10803333Firstname 10803333Surname
'''))

        # Verify the merge of dup2 in dup1
        c = s.url('=' + abj + '/%s/referents_students' % ys)
        c = replace_dates(c)
        assert('"Ldup1",[C("ue2.master","tt.master","DATE"),C(),C("10801111","tt.master","DATE"),C("10802222","*","DATE"),C("","*","DATE","",[["10803333","DATE","*","V"]])]' in c)
        assert('"Ldup2",[C("","*","DATE","",[["ue2.master","DATE","tt.master","V"]]),C("","*","DATE"),C("","*","DATE","",[["10802222","DATE","tt.master","V"]]),C("","*","DATE","",[["10803333","DATE","tt.master","V"]]),C()]' in c)

        c = s.url('=ue2.master/')
        assert('"10801111Firstname"' in c)

    if do('ticket_suivi'):
        # Check the redirection for bad ticket on suivi
        c = s.url('=%s/' % abj)
        ticket = c.split('var ticket="')[1].split('"')[0]
        filename = 'TMP/TICKETS/%s' % ticket
        with open(filename, 'r', encoding='utf-8') as file:
            content = file.read()
        new_content = re.sub(r'[0-9.]*\)$', '1111)', content)
        with open(filename, 'w', encoding='utf-8') as file:
            file.write(new_content)
        # Now the ticket is no more valid
        ss.start()
        c = ss.url('%s?ticket=%s' % (abj, abj))
        assert('window.location = %s' % json.dumps(ss.get_url(ys + '/' + abj)) in c)
        with open(filename, 'w', encoding='utf-8') as file:
            file.write(content)

    if do('remove_ticket'):
        s.url('=' + root + '/')
        assert os.path.exists('TMP/TICKETS/' + root)
        utilities.mkpath('TMP/TICKETS_TMP/' + root + '/foo')
        try:
            s.url('=' + root + '/logout', stop_if_error=False, display_log_if_error=False)
        except:
            print()
        for i in range(1000):
            time.sleep(0.01)
            if not os.path.exists('TMP/TICKETS/' + root):
                break
        else:
            ticket_no_destroyed
        s.stop()
        s.restart()
        ss.stop()
        ss.restart()

    if do('send_mail'):
        if os.path.exists("xxx.mail"):
            os.unlink('xxx.mail')
        c = s.post('=' + root + '/send_mail',
                fields = ( ("subject", "_MAIL_SUB_"),
                           ("message", "_MAIL_MSG_"),
                           ("recipients", "_MAIL_@_MAIL_\001root"),
                           ("titles", ""),
                         ),
               )
        assert '100.0%' in c
        time.sleep(0.1)
        with open('xxx.mail') as f:
            c = f.read()
            assert c.count('_MAIL_MSG_') == 3
            assert c.count('_MAIL_SUB_') == 3
            assert '[FOR_ARCHIVAGE]' in c

        os.unlink('xxx.mail')
        c = s.post('=' + root + '/send_mail',
                fields = ( ("subject", "_MAIL_SUB_ [A] [B]"),
                           ("message", "_MAIL_MSG_ [A] [B]"),
                           ("recipients", "_MAIL_@_MAIL_\002VaX\002VaY\001root\002VbX\002VbY"),
                           ("titles", "A\001B"),
                           ("archive", "%s\001toto@fr" % abj),
                         ),
               )
        assert '100.0%' in c
        time.sleep(0.1)
        with open('xxx.mail') as f:
            c = f.read()
            assert c.count('_MAIL_MSG_ VbX VbY') == 1
            assert c.count('_MAIL_MSG_ VaX VaY') == 1
            assert c.count('_MAIL_MSG_ [A] [B]') == 1
            assert c.count('_MAIL_SUB_ VbX VbY') == 1
            assert c.count('_MAIL_SUB_ VaX VaY') == 1
            assert c.count('_MAIL_SUB_ [A] [B]') == 1
            assert '[FOR_ARCHIVAGE]' in c

        os.unlink('xxx.mail')
        c = s.post('=' + root + '/send_mail',
                fields = ( ("subject", "_MAIL_SUB_"),
                           ("message", "_MAIL_MSG_"),
                           ("recipients", "_MAIL_@_MAIL_\001root"),
                           ("cc", "cc1\001cc2@x.fr"),
                           ("titles", ""),
                           ("noreply", "1"),
                         ),
               )
        assert '100.0%' in c

        os.unlink('xxx.mail')
        c = s.post('=' + root + '/send_mail',
                   fields = ( ("subject", "_MAIL_SUB_"),
                              ("message", "_MAIL_MSG_"),
                              ("recipients", ""),
                              ("cc", ""),
                              ("titles", ""),
                              ("noreply", "1"),
                            ))
        # assert("cc2@x.fr" in utilities.read_file('xxx.mail'))

    if do('bookmark'):
        s.url('=' + abj + '/%s/UE-bookmark' % ys)
        c = s.url('=' + abj + '/%s/UE-bookmark' % ys + '/1/0/table_attr_masters/' + abj)
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-bookmark/bookmark' % ys)
        assert(c == ok_png_bin)
        c = utilities.manage_key('LOGINS', os.path.join(abj, 'bookmarked'))
        assert(c == "[\n(%s, 'Printemps', 'UE-bookmark'),\n]" % year)
        c = s.url('=' + abj + '/%s/UE-bookmark/delete_this_table' % ys)
        assert(deletion_done in c)
        retry(lambda: utilities.manage_key('LOGINS', os.path.join(abj, 'bookmarked')),
              lambda c: c == "[\n]")

    if do('login_list'):
        c = s.url('=' + abj + '/login_list/BOA')
        assert c == 'the_full_login_list("BOA",[["k10", "Bill", "BOA", "bboa@africa.net", null, null, null, null, null]], "student");the_full_login_list("BOA",[["k10", "Bill", "BOA", "bboa@africa.net", null, null, null, null, null]], "teacher");'

        s.url('=%s/%s/UE-loginlist' % (abj, ys))

        c = s.url('=%s/tablebuffer/%s/UE-loginlist' % (root, ys))
        assert(r'login_list(' not in c)

        c = s.url('=%s/%s/UE-loginlist/1/0/login_list/BOA/xxxx' % (abj, ys))
        assert(c == ok_png)

        s.url('=%s/tablebuffer/%s/UE-loginlist' % (root, ys))

    if do('clients'):
        c = s.url('=' + root + '/clients/' + str(year))
        assert('lines = {"0": [C("Browser")' in c)

    if do('backtrace'):
        shutil.rmtree('LOGS/BACKTRACES', ignore_errors=True)
        s.url('=' + root + '/backtrace') # Create a backtrace
        c = s.url('=' + root + '/backtrace')
        c = c.split('src="')
        urls = [v.split('"')[0] for v in c[1:]]
        c = s.url('=' + root + '/backtrace/' + urls[0])
        assert('<a target="list" class="important" onfocus="dofocus(event)" href="list/important/' in c)
        c = s.url('=' + root + '/backtrace/' + urls[1])
        assert('<a target="backtrace" onclick="dofocus(event)" class="important" href="' in c)
        c = s.url('=' + root + '/backtrace/' + urls[2])
        assert('<small>PLUGINS/backtrace.py</small></small>/<b>backtrace_home</b>' in c)

    if do('number'):
        ss.start()
        c = s.url('=' + root + '/%s/UE-numbers' % ys)
        c = s.url('=' + root + '/%s/UE-numbers' % ys +
                  '/1/0/column_attr_type/col_0/Text')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-numbers' % ys +
                  '/1/1/cell_change/col_0/0_0/FOOBAR42')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-numbers' % ys +
                  '/1/2/cell_change/0_0/0_0/10800000') # Student ID
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-numbers' % ys +
                  '/1/3/column_attr_course_dates/col_1/20200101')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-numbers' % ys +
                  '/1/4/cell_change/0_4/L1/1')
        assert(c == ok_png)

        c = ss.url('=' + root + '/' + ys + '/number/FOOBAR42/UE-numbers')
        assert('"comment": "Number of FOOBAR42",' in c)
        assert('"0": [C("10800000"),C(),C(),C(1),C()]' in c)

        c = ss.url('=' + root + '/' + ys + '/count/FOOBAR42/UE-numbers')
        assert('"comment": "Number of FOOBAR42",' in c)
        assert('"0": [C("10800000"),C(),C(),C(1),C()]' in c)
        assert(re.findall('"columns": "20[0-9][0-9]-[0-1][0-9]-[0-3][0-9]_[A-Z][a-z]*",', c) )

        c = ss.url('=' + root + '/' + ys + '/count/FOOBAR42/bugUE-numbers')
        assert('Number of' not in c)
        c = ss.url('=' + root + '/' + ys + '/number/FOOBAR42/BADUE-numbers')
        assert('lines = {}' in c)

    if do('gc_state') and not search_leak:
        c = s.url('=' + root + '/atomic_checker')
        assert(' &lt;class &#x27;int&#x27;&gt; ' in c)
        c = s.url('=' + root + '/memory_size_checker', timeout=1, max_size=10000)
        assert('\n0 → ' in c)
        assert('Thread immortal=True' in c)

        s.url('=' + root + '/%s/UE-reqlost' % ys) # Create a lost request
        s.url('=' + root + '/%s/UE-reqlost' % ys + '/1/1/column_attr_type/col_0/COUCOU', timeout=1)
        c = s.url('=' + root + '/gcbig')
        assert('</pre>Total:' in c)
        c = s.url('=' + root + '/threads')
        assert('update_computed_values_fast' in c)
        c = s.url('=' + root + '/locks')
        assert('Locked   dispatch_request [TOMUSS.plugin]' in c)
        c = s.url('=' + root + '/caches')
        assert('</table>' in c)
        assert('firstname surname to login' in c)
        c = s.url('=' + root + '/gc')
        # assert('/type/UE">UE</a>' in c)
        assert('type/CellValue">CellValue</a>' in c)
        c = s.url('=' + root + '/type/list')
        for i in range(5):
            cc = c.split('/object/')[1 + 10*i].split('"')[0]
            cc = s.url('=' + root + '/object/' + cc)
            assert(cc.startswith(b'\x89PNG'))

    if do('configs'):
        c = s.url('=' + root + '/0/Dossiers/config_room')
        assert('"title": "Place",' in c)
        c = s.url('=' + root + '/0/Dossiers/config_room/2/0/cell_change/a/0_0/R1')
        assert(c == ok_png)
        c = s.url('=' + root + '/0/Dossiers/config_room/2/1/cell_change/b/0_0/100-200')
        assert(c == ok_png)
        c = s.url('=' + root + '/0/Dossiers/config_room/2/2/cell_change/c/0_0/foobar')
        assert(c == bug_png)
        c = s.url('=' + root + '/0/Dossiers/config_room/2/3/cell_change/b/0_0/10-20%2030-40%20-15%20-5%20-50')
        assert(c == ok_png)
        # To exercise regression tests
        c = s.url('=' + root + '/0/Dossiers/config_room/2/4/cell_change/b/0_1/20-10')
        assert(c == ok_png)
        c = s.url('=' + root + '/0/Dossiers/config_room/2/5/cell_change/b/0_1/10-20%20-10%20-20')
        assert(c == ok_png)

    if do('acls_check'):
        c = s.url('=' + root + '/0/Dossiers/config_acls')
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=' + root + '/0/Dossiers/config_acls/%s/0/cell_change/a/LL1/user.2' % page)
        assert(c == ok_png)
        c = s.url('=' + root + '/0/Dossiers/config_acls/%s/1/cell_change/b/LL1/staff' % page)
        assert(c == ok_png)
        c = s.url('=' + root + '/acls_check')
        assert('C("ue9.master"),C("ue9.masterFirstname"),C("ue9.masterSurname"),C("mail@mail.fr")' in c)

    if do('all_ips'):
        ss.start()
        create_u2()
        c = ss.url('=' + root + '/ip')
        assert("(-999, 'INF20UE2')" in c  or "[(-998, 'INF22UE9')]" in c)
        # assert("(2, 'INF20UE2')" in c or "(1, 'INF20UE2')" in c  or  "(1, 'groupcolumn')" in c)

    if do('import-zip'):
        utilities.write_file("xxx1.txt", "File content 1")
        utilities.write_file("xxx2.txt", "File content 2")
        utilities.write_file("xxx3.txt", "File content 3")
        zf = zipfile.ZipFile(open("xxx.zip", "wb"),
                         mode="w", compression=zipfile.ZIP_DEFLATED)
        zf.write("xxx1.txt", "10800000#toto.txt")
        zf.write("xxx2.txt", "10800001_titi.txt")
        zf.write("xxx3.txt", "10800002_tata.txt")
        zf.write("xxx3.txt", "10800003_tutu.txt")
        zf.write("xxx3.txt", "__MACOSX/toto.txt")
        zf.close()

        s.url('=' + abj + '/%s/UE-importzip' % ys)
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/0/column_attr_title/C0/ColName')
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/1/cell_change/0_0/L1/10800000')
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/2/cell_change/0_0/L2/10800001')
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/3/cell_change/0_0/L3/10800002')
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/4/cell_change/0_1/L2/FOO')
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/5/cell_change/0_2/L2/BAR')
        c = s.post('=' + abj + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '0') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx.zip", encoding='bytes')), )
               )
        assert('There is a bug' in c)
        s.url('=' + abj + '/%s/UE-importzip' % ys + '/1/6/column_attr_type/C0/Upload')
        c = s.post('=' + abj + '/%s/bugUE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '0') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx.zip", encoding='bytes')), )
               )
        assert('There is a bug' in c)
        s.url('=' + root + '/%s/UE-importzip' % ys)
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/0/table_attr_masters/' + root)
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/1/cell_change/C0/L3/7.77')
        assert(c == ok_png)
        c = s.post('=' + abj + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '0') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx.zip", encoding='bytes')), )
               )
        c = c.replace('No virus found.\n', '')

        assert(c == '''<br>10800000#toto.txt
<br>Wait (maybe some minutes) the message indicating the uploading success.
<br><span>Uploaded file size in bytes: 14</span>
<br><span>Uploaded file type: text/plain; charset=us-ascii</span>
<!--OK--[0.014, 'text/plain; charset=us-ascii toto.txt']--OK--><br>10800001_titi.txt
<br>Wait (maybe some minutes) the message indicating the uploading success.
<br><span>Uploaded file size in bytes: 14</span>
<br><span>Uploaded file type: text/plain; charset=us-ascii</span>
<!--OK--[0.014, 'text/plain; charset=us-ascii titi.txt']--OK--><br>10800002_tata.txt
is not empty<br>10800003_tutu.txt
Can't find the student login<br>__MACOSX/toto.txt
<span class="emoji">🗑</span><p>THE END''')
        c = utilities.read_file('UPLOAD/%s/UE-importzip/C0/L1' % ys, encoding="bytes")
        assert(c == b"File content 1")
        c = utilities.read_file('UPLOAD/%s/UE-importzip/C0/L2' % ys, encoding="bytes")
        assert(c == b"File content 2")

        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/2/column_attr_locked/C0/1')
        assert(c == ok_png)
        c = s.post('=' + invited + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '0') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx.zip", encoding='bytes')), )
               )
        assert('There is a bug' in c)
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/3/column_attr_locked/C0/0')
        assert(c == ok_png)

        c = s.post('=' + abj + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '-1') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx.zip", encoding='bytes')), )
               )
        assert(c.count("Can't find the student login") == 4)

        c = s.post('=' + abj + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '3') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx.zip", encoding='bytes')), )
               )
        assert(c.count("Can't find the student login") == 4)

        c = s.post('=' + abj + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.zip"), ('data_col_key', '0') ),
                   files = ( ("data", "FOO.ZIP", utilities.read_file("xxx1.txt", encoding='bytes')), )
               )
        assert('This is not a valid ZIP file' in c)

        # Import a PDF
        c = s.post('=' + abj + '/%s/UE-importzip/import_zip/C0' % ys,
                   fields = ( ("filename", "foo.pdf"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert(c == '''window.importPDF = new window.parent.ImportPDF("%s_Printemps_UE-importzip_C0",8) ;window.importPDF.add();window.importPDF.add();window.importPDF.add();window.importPDF.add();window.importPDF.add();window.importPDF.add();window.importPDF.add();window.importPDF.add();''' % year)

        c = s.post('=' + abj + '/%s/UE-importzip/upload_pdf/1/C0' % ys,
                    fields = ( ('L1', '2\0012'), # Pages 2 and 3
                               ('L2', '5\0013'), # Pages 5 - 7
                               ('L4', '1\0011'), # Pages 1 NOT IN TABLE
                             ),
               )
        c = c.replace('No virus found.\n', '')
        assert(' : 2-3<br><br>Wait (maybe some minutes) the message indicating the uploading success.\n<br><span>Uploaded file size in bytes: 167' in c)
        assert('<!--OK--[16.7' in c)
        assert(' : 5-7<br><br>Wait (maybe some minutes) the message indicating the uploading success.\n<br><span>Uploaded file size in bytes: 249' in c)
        assert('<!--OK--[24.9' in c)
        assert('L4 does not exists' in c)
        assert('THE END' in c)
        c = utilities.read_file('UPLOAD/%s/UE-importzip/C0/L1' % ys, encoding="bytes")
        assert( 16700 < len(c) < 16800 )
        c = utilities.read_file('UPLOAD/%s/UE-importzip/C0/L2' % ys, encoding="bytes")
        assert( 24900 < len(c) < 25000 )
        assert(not os.path.exists('UPLOAD/%s/UE-importzip/C0/L3' % ys))

        # download a ZIP
        c = s.url('=' + abj + '/%s/UE-importzip' % ys + '/upload_zip/C0/L1/L99/L2/L2/L3', read_bytes=True)
        utilities.write_file('xxx.zip', c, encoding='bytes')
        zf = zipfile.ZipFile(open('xxx.zip', 'rb+'), mode="r")
        assert(sorted(zf.namelist()) == ['%s_Printemps_UE-importzip_ColName/10800000#__ColName.pdf' % year,
                                         '%s_Printemps_UE-importzip_ColName/10800001#FOO_BAR_ColName.pdf' % year
                                        ])
        with zf.open('%s_Printemps_UE-importzip_ColName/10800000#__ColName.pdf' % year, "r") as f:
            assert( 16700 < len(f.read()) < 16800 )
        with zf.open('%s_Printemps_UE-importzip_ColName/10800001#FOO_BAR_ColName.pdf' % year, "r") as f:
            assert( 24900 < len(f.read()) < 25000 )

        s.url('=' + root) # Activate root ticket
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/2/comment_change/C0/L1/badmime')
        assert(c == ok_png)

        # download a ZIP for all
        c = s.url('=' + abj + '/%s/UE-importzip' % ys + '/upload_zip/C0/*/afilename', read_bytes=True)
        utilities.write_file('xxx.zip', c, encoding='bytes')
        zf = zipfile.ZipFile(open('xxx.zip', 'rb+'), mode="r")
        ys_ = ys.replace('/', '_')
        assert(sorted(zf.namelist()) == [ys_ + '_UE-importzip_ColName/10800000#__unknown',
                                         ys_ + '_UE-importzip_ColName/10800001#FOO_BAR_ColName.pdf'])

        # use group
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/3/cell_change/0_0/L4/10800004')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/4/column_attr_groupcolumn/C0/Grp')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/5/cell_change/0_3/L4/GRP1')
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-importzip' % ys + '/2/6/cell_change/0_3/L1/GRP1')
        assert(c == ok_png)

        # download a ZIP for 10800004
        c = s.url('=' + abj + '/%s/UE-importzip' % ys + '/upload_zip/C0/L4/afilename', read_bytes=True)
        utilities.write_file('xxx.zip', c, encoding='bytes')
        zf = zipfile.ZipFile(open('xxx.zip', 'rb+'), mode="r")
        ys_ = ys.replace('/', '_')
        assert(zf.namelist() == ['%s_Printemps_UE-importzip_ColName/10800000#__unknown' % year])

    if do('clean'):
        def put_in_past(ue):
            filename = 'DBregtest/Y{}/S{}/{}.py'.format(year, semester, ue)
            c = utilities.read_file(filename)
            utilities.write_file(filename, c.replace(str(year), '2000'))
            s.url('={}/{}/{}/page_unload'.format(root, ys, ue))
            os.utime(filename, (time.time(), time.time() - 86400))

        s.url('=' + abj + '/%s/UE-empty-today' % ys)
        s.url('=' + abj + '/%s/UE-empty' % ys)
        put_in_past('UE-empty')
        s.url('=' + abj + '/%s/UE-empty-with-line' % ys)
        s.url('=' + abj + '/%s/UE-empty-with-line' % ys + '/1/0/cell_change/0_0/L1/')
        put_in_past('UE-empty-with-line')
        c = s.url('=' + root + '/clean/%s' % ys)
        assert('<b>UE-empty</b><br>' in c)
        assert('<a href="UE-empty-today">UE-empty-today</a> : Created/Modified today<br>' in c)
        assert('<a href="UE-empty-with-line">UE-empty-with-line</a> : 1 lines<br>' in c)
        assert(not os.path.exists('DBregtest/Y{}/S{}/UE-empty.py'.format(year, semester)))
        assert(os.path.exists('DBregtest/Y{}/S{}/UE-empty-today.py'.format(year, semester)))
        assert(os.path.exists('DBregtest/Y{}/S{}/UE-empty-with-line.py'.format(year, semester)))

        s.url('=' + abj + '/0/Public/UE-empty')
        s.url('=' + abj + '/0/Public/UE-empty/page_unload')
        s.url('=' + abj + '/0/Public/UE-empty2') # No active pages, so deleted
        utilities.write_file('DBregtest/Y0/SPublic/empty.py', '')
        c = s.url('=' + root + '/clean_other')
        assert('<br>DBregtest/Y0/SPublic/UE-empty2.py : 0 col. DELETED<br>' in c)
        assert('<br>DBregtest/Y0/SPublic/UE-empty.py : 0 col. DELETED<br>' in c)
        assert('<br>DBregtest/Y0/SPublic/empty.py : 0 col. not deleted, look the content, it is a strange file.<br>' in c)

    if do('groupe'):
        s.url('=' + abj + '/%s/UE-groupe' % ys)
        s.url('=' + abj + '/%s/UE-groupe' % ys + '/1/0/cell_change/0_0/L1/10800000')
        time.sleep(0.1) # student list update erasing grp
        s.url('=' + abj + '/%s/UE-groupe' % ys + '/1/1/cell_change/0_3/L1/A')
        s.url('=' + abj + '/%s/UE-groupe' % ys + '/1/2/cell_change/0_4/L1/1')

        s.url('=' + abj)
        ss.start()
        c = ss.url('={}/{}/groupe'.format(abj, ys))
        assert (r''':8888/={0}/{1}/Printemps/UE-groupe">UE-groupe</a><td>1<td><a href="groupe/UE-groupe(1).csv" onmouseover="display('	10800000 A {0}\x3Cbr\x3E\n')">CSV</script></a><td>A'''.format(abj, year) in c)

        c = ss.url('={}/{}/groupe/UE-groupe(1).csv'.format(abj, ys))
        assert('10800000;;;A' in c)

    if do('annotate'):
        shutil.rmtree('UPLOAD/%s/UE-annotate' % ys, ignore_errors=True)
        s.url('=' + abj + '/%s/UE-annotate' % ys)
        s.url('=' + abj + '/%s/UE-annotate' % ys + '/1/0/column_attr_type/C0/Annotate')
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + "/1/1/column_attr_annotate/C0/%7B%22images%22:%5B%22A%22%5D,%22options%22:%5B%7B%22colors%22:%22%23000,%23F00,%230A0,%2300F%22,%22color%22:%22%23000%22,%22widths%22:%221,4,8,12%22,%22width%22:%221%22,%22sizes%22:%2218,24,32,48%22,%22size%22:%2224%22,%22modes%22:%22Edit,Text,Draw,Highlight%22,%22mode%22:%22Text%22%7D,%7B%22colors%22:%22%23000,%23F00,%230A0,%2300F%22,%22color%22:%22%23F00%22,%22widths%22:%221,4,8,12%22,%22width%22:%221%22,%22sizes%22:%2218,24,32,48%22,%22size%22:%2224%22,%22modes%22:%22Edit,Text,Draw,Highlight,-1,+1,-0.5,+0.5%22,%22mode%22:%22+1%22%7D%5D%7D")
        assert(c == ok_png)

        # Import Errors
        c = s.post('=' + abj + '/%s/bugUE-annotate/annotator_upload/C0' % ys,
                   fields = ( ("letter", "A"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('TABLE???' in c)
        c = s.post('=' + abj + '/%s/UE-annotate/annotator_upload/C99' % ys,
                   fields = ( ("letter", "A"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('COL???' in c)
        c = s.post('=%s/%s/UE-annotate/annotator_upload/C0' % (abj, ys),
                   fields = ( ("letter", "AA"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('Hacker!' in c)

        # Import a PDF
        c = s.post('=' + abj + '/%s/UE-annotate/annotator_upload/C0' % ys,
                   fields = ( ("letter", "A"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert(c == 'OK!<script>window.parent.annotator_add("A")</script>')
        png = s.url('=' + abj + '/%s/UE-annotate/annotator_get/C0/A.png' % ys, read_bytes=True)
        utilities.write_file('xxx.png', png, encoding='bytes')
        import PIL.Image
        image = PIL.Image.open('xxx.png')
        assert(image.size == (1240, 14024))

        c = s.url('=' + abj + '/%s/UE-annotate' % ys + '/1/2/cell_change/0_0/L1/10800001')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + "/1/3/column_attr_modifiable/C0/2")
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-annotate' % ys)
        assert('"modifiable": 2' in c)
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + "/1/4/column_attr_cell_writable/C0/=|@|%23[]")
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + "/1/5/column_attr_title/C0/title")
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + '/1/6/cell_change/0_0/L9/10800002')
        assert(c == ok_png)

        # When the student load, a subject is selected
        png = s.url('=10800002/%s/UE-annotate/annotator_get/C0/L9' % ys, read_bytes=True)
        utilities.write_file('xxx.png', png, encoding='bytes')
        import PIL.Image
        image = PIL.Image.open('xxx.png')
        assert(image.size == (1240, 14024))
        c = s.url('=' + abj + '/%s/UE-annotate' % ys)
        c = replace_dates(c)
        assert('P("L1",[C("10800001","%s","DATE"),C(),C(),C(),C(),C(),C()]);' % abj in c)
        assert('P("L9",[C("10800002","%s","DATE"),C(),C(),C(),C(),C(),C("","","","A",[["A","DATE","","C"]])]);' % abj in c)

        # Record ticket
        ss.start()
        c = ss.url('=10800001/%s' % ys)
        assert('"type": "Annotate"' in c)

        # Set the comment with the subject letter
        c = s.url('=10800001/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert(c == '')

        c = s.url('=' + abj + '/%s/UE-annotate' % ys)
        assert('"),C(),C(),C(),C(),C(),C("","","","A",[["A","' in c)

        # Student answer
        actions = [
            ['new', 1000000000, 1000000, {"type": "textbox", "text": "foo"}],
        ]
        c = s.post('=10800001/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert(c == 'OK!')

        c = s.post('=10800001/%s/UE-annotate' % ys + '/annotator_record/C0/L2',
            fields = ( ("content", json.dumps(actions)), ))
        assert("Can't find line" in c)

        # Renumbering of the new ID of the student answer
        c = s.url('=10800001/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert(c == '''["new", 1000000000, 1, {"type": "textbox", "text": "foo"}, "10800001"]\n''')

        # Student answer a text with a background (forbiden)
        actions = [
            ['new', 1000000000, 1000000, {"type": "textbox", "text": "foo", "backgroundColor": "#C00A"}],
        ]
        c = s.post('=10800001/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert('There is a bug' in c)

        # Check nothing recorded
        c = s.url('=10800001/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert(c == '''["new", 1000000000, 1, {"type": "textbox", "text": "foo"}, "10800001"]\n''')

        # Student try to hack
        actions = [
            ['new', 1000000000, 1000001, {"type": "textbox", "text": "foo"}],
        ]
        c = s.post('=10800001/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert('There is a bug' in c)

        # The student is PRST
        c = s.url('=' + abj + '/%s/UE-annotate' % ys)
        assert('C("PRST","10800001"' in c)


        # Teacher grade
        actions = [
            ['new', 1000000000, 1000000, {"type": "textbox", "text": "foo"}],
            ['chg', 1000000001, 1000000, {"left": 1000, "backgroundColor": "#C00A"}],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert(c == 'OK!')

        # Renumbering of the new ID
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert(c == '''["new", 1000000000, 1, {"type": "textbox", "text": "foo"}, "10800001"]
["new", 1000000000, 2, {"type": "textbox", "text": "foo"}, "%s"]
["chg", 1000000001, 2, {"left": 1000, "backgroundColor": "#C00A"}, "%s"]
''' % (abj, abj))

        # Test del
        actions = [
            ['new', 1000000002, 1000000, {"type": "textbox", "text": "bar"}],
            ['del', 1000000003, 1000000, {}],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert(c == 'OK!')
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert('["del", 1000000003, 3, {}, "tt.master"]' in c)

        actions = [
            ['chg', 1000000004, 3, {"type": "textbox", "text": "gotogarbage"}],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert(c == 'OK!')
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert('gotogarbage' not in c)

        # Add smiley
        actions = [
            ['new', 1000000005, 1000000,
             {"type": "textbox", "text": "😊", "backgroundColor": "#FFF"}],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert(c == 'OK!')
        c = s.url('=' + abj + '/%s/UE-annotate' % ys + '/annotator_load/C0/L1')
        assert('["new", 1000000005, 4,' in c)

        # Hacker
        actions = [
            ['new', 1000000005, 1000000, {"type": "textbox"}, "login"],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert('There is a bug' in c)

        # Forbid modification
        actions = [
            ['chg', 1000000010, 1, {"left": 1000}],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert('There is a bug' in c)

        # Add a grade
        actions = [
            ['new', 1000000020, 1000000, {"type": "textbox", "text": "+0.75", "backgroundColor": "#C00A"}],
        ]
        c = s.post('=' + abj + '/%s/UE-annotate' % ys + '/annotator_record/C0/L1',
            fields = ( ("content", json.dumps(actions)), ))
        assert('OK!' in c)

        c = s.url('=' + abj + '/%s/UE-annotate' % ys)
        assert('C(0.75,"%s"' % abj  in  c)
        assert('"A 😊"' in c)

        s.url('=' + root)
        c = s.post('=%s/%s/UE-annotate/annotator_upload/C0' % (root, ys),
                   fields = ( ("letter", "B"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert(red1 in c)

        # Not allowed
        c = s.url('=10800001/%s/UE-annotate' % ys + '/annotator_load/C0/L2')
        assert(c == '')

    if do('abi_is'):
        ss.start()
        s.url('=' + abj + '/%s/UE-abiis' % ys)
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/0/cell_change/0_0/L1/10800001')
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/1/column_attr_abi_is/C1/1')
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/2/column_attr_title/C0/A')
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/3/column_attr_type/C1/Moy')
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/4/column_attr_columns/C1/A')
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/5/cell_change/C0/L1/UNJLEAVE')
        s.url('=' + abj + '/%s/UE-abiis' % ys + '/1/6/column_attr_abi_is/C1/A')
        c = ss.url('=10800001/%s' % ys)
        assert('["DEF", "?"], ["UNJLEAVE",' in c)

    if do('abj_is'):
        ss.start()
        s.url('=' + abj + '/%s/UE-abjis' % ys)
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/0/column_attr_title/C0/A')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/1/column_attr_title/C1/B')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/2/column_attr_type/C2/Moy')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/3/column_attr_abj_is/C2/0')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/4/column_attr_columns/C2/A%20B')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/5/column_attr_best/C2/1')

        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/6/cell_change/0_0/L1/10800001')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/7/cell_change/C0/L1/JUSLEAVE')
        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/8/cell_change/C1/L1/1')
        time.sleep(0.1)
        c = ss.url('=10800001/%s' % ys)
        c = replace_dates(c)
        assert('["JUSLEAVE", "%s", "DATE"], [1.0, "%s", "DATE"], [NaN, "?"]],' % (abj, abj) in c)

        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/9/column_attr_abj_is/C2/3')
        retry(lambda: replace_dates(ss.url('=10800001/%s' % ys)),
              lambda c:'["JUSLEAVE", "%s", "DATE"], [1.0, "%s", "DATE"], [1.0, "*", "", ""]],' % (abj, abj) in c)

        c = s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/10/column_attr_abj_is/C2/a')
        assert(c == bad_png)

        s.url('=' + abj + '/%s/UE-abjis' % ys + '/1/11/column_attr_best/C2/bug')
        assert(c == bad_png)

    if do('icone'):
        s.url('=%s/%s/UE-icoABI' % (abj, ys))
        c = s.url('=%s/%s/UE-icoABI/1/0/column_attr_title/C1/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/1/column_attr_title/C2/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/2/column_attr_type/C1/Note' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/3/column_attr_type/C2/Note' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/4/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/5/cell_change/C1/L1/ULV' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/6/cell_change/C2/L1/GARBAGE' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/7/column_attr_title/C3/C3' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/8/column_attr_title/C4/C4' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/9/column_attr_type/C3/Enumeration' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/10/column_attr_type/C4/Note' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/11/cell_change/C3/L1/GARBAGE' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-icoABI/1/12/cell_change/C4/L1/5.5' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        c = ss.url('=' + root + '/%s/_10800001' % ys)
        utilities.write_file('xxx.png', c, encoding='bytes')
        import PIL.Image
        image = PIL.Image.open('xxx.png')
        print(image.size)

        c = ss.url('=' + root + '/%s/_1234567' % ys)
        assert(c == utilities.read_file('FILES/tip.png', encoding='bytes'))

        c = ss.url('=' + root + '/%s/timeline/10800001' % ys)
        assert('<script>timeline(' in c)

        utilities.mkpath('DBregtest/LOGINS/108/10800001')
        utilities.write_file('DBregtest/LOGINS/108/10800001/grades',
            '''20201217090909 {0} {1} UE 0.333
20201217090909 {0} {1} UE PRST
20201217090909 {0} {1} UE JUSLEAVE
20201217090909 {0} {1} UE UNJLEAVE'''.format(year, semester))
        c = ss.url('=' + root + '/%s/_10800001' % ys)
        utilities.write_file('xxx.png', c, encoding='bytes')
        image = PIL.Image.open('xxx.png')
        assert(image.size == (9, 4))


    if do('reload_plugins'):
        c = s.url('=' + root + '/reload_plugins')
        assert('True' not in c)
        assert(c.count('Reloaded') == 3)
        assert(c.count('False') > 10)
        libjs = s.url('lib.js')

        modify = ['FILES/doc_table.html', 'PLUGINS/suivi_share.js',
            'PLUGINS/create_table.py', 'COLUMN_TYPES/bool.py',
            'ATTRIBUTES/columnred.py']
        contents = []

        for filename in modify:
            contents.append(utilities.read_file(filename))
        try:
            for filename, content in zip(modify, contents):
                utilities.write_file(filename, content + '\n\t \t\n')
            c = s.url('=' + root + '/reload_plugins')
            assert(c.count('Reloaded') == 4)
            assert(c.count('True') == 2)
            retry(lambda: s.url('lib.js'), lambda c: c != libjs)
        finally:
            for filename, content in zip(modify, contents):
                utilities.write_file(filename, content)

    if do('delete-student'): # The tests is for coverage of configuration.student_removed()
        c = s.url('=%s/%s/UE-delete-student' % (abj, ys))
        c = s.url('='+abj+'/%s/UE-delete-student/1/0/cell_change/0_0/L0/11111111' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-delete-student/1/1/cell_change/0_0/L0/' % ys)
        assert(c == ok_png)

    if do('invitation'):
        s.url('=' + abj + '/%s/UE-invitation' % ys)
        s.url('=' + abj + '/%s/UE-invitation' % ys + '/1/0/cell_change/0_0/L1/10800001')

        for typ in ("sharable", "sharable_suivi", "one_shot"):
            c = s.post('=%s/%s/UE-invitation/invitation' % (abj, ys),
                fields = ( ("subject", "SUBJECT"),
                           ("message", "MESSAGE"),
                           ("days", "1"),
                           ("type", typ),
                           ("recipients", "a@x.fr." + typ),
                         ))
            assert("x.style.width = '100.0%'" in c)
            cc = retry(lambda: ('MESSAGE\n' + utilities.read_file("xxx.mail")).split("MESSAGE\n")[-2].split('\n')[0],
                      lambda c: '/invitation_accept/%s/UE-invitation/a@x.fr.%s/%s/' % (ys, typ, typ) in c)
            cc = '=10800002%s/invitation_accept/' % typ + cc.split('invitation_accept/')[1]
            assert('The invitation link is bad' in s.url(cc.replace('a@x.fr', 'b@x.fr')))
            c = s.url(cc)
            assert('You accepted the invitation.' in c)
            c = s.url(cc)
            assert('You are already in the table' in c)
            c = s.url('=' + abj + '/%s/UE-invitation' % ys)
            assert('10800001' in c)
            assert('a@x.fr.' + typ in c)

            c = s.url('=10900002%s/invitation_accept/' % typ + cc.split('invitation_accept/')[1])
            if "sharable" in typ:
                assert('You accepted the invitation.' in c)
            else:
                assert('This invitation has yet been used' in c)

        c = s.post('=%s/%s/UE-invitation/invitation' % (abj, ys),
            fields = ( ("subject", "SUBJECT"),
                       ("message", "MESSAGE"),
                       ("days", "0"),
                       ("type", "sharable"),
                       ("recipients", "z@x.fr"),
                     ))
        c = retry(lambda: utilities.read_file("xxx.mail").split("MESSAGE\n")[-2].split('\n')[0],
                 lambda c: '/invitation_accept/%s/UE-invitation/z@x.fr/sharable/' % ys in c)

        c = '=10800002/invitation_accept/' + c.split('invitation_accept/')[1]
        c = s.url(c)
        assert('The invitation link has expired' in c)

    if do('get_columns'):
        s.url('=' + abj + '/%s/UE-getcolumns' % ys)
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/0/column_attr_title/C0/A')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/1/column_attr_type/C0/Prst')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/2/cell_change/0_0/L1/10800001')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/3/cell_change/0_0/L2/10800002')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/4/cell_change/0_0/L3/10800003')
        time.sleep(0.1)
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/5/cell_change/0_3/L1/A')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/6/cell_change/0_3/L2/A')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/7/cell_change/0_3/L3/B')
        s.url('=' + abj + '/%s/UE-getcolumns' % ys + '/1/8/cell_change/0_4/L1/a')
        c = s.url('=' + abj + '/%s/UE-getcolumns/get_columns' % ys)
        assert( 'the_columns(columns,{"a\\u0001A": 1, "A": 1, "B": 1},0);' in c)
        c = s.url('=' + abj + '/%s/UE-badgetcolumns/get_columns' % ys)
        assert('the_columns([], [])' in c)

        c = s.url('=%s/0/Dossiers/config_table' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_table/%s/0/cell_change/0_2/display_sequence_column/[^Z][^Z].*' % (root, page))
        assert(c == ok_png)
        s.url('=' + abj + '/%s/ZZ-getcolumns' % ys)
        s.url('=' + abj + '/%s/ZZ-getcolumns' % ys + '/1/0/column_attr_title/C0/A')
        s.url('=' + abj + '/%s/ZZ-getcolumns' % ys + '/1/1/column_attr_type/C0/Prst')
        s.url('=' + abj + '/%s/ZZ-getcolumns' % ys + '/1/2/cell_change/0_0/L1/10800001')
        time.sleep(0.1)
        s.url('=' + abj + '/%s/ZZ-getcolumns' % ys + '/1/3/cell_change/0_3/L1/A')
        s.url('=' + abj + '/%s/ZZ-getcolumns' % ys + '/1/4/cell_change/0_4/L1/a')
        c = s.url('=' + abj + '/%s/ZZ-getcolumns/get_columns' % ys)
        assert('the_columns(columns,{"A": 1},1)' in c)

    if do('share'):
        s.url('=' + abj + '/%s/UE-share' % ys)
        s.url('=' + abj + '/%s/UE-share' % ys + '/1/0/cell_change/0_0/L1/10800001')
        s.url('=' + abj + '/%s/UE-share' % ys + '/1/1/column_attr_title/C0/A')
        s.url('=' + abj + '/%s/UE-share' % ys + '/1/2/cell_change/C0/L1/5.551')
        ss.start()
        c = ss.url('=10800001/%s' % ys)
        assert('5.551' in c)
        ticket = c.split('ticket="')[1].split('"')[0]
        c = s.url('=%s/share_store/add/toto/1/1/foobar' % ticket)
        assert("The link has been created" in c)
        c = ss.url('=10800001/%s' % ys)
        assert('"toto": {"photo": 1, "upload": 1, "comment": "foobar", "time": ' in c)
        c = ss.url('shared/10800001/toto')
        assert('username="anonymous"' in c)
        assert('"ShareSemesters"' in c)
        assert('"ShareLogin"' in c)

        c = s.url('shared/10800001/toto/XXX/picture/10800001.JPG') # for coverage
        c = s.url('shared/10800001/toto/upload/%s/UE-share/C0/x' % ys)  # for coverage
        assert("Can't find line" in c)

        c = s.url('=%s/share_store/del/toto' % ticket)
        assert('The link has been deleted' in c)
        c = ss.url('shared/10800001/toto')
        assert('Invalid or deleted shared link' in c)

        # Create another link with less authorization
        c = s.url('=%s/share_store/add/titi/0/0/foobar' % ticket)
        assert("The link has been created" in c)
        c = ss.url('shared/10800001/titi/%s' % year)
        assert('username="anonymous"' in c)
        c = ss.url('shared/10800001/titi/%s' % ys)
        assert('username="anonymous"' in c)
        try:
            s.url('shared/10800001/titi/XXX/picture/10800001.JPG')
            bug
        except:
            pass
        try:
            c = s.url('shared/10800001/titi/upload/%s/UE-share/C0/x' % ys)
            bug
        except:
            pass
        c = s.url('shared/10800001/titi/del')
        assert("The link has been delete" in c)

    if do('share2'):
        cc = "{'toto':{'photo': 1, 'upload': 1, 'comment': 'foobar', 'time': 0}}"
        utilities.manage_key('LOGINS', '10800001/share', content=cc)
        ss.start()
        c = ss.url('shared/10800001/toto')
        assert('Invalid or deleted shared link' in c)
        assert(utilities.manage_key('LOGINS', '10800001/share') == cc)
        c = ss.url('=10800001/%s' % ys)
        ticket = c.split('ticket="')[1].split('"')[0]
        c = s.url('=%s/share_store/add/titi/1/1/foobar2' % ticket)
        assert('The link has been created' in c)
        c = utilities.manage_key('LOGINS', '10800001/share')
        assert("'titi':{'photo': 1, 'upload': 1, 'comment': 'foobar2', 'time':" in c)
        assert('toto' not in c)

        c = s.url('=%s/share_store/bug/titi/1/1/foobar2' % ticket)
        assert('???' == c)

    if do('badname'):
        ss.start()
        c = ss.url('=' + root + '/badname')
        assert('</table>' in c)

    if do('search'):
        create_u2()
        c = s.url('=' + abj + '/search/aigle')
        assert('k09' in c)
        assert('</table>' in c)
        c = s.url('=' + abj + '/search/U')
        assert(c == '?')
        c = s.url('=' + abj + '/search/UE1')
        assert('UE1' in c)
        c = s.url('=' + abj + '/search/UE-UE1')
        assert('UE1' in c)
        c = s.url('=' + abj + '/search/AQWXSZEDCVFR')
        assert('</table>?' in c)

    if do('create-table'):
        c = s.url('=' + abj + '/create_table/%s/UE-FOOBAR/FB_TITLE/super.user/1' % ys)
        assert('The table has been created' in c)
        c = utilities.read_file('DBregtest/Y%s/S%s/UE-FOOBAR.py' % (year, semester))
        c = replace_dates(c)
        assert('''table_attr('masters',1,['%s'],'DATE')
table_attr('table_title',1,'FB_TITLE','DATE')
table_attr('teachers',1,['super.user'],'DATE')''' % abj)

        c = s.url('=' + abj + '/create_table/%s/UE-FOOBAR/FB_TITLE/super.user%%20bad/1' % ys)
        assert('Unknown login' in c)
        assert('The table is yet created' in c)

    if do('log'):
        utilities.unlink_safe('LOGS/GUI_record/' + abj)
        utilities.unlink_safe('LOGS/help')
        c = s.url('log/help/foobar')
        assert(c == '')
        c = utilities.read_file('LOGS/help')
        assert(c == 'foobar\n')
        s.url('log/bad_selector/ggg') # for coverage
        s.url('log/help/foobar2') # for coverage

        s.url('=%s' % abj) # Create ticket
        c = s.post('=%s/gui_record' % abj,
            fields = ( ("table", "SUBJECT"),
                       ("start", '1000000'),
                       ("data", '[]'),
                     ))
        assert(c == '')
        record = utilities.read_file('LOGS/GUI_record/' + abj)
        assert(record == 'R("SUBJECT","1000000",[])\n')

        c = s.post('=%s/gui_record' % abj,
            fields = ( ("table", "SUBJECT"),
                       ("start", '1000000'),
                       ("data", 'error'),
                     ))
        assert(c == '')
        c = utilities.read_file('LOGS/GUI_record/' + abj)
        assert(record == c)

    if do('autoupdate'):
        create_u2()
        c = s.url('=%s/auto_update/%s/UE-INF20UE2' % (root, ys))
        assert('The student lists are now up to date' in c)

        c = s.url('=%s/auto_update' % root)
        assert('The student lists are now up to date' in c)

    if do('fix_modifiable'):
        # Make table modifiable
        c = s.url('=%s/%s/UE-fixmod' % (root, ys))
        c = s.url('=%s/%s/UE-fixmod/1/0/table_attr_modifiable/0' % (root, ys))
        c = s.url('=%s/%s/UE-fixmod/1/1/table_attr_modifiable/1' % (root, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-fixmod/1/2/table_attr_modifiable/bug' % (root, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-fixmod/1/3/table_attr_modifiable/1' % (root, ys))
        assert(c == ok_png)

        # Copy the table in the past
        name = 'DBregtest/Y%s/S%s/UE-fixmod.py'
        c = utilities.read_file(name % (year, semester))
        c = c.replace("'%s" % year, "'2000")
        name = name % (year - 2, semester)
        utilities.mkpath('/'.join(name.split('/')[:-1]))
        utilities.write_file(name, c)

        os.utime(name, (999999999, 1)) # Put the file in the past

        c = s.url('=%s/fix_modifiables' % root)
        assert(c.count('UE-fixmod') == 3)
        assert('lines = {"0":' in c)

        c = s.url('=%s/fix_modifiable/%s/%s/UE-fixmod' % (root, year-2, semester))
        assert(c == ok_png_bin)

        c = utilities.read_file(name)
        assert("table_attr('modifiable',0,0,'" in c)

        c = s.url('=%s/fix_modifiable/%s/%s/UE-bugfixmod' % (root, year-2, semester))
        assert(c == bug_png_bin)

    if do('favorite_student'):
        # Toggle
        c = s.url('=%s/favorite_student/18000001' % abj)
        assert(c == ok_png_bin)
        c = utilities.read_file('DBregtest/LOGINS/%s/%s/favstu' % (abj[:3], abj))
        assert(c == "[\n'18000001',\n]")
        c = s.url('=%s/favorite_student/18000001' % abj)
        assert(c == ok_png_bin)
        c = utilities.read_file('DBregtest/LOGINS/%s/%s/favstu' % (abj[:3], abj))
        assert(c == "[\n]")

    if do('picture_upload'):
        create_u2()
        c = s.url('=%s' % abj)
        c = s.post('=%s/my_picture_upload' % abj,
                files = ( ("datafile", "picture.png", ok_png_bin), )
               )
        assert(c.count(': False') == 2)
        c = s.url('=%s/0/Dossiers/config_table' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_table/%s/0/cell_change/0_2/allow_teacher_picture_upload/(1,2)'
            % (root, page))
        assert(c == bug_png)
        c = s.url('=%s/0/Dossiers/config_table/%s/1/cell_change/0_0/allow_teacher_picture_upload/(1,2)'
            % (root, page))
        assert(c == bad_png)
        c = s.url('=%s/0/Dossiers/config_table/%s/2/cell_change/0_2/allow_teacher_picture_upload/True'
            % (root, page))
        assert(c == ok_png)
        s.url('=' + 'ue1.master')
        c = s.post('=%s/my_picture_upload' % 'ue1.master',
                files = ( ("datafile", "picture.png", ok_png_bin), )
               )
        assert('<img src=' in c)
        assert(c.count(': False') == 0)
        assert(os.path.exists("PICTURES/%s.JPG" % 'ue1.master'))
        # assert(os.path.exists("PICTURES/%s-.JPG" % 'ue1.master')) # No more created

        # Restore masters
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys)
        page2 = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=' + root + '/%s/UE-INF20UE2' % ys +
              '/%s/0/table_attr_masters/ue1.master' % page2)
        assert(c == ok_png)

        ss.start()
        retry(lambda: ss.url('=10800000/%s' % ys),
            lambda c: '"masters": [["ue1.masterFirstname", "ue1.masterSurname", "mail@mail.fr"]]' in c)

        c = s.url('=%s/0/Dossiers/config_table/%s/3/cell_change/0_2/show_teacher_pictures/True'
            % (root, page))
        assert(c == ok_png)
        ss.stop()
        ss.restart()
        retry(lambda: ss.url('=10800000/%s' % ys),
            lambda c: '"masters": [["ue1.masterFirstname", "ue1.masterSurname", "mail@mail.fr", "ue1.master"]]' in c)

        s.url('=10800001/picture/ue1.master.JPG') # For coverage
        s.url('=10800001/picture/bugue1.master.JPG') # For coverage


    if do('profilingyear'):
        c = s.url('=%s/profiling/%s' % (root, year))
        assert('runlog(columns, lines)' in c)
        assert('C("static-file")' in c)

    if do('getref'):
        c = s.url('=' + abj + '/%s/referents_students' % ys)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/0/cell_change/a/L2/%s' % (page, abj))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/1/cell_change/a/L3/ue2.master' % (page,))
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/referents_students' % ys +
              '/%s/2/cell_change/a/L4/foobar' % (page,))
        assert(c == ok_png)

        c = s.url('=%s' % (root,))
        retry(lambda: s.post('=%s/referent_set_pairs' % (root,),
                             fields = (('lines', 'foobar 10800001\n'), )),
              lambda c: '10800001 : 10800001Firstname 10800001Surname\n' in c
        )

        c = s.url('=%s/%s/UE-getref' % (abj, ys))
        c = s.url('=%s/%s/UE-getref/1/0/column_attr_type/C0/Get_Referent' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-getref/1/1/column_attr_columns/C0/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-getref/1/2/cell_change/0_0/L1/10800001' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-getref' % (abj, ys))
        c = replace_dates(c)
        assert('P("L1",[C("10800001","tt.master","DATE"),C(),C(),C(),C(),C(),C("foobar","*","DATE")]);' in c)

        c = s.url('=%s' % (root,))
        c = s.post('=%s/referent_set_pairs' % (root,),
                fields = (('lines', 'ue2.master 10800001\n'), ))
        assert('10800001 : 10800001Firstname 10800001Surname\n' in c)

        # Create a notepad (stats coverage)
        c = s.url('=ue2.master/%s/Referents/ue2__master' % uyear)
        assert('[C("10800001","*"' in c)
        c = s.url('=ue2.master/%s/Referents/ue2__master/2/0/cell_change/0_0/L1/' % uyear)
        assert( c == ok_png) # for coverage

        c = s.url('=%s/%s/UE-getref/1/3/update_content/column' % (abj, ys))
        retry(lambda: replace_dates(s.url('=%s/%s/UE-getref' % (abj, ys))),
              lambda c: 'P("L1",[C("10800001","tt.master","DATE"),C(),C(),C(),C(),C(),C("ue2.master","*","DATE","",[["foobar","DATE","*","V"]' in c)

        c = s.url('=' + abj + '/referent/10800001')
        assert('ue2.master' in c)
        c = s.url('=' + abj + '/referent/10111111')
        assert('No referent teacher' in c)

        ss.start()

        retry(lambda: ss.url('=ue2.master/%s/10800001' % ys), lambda c: 'You are the only user seeing' in c)
        c = ss.url('=10800001/%s' % ys)
        assert('You are the only user seeing' not in c)

        c = s.url('=%s/referent_set/%s/10800001' % (root, root))
        assert("10800001: has yet a referent teacher" in c)

        c = s.url('=%s/referent_set_force/foobar/10800001' % (root,))
        assert('is now referent teacher of:\n10800001 : 10800001Firstname 10800001Surname' in c)

        open('DBregtest/Y%s/SReferents/badfile' % uyear, "w").close() # for coverage

        retry(lambda: ss.url('=%s/*3' % root), lambda c: '#of referent teachers: ' in c)

        c = s.url('=%s/%s/UE-getref/1/4/update_content/column' % (abj, ys))
        c = s.url('=%s/%s/UE-getref' % (abj, ys))
        c = replace_dates(c)
        assert('P("L1",[C("10800001","tt.master","DATE"),C(),C(),C(),C(),C(),C("foobar","*","DATE","",[["foobar","DATE","*","V"],["ue2.master"' in c
            or 'P("L1",[C("10800001","tt.master","DATE"),C(),C(),C(),C(),C(),C("foobar","*","DATE","",[["foobar","DATE","*","V"],["","DATE","*","V"],["ue2.master"' in c)

        c = s.url('=%s/orphan_students/10800001/' % (root,))
        c = s.url('=%s/orphan_students/10800001' % (root,))
        assert('has no referent teacher' in c)
        c = s.url('=%s/%s/UE-getref/1/5/update_content/column' % (abj, ys))
        c = s.url('=%s/%s/UE-getref' % (abj, ys))
        c = replace_dates(c)
        assert('P("L1",[C("10800001","tt.master","DATE"),C(),C(),C(),C(),C(),C("","*","DATE","",[["foobar",' in c)

    # if do('getref2'):
    #     # Only for coverage
    #     s.url('=%s/%s/BAD/UE-getref' % (abj, year))
    #     c = s.url('=%s/%s/BAD/UE-getref/1/0/column_attr_type/C0/Get_Referent' % (abj, year))
    #     assert( c == ok_png)

    if do('remove_history'):
        c = s.url('=%s/0/Dossiers/config_table/remove_history/no' % (root,))
        assert('Follow this link to confirm this action' in c)
        c = s.url('=%s/0/Dossiers/config_table/remove_history/ok' % (root,))
        assert('OK: Copy seems fine' in c)
        c = s.url('=%s/0/Dossiers/config_table' % (root,))
        assert('runlog(columns, lines)' in c)

        c = s.url('=%s/0/Dossiers/config_table/remove_history/no' % (abj,))
        assert('you are not a table master' in c)

        c = s.url('=%s/0/Dossiers/config_table' % (root,))
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_table/%s/0/table_attr_modifiable/0' % (root, page))
        assert( c == ok_png)
        c = s.url('=%s/0/Dossiers/config_table/remove_history/no' % (root,))
        assert('nobody can change the content' in c)
        c = s.url('=%s/0/Dossiers/config_table/%s/1/table_attr_modifiable/1' % (root, page))
        assert( c == ok_png)

    if do('notation'):
        s.url('=%s/%s/UE-notation' % (abj, ys))
        c = s.url('=%s/%s/UE-notation/1/0/column_attr_type/C0/Notation' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/1/column_attr_groupcolumn/C0/Grp' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/2/cell_change/0_0/L1/18000001' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/3/cell_change/0_0/L2/18000002' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/4/cell_change/0_3/L1/A' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/5/cell_change/0_3/L2/A' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/6/cell_change/C0/L1/1.11' % (abj, ys))
        assert( c == ok_png)
        # The comment must disseminate to other group students
        c = s.url('=%s/%s/UE-notation/1/7/comment_change/C0/L1/FOOnotationBAR' % (abj, ys))
        assert( c == ok_png)
        ss.start()
        c = ss.url('=18000002/%s' % ys)
        assert('FOOnotationBAR' in c)
        assert('"nr_in_grp": 2' in c)

        c = s.url('=%s/%s/UE-notation/1/8/column_attr_modifiable/C0/2' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/9/cell_change/C0/L1/' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=18000001/%s/UE-notation/comment/C0/L1/{"":"5$2F20$20jjj"}' % (ys, ))
        assert(red in c)
        c = s.url('=18000001/%s/UE-notation/comment/C0/L1/{"":"$3F$2F20$20jjj"}' % (ys, ))
        assert('OK!' in c)

        # No group
        c = s.url('=%s/%s/UE-notation/1/10/cell_change/0_0/L3/18000003' % (abj, ys))
        assert( c == ok_png)
        c = retry(lambda: ss.url('=%s/%s/18000003' % (abj, ys)), lambda c: '"nr_in_grp": 0' in c)
        assert('"group_members": []' in c)

        # group of one
        c = s.url('=%s/%s/UE-notation/1/11/cell_change/0_0/L4/18000004' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-notation/1/12/cell_change/0_3/L4/ALONE' % (abj, ys))
        assert( c == ok_png)
        c = retry(lambda: ss.url('=%s/%s/18000004' % (abj, ys)), lambda c: '"nr_in_grp": 0' in c)
        assert('"group_members": []' in c)

    if do('speak'):
        c = s.url('=%s/0/Dossiers/config_table/column_speak/2/0/logo/language' % root)
        assert(c == 'espeak not installed' or 'RIFF' in c and 'WAVE' in c)

    if do('change_identity'):
        c = s.url('=%s-42/change_identity/%s' % (root, abj))
        assert('Identity change done' in c)
        c = s.url('=%s-42' % root)
        assert('username="%s";' % abj in c)

    if do('suivi_rss'):
        c = s.url('=' + root + '/0/Dossiers/config_suivi_rss')
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/0/cell_change/0/L1/0' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/1/cell_change/1/L1/3' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/2/cell_change/2/L1/staff' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/3/cell_change/4/L1/file:FILES$2Fnews.xml' % (root, page))
        assert(c == ok_png)
        ss.start()
        c = ss.url("=" + abj)
        assert('["RSSStream", [[' in c)
        c = ss.url("=" + abj + '/' + abj)
        assert('["RSSStream", [[' in c)

        # For coverage only
        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/4/cell_change/1/L1/-3' % (root, page))
        assert(c == ok_png)
        c = ss.url("=" + abj)
        retry(lambda: ss.url("=" + abj), lambda c: "01/03/2010" in c)
        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/5/cell_change/4/L1/file:BAD' % (root, page))
        assert(c == ok_png)
        c = ss.url("=" + abj)
        assert('["RSSStream", [[' in c)

        c = s.url('=%s/0/Dossiers/config_suivi_rss/%s/6/cell_change/1/L1/-3' % (root, page))
        assert(c == ok_png)
        c = ss.url("=" + abj)
        assert('["RSSStream", [[' in c)

    if do('extracts'):
        c = s.url('=user.2/save_preferences/can_bring_a_pc=1')
        assert( c == ok_png_bin)
        c = s.url('=k01/save_preferences/commute_time=4242')
        assert( c == ok_png_bin)
        c = s.url('=k01/save_preferences/working_hours_per_week=42')
        assert( c == ok_png_bin)

        c = s.url('=%s/%s/UE-extracts' % (abj, ys))
        c = s.url('=%s/%s/UE-extracts/1/0/column_attr_type/C0/Phone' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/1/column_attr_columns/C0/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/2/column_attr_type/C1/Can_Bring_A_Pc' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/3/column_attr_columns/C1/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/4/column_attr_type/C2/Mail' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/5/column_attr_columns/C2/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/6/column_attr_type/C3/First_Registration' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/7/column_attr_columns/C3/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/8/cell_change/0_0/L1/user.2' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/9/cell_change/0_0/L2/k01' % (abj, ys)) # for coverage
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/10/column_attr_type/C4/Commute_Time' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/11/column_attr_columns/C4/ID' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/12/column_attr_type/C5/Working_Hours' % (abj, ys))
        assert( c == ok_png)
        c = s.url('=%s/%s/UE-extracts/1/13/column_attr_columns/C5/ID' % (abj, ys))
        assert( c == ok_png)
        c = retry(lambda: replace_dates(s.url('=%s/%s/UE-extracts' % (abj, ys))),
            lambda c: 'C("telephoneNumber","*","DATE"),C("YES","*","DATE"),C("mail@mail.fr","*","DATE"),C("0","*","DATE")' in c)
        assert('C("telephoneNumber","*","DATE"),C("","*","DATE"),C("bbonobo@africa.net","*","DATE"),C("0","*","DATE"),C(4242,"*","DATE","",[["","DATE","*","V"]]),C(42,"*","DATE","",[' in c)
    if do('stat_preferences'):
        ss.start()
        c = ss.url('=%s/stat_preferences' % (root,))
        assert('runlog(columns, lines)' in c)

    if do('picture'):
        if os.path.exists('PICTURES/k01-.JPG'):
            os.unlink('PICTURES/k01-.JPG')
        c = s.url('=%s/picture/k01.JPG' % (root,))
        assert(6500 < len(c) < 6600) # Because of resizing of the original
        c = s.url('=%s/picture-icon/k01.JPG' % root)
        assert(1900 < len(c) < 2200)
        c = s.url('=k01/picture/k01.JPG')
        assert(6500 < len(c) < 6600) # Because of resizing of the original
        c = s.url('=k01/picture-icon/k01.JPG')
        assert(1900 < len(c) < 2200)
        c = s.url('=k02/picture/k01.JPG')
        assert(5650 < len(c) < 5750)

    if do('attributes'):
        s.url('=%s/%s/UE-attributes' % (abj, ys))
        c = s.url('=%s/%s/UE-attributes/1/0/table_attr_masters/%s' % (abj, ys, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/1/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/2/column_attr_clamp/C0/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/3/table_attr_official_ue/0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/4/column_attr_mcq/C1/' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/5/column_attr_mcq/C1/{}' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/6/column_attr_weight/C1/+1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/7/column_attr_locked/C1/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/8/column_attr_locked/C1/a' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/9/column_attr_rounding/C1/' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/10/column_attr_rounding/C1/q' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/11/table_attr_rounding/C1/q' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/12/table_attr_rounding/C1/9' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/13/column_attr_hidden/C1/q' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/14/column_attr_weight/C1/g' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/15/column_attr_weight/0_0/1' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/16/column_attr_title/C1/my_title' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/17/column_attr_title/C2/my_title' % (abj, ys))
        assert(c == bug_png)
        c = s.url('=%s/%s/UE-attributes/1/18/column_attr_minmax/C2/a;b' % (abj, ys))
        assert(c == ok_png) # unchecked server side
        c = s.url('=%s/%s/UE-attributes/1/19/column_attr_clamp/C0/a' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/20/table_attr_dates/01%%2F02%%2F2021%%2001%%2F01%%2F2021' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/21/column_attr_freezed/C0/BUG' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/22/table_attr_modifiable/2' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-attributes/1/23/table_attr_modifiable/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-attributes/1/24/table_attr_modifiable/1' % (abj, ys))
        assert(c == ok_png)

    if do('UE1'):
        # For the coverage of teacher.py
        c = s.url('=%s/%s/UE-UE1' % (abj, ys))
        c = s.url('=%s/%s/UE-UE1/1/0/cell_change/0_0/L1/user.3' % (abj, ys))
        assert(c == ok_png)

    if do('config_plugin'):
        c = s.url('=%s/0/Dossiers/config_plugin' % (root,))
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_plugin/%s/0/cell_change/invited/page_unload/' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_plugin/%s/1/cell_change/invited/page_unload/bug' % (root, page))
        assert(c == bug_png)
        c = s.url('=%s/0/Dossiers/config_plugin/%s/2/cell_change/invited/page_unload/("grp:foo","%s")' % (root, page, invited))
        assert(c == ok_png)
        s.url('=%s/%s/UE-UE1' % (abj, ys))
        c = s.url('=%s/%s/UE-UE1/page_unload' % (abj, ys))
        assert(c == 'bad_url')
        c = s.url('=%s/%s/UE-UE1/page_unload' % (invited, ys))
        assert('Unload done' in c)

    if do('bilan'):
        # Create a fake 'resume' (should be computed by SCRIPTS/bilan.py)
        utilities.mkpath('DBregtest/LOGINS/p08/p0800001')
        utilities.write_file('DBregtest/LOGINS/p08/p0800001/resume',
            '''{'UE-UE1':[[%s,'%s',42,0,0,0.1234,7]],}''' % (year, semester))
        c = s.url('=%s/bilan/p0800001' % (root,))
        assert('''bilan("p0800001",{'UE-UE1':[[%s,'Printemps',42,0,0,0.1234,7]],},"P0800001Firstname","P0800001SURNAME","mail@mail.fr",{''' % year
            in c)
        c = s.url('=%s/bilan/p0803301' % (root,))
        assert('This student is unknown of TOMUSS' in c)
        c = s.url('=%s/bilan/truc' % (root,))
        assert('bad_url' in c)

    if do('tickets'):
        c = s.url('=%s/tickets' % (root,))
        assert('lines = {"0": ' in c)
        assert('C("TJRrAP"),C("\'Python-urllib/' in c)
        c = s.url('=%s/check_ticket/timestamp' % (root,))
        assert('window.check_ticket = "True"' in c)

    if do('logout'):
        c = s.post('/logout/redirection')
        assert(c == '')
        c = s.post('/logout/https:$2F$2Fredirection')
        assert(c == '')
        s.url('=' + abj)
        # XXX Must uncomment the 2 next lines
        c = s.url('allow/%s' % abj)
        assert('window.close()' in c)
        # c = s.post('/logout/' + abj)
        # assert(c == '')

    if do('files'):
        c = s.url('ok.png')
        assert(c == ok_png_bin)
        c = s.url('files/{}/documentation.js'.format(configuration.version))
        assert(len(utilities.compressBuf(utilities.read_file('FILES/documentation.js')))
               - len(c) < 200)
        utilities.mkpath('TMP/0')
        utilities.write_file("TMP/0/foo_UNCACHED.txt", 'mmm')
        c = s.url('files/0/foo_UNCACHED.txt')
        assert(abs(len(c) - 23) < 5)

    if do('rewrite'):
        c = s.url('=%s/0/Dossiers/config_table/rewrite' % root)
        assert("cell_change(0,'0_0','language','language'," in c)
        c = s.url('=%s/0/Dossiers/config_table/0/aytrytr' % root)
        assert(c == 'bad_url')
        c = s.url('=%s/0/Dossiers/config_table' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_table/%s/end_of_load' % (root, page))
        assert c == ''
        c = s.url('=%s/0/Dossiers/config_table/%s/0/table_attr_default_nr_columns/a' % (root, page))
        assert(c == ok_png) # stor 0 in place of "a"
        c = s.url('=%s/0/Dossiers/config_table/%s/1/table_attr_default_nr_columns/4' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/nonono/page_unload' % root)
        assert('This table does not exist!' in c)

        c = s.url('=%s/0/Dossiers/config_table/bookmark' % abj)
        assert(c == ok_png_bin)
        c = s.url('=%s' % abj)
        assert('["HomeUEBookmarked", [[0, "Dossiers", "config_table"]]]' in c)
        c = s.url('=%s/0/Dossiers/config_table/bookmark' % abj)
        assert(c == ok_png_bin)
        c = s.url('=%s' % abj)
        assert('["HomeUEBookmarked", []' in c)

        if os.path.exists('LOGS/key_history'):
            os.unlink('LOGS/key_history')
        try:
            s.url('=%s/0/Dossiers/config_table/%s/key_history/AKEY' % (root, page),
                stop_if_error=False, display_log_if_error=False)
            bug
        except:
            assert '0/Dossiers/config_table AKEY' in utilities.read_file('LOGS/key_history')
            print()

    if do('MCQ'):
        s.url('=%s/%s/UE-MCQ' % (abj, ys))
        c = s.url('=%s/%s/UE-MCQ/1/0/column_attr_type/C0/MCQ' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-MCQ/1/1/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=10800001/%s/UE-MCQ/comment/C0/L1/COMMENTMCQ' % (ys, ))
        assert(red in c)
        c = s.url('=%s/%s/UE-MCQ/1/2/column_attr_modifiable/C0/2' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=10800001/%s/UE-MCQ/comment/C0/L1/COMMENTMCQ' % (ys, ))
        assert(c == 'OK!')
        c = s.url('=10800001/%s/UE-MCQ/cell/C0/L1/{}' % (ys, ))
        assert(red in c)
        c = s.url('=10800001/%s/UE-MCQ/cell/C0/L1/NO' % (ys,))
        assert(c == 'OK!')

        mcq = '[{"q1":["QA",{"c295":["CA",{"priority":0}],"c770":["CB",{"priority":1}]},{"radio":1,"grade":[[["c295"],["c770"],null]],"priority":0}]},{"same_questions":0,"radio":1,"gui":"expert","shuffle_questions":1,"shuffle_choices":1,"good_answer":1,"no_answer":0,"bad_answer":"-1/(#-1)","progressive":"bad","group":"","obfuscate":1,"choice_CSS":"","question_CSS":"","scoring_scale":1,"stats":0,"nr_question_in_group":{},"nr_question_in_group":{}}]'.replace('#', '$23').replace('/', '$2F')
        c = s.url('=%s/%s/UE-MCQ/1/3/column_attr_mcq/C0/%s' % (abj, ys, mcq))
        assert(c == ok_png)

        # A send alert
        c = s.url('=%s/send_alert/UE-MCQ/message' % (root,))
        assert('Message sent : message' in c)
        c = s.url('=%s/tablebuffer/%s/UE-MCQ' % (root, ys))
        assert(r'<td>&lt;script&gt;alert(_(&quot;ALERT_message_for&quot;)+&quot;UE-MCQ\n\nmessage&quot;);&lt;/script&gt;' in c)

        ss.start()
        c = ss.url('=10800001')

        c = s.url('=%s/%s/UE-MCQ/1/4/column_attr_cell_writable/C0/=YES|$23[]' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-MCQ/1/5/cell_change/C0/L1/YES' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=10800001')
        assert("c295" in c)
        assert('"grade":' not in c)
        c = s.url('=%s/%s/UE-MCQ/1/6/cell_change/C0/L1/NO' % (abj, ys))
        assert(c == ok_png)
        retry(lambda: ss.url('=10800001'), lambda c: "c295" not in c)
        c = s.url('=%s/%s/UE-MCQ/1/7/column_attr_modifiable/C0/0' % (abj, ys))
        assert(c == ok_png)
        retry(lambda: ss.url('=10800001'), lambda c: "c295" not in c)
        mcq = '[{"q1":["QA",{"c295":["CA",{"priority":0}],"c770":["CB",{"priority":1}]},{"radio":1,"grade":[[["c295"],["c770"],null]],"priority":0}]},{"same_questions":0,"radio":1,"gui":"expert","shuffle_questions":1,"shuffle_choices":1,"good_answer":1,"no_answer":0,"bad_answer":"-1/(#-1)","progressive":"bad","group":"","obfuscate":1,"choice_CSS":"","question_CSS":"","scoring_scale":1,"stats":0,"nr_question_in_group":{},"nr_question_in_group":{},"show_answers":1}]'.replace('#', '$23').replace('/', '$2F')
        c = s.url('=%s/%s/UE-MCQ/1/8/column_attr_mcq/C0/%s' % (abj, ys, mcq))
        assert(c == ok_png)
        retry(lambda: ss.url('=10800001'), lambda c: "c295" not in c)
        c = s.url('=%s/%s/UE-MCQ/1/9/cell_change/C0/L1/5.55' % (abj, ys))
        assert(c == ok_png)
        retry(lambda: ss.url('=10800001'), lambda c: "c295" in c)


    if do('moreabj'):
        # 'abj' must be executed before
        s.url('=' + root + '/%s/UE-UE1' % ys)
        c = s.url('=%s/%s/abj/list_mail' % (abj, ys))
        assert('BEWARE, all the mails will be sent' in c)
        c = s.url('=%s/%s/abj/send_mail' % (abj, ys))
        assert('mail sent' in c)
        c = s.url('=%s/%s/abjs' % (abj, ys))
        assert("abj_no" in c)
        c = s.url('=%s/%s/abj/alpha_author.xls' % (abj, ys))
        assert('sort(cmp_name)' in c)
        c = s.url('=%s/%s/abj/html/.*' % (abj, ys))
        assert('sort(cmp_name)' in c)
        c = s.url('=%s/%s/abj/html/REGEXP' % (abj, ys))
        assert('You must indicate a regular expression' in c)
        c = s.url('=%s/%s/abj/send_mail' % (abj, ys))
        assert('BEWARE, all the mails will be sent' not in c)
        c = s.url('=%s/%s/abj/table' % (abj, ys))
        assert('runlog' in c)

    if do('nonsemester'):
        c = s.url('=%s/0/Dossiers/config_table' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_table/%s/0/cell_change/0_2/ue_not_per_semester/ZZ.*' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/Automne/ZZtop' % (abj, uyear))
        start, end, _garbage = c.split('dates:[')[1].split(',', 2)
        assert (float(end[:-1]) - float(start)) > 30000000 # one year

        c = s.url('=%s/%s/Printemps/ZZtop' % (abj, uyear+1))
        c = s.url('=%s/%s/Automne/ZZtop/1/0/cell_change/0_0/L1/18000001' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Automne/ZZtop/1/1/table_attr_masters/%s' % (abj, uyear, abj))
        assert(c == ok_png)

        c = s.url('=%s/%s/Printemps/ZZtop' % (abj, uyear+1))
        # XXX Missing semester : why ?
        assert(r'all_the_semesters = "\x3Coption value=\"2008/Test\"\x3ESand Box\x3C/option\x3E"' in c)

        c = s.url('=%s/%s/Automne/ZZtop/delete_this_table' % (abj, uyear))
        assert(deletion_done in c)

    if do('badcol'):
        s.url('=%s/%s/UE-badcol' % (abj, ys))
        c = s.url('=%s/%s/UE-badcol/1/0/comment_change/C0/L1/ID' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-badcol/1/1/column_delete/C0/L1/ID' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-badcol/1/2/column_attr_red/C0/[BUG]=42' % (abj, ys))
        assert(c == ok_png) # Make more cover when running bilan
        c = s.url('=%s/%s/UE-badcol/1/3/column_attr_locked/C0/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-badcol/1/4/cell_change/C0/0_0/X' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-badcol/1/5/comment_change/C0/0_0/X' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-badcol/1/6/column_attr_cell_writable/C0/=NEVER' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-badcol/1/7/comment_change/C0/0_0/Y' % (abj, ys))
        assert(c == bad_png)

    if do('grpselect'):
        c = s.url('=%s/%s/UE1@_O' % (abj, ys))
        assert("k02" in c)
        assert("k09" in c)
        assert("k12" in c)
        assert("k01" not in c)
        assert("k03" not in c)
        assert("k04" not in c)

    if do('teachers'):
        c = s.url('=%s/%s/UE-teachers' % (abj, ys))
        c = s.url('=%s/%s/UE-teachers/1/0/table_attr_teachers/%s%%20%s' % (abj, ys, abj, 'ue1.master'))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-teachers/1/1/table_attr_teachers/%s%%20%s' % (abj, ys, abj, 'do-not-exists'))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-teachers' % (invited, ys))
        assert("You are not allowed to" in c)
        c = s.url('=%s/%s/UE-teachers' % ('ue1.master', ys))
        assert("runlog" in c)

    if do('upload2'):
        shutil.rmtree('UPLOAD/%s/UE-upload2' % ys, ignore_errors=True)

        c = s.url('=' + abj +'/%s/UE-upload2' % ys)
        c = s.url('='+abj+'/%s/UE-upload2/1/0/column_attr_type/A/Upload'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload2/1/1/column_attr_modifiable/A/2'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload2/1/2/cell_change/0_0/L1/10800001'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload2/1/3/cell_change/A/L1/NO'%ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload2/1/4/column_attr_modifiable/B/2'%ys)
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload2/1/5/column_attr_private/A/%s' % (abj, ys, abj))
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-upload2/1/6/cell_change/0_0/L2/10800002'%ys)
        assert(c == ok_png)
        ss.start()
        s.url('=10800001/bad_url')
        c = s.post('=10800001/%s/UE-upload2/upload_post/A/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), ))
        assert('not allowed to modify' in c)

        c = s.post('=10800001/%s/UE-badupload2/upload_post/A/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), ))
        assert("Can't find table" in c)
        c = s.post('=10800001/%s/UE-upload2/upload_post/BAD/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), ))
        assert("Can't find column" in c)
        c = s.post('=10800001/%s/UE-upload2/upload_post/B/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), ))
        assert("Not an ('Upload', 'Annotate') column type" in c)

        c = s.url('%s/UE-upload2/upload_get_public/A/L1' % ys)
        assert("No anonymous access allowed" in c)

        c = s.url('=%s/%s/UE-upload2/upload_get/A/L1' % (abj, ys))
        assert("Can't find this file" in c)

        c = s.url('=%s/%s/UE-upload2/upload_get/A/L1' % (invited, ys))
        assert("This is private data" in c)

        c = s.url('=10800001/%s/UE-upload2/upload_get/A/L2' % (ys,))
        assert("You are not allowed to read/modify this value" in c)

        c = s.url('=10800002/%s/UE-upload2/upload_get/A/L2' % (ys,))
        assert("Can't find this file" in c)

        c = s.url('=' + abj + '/%s/UE-upload2/comment/A/L1/0' % ys)
        assert(red in c)


    if do('import3'):
        utilities.mkpath('TMP/0')
        utilities.write_file('TMP/0/xxx.csv', '''10800001\t3.45\t4.56
    10800002\t5.67\t6.78
    ''')
        s.url('=%s/%s/UE-import3' % (abj, ys))
        u = s.get_url('files/0/xxx.csv#2').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/0/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)
        c = s.url('=' + root + '/%s/UE-import3' % ys)
        assert('xxx.csv' not in c) # confidential
        c = s.url('=%s/%s/UE-import3/1/1/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-import3' % (abj, ys))
        assert('3.45' in c)

        # For coverage
        u = s.get_url('files/0/doesnotexists.csv#2').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/2/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)
        u = s.get_url('files/0/xxx.csv#99').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/3/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)
        u = s.get_url('files/0/xxx.csv#-1').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/4/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)
        u = s.get_url('files/0/xxx.csv#a').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/5/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)

        # Remove import
        c = s.url('=%s/%s/UE-import3/1/6/column_attr_url_import/A/' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-import3' % (abj, ys))
        c = replace_dates(c)
        assert('C(3.45,"?","DATE","",[[3.45,"DATE","*","V"]])' in c)

        # import comment
        u = s.get_url('files/0/xxx.csv#2#').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/7/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-import3/1/8/cell_change/0_0/L2/10800002' % (abj, ys))
        assert(c == ok_png)
        ## Allow 'abj' to erase '?' value
        #c = s.url('=%s/%s/UE-import3/1/9/table_attr_masters/%s' % (abj, ys, abj))
        #assert(c == ok_png)
        ## Erase cell content to allow import comment
        #c = s.url('=%s/%s/UE-import3/1/10/cell_change/A/L1/' % (abj, ys))
        #assert(c == ok_png)
        c = s.url('=%s/%s/UE-import3' % (abj, ys))
        c = replace_dates(c)
        assert('C(),C("","","","5.67",[["5.67","DATE","*","C"]])])' in c)
        assert('C(),C(3.45,"?","DATE","3.45"' in c)

        # For coverage: missing #
        u = s.get_url('files/0/doesnotexists.csv').replace('/', '$2F')
        c = s.url('=%s/%s/UE-import3/1/9/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png)

        # Bad import column
        u = s.get_url('files/0/xxx.csv').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/10/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png) # XXX should not be ok
        c = s.url('='+abj+'/%s/UE-import3/1/11/update_content/column' % ys)
        assert(c == ok_png)
        u = s.get_url('files/0/xxx.csv#BAD').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-import3/1/12/column_attr_url_import/A/%s' % (abj, ys, u))
        assert(c == ok_png) # XXX should not be ok
        c = s.url('='+abj+'/%s/UE-import3/1/13/update_content/column' % ys)
        assert(c == ok_png)

    if do('annotate2'):
        shutil.rmtree('UPLOAD/%s/UE-annotate2' % ys, ignore_errors=True)
        s.url('=' + abj + '/%s/UE-annotate2' % ys)
        c = s.url('=' + abj + '/%s/UE-annotate2' % ys + '/1/0/column_attr_type/C0/Annotate')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-annotate2' % ys + "/1/1/column_attr_annotate/C0/%7B%22images%22:%5B%5D,%22options%22:%5B%7B%22colors%22:%22%23000,%23F00,%230A0,%2300F%22,%22color%22:%22%23000%22,%22widths%22:%221,4,8,12%22,%22width%22:%221%22,%22sizes%22:%2218,24,32,48%22,%22size%22:%2224%22,%22modes%22:%22Edit,Text,Draw,Highlight%22,%22mode%22:%22Text%22%7D,%7B%22colors%22:%22%23000,%23F00,%230A0,%2300F%22,%22color%22:%22%23F00%22,%22widths%22:%221,4,8,12%22,%22width%22:%221%22,%22sizes%22:%2218,24,32,48%22,%22size%22:%2224%22,%22modes%22:%22Edit,Text,Draw,Highlight,-1,+1,-0.5,+0.5%22,%22mode%22:%22+1%22%7D%5D%7D")
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-annotate2' % ys + '/1/2/cell_change/0_0/L1/18000001')
        assert(c == ok_png)

        ss.start()
        s.url('=18000001')
        c = s.post('=18000001/%s/UE-annotate2/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('You are not allowed to modify this value' in c)

        c = s.url('=' + abj + '/%s/UE-annotate2' % ys + '/1/3/column_attr_modifiable/C0/2')
        assert(c == ok_png)

        # Import a PDF by the student
        c = s.post('=18000001/%s/UE-annotate2/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.pdf"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('Uploaded file type: image/png; charset=binary' in c)

        c = s.post('=18000001/%s/UE-annotate2/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.xxx"), ),
                   files = ( ("data", "FOO.xxx", b'not a pdf'), ),
               )
        assert('Unknown value: <p><b>foo.xxx</b>')

        c = s.post('=18000001/%s/UE-annotate2/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.xxx"), ),
                   files = ( ("data", "FOO.pdf", b'not a pdf'), ),
               )
        assert('Unknown value: <p><b>foo.pdf</b>')

        c = s.post('=18000001/%s/UE-annotate2/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.pdf"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('The uploaded file has the same size than the previous one' in c)

        c = s.url('=' + abj + '/%s/UE-annotate2' % ys + '/1/4/column_attr_upload_max/C0/1')
        assert(c == ok_png)
        c = s.post('=18000001/%s/UE-annotate2/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.pdf"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('too many bytes in your file' in c)

        png = s.url('=18000001/%s/UE-annotate2/upload_get/C0/L1/toto.txt' % ys)
        utilities.write_file('xxx.png', png, encoding='bytes')
        import PIL.Image
        image = PIL.Image.open('xxx.png')
        assert(image.size == (1240, 14024))

    if do('upload_group'):
        shutil.rmtree('UPLOAD/%s/UE-upload_group' % ys, ignore_errors=True)
        s.url('=%s/%s/UE-upload_group' % (abj, ys))
        c = s.url('=%s/%s/UE-upload_group/1/0/column_attr_type/C0/Upload' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/1/cell_change/0_0/L1/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/2/cell_change/0_0/L2/18000002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/3/cell_change/0_3/L1/GRPA' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/4/cell_change/0_3/L2/GRPA' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/5/column_attr_groupcolumn/C0/Grp' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/6/column_attr_modifiable/C0/2' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        s.url('=18000001')
        c = s.post('=18000001/%s/UE-upload_group/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"A TXT CONTENT"), )
               )
        assert('<p style="background:#8F8">' in c)
        c = s.url('=18000001/%s/UE-upload_group/upload_get/C0/L1/toto.txt' % ys)
        assert(c == 'A TXT CONTENT')
        s.url('=18000002') # Activate ticket
        retry(lambda: s.url('=18000002/%s/UE-upload_group/upload_get/C0/L2/toto.txt' % ys),
              lambda c: c == 'A TXT CONTENT')

        # Switch comment line
        c = s.url('=%s/%s/UE-upload_group' % (root, ys))
        c = s.url('=%s/%s/UE-upload_group/3/0/table_attr_masters/%s' % (root, ys, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/7/comment_change/C0/L1/' % (abj, ys))
        assert(c == ok_png)
        u = 'image/png; charset=binary'.replace('/', '$2F').replace(' ', '$20')
        c = s.url('=%s/%s/UE-upload_group/1/8/comment_change/C0/L2/%s' % (abj, ys, u))
        assert(c == ok_png)
        c = s.url('=18000001/%s/UE-upload_group/upload_get/C0/L1/toto.txt' % ys)
        assert(c == 'A TXT CONTENT')

        # Duplicate comment
        c = s.url('=%s/%s/UE-upload_group/1/9/comment_change/C0/L1/%s' % (abj, ys, u))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/10/column_attr_cell_writable/C0/!=' % (abj, ys))
        assert(c == ok_png)
        c = s.post('=18000002/%s/UE-upload_group/upload_post/C0/L2' % ys,
                   fields = ( ("filename", "foo2.txt"), ),
                   files = ( ("data", "FOO2.TXT", b"A TXT CONTENT 2"), )
               )
        assert('Replace the uploaded file of ' in c)

        # Coverage of entity multiple logins
        c = s.url('=%s/%s/UE-upload_group/1/11/column_attr_type/C1/Firstname' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/1/12/column_attr_columns/C1/ID' % (abj, ys))
        assert(c == ok_png)

        # Unwritable table
        c = s.url('=%s/%s/UE-upload_group/3/1/table_attr_modifiable/0' % (root, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-upload_group/3/2/column_delete/C0' % (root, ys))
        assert(c == bad_png)
        c = s.post('=18000001/%s/UE-upload_group/upload_post/C0/L1' % ys,
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"A TXT CONTENT"), )
               )
        assert('not allowed to modify' in c)

        c = s.url('=%s/%s/UE-upload_group/page_unload' % (root, ys))
        assert('Unload done' in c)

    if do('config_login'):
        c = s.url('=%s/0/Dossiers/config_login' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_login/%s/0/cell_change/a/L1/ue7.master' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_login/%s/1/cell_change/b/L1/ue8.master' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_login/%s/2/cell_change/c/L1/Hello' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_login/%s/3/cell_change/a/L2/18000000' % (root, page))
        assert(c == ok_png) # For coverage (error message on browser)
        c = s.url('=ue7.master/%s/UE-login' % (ys,))
        page7 = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=ue7.master/%s/UE-login/%s/0/cell_change/0_0/L1/7' % (ys, page7))
        assert(c == ok_png)
        c = s.url('=ue8.master/%s/UE-login' % (ys,))
        page8 = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=ue8.master/%s/UE-login/%s/0/cell_change/0_0/L2/8' % (ys, page8))
        assert(c == ok_png)
        c = s.url('=ue8.master/%s/UE-login/%s/1/cell_change/0_0/L1/82' % (ys, page8))
        assert(c == bad_png)
        c = s.url('=ue7.master/%s/UE-login/%s/1/cell_change/0_0/L2/83' % (ys, page7))
        assert(c == ok_png)

    if do('tablecopy2'):
        c = s.url('=%s/0/Dossiers/config_table/tablecopy/0/Dossiers/columns' % root)
        assert('Read only table' in c)

        c = s.url('=%s/0/Dossiers/private' % root)
        c = s.url('=%s/0/Dossiers/private/tablecopy/0/Dossiers/columns' % abj)
        assert('Hacker!' in c)

        c = s.url('=%s/0/Dossiers/private/tablecopy/0/Dossiers/columns/copypy' % root)
        assert('copypy' in c)
        assert('OK: Copy seems fine' in c)

        c = s.url('=%s/0/Dossiers/private/tablecopy/0/Dossiers/columns' % root)
        assert('Impossible to copy' in c)

        c = s.url('=%s/0/Dossiers/private/1/0/column_attr_private/C0/' % root)
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/private/1/1/column_attr_private/C0/ue1.master' % root)
        assert(c == ok_png)

        c = s.url('=%s/0/Dossiers/private/tablecopy/0/Dossiers/columns/copypy' % root)
        assert('content→columns' not in c)
        assert('copypy' in c)
        assert('OK: Copy seems fine' in c)

        c = s.url('=%s/0/Dossiers/private/tablecopy/0/Dossiers/content/copypy' % root)
        assert('content → columns' in c)
        assert('copypy' in c)
        assert('OK: Copy seems fine' in c)

        c = s.url('=%s/0/Dossiers/private/tablecopy/0/Dossiers/badone/copypy' % root)
        assert('badone → columns' in c)

    if do('config_cache'):
        c = s.url('=%s/0/Dossiers/config_cache' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_cache/%s/0/cell_change/2/COLUMN_TYPES.text.read_url_not_cached/1' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cache/%s/1/cell_change/2/COLUMN_TYPES.text.read_url_not_cached/a' % (root, page))
        assert(c == bug_png)
        c = s.url('=%s/0/Dossiers/config_cache/%s/2/cell_change/1/COLUMN_TYPES.text.read_url_not_cached/comment' % (root, page))
        assert(c == ok_png)

    if do('suivistat'):
        create_tt()
        create_referents()
        create_u2()
        ss.start()
        c = ss.url('=' + abj + '/%s/%s' % (ys, root) )
        assert('<br class="noprint">' in c)

        c = ss.url('=' + root + '/%s/uninterested' % ys )
        assert('10800000' in c)
        assert('<td>10800000</td><td>' in c)

        c = ss.url('=' + root + '/%s/*3' % ys )
        assert(_("MSG_suivi_referents_nr_referents") in c)
        assert(_("MSG_suivi_referents_nr_students") in c)

        c = ss.url('=' + abj + '/%s/*1' % ys )
        assert(_("TITLE_suivi_bad_student") in c)
        assert("<tr><th>UE-numbers<td>10800000<td><td><td>['FOOBAR42']</tr>" in c
            or "<tr><th>UE-INF20UE2<td>10800001<td><td><td>[22.22, 'ABIGAT']</tr>" in c)
        #assert("<tr><th>UE-wc<td><td><td><td>[12.34, 12.35, 4.44]</tr>" in c)
        #assert("<tr><th>UE-repetition<td><td><td><td>['QQQQ', 'EEEE']</tr>" in c)

        c = ss.url('=' + root + '/%s/referents.csv' % ys )
        assert(';10800000;MARTIN;Jacques;surnametoto;Firstnametoto;mail@toto' in c)

    if do('suivitable'):
        create_tt()
        create_referents()
        create_u2()
        assert(os.system("cd REGTEST_SERVER ; make coverage-in") == 0)
        ss.start()

        c = ss.url('=' + abj + '/%s/*' % ys )
        assert(_("COL_TITLE_nb_cells_entered") in c)
        # le dictionnaire n'est pas toujours dans le meme ordre
        assert(': [C("%s"),' % abj in c)


        c = ss.url('=' + abj + '/%s/*2' % ys )
        assert(_("COL_TITLE_nb_teachers") in c)
        # assert('P([C("teachers","*"),' in c)
        # le dictionnaire n'est pas toujours dans le meme ordre
        assert(': [C("UE-INF20UE2"),' in c)

    if do('suivitable2'):
        # Some referent notepads have been created
        # So the stats are more complete
        # Here only for coverage
        ss.url('=' + abj + '/%s/*' % ys)

    if do('badpassword'):
        c = s.url('=ue4.master')
        assert('home3' in c)
        c = s.url('=ue5.master')
        assert('You must change your password' in c)

    if do('ifelse'):
        s.url('=%s/%s/UE-ifelse' % (abj, ys))
        c = s.url('=%s/%s/UE-ifelse/1/0/column_attr_title/C0/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/1/column_attr_title/C1/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/2/column_attr_type/C2/If_Else' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/3/cell_change/C0/L1/IFELSE=A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/4/cell_change/C1/L1/IFELSE=B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/5/cell_change/0_0/L1/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/6/column_attr_columns/C2/A%%20B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-ifelse/1/7/column_attr_test_if/C2/[ID]=18000001' % (abj, ys))
        assert(c == ok_png)

        ss.start()
        retry(lambda: ss.url('=18000001'), lambda c: c.count("IFELSE=A") == 2 and c.count("IFELSE=B") == 1)

        c = s.url('=%s/%s/UE-ifelse/1/8/column_attr_test_if/C2/[ID]=18000000' % (abj, ys))
        assert(c == ok_png)

        retry(lambda: ss.url('=18000001'),
              lambda c: c.count("IFELSE=A") == 1 and c.count("IFELSE=B") == 2)

        c = s.url('=%s/%s/UE-ifelse/column_speak/8/0/18000001' % (root, ys))
        assert(c == 'espeak not installed' or 'RIFF' in c and 'WAVE' in c)

        c = s.url('=%s/%s/badUE-ifelse/column_speak/8/0/18000001' % (root, ys))
        assert(c == '')

    if do('prof'):
        if os.path.exists('xxx.prof'):
            os.unlink('xxx.prof')
        c = s.url('PROFILE_THIS_URL/=%s' % root)
        assert('],"Home");' in c)
        retry(lambda:'', lambda c: os.path.exists('xxx.prof'))

        os.unlink('xxx.prof')
        ss.start()
        c = ss.url('PROFILE_THIS_URL/=%s/%s' % (root, ys))
        assert('<br class="noprint">' in c)
        retry(lambda:'', lambda c: os.path.exists('xxx.prof'))

    if do('tmp'):
        utilities.mkpath('TMP/TICKETS_TMP/' + root)
        utilities.write_file('TMP/TICKETS_TMP/' + root + '/my_file', 'hello')
        c = s.url('=%s/tmp/my_file' % root)
        assert(c == 'hello')

    if do('acls2'):
        c = s.url('=%s/0/Dossiers/config_table' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('='+root+'/0/Dossiers/config_table/%s/0/cell_change/0_2/login_not_teacher/("ue6.master",)' % page)
        assert(c == ok_png)

    if do('config_display'):
        c = s.url('=%s' % root)
        assert("HomeUEBookmarked" in c)
        c = s.url('=%s/0/Dossiers/config_display' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('='+root+'/0/Dossiers/config_display/%s/0/cell_change/v/HomeUEBookmarked/0' % page)
        assert(c == ok_png)
        c = s.url('=%s' % root)
        assert('HomeUEBookmarked' not in c)

    if do('column'):
        s.url('=' + abj + '/%s/UE-column' % ys)
        c = s.url('=' + abj + '/%s/UE-column' % ys + '/1/0/column_attr_author/C0/xxx')
        assert(c == bad_png)
        c = s.url('=' + abj + '/%s/UE-column' % ys + '/1/1/column_delete/C0')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-column' % ys + '/1/2/column_attr_title/C0/C0title')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-column' % ys + '/1/3/cell_change/C0/L1/5.55')
        assert(c == ok_png)
        c = s.url('=' + abj + '/%s/UE-column' % ys + '/1/4/column_delete/C0')
        assert(c == bad_png)

    if do('incomplete'):
        s.url('=%s/%s/UE-incomplete' % (abj, ys))
        c = s.url('=%s/%s/UE-incomplete/1/0/column_attr_title/C0/C0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/1/column_attr_title/C1/C1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/2/column_attr_visibility/C1/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/3/column_attr_type/C2/Moy' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/4/column_attr_columns/C2/C0%%20C1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/5/column_attr_rounding/C2/0.1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/6/cell_change/C0/L1/1.111' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/7/cell_change/C1/L1/17.333' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/8/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        c = ss.url('=18000000')
        assert('[9.2, "*"]' in c)
        assert('17.3' not in c)

        c = s.url('=%s/%s/UE-incomplete/1/9/column_attr_visibility/C1/4' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/10/column_attr_visibility/C1/7' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-incomplete/1/11/column_attr_minmax/C1/[-20;20]' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=18000000')
        assert('17.3' in c)


        c = s.url('=%s/%s/UE-incomplete/1/12/column_attr_visibility/C1/0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-incomplete/1/13/column_attr_visibility_date/C1/<17' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=18000000')
        assert('17.3' not in c)
        c = s.url('=%s/%s/UE-incomplete/1/14/column_attr_visibility_date/C1/<18' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=18000000')
        assert('17.3' in c)
        c = s.url('=%s/%s/UE-incomplete/1/15/column_attr_visibility_date/C1/<[bad]' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=18000000')
        assert('17.3' not in c)

        c = s.url('=%s/%s/UE-incomplete/1/16/column_attr_visibility/C1/6' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=18000000')
        assert('17.3' not in c)
        c = s.url('=%s/%s/UE-incomplete/1/17/column_attr_visibility_date/C1/' % (abj, ys))
        assert(c == ok_png)
        c = ss.url('=18000000')
        assert('17.3' in c)

        c = ss.url('=' + abj)
        assert('17.3' not in c)

    if do('visible'):
        s.url('=%s/%s/UE-visible' % (abj, ys))
        c = s.url('=%s/%s/UE-visible/1/0/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        c = ss.url("=18000000")
        assert('UE-visible' in c)
        c = s.url('=%s/%s/UE-visible/1/1/table_attr_official_ue/0' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=18000000")
        assert('UE-visible' not in c)
        c = s.url('=%s/%s/UE-visible/1/2/table_attr_official_ue/1' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=18000000")
        assert('UE-visible' in c)

    if do('delete_ro'):
        s.url('=%s/%s/UE-delete_ro' % (abj, ys))
        c = s.url('=%s/%s/UE-delete_ro/1/0/table_attr_masters/%s' % (abj, ys, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-delete_ro/1/1/table_attr_modifiable/0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-delete_ro/delete_this_table' % (abj, ys))
        assert("Unmodifiable table are not deletable" in c)

    if do('bad-master'):
        s.url('=%s/%s/UE-badmaster' % (abj, ys))
        c = s.url('=%s/%s/UE-badmaster/1/0/table_attr_masters/foobar' % (abj, ys))
        assert(c == bad_png)
        c = s.url('=%s/%s/UE-badmaster/1/1/table_attr_masters/%s' % (abj, ys, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-badmaster/1/2/table_attr_masters/' % (abj, ys))
        assert(c == ok_png)

    if do('updatecontent2'):
        # Unregistred student line become modifiable evan if extracted data
        students = ['1234560{0} FN{0} SN{0} M{0}@x.y A {0}'.format(i)
                    for i in range(7)]
        utilities.write_file('xxx.students', '\n'.join(students[0:4]))
        s.url('=%s/%s/UE-update2' % (abj, ys))
        c = s.url('=%s/%s/UE-update2/1/0/column_attr_title/C0/C0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-update2/1/1/cell_change/C0/1_0/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-update2/1/2/column_attr_type/C1/Code_Etape' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-update2/1/3/column_attr_columns/C1/ID' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-update2/1/4/update_content/column' % (abj, ys))
        assert(c == ok_png)

        c = s.url('=%s/%s/UE-update2' % (abj, ys))
        c = replace_dates(c)
        assert('P("1_0",[C(),C(),C(),C(),C(),C("non","*","DATE"),C(1.0,"tt.master","DATE"),C("","*","DATE")]);' in c)
        assert('P("0_3",[C("12345603","*","DATE"),C("FN3","*","DATE"),C("SN3","*","DATE"),C("A","*","DATE"),C("3","*","DATE"),C("ok","*","DATE"),C(),C("etape-12345603","*","DATE","",[["","DATE","*","V"]])]);' in c)

        utilities.write_file('xxx.students', '\n'.join(students[:3]))
        c = s.url('=%s/%s/UE-update2/1/5/update_content/add/remove' % (abj, ys))
        assert(c == ok_png)

        c = s.url('=%s/%s/UE-update2' % (abj, ys))
        c = replace_dates(c)
        assert('P("1_0",[C(),C(),C(),C(),C(),C("non","*","DATE"),C(1.0,"tt.master","DATE"),C("","*","DATE")]);' in c)
        assert('P("0_3",[C("","*","DATE","",[["12345603","DATE","*","V"]]),C("","*","DATE","",[["FN3","DATE","*","V"]]),C("","*","DATE","",[["SN3","DATE","*","V"]]),C("","*","DATE","",[["A","DATE","*","V"]]),C("","*","DATE","",[["3","DATE","*","V"]]),C("","*","DATE","",[["ok","DATE","*","V"]]),C(),C("","*","DATE","",[["","DATE","*","V"],["etape-12345603","DATE","*","V"]])])' in c)

    if do('filtre-historique'):
        s.url('=%s/%s/UE-histo' % (abj, ys))
        c = s.url('=%s/%s/UE-histo/1/0/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-histo/1/1/column_attr_visibility_date/C0/!:~17.77' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-histo/1/2/cell_change/C0/L1/18.88' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        c = ss.url("=18000000")
        assert('[18.88' in c)
        c = s.url('=%s/%s/UE-histo/1/3/cell_change/C0/L1/17.77' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-histo/1/4/cell_change/C0/L1/18.88' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-histo/1/5/column_attr_title/C0/C0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-histo/1/6/column_attr_url_import/C1/:C0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-histo' % (abj, ys))
        c = replace_dates(c)
        assert('''C("[(18.88, "DATE", '%s', 'V'), (17.77, "DATE", '%s', 'V')]","*","DATE")'''
            % (abj, abj) in c)
        ss.start()
        c = ss.url("=18000000")
        assert('["[(18.88,' in c)
        assert('[18.88' not in c)

        # Create a fake 'tables' (should be computed by SCRIPTS/bilan.py)
        utilities.mkpath('DBregtest/LOGINS/%s/%s' % (abj[:3], abj))
        utilities.write_file('DBregtest/LOGINS/%s/%s/tables' % (abj[:3], abj),
            "{(2021, 'Printemps', 'UE-INF1001L'):4242}")
        c = ss.url("=%s" % abj)
        assert('"Tables", [[2021, "Printemps", "UE-INF1001L", 4242]]' in c)

        # Textual
        c = ss.url("=%s/%s/18000000/*" % (abj, ys))
        assert('],"Textual")' in c)

    if do('rgpd'):
        ss.start()
        c = ss.url("=%s/18000000" % invited)
        assert("UE-UE5" in c)
        c = s.url('=18000000/save_preferences/private_suivi=1')
        assert(c == ok_png_bin)
        retry(lambda: ss.url("=%s/18000000" % invited), lambda c:  "UE-UE5" not in c)
        c = s.url("=%s/ask_share/18000000" % invited)
        assert('Share your TOMUSS page with' in c)
        c = ss.url('=18000000')
        assert('"share_with_%s": -1' % invited in c)
        c = s.url('=18000000/save_preferences/share_with_%s=1' % invited)
        assert(c == ok_png_bin)
        c = ss.url("=%s/18000000" % invited)
        assert("UE-UE5" in c)

    if do('config_cron'):
        c = s.url('=%s/0/Dossiers/config_cron' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        now = time.strftime('%Y-%m-%d%%20%H:%M:%S', time.localtime(time.time()+4))
        c = s.url('=%s/0/Dossiers/config_cron/%s/0/cell_change/a/x/%s' % (root, page, now))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/1/cell_change/b/x/0' % (root, page))
        assert(c == ok_png)
        action = "plugin:%s@0/Dossiers/config_cron/cell/e/x/PluginTestOk" % root
        action = action.replace("/", "$2F")
        c = s.url('=%s/0/Dossiers/config_cron/%s/2/cell_change/d/x/%s' % (root, page, action))
        assert(c == ok_png)

        c = s.url('=%s/0/Dossiers/config_cron/%s/3/cell_change/a/y/%s' % (root, page, now))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/4/cell_change/d/y/python:True' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/5/cell_change/a/z/%s' % (root, page, now))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/6/cell_change/d/z/python:Tru' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/7/cell_change/a/t/%s' % (root, page, now))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/8/cell_change/d/t/' % (root, page))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/9/cell_change/a/u/%s' % (root, page, now))
        assert(c == ok_png)
        c = s.url('=%s/0/Dossiers/config_cron/%s/10/cell_change/d/u/python:1%%2F0' % (root, page))
        assert(c == ok_png)

        retry(lambda: s.url('=%s/0/Dossiers/config_cron' % root),
              lambda c: '"PluginTestOk"' in c)

        c = s.url('=%s/cron' % (root,))
        assert('Average' in c)

    if do('config_home'):
        c = s.url('=%s/0/Dossiers/config_home' % root)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Dossiers/config_home/%s/0/cell_change/0/0/x' % (root, page))
        assert(c == ok_png)

    if do('cow'):
        s.url('=%s/%s/UE-cow' % (abj, ys))
        c = s.url('=%s/%s/UE-cow/1/0/column_attr_title/C0/C0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-cow/1/1/column_attr_type/C1/COW' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-cow/1/2/column_attr_columns/C1/C0' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-cow/1/3/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-cow/1/4/cell_change/C0/L1/1.21' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        time.sleep(0.1)
        c = ss.url("=18000000")
        assert(c.count('1.21') == 6)
        c = s.url('=%s/%s/UE-cow/1/5/cell_change/C1/L1/1.27' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=18000000")
        assert(c.count('1.21') == 3)
        assert(c.count('1.27') == 3)
        c = s.url('=%s/%s/UE-cow/1/6/cell_change/C0/L1/1.29' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=18000000")
        assert(c.count('1.29') == 3)
        assert(c.count('1.27') == 3)
        c = s.url('=%s/%s/UE-cow/1/7/cell_change/C0/L1/1.28' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=18000000")
        assert(c.count('1.28') == 3)
        c = s.url('=%s/%s/UE-cow/1/8/cell_change/C1/L1/' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=18000000")
        assert(c.count('1.28') == 6)

    if do('nan'):
        s.url('=%s/%s/UE-nan' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/0/column_attr_title/C0/C0' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/1/column_attr_title/C1/C1' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/2/column_attr_type/C2/Moy' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/3/column_attr_columns/C2/C0%%20C1' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/4/cell_change/0_0/L1/18000000' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/5/table_attr_masters/%s' % (abj, ys, abj))
        ss.start()
        time.sleep(0.1)
        c = ss.url("=18000000")
        assert('[], [], [NaN, "?"]]' in c)
        s.url('=%s/%s/UE-nan/1/6/cell_change/C0/L1/1' % (abj, ys))
        s.url('=%s/%s/UE-nan/1/7/cell_change/C1/L1/2' % (abj, ys))
        c = ss.url("=18000000")
        assert('], [1.5' in c)
        c = s.url('=%s/%s/UE-nan/1/8/cell_change/C2/L1/16.43' % (abj, ys))
        assert(c == bad_png)

        s.url('=%s/%s/UE-nan/1/9/cell_change/0_0/L2/18000001' % (abj, ys))
        c = s.url('=%s/%s/UE-nan/1/10/cell_change/C2/L2/16.43' % (abj, ys))
        assert(c == ok_png)

        retry(lambda: ss.url("=18000001"), lambda c: '], [16.43' in c)

        # COW on Moy
        c = s.url('=%s/%s/UE-nan/1/11/column_attr_title/C2/C2' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-nan/1/12/column_attr_type/C3/COW' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-nan/1/13/column_attr_columns/C3/C2' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-nan/1/14/cell_change/C3/L1/17.41' % (abj, ys))
        assert(c == ok_png)

        retry(lambda: ss.url("=18000000"), lambda c: '], [17.41' in c)

    if do('unknown'):
        s.url('=%s/%s/UE-unknown' % (abj, ys))
        s.url('=%s/%s/UE-unknown/1/0/cell_change/0_0/L1/00000000' % (abj, ys))
        c = replace_dates(s.url('=%s/%s/UE-unknown' % (abj, ys)))
        assert('C("00000000","tt.master","DATE"),C("","*","DATE"),C("","*","DATE")' in c)

    if do('users'):
        # Do not start alone, take the previous test
        c = s.url('=%s/users' % (root,))
        assert('>{}<'.format(root) in c)
        c = s.url('=%s/users/_doc_' % (root,))
        assert('</table>' in c)
        c = s.url('=%s/users/%s' % (root, root))
        nb = c.count('<tr>')
        assert(nb >= 10)
        c = s.url('=%s/users/_update_/%s' % (root, root))
        assert(c.count('<tr>') == nb)
        c = s.url('=%s/users/_UPDATE_/%s' % (root, root))
        assert(c.count('<tr>') > nb)
        c = s.url('=%s/users/_clear_/%s' % (root, root))
        assert(c.split('</table>')[0].count('<tr>') == 3)
        assert(c.split('</table>')[1].count('<tr>') == 1)
        c = s.url('=%s/users/_cleanup_/0' % (root,))
        assert('THE END' in c) # and  '*' in c)
        c = s.url('=%s/users/_UPDATE_/%s' % (root, root))
        assert(c.count('<tr>') > nb)
        c = s.url('=%s/users/_erase_/%s' % (root, root))
        assert('THE END' in c and ('*' in c or '@' in c))

    #if do('suivitable3'):
    #    ss.start()
    #    ss.url('=' + abj + '/%s/*2' % ys )

    if do('teachers2'):
        remove_abjs()
        create_u2()
        c = s.url('=%s/%s/Teachers/UE-teach' % (abj, uyear))
        c = s.url('=%s/%s/Teachers/UE-teach/2/0/cell_change/10Mail/L1/test1@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/1/cell_change/20UE/L1/UE-INF20UE2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/2/cell_change/30Semester/L1/%s' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/3/cell_change/10Mail/L2/test2@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/4/cell_change/20UE/L2/UE-INF20UE2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/5/cell_change/30Semester/L2/%s' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/6/cell_change/40TD/L2/A' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/7/cell_change/10Mail/L3/test3@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/8/cell_change/20UE/L3/UE-INF20UE2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/9/cell_change/30Semester/L3/BadSemester' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/10/cell_change/10Mail/L4/test4@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/11/cell_change/20UE/L4/BadUE' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/12/cell_change/30Semester/L4/%s' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/13/cell_change/10Mail/L6/test6@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/14/cell_change/20UE/L6/UE-INF20UE2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/15/cell_change/30Semester/L6/%s' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/16/cell_change/40TD/L6/A%%20B+C' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/17/cell_change/10Mail/L7/test6@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/18/cell_change/20UE/L7/UE-INF20UE2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/19/cell_change/30Semester/L7/%s' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/20/cell_change/40TD/L7/B42%%20B42' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/21/cell_change/80MailResp/L7/%s' % (abj, uyear, configuration.yes))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/22/cell_change/90SendXLS/L7/%s' % (abj, uyear, configuration.no))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/23/cell_change/95Duration/L1/90' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach/2/24/cell_change/95Duration/L7/90' % (abj, uyear))
        assert(c == ok_png)
        if os.path.exists("xxx.mail"):
            os.unlink('xxx.mail')
        c = s.post('=%s/teachers_mail' % (abj,),
                   fields = ( ("year", str(uyear)), ('ue', 'UE-teachBAD'), ('lin_ids', 'L1') ),
                  )
        assert('modifications are forbidden' in c)
        c = s.post('=%s/teachers_mail' % (abj,),
                   fields = ( ("year", str(uyear)), ('ue', 'UE-teach'), ('lin_ids', 'L1 L2 L3 L6 L4 L5 L7') ),
                  )
        assert(c.count("BadSemester/UE-INF20UE2 This table does not exist!") == 1)
        assert(c.count("%s/BadUE This table does not exist!" % semester) == 1)
        assert('Teachers/UE-teach</h1>' in c)
        assert('<div>line_id=L5</div>' in c)
        c = utilities.read_file('xxx.mail')
        assert("""MIME-Version: 1.0
From: mail@mail.fr
To: test1@fr
Subject: UE-INF20UE2""" in c)
        assert(len(c) > 15000)
        n = c.count('openxmlformats')
        if n not in (4,):
            raise ValueError(n)
        n = c.count('Content-Type: multipart/related')
        if n not in (4,):
            raise ValueError(n)

        #c = s.url('=%s/%s/UE-INF20UE2' % (abj, ys))
        #page = c.split('page_id = "')[1].split('"')[0]
        #c = s.url('=%s/%s/UE-INF20UE2/%s/0/table_attr_masters/%s' % (abj, ys, page, abj))
        #assert(c == ok_png)
        c = s.url('=ue1.master/%s/UE-INF20UE2/delete_this_table' % (ys,))
        assert('Deletion done' in c)
        del done['create_u2']
        create_u2()

        s.url('=%s/%s/P/UE-teach' % (abj, uyear))
        c = s.url('=%s/%s/P/UE-teach/2/0/table_attr_masters/%s%%20%s' % (abj, uyear, abj, root))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/UE-teach/2/1/column_attr_comment/6/{"pc_abi":"0.5","student_rule":".*","abi_content":"{}"}' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=10800000/SVC/'
            + sign('UE-teach/c/45/PRST/UE-INF20UE2%%20%s%%20%%20%s' % (abj, semester), '%s/P' % uyear))
        assert('1 PRST' in c)
        os.unlink('xxx.mail')
        c = s.url('=%s/SVC/' % abj + sign('UE-teach/c/45//', '%s/P' % uyear))
        assert('1 PRST' in c)
        time.sleep(0.1)
        retry(lambda: utilities.read_file('xxx.mail') if os.path.exists('xxx.mail') else '',
              lambda c: 'UNJLEAVE had not been recorded because they are 66% (>50%)' in c)

        c = s.url('=%s/%s/P/UE-teach/delete_this_table' % (abj, uyear))
        assert('Deletion done' in c)
        s.url('=%s/%s/P/UE-teach' % (abj, uyear))
        c = s.url('=%s/%s/P/UE-teach/2/0/table_attr_masters/%s%%20%s' % (abj, uyear, abj, root))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/UE-teach/2/1/column_attr_comment/6/{"pc_abi":"1","student_rule":".*","abi_content":"{}","abi_cc":"abi_cc@mail.fr"}' % (abj, uyear))
        assert(c == ok_png)
        # Verify mail CC to table manager on ABI
        c = s.url('=18000001/SVC/'
            + sign('UE-teach/c/46/PRST/UE-INF20UE2%%20%s%%20A%%20%s' % (abj, semester), '%s/P' % uyear))
        assert('1 PRST' in c)
        if os.path.exists("xxx.mail"):
            os.unlink('xxx.mail')
        c = s.url('=%s/SVC/' % abj + sign('UE-teach/c/46//', '%s/P' % uyear))
        assert('1 PRST' in c)
        time.sleep(0.1)
        retry(lambda: utilities.read_file('xxx.mail') if os.path.exists('xxx.mail') else '',
              lambda c: 'CC: mail@mail.fr' in c
                        and 'CC: abi_cc@mail.fr' in c
                        and 'Subject: UNJLEAVE UE-INF20UE2 tt.master' in c)

        c = s.url('=%s/%s/UE-INF20UE2/attendance' % (abj, ys))
        assert('"1": [C("10800000"),C("MARTIN"),C("Jacques"),C("1"),C("UNJLEAVE","","","tt.master")]' in c)

    if do('extract2'):
        # Key test
        s.url('=' + abj + '/%s/UE-extract-src' % ys)
        c = s.url('=%s/%s/UE-extract-src/1/0/column_attr_title/C0/VALUE' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-src/1/1/cell_change/0_0/L1/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-src/1/2/cell_change/0_0/L2/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-src/1/3/cell_change/C0/L1/1024' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-src/1/4/cell_change/C0/L2/2048' % (abj, ys))
        assert(c == ok_png)

        s.url('=' + abj + '/%s/UE-extract-dst' % ys)
        c = s.url('=%s/%s/UE-extract-dst/1/0/column_attr_title/C0/KEY' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/1/cell_change/0_0/L1/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/2/cell_change/0_0/L2/18000002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/3/cell_change/0_0/L3/18000003' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/4/cell_change/C0/L1/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/5/cell_change/C0/L2/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/6/cell_change/C0/L3/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/7/column_attr_url_import/C1/UE-extract-src$2FVALUE[KEY]' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-extract-dst/1/8/column_attr_url_import/C2/UE-extract-src$2FVALUE[BAD]' % (abj, ys))
        assert(c == ok_png)

        c = s.url('=' + abj + '/%s/UE-extract-dst' % ys)
        assert(c.count('C(1024') == 1)
        assert(c.count('C(2048') == 2)

    if do('find_cycles') and not search_leak:
        c = s.url('=%s/find_cycles' % root)
        assert("&lt;class &#x27;tuple&#x27;&gt;<br>" in c)

    if do('find_garbage') and not search_leak:
        c = s.url('=%s/find_garbage' % root)
        assert("Press Escape to free objects" in c)

    if do('dashboard'):
        c = s.url('=%s/dashboard' % root)
        key = c.split("<script>initialize(")[1].split('"')[-2]
        c = s.url('dashboard_data/%s' % key)
        assert(c.startswith('['))
        c = s.url('dashboard_data/badkey')
        assert(c.startswith('Bad'))

    if do('qrcode'):
        s.url('=%s/%s/UE-qrcode' % (abj, ys))
        c = s.url('=%s/%s/UE-qrcode/1/0/column_attr_type/C0/Prst' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-qrcode/1/1/cell_change/0_0/L1/10800000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-qrcode/1/2/cell_change/0_0/L2/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-qrcode/1/3/cell_change/0_0/L3/10800002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-qrcode/1/4/column_attr_type/C1/Prst' % (abj, ys))
        assert(c == ok_png)


        s.url('=%s/%s/UE-qrcode' % (root, ys))
        c = s.url('=%s/%s/UE-qrcode/2/0/cell_change/C0/L2/PRST' % (root, ys))
        assert(c == ok_png)


        c = s.url('=%s/%s/UE-qrcode/qrcode_new/6' % (abj, ys))
        assert('["10800001Surname", "10800001Firstname", "10800001", "PRST", false, false]' in c)
        assert('["MARTIN", "Jacques", "10800000", "", true, false]' in c)
        qrcode = c.split('var qrcode = ')[1].split('"')[1]

        def read():
            while not os.path.exists('xxx.qrcode'):
                time.sleep(0.01)
            time.sleep(0.02)
            c = utilities.read_file('xxx.qrcode')
            shutil.rmtree('xxx.qrcode', ignore_errors=True)
            return c

        shutil.rmtree('xxx.qrcode', ignore_errors=True)
        c = s.url('=%s/%s/UE-qrcode/1/5/cell_change/C0/L1/PRST' % (abj, ys))
        c = read()
        assert('update("10800000","PRST",1)' in c)
        assert('update_stats(0)' in c)

        c = s.url('=%s/qrcode_update/badqrcode/set/10800000/UNJLEAVE' % abj)
        assert(c == bug_png_bin)

        c = s.url('=%s/qrcode_update/%s/set/10800000/UNJLEAVE' % (abj, qrcode))
        assert(c == ok_png_bin)
        c = read()
        assert('update("10800000","UNJLEAVE",1);update_stats(0);' in c)

        c = s.url('=%s/qrcode_update/%s/set/10800000/PRST' % (abj, qrcode))
        assert(c == ok_png_bin)
        c = read()
        assert('update("10800000","PRST",1);update_stats(0);' in c)

        c = s.url('=%s/qrcode_update/%s/set/10800001/PRST' % (abj, qrcode))
        assert(c == ok_png_bin)
        c = read()
        assert('update("10800000","PRST",1);update_stats(0);' in c)

        c = s.url('=%s/qrcode_update/%s/set/10800001/JUSLEAVE' % (abj, qrcode))
        assert(c == bad_png_bin)
        c = read()
        assert('update("10800000","PRST",1);update_stats(0);' in c)


        c = s.url('=10800002/qrcode_check/%s' % qrcode)
        assert('#F00' in c)

        c = s.url('=10800002/qrcode_check/bad')
        assert('#F00' in c)

        c = s.url('=%s/qrcode_update/%s/start' % (abj, qrcode))
        assert(c == ok_png_bin)
        c = read()
        assert('update_time(60)' in c)

        c = s.url('=10899992/qrcode_check/%s' % qrcode)
        assert('not registered' in c)

        c = s.url('=10800002/qrcode_check/%s' % qrcode)
        assert('#0F0' in c)
        c = read()
        assert('update_stats(1)' in c)

        c = s.url('=10800001/qrcode_check/%s' % qrcode)
        assert('#0F0' in c)

        c = s.url('=%s/qrcode_update/%s/stop' % (abj, qrcode))
        assert(c == ok_png_bin)
        c = read()
        assert('update_time(0)' in c)


        c = s.url('=%s/%s/UE-qrcode/qrcode_new/7/L1' % (abj, ys))
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        s.url('=%s/qrcode_update/%s/start' % (abj, qrcode))

        c = s.url('=10800000/qrcode_check/%s' % qrcode)
        assert('#0F0' in c and '<br>UE-qrcode<br></body>' in c)
        c = s.url('=10800000/qrcode_check/%s' % qrcode)
        assert('#0F0' in c and '<br>UE-qrcode<br></body>' not in c)
        c = s.url('=10800001/qrcode_check/%s' % qrcode)
        assert('#F00' in c)

        c = s.url('=%s/%s/UE-qrcode/qrcode_new/7/grpseq//' % (abj, ys))
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url('=%s/%s/UE-qrcode/qrcode_new/7/grpseq/seq/' % (abj, ys))
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url('=%s/%s/UE-qrcode/qrcode_new/7/grpseq/seq/grp' % (abj, ys))
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url('=%s/%s/UE-qrcode/qrcode_new/7/grpseq/[["",""]]' % (abj, ys))
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url('=%s/%s/UE-qrcode/qrcode_new/7/grpseq/[]' % (abj, ys))
        qrcode = c.split('var qrcode = ')[1].split('"')[1]

        c = s.url('=%s/qrcode_debug' % abj)
        assert(c.count('UE-qrcode/') == 6
            and ", '10800000') 0<br>" in c
            and ", '10800001') 0<br>" in c
            and ", '10800002') 0<br>" in c
            )

    if do('teachers3'):
        # Coverage for sequence and other column group in Teacher TD
        create_u2()
        c = s.url('=%s/%s/Teachers/UE-teach3' % (abj, uyear))
        c = s.url('=%s/%s/Teachers/UE-teach3/2/0/cell_change/10Mail/L1/test1@fr' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach3/2/1/cell_change/20UE/L1/UE-INF20UE2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach3/2/2/column_attr_empty_is/30Semester/%s' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/UE-teach3/2/3/cell_change/40TD/L1/ID=1%%201-A%%20ID=2-B' % (abj, uyear))
        assert(c == ok_png)
        c = s.post('=%s/teachers_mail' % (abj,),
                   fields = ( ("year", str(uyear)), ('ue', 'UE-teach3'), ('lin_ids', 'L1') ),
                  )
        assert('THE END' in c)

        c = s.url('=18000000/SVC/'
                  + sign('UE-teach3/C/200/PRST/UE-INF20UE2%20Teacher1%20Grp=A+Grp=-A+ZZZ=B%20{}'
                         .format(semester)))
        assert('1 PRST' in c)
        c = s.url('=18000000/SVC/' + sign('UE-teach3/C/200//'))
        assert('1 PRST' in c)

        c = s.url('=%s/%s/Teachers/UE-teach3/2/4/cell_change/40TD/L1/GGG=A' % (abj, uyear))
        assert(c == ok_png)
        c = s.post('=%s/teachers_mail' % (abj,),
                   fields = ( ("year", str(uyear)), ('ue', 'UE-teach3'), ('lin_ids', 'L1') ),
                  )
        assert('this column does not exists : «GGG»' in c)

    if do('SVC'):
        today = time.strftime('%Y-%m-%d_%H:')
        c = s.url('=%s/%s/UE-SVC' % (abj, ys))
        assert('initialize()' in c)
        c = s.url('=%s/%s/UE-SVC/1/0/column_attr_type/C0/Prst' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/1/column_attr_title/C0/%s00' % (abj, ys, today))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/2/column_attr_type/C1/Prst' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/3/column_attr_title/C1/%s59' % (abj, ys, today))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/4/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/5/cell_change/0_0/L2/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/6/table_attr_masters/%s' % (abj, ys, abj))
        assert(c == ok_png)

        c = s.url('=18000000/SVC/' + sign('UE-SVC/6/1/value/comment'))
        assert('1 value' in c)
        assert('\ncomment#1' in c)
        assert('\nvalue\n' in c)
        c = replace_dates(s.url('=%s/%s/UE-SVC' % (abj, ys)))
        assert('C("value","18000000","DATE","comment#1",[' in c)
        c = s.url('=18000002/SVC/' + sign('UE-SVC/6/1/hell/'))
        assert('The student is not registered for this UE.' in c)
        c = s.url('=18000002/SVC/' + sign('UE-SVC/6/1//')) # Close QRCode0
        assert('1 value' in c)
        assert('\nvalue\n' not in c)
        assert('>hell<' not in c)
        assert('\n#1\n<br>' in c)
        assert('>Answering is no more possible<' in c)
        c = s.url('=18000000/SVC/' + sign('UE-SVC/6/1/PRST/'))
        assert('1 value' in c)
        assert('\n#1\n<br>' in c)
        assert('\nvalue\n' in c)
        assert('>Answering is no more possible<' in c)

        c = s.url('=18000000/SVC/' + sign('UE-SVC/6/2/PRST/'))
        assert('1 PRST' in c)
        assert('\n#2\n' in c)
        assert('\nPRST\n' in c)
        assert('href="regtest_rgpd_link"' in c)
        assert('>Answering is no more possible<' not in c)
        c = replace_dates(s.url('=%s/%s/UE-SVC' % (abj, ys)))
        assert('C("PRST","18000000","DATE","#2",[["comment#1","DATE","18000000","C"],["value","DATE","18000000","V"],["#2","DATE","18000000","C"]]' in c)

        c = s.url('=18000000/SVC/' + sign('UE-SVC/6/2/PRST/') + "badsign")
        assert(c == 'HACKER')

        c = s.url('=18000001/SVC/' + sign('UE-SVC/D/3/PRST/'))
        assert('1 PRST' in c)
        assert('\n#3' in c)
        assert('\nPRST\n' in c)
        assert('>Answering is no more possible<' not in c)

        c = replace_dates(s.url('=%s/%s/UE-SVC' % (abj, ys)))
        assert('C("PRST","18000001","DATE","#3",[["#3","DATE","18000001","C"]])' in c)

        c = s.url('=%s/%s/UE-SVC/1/7/column_attr_cell_writable/C0/X' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC/1/8/column_attr_cell_writable/C1/X' % (abj, ys))
        assert(c == ok_png)

        c = s.url('=18000000/SVC/' + sign('UE-SVC/D/3/ABINJ/'))
        assert('1 PRST' in c)
        assert('\n#3' in c)
        assert('\nPRST\n' in c or '\n???\n' in c) # Realtime dependent
        assert('>You are not allowed to modify this value.<' in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC/D/3/ABINJ/'))
        assert('1 PRST' in c)
        assert('\n#3' in c)
        assert('\nPRST\n' in c)
        assert('>You are not allowed to modify this value.<' in c)

        s.url('=%s/%s/UE-SVC2' % (abj, ys))
        c = s.url('=%s/%s/UE-SVC2/1/0/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC2/1/1/cell_change/0_0/L2/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=18000000/SVC/' + sign('UE-SVC2/D/42/PRST/'))
        assert(c == 'No column')
        c = s.url('=18000000/SVC/' + sign('UE-SVC2/C/43/PRST/'))
        assert('1 PRST' in c)
        assert('\n#43' in c)
        assert('\nPRST\n' in c)
        c = s.url('=%s/%s/UE-SVC2' % (abj, ys))
        assert('Col({"type": "Prst", "author": "?", "position": 6, "qrcode_prst": [43], "title": "20' in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC2/C/44/PRST/'))
        assert('1 PRST' in c)
        assert('\n#44' in c)
        assert('\nPRST\n' in c)
        c = s.url('=%s/%s/UE-SVC2' % (abj, ys))
        assert('Col({"type": "Prst", "author": "?", "position": 6, "qrcode_prst": [43, 44], "title": "20' in c)
        c = s.url('=%s/SVC/' % abj + sign('UE-SVC2/C/43//'))
        assert('1 PRST' in c)
        assert('\n#43\n<br>' in c)
        assert('\n???\n' not in c)
        assert('Answering is no more possible' in c)
        c = s.url('=%s/%s/UE-SVC2' % (abj, ys))
        assert('Col({"type": "Prst", "author": "?", "position": 6, "qrcode_prst": [44, -43], "title": "20' in c)
        c = s.url('=18000003/SVC/' + sign('UE-SVC2/C/44/PRST/'))
        assert('2 PRST' in c)
        assert('\n#44' in c)
        assert('\nPRST\n' in c)
        c = s.url('=18000003/SVC/' + sign('UE-SVC2/C/44/ABINJ/'))
        assert('1 PRST' in c)
        assert('1 ABINJ' in c)
        assert('\n#44' in c)
        assert('\nABINJ\n' in c)
        c = s.url('=%s/%s/UE-SVC2' % (abj, ys))
        assert('18000003' in c)

    if do('SVC-ABI'):
        # Define the UE with students
        c = s.url('=%s/%s/UE-SVC-ABI' % (abj, ys))
        assert('initialize()' in c)
        c = s.url('=%s/%s/UE-SVC-ABI/1/0/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI/1/1/cell_change/0_0/L2/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI/1/2/cell_change/0_0/L3/18000002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI/1/3/cell_change/0_0/L4/18000003' % (abj, ys))
        assert(c == ok_png)
        time.sleep(0.1) # update student erase the group
        c = s.url('=%s/%s/UE-SVC-ABI/1/4/cell_change/0_3/L1/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI/1/5/cell_change/0_3/L2/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI/1/6/cell_change/0_3/L3/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI/1/7/cell_change/0_3/L4/B' % (abj, ys))
        assert(c == ok_png)

        s.url('=%s/%s/UE-SVC-PRST' % (abj, ys)) # Where PRST are recorded
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST/C/99/PRST/UE-SVC-ABI%20Teacher1%20%20{}'.format(semester)))
        assert('1 PRST' in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC-PRST/C/99/PRST/UE-SVC-ABI%20Teacher1%20%20{}'.format(semester)))
        assert('2 PRST' in c)
        c = s.url('=18000002/SVC/' + sign('UE-SVC-PRST/C/99/PRST/UE-SVC-ABI%20Teacher1%20%20{}'.format(semester)))
        assert('3 PRST' in c)
        if os.path.exists("xxx.mail"):
            os.unlink('xxx.mail')
        c = s.url('=toto/SVC/' + sign('UE-SVC-PRST/C/99//'))
        assert('Answering is no more possible' in c)
        c = s.url('=%s/%s/UE-SVC-PRST' % (abj, ys))
        assert(',C("UNJLEAVE","","' in c)


        s.url('=%s/%s/UE-SVC-PRST2' % (abj, ys)) # Where PRST are recorded
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST2/C/99/PRST/UE-SVC-ABI%20Teacher2%20A%20{}'.format(semester)))
        assert('1 PRST' in c)
        c = s.url('=toto/SVC/' + sign('UE-SVC-PRST2/C/99//'))
        assert('Answering is no more possible' in c)
        c = s.url('=%s/%s/UE-SVC-PRST2' % (abj, ys))
        assert(',C("UNJLEAVE","","' not in c)

        s.url('=%s/%s/UE-SVC-PRST3' % (abj, ys)) # Where PRST are recorded
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST3/C/99/PRST/UE-SVC-ABI%20Teacher3%20A%20{}'.format(semester)))
        assert('1 PRST' in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC-PRST3/C/99/PRST/UE-SVC-ABI%20Teacher3%20A%20{}'.format(semester)))
        assert('2 PRST' in c)
        c = s.url('=toto/SVC/' + sign('UE-SVC-PRST3/C/99//'))
        assert('Answering is no more possible' in c)
        c = s.url('=%s/%s/UE-SVC-PRST3' % (abj, ys))
        assert(c.count(',C("UNJLEAVE","","') == 1)

        s.url('=%s/%s/UE-SVC-PRST4' % (abj, ys)) # Where PRST are recorded
        c = s.url('=toto/SVC/' + sign('UE-SVC-PRST4/C/99//'))
        assert('Answering is no more possible' in c)
        c = s.url('=%s/%s/UE-SVC-PRST4' % (abj, ys))
        assert(c.count(',C("UNJLEAVE","","') == 0)

        s.url('=%s/%s/UE-SVC-PRST5' % (abj, ys)) # Where PRST are recorded
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST5/C/99/PRST/UE-SVC-ABI%20Teacher4%20A%20{}'.format(semester)))
        assert('1 PRST' in c)
        assert(_("MSG_QRCode_unexpected") not in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC-PRST5/C/99/PRST/UE-SVC-ABI%20Teacher4%20B%20{}'.format(semester)))
        assert('2 PRST' in c)
        assert(_("MSG_QRCode_unexpected") in c)
        # Multiple times to be sure the error message is flushed in xxx.tomuss.stderr
        for i in range(30):
            c = s.url('=toto/SVC/' + sign('UE-SVC-PRST5/C/99//'))
            assert('Answering is no more possible' in c)
            c = utilities.read_file('xxx.tomuss.stderr', encoding='bytes')
            if b'<b>get_comment</b>' in c:
                break
        else:
            raise ValueError("Unbreaked for")
        c = s.url('=%s/%s/UE-SVC-PRST5' % (abj, ys))
        assert(c.count(',C("UNJLEAVE","","') == 1)

        yp = str(uyear) + '/P'
        c = s.url('=%s/%s/UE-SVC-PRST6' % (root, yp)) # Where PRST are recorded
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/%s/UE-SVC-PRST6/%s/0/table_attr_masters/%s' % (root, yp, page, root))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-PRST6/%s/1/column_attr_comment/6/{"student_rule":".*","pc_abi":"0.4001","abi_content":"{}"}' % (root, yp, page))
        assert(c == ok_png)
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST6/C/99/PRST/UE-SVC-ABI%20Teacher5%20A%20{}%2042'.format(semester), yp))
        assert('1 PRST' in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC-PRST6/C/99/PRST/UE-SVC-ABI%20Teacher5%20B%20{}%2042'.format(semester), yp))
        assert('2 PRST' in c)
        c = s.url('=toto/SVC/' + sign('UE-SVC-PRST6/C/99//', yp))
        assert(len(os.listdir("TMP/SVC")) == 0)
        assert('Answering is no more possible' in c)
        c = utilities.read_file('xxx.tomuss.stderr', encoding='bytes')
        assert(b'QRCode BUG in' in c)
        c = s.url('=%s/%s/UE-SVC-PRST6' % (abj, yp))
        assert(c.count(',C("UNJLEAVE","","') == 1)

        # Mails are sent to student only for «P» semesters
        for i in range(100):
            if os.path.exists('xxx.mail'):
                c = utilities.read_file('xxx.mail')
                if ('UNJLEAVE UE-SVC-ABI Teacher1' not in c
                        and 'UNJLEAVE UE-SVC-ABI Teacher3' not in c
                        and 'UNJLEAVE UE-SVC-ABI Teacher4' not in c
                        and 'UNJLEAVE UE-SVC-ABI Teacher5' in c
                   ):
                    break
            time.sleep(0.1)
        else:
            bug

        s.url('=%s/%s/UE-SVC-PRST7' % (abj, ys)) # Where PRST are recorded
        # Bad UE name
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST7/C/99/PRST/UE-SVCABI%20Teacher5%20A%20{}'.format(semester)))
        assert('1 PRST' in c)
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST7/C/99//'))
        assert(c.count(',C("UNJLEAVE","","') == 0)


        # The student go to the bad group
        s.url('=%s/%s/UE-SVC-PRST8' % (abj, ys)) # Where PRST are recorded
        # 18000000 18000001 group A
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST8/C/99/PRST/UE-SVC-ABI%20Teacher1%20A%20{}'.format(semester)))
        assert('1 PRST' in c)
        c = s.url('=18000001/SVC/' + sign('UE-SVC-PRST8/C/99/PRST/UE-SVC-ABI%20Teacher1%20A%20{}'.format(semester)))
        assert('2 PRST' in c)
        # 18000002 group B
        c = s.url('=18000002/SVC/' + sign('UE-SVC-PRST8/C/98/PRST/UE-SVC-ABI%20Teacher2%20B%20{}'.format(semester)))
        assert('1 PRST' in c)
        # group A close session
        c = s.url('=18000000/SVC/' + sign('UE-SVC-PRST8/C/99//'))
        assert('Answering is no more possible' in c)
        assert('2 PRST' in c)
        # PRST is not erase
        c = s.url('=%s/%s/UE-SVC-PRST8' % (abj, ys))
        assert('C("PRST","18000002"' in c)

        # Reporting
        c = s.url('=%s/P_reporting/%s' % (abj, uyear))
        assert("window.open" in c)

        c = s.url('=%s/P_reporting/%s//%s-08-01/%s-08-01/////0' % (abj, uyear, uyear, uyear+2))
        assert('TITLE="UE-SVC-ABI Teacher5 A Printemps 42#99 (0min)">' in c)

        c = s.url('=%s/P_reporting/%s/X///////0' % (abj, uyear))
        assert('</table>' in c)
        c = s.url('=%s/P_reporting/%s/etape-18000002/%s-08-01/%s-08-01/.*////0' % (abj, uyear, uyear, uyear+2))
        assert(c.count('<table') == 3)

        c = s.url('=%s/P_reporting/%s/etape-18000002/%s-11-01/%s-08-01/////1/true' % (abj, uyear, uyear, uyear+2))
        assert(c.count('<table') == 3)

        c = s.url('=%s/P_reporting/%s/etape-18000002' % (abj, uyear))
        assert('value="etape-18000002">' in c)
        assert('value="{}">'.format(time.strftime('%d/%m/%Y')) in c)

        c = s.url('=%s/P_reporting/%s//%s-08-01/%s-08-01//true///1' % (abj, uyear, uyear, uyear+2))
        assert('<h1>18000001 18000001Surname 18000001Firstname</h1>' in c)
        assert('<td>42<td>PRST<td class="ue">SVC-ABI<br>?<td class="teacher">Teacher5Firstname<br>teacher5Surname</tr>' in c)

        c = s.url('=%s/P_reporting/%s//%s-08-01/%s-08-01//true///0/true' % (abj, uyear, uyear, uyear+2))
        assert('<h1>18000001 18000001Surname 18000001Firstname</h1>' in c)
        assert('<td>PRST<td class="ue">SVC-ABI<br>?</tr>' in c)

        c = s.url('=%s/%s/UE-SVC-ABI/attendance' % (abj, ys))
        assert('''lines = {"0": [C("18000000"),C("18000000Surname"),C("18000000Firstname"),C("0"),C("PRST","","","Teacher5")],
"1": [C("18000001"),C("18000001Surname"),C("18000001Firstname"),C("0"),C("PRST","","","Teacher5")],
"2": [C("18000002"),C("18000002Surname"),C("18000002Firstname"),C("1"),C("UNJLEAVE","","","Teacher5")]} ;''' in c)

    if do('SVC-ABI2'):
        # Test group A+B
        s.url('=%s/%s/UE-SVC-ABI2' % (abj, ys))
        c = s.url('=%s/%s/UE-SVC-ABI2/1/0/cell_change/0_0/L1/18000000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/1/cell_change/0_0/L2/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/2/cell_change/0_0/L3/18000002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/3/cell_change/0_0/L4/18000003' % (abj, ys))
        assert(c == ok_png)
        time.sleep(0.1)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/4/cell_change/0_3/L1/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/5/cell_change/0_3/L2/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/6/cell_change/0_3/L3/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/7/cell_change/0_3/L4/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/8/cell_change/0_0/L5/18000004' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/9/cell_change/0_0/L6/18000005' % (abj, ys))
        assert(c == ok_png)
        time.sleep(0.1)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/10/cell_change/0_3/L5/A42' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/1/11/cell_change/0_3/L6/B42' % (abj, ys))
        assert(c == ok_png)

        yp = str(uyear) + '/P'
        c = s.url('=%s/%s/UE-SVC-ABI2' % (abj, yp)) # Where PRST are recorded
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/%s/UE-SVC-ABI2/%s/0/table_attr_masters/%s' % (abj, yp, page, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-ABI2/%s/1/column_attr_comment/6/{"student_rule":".*","pc_abi":"0.9","abi_content":"{}"}' % (abj, yp, page))
        assert(c == ok_png)
        c = s.url('=18000000/SVC/' + sign('UE-SVC-ABI2/C/99/PRST/UE-SVC-ABI2%20Teacher1%20A+B%20{}'.format(semester), yp))
        assert('1 PRST' in c)
        c = s.url('=18000002/SVC/' + sign('UE-SVC-ABI2/C/99/PRST/UE-SVC-ABI2%20Teacher1%20A+B%20{}'.format(semester), yp))
        assert('2 PRST' in c)
        c = s.url('=18000004/SVC/' + sign('UE-SVC-ABI2/C/99/PRST/UE-SVC-ABI2%20Teacher1%20A+B%20{}'.format(semester), yp))
        assert('2 PRST' in c)
        assert('1 PRST ∉ UE-SVC-ABI2(A+B)' in c)

        c = s.url('=toto/SVC/' + sign('UE-SVC-ABI2/C/99//', yp))
        assert('Answering is no more possible' in c)
        c = replace_dates(s.url('=%s/%s/UE-SVC-ABI2' % (abj, yp)))
        assert(c.count(',C("UNJLEAVE","","') == 2)

    if do('SVC-expire'):
        s.url('=%s/%s/UE-SVCex' % (abj, ys))
        c = s.url('=%s/%s/UE-SVCex/1/0/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=18000000/SVC/' + sign('UE-SVCex/C/222/PRST/UE-SVCex%20Teacher3%20%20{}'.format(semester), ys))
        assert('1 PRST' in c)
        assert(not os.path.exists('TMP/SVC/%s#Printemps#UE-SVCex#C#222' % year))

        s.url('=%s/%s/UE-SVCpex' % (abj, ys))
        c = s.url('=%s/%s/UE-SVCpex/1/0/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        yp = str(uyear) + '/P'
        s.url('=%s/%s/UE-SVCpex' % (abj, yp))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVCpex/2/0/table_attr_masters/%s' % (abj, yp, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVCpex/2/1/column_attr_comment/6/{"student_rule":".*","pc_abi":"1","abi_content":"{}"}' % (abj, yp))
        assert(c == ok_png)

        c = s.url('=18000000/SVC/' + sign('UE-SVCpex/C/223/PRST/UE-SVCpex%20Teacher4%20%20{}'.format(semester), yp))
        assert('1 PRST' in c)
        token = 'TMP/SVC/%s#P#UE-SVCpex#C#0#223' % uyear
        assert(os.path.exists(token))

        if os.path.exists("xxx.mail"):
            os.unlink('xxx.mail')
        c = s.url('=%s/expire_qrcode' % root)
        assert(not os.path.exists(token))
        retry(lambda: os.path.exists('xxx.mail') and utilities.read_file('xxx.mail'),
              lambda c: 'Subject: UNJLEAVE UE-SVCpex Teacher4' in str(c))

    if do('SVC-grp'):
        s.url('=%s/%s/UE-SVCgrp' % (abj, ys))
        c = s.url('=%s/%s/UE-SVCgrp/1/0/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVCgrp/1/1/column_attr_title/Cgrp/group' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVCgrp/1/2/cell_change/Cgrp/L1/Grp%%3AA' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVCgrp/1/3/column_attr_title/CPrst/present' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVCgrp/1/4/column_attr_type/CPrst/Prst' % (abj, ys))
        assert(c == ok_png)

        c = s.url('=10800001/SVC/' + sign('UE-SVCgrp/7/222/PRST/Teacher3%20group==Grp:B'.format(semester)))
        assert('<div style="background: #F00">group Grp:B≠Grp:A</div>' in c)

        c = s.url('=10800001/SVC/' + sign('UE-SVCgrp/7/222/PRST/Teacher3%20group==Grp:A'.format(semester)))
        assert('<b>1 PRST</b>' in c)

    if do('SVC-abj'):
        # Create student list
        s.url('=%s/%s/UE-SVC-abj' % (abj, ys))
        c = s.url('=%s/%s/UE-SVC-abj/1/0/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-abj/1/1/cell_change/0_0/L2/10800002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-abj/1/2/cell_change/0_0/L3/10800003' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-abj/1/3/cell_change/0_0/L4/10800004' % (abj, ys))
        assert(c == ok_png)

        # Create presence table
        yp = str(uyear) + '/P'
        s.url('=%s/%s/UE-SVC-abj' % (abj, yp))
        c = s.url('=%s/%s/UE-SVC-abj/2/0/table_attr_masters/%s' % (abj, yp, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-SVC-abj/2/1/column_attr_comment/6/{"student_rule":".*","pc_abi":"0.9","abi_content":"{}"}' % (abj, yp))
        assert(c == ok_png)

        # Create ABJ
        c = s.url('=' + abj + '/%s/abj/0/10800002/addabj/%s/A/%s/A/com4' % (
            ys, abj_date_previous, abj_date_next))
        assert('display_abjs' in c)
        c = s.url('=' + abj + '/%s/abj/0/10800003/addabj/%s/M/%s/M/com4' % (
            ys, abj_date_old, abj_date_previous))
        assert('display_abjs' in c)
        c = s.url('=' + abj + '/%s/abj/0/10800004/addabj/%s/M/%s/A/com4' % (
            ys, abj_date_next, abj_date_next))
        assert('display_abjs' in c)

        # Check ABJ
        c = s.url('=10800001/SVC/' + sign('UE-SVC-abj/C/223/PRST/UE-SVC-abj%20Teacher4@nowhere%20%20{}%2090'.format(semester), yp))
        assert('1 PRST' in c)
        if os.path.exists("xxx.mail"):
            os.unlink('xxx.mail')
        c = s.url('=10800001/SVC/' + sign('UE-SVC-abj/C/223//', yp))
        assert('1 PRST' in c)
        c = s.url('=%s/%s/UE-SVC-abj' % (abj, yp))
        assert(c.count(configuration.abi) == 4)
        assert(c.count(configuration.abj) == 1)
        assert(c.count(configuration.pre) == 1)
        remove_abjs()
        retry(lambda: os.path.exists('xxx.mail') and utilities.read_file('xxx.mail'),
              lambda c:
                'Subject: UNJLEAVE UE-SVC-abj\n' in str(c)
                and 'UNJLEAVE 10800004Surname 10800004Firstname' in str(c)
                and 'JUSLEAVE 10800002Surname 10800002Firstname' in str(c)
             )

    if do('import4'):
        s.url('=%s/%s/UE-import4' % (abj, ys))
        c = s.url('=%s/%s/UE-import4/1/0/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-import4/1/1/cell_change/0_0/L2/10800002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-import4/1/2/column_attr_cell_writable/C0/[ID]=10800001' % (abj, ys))
        assert(c == ok_png)

        utilities.mkpath('TMP/0')
        utilities.write_file('TMP/0/xxx.csv', '10800001\t3.45\n10800002\t5.67\n')
        u = s.get_url('files/0/xxx.csv#2').replace('/', '$2F').replace('#', '$23')
        # Import a CSV file
        c = s.url('=%s/%s/UE-import4/1/3/column_attr_url_import/C0/%s' % (abj, ys, u))
        assert(c == ok_png)

        c = replace_dates(s.url('=%s/%s/UE-import4' % (abj, ys)))
        assert('C(3.45,"*","DATE")' in c)
        assert('5.67' not in c)

        # A lingering thread overwrite with the 3.45 value in some time...
        time.sleep(1)

        c = s.url('=%s/%s/UE-import4/1/4/column_attr_url_import/C0/UE-import4$2FID' % (abj, ys))
        assert(c == ok_png)
        retry(lambda: s.url('=%s/%s/UE-import4' % (abj, ys)),
              lambda c: c.count('10800001') == 6 and c.count('10800002') == 4)

    if do('popup_on_red_line'):
        s.url('=' + abj + '/%s/UE-porl' % ys)
        c = s.url('='+abj+'/%s/UE-porl/1/0/cell_change/0_0/L0/11111111' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-porl/1/1/table_attr_popup_on_red_line/0' % ys)
        assert(c == ok_png)
        c = s.url('='+abj+'/%s/UE-porl/1/2/update_content/remove' % ys)
        assert(c == ok_png)
        c = replace_dates(s.url('=' + abj + '/%s/UE-porl' % ys))
        # Student not removed
        assert('P("L0",[C("11111111","tt.master","DATE"),C("11111111Firstname","*","DATE"),C("11111111Surname","*","DATE"),C("","*","DATE"),C("","*","DATE"),C("ok","*","DATE")]);' in c)

    if do('export_xls'):
        create_u2()
        c = s.url('=%s/%s/UE-INF20UE2' % (root, ys))
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/%s/UE-INF20UE2/%s/0/column_attr_red/C1/=' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/1/column_attr_green/C1/!=' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/2/column_attr_redtext/C1/x' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/3/column_attr_greentext/C1/>0' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/4/comment_change/C1/L1/X' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/5/table_attr_comment/a_table_comment' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/%s/6/column_attr_type/C2/URL' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-INF20UE2/export_xls/1,-2/wrap/toto.xlsx' % (root, ys), read_bytes=True)
        assert(c.startswith(b'PK'))
        assert(b'worksheets' in c)
        assert(7000 < len(c) < 8000)

        c = s.url('=%s/%s/NoTable/export_xls/1,-2/toto.xlsx' % (root, ys), read_bytes=True)
        assert(c == b'')

        s.url('=%s/%s/UE-privatexls' % (root, ys))
        c = s.url('=%s/%s/UE-privatexls/1/0/table_attr_private/1' % (root, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-privatexls/export_xls/1,-2/toto.xlsx' % (root, ys), read_bytes=True)
        assert(c.startswith(b'PK'))
        c = s.url('=%s/%s/UE-privatexls/export_xls/1,-2/toto.xlsx' % (abj, ys), read_bytes=True)
        assert(c == b'PRIVATE')

    if do('column_update'):
        # Test commit : 00a96b9be7c6ed704debd7da706f6e32ddb23eb9
        c = s.url('=%s/%s/Teachers/colupdate' % (abj, uyear))
        assert('initialize()' in c)
        c = s.url('=%s/%s/Teachers/colupdate/2/0/table_attr_masters/%s' % (abj, uyear, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/colupdate/2/1/column_attr_repetition/10Mail/2' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/Teachers/colupdate/page_unload' % (root, uyear))
        assert(_("MSG_page_unload_after") in c)
        # Only for coverage
        s.url('=%s/%s/Teachers/colupdate' % (abj, uyear))

    if do('analyser'):
        c = s.url('=%s/%s/UE-analyser' % (abj, ys))
        assert('initialize()' in c)
        c = s.url('=%s/%s/UE-analyser/1/0/column_attr_type/C1/Analyser' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/1/1/column_attr_columns/C1/ID' % (abj, ys))
        assert(c == ok_png)
        grade_script = '''
comment = STUDENTS.add_competence('Competence', 10)
done = STUDENTS.add_competence('Done', 1, True)
for student in STUDENTS:
    student.grade(done, 1, 'done')
    for cell in student:
        if '0' in cell.new.value:
            student.grade(comment, 1, cell.new.value)
        if cell.new.with_upload():
            student.grade(comment, 1)
'''
        analyser_current_attrs_dict = {
            "script_G_upload": 1,
            "script_G_old_upload": 1,
            "script_G_on_upload": 1,
            "script_U_upload": 1,
            "script_U_old_upload": 1,
            "script_U_on_upload": 1,
            "script_G": grade_script,
            "script_U": 'for student in STUDENTS: student.message_green("GM")'
        }
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')

        c = s.url('=%s/%s/UE-analyser/1/2/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/1/3/cell_change/0_0/L1/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/1/4/cell_change/0_0/L2/18000002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/1/5/column_attr_type/C2/Upload' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/1/6/column_attr_title/C2/upload' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/1/7/column_attr_columns/C1/ID$20upload' % (abj, ys))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser/analyser_grade/6' % (abj, ys),
                   fields=(("line_ids", "L1 L2"),))
        assert(c == 'Students sent to grading: 1/2 2/2<p> 18000001 18000002')

        c = s.url('=%s/%s/UE-analyser' % (abj, ys))
        assert(r'"{\"c000\": \"1/10 18000002\", \"c001\": \"1/1 done\"}"' in c)
        assert(r'"comment": "[\"c000 1Q10 Competence\", \"c001 1B1 Done\"]"' in c)

        c = s.post('=%s/%s/UE-analyser/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), )
               )
        assert('Uploaded file size in bytes: 16' in c)
        assert('>GM<' in c)

        retry(lambda: s.url('=%s/%s/UE-analyser' % (abj, ys)),
              lambda c: '2/10 18000001' in c)

        c = s.url('=%s/%s/UE-analyser/1/8/table_attr_masters/%s' % (abj, ys, root))
        assert(c == ok_png)

        c = s.url('=%s/%s/UE-analyser' % (root, ys))
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/%s/UE-analyser/%s/0/cell_change/C1/L1/12.34' % (root, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser/%s/1/comment_change/C1/L1/' % (root, ys, page))
        assert(c == ok_png)

        c = s.post('=%s/%s/UE-analyser/analyser_grade/6' % (abj, ys),
                   fields=(("line_ids", "L1 L2"),))
        assert('Students sent to grading: 1/2 2/2<p> <span style="background: #F00">18000001</span> 18000002'
            in c)

        c = s.url('=%s/%s/UE-analyser/analyser_zip/6' % (abj, ys), read_bytes=True)
        c = io.BytesIO(c)
        to_analyse = zipfile.ZipFile(c, mode="r")
        assert(sorted([f.filename for f in to_analyse.filelist])
            == ['1', '2', 'YourScript.py', 'analyser.py', 'students.csv'])

    if do('analyser2'):
        c = s.url('=%s/%s/UE-analyser2' % (abj, ys))
        assert('initialize()' in c)
        c = s.url('=%s/%s/UE-analyser2/1/0/column_attr_type/C1/Analyser' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/1/column_attr_columns/C1/ID' % (abj, ys))
        assert(c == ok_png)
        analyser_current_attrs_dict = {
            "script_G_upload": 1,
            "script_G_old_upload": 1,
            "script_G_on_upload": 1,
            "script_U_upload": 1,
            "script_U_old_upload": 1,
            "script_U_on_upload": 1,
            "script_G": 'bad script1',
            "script_U": 'for student in STUDENTS: student.message_red("RM")'
        }
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')

        c = s.url('=%s/%s/UE-analyser2/1/2/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/3/cell_change/0_0/L1/18000001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/4/cell_change/0_0/L2/18000002' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/5/column_attr_type/C2/Upload' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/6/column_attr_title/C2/upload' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/7/column_attr_columns/C1/ID$20upload' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/8/cell_change/0_0/L3/' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/1/9/column_attr_type/C3/Analyser' % (abj, ys))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser2/analyser_grade/6' % (abj, ys),
                   fields=(("line_ids", "L1 L2 L3 L4"),))
        assert('Students sent to grading: 1/4 2/4' in c)
        assert('3/4' not in c)
        assert('4/4' not in c)
        assert('<pre>Traceback' in c)
        assert('bad script' in c)

        c = s.post('=%s/%s/UE-analyser2/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content"), )
               )
        assert('Uploaded file size in bytes: 16' in c)
        assert('>RM<' in c)

        analyser_current_attrs_dict['script_U'] = ''
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-analyser2/1/10/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser2/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content2"), )
               )
        assert('successfully' in c)


        analyser_current_attrs_dict['script_U'] = 'bad script2'
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-analyser2/1/11/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser2/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content3"), )
               )
        assert('successfully' in c)
        assert('THERE IS A PROBLEM, CONTACT YOUR TEACHER' in c)

        analyser_current_attrs_dict["script_G_on_upload"] = 0
        analyser_current_attrs_dict['script_U'] = ''
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-analyser2/1/12/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser2/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content4"), )
               )
        assert('successfully' in c)
        assert('THERE IS A PROBLEM, CONTACT YOUR TEACHER' not in c)

        analyser_current_attrs_dict["script_G_on_upload"] = 1
        analyser_current_attrs_dict["script_G"] = ''
        analyser_current_attrs_dict["script_U_on_upload"] = 0
        analyser_current_attrs_dict['script_U'] = 'a = 1'
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-analyser2/1/13/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser2/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content5"), )
               )
        assert('successfully' in c)
        assert('THERE IS A PROBLEM, CONTACT YOUR TEACHER' not in c)


        analyser_current_attrs_dict['script_U'] = '''
a = ''
for i in range(20):
    a += 100000000 * str(i)
for s in STUDENTS:
       s.message_red("Impossible")
'''
        analyser_current_attrs = json.dumps(analyser_current_attrs_dict
            ).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')
        c = s.url('=%s/%s/UE-analyser2/1/14/column_attr_analyser_config/C1/%s'
            % (abj, ys, analyser_current_attrs))
        assert(c == ok_png)
        c = s.post('=%s/%s/UE-analyser2/upload_post/C2/L1' % (abj, ys),
                   fields = ( ("filename", "foo.txt"), ),
                   files = ( ("data", "FOO.TXT", b"the file content6"), )
               )
        assert('THERE IS A PROBLEM' in c)
        assert("Impossible" not in c)

        c = s.url('=%s/%s/UE-analyser2/1/15/table_attr_masters/%s' % (abj, ys, abj))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-analyser2/delete_this_table' % (abj, ys))
        assert('Deletion done' in c)

    if do('get_pair'):
        c = s.url('=%s/%s/UE-get_pair/get_pair/6/0/42' % (abj, ys), read_bytes=True)
        assert(len(c) == 302)
        c = s.url('=%s/%s/UE-get_pair/get_pair/6/1/42' % (abj, ys), read_bytes=True)
        assert(len(c) == 302)
        c = s.url('=%s/%s/UE-get_pair/get_pair/6/1/42/colTitle==Value' % (abj, ys), read_bytes=True)
        assert(len(c) == 369)

    if do('reporting_xls'):
        s.url('=%s/%s/P/report' % (abj, uyear)) # Where PRST are recorded
        c = s.url('=18051234/SVC/' + sign('report/C/42/PRST/UE-Code%20TeacherName%20TD%20{}%2075'.format(semester),
                                          '%s/P' % uyear))
        assert('1 PRST' in c)
        c = s.url('=%s/%s/P/report/2/0/column_attr_title/10/%s-12-31_10:00' % (abj, uyear, uyear+1))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/report/2/1/cell_change/10/0_0/UNJLEAVE' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/report/2/2/comment_change/10/0_0/UE%%20Login%%20TD%%20Printemps%%2060060$2343' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/report/2/3/column_attr_title/11/%s-12-31_20:00' % (abj, uyear, uyear+1))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/report/2/4/cell_change/11/0_0/JUSLEAVE' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/report/2/5/comment_change/11/0_0/UE%%20Login%%20TD%%20Printemps%%20120120$2343' % (abj, uyear))
        assert(c == ok_png)

        def checkxls(fct, search_day=True):
            utilities.write_file('xxx.zip', c, encoding='bytes')
            zf = zipfile.ZipFile(open('xxx.zip', 'rb+'), mode="r")
            values = []
            with zf.open('xl/sharedStrings.xml', "r") as f:
                for value in f.read().decode('utf-8').split('<t>'):
                    values.append(value.split('</t>')[0])
            # values.pop(0) # Index starts at 1 not 0
            if search_day:
                assert(('%s-12-31' % (uyear+1)) in values)
            student_id = f' t="s"><v>{values.index("18051234")}</v>'
            for filename in zf.namelist():
                if filename.startswith('xl/worksheets'):
                    with zf.open(filename, "r") as f:
                        content = f.read().decode('utf-8')
                        for row in content.split('<row '):
                            if student_id in row:
                                if fct(row):
                                    return # Found it !
                                print(row)
            raise ValueError("Can't find value")

        # Reporting by nbr
        c = s.url('=%s/P_reporting_xls/%s//%s-01-01/%s-01-01//0/' % (abj, uyear, uyear, uyear+2), read_bytes=True)
        checkxls(lambda v: ' s="1"><v>1</v>' in v and ' s="3"><v>1</v>' in v)

        # Reporting by hours
        c = s.url('=%s/P_reporting_xls/%s//%s-01-01/%s-01-01//1/' % (abj, uyear, uyear, uyear+2), read_bytes=True)
        checkxls(lambda v: ' s="1"><v>1.25</v>' in v  and  ' s="3"><v>1001</v>' in v  and  ' s="2"><v>2002</v>' in v)

        # To do coverage 20..→....
        c = s.url('=%s/P_reporting_xls/%s//20//.*/1/' % (abj, uyear), read_bytes=True)
        checkxls(lambda v: ' s="1"><v>1.25</v>' in v  and  ' s="3"><v>1001</v>' in v  and  ' s="2"><v>2002</v>' in v)

        # Empty
        c = s.url('=%s/P_reporting_xls/%s//30///1/' % (abj, uyear), read_bytes=True)
        checkxls(lambda v: ' s="1"><v>1.25</v>' not in v  and  ' s="3"><v>1001</v>' not in v, search_day=False)

    if do('referent_get_table'):
        create_u2()
        c = s.url('=%s/referent_set/%s/UE-INF20UE2' % (root, root))
        assert('10800000' in c and '10800011' in c)
        c = s.url('=%s/referent_set/%s/UE-INF20UE2:A:2' % (root, root))
        assert('10800000' not in c)
        c = s.url('=%s/referent_set/%s/UE-INF20UE2:B' % (root, root))
        assert('10800000' not in c)
        c = s.url('=%s/referent_set/%s/UE-INF20UE2:A' % (root, root))
        assert('10800000' in c)

    if do('table_access'):
        create_u2()
        c = s.url('=p0800000/%s/UE-INF20UE2' % ys)
        assert('<script>window.location =' in c)

        c = s.url('=p0800000/%s/UE-INF20UE2/0/0' % ys)
        assert(c == '') # Hacker direct access
        c = s.url('=p0800000/%s/UE-INF20UE2/0/end_of_load' % ys)
        assert(c == '') # Hacker direct access
        try:
            c = s.url('=p0800000/%s/UE-INF20UE2/0/key_history/x' % ys,
                      stop_if_error=False, display_log_if_error=False)
            raise ValueError('This page does not generate an answer')
        except http.client.RemoteDisconnected:
            pass
        try:
            c = s.post('=p0800000/gui_record', fields = (('table', "%s/UE-INF20UE2"%ys),))
        except http.client.RemoteDisconnected:
            pass

        actions = [0, year, semester, 'UE-INF20UE2', 0, []]
        c = s.post('pageaction?ticket=p0800000', fields = (('content', json.dumps(actions)),), binary=True)
        assert(c == bug_png_bin)
        actions[5] = ['x']
        c = s.post('pageaction?ticket=p0800000', fields = (('content', json.dumps(actions)),), binary=True)
        assert(c == b'') # Hacker direct access

        # Open the table to the student
        c = s.url('=' + root + '/0/Dossiers/config_table')
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('='+root+'/0/Dossiers/config_table/%s/0/cell_change/0_2/exceptions/{"p0800000":".*UE-INF20UE2"}' % page)
        assert(c == ok_png)

        c = s.url('=p0800000/%s/UE-INF20UE2' % ys)
        assert('runlog' in c)
        page = c.split('page_id = "')[1].split('"')[0]

        # Not the other students
        c = s.url('=p0800001/%s/UE-INF20UE2' % ys)
        assert('<script>window.location =' in c)

        # Modify a value
        c = s.url('=p0800000/%s/UE-INF20UE2/%s/1/cell_change/col_0/StudentLine/1' % (ys, page))
        assert(c == ok_png)

        # Answer page
        c = s.url('=p0800000/%s/UE-INF20UE2/%s/1' % (ys, page))
        assert('<script>set_updating(0);</script>' in c)

        c = s.url('=p0800000/%s/UE-INF20UE2/%s/end_of_load' % (ys, page))
        assert(c == '')

    if do('alert'):
        alert = [
            {'filter': '[ID]=10800001 9.99',
             'to': 'MAIL@xxx.fr [B]',
             'cc': 'MAIL@yyy.fr @[B]',
             'subject': 'Value=[A] Author=@[A] Comment=#[B] ?[B] [undefined]',
             'body': 'VALUE=[A] AUTHOR=@[A] COMMENT=#[B]',
            },
            {'cc': 'empty@alert.fr', 'subject': 'plain text', 'body': 'unfinished [ here'},
            {'filter': '=NEVER'}, # For coverage
            ]

        s.url('=%s/%s/UE-alert' % (abj, ys))
        c = s.url('=%s/%s/UE-alert/1/0/column_attr_title/A/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-alert/1/1/column_attr_alert/A/%s' % (abj, ys,
            json.dumps(alert).replace(' ', '$20').replace('/', '$2F').replace('#', '$23')))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-alert/1/2/cell_change/0_0/L1/10800001' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-alert/1/3/column_attr_title/B/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-alert/1/4/column_attr_type/B/Text' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-alert/1/5/comment_change/B/L1/mail@zzz.fr' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-alert/1/6/cell_change/B/L1/mail@yyy.fr' % (abj, ys))
        assert(c == ok_png)
        utilities.unlink_safe('xxx.mail', do_backup=False)
        c = s.url('=%s/%s/UE-alert/1/7/cell_change/A/L1/9.99' % (abj, ys))
        assert(c == ok_png)
        retry(lambda: utilities.read_file('xxx.mail') if os.path.exists('xxx.mail') else '',
              lambda c:
              'Subject: Value=9.99 Author=tt.master Comment=mail@zzz.fr' in c
              and 'To: marianne.tery@univ-lyon1.fr' in c
              and 'From: marianne.tery@univ-lyon1.fr' in c
              and 'VALUE=9.99 AUTHOR=tt.master COMMENT=mail@zzz.fr' in c
              and 'CC: MAIL@yyy.fr' in c
              and 'CC: mail@mail.fr' in c
              and 'CC: empty@alert.fr' in c
              )

    if do('star'):
        s.url('=%s/%s/UE-star' % (abj, ys))
        c = s.url('=%s/%s/UE-star/1/0/cell_change/0_0/L1/10804242' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/1/column_attr_title/A/A' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/2/column_attr_title/B/B' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/3/cell_change/A/L1/1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/4/cell_change/B/L1/5' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/5/column_attr_type/C/Moy' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/6/column_attr_columns/C/*Note' % (abj, ys))
        assert(c == ok_png)
        ss.start()
        c = ss.url("=10804242")
        c = replace_dates(c)
        assert('[1.0, "tt.master", "DATE"], [5.0, "tt.master", "DATE"], [3.0, "*"]]' in c)
        c = s.url('=%s/%s/UE-star/1/7/column_attr_type/C/Nmbr' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-star/1/8/column_attr_columns/C/*' % (abj, ys))
        assert(c == ok_png)
        c = ss.url("=10804242")
        c = replace_dates(c)
        assert('[1.0, "tt.master", "DATE"], [5.0, "tt.master", "DATE"], [8, "*", "", ""]]' in c)

    if do('update_abj'):
        create_u2()
        c = s.url('=%s/%s/abj/-1/10800000/addabj/30/01/%d/M/30/01/%d/A/Motif' % (
            abj, ys, year, year))
        assert("window.parent.display_abjs([['30/1/%dM','30/1/%dA','%s',\"Motif\",[" % (year, year, abj) in c)

    if do('animaux'):
        c = s.url('=%s/%s/demo_animaux' % (abj, ys))
        assert('LEZARD' in c)
        c = s.url('=%s/%s/demo_animaux-99' % (abj, ys))
        assert('k099' in c)

    if do('qrcode_etape'):
        create_u2()
        c = s.url('=%s/%s/UE-INF20UE2/qrcode_new/-200101011100180/grpseq/[["A","1","Grp","Seq"],["B","2"]]' % (abj, ys))
        assert('code = "UE-INF20UE2"' in c)
        assert('column = "2001-01-01_11:00"' in c)
        assert('/P/etape-10800000' in c)
        qrcode = c.split('var qrcode = "')[1].split('"')[0]
        # Teacher set PRST
        c = s.url('=%s/qrcode_update/%s/set/10800000/PRST/42' % (abj, qrcode))
        assert(c == ok_png_bin)
        # PRST in the P table
        c = s.url('=%s/%s/P/etape-10800000' % (abj, uyear))
        assert('"title": "2001-01-01_11:00"' in c)
        assert('"UE-INF20UE2 tt.master Seq1GrpA+2B Printemps 180#1"' in c)
        assert('C("PRST","%s",' % abj in c)
        # Student set PRST
        c = s.url('=%s/qrcode_update/%s/start/42' % (abj, qrcode));
        assert(c == ok_png_bin)

        c = s.url('=10800000/qrcode_check/%s' % qrcode)
        assert('background:#0F0;' in c)
        assert(f'<br>{uyear}<br>P<br>etape-10800000<br>2001-01-01_11:00' in c)

        # PRST in the past
        c = s.url('=%s/%s/UE-INF20UE2/qrcode_new/-200101011000180/grpseq/[["A","1"],["B","2"]]' % (abj, ys))
        assert('code = "UE-INF20UE2"' in c)
        assert('column = "2001-01-01_10:00"' in c)
        assert('/P/etape-10800000' in c)
        qrcode = c.split('var qrcode = "')[1].split('"')[0]
        # Teacher set PRST
        c = s.url('=%s/qrcode_update/%s/set/10800000/PRST/42' % (abj, qrcode))
        assert(c == ok_png_bin)
        c = s.url('=%s/%s/P/etape-10800000' % (abj, uyear))
        assert('"title": "2001-01-01_10:00"' in c)
        assert('"UE-INF20UE2 tt.master 1A+2B Printemps 180#2"' in c)
        assert(c.count('C("PRST","%s",' % abj) == 2)

        # Check report
        c = s.url('=%s/P_reporting/%s//2000-01-01/2002-08-01/////0' % (abj, uyear))
        assert('<span class="hour">10:00</span><th><span class="hour">11:00</span>' in c)
        assert('<tr><td>2001-01-01_10:00<td>PRST<td>UE-INF20UE2    <td>tt.master<td>0</tr>' in c)
        assert('<tr><td>2001-01-01_11:00<td>PRST<td>UE-INF20UE2    <td>tt.master<td>0</tr>' in c)

    if do('abi2abj'):
        abi_year = year
        abi_month = 5
        abi_day = 7
        abi_P = '%s-%02d-%02d_11:00' % (abi_year, abi_month, abi_day)

        c = s.url('=%s/%s/P/abi2abj' % (abj, uyear))
        c = s.url('=%s/%s/P/abi2abj/2/0/cell_change/0/L1/10810000' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/abi2abj/2/1/column_attr_title/C1/%s' % (abj, uyear, abi_P))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/abi2abj/2/2/cell_change/C1/L1/UNJLEAVE' % (abj, uyear))
        assert(c == ok_png)

        def date_for_abj(year, month, day):
            '''French date'''
            return str(day) + '/' + str(month) + '/' + str(year)

        c = s.url('=' + abj + '/%s/abj/0/10810000/display' % ys)
        assert('display_abjs([])' in c)
        c = s.url('=%s/%s/abj/0/10810000/addabj/%s/M/%s/M/com142' % (
            abj, ys, date_for_abj(abi_year, abi_month, abi_day), date_for_abj(abi_year, abi_month, abi_day)))
        c = s.url('=%s/%s/abj/0/10810000/addabj/%s/A/%s/A/com242' % (
            abj, ys, date_for_abj(abi_year, abi_month, abi_day), date_for_abj(abi_year, abi_month, abi_day)))
        c = s.url('=%s/%s/abj/0/10810000/addabj/%s/A/%s/A/com342' % (
            abj, ys, date_for_abj(abi_year, abi_month, abi_day-1), date_for_abj(abi_year, abi_month, abi_day-1)))
        assert('com142",[["2' in c)
        assert('com242",[]]' in c)
        assert('com342",[]]' in c)

        c = s.url('=%s/abi2abj/10810000/%s/P/abi2abj/XXX' % (abj, uyear))
        assert('BUG' in c)
        c = s.url('=%s/abi2abj/00000000/%s/P/abi2abj/%s' % (abj, uyear, abi_P))
        assert('BUG' in c)
        c = s.url('=%s/abi2abj/10810000/%s/P/XXX/%s' % (abj, uyear, abi_P))
        assert('BUG' in c)
        c = s.url('=%s/abi2abj/10810000/%s/P/abi2abj/%s' % (abj, uyear, abi_P))
        assert('#0F0' in c)
        c = s.url('=%s/abi2abj/10810000/%s/P/abi2abj/%s' % (abj, uyear, abi_P))
        assert('#F00' in c)
        assert('This JL yet exists!')
        # Check with @
        c = s.url('=%s/%s/P/abi2abj/2/3/cell_change/C1/L1/UNJLEAVE@' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/abi2abj/10810000/%s/P/abi2abj/%s' % (abj, uyear, abi_P))
        assert('#0F0' in c)
        c = s.url('=%s/%s/P/abi2abj' % (abj, uyear))
        assert('JUSLEAVE@' in c)

    if do('abi2abj2'):
        s.url('=%s/%s/abi2abj2' % (abj, ys))
        c = s.url('=%s/%s/abi2abj2/1/0/cell_change/0_0/L1/10820000' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/abi2abj2/1/1/column_attr_title/C1/foobar' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/abi2abj2/1/2/column_attr_type/C1/Prst' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/abi2abj2/1/3/cell_change/C1/L1/UNJLEAVE' % (abj, ys))
        assert(c == ok_png)
        for _x in range(20):
            if 'UNJLEAVE' in (utilities.manage_key('LOGINS', '10820000/grades') or ''):
                break
            time.sleep(0.1)
        else:
            bug
        c = s.url('=' + abj + '/%s/abj/0/10820000/display' % ys)
        assert('display_abjs([])' in c)
        c = s.url('=%s/%s/abj/0/10820000/addabj/01/09/%s/M/01/09/%s/M/com1422' % (abj, ys, uyear, uyear+2))
        assert("display_abjs([['1/9/" in c)
        assert('"abi2abj2", "UNJLEAVE"' in c)
        date = c.split('"com1422",[["')[1].split('"')[0]

        c = s.url('=%s/abi2abj/10820000/%s/abi2abj2/%s' % (abj, ys, date))
        assert('OK' in c)
        for _x in range(20):
            grades = utilities.manage_key('LOGINS', '10820000/grades') or ''
            if 'JUSLEAVE' in grades and  'UNJLEAVE' not in grades:
                break
            time.sleep(0.1)
        else:
            bug

        c = s.url('=%s/abi2abj/10820000/%s/abi2abj2/baddate' % (abj, ys))
        assert('cell not found' in c)

    if do('Tables'):
        create_u2()
        c = s.url('=%s/%d/Tables/test' % (abj, uyear))
        c = s.url('=%s/%d/Tables/test/4/0/cell_change/ID/L1/%s$2FUE-INF20UE2' % (abj, uyear, semester))
        assert(c == ok_png)
        c = s.url('=%s/%d/Tables/test' % (abj, uyear))
        assert('C("[\'ue1.master\']"' in c)

        shutil.rmtree('LOGS/BACKTRACES', ignore_errors=True)
        c = s.url('=%s/%d/Tables/test/4/1/cell_change/comment/L1/new_comment' % (abj, uyear))
        assert(c == ok_png)
        for i in range(100):
            backtraces = glob.glob('LOGS/BACKTRACES/*/*')
            if backtraces:
                break
            time.sleep(0.1)
        else:
            bug
        with open(backtraces[0], 'r', encoding='utf-8') as f:
            assert('bad_auth' in f.read())

        c = s.url('=%s/%d/Tables/test/4/2/table_attr_masters/%s$20ue1.master$20ue2.master' % (abj, uyear, abj))
        assert(c == ok_png)

        c = s.url('=ue1.master/%d/Tables/test' % uyear)
        assert('new_comment' in c) # XXX should not be there, but complex to fix
        c = s.url('=ue1.master/%d/Tables/test/6/0/cell_change/comment/L1/NEW_COMMENT' % uyear)
        assert(c == ok_png)
        c = s.url('=ue1.master/%d/Tables/test/6/1/cell_change/rounding/L1/0' % uyear)
        assert(c == ok_png)
        c = s.url('=ue1.master/%d/Tables/test/6/2/cell_change/teachers/L1/["ue1.master","%s"]' % (uyear, abj))
        assert(c == ok_png)

        # Store illegals
        c = s.url('=ue1.master/%d/Tables/test/6/3/cell_change/teachers/L1/bad' % uyear)
        assert(c == ok_png)
        c = s.url('=ue1.master/%d/Tables/test/6/4/cell_change/teachers/L1/{1:5}' % uyear)
        assert(c == ok_png)
        c = s.url('=ue1.master/%d/Tables/test/6/5/cell_change/rounding/L1/3' % uyear)
        assert(c == ok_png)

        # Add bad column name

        time.sleep(1.1)
        c = s.url('=%s/%s/UE-INF20UE2' % (abj, ys))
        assert('rounding:0,' in c)
        assert('comment:"NEW_COMMENT",' in c)
        assert("teachers:['ue1.master', '%s']," % abj in c)

        # Add User column
        c = s.url('=ue1.master/%d/Tables/test/6/6/column_attr_title/C/bad_title' % uyear)
        assert(c == ok_png)

        # Store bad table
        c = s.url('=ue1.master/%d/Tables/test/6/7/cell_change/ID/L2/bad_table' % uyear)
        assert(c == ok_png)

        # For coverage: reading the same value
        c = s.url('=ue1.master/%d/Tables/test/6/8/update_content/column' % uyear)
        assert(c == ok_png)
        # Bad JSON
        c = s.url('=ue1.master/%d/Tables/test/6/9/cell_change/columns/L1/[' % uyear)
        assert(c == ok_png)

        # Add a column
        c = s.url('=ue1.master/%d/Tables/test/6/10/cell_change/columns/L1/[{"title":"C1","red":"RED","green":"X"},{"title":"C2","red":"GREEN"},{"red":"BLUE"},{"title": "C3","visibility_date":"21000101"},{"title":"C4", "BADATTR":"BADATTR"}]' % uyear)
        assert(c == ok_png)
        time.sleep(2.1)
        c = s.url('=%s/%s/UE-INF20UE2' % (abj, ys))
        assert('"red": "RED"' in c)
        assert('"red": "GREEN"' in c)
        assert('"red": "BLUE"' not in c)
        c = s.url('=ue1.master/%d/Tables/test/6/11/cell_change/columns/L1/[{"title":"C1","red":"DER","green":"X"}]' % uyear)
        assert(c == ok_png)
        time.sleep(1.1)
        c = s.url('=%s/%s/UE-INF20UE2' % (abj, ys))
        assert('"red": "DER"' in c)

        # Change user column
        c = s.url('=ue1.master/%d/Tables/test/6/12/cell_change/C/L1/foobar' % uyear)
        assert(c == ok_png)

        # Forbiden change
        c = s.url('=ue2.master/%d/Tables/test' % uyear)
        c = s.url('=ue2.master/%d/Tables/test/7/0/cell_change/columns/L1/[{"title":"C1","red":"DRE"}]' % uyear)
        assert(c == ok_png)
        time.sleep(1.1)
        c = s.url('=%s/%s/UE-INF20UE2' % (abj, ys))
        assert('"red": "DER"' in c) # Unchanged

    if do('competences'):
        utilities.mkpath('DBregtest/Y0/SCompetences')
        with open('DBregtest/Y0/SCompetences/test.py', 'w', encoding='utf-8') as f:
            with open('SCRIPTS/test_gui_competences.py', 'r', encoding='utf-8') as g:
                f.write(g.read())
        c = s.url('=%s/0/Competences/t' % abj)
        page = c.split('page_id = "')[1].split('"')[0]
        c = s.url('=%s/0/Competences/t/%s/0/cell_change/0/L1/o9' % (abj, page))
        assert(c == ok_png)
        c = s.url('=%s/competences' % abj)
        assert(json.loads(c) == {"vocabulary": {"m": ["Informatique"], "c": ["savoir coder"], "f": ["Coder en java"], "b": ["Description creer aplication"], "o": ["Non not\u00e9", "Mauvais", "Bien"]}, "choices": ["m0", "m0b0", "m0b0c0", "m0b0c0f0", "o0", "o1", "o2"], "vocabulary_name": {"m": "description informatique", "c": "competence", "f": "utilisation de langages"}, "trees": {"(INF).*": ["m0", "m0b0"], "test": ["m0o1,2", "m0b0o1"]}, "trees_ue": {"(INF9999L|INF8888L).*": ["m0b0c0f0"]}, "merges": {"m0b0c0": {"method": "average", "fail": [], "possible_results": []}}, "errors": ["Invalid line: m0 \t [\"Non not\u00e9\",\"Mauvais\",\"Bien\"]"], "observations": {"m0": ["Non not\u00e9", "Mauvais", "Bien"]}})

    if do('abis'):
        utilities.mkpath('DBregtest/LOGINS/108/10888888')
        # 4240 ABI in 2 tables
        with open('DBregtest/LOGINS/108/10888888/resume', 'w', encoding='utf-8') as f:
            f.write("""
            {
            'UE-x': [[%d,'Automne',0,40,0,0.666,1],
                     [2000,'Automne',0,-100000,0,0,0]],
            'UE-y': [[%d,'Automne',0,4000,0,0.666,1],
                     [%d,'Printemps',0,200,0,0.777,1],
                     [2000,'Automne',0,-100000,0,0,0]]
            }""".replace(' ', '') % (uyear, uyear, uyear+1))
        c = s.url('=%s/%s/UE-abis' % (abj, ys))
        c = s.url('=%s/%s/UE-abis/1/0/column_attr_title/C1/C1' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-abis/1/1/column_attr_type/C1/Abis' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-abis/1/2/column_attr_columns/C1/ID' % (abj, ys))
        assert(c == ok_png)
        c = s.url('=%s/%s/UE-abis/1/3/cell_change/0_0/L1/10888888' % (abj, ys))
        assert(c == ok_png)

        # 2 ABI In P template

        abi_year = year
        abi_month = 5
        abi_day = 7
        c = s.url('=%s/%s/P/abis' % (abj, uyear))
        abi_P = '%s-%02d-%02d' % (abi_year, abi_month, abi_day)
        c = s.url('=%s/%s/P/abis/2/0/cell_change/0/L1/10888888' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/abis/2/1/column_attr_title/C1/%s_10:00' % (abj, uyear, abi_P))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/abis/2/2/cell_change/C1/L1/UNJLEAVE' % (abj, uyear))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/abis/2/3/column_attr_title/C2/%s_11:00' % (abj, uyear, abi_P))
        assert(c == ok_png)
        c = s.url('=%s/%s/P/abis/2/4/cell_change/C2/L1/UNJLEAVE@' % (abj, uyear))
        assert(c == ok_png)

        time.sleep(0.2)
        c = s.url('=%s/%s/UE-abis' % (abj, ys))
        assert('C(4242,"*",' in c)

    if do('notes'):
        c = s.url('=%s/record_note/k01/foobar' % abj)
        assert(c == ok_png_bin)
        assert('foobar' in utilities.manage_key('LOGINS', os.path.join(abj, 'notes')))

        c = s.url('=%s/record_note/k01/barfoo' % abj)
        assert(c == ok_png_bin)
        c = s.url('=' + abj + '/login_list/Bernard')
        assert('"note": "barfoo"' in c)

        c = s.url('=%s/favorite_student/k01' % abj)
        assert(c == ok_png_bin)
        c = s.url('=%s' % abj)
        assert('barfoo' in c)

    if do('getline'):
        create_u2()
        c = s.url('=%s/%s/UE-INF20UE2/getline/10800000' % (abj, ys))
        assert(c == '[["ID", "10800000"], ["Firstname", "Jacques"], ["Surname", "MARTIN"], ["TITLE0", 11.11]]')
        c = s.url('=%s/%s/UE-INF20UE2/getline/10800x00' % (abj, ys))
        assert(c == '[]')
        c = s.url('=%s/%s/UE-INF20UE2-DONOTEXISTS/getline/10800000' % (abj, ys))
        assert(c == '[]')

    if do('maj-student-list'):
        c = s.url('=%s/%s/demo_animaux-1' % (abj, ys))
        page = c.split('page_id = "')[1].split('"')[0]
        table_creation_date = c.split('table_creation_date = "')[1].split('"')[0]
        table_load_time = c.split('table_load_time = "')[1].split('"')[0]

        # Keep active

        file = s.url('=%s/%s/demo_animaux-1/%s/0.0.%s.%s.%s' % (abj, ys, page,
            table_creation_date, int(time.time()), int(time.time())), returns_file=True)
        file.read()

        c = s.url('=%s/%s/demo_animaux-1/%s/0/update_content' % (abj, ys, page))
        assert(c == ok_png)

        # Check if active

        c = s.url('=%s/stat' % root)
        assert(re.findall('demo_animaux-1</a></td><td>1/3<small>[^\n]*MU<', c))

        # Wait the end of student list update

        for i in range(50):
            content = utilities.read_file('DBregtest/Y%d/S%s/demo_animaux-1.py' % (year, semester))
            if "column_attr('columns',1,'Avg.TP','TP1 TP2 TP3'" in content:
                break
            time.sleep(0.1)
        else:
            print(content)
            raise ValueError("Not found old registered")
        time.sleep(0.1)

        # Add a student, must not be unregistered

        c = s.url('=%s/%s/demo_animaux-1/%s/1/cell_change/0_0/L1/k99' % (abj, ys, page))
        assert(c == ok_png)

        for i in range(50):
            content = utilities.read_file('DBregtest/Y%d/S%s/demo_animaux-1.py' % (year, semester))
            if "cell_change(0,'0_5','L1','non'" in content:
                print(content)
                raise ValueError("Must not be unregistered")
            if "cell_change(0,'0_4','L1',''" in content:
                break
            time.sleep(0.1)
        else:
            print(content)
            raise ValueError("Not found new registered")

        # Force student list update

        c = s.url('=%s/%s/demo_animaux-1/%s/2/update_content' % (abj, ys, page))
        assert(c == ok_png)

        for i in range(50):
            content = utilities.read_file('DBregtest/Y%d/S%s/demo_animaux-1.py' % (year, semester))
            if "cell_change(0,'0_5','L1','non'," in content:
                print("\ti=", i)
                break
            time.sleep(0.1)
        else:
            raise ValueError("Not found unregistered")

        # Try with auto_update

        c = s.url('=%s/%s/demo_animaux-1/%s/2/table_attr_official_ue/1' % (abj, ys, page))
        assert(c == ok_png)
        c = s.url('=%s/%s/demo_animaux-1/%s/3/cell_change/0_0/L2/k98' % (abj, ys, page))
        assert(c == ok_png)

        assert(c == ok_png)
        for i in range(50):
            content = utilities.read_file('DBregtest/Y%d/S%s/demo_animaux-1.py' % (year, semester))
            if "cell_change(0,'0_5','L2','non'" in content:
                print(content)
                raise ValueError("Must not be unregistered")
            if "cell_change(0,'0_4','L2',''" in content:
                break
            time.sleep(0.1)
        else:
            print(content)
            raise ValueError("Not found new registered")

        feedback = s.url('=%s/auto_update' % root, returns_file=True)
        while True:
            data = feedback.read()
            if b'demo_animaux-1' in data and b'lists are now up to date' in data:
                feedback.close()
                break
            if not data:
                raise ValueError('Not updated')
        for i in range(50):
            content = utilities.read_file('DBregtest/Y%d/S%s/demo_animaux-1.py' % (year, semester))
            if "cell_change(0,'0_5','L2','non'," in content:
                print("\ti=", i)
                break
            time.sleep(0.1)
        else:
            raise ValueError("Not found unregistered")

    if do('pdf-form'):
        shutil.rmtree(f'UPLOAD/{ys}/UE-INF20UE2', ignore_errors=True)
        create_u2()
        c = s.url(f'={root}/{ys}/UE-INF20UE2/1/0/column_attr_type/Anno/Annotate')
        assert(c == ok_png)
        c = s.url(f'={root}/{ys}/UE-INF20UE2/1/1/column_attr_modifiable/Anno/2')
        assert(c == ok_png)
        try:
            s.url('=10800000', stop_if_error=False, display_log_if_error=False) # Create ticket in order to use POST after
        except: # pylint: disable=bare-except
            pass
        c = s.post(f'=10800000/{ys}/UE-INF20UE2/upload_post/Anno/0_0',
                   fields = ( ("filename", "foo2.txt"), ),
                   files = ( ("data", "FOO2.TXT", b"A TXT CONTENT 2"), )
               )
        assert('Only PDF files are allowed' in c)
        c = s.post(f'=10800000/{ys}/UE-INF20UE2/upload_post/Anno/0_0',
                   fields = ( ("filename", "foo2.pdf"), ),
                   files = ( ("data", "FOO2.PDF",utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('PDF → PNG OK' in c)


        # Test loading student PDF(PNG format) if it is not a form
        c = s.url(f'={invited}/{ys}/UE-INF20UE2/annotator_form/Anno/0_0/foo.zip', read_bytes=True)
        utilities.write_file('xxx.zip', c, encoding='bytes')
        zf = zipfile.ZipFile(open('xxx.zip', 'rb+'), mode="r")
        assert(sorted(zf.namelist()) == ['2024_Printemps_UE-INF20UE2_/10800000#Jacques_MARTIN_doc.png'])

        # Upload form background
        c = s.post(f'={root}/{ys}/UE-INF20UE2/annotator_upload/Anno',
                   fields = ( ("letter", "A"), ),
                   files = ( ("data", "FOO.PDF", utilities.read_file("REGTEST_SERVER/tests.pdf", encoding='bytes')), )
               )
        assert('window.parent.annotator_add("A")' in c)

        # Set the ID on the background
        c = s.url(f'={invited}/{ys}/UE-INF20UE2/annotator_record/Anno/{abj}')
        assert("Can't find line" in c)
        c = s.post(f'={root}/{ys}/UE-INF20UE2/annotator_record/Anno/{root}',
                   fields = ( ("content", '[[]]'), ))
        assert('There is a bug' in c)
        c = s.post(f'={root}/{ys}/UE-INF20UE2/annotator_record/Anno/{root}',
                   fields = ( ("content",
                               json.dumps([["new",
                                           1702632218,
                                           1,
                                           {"type": "text", "left": 422.79, "top": 269.7,
                                            "width": 59.76, "height": 33.9, "fill": "#FFF",
                                            "stroke": "#000", "backgroundColor": "#FFFA", "text": "ID",
                                            "fontSize": 30, "scaleY": 1
                                           }]]) + '\n'
                              ),
                            )
                  )
        assert('OK!' in c)

        # Get the ZIP of PDF with form filled of bad table/column
        c = s.url(f'={invited}/{ys}/UE-INF20UE2-bad/annotator_form/Anno/0_0/foo.zip', read_bytes=True)
        assert(c == b'')
        c = s.url(f'={invited}/{ys}/UE-INF20UE2/annotator_form/Ano/0_0/foo.zip', read_bytes=True)
        assert(c == b'')

        # Get the ZIP of PDF with form filled
        c = s.url(f'={invited}/{ys}/UE-INF20UE2/annotator_form/Anno/0_0/foo.zip', read_bytes=True)
        utilities.write_file('xxx.zip', c, encoding='bytes')
        zf = zipfile.ZipFile(open('xxx.zip', 'rb+'), mode="r")
        assert(sorted(zf.namelist()) == ['TOMUSS/10800000#Jacques_MARTIN.pdf'])
        f = zf.open('TOMUSS/10800000#Jacques_MARTIN.pdf', "r")
        utilities.write_file('xxx.pdf', f.read(), encoding='bytes')
        f.close()
        with os.popen('pdftotext xxx.pdf -', 'r') as file:
            c = file.read()
            assert('Page 1 one\n\n10800000\n' in c)
            assert('Page 8 eight\n' in c)

    if do('qrcode_past'):
        c = s.url(f'={abj}/{ys}/qrcode_past')
        c = s.url(f'={abj}/{ys}/qrcode_past/1/0/cell_change/0_0/L1/18000001')
        assert(c == ok_png)

        # 1 january 12:00, 90 minutes
        c = s.url(f'={abj}/{ys}/qrcode_past/qrcode_new/-20010101120090/grpseq/[["A","1"]]')
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url(f'={abj}/qrcode_update/{qrcode}/set/18000001/PRST/42')
        assert(c == ok_png_bin)
        c = s.url(f'={abj}/{uyear}/P/etape-18000001')
        assert('2001-01-01_12:00' in c)

        # 1 january 13:00, 90 minutes
        c = s.url(f'={abj}/{ys}/qrcode_past/qrcode_new/-20010101130090/grpseq/[["A","1"]]')
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url(f'={abj}/qrcode_update/{qrcode}/set/18000001/PRST/42')
        assert(c == ok_png_bin)
        c = s.url(f'={abj}/{uyear}/P/etape-18000001')
        assert('2001-01-01_13:00' in c)
        assert('"position": 8' in c)

        # 1 january 11:00, 90 minutes
        c = s.url(f'={abj}/{ys}/qrcode_past/qrcode_new/-20010101110090/grpseq/[["A","1"]]')
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url(f'={abj}/qrcode_update/{qrcode}/set/18000001/PRST/42')
        assert(c == ok_png_bin)
        c = s.url(f'={abj}/{uyear}/P/etape-18000001')
        assert('2001-01-01_11:00' in c)
        assert('"position": 6.5' in c)

        # 1 january 11:30, 90 minutes
        c = s.url(f'={abj}/{ys}/qrcode_past/qrcode_new/-20010101113090/grpseq/[["A","1"]]')
        qrcode = c.split('var qrcode = ')[1].split('"')[1]
        c = s.url(f'={abj}/qrcode_update/{qrcode}/set/18000001/PRST/42')
        assert(c == ok_png_bin)
        c = s.url(f'={abj}/{uyear}/P/etape-18000001')
        assert('2001-01-01_11:30' in c)
        assert('"position": 6.75' in c)

    ######################################
    # Must be kept last: broke coverage
    ######################################
    if do('restart'):
        c = s.url('=%s/restart_tomuss/0.01/0.01' % root)
        assert('GO GO GO GO' in c)
        for i in range(10):
            time.sleep(2)
            try:
                s.url('=%s' % root)
            except:
                pass
            break
        else:
            bug

if '1' in sys.argv:
   sys.argv.remove('1')
   only_once = True
else:
   only_once = False

coverage = 'coverage' in sys.argv
if coverage:
    sys.argv.remove('coverage')

search_leak = 'leak' in sys.argv
if search_leak:
    sys.argv.remove('leak')

os.system("ps -fle | grep ./tomuss")

n = 0
while True:
    os.rename('TMP', 'TMP.original')
    start = Server.start_time = time.time()
    exit_status = 1
    try:
        if 'only-sync' not in sys.argv:
                tests()
        else:
                s = Server(coverage=coverage, search_leak=search_leak)
                s.start()
        if coverage:
            if os.system("cd REGTEST_SERVER ; make coverage-in") != 0:
                raise ValueError("make coverage-in failed")
        exit_status = 0
        print('Test fine')
    except AssertionError:
        if c == '':
            print('assert: empty !!!!!')
        elif c == bad_png:
            print('assert: bad_png')
        elif c == ok_png:
            print('assert: ok_png')
        elif c == bug_png:
            print('assert: bug_png')
        elif c == bad_png_bin:
            print('assert: bad_png_bin')
        elif c == ok_png_bin:
            print('assert: ok_png_bin')
        elif c == bug_png_bin:
            print('assert: bug_png_bin')
        else:
            print("Unknown value:", str(c)[:10000])
            f = open('xxx.html', 'w', encoding = "utf-8")
            f.write(c)
            f.close()
        print('End of regressions tests : failure')
        traceback.print_exc(file=sys.stdout)
        break
    except KeyboardInterrupt:
        exit_status = '^C'
        print()
        print(('*'*79 + '\n')*5)
        print("         Wait a second, servers are being stopped and files cleaned\n")
        print(('*'*79 + '\n')*5)
        break
    except:
        traceback.print_exc(file=sys.stdout)
        break
    finally:
        if exit_status != '^C':
            with open("regtests.results", "a") as log:
                if exit_status:
                    more = do.current
                else:
                    more = 'OK'
                log.write(f"{time.strftime('%Y%m%d-%H:%M:%S')} {int(time.time() - start):4}sec {more}\n")
        try:
            try:
                s.stop()
            except:
                print('Not stopped')
                shutil.rmtree('DBregtest', ignore_errors=True)
            if ss and ss.started:
                ss.stop()

            keep = 'DBregtest/Y%s/S%s/UE-UE1.py' % (year, semester)
            if os.path.exists(keep):
                content = utilities.read_file(keep)
            else:
                content = None
            print("exit_status", exit_status)
            if exit_status == 0 and do('sync'):
                print("Start sync.py")
                assert(os.system("cd REGTEST_SERVER ; ./sync.py "
                                 + ('coverage' if coverage else '')) == 0)

            if content:
                utilities.write_file(keep, content) # For coverage of teacher.py

        finally:

            if os.path.exists('TMP.regtest'):
                shutil.rmtree('TMP.regtest')
            os.rename('TMP', 'TMP.regtest')
            os.rename('TMP.original', 'TMP')

    if len(sys.argv) == 1:
        print("Start browser tests")
        if os.system('SCRIPTS/test_gui.py 1 hidden'):
            sys.exit(1)

    if only_once:
        break

sys.exit(exit_status)
